/*******************************************************************************
* FILE NAME: ibase.cpp                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class IBase.                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147482624 )   // IBASE_PRIORITY

#define INCL_DOSMODULEMGR

extern "C" {
  #include <iwindefs.h>
  #include <stdlib.h>
}

#include <new.h>

#include <istream.hpp>

#include <ibase.hpp>

#ifndef _ISTRING_
  #include <istring.hpp>
#endif

#ifndef _ICCONST_
  #include <icconst.h>
#endif

#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBASE_CPP_
  #include <ipagetun.h>
#endif

// Local struct for library initialization.
struct ICLUI_Init : IException::TraceFn
{
  ICLUI_Init();
virtual void
  write ( const char *exceptionText );
static void
  newHandler ( );
};

// Dump Memory statistics
#ifdef __DEBUG_ALLOC__
  class IDumpAllRemainingStorage
  {
  public:
    ~IDumpAllRemainingStorage()
     {
        _dump_allocated(20);
     }
  };


#pragma data_seg(ICLNonConst)
  static IDumpAllRemainingStorage dumpIt;
#pragma data_seg()
#endif

#pragma data_seg(ICLNonConst)
  //   initialize synonyms for IException::Severity
  IException::Severity
    IBase::unrecoverable = IException::unrecoverable,
    IBase::recoverable   = IException::recoverable;

static ICLUI_Init initializer;

#pragma data_seg()





ICLUI_Init ::   ICLUI_Init()
  {
  // Add this object as exception trace function:
  IException::setTraceFunction( *this );
  // Register newHandler function:
  set_new_handler( ICLUI_Init::newHandler );
  }


void ICLUI_Init :: write ( const char *exceptionText )
  {
  ITrace log;
  log.write( exceptionText );
  }

void ICLUI_Init :: newHandler ( )
  {
  ITHROWLIBRARYERROR(IC_HEAP_EXHAUSTED, IErrorInfo::outOfMemory, IException::unrecoverable);
  }


/*------------------------------------------------------------------------------
| Statics & functions related to message file handling|
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
char
 *IBase::msgFile = 0;

static const char
 defaultFile[] = "CPPOOC3U.MSG",
 msgFileVar[]  = "ICLUI MSGFILE";
#pragma data_seg()

char* IBase :: messageFile ( )
  {
  if ( !IBase::msgFile )
    {
    const char *pFileName = getenv( msgFileVar );
    if ( !pFileName )
      pFileName = defaultFile;
    IBase::setMessageFile( pFileName );
    }
  return msgFile;
  }

void IBase :: setMessageFile ( const char *msgFileName )
  {
  if ( msgFileName )
    {
    if (IBase::msgFile)
       delete [] IBase::msgFile;
    IBase::msgFile = new char[strlen(msgFileName) + 1];
    strcpy(IBase::msgFile, msgFileName);
    }
  }

/*------------------------------------------------------------------------------------------------
| messageText - returns the message specified by "messageId", |
| with the specified text strings inserted at the proper places.       |
-------------------------------------------------------------------------------------------------*/
IMessageText IBase :: messageText ( unsigned long messageId,
                                    const char*   textInsert1,
                                    const char*   textInsert2,
                                    const char*   textInsert3,
                                    const char*   textInsert4,
                                    const char*   textInsert5,
                                    const char*   textInsert6,
                                    const char*   textInsert7,
                                    const char*   textInsert8,
                                    const char*   textInsert9 )
  {
  IMessageText msg(messageId, IBase::messageFile(), textInsert1, textInsert2, textInsert3,
                   textInsert4, textInsert5, textInsert6, textInsert7, textInsert8,
                   textInsert9);
  return msg;
  }

ostream &operator << ( ostream     &aStream,
                       const IBase &anObject )
  {
  return aStream << anObject.asString();
  }

IString IBase :: asString ( ) const
  {
  return "IBase object";
  }

IString IBase :: asDebugInfo ( ) const
  {
  IString result = "IBase(@";
  result += IString( (unsigned long)this ).d2x();
  result += ")";
  return result;
  }

IBase::Version IBase :: version ( )
  {
  Version
    current = { IC_MAJOR_VERSION, IC_MINOR_VERSION };
  return current;
  }
