/*******************************************************************************
* FILE NAME: ibuffer0.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ibuffer0.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C" {
  #include <string.h>
  #include <stdlib.h>
  #define INCL_DOSNLS
  #include <iwindefs.h>
  }

#include <ibuffer.hpp>

#ifndef _ISTRING_
  #include <istring.hpp>
#endif

#ifndef _IDBCSBUF_
  #include <idbcsbuf.hpp>
#endif

#ifndef _IEXCBASE_
  #include <iexcbase.hpp>
#endif

#ifndef _ICCONST_
  #include <icconst.h>
#endif

#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBUFFER0_CPP_
  #include <ipagetun.h>
#endif
#pragma data_seg(ICLNonConst)
char
  IBuffer::dbcsTable[ 256 ];
#pragma data_seg()

/*------------------------------------------------------------------------------
| IBuffer::className                                                           |
------------------------------------------------------------------------------*/
const char * IBuffer :: className ( ) const
  {
  return "IBuffer";
  }

/*------------------------------------------------------------------------------
| IBuffer::asDebugInfo                                                         |
------------------------------------------------------------------------------*/
IString IBuffer :: asDebugInfo ( ) const
  {
  IString result( this->className() );
  result += "(@";
  result += IString( (unsigned long) this );
  result += ",refs=";
  result += IString( this->useCount() );
  result += ",len=";
  result += IString( this->length() );
  result += ",data=[";
  if ( this->length() > 23 )
    {
    result += IString( this->contents(), 10 );
    result += "...";
    result += IString( this->contents() + this->length() - 10, 10 );
    }
  else
    result += this->contents();
  result += "])";
  return result;
  }


/*------------------------------------------------------------------------------
| IBuffer :: charType                                                          |
------------------------------------------------------------------------------*/
IStringEnum::CharType IBuffer :: charType ( unsigned ) const
  {
  return IStringEnum::sbcs;
  }

/*------------------------------------------------------------------------------
| IBuffer::allocate                                                            |
------------------------------------------------------------------------------*/
IBuffer *IBuffer :: allocate ( unsigned newLen ) const
  {
  return new ( newLen ) IBuffer ( newLen );
  }

/*------------------------------------------------------------------------------
| IBuffer::IBuffer                                                             |
------------------------------------------------------------------------------*/
IBuffer :: IBuffer ( unsigned length )
  : refs( 1 ), len( length )
  {
  data[ length ] = '\0';
  }

/*------------------------------------------------------------------------------
| IBuffer::~IBuffer                                                            |
------------------------------------------------------------------------------*/
IBuffer :: ~IBuffer ( )
  {
  }

/*------------------------------------------------------------------------------
| IBuffer::nullBuffer                                                          |
|                                                                              |
| IBuffer with "null" value.                                                   |
|                                                                              |
| This buffer is the "default" buffer used to construct IStrings.              |
|                                                                              |
| All "null" IStrings will point to this instance.  It's "contents"            |
| are a single null character.  This ensures that using a null                 |
| IString equates to using a null string (in more of the C sense).             |
| When running on a DBCS machine, the null buffer will be an instance of       |
| IDBCSBuffer and a new nullIBuffer is used when a null IBuffer instance is    |
| needed on DBCS machines.                                                     |
------------------------------------------------------------------------------*/
IBuffer *IBuffer::nullBuffer = 0;
IBuffer *IBuffer::nullIBuffer = 0;

/*------------------------------------------------------------------------------
| IBuffer::null                                                                |
------------------------------------------------------------------------------*/
IBuffer *IBuffer :: null ( ) const
  {
  if ( nullIBuffer && ( nullIBuffer->className() == this->className() ))
    {
    return nullIBuffer;
    }
  return IBuffer::defaultBuffer();
  }

/*------------------------------------------------------------------------------
| IBuffer::defaultBuffer                                                       |
------------------------------------------------------------------------------*/
IBuffer *IBuffer :: defaultBuffer ( )
  {
  return ( nullBuffer ) ?
    nullBuffer
    :
    ( nullBuffer = IBuffer::initialize() );
  }

/*------------------------------------------------------------------------------
| IBuffer::initialize                                                          |
|                                                                              |
| Build DBCS table and return pointer to appropriate (null) IBuffer.           |
------------------------------------------------------------------------------*/
IBuffer *IBuffer :: initialize ( )
  {
  unsigned      n = 0;
  signed int           i;
  Boolean       dbcs = false;

  UCHAR         buffer[12];
  COUNTRYCODE   countryCode;
  countryCode.country = 0;     // 0 means return DBCS information for
  countryCode.codepage = 0;    //  the current process code page.
  DosQueryDBCSEnv( sizeof(buffer), &countryCode, (PCHAR)buffer );

  if ( buffer[0] | buffer[1] )
  {
    dbcs = true;
    for (i = 0; i < sizeof(dbcsTable); i++)
    {
      if ( i >= buffer[n] && i <= buffer[n+1] )
        dbcsTable[i] = 1;
      else
        dbcsTable[i] = 0;
      if ( i == buffer[n+1] )
        n += 2;
    }
  }
  else
  {
    for (i = 0; i < sizeof(dbcsTable); i++)
      dbcsTable[i] = 0;
    // Test code: Query IDBCSTABLE environment variable and
    // use its value as valid DBCS first byte codes.
    char *var = getenv( "IDBCSTABLE" );
    if ( var )
      while ( *var )
        {
        dbcs = true;
        dbcsTable[ (unsigned)(*var++) ] = 1;
        }
  }
  if ( dbcs )
    {
    nullIBuffer = new (0) IBuffer( 0 );
    return new (0) IDBCSBuffer( 0 );
    }
  else
    return new (0) IBuffer( 0 );
  }

/*------------------------------------------------------------------------------
| IBuffer::next                                                                |
------------------------------------------------------------------------------*/
char *IBuffer :: next ( const char *prev )
  {
  return (char*)prev++;
  }

/*------------------------------------------------------------------------------
| IBuffer::next                                                                |
------------------------------------------------------------------------------*/
const char *IBuffer :: next ( const char *prev ) const
  {
  return prev++;
  }

/*------------------------------------------------------------------------------
| IBuffer::IBuffer                                                             |
|                                                                              |
| Construct a buffer of the specified length.  The "data" member               |
| array must actually be of length 1 greater than the argument                 |
| value (this is achieved automatically via use of the overloaded              |
| operator new for class IBuffer).                                             |
|                                                                              |
| The terminating (extra) byte is set to null.                                 |
|                                                                              |
| Notes:                                                                       |
|   1. This method is protected.  IBuffers must be obtained by using           |
|      IBuffer::nullBuffer and subsequent newBuffer calls to existing          |
|      IBuffer objects.  The only non-heap instance of this class is           |
|      the static IBuffer::nullBuffer object.                                  |
|   2. This method should be inline (in context of IBuffer::newBuffer).        |
------------------------------------------------------------------------------*/
// See ibuffer.inl

/*------------------------------------------------------------------------------
| IBuffer::newBuffer                                                           |
|                                                                              |
| Allocate a new IBuffer of the same type as the receiver and initialize it    |
| with the provided data.                                                      |
------------------------------------------------------------------------------*/
IBuffer *IBuffer :: newBuffer ( const void *p1,
                                unsigned    len1,
                                const void *p2,
                                unsigned    len2,
                                const void *p3,
                                unsigned    len3,
                                char        padChar )
  const
  {
  unsigned
    newLen = checkAddition( checkAddition( len1, len2 ),  len3 );
  IBuffer *buffer;
  if ( newLen )
    {
    buffer = this -> allocate( newLen );
    char *p = buffer->contents();
    // Copy first portion (or pad).
    if ( p1 )
      memcpy( p, p1, len1 );
    else
      memset( p, padChar, len1 );
    p += len1;
    // Copy second portion (or pad).
    if ( p2 )
      memcpy( p, p2, len2 );
    else
      memset( p, padChar, len2 );
    p += len2;
    // Copy third portion (or pad).
    if ( p3 )
      memcpy( p, p3, len3 );
    else
      memset( p, padChar, len3 );
    }
  else
    {
    buffer = this->null();
    buffer -> addRef();
    }
  return buffer;
  }

/*------------------------------------------------------------------------------
| IBuffer :: operator new                                                      |
|                                                                              |
| Allocate an IBuffer object with "data" array of requested size.              |
|                                                                              |
| Notes:                                                                       |
|   1. The argument size is the size of the string to be placed in             |
|      the buffer, *excluding* the terminating null.  The total amount         |
|      of space is thus the size of the IBuffer header (use count and          |
|      length fields), plus the string length (as given by the argument),      |
|      plus 1 more byte (for the terminating null).  Since one byte of         |
|      the data array is declared in IBuffer, the proper amount of             |
|      space is calculated by the expression used below.                       |
------------------------------------------------------------------------------*/
// See ibuffer.inl

/*------------------------------------------------------------------------------
| IBuffer::operator delete                                                     |
|                                                                              |
| Delete the storage using the inverse of the operator used to allocate it.    |
------------------------------------------------------------------------------*/
// See ibuffer.inl

/*------------------------------------------------------------------------------
| IBuffer :: addRef                                                            |
|                                                                              |
| Increment use count for this IBuffer.                                        |
------------------------------------------------------------------------------*/
// See ibuffer.inl

/*------------------------------------------------------------------------------
| IBuffer :: removeRef                                                         |
|                                                                              |
| Decrement the use count of the receiver.                                     |
|                                                                              |
| If the resulting use count is now zero, release the IBuffer by               |
| calling delete against "this."                                               |
|                                                                              |
| Notes:                                                                       |
|   1. This function appears dangerous.  However, a call to this function      |
|      is interpreted as giving up access to the associated buffer.  At        |
|      that point, only other references are valid (and if there are none,     |
|      then the receiver can be deleted).                                      |
------------------------------------------------------------------------------*/
// See ibuffer.inl

/*------------------------------------------------------------------------------
| IBuffer :: rightJustify                                                      |
|                                                                              |
| This function right justifies the receiver in a new buffer of the given      |
| length and returns the address of the new buffer.                            |
|                                                                              |
| The right justification algorithm is as follows:                             |
|                                                                              |
|   1. Calculate how much padding will be needed on the left.                  |
|   2. Calculate how many bytes of the current buffer will be                  |
|      transferred to the right-justified result.                              |
|   3. return a new buffer comprised of the "prefix" pad bytes                 |
|      and the portion of the receiver calculated at step 2.                   |
|                                                                              |
| Notes:                                                                       |
|   1. This function (and likewise all the similar "editing" functions)        |
|      is usually called in the following context:                             |
|         IString::xxxxx ( )                                                   |
|           {                                                                  |
|           ...                                                                |
|           IString old( self.pBuffer );                                       |
|           self.pBuffer = old.pBuffer->rightJustify( ... );                   |
|           ...                                                                |
|           }                                                                  |
|   2. This function appears here (rather than with the other editing          |
|      functions in ibuffer7.cpp) because it is called from the conversion     |
|      functions in istring9/ibuffer9.  Placing it here avoids pulling in      |
|      all the editing functions when only conversion is needed.               |
------------------------------------------------------------------------------*/
IBuffer * IBuffer :: rightJustify ( unsigned newLength,
                                    char     padCharacter )
  {
  if ( newLength != this->length() )
     {
     unsigned prefix       = ( newLength > this->length() )
                               ? newLength - this->length()
                               : 0,
              fromReceiver = ( this->length() < newLength ) ?
                               this->length()
                               :
                               newLength;
     return this->newBuffer( 0,
                             prefix,
                             this->contents() + this->length() - fromReceiver,
                             fromReceiver,
                             0, 0,
                             padCharacter );
     }
  return this;
  }

/*------------------------------------------------------------------------------
| IBuffer::remove                                                              |
|                                                                              |
| If the buffer will remain unmodified (starting index is beyond the end or    |
| receiver is null), then the receiver is returned unmodified.                 |
|                                                                              |
| Else, if the number of characters to delete is zero, or passes the end of    |
| the receiver, then it is defaulted to the rest of the buffer. Finally, the   |
| new buffer is allocated and filled in via a call to newBuffer().             |
|                                                                              |
| Notes:                                                                       |
|   1. See notes 1. and 2. under IBuffer::rightJustify.                        |
------------------------------------------------------------------------------*/
IBuffer * IBuffer :: remove ( unsigned startPos,
                              unsigned numChars )
  {
  if ( startPos <= this->length()
       &&
       this->length() != 0 )
    {
    // Consider startPos 0 to be 1:
    if ( startPos == 0 )
      startPos++;
    // Default numChars to rest of string:
    if ( numChars > this->length() - startPos )
      numChars = this->length() - startPos + 1;

    // Initialize from current contents before/after deleted chars:
    return this->newBuffer( this->contents(),
                            startPos - 1,
                            this->contents() + startPos + numChars - 1,
                            this->length() - numChars - startPos + 1,
                            0, 0,
                            0 );
    }
  return this;
  }

/*------------------------------------------------------------------------------
| IBuffer::overflow                                                            |
|                                                                              |
| Throw "overflow" exception.                                                  |
------------------------------------------------------------------------------*/
unsigned IBuffer :: overflow ( )
  {
  ITHROWLIBRARYERROR( IC_ISTRING_OVERFLOW,
                      IErrorInfo::invalidRequest,
                      IException::recoverable );
  return 0;
  }
