/*******************************************************************************
* FILE NAME: ibuffer1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of the IBuffer testing               *
*   functions.                                                                 *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #include <ctype.h>
  }

#include <ibuffer.hpp>

#include <istrtest.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBUFFER1_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IBuffer::isAlnum                                                             |
| IBuffer::isAlpha                                                             |
| IBuffer::isAscii                                                             |
| IBuffer::isCntrl                                                             |
| IBuffer::isDigits                                                            |
| IBuffer::isGraphics                                                          |
| IBuffer::isHexDigits                                                         |
| IBuffer::isLowerCase                                                         |
| IBuffer::isPrintable                                                         |
| IBuffer::isPunctuation                                                       |
| IBuffer::isUpperCase                                                         |
| IBuffer::isWhitespace                                                        |
|                                                                              |
| Each of these functions is implemented by constructing (implicitly)          |
| an IStringTest with the corresponding standard library function.             |
| This IStringTest is then applied to the buffer contents using the            |
| indexOfAnyBut member function (which returns the index of the first          |
| character that fails the test).  This "index" is then negated to             |
| achieve the desired result:                                                  |
|   0  (no characters failed the test)  -> "true"                              |
|   >0 (some character failed the test) -> "false"                             |
|                                                                              |
| Notes:                                                                       |
|   1. This simplistic implementation fails NLS/DBCS criteria, the             |
|      determination of alpha/lowercase/etc. must be made on a per             |
|      code page basis.                                                        |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isAlphanumeric ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isalnum ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isAlphabetic                                                        |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isAlphabetic ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isalpha ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isASCII                                                             |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isASCII ( ) const
  {
  unsigned    i = this->length();
  const unsigned char *p = (unsigned char *) this->contents();

  if (!i)  return false;      // return false for null string

  while ( i-- )
    if ( *p++ > 127 )
      return false;
  return true;
  }

/*------------------------------------------------------------------------------
| IBuffer::isControl                                                           |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isControl ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( iscntrl ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isDigits                                                            |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isDigits ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isdigit ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isGraphics                                                          |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isGraphics ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isgraph ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isHexDigits                                                         |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isHexDigits ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isxdigit ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isLowerCase                                                         |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isLowerCase ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( islower ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isPrintable                                                         |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isPrintable ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isprint ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isPunctuation                                                       |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isPunctuation ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( ispunct ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isUpperCase                                                         |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isUpperCase ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isupper ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isWhiteSpace                                                        |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isWhiteSpace ( ) const
  {
    if (!this->length())
       return false;           // return false for null string

    return !this->indexOfAnyBut( IStringTest( isspace ) );
  }

/*------------------------------------------------------------------------------
| IBuffer::isDBCS                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isDBCS ( ) const
  {
  return false;
  }

/*------------------------------------------------------------------------------
| IBuffer::isMBCS                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isMBCS ( ) const
  {
  return false;
  }

/*------------------------------------------------------------------------------
| IBuffer::isSBCS                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isSBCS ( ) const
  {
  return true;
  }

/*------------------------------------------------------------------------------
| IBuffer::includesMBCS                                                        |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: includesMBCS ( ) const
  {
  return false;
  }

/*------------------------------------------------------------------------------
| IBuffer::includesDBCS                                                        |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: includesDBCS ( ) const
  {
  return false;
  }

/*------------------------------------------------------------------------------
| IBuffer::includesSBCS                                                        |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: includesSBCS ( ) const
  {
  return true;
  }

/*------------------------------------------------------------------------------
| IBuffer::isValidDBCS                                                         |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isValidDBCS ( ) const
  {
  return true;
  }
/*------------------------------------------------------------------------------
| IBuffer::isValidMBCS                                                         |
------------------------------------------------------------------------------*/
IBase::Boolean IBuffer :: isValidMBCS ( ) const
  {
  return true;
  }
