/*******************************************************************************
* FILE NAME: ibuffer2.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of the IBuffer comparison            *
*   function "compare".                                                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #include <string.h>
  }

#include <ibuffer.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBUFFER2_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IBuffer::compare                                                             |
|                                                                              |
| Initialize result to "less than".                                            |
|                                                                              |
| If neither string is null, we compare as many bytes as in the                |
| shorter of the two, using memcmp.                                            |
|                                                                              |
| If these sections compare equal, then, if the lengths are the same,          |
| set the result to "equal."  Otherwise, set the result to either              |
| "less than" or "greater than," depending on which string is longer           |
| (the longer one is "greater").                                               |
|                                                                              |
| If these sections aren't equal, then simply set the result to                |
| "less than" or "greater than" according to the result of the call            |
| to memcmp.                                                                   |
|                                                                              |
| If the receiver isn't null, but the argument is, then return                 |
| "greater than."  If both the receiver and argument are null,                 |
| return "equal."                                                              |
|                                                                              |
| Notes:                                                                       |
|   1. A quick check is made to see if the receiver's buffer and the           |
|      argument coincide (in which case "equal" is returned).                  |
------------------------------------------------------------------------------*/
IBuffer::Comparison IBuffer :: compare ( const void*  pArg,
                                         unsigned     argLen ) const
  {
  Comparison result = lessThan;
  if ( this->contents() == pArg
       &&
       this->length() == argLen )
    result = equal;
  else
    if ( this->length() > 0 )
      if ( argLen > 0 )
        {
        int rc = memcmp( this->contents(),
                         pArg,
                         ( this->length() < argLen ) ?
                           this->length()
                         :
                           argLen );
        if ( rc == 0 )
          result = ( this->length() == argLen ) ?
                     equal
                   :
                     ( this->length() < argLen ) ?
                       lessThan
                     :
                       greaterThan;
        else
          result = ( rc < 0 ) ? lessThan : greaterThan;
        }
      else
        result = greaterThan;
    else
      if ( argLen == 0 )
        result = equal;
  return result;
  }
