/*******************************************************************************
* FILE NAME: ibuffer5.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of the IBuffer accessing             *
*   function "subString".                                                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #include <string.h>
  }

#include <ibuffer.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBUFFER5_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IBuffer::subString                                                           |
|                                                                              |
| If the starting position is zero, then start at beginning of string.         |
|                                                                              |
| Then, we calculate how much of the substring can come from the receiver.     |
| This is either the tail end starting at startPos, or 0 (if startPos is       |
| off the end already).                                                        |
|                                                                              |
| Next, we reduce fromReceiver to the requested length (if smaller).           |
|                                                                              |
| Finally, we allocate a new buffer via a call to newBuffer(), initializing    |
| from the receiver (for length fromReceiver) and padding with the             |
| argument pad character if extra is required.                                 |
------------------------------------------------------------------------------*/
IBuffer *IBuffer :: subString ( unsigned startPos,
                                unsigned len,
                                char     padCharacter ) const
  {
  // Map startPos=0 to startPos=1...
  if ( startPos == 0 )
     startPos = 1;
  // Calculate how much can come from receiver.
  unsigned int fromReceiver = ( startPos <= this->length() ) ?
                                this->length() - startPos + 1
                              :
                                0;
  // Reduce if request length is shorter...
  if ( len < fromReceiver )
    fromReceiver = len;
  // Initialize what we can from the receiver and leave
  // room for the remainder.
  return this->newBuffer( this->contents() + startPos - 1, fromReceiver,
                          0, len - fromReceiver,
                          0, 0,
                          padCharacter );
  }
