/*******************************************************************************
* FILE NAME: idbcsb5.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*  This file contains the implementation of the IDBCSBuffer accessing          *
*  function "subString".                                                       *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #include <string.h>
  }

#include <ibuffer.hpp>
#include <idbcsbuf.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDBCSB5_CPP_
  #include <ipagetun.h>
#endif
/*------------------------------------------------------------------------------
| IDBCSBuffer::subString                                                       |
|                                                                              |
| Allocate a new buffer by calling the inherited version of this function.     |
|                                                                              |
| Then, if the substring split a DBCS character at either end, convert that    |
| character to the pad character.                                              |
------------------------------------------------------------------------------*/
IBuffer *IDBCSBuffer :: subString ( unsigned startPos,
                                    unsigned len,
                                    char     padCharacter ) const
{
   IBuffer *result;

   if ( startPos == 0 )
      startPos = 1;

   //-------------------------------------------------------------------------
   // Calculate how much can come from receiver.
   //-------------------------------------------------------------------------
   unsigned int fromReceiver = ( startPos <= this->length() )
                                 ? this->length() - startPos + 1
                                 : 0;

   //-------------------------------------------------------------------------
   // Default length to rest of string.
   //-------------------------------------------------------------------------
   if ( len < fromReceiver )
      fromReceiver = len;

   //-------------------------------------------------------------------------
   // Initialize what we can from the receiver and leave
   // room for the remainder.
   //-------------------------------------------------------------------------
   result = this->newBuffer( this->contents() + startPos - 1, fromReceiver,
                             0, len - fromReceiver,
                             0, 0,
                             padCharacter );

   //-------------------------------------------------------------------------
   // If the byte at startPos is the second byte of a DBCS character,
   // then that byte should be replaced with a padCharacter.
   // If the byte at (startPos + len - 1) is the first byte of a DBCS
   // character, then that byte should be replaced with a padCharacter.
   //-------------------------------------------------------------------------
   for ( unsigned i = 0; startPos + i <= this->length (); i++ ) {
     if ( this->charType( startPos + i ) > IStringEnum::mbcs1 )
       result->contents()[i] = padCharacter;
     else
       break;
   }

   if ( startPos + fromReceiver <= this->length () )
   {
     IStringEnum::CharType typeOfFirstAfterSubstring =
       this->charType ( startPos + len );
     if ( typeOfFirstAfterSubstring > IStringEnum::mbcs1 )
       for ( unsigned i = 1; i < typeOfFirstAfterSubstring && i <= len; i++)
         result->contents()[len - i] = padCharacter;
   }

   return result;
}
