/*******************************************************************************
* FILE NAME: iexcept.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iexcept.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>

extern "C"
{
   #define INCL_WINERRORS
   #define INCL_DOSFILEMGR
   #define INCL_DOSMISC
   #define INCL_WINWINDOWMGR
   #include <os2.h>
  #include <string.h>
  #include <errno.h>
}

#include <iexcept.hpp>
#include <icconst.h>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IEXCEPT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IErrorInfo::IErrorInfo                                                       |
------------------------------------------------------------------------------*/
IErrorInfo::IErrorInfo ( )
{ }

/*------------------------------------------------------------------------------
| IErrorInfo::~IErrorInfo                                                      |
------------------------------------------------------------------------------*/
IErrorInfo::~IErrorInfo ( )
{ }

/*------------------------------------------------------------------------------
| IErrorInfo::throwError                                                       |
------------------------------------------------------------------------------*/
void IErrorInfo :: throwError ( const IExceptionLocation&  location,
                                ExceptionType              name,
                                IException::Severity       severity,
                                IException::ErrorCodeGroup errorGroup )
{
  switch (name)
  {
    case accessError:
    {
      IAccessError exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case deviceError:
    {
      IDeviceError exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case invalidParameter:
    {
      IInvalidParameter exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case invalidRequest:
    {
      IInvalidRequest exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case outOfSystemResource:
    {
      IOutOfSystemResource exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case outOfWindowResource:
    {
      IOutOfWindowResource exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case outOfMemory:
    {
      IOutOfMemory exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case resourceExhausted:
    {
      IResourceExhausted exc(text(), errorId(), severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
  } /* endswitch */
}

/*------------------------------------------------------------------------------
| IGUIErrorInfo::IGUIErrorInfo                                                 |
------------------------------------------------------------------------------*/
IGUIErrorInfo::IGUIErrorInfo ( const char *pGUIFunctionName )
{
  HAB habTemp = WinQueryAnchorBlock( HWND_DESKTOP );
  PERRINFO perri = WinGetErrorInfo( habTemp );
  this->perrinfoCl = (void*)perri;
  char *pErrText;

  if ((perri != 0)  &&  (perri->cDetailLevel > 0UL))
  {
    unsigned long *pul = (unsigned long*)((char*)perri+ (perri->offaoffszMsg));
    pErrText = ((char*)perri + *pul);
  }
  else
    pErrText = "No error text is available.";

  if (pGUIFunctionName)
  {
    this->pClFuncNameText = new char[strlen(pGUIFunctionName) +
                                     strlen(pErrText)+2];
    strcpy(this->pClFuncNameText, pGUIFunctionName);
    strcat(this->pClFuncNameText, ":");
    strcat(this->pClFuncNameText, pErrText);
  }
  else
  {
    this->pClFuncNameText = new char[strlen(pErrText)+1];
    strcpy(this->pClFuncNameText, pErrText);
  }
}

/*------------------------------------------------------------------------------
| IGUIErrorInfo::~IGUIErrorInfo                                                |
------------------------------------------------------------------------------*/
IGUIErrorInfo::~IGUIErrorInfo ( )
{
  if (this->perrinfoCl)
    WinFreeErrorInfo((PERRINFO)this->perrinfoCl);

  if (this->pClFuncNameText)
    delete [] this->pClFuncNameText;
}

/*------------------------------------------------------------------------------
| IGUIErrorInfo::text                                                          |
------------------------------------------------------------------------------*/
const char* IGUIErrorInfo::text ( ) const
{
  return( this->pClFuncNameText );
}

/*------------------------------------------------------------------------------
| IGUIErrorInfo::errorId                                                       |
------------------------------------------------------------------------------*/
unsigned long IGUIErrorInfo::errorId ( ) const
{
  PERRINFO perri = (PERRINFO)this->perrinfoCl;
  if (perri)
    return( (unsigned short)perri->idError );
  else
    return( 0 );
}

/*------------------------------------------------------------------------------
| IGUIErrorInfo::operator const char*                                          |
------------------------------------------------------------------------------*/
IGUIErrorInfo::operator const char* () const
{
  return( this->pClFuncNameText );
}

/*------------------------------------------------------------------------------
| IGUIErrorInfo::isAvailable                                                   |
------------------------------------------------------------------------------*/
Boolean IGUIErrorInfo::isAvailable ( ) const
{
  PERRINFO perri = (PERRINFO)this->perrinfoCl;
  if (perri == 0)
    return( 0 );
  else
  {
    if (perri->idError != 0)
      return( 1 );
    else
      return( 0 );
  }
}

/*------------------------------------------------------------------------------
| IGUIErrorInfo::throwGUIError                                                 |
------------------------------------------------------------------------------*/
void IGUIErrorInfo :: throwGUIError ( const char*               pFunctionName,
                                      const IExceptionLocation  &location,
                                      IErrorInfo::ExceptionType name,
                                      IException::Severity      severity )
{
  IGUIErrorInfo errInfo(pFunctionName);
  errInfo.throwError(location, name, severity, IException::presentationSystem );
}

/*------------------------------------------------------------------------------
| ISystemErrorInfo::ISystemErrorInfo                                           |
------------------------------------------------------------------------------*/
ISystemErrorInfo::ISystemErrorInfo ( unsigned long ulSystemErrorId,
                                     const char    *pSystemFunctionName )
                  : ulClErrorId(ulSystemErrorId),
                    bClError(true)
{
  char szText[512];
  unsigned long ulRc;
  unsigned long ulBootDrive;                               /* QSV_BOOT_DRIVE */
  DosQuerySysInfo( 5, 5, &ulBootDrive, sizeof(ulBootDrive) );
  unsigned char ucA = 'A';
  unsigned char ucBootDrive = (unsigned char)(ucA + (ulBootDrive - 1));
  char szFileName[80];
  szFileName[0] = ucBootDrive;
  szFileName[1] = '\0';
  strcat(szFileName, "\\OS2\\SYSTEM\\OSO001.MSG");
  unsigned long ulMessageLength;
  ulRc = DosGetMessage( 0, 0, (PCHAR)szText, 255,
                        ulSystemErrorId, (PSZ)szFileName,
                        &ulMessageLength );

  if (ulRc)
  {
    this->bClError = false;
    ulMessageLength = 27;
    strcpy(szText, "No error text is available.");
  }

  /********************************************************************/
  /* Remove CRLF control characters if found                          */
  /********************************************************************/
  if ((szText[ulMessageLength-2] == '\r')  &&
      (szText[ulMessageLength-1] == '\n'))
  {
    ulMessageLength = ulMessageLength - 2;
  }

  if (pSystemFunctionName)
  {
    this->pClErrorText = new char[ulMessageLength +
                                  strlen(pSystemFunctionName) + 2];
    strcpy(this->pClErrorText, pSystemFunctionName);
    strcat(this->pClErrorText, ":");
    strncat(this->pClErrorText, szText, ulMessageLength);
  }
  else
  {
    this->pClErrorText = new char[ulMessageLength + 1];
    strncpy(this->pClErrorText, szText, ulMessageLength);
    this->pClErrorText[ulMessageLength] = '\0';
  }
}

/*------------------------------------------------------------------------------
| ISystemErrorInfo::~ISystemErrorInfo                                          |
------------------------------------------------------------------------------*/
ISystemErrorInfo::~ISystemErrorInfo ( )
{
  if (this->pClErrorText)
    delete [] this->pClErrorText;
}

/*------------------------------------------------------------------------------
| ISystemErrorInfo::text                                                       |
------------------------------------------------------------------------------*/
const char* ISystemErrorInfo::text ( ) const
{
  return( (const char*)this->pClErrorText );
}

/*------------------------------------------------------------------------------
| ISystemErrorInfo::operator const char*                                       |
------------------------------------------------------------------------------*/
ISystemErrorInfo::operator const char* () const
{
  return( text() );
}

/*------------------------------------------------------------------------------
| ISystemErrorInfo::errorId                                                    |
------------------------------------------------------------------------------*/
unsigned long ISystemErrorInfo::errorId ( ) const
{
  return( this->ulClErrorId );
}

/*------------------------------------------------------------------------------
| ISystemErrorInfo::isAvailable                                                |
------------------------------------------------------------------------------*/
Boolean ISystemErrorInfo::isAvailable ( ) const
{
  if (!this->bClError)
    return( 0 );

  return( 1 );
}

/*------------------------------------------------------------------------------
| ISystemErrorInfo::throwSystemError                                           |
------------------------------------------------------------------------------*/
void ISystemErrorInfo ::
                 throwSystemError ( unsigned long             ulSystemErrorId,
                                    const char                *pFunctionName,
                                    const IExceptionLocation  &location,
                                    IErrorInfo::ExceptionType name,
                                    IException::Severity      severity )
{
  ISystemErrorInfo errInfo(ulSystemErrorId, pFunctionName);

  switch (name)
  {
    case accessError:
    {
      IAccessError exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case deviceError:
    {
      IDeviceError exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case invalidParameter:
    {
      IInvalidParameter exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case invalidRequest:
    {
      IInvalidRequest exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case outOfSystemResource:
    {
      IOutOfSystemResource exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case outOfWindowResource:
    {
      IOutOfWindowResource exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case outOfMemory:
    {
      IOutOfMemory exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case resourceExhausted:
    {
      IResourceExhausted exc(errInfo.text(), errInfo.errorId(), severity);
      exc.setErrorCodeGroup(IException::operatingSystem);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
  } /* endswitch */
}

/*------------------------------------------------------------------------------
| ICLibErrorInfo::~IClibErrorInfo                                              |
------------------------------------------------------------------------------*/
ICLibErrorInfo::~ICLibErrorInfo ( )
{
  if (this->pClErrorText)
    delete [] this->pClErrorText;
}

/*------------------------------------------------------------------------------
| ICLibErrorInfo::text                                                         |
------------------------------------------------------------------------------*/
const char* ICLibErrorInfo::text ( ) const
{
  return( (const char*)this->pClErrorText );
}

/*------------------------------------------------------------------------------
| ICLibErrorInfo::operator const char*                                         |
------------------------------------------------------------------------------*/
ICLibErrorInfo::operator const char* () const
{
  return( text() );
}

/*------------------------------------------------------------------------------
| ICLibErrorInfo::errorId                                                      |
------------------------------------------------------------------------------*/
unsigned long ICLibErrorInfo::errorId ( ) const
{
  return( this->ulClErrno );
}

/*------------------------------------------------------------------------------
| ICLibErrorInfo::isAvailable                                                  |
------------------------------------------------------------------------------*/
IBase::Boolean ICLibErrorInfo::isAvailable ( ) const
{
  return( this->bClError );
}

/*------------------------------------------------------------------------------
| ICLibErrorInfo::ICLibErrorInfo                                               |
|                                                                              |
|  The strerror C function is used to retrieve the error text for errno.       |
------------------------------------------------------------------------------*/
ICLibErrorInfo::ICLibErrorInfo ( const char *pCLibFunctionName )
{
  char *pErrorStr = strerror(errno);
  char noErrorText[] = "No Error text is available";

  this->ulClErrno = (unsigned long) errno;

  this->bClError = strlen(pErrorStr) ? true : false;

  if (pCLibFunctionName)
  {
    if (this->bClError)
    {
      this->pClErrorText = new char[strlen(pErrorStr) +
                                    strlen(pCLibFunctionName) + 2];
      strcpy(this->pClErrorText, pCLibFunctionName);
      strcat(this->pClErrorText, ":");
      strcat(this->pClErrorText, pErrorStr);
    }
    else
    {
      this->pClErrorText = new char[strlen(noErrorText) +
                                    strlen(pCLibFunctionName) + 2];
      strcpy(this->pClErrorText, pCLibFunctionName);
      strcat(this->pClErrorText, ":");
      strcat(this->pClErrorText, noErrorText);
    }
  }
  else
  {
    if (this->bClError)
    {
      this->pClErrorText = new char[strlen(pErrorStr) + 1];
      strcpy(this->pClErrorText, pErrorStr);
    }
    else
    {
      this->pClErrorText = new char[strlen(noErrorText) + 1];
      strcpy(this->pClErrorText, noErrorText);
    }
  }
}

/*------------------------------------------------------------------------------
| ICLibErrorInfo::throwCLibError                                               |
------------------------------------------------------------------------------*/
void ICLibErrorInfo::throwCLibError( const char                *pFunctionName,
                                     const IExceptionLocation  &location,
                                     IErrorInfo::ExceptionType name,
                                     IException::Severity      severity )
{
  ICLibErrorInfo clibError( pFunctionName );
  clibError.throwError( location, name, severity, IException::CLibrary );
}

/*------------------------------------------------------------------------------
| IExcept__assertParameter                                                     |
------------------------------------------------------------------------------*/
void IExcept__assertParameter ( const char               *pAssertExpression,
                                const IExceptionLocation &location )
{
  IInvalidParameter invParm(IBase::messageText(IC_ASSERTPARM,
                                               pAssertExpression),
                            IC_ASSERTPARM,
                            IException::recoverable);
  invParm.setErrorCodeGroup(IException::other);
  invParm.addLocation(location);
  invParm.logExceptionData();

  #if (INO_EXCEPTIONS_SUPPORT)
    invParm.terminate();
  #else
    throw invParm;
  #endif
}

/*------------------------------------------------------------------------------
| IExcept__assertState                                                         |
------------------------------------------------------------------------------*/
void IExcept__assertState ( const char               *pAssertExpression,
                            const IExceptionLocation &location )
{
  IInvalidRequest invReq(IBase::messageText(IC_ASSERTPARM, pAssertExpression),
                         IC_ASSERTPARM,
                         IException::recoverable);
  invReq.setErrorCodeGroup(IException::other);
  invReq.addLocation(location);
  invReq.logExceptionData();

  #if (INO_EXCEPTIONS_SUPPORT)
    invReq.terminate();
  #else
    throw invReq;
  #endif
}

/*------------------------------------------------------------------------------
| IExcept__throwLibraryError                                                   |
------------------------------------------------------------------------------*/
void IExcept__throwLibraryError ( unsigned long             ulMessageId,
                                  const IExceptionLocation  &location,
                                  IErrorInfo::ExceptionType name,
                                  IException::Severity      severity,
                                  const char                *pText )
{
  IMessageText errInfo = IBase::messageText(ulMessageId, pText);

  switch (name)
  {
    case IErrorInfo::accessError:
    {
      IAccessError exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IErrorInfo::deviceError:
    {
      IDeviceError exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IErrorInfo::invalidParameter:
    {
      IInvalidParameter exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IErrorInfo::invalidRequest:
    {
      IInvalidRequest exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IErrorInfo::outOfSystemResource:
    {
      IOutOfSystemResource exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IErrorInfo::outOfWindowResource:
    {
      IOutOfWindowResource exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IErrorInfo::outOfMemory:
    {
      IOutOfMemory exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IErrorInfo::resourceExhausted:
    {
      IResourceExhausted exc(errInfo.text(), ulMessageId, severity);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
  } /* endswitch */
}

/*------------------------------------------------------------------------------
| IExcept__throwError                                                          |
------------------------------------------------------------------------------*/
void IExcept__throwError ( unsigned long              messageId,
                           const IExceptionLocation&  location,
                           const char*                messageFile,
                           IException::ErrorCodeGroup errorGroup,
                           IErrorInfo::ExceptionType  name,
                           IException::Severity       severity,
                           const char*                substitutionText )
{
  IMessageText errInfo = IMessageText( messageId,
                                       messageFile,
                                       substitutionText );

  switch (name)
  {
    case IErrorInfo::accessError:
    {
      IAccessError exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IErrorInfo::deviceError:
    {
      IDeviceError exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IErrorInfo::invalidParameter:
    {
      IInvalidParameter exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IErrorInfo::invalidRequest:
    {
      IInvalidRequest exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IErrorInfo::outOfSystemResource:
    {
      IOutOfSystemResource exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IErrorInfo::outOfWindowResource:
    {
      IOutOfWindowResource exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IErrorInfo::outOfMemory:
    {
      IOutOfMemory exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
    case IErrorInfo::resourceExhausted:
    {
      IResourceExhausted exc(errInfo.text(), messageId, severity);
      exc.setErrorCodeGroup(errorGroup);
      exc.addLocation(location);
      exc.logExceptionData();
      #if (INO_EXCEPTIONS_SUPPORT)
        exc.terminate();
      #else
        throw exc;
      #endif
    }
  } /* endswitch */
}

