/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <iibstkss.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IIBSTKSS_CPP_
  #include <ipagetun.h>
#endif


IBoolean IBSTKeySortedSetImpl::
add (void const* element, IBSTKeySortedSetImplLnode*& cursor)
{ IBoolean rv;
  IBSTKeySortedSetImplLnode* lnode = newLnode (element);

  if (isEmpty ()) {
    ivRoot = lnode;
    ivHeight = 1;
    ivNoElements = 1;
    cursor = lnode;
    rv = True;
  }
  else {
    IBSTKeySortedSetImplNode* node;
    void* key;

    ITRY {
      key = newKey ();
    }
    IENDTRY

    ICATCH (IOutOfMemory exc) {
      deleteLnode (lnode);
      IRETHROW (exc);
    }
    IENDCATCH

    ITRY {
      if (addAid (ivRoot, lnode, node, key, cursor)) {
        //
        // element added to the tree
        //
        ivNoElements += 1;
        rv = True;
      }
      else {
        //
        // element already in tree
        //
        deleteLnode (lnode);
        rv = False;
      }
    }
    IENDTRY

    ICATCH (IOutOfMemory) {
      IASSERT (node != 0);

      //
      // Only thing we can do is to delete the node, it is not
      // referenced to any more.
      //
      ivNoElements -= removeAllSubtree (node);
      deleteKey (key);
    }
    IENDCATCH

    if (node != 0) {
      //
      // the root has splitted, create a new root
      //
      IBSTKeySortedSetImplInode* inode;

      ITRY {
        inode = newInode ();
      }
      IENDTRY

      ICATCH (IOutOfMemory exc) {
        ivNoElements -= removeAllSubtree (node);
        deleteKey (key);
        IRETHROW (exc);
      }
      IENDCATCH

      if (ivRoot->isLeaf ()) {
        IASSERT (node->isLeaf ());

        if (isKeyLessThanLnode
              (key, (IBSTKeySortedSetImplLnode*) ivRoot)) {
          //
          // added key is smaller than already present key
          //
          copyKeyFromLnode (key, (IBSTKeySortedSetImplLnode*) ivRoot);
          copyKey (inode->ivKeys, 0, key);
          inode->ivPtrs[0] = node;
          inode->ivPtrs[1] = ivRoot;
        }
        else {
          //
          // added key is largest in new node
          //
          copyKey (inode->ivKeys, 0, key);
          inode->ivPtrs[0] = ivRoot;
          inode->ivPtrs[1] = node;
        }
      }
      else {
        //
        // root is an interior node
        // this means that the new node is always larger
        // than the key added
        //
        copyKey (inode->ivKeys, 0, key);
        inode->ivPtrs[0] = ivRoot;
        inode->ivPtrs[1] = node;
      }

      inode->ivm = 2;
      ivRoot = inode;
      ivHeight += 1;
    }

    deleteKey (key);
  }

  return rv;
}

void IBSTKeySortedSetImpl::
removeAt (IBSTKeySortedSetImplLnode* const& cursor)
{
  if (ivRoot->isLeaf ()) {
    IASSERT (cursor == ivRoot);

    deleteLnode ((IBSTKeySortedSetImplLnode*) ivRoot);
    ivRoot = 0;
    ivHeight = 0;
  }
  else {
    //
    // the root points to an interior node
    //
    removeAid ((IBSTKeySortedSetImplInode*) ivRoot, cursor);

    if (((IBSTKeySortedSetImplInode*) ivRoot)->ivm < 2) {
      //
      // root has become obsolete
      // root must have at least two children
      //
      IBSTKeySortedSetImplNode* tmpNode =
        ((IBSTKeySortedSetImplInode*) ivRoot)->ivPtrs[0];
      deleteInode ((IBSTKeySortedSetImplInode*) ivRoot);
      ivHeight -= 1;
      ivRoot = tmpNode;
    }
  }

  ivNoElements -= 1;

  return;
}

INumber IBSTKeySortedSetImpl::
removeAll (void* predicate, void* env)
{ IBSTKeySortedSetImplLnode *current, *next;
  INumber removed = 0;
  for (setToFirst (current), setToFirst (next);
       current != 0;
       current = next) {
    setToNext (next);
    if (constantFunctionIteration(predicate, env, current)) {
      removeAt (current);
      removed++;
    }
  }
  return removed;
}

IBoolean IBSTKeySortedSetImpl::
setToFirst (IBSTKeySortedSetImplLnode*& cursor) const
{ IBoolean rv;

  if (ivRoot == 0) {
    cursor = 0;
    rv = False;
  }
  else {
    cursor = minimumLeaf (ivRoot);
    rv = True;
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
setToNext (IBSTKeySortedSetImplLnode*& cursor) const
{ IBoolean rv;

  if (ivRoot->isLeaf ()) {
    rv = False;
    cursor = 0;
  }
  else {
    rv = setToNextSubtree (cursor, (IBSTKeySortedSetImplInode*) ivRoot);
    if (! rv) {
      cursor = 0;
    }
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
allElementsDo (void* function, void* env)
{ if (ivRoot != 0)
    return allElementsDoSubtree (function, env, ivRoot);
  else
    return True;
}

IBoolean IBSTKeySortedSetImpl::
allElementsDo (void* iterator)
{ if (ivRoot != 0)
    return allElementsDoSubtree(iterator, ivRoot);
  else
    return True;
}

IBoolean IBSTKeySortedSetImpl::
allElementsDo (void* function, void* env) const
{ if (ivRoot != 0)
    return allElementsDoSubtree(function, env, ivRoot);
  else
    return True;
}

IBoolean IBSTKeySortedSetImpl::
allElementsDo (void* iterator) const
{ if (ivRoot != 0)
    return allElementsDoSubtree(iterator, ivRoot);
  else
    return True;
}

//
// IKeyCollection
//

IBoolean IBSTKeySortedSetImpl::
containsAllKeysFrom (IBSTKeySortedSetImpl const& tree) const
{ IBoolean rv;

  if (tree.ivRoot == 0) {
    rv = True;
  }
  else {
    IBSTKeySortedSetImplLnode* dummyCursor;

    rv = containsAllKeysFromSubtree (tree.ivRoot, dummyCursor);
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
containsElementWithKey (void const* key) const
{ IBoolean rv;
  IBSTKeySortedSetImplLnode* dummyCursor;

  if (ivRoot == 0) {
    rv = False;
  }
  else {
    rv = locateElementWithKeySubtree (key, dummyCursor, ivRoot);
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
locateElementWithKey (void const* key,
                      IBSTKeySortedSetImplLnode*& cursor) const
{ IBoolean rv;

  if (ivRoot == 0) {
    cursor = 0;
    rv = False;
  }
  else {
    rv = locateElementWithKeySubtree (key, cursor, ivRoot);
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
replaceElementWithKey (void const* element)
{ IBSTKeySortedSetImplLnode* dummyCursor;

  return replaceElementWithKey (element, dummyCursor);
}

IBoolean IBSTKeySortedSetImpl::
replaceElementWithKey (void const* element,
                       IBSTKeySortedSetImplLnode*& cursor)
{ IBoolean rv;

  if (locateElementWithKeyOfElement (element, cursor)) {
    replaceAt (cursor, element);
    rv = True;
  }
  else {
    rv = False;
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
addOrReplaceElementWithKey (void const* element)
{ IBSTKeySortedSetImplLnode* dummyCursor;

  return addOrReplaceElementWithKey (element, dummyCursor);
}

IBoolean IBSTKeySortedSetImpl::
addOrReplaceElementWithKey (void const* element,
                            IBSTKeySortedSetImplLnode*& cursor)
{ IBoolean rv;

  if (add (element, cursor)) {
    rv = True;
  }
  else {
    //
    // element with key already in tree, cursor points to it
    //
    replaceAt (cursor, element);
    rv = False;
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
removeElementWithKey (void const* key)
{ IBSTKeySortedSetImplLnode* cursor;
  IBoolean rv;

  if (locateElementWithKey (key, cursor)) {
    removeAt (cursor);
    rv = True;
  }
  else {
    rv = False;
  }

  return rv;
}

void const* IBSTKeySortedSetImpl::
elementWithKey (void const* key) const
{ IBSTKeySortedSetImplLnode* cursor;

  IBoolean b = locateElementWithKey (key, cursor);
  ICHECK (containsElementWithKey(key), INotContainsKeyException,
          IKeyNotContainedText);
  return elementAt (cursor);
}

void* IBSTKeySortedSetImpl::
elementWithKey (void const* key)
{ IBSTKeySortedSetImplLnode* cursor;

  IBoolean b = locateElementWithKey (key, cursor);
  ICHECK (containsElementWithKey(key), INotContainsKeyException,
          IKeyNotContainedText);
  return elementAt (cursor);
}

//
// IOrderedCollection
//

void IBSTKeySortedSetImpl::
removeFirst ()
{ IBSTKeySortedSetImplLnode* cursor;

  IBoolean b = setToFirst (cursor);
  IASSERT (b);
  removeAt (cursor);
}

void IBSTKeySortedSetImpl::
removeLast ()
{ IBSTKeySortedSetImplLnode* cursor;

  IBoolean b = setToLast (cursor);
  IASSERT (b);
  removeAt (cursor);
}

void IBSTKeySortedSetImpl::
removeAtPosition (IPosition position)
{ IBSTKeySortedSetImplLnode* cursor;
  setToPosition (position, cursor);
  removeAt (cursor);
}

void const* IBSTKeySortedSetImpl::
lastElement () const
{ IBSTKeySortedSetImplLnode* cursor;

  IBoolean b = setToLast (cursor);
  IASSERT (b);
  return elementAt (cursor);
}

void const* IBSTKeySortedSetImpl::
elementAtPosition (IPosition position) const
{ IBSTKeySortedSetImplLnode* cursor;

  setToPosition (position, cursor);
  return elementAt (cursor);
}

IBoolean IBSTKeySortedSetImpl::
setToLast (IBSTKeySortedSetImplLnode*& cursor) const
{ IBoolean rv;

  if (ivRoot == 0) {
    cursor = 0;
    rv = False;
  }
  else {
    cursor = maximumLeaf (ivRoot);
    rv = True;
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
setToPrevious (IBSTKeySortedSetImplLnode*& cursor) const
{ IBoolean rv;

  if (ivRoot->isLeaf ()) {
    cursor = 0;
    rv = False;
  }
  else {
    rv = setToPreviousSubtree
           (cursor, (IBSTKeySortedSetImplInode*) ivRoot);
    if (! rv) {
      cursor = 0;
    }
  }

  return rv;
}

void IBSTKeySortedSetImpl::
setToPosition (IPosition position,
               IBSTKeySortedSetImplLnode*& cursor) const
{ IPosition i;

  if (position - 1 < ivNoElements - position) {
    //
    // better to step forward
    //
    IBoolean b = setToFirst (cursor);
    IASSERT (b);
    i = 1;
    while (i < position) {
      IBoolean b = setToNext (cursor);
      IASSERT (b);
      i++;
    }
  }
  else {
    //
    // better to step backward
    //
    IBoolean b = setToLast (cursor);
    IASSERT (b);
    i = ivNoElements;
    while (i > position) {
      IBoolean b = setToPrevious (cursor);
      IASSERT (b);
      i--;
    }
  }
}

#ifndef __BIN_COMPAT_BASE_43__
IPosition IBSTKeySortedSetImpl::
position (IBSTKeySortedSetImplLnode* const& cursor) const
{ IBSTKeySortedSetImplLnode *tmpcursor;
  IPosition pos = 1;

  setToFirst(tmpcursor);
  while (tmpcursor != cursor) {
    setToNext (tmpcursor);
    pos++;
  }
  return pos;
}
#endif

IBoolean IBSTKeySortedSetImpl::
isFirst (IBSTKeySortedSetImplLnode* const& cursor) const
{ IBSTKeySortedSetImplLnode* tmpCursor;

  IBoolean b = setToFirst (tmpCursor);
  IASSERT (b);
  return cursor == tmpCursor;
}

IBoolean IBSTKeySortedSetImpl::
isLast (IBSTKeySortedSetImplLnode* const& cursor) const
{ IBSTKeySortedSetImplLnode* tmpCursor;

  IBoolean b = setToLast (tmpCursor);
  IASSERT (b);
  return cursor == tmpCursor;
}

//
// ISortedCollection
//

//
// for all collections
//

IBSTKeySortedSetImpl& IBSTKeySortedSetImpl::
operator = (IBSTKeySortedSetImpl const& tree)
{ if (this != &tree) {
    removeAll ();
    if (tree.ivRoot != 0) {
      ivRoot = copySubtree (tree.ivRoot);
      ivHeight = tree.ivHeight;
      ivNoElements = tree.ivNoElements;
    }
  }

  return (*this);
}

//
// for all ordered collections
//

long IBSTKeySortedSetImpl::
compare (IBSTKeySortedSetImpl const& tree, void *function) const
{ IBSTKeySortedSetImplLnode* c1;
  IBSTKeySortedSetImplLnode* c2;

  IBoolean next1 = setToFirst (c1);
  IBoolean next2 = tree.setToFirst (c2);

  long compare = 0;
  while (next1 && next2 && compare == 0) {
    compare = functionComparison(function, c1, c2);
    if (compare == 0) {
      next1 = setToNext (c1);
      next2 = tree.setToNext (c2);
    }
  }

  if (compare == 0 && (next1 || next2)) {
    //
    // we have to return a result based on the size of the collection
    //
    compare = (long) numberOfElements () - tree.numberOfElements ();
  }

  return compare;
}

//
// PRIVATE methods
//

IBoolean IBSTKeySortedSetImpl::
allElementsDoSubtree (void* function, void* env,
                      IBSTKeySortedSetImplNode* current)
{ IASSERT (current != 0);

  IBoolean rv = True;

  if (current->isLeaf ()) {
    rv = functionIteration (function, env, current);
  }
  else {
    IBSTKeySortedSetImplInode const* inode =
      (IBSTKeySortedSetImplInode const*) current;

    rv = True;
    for (IPosition i = 0; i < inode->ivm && rv; i++) {
      rv = allElementsDoSubtree (function, env, inode->ivPtrs[i]);
    }
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
allElementsDoSubtree (void* iterator, IBSTKeySortedSetImplNode* current)
{ IASSERT (current != 0);

  IBoolean rv = True;

  if (current->isLeaf ()) {
    rv = iteratorIteration (iterator, current);
  }
  else {
    IBSTKeySortedSetImplInode const* inode =
      (IBSTKeySortedSetImplInode const*) current;

    rv = True;
    for (IPosition i = 0; i < inode->ivm && rv; i++) {
      rv = allElementsDoSubtree (iterator, inode->ivPtrs[i]);
    }
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
allElementsDoSubtree (void* function, void* env,
                      IBSTKeySortedSetImplNode const* current) const
{ IASSERT (current != 0);

  IBoolean rv = True;

  if (current->isLeaf ()) {
    rv = constantFunctionIteration (function, env, current);
  }
  else {
    IBSTKeySortedSetImplInode const* inode =
      (IBSTKeySortedSetImplInode const*) current;

    rv = True;
    for (IPosition i = 0; i < inode->ivm && rv; i++) {
      rv = allElementsDoSubtree (function, env, inode->ivPtrs[i]);
    }
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
allElementsDoSubtree (void* iterator,
                      IBSTKeySortedSetImplNode const* current) const
{ IASSERT (current != 0);

  IBoolean rv = True;

  if (current->isLeaf ()) {
    rv = constantIteratorIteration (iterator, current);
  }
  else {
    IBSTKeySortedSetImplInode const* inode =
      (IBSTKeySortedSetImplInode const*) current;

    rv = True;
    for (IPosition i = 0; i < inode->ivm && rv; i++) {
      rv = allElementsDoSubtree (iterator, inode->ivPtrs[i]);
    }
  }

  return rv;
}

IPosition IBSTKeySortedSetImpl::
binarySearchInPage (IBSTKeySortedSetImplInode const* inode,
                    IBSTKeySortedSetImplLnode const* lnode) const
{ IPosition startIndex = 0;
  IPosition between;
  IPosition endIndex = inode->ivm - 1;

  while (startIndex < endIndex) {
    between = (startIndex + endIndex) / 2;
    if (isKeyGreaterThanLnode (inode->ivKeys, between, lnode)) {
      endIndex = between;
    }
    else {
      startIndex = between + 1;
    }
  }

  return endIndex;
}

IPosition IBSTKeySortedSetImpl::
binarySearchInPageWithKey (IBSTKeySortedSetImplInode const* inode,
                           void const* key) const
{ IPosition startIndex = 0;
  IPosition between;
  IPosition endIndex = inode->ivm - 1;

  while (startIndex < endIndex) {
    between = (startIndex + endIndex) / 2;
    if (isKeyGreaterThanKey (inode->ivKeys, between, key)) {
      endIndex = between;
    }
    else {
      startIndex = between + 1;
    }
  }

  return endIndex;
}

IPosition IBSTKeySortedSetImpl::
binarySearchInPageWithElement (IBSTKeySortedSetImplInode const* inode,
                               void const* element) const
{
  IPosition startIndex = 0;
  IPosition between;
  IPosition endIndex = inode->ivm - 1;

  while (startIndex < endIndex) {
    between = (startIndex + endIndex) / 2;
    if (isKeyGreaterThanElement (inode->ivKeys, between, element)) {
      endIndex = between;
    }
    else {
      startIndex = between + 1;
    }
  }

  return endIndex;
}

IBoolean IBSTKeySortedSetImpl::
setToNextSubtree (IBSTKeySortedSetImplLnode*& cursor,
                  IBSTKeySortedSetImplInode const* inode) const
{ IASSERT (! inode->isLeaf ());

  IBoolean rv;

  IPosition index = binarySearchInPage (inode, cursor);

  if (inode->ivPtrs[index] == cursor ||
      ! setToNextSubtree
          (cursor,
           (IBSTKeySortedSetImplInode const*) inode->ivPtrs[index])) {
    //
    // try to let cursor point to first element of next inode
    //
    if (index < inode->ivm - 1) {
      //
      // set to next lnode
      //
      cursor = minimumLeaf (inode->ivPtrs[index+1]);
      rv = True;
    }
    else {
        rv = False;
    }
  }
  else {
    //
    // recursive call to setToNextSubtree must have returned True in
    // order ro come here, propagate this return value!
    //
    rv = True;
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
setToPreviousSubtree (IBSTKeySortedSetImplLnode*& cursor,
                      IBSTKeySortedSetImplInode const* inode) const
{ IASSERT (! inode->isLeaf ());

  IBoolean rv;

  IPosition index = binarySearchInPage (inode, cursor);

  if (inode->ivPtrs[index] == cursor ||
      ! setToPreviousSubtree
          (cursor,
           (IBSTKeySortedSetImplInode const*) inode->ivPtrs[index])) {
    //
    // try to let cursor point to previous element of inode
    //
    if (index > 0) {
      //
      // set to previous lnode
      //
      cursor = maximumLeaf (inode->ivPtrs[index-1]);
      rv = True;
    }
    else {
      rv = False;
    }
  }
  else {
    //
    // recursive call to setToPreviousSubtree must have returned True in
    // order ro come here, propagate this return value!
    //
    rv = True;
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
addAid (IBSTKeySortedSetImplNode* page, // starting page for search (IN)
        IBSTKeySortedSetImplLnode* lnode,// element to add (IN)
        IBSTKeySortedSetImplNode*& node, // pointer to a new node
                                         // created here (OUT)
        void* key,      // keyest keyvalue of interior node (OUT)
        IBSTKeySortedSetImplLnode*& cursor) // to point to added
                                            // or found element (OUT)
{ IBoolean rv;

  if (page->isLeaf ()) {
    IBSTKeySortedSetImplLnode*
      lnPage = (IBSTKeySortedSetImplLnode*) page;

    if (isLnodeKeyEqualTo (lnPage, lnode)) {
      //
      // element already in tree
      //
      cursor = lnPage;
      node = 0;
      rv = False;
    }
    else {
      cursor = lnode;
      node = lnode;
      copyKeyFromLnode (key, lnode);
      rv = True;
    }
  }
  else {
    IBSTKeySortedSetImplInode*
      inPage = (IBSTKeySortedSetImplInode*) page;

    void* tmpKey;

    IPosition index = binarySearchInPage (inPage, lnode);
    rv = addAid (inPage->ivPtrs[index], lnode, node, key, cursor);

    if (node != 0) {
      //
      // node has to be inserted in the page
      //
      IPosition i;

      tmpKey = newKey ();

      if (inPage->ivm < ivTreeOrder) {
        //
        // node can be inserted in this page, move elements
        //
        for (i = inPage->ivm - 1; i >= index + 1; i--) {
          inPage->ivPtrs[i+1] = inPage->ivPtrs[i];
          copyKey (inPage->ivKeys, i, inPage->ivKeys, i-1);
        }

        //
        // room available in key[index] and ptr[index+1]
        // if index is 0 keys have to be compared
        //
        if (index == 0 || inPage->ivPtrs[index]->isLeaf ())
          copyKeyFromLnode (tmpKey,minimumLeaf (inPage->ivPtrs[index]));

        if ((index == 0 || inPage->ivPtrs[index]->isLeaf ()) &&
            isKeyLessThan (key, tmpKey)) {
          inPage->ivPtrs[index+1] = inPage->ivPtrs[index];
          inPage->ivPtrs[index] = node;
          copyKey (inPage->ivKeys, index, tmpKey);
        }
        else {
          inPage->ivPtrs[index+1] = node;
          copyKey (inPage->ivKeys, index, key);
        }

        inPage->ivm += 1;
        node = 0;
      }
      else {
        //
        // overflow
        // redistribute elements between current and new page
        // determine a new key value
        // first create temp storage
        // this storage is allocated as an internal node
        // using its overloaded new operator
        // note that this internal node provides storage for one
        // extra pointer and one extra key
        // we also need some temp storage for a key and for a node
        // pointer. This is because we want to insert a key and a
        // pointer and we already have a full page (inPage)
        // Just allocating an interior node as temporary storage
        // is therefore insufficient.
        // The key storage is still available as tmpKey
        // So only create a tmp node ptr
        //
        IBSTKeySortedSetImplInode* inode = newInode ();
        IBSTKeySortedSetImplInode* tmpStore;

        ITRY {
          tmpStore = newInode ();
        }
        IENDTRY

        ICATCH (IOutOfMemory exc) {
          deleteInode (inode);
          IRETHROW (exc);
        }
        IENDCATCH

        IBSTKeySortedSetImplNode* tmpNode;

        //
        // put the first part of inPage in the temporary storage
        //
        for (i = 0; i < index; i++) {
          copyKey (tmpStore->ivKeys, i, inPage->ivKeys, i);
          tmpStore->ivPtrs[i] = inPage->ivPtrs[i];
        }

        if (index == 0 || inPage->ivPtrs[index]->isLeaf ())
          copyKeyFromLnode (tmpKey,minimumLeaf (inPage->ivPtrs[index]));

        if ((index == 0 || inPage->ivPtrs[index]->isLeaf ()) &&
            isKeyLessThan (key, tmpKey)) {
          tmpStore->ivPtrs[index] = node;
          if (index == ivTreeOrder - 1) {
            //
            // remember node and key, they don't fit in tmpstore
            //
            tmpNode = inPage->ivPtrs[index];
            //copyKey (tmpKey, tmpKey);
          }
          else {
            tmpStore->ivPtrs[index+1] = inPage->ivPtrs[index];
            copyKey (tmpStore->ivKeys, index, tmpKey);
          }
        }
        else {
          tmpStore->ivPtrs[index] = inPage->ivPtrs[index];
          if (index == ivTreeOrder - 1) {
            //
            // remember node and key, they don't fit in tmpstore
            //
            tmpNode = node;
            copyKey (tmpKey, key);
          }
          else {
            tmpStore->ivPtrs[index+1] = node;
            copyKey (tmpStore->ivKeys, index, key);
          }
        }

        //
        // put the other part of inPage in the temporary storage
        //
        for (i = index + 1; i < ivTreeOrder - 1; i++) {
          copyKey (tmpStore->ivKeys, i, inPage->ivKeys, i-1);
          tmpStore->ivPtrs[i+1] = inPage->ivPtrs[i];
        }

        //
        // Store the last key and the last pointer of inPage
        // in the temp pointer and the temp key, because
        // they do not fit in tmpStore if this is not already done
        //
        if (index != ivTreeOrder - 1) {
          tmpNode = inPage->ivPtrs[ivTreeOrder-1];
          copyKey (tmpKey, inPage->ivKeys, ivTreeOrder-2);
        }

        //
        // now determine new key, and split
        //
        IPosition keyindex = ((ivTreeOrder + 1) / 2) - 1;

        //
        // Put the first half of the temp storage back in inPage
        //
        for (i = 0 ; i < keyindex; i++) {
          copyKey (inPage->ivKeys, i, tmpStore->ivKeys, i);
          inPage->ivPtrs[i] = tmpStore->ivPtrs[i];
        }
        inPage->ivPtrs[keyindex] = tmpStore->ivPtrs[keyindex];
        inPage->ivm = keyindex + 1;

        //
        // Store the key that will be used in the previous level
        // of recursion to insert in the parent inode
        //
        copyKey (key, tmpStore->ivKeys, keyindex);

        //
        // Put the latter half of the temp storage in inode
        //
        for (i = keyindex + 1; i < ivTreeOrder - 1; i++) {
          copyKey (inode->ivKeys, i-keyindex-1, tmpStore->ivKeys, i);
          inode->ivPtrs[i-keyindex-1] = tmpStore->ivPtrs[i];
        }
        copyKey (inode->ivKeys, ivTreeOrder-keyindex-2, tmpKey);
        inode->ivPtrs[ivTreeOrder-keyindex-2] =
                                       tmpStore->ivPtrs[ivTreeOrder-1];
        inode->ivPtrs[ivTreeOrder-keyindex-1] = tmpNode;
        inode->ivm = ivTreeOrder-keyindex;

        node = inode;

        //
        // remove the temporary storage
        //
        deleteInode (tmpStore);
      }

      //
      // remove temporary key
      //
      deleteKey (tmpKey);
    }
  }

  return rv;
}

IBSTKeySortedSetImplLnode* IBSTKeySortedSetImpl::
minimumLeaf (IBSTKeySortedSetImplNode* node) const
{ IASSERT (node != 0);

  while (! node->isLeaf ()) {
    node = ((IBSTKeySortedSetImplInode*) node)->ivPtrs[0];
  }

  return (IBSTKeySortedSetImplLnode*) node;
}

IBSTKeySortedSetImplLnode* IBSTKeySortedSetImpl::
maximumLeaf (IBSTKeySortedSetImplNode* node) const
{ IASSERT (node != 0);

  while (! node->isLeaf ()) {
    node = ((IBSTKeySortedSetImplInode*) node)->
             ivPtrs[((IBSTKeySortedSetImplInode*) node)->ivm - 1];
  }

  return (IBSTKeySortedSetImplLnode*) node;
}

INumber IBSTKeySortedSetImpl::
removeAllSubtree (IBSTKeySortedSetImplNode* node)
{ IASSERT (node != 0);

  INumber n;

  if (node->isLeaf ()) {
    deleteLnode ((IBSTKeySortedSetImplLnode*) node);
    n = 1;
  }
  else {
    IBSTKeySortedSetImplInode*
      inode = (IBSTKeySortedSetImplInode*) node;

    n = 0;
    for (IPosition i = 0; i < inode->ivm; i++) {
      n += removeAllSubtree (inode->ivPtrs[i]);
    }

    deleteInode (inode);
  }

  return n;
}

void IBSTKeySortedSetImpl::
addAllFromSubtree (IBSTKeySortedSetImplNode const* node,
                   IBSTKeySortedSetImplLnode*& dummyCursor)
{ IASSERT (node != 0);

  if (node->isLeaf ()) {
    add (getLnodeElement ((IBSTKeySortedSetImplLnode const*) node),
         dummyCursor);
  }
  else {
    IBSTKeySortedSetImplInode const* inode =
      (IBSTKeySortedSetImplInode const*) node;

    for (IPosition i = 0; i < inode->ivm; i++) {
    addAllFromSubtree (inode->ivPtrs[i], dummyCursor);
    }
  }

  return;
}

void IBSTKeySortedSetImpl::
removeAid (IBSTKeySortedSetImplInode* inode,
           IBSTKeySortedSetImplLnode* cursor)
{ IPosition index = binarySearchInPage (inode, cursor);

  if (inode->ivPtrs[index] == cursor) {
    deleteLnode ((IBSTKeySortedSetImplLnode*) inode->ivPtrs[index]);

    //
    // shift elements to the left
    //
    for (IPosition i = index; i < inode->ivm - 1; i++) {
      inode->ivPtrs[i] = inode->ivPtrs[i+1];
    }

    for (i = index; i < inode->ivm - 2; i++) {
      copyKey (inode->ivKeys, i, inode->ivKeys, i+1);
    }

    inode->ivm -= 1;
  }
  else {
    //
    // node is at level two or higher
    //

    IASSERT (! inode->ivPtrs[index]->isLeaf ());

    removeAid
      ((IBSTKeySortedSetImplInode*) inode->ivPtrs[index], cursor);

    if (isUnderflown
         ((IBSTKeySortedSetImplInode*) inode->ivPtrs[index])) {
      handleUnderflow
        (inode,
          (IBSTKeySortedSetImplInode*) inode->ivPtrs[index], index);
    }
  }

  return;
}

void IBSTKeySortedSetImpl::
handleUnderflow (IBSTKeySortedSetImplInode* parentNode,
                 IBSTKeySortedSetImplInode* childNode,
                 IPosition indexOfChild)
{
  IBSTKeySortedSetImplInode* sibling;

  if (indexOfChild > 0) {
    //
    // it has a left sibling
    //
    sibling =
      (IBSTKeySortedSetImplInode*) parentNode->ivPtrs[indexOfChild-1];

    if (sibling->ivm == (ivTreeOrder + 1) / 2) {
      //
      // childnode can be moved into sibling
      //
      joinInodes (sibling, childNode, parentNode, indexOfChild - 1);
      deleteInode (childNode);
    }
    else {
      //
      // sibling has > [m/2] sons, move some elements of sibling
      // in child, determine the number of movements
      //
      INumber noMoves;
      IPosition i;

      noMoves = ((sibling->ivm + childNode->ivm) / 2) - childNode->ivm;
      IASSERT (noMoves > 0);

      //
      // make room in childnode
      //
      for (i = childNode->ivm - 1; i > 0; i--) {
        copyKey (childNode->ivKeys, i+noMoves-1,
                        childNode->ivKeys, i-1);
        childNode->ivPtrs[i+noMoves] = childNode->ivPtrs[i];
      }
      childNode->ivPtrs[noMoves] = childNode->ivPtrs[0];

      //
      // adjust parentnode
      //
      copyKey (childNode->ivKeys, noMoves-1,
                      parentNode->ivKeys, indexOfChild-1);
      copyKey (parentNode->ivKeys, indexOfChild-1,
                      sibling->ivKeys, sibling->ivm-noMoves-1);

      //
      // now move elements of sibling to childnode
      //
      childNode->ivPtrs[0] = sibling->ivPtrs[sibling->ivm-noMoves];
      for (i = 1; i < noMoves; i++) {
        childNode->ivPtrs[i] = sibling->ivPtrs[sibling->ivm-noMoves+i];
        copyKey (childNode->ivKeys, i-1,
                        sibling->ivKeys, sibling->ivm-noMoves+i-1);
      }

      childNode->ivm += noMoves;
      sibling->ivm -= noMoves;
    }
  }
  else {
    //
    // we must look at the right sibling
    //
    sibling =
      (IBSTKeySortedSetImplInode*) parentNode->ivPtrs[indexOfChild+1];
    if (sibling->ivm == (ivTreeOrder + 1) / 2) {
      //
      // move sibling into childnode
      //
      joinInodes (childNode, sibling, parentNode, indexOfChild);
      deleteInode (sibling);
    }
    else {
      //
      // sibling has > [m/2] sons, move some elements of sibling
      // in child, determine the number of movements
      //
      INumber noMoves;
      IPosition i;

      noMoves = ((sibling->ivm + childNode->ivm) / 2) - childNode->ivm;
      IASSERT (noMoves > 0);

      //
      // move elements of sibling to childnode
      //
      for (i = 0; i < noMoves - 1; i++) {
        copyKey (childNode->ivKeys, childNode->ivm+i,
                        sibling->ivKeys, i);
        childNode->ivPtrs[childNode->ivm+i] = sibling->ivPtrs[i];
      }
      childNode->ivPtrs[childNode->ivm+noMoves-1] =
                                      sibling->ivPtrs[noMoves-1];

      //
      // adjust parentnode
      //
      copyKey (childNode->ivKeys, childNode->ivm-1,
                      parentNode->ivKeys, indexOfChild);
      copyKey (parentNode->ivKeys, indexOfChild,
                      sibling->ivKeys, noMoves-1);

      //
      // rearrange sibling node
      //
      for (i = 0; i < sibling->ivm - noMoves - 1; i++) {
        copyKey (sibling->ivKeys, i, sibling->ivKeys, i+noMoves);
        sibling->ivPtrs[i] = sibling->ivPtrs[i+noMoves];
      }
      sibling->ivPtrs[sibling->ivm-noMoves-1] =
                                      sibling->ivPtrs[sibling->ivm-1];

      childNode->ivm += noMoves;
      sibling->ivm -= noMoves;
    }
  }

  return;
}

void IBSTKeySortedSetImpl::
joinInodes (IBSTKeySortedSetImplInode* leftInode,
            IBSTKeySortedSetImplInode* rightInode,
            IBSTKeySortedSetImplInode* parentInode,
            IPosition indexOfLeftInode)
{ IPosition i;

  copyKey (leftInode->ivKeys, leftInode->ivm-1,
                  parentInode->ivKeys, indexOfLeftInode);
  leftInode->ivPtrs[leftInode->ivm] = rightInode->ivPtrs[0];
  leftInode->ivm += 1;

  for (i = 0; i < rightInode->ivm - 1; i++) {
    copyKey (leftInode->ivKeys, leftInode->ivm-1,
                    rightInode->ivKeys, i);
    leftInode->ivPtrs[leftInode->ivm] = rightInode->ivPtrs[i+1];
    leftInode->ivm += 1;
  }

  //
  // adjust parent node
  //
  for (i = indexOfLeftInode; i < parentInode->ivm - 2; i++) {
    copyKey (parentInode->ivKeys, i,
                    parentInode->ivKeys, i+1);
    parentInode->ivPtrs[i+1] = parentInode->ivPtrs[i+2];
  }
  parentInode->ivm -= 1;

  return;
}

IBSTKeySortedSetImplNode* IBSTKeySortedSetImpl::
copySubtree (IBSTKeySortedSetImplNode const* node)
{ IASSERT (node != 0);

  IBSTKeySortedSetImplNode* newNode;

  if (node->isLeaf ()) {
    newNode = newLnodeFromLnode (node);
  }
  else {
    IBSTKeySortedSetImplInode const* inode =
      (IBSTKeySortedSetImplInode const*) node;
    IBSTKeySortedSetImplInode* newInode = this->newInode (inode);

    for (IPosition i = 0; i < inode->ivm; i++) {
      ITRY {
        newInode->ivPtrs[i] = copySubtree (inode->ivPtrs[i]);
      }
      IENDTRY

      ICATCH (IOutOfMemory exc) {
        newInode->ivm = i;
        removeAllSubtree (newInode);
        IRETHROW (exc);
      }
      IENDCATCH
    }

    newNode = newInode;
  }

  return newNode;
}

IBoolean IBSTKeySortedSetImpl::
locateWithKeySubtree (void const* key,
                      IBSTKeySortedSetImplLnode*& cursor,
                      IBSTKeySortedSetImplNode* node) const
{ IBoolean rv;

  if (node->isLeaf ()) {
    IBSTKeySortedSetImplLnode*
      lnode = (IBSTKeySortedSetImplLnode*) node;

    if (isLnodeKeyEqualToKey (lnode, key)) {
      cursor = lnode;
      rv = True;
    }
    else {
      cursor = 0;
      rv = False;
    }
  }
  else {
    IBSTKeySortedSetImplInode*
      inode = (IBSTKeySortedSetImplInode*) node;

    IPosition index = binarySearchInPageWithKey (inode, key);

    rv = locateWithKeySubtree (key, cursor, inode->ivPtrs[index]);
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
containsAllKeysFromSubtree
        (IBSTKeySortedSetImplNode const* node,
         IBSTKeySortedSetImplLnode*& dummyCursor) const
{ IBoolean rv;

  if (node->isLeaf ()) {
    rv = locateElementWithKey ((IBSTKeySortedSetImplLnode const*) node,
                               dummyCursor);
  }
  else {
    IBSTKeySortedSetImplInode const* inode =
      (IBSTKeySortedSetImplInode const*) node;

    rv = True;
    for (IPosition i = 0; i < inode->ivm && rv; i++) {
      rv = containsAllKeysFromSubtree (inode->ivPtrs[i], dummyCursor);
    }
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
locateElementWithKeySubtree (void const* key,
                             IBSTKeySortedSetImplLnode*& cursor,
                             IBSTKeySortedSetImplNode* node) const
{ IBoolean rv;

  if (node->isLeaf ()) {
    IBSTKeySortedSetImplLnode*
      lnode = (IBSTKeySortedSetImplLnode*) node;

    if (isLnodeKeyEqualToKey (lnode, key)) {
      cursor = lnode;
      rv = True;
    }
    else {
      cursor = 0;
      rv = False;
    }
  }
  else {
    IBSTKeySortedSetImplInode*
      inode = (IBSTKeySortedSetImplInode*) node;

    IPosition index = binarySearchInPageWithKey (inode, key);

    rv = locateElementWithKeySubtree
           (key, cursor, inode->ivPtrs[index]);
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
locateElementWithKeyOfElement (void const* element,
                               IBSTKeySortedSetImplLnode*& cursor) const
{ IBoolean rv;

  if (ivRoot == 0) {
    cursor = 0;
    rv = False;
  }
  else {
    rv = locateElementWithKeyOfElementSubtree (element, cursor, ivRoot);
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
locateElementWithKeyOfElementSubtree
        (void const* element,
         IBSTKeySortedSetImplLnode*& cursor,
         IBSTKeySortedSetImplNode* node) const
{ IBoolean rv;

  if (node->isLeaf ()) {
    IBSTKeySortedSetImplLnode*
      lnode = (IBSTKeySortedSetImplLnode*) node;

    if (isLnodeKeyEqualToKeyOfElement (lnode, element)) {
      cursor = lnode;
      rv = True;
    }
    else {
      cursor = 0;
      rv = False;
    }
  }
  else {
    IBSTKeySortedSetImplInode*
      inode = (IBSTKeySortedSetImplInode*) node;

    IPosition index = binarySearchInPageWithElement (inode, element);

    rv = locateElementWithKeyOfElementSubtree (element,
                                              cursor,
                                              inode->ivPtrs[index]);
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
containsElementWithKey (IBSTKeySortedSetImplLnode const* lnode) const
{ IBSTKeySortedSetImplLnode* dummyCursor;

  return locateElementWithKey (lnode, dummyCursor);
}

IBoolean IBSTKeySortedSetImpl::
locateElementWithKey (IBSTKeySortedSetImplLnode const* lnode,
                      IBSTKeySortedSetImplLnode*& cursor) const
{ IBoolean rv;

  if (ivRoot == 0) {
    cursor = 0;
    rv = False;
  }
  else {
    rv = locateElementWithKeySubtree (lnode, cursor, ivRoot);
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
locateElementWithKeySubtree (IBSTKeySortedSetImplLnode const* lnode,
                             IBSTKeySortedSetImplLnode*& cursor,
                             IBSTKeySortedSetImplNode* node) const
{ IBoolean rv;

  if (node->isLeaf ()) {
    if (isLnodeKeyEqualTo
          (((IBSTKeySortedSetImplLnode*) node), lnode)) {
      cursor = (IBSTKeySortedSetImplLnode*) node;
      rv = True;
    }
    else {
      cursor = 0;
      rv = False;
    }
  }
  else {
    IBSTKeySortedSetImplInode*
      inode = (IBSTKeySortedSetImplInode*) node;

    IPosition index = binarySearchInPage (inode, lnode);

    rv = locateElementWithKeySubtree
           (lnode, cursor, inode->ivPtrs[index]);
  }

  return rv;
}

IBoolean IBSTKeySortedSetImpl::
checkNode (IBSTKeySortedSetImplLnode const* node,
           IBSTKeySortedSetImplNode const* current) const
{ if (current->isLeaf ()) {
    return node == current;
  }
  else {
    IBSTKeySortedSetImplInode const* inode =
      (IBSTKeySortedSetImplInode const*) current;

    for (IPosition i = 0; i < inode->ivm; i++) {
      if (checkNode (node, inode->ivPtrs[i]))
        return True;
    }
    return False;
  }
}

IBoolean IBSTKeySortedSetImpl::
checkNode (IBSTKeySortedSetImplLnode const* node) const
{ if (node == 0)
    return True;
  else if (ivRoot != 0)
    return checkNode (node, ivRoot);
  else
    return False;
}

IBoolean IBSTKeySortedSetImpl::
isConsistent () const
{ return True;
}
