/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <iihshkb.h>
#include <ibexcept.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IIHSHKB_CPP_
  #include <ipagetun.h>
#endif


void
IHashKeyBagImpl::destruct ()
{ removeBlock (ivTable);
  removeBlock (ivCollList);
}

IBoolean
IHashKeyBagImpl::add (void const* element,
                     unsigned long hashvalue,
                     Cursor& cursor) {
  IASSERT (hashvalue < ivNoEntries);

  IHashKeyBagImplNode* node = newNode(element);

  if (ivTable[hashvalue] != 0)
       {
       ivCollList[hashvalue]++;
       }

   //
   // link element in bucket
   //
  node->ivNext = ivTable[hashvalue];
  ivTable[hashvalue] = node;

  ivNoElements++;

   //
   // Let cursor point to newly added element
   //
  cursor.ivEntryNumber = hashvalue;
  cursor.ivNode = node;

  if (ivNoEntries * 2 < ivNoElements)
    resize (node, cursor.ivEntryNumber);

  return True;
}

IBoolean
IHashKeyBagImpl::add (void const* element, unsigned long hashvalue) {
  IASSERT (hashvalue < ivNoEntries);

  IHashKeyBagImplNode* node = newNode(element);

  if (ivTable[hashvalue] != 0)
       {
       ivCollList[hashvalue]++;
       }

   //
   // link element in bucket
   //
  node->ivNext = ivTable[hashvalue];
  ivTable[hashvalue] = node;

  ivNoElements++;

  if (ivNoEntries * 2 < ivNoElements) {
    INumber dummy;
    resize (node, dummy);
  }

  return True;
}

void
IHashKeyBagImpl::addAllFrom (
  IHashKeyBagImpl const& hashtable
)
{
  Cursor c;

  for (unsigned long i = 0; i < hashtable.ivNoEntries; i++)
       {
       IHashKeyBagImplNode* tmp = hashtable.ivTable[i];
       while (tmp != 0)
           {
           //
           // New element is added at the start of the bucket.
           //
           add(hashtable.elementAt (tmp),
               getHashvalue(keyAt (tmp), ivNoEntries),
               c);
           tmp = tmp->ivNext;
           }
       }

  return;
}

void const*
IHashKeyBagImpl::elementAt (Cursor const& c)
const
{
  IASSERT (c.ivNode != 0);

  return elementAt (c.ivNode);
}

void*
IHashKeyBagImpl::elementAt (Cursor const& c)
{
  IASSERT (c.ivNode != 0);

  return elementAt (c.ivNode);
}

void IHashKeyBagImpl::
operator= (IHashKeyBagImpl const& hashtable)
{ if (this != & hashtable) {
    removeAll();
    copyHashtable(hashtable);
  }
}

void const*
IHashKeyBagImpl::anyElement ()
const
{
  Cursor cursor;

  setToFirst(cursor);
  IASSERT (cursor.ivNode != 0);

  return elementAt (cursor.ivNode);
}

void
IHashKeyBagImpl::removeAt (
  Cursor const& cursor // cursor to element to be removed (IN)
)
{
  IASSERT (cursor.ivNode != 0);

  if (cursor.ivNode == ivTable[cursor.ivEntryNumber])
       {
       //
       // remove first element in a bucket.
       //
       ivTable[cursor.ivEntryNumber] = cursor.ivNode->ivNext;
       }
  else
       {
       //
       // remove element somewhere in bucket, find previous
       //

       IHashKeyBagImplNode* prev = ivTable[cursor.ivEntryNumber];
       while (prev->ivNext != cursor.ivNode)
           {
           //
           // find previous of element
           //
           prev = prev->ivNext;
           IASSERT (prev != 0);
           }

       //
       // link next of previous to next of element to remove
       //
       prev->ivNext = cursor.ivNode->ivNext;
       }

  deleteNode(cursor.ivNode);
  ivNoElements--;

  if (ivCollList[cursor.ivEntryNumber] > 0)
       {
       ivCollList[cursor.ivEntryNumber]--;
       }

  return;
}

INumber
IHashKeyBagImpl::removeAll (void* predicate, void* env) {
  Cursor c, next;
  INumber removed = 0;
  for (setToFirst (c), setToFirst (next); c.ivNode != 0; c = next) {
    setToNext (next);
    if (constantFunctionIteration(predicate, env, c.ivNode)) {
      removeAt (c);
      removed++;
    }
  }
  return removed;
}

void
IHashKeyBagImpl::replaceAt (
  Cursor const& cursor, // points to element to be replaced (IN)
  void const* element // new element (IN)
)
{
  IASSERT (cursor.ivNode != 0);

  copyFrom (cursor.ivNode, element);

  return;
}

void
IHashKeyBagImpl::removeAll ()
{
  for (unsigned long i = 0; i < ivNoEntries; i++)
       {
       IHashKeyBagImplNode* tmp1 = ivTable[i];
       while (tmp1 != 0)
           {
           IHashKeyBagImplNode* tmp2 = tmp1->ivNext;
           deleteNode(tmp1);
           tmp1 = tmp2;
           }
       ivTable[i] = 0;
       ivCollList[i] = 0;
       }
  ivNoElements = 0;

  return;
}


IBoolean
IHashKeyBagImpl::isBounded ()
const
{
  return False;
}

unsigned long
IHashKeyBagImpl::maxNumberOfElements ()
const
{
  IASSERT (False);

  return 0;
}

IBoolean
IHashKeyBagImpl::isFull () const {
  return False;
}

IBoolean IHashKeyBagImpl::setToFirst (Cursor& cursor) const {
  cursor.ivEntryNumber = 0;

  if (isEmpty())
       {
       //
       // setToFirst will return very fast on an empty hashtable
       //
       cursor.ivNode = 0;
       }
  else
       {
       //
       // find first non-empty bucket
       //
       while (ivTable[cursor.ivEntryNumber] == 0 &&
              cursor.ivEntryNumber < ivNoEntries - 1)
           {
           cursor.ivEntryNumber++;
           }
       cursor.ivNode = ivTable[cursor.ivEntryNumber];
       }

  return (cursor.ivNode != 0);
}

IBoolean
IHashKeyBagImpl::setToNext (Cursor& cursor) const {
  IASSERT (cursor.ivNode != 0);

  cursor.ivNode = cursor.ivNode->ivNext;
  if (cursor.ivNode == 0) {
    //
    // end of the bucket,
    // try to find new non-empty bucket
    //
    while (cursor.ivEntryNumber < ivNoEntries - 1) {
      cursor.ivEntryNumber++;
      if (ivTable[cursor.ivEntryNumber] != 0) {
        cursor.ivNode = ivTable[cursor.ivEntryNumber];
        break;
      }
    }
  }

  return (cursor.ivNode != 0);
}

IBoolean
IHashKeyBagImpl::allElementsDo (void* function, void* env) {
  Cursor c;

  if (setToFirst(c)) {
    while (functionIteration(function, env, c.ivNode) &&
           setToNext(c)) {
    }
  }

  //
  // return True if all elements have been processed,
  // than the cursor is null
  //
  return c.ivNode == 0;
}

IBoolean
IHashKeyBagImpl::allElementsDo (void* iterator) {
  Cursor c;

  if (setToFirst(c)) {
    while (iteratorIteration(iterator,c.ivNode) &&
           setToNext(c)) {
    }
  }

  //
  // return True if all elements have been processed,
  // than the cursor is null
  //
  return c.ivNode == 0;
}

IBoolean
IHashKeyBagImpl::allElementsDo (void* function, void* env) const {
  Cursor c;

  if (setToFirst(c)) {
    while (constantFunctionIteration(function, env, c.ivNode) &&
           setToNext(c)) {
    }
  }

  //
  // return True if all elements have been processed,
  // than the cursor is null
  //
  return c.ivNode == 0;
}

IBoolean
IHashKeyBagImpl::allElementsDo (void* iterator) const {
  Cursor c;

  if (setToFirst(c)) {
    while (constantIteratorIteration(iterator,c.ivNode) &&
           setToNext(c)) {
    }
  }

  //
  // return True if all elements have been processed,
  // than the cursor is null
  //
  return c.ivNode == 0;
}

IBoolean
IHashKeyBagImpl::containsElementWithKey (
  void const* key, // pointer to search key (IN)
  unsigned long hashvalue // the hashed value of the element
) const
{
  Cursor cursor;

  return locateElementWithKey(key, hashvalue, cursor);
}

IBoolean
IHashKeyBagImpl::containsAllKeysFrom
         (IHashKeyBagImpl const& hashtable) const
{
  Cursor cursor;

  if (hashtable.setToFirst(cursor))
       {
       void const* key;

       do {
          key = keyAt (cursor.ivNode);
       }
       while (containsElementWithKey
                        (key, getHashvalue(key, ivNoEntries))
              && hashtable.setToNext(cursor));
       }

  return cursor.ivNode == 0;
}

unsigned long
IHashKeyBagImpl::numberOfElementsWithKey (
  void const* key,
  unsigned long hashvalue
) const
{
  unsigned long n;
  Cursor cursor;

  if (locateElementWithKey(key, hashvalue, cursor))
       {
       n = 1;
       while (locateNextElementWithKey(key, hashvalue, cursor))
           {
           n++;
           }
       }
  else
       {
       n = 0;
       }

  return n;
}

IBoolean
IHashKeyBagImpl::locateElementWithKey (
  void const* key, // pointer to search key (IN)
  unsigned long hashvalue, // the hashed value of the element
  Cursor& cursor // cursor to point to the element
) const
{
  cursor.ivEntryNumber = hashvalue;
  cursor.ivNode = ivTable[cursor.ivEntryNumber];

  while (! (cursor.ivNode == 0 ||
             isKeyEqualToKey(cursor.ivNode, key)))
       {
       //
       // while element available and key not equal to argument key,
       // try next one
       //
       cursor.ivNode = cursor.ivNode->ivNext;
       }

  return cursor.ivNode != 0;
}

IBoolean
IHashKeyBagImpl::locateElementWithKeyOfElement (
  void const* element, // pointer to element with search key (IN)
  unsigned long hashvalue, // the hashed value of the element
  Cursor& cursor // cursor to point to the element
) const
{
  cursor.ivEntryNumber = hashvalue;
  cursor.ivNode = ivTable[cursor.ivEntryNumber];

  while (! (cursor.ivNode == 0 ||
             isKeyEqualToElement(cursor.ivNode, element)))
       {
       //
       // while element available and key not equal to argument key,
       // try next one
       //
       cursor.ivNode = cursor.ivNode->ivNext;
       }

  return cursor.ivNode != 0;
}

IBoolean
IHashKeyBagImpl::replaceElementWithKey (
  void const* element,
  unsigned long hashvalue,
  Cursor& cursor
)
{
  IBoolean rv;

  if (locateElementWithKeyOfElement(element, hashvalue, cursor))
       {
       replaceAt(cursor, element);
       rv = True;
       }
  else
       {
       rv = False;
       }

  return rv;
}

IBoolean
IHashKeyBagImpl::replaceElementWithKey (
  void const* element,
  unsigned long hashvalue
)
{
  IBoolean rv;
  Cursor cursor;

  if (locateElementWithKeyOfElement(element, hashvalue, cursor))
       {
       replaceAt(cursor, element);
       rv = True;
       }
  else
       {
       rv = False;
       }

  return rv;
}


IBoolean
IHashKeyBagImpl::locateOrAddElementWithKey (
  void const* element,
  unsigned long hashvalue,
  Cursor& cursor
)
{
  IBoolean rv = locateElementWithKeyOfElement
                  (element, hashvalue, cursor);

  if (! rv)
     add(element, hashvalue, cursor);

  return rv;
}

IBoolean
IHashKeyBagImpl::locateOrAddElementWithKey (
  void const* element,
  unsigned long hashvalue
)
{
  Cursor cursor;
  IBoolean rv = locateElementWithKeyOfElement
                  (element, hashvalue, cursor);

  if (! rv)
     add(element, hashvalue);

  return rv;
}


IBoolean
IHashKeyBagImpl::addOrReplaceElementWithKey (
  void const* element,
  unsigned long hashvalue,
  Cursor& cursor
)
{
  IBoolean rv;

  if (locateElementWithKeyOfElement(element, hashvalue, cursor))
       {
       replaceAt(cursor, element);
       rv = False;
       }
  else
       {
       rv = add(element, hashvalue, cursor);
       rv = True;
       }

  return rv;
}

IBoolean
IHashKeyBagImpl::addOrReplaceElementWithKey (
  void const* element,
  unsigned long hashvalue
)
{
  IBoolean rv;
  Cursor cursor;

  if (locateElementWithKeyOfElement(element, hashvalue, cursor))
       {
       replaceAt(cursor, element);
       rv = False;
       }
  else
       {
       rv = add(element, hashvalue);
       rv = True;
       }

  return rv;
}


IBoolean
IHashKeyBagImpl::removeElementWithKey (
  void const* key,
  unsigned long hashvalue
)
{
  IBoolean rv;
  Cursor cursor;

  if (locateElementWithKey(key, hashvalue, cursor))
       {
       removeAt(cursor);
       rv = True;
       }
  else
       {
       rv = False;
       }

  return rv;
}

void const*
IHashKeyBagImpl::elementWithKey (void const* key,
                                 INumber hashvalue) const
{
  Cursor cursor;

  ICHECK (locateElementWithKey(key, hashvalue, cursor),
          INotContainsKeyException, IKeyNotContainedText);
  return elementAt (cursor.ivNode);
}

void*
IHashKeyBagImpl::elementWithKey (void const* key, INumber hashvalue) {
  Cursor cursor;

  ICHECK (locateElementWithKey(key, hashvalue, cursor),
          INotContainsKeyException, IKeyNotContainedText);
  return elementAt (cursor.ivNode);
}

IBoolean
IHashKeyBagImpl::locateNextElementWithKey (
  void const* key, // pointer to search key (IN)
  unsigned long hashvalue, // the hashed value of the element
  Cursor& cursor // cursor to point to the element
) const
{
  IASSERT (cursor.ivNode != 0);
  IASSERT (cursor.ivEntryNumber == hashvalue);

  do {
    cursor.ivNode = cursor.ivNode->ivNext;
  } while (! (cursor.ivNode == 0 ||
               isKeyEqualToKey(cursor.ivNode, key)));

  return cursor.ivNode != 0;
}

INumber
IHashKeyBagImpl::removeAllElementsWithKey (
  void const* key, // key of elements to be removed (IN)
  unsigned long hashvalue // hashvalue of key (IN)
)
{
  IASSERT (hashvalue < ivNoEntries);

  Cursor cursor, next;
  IBoolean count = 0;

  cursor.ivEntryNumber = hashvalue;
  cursor.ivNode = ivTable[cursor.ivEntryNumber];

  if (cursor.ivNode != 0)
       {
       next = cursor;
       do
           {
           setToNext (next);
           if (isKeyEqualToKey(cursor.ivNode, key))
               {
               removeAt(cursor);
               count++;
               }
           cursor = next;
           }
       while (cursor.ivNode != 0 && cursor.ivEntryNumber == hashvalue);
       }

  return count;
}

INumber
IHashKeyBagImpl::numberOfDifferentKeys () const
{ Cursor c;
  INumber result = 0;
  for (setToFirst (c); c.ivNode != 0; setToNextWithDifferentKey (c))
    result++;
  return result;
}

IBoolean
IHashKeyBagImpl::setToNextWithDifferentKey (Cursor& cursor) const
{ void const* key = keyAt (cursor.ivNode);
  do {
    setToNext (cursor);
  }
  while (cursor.ivNode != 0 && isKeyEqualToKey (cursor.ivNode, key));
  return cursor.ivNode != 0;
}

void
IHashKeyBagImpl::createHashtable (IHashKeyBagImpl const& hashtable)
{ createHashtable (hashtable.ivNoEntries);
  copyHashtable (hashtable);
}

void
IHashKeyBagImpl::createHashtable (
  unsigned long noEntries // number of entries in the Hashtable (IN)
)
{
  if (noEntries == 0)
    noEntries = 1;
  ivNoElements = 0;
  ivNoEntries = 0;

  ivTable = (IHashKeyBagImplNode**) 0; // in case newNodeBlock() fails
  ivTable = (IHashKeyBagImplNode**)
             newNodeBlock (noEntries);
  ivNoEntries = noEntries;

  ITRY {
       ivCollList = (unsigned long*) 0; // in case newCountBlock() fails
       ivCollList = (unsigned long*)
                    newCountBlock (ivNoEntries);
  }
  IENDTRY

  ICATCH (IOutOfMemory exc) {
       //
       // remove already allocated memory
       //
       removeBlock (ivTable);
       ivTable = (IHashKeyBagImplNode**) 0;
       ivNoEntries = 0;
       IRETHROW (exc);
  }
  IENDCATCH

  for (unsigned long i = 0; i < ivNoEntries; i++)
       {
       ivTable[i] = 0;
       ivCollList[i] = 0;
       }

  return;
}

//
// PRIVATE methods
//

void
IHashKeyBagImpl::copyHashtable (
  IHashKeyBagImpl const& hashtable // Hashtable to make copy of (IN)
)
{
  IASSERT (ivNoElements == 0);

  if (ivNoEntries == hashtable.ivNoEntries)
       {
       //
       // copy contents of bucket table and collision list
       //
       for (unsigned long i = 0; i < ivNoEntries; i++)
           {
           IHashKeyBagImplNode* node;
           IHashKeyBagImplNode* tmpNode;

           ivCollList[i] = hashtable.ivCollList[i];
           ivTable[i] = 0;

           tmpNode = hashtable.ivTable[i];
           while(tmpNode != 0)
               {
               // copy constructor for a Hash element

               ITRY {
                   node = copyNode(tmpNode);
               }
               IENDTRY

               ICATCH (IOutOfMemory exc) {
                   for (++i; i < ivNoEntries; i++)
                       {
                       //
                       // leave table in consistent state.
                       //
                       ivTable[i] = 0;
                       }
                   removeAll();
                   IRETHROW (exc);
               }
               IENDCATCH

               node->ivNext = ivTable[i];
               ivTable[i] = node;

               tmpNode = tmpNode->ivNext;
               }
           }
       ivNoElements = hashtable.ivNoElements;
       }
  else
       {
       //
       // add per element,
       // hashvalue has to be computed for every element
       //
       addAllFrom(hashtable);
       }

  return;
}

void IHashKeyBagImpl::
resize (IHashKeyBagImplNode *newNode, INumber& newHashvalue)
{ INumber oldNoEntries = ivNoEntries;
  IHashKeyBagImplNode** oldTable = ivTable;
  removeBlock (ivCollList);

  createHashtable (ivNoEntries * 2 + 1);

  for (INumber i = 0; i < oldNoEntries; i++) {
    while (oldTable [i] != 0) {
      IHashKeyBagImplNode* node = oldTable[i];
      oldTable [i] = node->ivNext;
      unsigned long hashvalue =
        getHashvalue (keyAt (node), ivNoEntries);
      if (node == newNode)
        newHashvalue = hashvalue;

      if (ivTable[hashvalue] != 0)
        ivCollList[hashvalue]++;

      node->ivNext = ivTable[hashvalue];
      ivTable[hashvalue] = node;

      ivNoElements++;
    }
  }

  removeBlock (oldTable);
}

INumber
IHashKeyBagImpl::noEntries () const {
  return ivNoEntries;
}


IBoolean
IHashKeyBagImpl::checkCursor (Cursor const& cursor) const {
  Cursor c;

  for (setToFirst(c); c != cursor && setToNext(c);) {
  }

  return c.ivNode != 0;
}

IBoolean
IHashKeyBagImpl::isConsistent () const
{ return True;
}
