/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <iilnseq.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IILNSEQ_CPP_
  #include <ipagetun.h>
#endif


IBoolean ILinkedSequenceImpl::
checkNode (ILinkedSequenceImplNode const* node) const {
  for (ILinkedSequenceImplNode *current = ivFirst;
       current != 0;
       current = current->ivNext) {
    if (node == current)
      return True;
  }
  return False;
}

ILinkedSequenceImpl::ILinkedSequenceImpl ()
{ ivFirst = 0;
  ivLast = 0;
  ivNumberOfElements = 0;
}

ILinkedSequenceImpl::
ILinkedSequenceImpl (ILinkedSequenceImpl const&)
{ ivFirst = 0;
  ivLast = 0;
  ivNumberOfElements = 0;
}

ILinkedSequenceImpl::~ILinkedSequenceImpl ()
{
}

void ILinkedSequenceImpl::
addAllFrom (ILinkedSequenceImpl const& linkedSequence) {
  for (ILinkedSequenceImplNode *node = linkedSequence.ivFirst;
       node != 0;
       node = node->ivNext) {
    addAsLast (this->newNode (node));
  }
}

void ILinkedSequenceImpl::removeAt (ILinkedSequenceImplNode* node) {
  if (node == ivFirst)
    ivFirst = node->ivNext;
  if (node == ivLast)
    ivLast = node->ivPrevious;

  if (node->ivNext != 0)
    node->ivNext->ivPrevious = node->ivPrevious;
  if (node->ivPrevious != 0)
    node->ivPrevious->ivNext = node->ivNext;

  this->deleteNode (node);

  ivNumberOfElements--;
}

INumber
ILinkedSequenceImpl::removeAll (void* predicate, void* env) {
  ILinkedSequenceImplNode *current, *next;
  INumber removed = 0;
  for (setToFirst (current), setToFirst (next);
       current != 0;
       current = next) {
    setToNext (next);
    if (this->constantFunctionIteration(predicate, env, current)) {
      removeAt (current);
      removed++;
    }
  }
  return removed;
}

void ILinkedSequenceImpl::removeAll () {
  ILinkedSequenceImplNode *node2;
  for (ILinkedSequenceImplNode *node1 = ivFirst;
       node1 != 0;
       node1 = node2) {
    node2 = node1->ivNext;
    this->deleteNode (node1);
  }

  ivFirst = 0;
  ivLast = 0;
  ivNumberOfElements = 0;
}

INumber ILinkedSequenceImpl::numberOfElements () const {
  return ivNumberOfElements;
}

IBoolean ILinkedSequenceImpl::isEmpty () const {
  return ivNumberOfElements == 0;
}

IBoolean ILinkedSequenceImpl::isFull () const {
  return False;
}

IBoolean ILinkedSequenceImpl::
setToFirst (ILinkedSequenceImplNode*& node) const {
  node = ivFirst;
  return node != 0;
}

IBoolean ILinkedSequenceImpl::
setToNext (ILinkedSequenceImplNode*& node) const {
  node = node->ivNext;
  return node != 0;
}

IBoolean ILinkedSequenceImpl::
allElementsDo (void* iterationFunction, void* env) {
  for (ILinkedSequenceImplNode *node = ivFirst;
       node != 0;
       node = node->ivNext) {
    if (! this->functionIteration (iterationFunction, env, node))
      return False;
  }
  return True;
}

IBoolean ILinkedSequenceImpl::allElementsDo (void *iterationFunction,
                                       void* env) const {
  for (ILinkedSequenceImplNode *node = ivFirst;
       node != 0;
       node = node->ivNext) {
    if (! this->constantFunctionIteration
            (iterationFunction, env, node))
      return False;
  }
  return True;
}

IBoolean ILinkedSequenceImpl::allElementsDo (void* iterator) {
  for (ILinkedSequenceImplNode *node = ivFirst;
       node != 0;
       node = node->ivNext) {
    if (! this->iteratorIteration (iterator, node))
      return False;
  }
  return True;
}

IBoolean ILinkedSequenceImpl::allElementsDo (void *iterator) const {
  for (ILinkedSequenceImplNode *node = ivFirst;
       node != 0;
       node = node->ivNext) {
    if (! this->constantIteratorIteration (iterator, node))
      return False;
  }
  return True;
}

void ILinkedSequenceImpl::removeFirst () {
  ILinkedSequenceImplNode* node;
  setToFirst (node);
  removeAt (node);
}

void ILinkedSequenceImpl::removeLast () {
  ILinkedSequenceImplNode* node;
  setToLast (node);
  removeAt (node);
}

void ILinkedSequenceImpl::removeAtPosition (IPosition position) {
  ILinkedSequenceImplNode* node;
  setToPosition (position, node);
  removeAt (node);
}

IBoolean ILinkedSequenceImpl::
setToLast (ILinkedSequenceImplNode*& node) const {
  node = ivLast;
  return node != 0;
}

IBoolean ILinkedSequenceImpl::
setToPrevious (ILinkedSequenceImplNode*& node) const {
  node = node->ivPrevious;
  return node != 0;
}

void ILinkedSequenceImpl::
setToPosition (IPosition position,
               ILinkedSequenceImplNode*& node) const {
  setToFirst (node);
  for (int i = 1; i < position; i++) {
    node = node->ivNext;
  }
}

#ifndef __BIN_COMPAT_BASE_43__
IPosition ILinkedSequenceImpl::
position (ILinkedSequenceImplNode const* node) const {
  ILinkedSequenceImplNode *tmpnode;
  IPosition pos = 1;

  setToFirst(tmpnode);
  while (tmpnode != node) {
    setToNext (tmpnode);
    pos++;
  }
  return pos;
}
#endif

IBoolean ILinkedSequenceImpl::
isFirst (ILinkedSequenceImplNode const* node) const {
  return node != 0 && node == ivFirst;
}

IBoolean ILinkedSequenceImpl::
isLast (ILinkedSequenceImplNode const* node) const {
  return node != 0 && node == ivLast;
}

long ILinkedSequenceImpl::
compare (ILinkedSequenceImpl const& linkedSequence,
         void* comparisonFunction) const {
  long result = 0;
  ILinkedSequenceImplNode *thisNode, *sequenceNode;

  for (thisNode = ivFirst, sequenceNode = linkedSequence.ivFirst;
       result == 0 && thisNode != 0 && sequenceNode != 0;
       thisNode = thisNode->ivNext,
         sequenceNode = sequenceNode->ivNext) {
    result = this->functionComparison
               (comparisonFunction, thisNode, sequenceNode);
  }

  if (result == 0) {
    if (thisNode != 0 && sequenceNode == 0)
      result = 1;
    else if (thisNode == 0 && sequenceNode != 0)
      result = -1;
  }

  return result;
}

ILinkedSequenceImplNode* ILinkedSequenceImpl::
addAsFirst (ILinkedSequenceImplNode* newNode) {
  return addAsNext (0, newNode);
}

ILinkedSequenceImplNode* ILinkedSequenceImpl::
addAsLast (ILinkedSequenceImplNode* newNode) {
  return addAsNext (ivLast, newNode);
}

ILinkedSequenceImplNode* ILinkedSequenceImpl::
addAsNext (ILinkedSequenceImplNode *node,
           ILinkedSequenceImplNode* newNode) {
  if (node == 0)
    newNode->ivNext = ivFirst;
  else
    newNode->ivNext = node->ivNext;

  newNode->ivPrevious = node;

  if (newNode->ivNext != 0)
    newNode->ivNext->ivPrevious = newNode;

  if (newNode->ivPrevious != 0)
    newNode->ivPrevious->ivNext = newNode;

  if (node == 0)
    ivFirst = newNode;

  if (node == ivLast)
    ivLast = newNode;

  ivNumberOfElements++;

  return newNode;
}

ILinkedSequenceImplNode* ILinkedSequenceImpl::
addAsPrevious (ILinkedSequenceImplNode *node,
               ILinkedSequenceImplNode* newNode) {
  return addAsNext (node->ivPrevious, newNode);
}

ILinkedSequenceImplNode* ILinkedSequenceImpl::
addAtPosition (IPosition position, ILinkedSequenceImplNode* newNode) {
  ILinkedSequenceImplNode *node;
  if (position == 1)
    node = 0;
  else {
    node = ivFirst;
    for (int i = 2; i < position; i++) {
      node = node->ivNext;
    }
  }
  return addAsNext (node, newNode);
}

void ILinkedSequenceImpl::sort (void* comparisonFunction) {
  ILinkedSequenceImplNode *current, *next, *last;

  for (last = ivLast; last != ivFirst; last = last->ivPrevious) {
    for (current = ivFirst; current != last; ) {
      next = current->ivNext;
      if (0 < this->functionComparison (comparisonFunction,
                                         current, current->ivNext)) {
        // exchange next and current
        current->ivNext = next->ivNext;
        next->ivPrevious = current->ivPrevious;
        if (current->ivNext != 0)
          current->ivNext->ivPrevious = current;
        if (next->ivPrevious != 0)
          next->ivPrevious->ivNext = next;
        current->ivPrevious = next;
        next->ivNext = current;
        if (next == last) {
          last = current;
          if (next == ivLast) {
            ivLast = current;
          }
        }
        if (current == ivFirst) {
          ivFirst = next;
        }
      }
      else {
        current = next;
      }
    }
    // maximum elements sorted in last .. ivLast
  }
}

IBoolean ILinkedSequenceImpl::isConsistent () const
{ INumber nrOfElm = 0;
  if (ivFirst != 0 && ivFirst->ivPrevious != 0)
    return False;
  for (ILinkedSequenceImplNode *n = ivFirst; n != 0; n = n->ivNext) {
    if (n->ivNext != 0 && n->ivNext->ivPrevious != n)
      return False;
    nrOfElm++;
  }
  return nrOfElm == ivNumberOfElements;
}
