/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <iitbseq.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IITBSEQ_CPP_
  #include <ipagetun.h>
#endif


void ITabularSequenceImpl::
addAllFrom (ITabularSequenceImpl const& sequence)
{ if (! ivTable.canAdd (sequence.ivTable.ivNumberOfElements))
    expand (sequence.ivTable.ivNumberOfElements);
  copy (sequence.ivTable);
}

void ITabularSequenceImpl::removeAt (INumber& index)
{ if (ivTable.ivFirst == index) {
    ivTable.incMod (ivTable.ivFirst);
    ivTable.incMod (index);
  }
  else if (ivTable.ivLast == index) {
    ivTable.decMod (ivTable.ivLast);
  }
  else if ((ivTable.ivFirst <= ivTable.ivLast &&
            index - ivTable.ivFirst > ivTable.ivLast - index) ||
           (ivTable.ivFirst > ivTable.ivLast &&
            index <= ivTable.ivLast)) {

    blockLeft(index, ivTable.ivLast - index);
    ivTable.ivLast--;
  }
  else {
    blockRight (ivTable.ivFirst + 1, index - ivTable.ivFirst);
    ivTable.ivFirst++;
    ivTable.incMod (index);
  }
  ivTable.ivNumberOfElements--;
}

INumber ITabularSequenceImpl::removeAll (void* predicate, void* env)
{ INumber index = ivTable.ivFirst;
  INumber removed = 0;
  for (INumber i = ivTable.ivNumberOfElements; i > 0; i--) {
    if (constantFunctionIteration (predicate, env, index)) {
      removeAt (index);
      removed++;
    }
    else
      ivTable.incMod (index);
  }
  return removed;
}

IBoolean ITabularSequenceImpl::setToFirst (Cursor& cursor) const
{ cursor.ivIsValid = ivTable.ivNumberOfElements != 0;
  cursor.ivIndex = ivTable.ivFirst;
  return cursor.ivIsValid;
}

IBoolean ITabularSequenceImpl::setToNext (Cursor& cursor) const
{ cursor.ivIsValid = cursor.ivIndex != ivTable.ivLast;
  ivTable.incMod (cursor.ivIndex);
  return cursor.ivIsValid;
}

IBoolean ITabularSequenceImpl::setToLast (Cursor& cursor) const
{ cursor.ivIsValid = ivTable.ivNumberOfElements != 0;
  cursor.ivIndex = ivTable.ivLast;
  return cursor.ivIsValid;
}

IBoolean ITabularSequenceImpl::setToPrevious (Cursor& cursor) const
{ cursor.ivIsValid = cursor.ivIndex != ivTable.ivFirst;
  ivTable.decMod (cursor.ivIndex);
  return cursor.ivIsValid;
}

void ITabularSequenceImpl::
setToPosition (IPosition position, Cursor& cursor) const
{ cursor.ivIsValid = True;
  cursor.ivIndex = ivTable.ivFirst;
  ivTable.addMod (cursor.ivIndex, position - 1);
}

#ifndef __BIN_COMPAT_BASE_43__
//IPosition ITabularSequenceImpl::
//position (Cursor const& cursor) const
//{ Cursor tmpcursor;
//  IPosition pos = 1;
//
//  setToFirst(tmpcursor);
//  while (tmpcursor.ivIndex != cursor.ivIndex) {
//    setToNext (tmpcursor);
//    pos++;
//  }
//  return pos;
//}
IPosition ITabularSequenceImpl::
position (Cursor const& cursor) const
{ return ivTable.pos (((Cursor const&) cursor).ivIndex)+1;
}
#endif

IBoolean ITabularSequenceImpl::
allElementsDo (void* iterationFunction, void* env)
{ INumber index = ivTable.ivFirst;
  for (INumber i = 0; i < ivTable.ivNumberOfElements; i++) {
    if (! functionIteration (iterationFunction, env, index))
      return False;
    ivTable.incMod (index);
  }
  return True;
}

IBoolean ITabularSequenceImpl::
allElementsDo (void *iterationFunction, void* env) const
{ INumber index = ivTable.ivFirst;
  for (INumber i = 0; i < ivTable.ivNumberOfElements; i++) {
    if (! constantFunctionIteration (iterationFunction, env, index))
      return False;
    ivTable.incMod (index);
  }
  return True;
}

IBoolean ITabularSequenceImpl::allElementsDo (void* iterator)
{ INumber index = ivTable.ivFirst;
  for (INumber i = 0; i < ivTable.ivNumberOfElements; i++) {
    if (! iteratorIteration (iterator, index))
      return False;
    ivTable.incMod (index);
  }
  return True;
}

IBoolean ITabularSequenceImpl::allElementsDo (void *iterator) const
{ INumber index = ivTable.ivFirst;
  for (INumber i = 0; i < ivTable.ivNumberOfElements; i++) {
    if (! constantIteratorIteration (iterator, index))
      return False;
    ivTable.incMod (index);
  }
  return True;
}

void ITabularSequenceImpl::
operator= (ITabularSequenceImpl const& sequence)
{ if (this != &sequence) {
    ivTable.ivLast = ivTable.ivAllocatedElements - 1;
    ivTable.ivFirst = ivTable.ivNumberOfElements = 0;
    if (! ivTable.canAdd (sequence.ivTable.ivNumberOfElements))
      expand (sequence.ivTable.ivNumberOfElements);
    copy (sequence.ivTable);
  }
}

long ITabularSequenceImpl::
compare (ITabularSequenceImpl const& sequence,
         void* comparisonFunction) const
{ long result = 0;
  INumber n1 = ivTable.ivNumberOfElements,
          n2 = sequence.ivTable.ivNumberOfElements;
  INumber index1 = ivTable.ivFirst,
          index2 = sequence.ivTable.ivFirst;

  for (INumber i1 = 0, i2 = 0;
       result == 0 && i1 < n1 && i2 < n2;
       i1++, i2++) {
    result = functionComparison
               (comparisonFunction,
                elementAt (index1),
                sequence.elementAt (index2));
    ivTable.incMod (index1);
    sequence.ivTable.incMod (index2);
  }

  if (result == 0) {
    if (i1 < n1 && i2 == n2)
      result = 1;
    else if (i1 == n1 && i2 < n2)
      result = -1;
  }

  return result;
}

void ITabularSequenceImpl::
addAsFirst (void const* element, Cursor& cursor)
{ if (! ivTable.canAdd (1))
    expand (1);

  ivTable.decMod (ivTable.ivFirst);

  copyFrom (ivTable.ivFirst, element);
  ivTable.ivNumberOfElements++;
  cursor.ivIsValid = True;
  cursor.ivIndex = ivTable.ivFirst;
}

void ITabularSequenceImpl::
addAsNext (void const* element, Cursor& cursor)
{ if (cursor.ivIndex == ivTable.ivLast)
    addAsLast (element, cursor);
  else {
    ivTable.incMod (cursor.ivIndex);
    addAsPrevious (element, cursor);
  }
}

void ITabularSequenceImpl::
addAsLast (void const* element, Cursor& cursor)
{ if (! ivTable.canAdd (1))
    expand (1);

  ivTable.incMod (ivTable.ivLast);

  copyFrom (ivTable.ivLast, element);
  ivTable.ivNumberOfElements++;
  cursor.ivIsValid = True;
  cursor.ivIndex = ivTable.ivLast;
}

void ITabularSequenceImpl::
addAsPrevious (void const* element, Cursor& cursor)
{ INumber &index = cursor.ivIndex;

  if (! ivTable.canAdd (1)) {
    index = ivTable.pos (index);
    expand (1);
  }

  if (ivTable.ivFirst == index) {
    ivTable.decMod (ivTable.ivFirst);
    index = ivTable.ivFirst;
  }
  else if (ivTable.ivFirst <= ivTable.ivLast &&
           (ivTable.ivFirst == 0 ||
            (ivTable.ivLast < ivTable.ivAllocatedElements - 1 &&
             index - ivTable.ivFirst > ivTable.ivLast - index))) {
    blockRight (index + 1, ivTable.ivLast - index + 1);
    ivTable.ivLast++;
  }
  else if (ivTable.ivFirst > ivTable.ivLast &&
           index <= ivTable.ivLast) {
    blockRight (index + 1, ivTable.ivLast - index + 1);
    ivTable.ivLast++;
  }
  else {
    blockLeft (ivTable.ivFirst - 1, index - ivTable.ivFirst);
    ivTable.ivFirst--;
    index--;
  }

  ivTable.ivNumberOfElements++;
  copyFrom (index, element);
}

void ITabularSequenceImpl::
addAtPosition (IPosition position, void const* element, Cursor& cursor)
{ if (position == ivTable.ivNumberOfElements + 1)
    addAsLast (element, cursor);
  else {
    setToPosition (position, cursor);
    addAsPrevious (element, cursor);
  }
}

void ITabularSequenceImpl::
quicksort (void* comparisonFunction, INumber from, INumber to)
{ INumber i = from, j = to;
  ivTable.incMod (i);
  for (;;) {
    while (i != j && (functionComparison
                        (comparisonFunction, i, from) <= 0))
      ivTable.incMod (i);
    while (i != j && (functionComparison
                        (comparisonFunction, j, from) >= 0))
      ivTable.decMod (j);
    if (i == j)
      break;
    swap (i, j);
    ivTable.incMod (i);
    if (i != j)
      ivTable.decMod (j);
  }
  if (functionComparison (comparisonFunction, i, from) > 0)
    ivTable.decMod (i);
  j = i;
  if (i != from) {
    swap (from, i);
    ivTable.decMod (i);
  }
  if (j != to)
    ivTable.incMod (j);
  if (from != i)
    quicksort (comparisonFunction, from, i);
  if (j != to)
    quicksort (comparisonFunction, j, to);
}

void ITabularSequenceImpl::sort (void* comparisonFunction)
{ if (ivTable.ivNumberOfElements > 1)
    quicksort (comparisonFunction, ivTable.ivFirst, ivTable.ivLast);
}

IBoolean ITabularSequenceImpl::checkIndex (INumber index) const
{ if (ivTable.ivNumberOfElements == 0)
    return False;
  else if (ivTable.ivFirst <= ivTable.ivLast)
    return ivTable.ivFirst <= index && index <= ivTable.ivLast;
  else
    return (ivTable.ivFirst <= index &&
              index <= ivTable.ivAllocatedElements) ||
            index <= ivTable.ivLast;
}

IBoolean ITabularSequenceImpl::isConsistent () const
{ INumber f = ivTable.ivFirst,
          l = ivTable.ivLast,
          a = ivTable.ivAllocatedElements,
          n = ivTable.ivNumberOfElements;
  return n % a == (l >= f ? l - f + 1 : l + a - f + 1) % a;
}
