/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <iexcbase.h>
#include <iitbtre.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IITBTRE_CPP_
  #include <ipagetun.h>
#endif

void ITabularTreeImpl::
initNode (ITabularTreeImplNode* node,
          ITabularTreeImplNode* parent) const
{ node->ivParent = parent;
  for (INumber n = 0; n < ivNumberOfChildren; n++)
    node->ivChildren [n] = 0;
}

INumber ITabularTreeImpl::
numberOfSubtreeElements (ITabularTreeImplNode const* node) const
{ IASSERT (node != 0);
  INumber result = 0;
  for (INumber n = 0; n < ivNumberOfChildren; n++) {
    if (node->ivChildren [n] != 0)
      result += numberOfSubtreeElements (node->ivChildren [n]);
  }
  return result + 1;
}

INumber ITabularTreeImpl::
numberOfSubtreeLeaves (ITabularTreeImplNode const* node) const
{ IASSERT (node != 0);
  INumber result = 0;
  for (INumber n = 0; n < ivNumberOfChildren; n++) {
    if (node->ivChildren [n] != 0)
      result += numberOfSubtreeLeaves (node->ivChildren [n]);
  }
  if (result == 0)
    // node is a leaf
    result = 1;
  return result;
}

IBoolean ITabularTreeImpl::
isLeaf (ITabularTreeImplNode const* node) const
{ for (INumber n = 0; n < ivNumberOfChildren; n++)
    if (node->ivChildren [n] != 0)
      return False;
  return True;
}

INumber ITabularTreeImpl::
position (ITabularTreeImplNode const* node) const
{ ITabularTreeImplNode* parent = node->ivParent;
  if (parent == 0)
    return 1;
  for (INumber n = 0; n < ivNumberOfChildren; n++)
    if (parent->ivChildren [n] == node)
      return n+1;
  IASSERT (False);
  return 1;
}

void ITabularTreeImpl::
attachSubtreeAsRoot (ITabularTreeImpl& tree, ITabularTreeImplNode* node)
{ if (node->ivParent == 0)
    tree.ivRoot = 0;
  else
    node->ivParent->ivChildren [tree.position (node) - 1] = 0;
  ivRoot = node;
  if (ivRoot != 0)
    ivRoot->ivParent = 0;
}

void ITabularTreeImpl::
attachSubtreeAsChild (ITabularTreeImplNode* parent,
                      IPosition pos,
                      ITabularTreeImpl& tree,
                      ITabularTreeImplNode* from)
{ if (from->ivParent == 0)
    tree.ivRoot = 0;
  else
    from->ivParent->ivChildren [tree.position (from) - 1] = 0;
  parent->ivChildren [pos-1] = from;
  if (from != 0)
    from->ivParent = parent;

}


IBoolean ITabularTreeImpl::
setToParent (ITabularTreeImplNode*& node) const
{ node = node->ivParent;
  return node != 0;
}

IBoolean ITabularTreeImpl::
setToFirstExistingChild (ITabularTreeImplNode*& node) const
{ for (INumber n = 0; n < ivNumberOfChildren; n++) {
    if (node->ivChildren [n] != 0) {
      node = node->ivChildren [n];
      return True;
    }
  }
  node = 0;
  return False;
}

IBoolean ITabularTreeImpl::
setToNextExistingChild (ITabularTreeImplNode*& node) const
{ ITabularTreeImplNode* parent = node->ivParent;
  if (parent != 0) {
    IBoolean skippedNode = False;
    for (INumber n = 0; n < ivNumberOfChildren; n++) {
      if (skippedNode) {
        if (parent->ivChildren [n] != 0) {
          node = parent->ivChildren [n];
          return True;
        }
      }
      else {
        if (parent->ivChildren [n] == node) {
          skippedNode = True;
        }
      }
    }
  }
  node = 0;
  return False;
}

IBoolean ITabularTreeImpl::
setToLastExistingChild (ITabularTreeImplNode*& node) const
{ for (INumber n = ivNumberOfChildren; n > 0; ) {
    n--;
    if (node->ivChildren [n] != 0) {
      node = node->ivChildren [n];
      return True;
    }
  }
  node = 0;
  return False;
}

IBoolean ITabularTreeImpl::
setToPreviousExistingChild (ITabularTreeImplNode*& node) const
{ ITabularTreeImplNode* parent = node->ivParent;
  if (parent != 0) {
    IBoolean skippedNode = False;
    for (INumber n = ivNumberOfChildren; n > 0; ) {
      n--;
      if (skippedNode) {
        if (parent->ivChildren [n] != 0) {
          node = parent->ivChildren [n];
          return True;
        }
      }
      else {
        if (parent->ivChildren [n] == node) {
          skippedNode = True;
        }
      }
    }
  }
  node = 0;
  return False;
}

IBoolean ITabularTreeImpl::
setToFirst (ITabularTreeImplNode*& node,
            ITreeIterationOrder order) const
{ node = ivRoot;
  if (ivRoot == 0)
    return False;
  if (order == IPostorder) {
    ITabularTreeImplNode* firstChild = node;
    while (setToFirstExistingChild (firstChild))
      node = firstChild;
  }
  return True;
}

IBoolean ITabularTreeImpl::
setToNext (ITabularTreeImplNode*& node, ITreeIterationOrder order) const
{ if (order == IPreorder) {
    ITabularTreeImplNode* firstChild = node;
    if (setToFirstExistingChild (firstChild)) {
      node = firstChild;
      return True;
    }
    else {
      ITabularTreeImplNode* nextChild = node;
      while (! setToNextExistingChild (nextChild)) {
        if (! setToParent (node))
          return False;
        nextChild = node;
      }
      node = nextChild;
      return True;
    }
  }
  else {
    ITabularTreeImplNode* nextChild = node;
    if (setToNextExistingChild (nextChild)) {
      node = nextChild;
      while (setToFirstExistingChild (nextChild))
        node = nextChild;
      return True;
    }
    else
      return setToParent (node);
  }
}

IBoolean ITabularTreeImpl::
setToLast (ITabularTreeImplNode*& node, ITreeIterationOrder order) const
{ node = ivRoot;
  if (ivRoot == 0)
    return False;
  if (order == IPreorder) {
    ITabularTreeImplNode* lastChild = node;
    while (setToLastExistingChild (lastChild))
      node = lastChild;
  }
  return True;
}

IBoolean ITabularTreeImpl::
setToPrevious (ITabularTreeImplNode*& node,
               ITreeIterationOrder order) const
{ if (order == IPostorder) {
    ITabularTreeImplNode* lastChild = node;
    if (setToLastExistingChild (lastChild)) {
      node = lastChild;
      return True;
    }
    else {
      ITabularTreeImplNode* previousChild = node;
      while (! setToPreviousExistingChild (previousChild)) {
        if (! setToParent (node))
          return False;
        previousChild = node;
      }
      node = previousChild;
      return True;
    }
  }
  else {
    ITabularTreeImplNode* previousChild = node;
    if (setToPreviousExistingChild (previousChild)) {
      node = previousChild;
      while (setToLastExistingChild (previousChild))
        node = previousChild;
      return True;
    }
    else
      return setToParent (node);
  }
}

IBoolean ITabularTreeImpl::
allElementsDo (void* function,
               ITreeIterationOrder order,
               void* env,
               IBoolean (*apply) (void *function,
                                  void* env, void* node),
               ITabularTreeImplNode* subtreeRoot)
{ if (order == IPreorder) {
    if (! (*apply) (function, env, subtreeRoot)) return False;
    for (int i = 0; i < ivNumberOfChildren; i++) {
      ITabularTreeImplNode* child = subtreeRoot->ivChildren [i];
      if (child != 0)
        if (! allElementsDo (function, order, env, apply, child))
          return False;
    }
  }
  else {
    for (int i = 0; i < ivNumberOfChildren; i++) {
      ITabularTreeImplNode* child = subtreeRoot->ivChildren [i];
      if (child != 0)
        if (! allElementsDo (function, order, env, apply, child))
          return False;
    }
    if (! (*apply) (function, env, subtreeRoot)) return False;
  }
  return True;
}

IBoolean ITabularTreeImpl::
allElementsDo (void* function,
               ITreeIterationOrder order,
               void* env,
               IBoolean (*apply) (void *function,
                                  void* env, void const* node),
               ITabularTreeImplNode* subtreeRoot) const
{ if (order == IPreorder) {
    if (! (*apply) (function, env, subtreeRoot)) return False;
    for (int i = 0; i < ivNumberOfChildren; i++) {
      ITabularTreeImplNode* child = subtreeRoot->ivChildren [i];
      if (child != 0)
        if (! allElementsDo (function, order, env, apply, child))
          return False;
    }
  }
  else {
    for (int i = 0; i < ivNumberOfChildren; i++) {
      ITabularTreeImplNode* child = subtreeRoot->ivChildren [i];
      if (child != 0)
        if (! allElementsDo (function, order, env, apply, child))
          return False;
    }
    if (! (*apply) (function, env, subtreeRoot)) return False;
  }
  return True;
}

IBoolean ITabularTreeImpl::
allElementsDo (void* iterator,
               ITreeIterationOrder order,
               IBoolean (*apply) (void* iterator, void* node),
               ITabularTreeImplNode* subtreeRoot)
{ if (order == IPreorder) {
    if (! (*apply) (iterator, subtreeRoot)) return False;
    for (int i = 0; i < ivNumberOfChildren; i++) {
      ITabularTreeImplNode* child = subtreeRoot->ivChildren [i];
      if (child != 0)
        if (! allElementsDo (iterator, order, apply, child))
          return False;
    }
  }
  else {
    for (int i = 0; i < ivNumberOfChildren; i++) {
      ITabularTreeImplNode* child = subtreeRoot->ivChildren [i];
      if (child != 0)
        if (! allElementsDo (iterator, order, apply, child))
          return False;
    }
    if (! (*apply) (iterator, subtreeRoot)) return False;
  }
  return True;
}

IBoolean ITabularTreeImpl::
allElementsDo (void* iterator,
               ITreeIterationOrder order,
               IBoolean (*apply) (void* iterator, void const* node),
               ITabularTreeImplNode* subtreeRoot) const
{ if (order == IPreorder) {
    if (! (*apply) (iterator, subtreeRoot)) return False;
    for (int i = 0; i < ivNumberOfChildren; i++) {
      ITabularTreeImplNode* child = subtreeRoot->ivChildren [i];
      if (child != 0)
        if (! allElementsDo (iterator, order, apply, child))
          return False;
    }
  }
  else {
    for (int i = 0; i < ivNumberOfChildren; i++) {
      ITabularTreeImplNode* child = subtreeRoot->ivChildren [i];
      if (child != 0)
        if (! allElementsDo (iterator, order, apply, child))
          return False;
    }
    if (! (*apply) (iterator, subtreeRoot)) return False;
  }
  return True;
}

IBoolean ITabularTreeImpl::
checkNode (ITabularTreeImplNode const* node) const
{ if (ivRoot != 0)
    return checkNodeInSubtree (node, ivRoot);
  else
    return False;
}

IBoolean ITabularTreeImpl::
checkNodeInSubtree (ITabularTreeImplNode const* node,
                    ITabularTreeImplNode const* current) const
{ if (node == current)
    return True;

  for (int i = 0; i < ivNumberOfChildren; i++)
    if (current->ivChildren [i] != 0 &&
        checkNodeInSubtree (node, current->ivChildren [i]))
      return True;

  return False;
}

IBoolean ITabularTreeImpl::
isConsistent () const
{ return ivRoot == 0 ||
           (ivRoot->ivParent == 0 && isConsistent (ivRoot));
}

IBoolean ITabularTreeImpl::
isConsistent (ITabularTreeImplNode const* node) const
{ for (int i = 0; i < ivNumberOfChildren; i++) {
    if (node->ivChildren [i] != 0 &&
        (node->ivChildren [i]->ivParent != node ||
         ! isConsistent (node->ivChildren [i])))
      return False;
  }
  return True;
}
