/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <iitdseq.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IITDSEQ_CPP_
  #include <ipagetun.h>
#endif

void IDilutedSequenceImpl::
addAllFrom (IDilutedSequenceImpl const& sequence)
{ if (! ivTable.canAdd (sequence.ivTable.ivNumberOfElements))
    expand (sequence.ivTable.ivNumberOfElements);
  copy (sequence.ivTable);
}

void IDilutedSequenceImpl::removeAt (INumber& index)
{ remove (index);
}

INumber IDilutedSequenceImpl::removeAll (void* predicate, void* env)
{ if (ivTable.ivNumberOfElements == 0)
    return 0;
  INumber index = ivTable.ivFirst;
  INumber removed = 0;
  for (;;) {
    IBoolean isLast = index == ivTable.ivLast;
    if (constantFunctionIteration (predicate, env, index)) {
      remove (index);
      removed++;
    }
    if (isLast)
      break;
    setToNext (index);
  }
  return removed;
}

IBoolean IDilutedSequenceImpl::setToFirst (Cursor& cursor) const
{ cursor.ivIsValid = ivTable.ivNumberOfElements != 0;
  cursor.ivIndex = ivTable.ivFirst;
  return cursor.ivIsValid;
}

IBoolean IDilutedSequenceImpl::setToNext (Cursor& cursor) const
{ cursor.ivIsValid = setToNext (cursor.ivIndex);
  return cursor.ivIsValid;
}

IBoolean IDilutedSequenceImpl::setToLast (Cursor& cursor) const
{ cursor.ivIsValid = ivTable.ivNumberOfElements != 0;
  cursor.ivIndex = ivTable.ivLast;
  return cursor.ivIsValid;
}

IBoolean IDilutedSequenceImpl::setToPrevious (Cursor& cursor) const
{ cursor.ivIsValid = setToPrevious (cursor.ivIndex);
  return cursor.ivIsValid;
}

void IDilutedSequenceImpl::
setToPosition (IPosition position, Cursor& cursor) const
{ cursor.ivIsValid = True;
  cursor.ivIndex = ivTable.ivFirst;
  for (IPosition i = 1; i < position; i++)
    setToNext (cursor.ivIndex);
}

#ifndef __BIN_COMPAT_BASE_43__
IPosition IDilutedSequenceImpl::
position (Cursor const& cursor) const
{ Cursor tmpcursor;
  IPosition pos = 1;

  setToFirst(tmpcursor);
  while (tmpcursor.ivIndex != cursor.ivIndex) {
    setToNext (tmpcursor);
    pos++;
  }
  return pos;
}
#endif

IBoolean IDilutedSequenceImpl::
allElementsDo (void* iterationFunction, void* env)
{ INumber index = ivTable.ivFirst;
  for (INumber i = 0; i < ivTable.ivNumberOfElements; i++) {
    if (! functionIteration (iterationFunction, env, index))
      return False;
    setToNext (index);
  }
  return True;
}

IBoolean IDilutedSequenceImpl::
allElementsDo (void *iterationFunction, void* env) const
{ INumber index = ivTable.ivFirst;
  for (INumber i = 0; i < ivTable.ivNumberOfElements; i++) {
    if (! constantFunctionIteration (iterationFunction, env, index))
      return False;
    setToNext (index);
  }
  return True;
}

IBoolean IDilutedSequenceImpl::allElementsDo (void* iterator)
{ INumber index = ivTable.ivFirst;
  for (INumber i = 0; i < ivTable.ivNumberOfElements; i++) {
    if (! iteratorIteration (iterator, index))
      return False;
    setToNext (index);
  }
  return True;
}

IBoolean IDilutedSequenceImpl::allElementsDo (void *iterator) const
{ INumber index = ivTable.ivFirst;
  for (INumber i = 0; i < ivTable.ivNumberOfElements; i++) {
    if (! constantIteratorIteration (iterator, index))
      return False;
    setToNext (index);
  }
  return True;
}

void IDilutedSequenceImpl::
operator= (IDilutedSequenceImpl const& sequence)
{ if (this != &sequence) {
    ivTable.ivLast = ivTable.ivAllocatedElements - 1;
    ivTable.ivFirst = ivTable.ivNumberOfElements = 0;
    if (! ivTable.canAdd (sequence.ivTable.ivNumberOfElements))
      expand (sequence.ivTable.ivNumberOfElements);
    copy (sequence.ivTable);
  }
}

long IDilutedSequenceImpl::
compare (IDilutedSequenceImpl const& sequence,
         void* comparisonFunction) const
{ long result = 0;
  INumber n1 = ivTable.ivNumberOfElements,
          n2 = sequence.ivTable.ivNumberOfElements;
  INumber index1 = ivTable.ivFirst,
          index2 = sequence.ivTable.ivFirst;

  for (INumber i1 = 0, i2 = 0;
       result == 0 && i1 < n1 && i2 < n2;
       i1++, i2++) {
    result = functionComparison
               (comparisonFunction,
                elementAt (index1),
                sequence.elementAt (index2));
    setToNext (index1);
    sequence.setToNext (index2);
  }

  if (result == 0) {
    if (i1 < n1 && i2 == n2)
      result = 1;
    else if (i1 == n1 && i2 < n2)
      result = -1;
  }

  return result;
}

void IDilutedSequenceImpl::
addAsFirst (void const* element, Cursor& cursor)
{ if (ivTable.ivNumberOfElements == 0)
    addAsLast (element, cursor);
  else {
    setToFirst (cursor);
    addAsPrevious (element, cursor);
  }
}

void IDilutedSequenceImpl::
addAsNext (void const* element, Cursor& cursor)
{ if (cursor.ivIndex == ivTable.ivLast)
    addAsLast (element, cursor);
  else {
    setToNext (cursor.ivIndex);
    addAsPrevious (element, cursor);
  }
}

void IDilutedSequenceImpl::
addAsLast (void const* element, Cursor& cursor)
{ if (! ivTable.canAdd (1))
    expand (1);

  cursor.ivIndex = ivTable.ivLast;
  ivTable.incMod (cursor.ivIndex);
  if (ivTable.ivNumberOfElements > 0 &&
      cursor.ivIndex == ivTable.ivFirst)
    makeFree (cursor.ivIndex);
  copyFrom (cursor.ivIndex, element);
  ivTable.ivNumberOfElements++;
  cursor.ivIsValid = True;
  ivTable.ivLast = cursor.ivIndex;
}

void IDilutedSequenceImpl::
addAsPrevious (void const* element, Cursor& cursor)
{ if (! ivTable.canAdd (1)) {
    cursor.ivIndex = ivTable.pos (cursor.ivIndex);
    expand (1);
  }

  IBoolean isFirst = cursor.ivIndex == ivTable.ivFirst;
  makeFree (cursor.ivIndex);
  if (isFirst)
    ivTable.ivFirst = cursor.ivIndex;
  ivTable.ivNumberOfElements++;
  copyFrom (cursor.ivIndex, element);
}

void IDilutedSequenceImpl::
addAtPosition (IPosition position, void const* element, Cursor& cursor)
{ if (position == ivTable.ivNumberOfElements + 1)
    addAsLast (element, cursor);
  else {
    setToPosition (position, cursor);
    addAsPrevious (element, cursor);
  }
}

void IDilutedSequenceImpl::sort (void* comparisonFunction)
{ INumber left;
  INumber last;
  INumber newLast;
  Cursor cursor;
  INumber& index = cursor.ivIndex;

  if (ivTable.ivNumberOfElements > 0) {
    last = ivTable.ivLast;
    while (last != ivTable.ivFirst) {
      setToFirst (cursor);
      left = index;
      while (left != last) {
        setToNext (cursor);

        if (functionComparison
              (comparisonFunction, index, left) < 0)
          swap (index, left);
        newLast = left;
        left = index;
      }
      last = newLast;
    }
  }
}

IBoolean IDilutedSequenceImpl::checkIndex (INumber index) const
{ if (ivTable.ivNumberOfElements == 0 || isRemoved (index))
    return False;
  else if (ivTable.ivFirst <= ivTable.ivLast)
    return ivTable.ivFirst <= index && index <= ivTable.ivLast;
  else
    return (ivTable.ivFirst <= index &&
              index <= ivTable.ivAllocatedElements) ||
            index <= ivTable.ivLast;
}

IBoolean IDilutedSequenceImpl::isConsistent () const
{ if (ivTable.ivNumberOfElements == 0) {
    INumber last = ivTable.ivLast;
    return ivTable.incMod (last) == ivTable.ivFirst;
  }
  else {
    if (isRemoved (ivTable.ivFirst) ||
        isRemoved (ivTable.ivLast))
      return False;
    INumber nrOfElm = 0;
    for (INumber index = ivTable.ivFirst;; ivTable.incMod (index)) {
      if (! isRemoved (index))
        nrOfElm++;
      if (index == ivTable.ivLast)
        break;
    }
    return nrOfElm == ivTable.ivNumberOfElements;
  }
}
