/*******************************************************************************
* FILE NAME: imsgtext.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imsgtext.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/


#include <ibase.hpp>
#include <new.h>
#include <imsgtext.hpp>

#if defined ( __MVS__ )

  #include <fstream.h>
  #include <stdlib.h>
#endif

extern "C"
  {
  #include <string.h>

#if defined (__OS2__)

    #define INCL_DOSMISC
    #include <os2.h>

#elif defined (_AIX)

    #include <limits.h>
    #include <locale.h>
    #include <nl_types.h>

#elif defined (IC_WIN)

    #define NOGDICAPMASKS
    #define NOVIRTUALKEYCODES
    #define NOWINMESSAGES
    #define NOWINSTYLES
    #define NOSYSMETRICS
    #define NOMENUS
    #define NOICONS
    #define NOKEYSTATES
    #define NOSYSCOMMANDS
    #define NORASTEROPS
    #define NOSHOWWINDOW
    #define OEMRESOURCE
    #define NOATOM
    #define NOCLIPBOARD
    #define NOCOLOR
    #define NOCTLMGR
    #define NODRAWTEXT
    #define NOGDI
    #define NOKERNEL
    #define NOUSER
    #define NOMB
    #define NOMEMMGR
    #define NOMETAFILE
    #define NOMINMAX
    #define NOMSG
    #define NOOPENFILE
    #define NOSCROLL
    #define NOSERVICE
    #define NOSOUND
    #define NOTEXTMETRIC
    #define NOWH
    #define NOWINOFFSETS
    #define NOCOMM
    #define NOKANJI
    #define NOHELP
    #define NOPROFILER
    #define NODEFERWINDOWPOS
    #include <windows.h>

#endif
  }

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMSGTEXT_CPP_
  #include <ipagetun.h>
#endif


IMessageText :: IMessageText  ( unsigned long messageId,
                                const char*   messageFileName,
                                const char*   textInsert1,
                                const char*   textInsert2,
                                const char*   textInsert3,
                                const char*   textInsert4,
                                const char*   textInsert5,
                                const char*   textInsert6,
                                const char*   textInsert7,
                                const char*   textInsert8,
                                const char*   textInsert9 )
                     : bClSuccess(0), pClText (0)
/****************************************************************/
/*                                                              */
/****************************************************************/
{
   void(*poldnh)() = set_new_handler(0);

#ifndef __OS2__
   if (messageFileName) {
#endif
      unsigned long ulCount = 0;
      char* insertTable[9];
      if (textInsert1) {         // build variable text insertion table
         ulCount++;
         insertTable[0] = (char*)textInsert1;
         if (textInsert2) {
            ulCount++;
            insertTable[1] = (char*)textInsert2;
            if (textInsert3) {
               ulCount++;
               insertTable[2] = (char*)textInsert3;
               if (textInsert4) {
                  ulCount++;
                  insertTable[3] = (char*)textInsert4;
                  if (textInsert5) {
                     ulCount++;
                     insertTable[4] = (char*)textInsert5;
                     if (textInsert6) {
                        ulCount++;
                        insertTable[5] = (char*)textInsert6;
                        if (textInsert7) {
                           ulCount++;
                           insertTable[6] = (char*)textInsert7;
                           if (textInsert8) {
                              ulCount++;
                              insertTable[7] = (char*)textInsert8;
                              if (textInsert9) {
                                 ulCount++;
                                 insertTable[8] = (char*)textInsert9;
                              }
                           }
                        }
                     }
                  }
               }
            }
         }
      }
      unsigned long ulMessageLength;

#if defined (__MVS__)

#define IMSGTEXT 256
      char messageText[IMSGTEXT];
      char* szText = 0;

      bClSuccess  = readMessageFromFile(messageText,messageFileName,
                                        messageId);

      if (bClSuccess)
      {
        szText = replaceVars(messageText,insertTable,ulCount);
        ulMessageLength = strlen (szText);
      }  /* endif */

#elif defined ( __OS2__ )

      char szText[512];
      bClSuccess =
      DosGetMessage(insertTable, ulCount, (PCHAR)szText,
                    (unsigned long)512, messageId,
                    (PSZ)messageFileName,
                    &ulMessageLength) == 0;

#elif defined (_AIX)

      setlocale (LC_ALL, "");
      nl_catd catalog = catopen ((char*)messageFileName, 0);
      char* szText = 0;
      char* szVarText = catgets (catalog, 1, messageId, 0);
      bClSuccess = szVarText != 0;
      if (bClSuccess) {
         szText = replaceVars (szVarText, insertTable, ulCount);
         ulMessageLength = strlen (szText);
      }

#elif defined (IC_WIN)
      char szText[512];
      this->bClSuccess = 0;
      HANDLE hMsgLibrary = LoadLibrary(messageFileName);
      if (hMsgLibrary)
      {
        /**************************************************************/
        /* Format the message from the supplied msg DLL               */
        /**************************************************************/
        if (ulMessageLength = FormatMessage(FORMAT_MESSAGE_FROM_HMODULE,
                                            hMsgLibrary,
                                            messageId,
                                            GetSystemDefaultLangID(),
                                            (LPTSTR)szText,
                                            512,
                                            NULL))

        {
          this->bClSuccess = 1;
        }
        FreeLibrary(hMsgLibrary);
      }

#endif

      if (bClSuccess)
      {
         pClText = new char[ulMessageLength + 1];
         if (pClText)
         {
            if ( (szText[ulMessageLength-2] == '\r') &&
                (szText[ulMessageLength-1] == '\n') )
                                                // get rid of CRLF
               ulMessageLength = ulMessageLength - 2;
            strncpy(pClText, szText, ulMessageLength);
            pClText[ulMessageLength] = '\0';
         }
      }

#if defined (__MVS__)

      delete [] szText;

#elif defined (_AIX)

      delete [] szText;
      catclose (catalog);

#endif

#ifndef __OS2__
   }   //   if (messageFileName) ....
#endif

   if (!pClText)  //   load default text if message text load failed
   {
      pClText = new char[39];
      if (pClText)
         strcpy(pClText, "Unable to load text from message file.");
   }

   set_new_handler(poldnh);
}

IMessageText :: IMessageText ( const IMessageText& text )
/****************************************************************/
/*                                                              */
/****************************************************************/
{
   void(*poldnh)() = set_new_handler(0);

   bClSuccess = text.bClSuccess;
   if (text.pClText)
   {
      pClText = new char[strlen(text.pClText) + 1];
      if (pClText)
         strcpy(pClText, text.pClText);
   }
   else pClText = 0;

   set_new_handler(poldnh);
}

IMessageText& IMessageText :: operator= ( const IMessageText& text )
/****************************************************************/
/*                                                              */
/****************************************************************/
{
   void(*poldnh)() = set_new_handler(0);

   bClSuccess = text.bClSuccess;
   if (pClText)
      delete [] pClText;
   if (text.pClText)
   {
      pClText = new char[strlen(text.pClText) + 1];
      if (pClText)
         strcpy(pClText, text.pClText);
   }
   else pClText = 0;

   set_new_handler(poldnh);

   return *this;
}

IMessageText::~IMessageText ( )
/****************************************************************/
/*                                                              */
/****************************************************************/
{
   if(pClText)
     delete [] pClText;
}

IMessageText::operator const char* () const
/****************************************************************/
/*                                                              */
/****************************************************************/
{
 return text();
}

const char* IMessageText::text ( ) const
/****************************************************************/
/*                                                              */
/****************************************************************/
{
  return pClText;
}

IMessageText& IMessageText :: setDefaultText (const char* defaultText)
/****************************************************************/
/*                                                              */
/****************************************************************/
{
   if (defaultText && !bClSuccess)
           // only set if we couldn't load text from message file
   {
      void(*poldnh)() = set_new_handler(0);

      char* pTmpText = pClText;
      pClText = new char[strlen(defaultText) + 1];
      if (pClText)
      {
         strcpy(pClText, defaultText);
         delete [] pTmpText;  // delete original default text
      }
      else
         pClText = pTmpText;

      set_new_handler(poldnh);
   }

   return *this;
}

#if defined (_AIX) || defined (__MVS__)

char* IMessageText :: replaceVars (const char* varText,
                                  char* insertTable[],
                                  unsigned long tableEntries)
/****************************************************************/
/*                                                              */
/****************************************************************/
{ char* replText = 0;

  if (varText [0] == 0) {
    replText = new char (1);
    replText [0] = 0;
  }
  else {
    unsigned replTextLen = 0;
    unsigned to = 0;
    do {
      unsigned var = 0;
      unsigned from = to;

      for (; varText [to] != 0; to++) {
       if (varText [to] == '%') {
         if (varText [to+1] == '%')
           to++;
         else if ('1' <= varText [to+1] && varText [to+1] <= '9') {
           var = varText [to+1] - '0';
           if (var <= tableEntries)
             break;
           else
             var = 0;
         }
       }
      }
      // to is index of '\0' or '%'

      char* oldReplText = replText;

      replTextLen += to - from;
      if (var != 0)
       replTextLen += strlen (insertTable [var-1]);

      replText = new char [replTextLen + 1];

      if (oldReplText) {
       strcpy (replText, oldReplText);
       delete [] oldReplText;
      }
      else
       replText [0] = 0;

      strncat (replText, &varText [from], to - from);

      if (var != 0) {
       strcat (replText, insertTable [var-1]);
       to += 2;
      }

    } while (varText [to] != 0);

  }
  return replText;
}

#endif

#if defined (__MVS__)

int IMessageText :: readMessageFromFile
                               (char*           messageText,
                                const char*     messageFileName,
                                unsigned long   messageId)
{
   static const unsigned int lprefix  =    3;
   static const unsigned int lmsgnum  =    4;
   static const unsigned int lsuffix  =    3;
   static const unsigned int ltext    =  256;
   static const unsigned int lmsgbuf  =  266;

   union Line
   {
      struct
      {
         char prefix[lprefix];
         char msgnum[lmsgnum];
         char suffix[lsuffix];
         char text[ltext];
      } record;

      char msgbuf[lmsgbuf];
   };

   Line line;
   unsigned int llength;

   ifstream messageFile(messageFileName);

   while (messageFile)
   {
     ws(messageFile).get(line.msgbuf, lmsgbuf, '\n');
     if (atol(line.record.msgnum) == messageId)
     {
       llength = strlen(line.record.text);
       strcpy(messageText, line.record.text);
       messageText[llength] = '\0';
       return 1;
     }
   } /* endwhile */

   return 0;
}

#endif
