/*******************************************************************************
* FILE NAME: ipoint.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ipoint.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
    #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
  #include <math.h>
  }

#include <ipoint.hpp>
#include <ihandle.hpp>
#include <istring.hpp>
#include <istream.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IPOINT_CPP_
  #include <ipagetun.h>
#endif

#define self (*this)
#define MIN(a, b) ( ( (a) <= (b) ) ? (a) : (b) )
#define MAX(a, b) ( ( (a) >= (b) ) ? (a) : (b) )

/*------------------------------------------------------------------------------
| IPair::asString                                                              |
------------------------------------------------------------------------------*/
IString IPair::asString() const
{
   IString str(IString("(") +
          IString(coord1()) +
          IString(',') +
          IString(coord2()) +
          IString(')'));
     return str;
}

/*------------------------------------------------------------------------------
| IPair::asDebugInfo                                                           |
------------------------------------------------------------------------------*/
IString IPair::asDebugInfo() const
  {
  IString result( "IPair(" );
  result += Inherited::asDebugInfo();
  result += "coord1=";
  result += IString( this->coord1() );
  result += "coord2=";
  result += IString( this->coord2() );
  return result;
  }

/*------------------------------------------------------------------------------
| IPair::distanceFrom                                                          |
------------------------------------------------------------------------------*/
double IPair :: distanceFrom( const IPair &aPair ) const
{
   long int dx = self.coordCl1 - aPair.coordCl1,
            dy = self.coordCl2 - aPair.coordCl2;
   return sqrt( dx * dx + dy * dy );
}


/*------------------------------------------------------------------------------
| IPair::minimum                                                               |
------------------------------------------------------------------------------*/
IPair IPair :: minimum(const IPair& pair) const
{
  return IPair(MIN(coord1(), pair.coord1()),
                MIN(coord2(), pair.coord2()) );
}

/*------------------------------------------------------------------------------
| IPair::maximum                                                               |
------------------------------------------------------------------------------*/
IPair IPair :: maximum(const IPair& pair) const
{
  return IPair(MAX(coord1(), pair.coord1()),
                MAX(coord2(), pair.coord2()) );
}

/*------------------------------------------------------------------------------
| operator <<                                                                  |
------------------------------------------------------------------------------*/
ostream &operator << ( ostream     &aStream,
                       const IPair &aPair )
  {
  return aStream << aPair.asString();
  }

/*------------------------------------------------------------------------------
| IPoint::asPOINTL                                                             |
------------------------------------------------------------------------------*/
POINTL IPoint::asPOINTL ( ) const
  {
  POINTL ptl = { x(), y() };
  return ptl;
  }

/*------------------------------------------------------------------------------
| ISize::asSIZEL                                                               |
------------------------------------------------------------------------------*/
SIZEL ISize::asSIZEL ( ) const
  {
  SIZEL sizl = { width(), height() };
  return sizl;
  }

