/*******************************************************************************
* FILE NAME: irect.cpp                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in irect.hpp.                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
   #include <iwindefs.h>
}

#include <irect.hpp>

#ifndef _ISTRING_
  #include <istring.hpp>
#endif

#ifndef _ISTREAM_
  #include <istream.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IRECT_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IRectangle::asString                                                         |
|                                                                              |
| Return the rectangle contents as an IString in the format                    |
|   IRectangle(left, bottom, right, top)                                       |
------------------------------------------------------------------------------*/
IString IRectangle::asString() const
{
 IString str(IString("IRectangle(") +
        IString(minX()) +
        IString(',') +
        IString(minY())+
        IString(',')+
        IString(maxX())+
        IString(',')+
        IString(maxY())+
        IString(')'));
 return str;
}

/*------------------------------------------------------------------------------
| IRectangle::asDebugInfo                                                      |
------------------------------------------------------------------------------*/
IString IRectangle::asDebugInfo() const
  {
  IString result( "IRectangle(" );
  result += Inherited::asDebugInfo();
  result += "origin=";
  result += this->origin.asString();
  result += "corner=";
  result += this->corner.asString();
  return result;
  }

/*------------------------------------------------------------------------------
| operator <<                                                                  |
------------------------------------------------------------------------------*/
ostream &operator << ( ostream          &aStream,
                       const IRectangle &aRectangle )
  {
  return aStream << aRectangle.asString();
  }

/*------------------------------------------------------------------------------
| IRectangle::asRECTL                                                          |
------------------------------------------------------------------------------*/
RECTL IRectangle :: asRECTL ( ) const
  {
  RECTL rectl = { minX(), minY(), maxX(), maxY() };
  return rectl;
  }

