/*******************************************************************************
* FILE NAME: istring7.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of the editing functions of          *
*   the class IString.                                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #include <ctype.h>
  }

#include <istring.hpp>

#if !defined( _IBUFFER_ )
  #include <ibuffer.hpp>
#endif

#if !defined( _ISTRTEST_ )
  #include <istrtest.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISTRING7_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IString::center                                                              |
------------------------------------------------------------------------------*/
IString & IString :: center ( unsigned newLength,
                              char      padCharacter )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->center( newLength, padCharacter ) );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::center                                                              |
------------------------------------------------------------------------------*/
IString IString :: center ( const IString &aString,
                unsigned        newLength,
                char            padCharacter )
  {
  return IString(aString).center( newLength, padCharacter );
  }


/*------------------------------------------------------------------------------
| IString::change                                                              |
------------------------------------------------------------------------------*/
IString & IString :: change ( const char *pPattern,
                              unsigned    patternLen,
                              const char *pReplacement,
                              unsigned    replacementLen,
                              unsigned    startPos,
                              unsigned    numChanges )
   {
   IString old( this->buffer() );
   this->setBuffer( this->buffer() -> change( pPattern,
                                              patternLen,
                                              pReplacement,
                                              replacementLen,
                                              startPos,
                                              numChanges ) );
   return *this;
   }

/*------------------------------------------------------------------------------
| IString::change                                                              |
------------------------------------------------------------------------------*/
IString IString :: change ( const IString &aString,
                            const IString &aPattern,
                            const IString &aReplacement,
                            unsigned       startPos,
                            unsigned       numChanges )
  {
  return IString(aString).change( aPattern,
                                  aReplacement,
                                  startPos,
                                  numChanges );
  }

/*------------------------------------------------------------------------------
| IString::change                                                              |
------------------------------------------------------------------------------*/
IString IString :: change ( const IString &aString,
                            const char    *pPattern,
                            const char    *pReplacement,
                            unsigned       startPos,
                            unsigned       numChanges )
  {
  return IString(aString).change( pPattern,
                                  pReplacement,
                                  startPos,
                                  numChanges );
  }

/*------------------------------------------------------------------------------
| IString::copy                                                                |
------------------------------------------------------------------------------*/
IString& IString :: copy ( unsigned numCopies )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->copy( numCopies ) );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::insert                                                              |
------------------------------------------------------------------------------*/
IString & IString :: insert ( const char *pString,
                              unsigned    len,
                              unsigned    index,
                              char        padCharacter )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->insert( pString, len,
                                           index,
                                           padCharacter ) );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::leftJustify                                                         |
------------------------------------------------------------------------------*/
IString & IString :: leftJustify ( unsigned newLength,
                                   char     padCharacter )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->leftJustify( newLength,
                                                padCharacter ) );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::lowerCase                                                           |
------------------------------------------------------------------------------*/
IString& IString :: lowerCase ( )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->lowerCase() );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::overlayWith                                                         |
------------------------------------------------------------------------------*/
IString& IString :: overlayWith ( const char *pOverlay,
                                  unsigned    len,
                                  unsigned    index,
                                  char        padCharacter )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->overlayWith( pOverlay,
                                                len,
                                                index,
                                                padCharacter ) );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::remove                                                              |
|                                                                              |
| Notes:                                                                       |
|   1. The member function IString::remove is implemented in istring0.cpp.     |
------------------------------------------------------------------------------*/
// See istring0.cpp

/*------------------------------------------------------------------------------
| IString::reverse                                                             |
------------------------------------------------------------------------------*/
IString& IString :: reverse ( )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->reverse() );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::rightJustify                                                        |
|                                                                              |
| Notes:                                                                       |
|   1. The member function IString::rightJustify is implemented in             |
|      istring0.cpp.                                                           |
------------------------------------------------------------------------------*/
// See istring0.cpp

/*------------------------------------------------------------------------------
| IString::strip                                                               |
------------------------------------------------------------------------------*/
IString & IString :: strip ( )
  {
  return this->strip( IStringTest( isspace ), IStringEnum::both );
  }

/*------------------------------------------------------------------------------
| IString::strip                                                               |
------------------------------------------------------------------------------*/
IString& IString :: strip ( const char             *pString,
                            unsigned                len,
                            IStringEnum::StripMode  mode )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->strip( pString, len, mode ) );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::strip                                                               |
------------------------------------------------------------------------------*/
IString& IString :: strip ( const IStringTest      &aTest,
                            IStringEnum::StripMode  mode )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->strip( aTest, mode ) );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::translate                                                           |
------------------------------------------------------------------------------*/
IString & IString :: translate ( const char *pInputChars,
                                 unsigned    inputLen,
                                 const char *pOutputChars,
                                 unsigned    outputLen,
                                 char        padCharacter )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->translate( pInputChars, inputLen,
                                              pOutputChars, outputLen,
                                              padCharacter ) );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::upperCase                                                           |
------------------------------------------------------------------------------*/
IString & IString :: upperCase ( )
  {
  IString old( this->buffer() );
  this->setBuffer( this->buffer()->upperCase() );
  return *this;
  }

/*------------------------------------------------------------------------------
| IString::upperCase                                                           |
------------------------------------------------------------------------------*/
IString IString :: upperCase ( const IString &aString )
  {
  return IString(aString).upperCase();
  }
