/*******************************************************************************
* FILE NAME: icslider.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icslider.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #define INCL_WINMESSAGEMGR
   #define INCL_WINWINDOWMGR
   #define INCL_WINDIALOGS
   #define INCL_WINBUTTONS
   #define INCL_OS2MM
   #define INCL_SW
   #define INCL_WINCIRCULARSLIDER
   #define IC_MULTIMEDIA
   #include <iwindefs.h>
   }

#ifndef _IRESLIB_
   #include <ireslib.hpp>
#endif

#ifndef _IWCNAME_
   #include <iwcname.hpp>
#endif
#ifndef _ISTRING_
   #include <istring.hpp>
#endif
#ifndef _IEVENT_
   #include <ievent.hpp>
#endif
#ifndef _IMULTSTA_
   #include <imultsta.hpp>
#endif

#ifndef _IPOINT_
   #include <ipoint.hpp>
#endif
#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif

#include <icslider.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICSLIDER_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLStaticConst)
const ICircularSlider::Style
  ICircularSlider::buttons           ( 0, ICSS_BUTTONS ),
  ICircularSlider::label             ( 0, ICSS_LABEL   ),
  ICircularSlider::displayValue      ( 0, ICSS_VALUE   ),
  ICircularSlider::jumpToPointer     = CSS_POINTSELECT,
  ICircularSlider::full360           = CSS_360,
  ICircularSlider::midpoint          = CSS_MIDPOINT,
  ICircularSlider::proportionalTicks = CSS_PROPORTIONALTICKS,
  ICircularSlider::noTicks           = CSS_NOTICKS,
  ICircularSlider::circularArm       = CSS_CIRCULARVALUE,
  ICircularSlider::classDefaultStyle ( WS_VISIBLE,
                                       ICSS_BUTTONS | ICSS_LABEL | ICSS_VALUE);
#pragma data_seg()

#pragma data_seg(ICLNonConst)
ICircularSlider::Style ICircularSlider::currentDefaultStyle ( WS_VISIBLE,
                                    ICSS_BUTTONS | ICSS_LABEL | ICSS_VALUE);
Boolean ICircularSlider::fRegistered = false;
#pragma data_seg()

/*------------------------------------------------------------------------------
| ICircularSlider::ICircularSlider                                             |
| Construct a new object.                                                      |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider::ICircularSlider(unsigned long Id,
                                 IWindow* parent,
                                 IWindow* owner,
                                const IRectangle& initial,
                                const Style& style)
               : fData(0)
{
   IASSERTPARM(parent!=0);     //parent must be valid
   // Save the extended style to make sure we have a copy of it stored
   setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

   if (!fRegistered)
   {
      fRegistered = true;
      IMultimediaStatics::registerCircularSlider();
   }

   IWindowHandle handle(create(Id, (PSZ)"", convertToGUIStyle( style ),
                               WC_CIRCULARSLIDER, parent->handle(),
                               owner->handle(), initial,
                               PVOID(0), PVOID(0)));

   IWindow::startHandlingEventsFor(handle);

}

/*------------------------------------------------------------------------------
| ICircularSlider::ICircularSlider                                             |
| Construct a new object.                                                      |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider::ICircularSlider(unsigned long Id,
                                 IWindow *parentDialog)
               : fData(0)
{
  setAutoDestroyWindow(false);
  startHandlingEventsFor(Id, parentDialog);
}

/*------------------------------------------------------------------------------
| ICircularSlider::ICircularSlider                                             |
| Construct a new object.                                                      |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider::ICircularSlider(const IWindowHandle& handle)
               : fData(0)
{
  setAutoDestroyWindow(false);
  startHandlingEventsFor(handle);
}

/*------------------------------------------------------------------------------
| ICircularSlider::~ICircularSlider                                            |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider::~ICircularSlider()
{
  if (fData)
     delete fData;
}

/*------------------------------------------------------------------------------
| ICircularSlider::defaultStyle                                                |
| Returns the current default style. This is the same as classDefaultStyle     |
| unless setDefaultStyle has been called.                                      |
------------------------------------------------------------------------------*/
ICircularSlider::Style ICircularSlider::defaultStyle()
{
   return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| ICircularSlider::setDefaultStyle                                             |
| Sets the default style for all subsequent circular sliders.                  |
|                                                                              |
------------------------------------------------------------------------------*/
void ICircularSlider::setDefaultStyle(const Style& style)
{
   currentDefaultStyle = style;
}

/*------------------------------------------------------------------------------
| ICircularSlider::convertToGUIStyle                                           |
| Converts a style object into a value appropriate for the presentation system.|
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long ICircularSlider :: convertToGUIStyle(const IBitFlag& guiStyle,
                                                   Boolean bExtOnly) const
{
  // Obtain the style from the class (ITextControl) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= extendedStyle() & IS_EXTMASK;
  }
  else
  {
    // Let the clean CSS_ styles, upper 12 bits of the lower word of the
    // style, flow thru
    ulStyle |= guiStyle.asUnsignedLong() & ICSS_MASK;

    // add notext style if labels are not desired
    if (!(extendedStyle() & label.asExtendedUnsignedLong()))
       ulStyle |= CSS_NOTEXT;

    // add nobuttons style if buttons are not desired
    if (!(extendedStyle() & buttons.asExtendedUnsignedLong()))
       ulStyle |= CSS_NOBUTTON;

    // add nonumber style if displaying the value is not desired
    if (!(extendedStyle() & displayValue.asExtendedUnsignedLong()))
       ulStyle |= CSS_NONUMBER;
  }

  return( ulStyle );
}

/*------------------------------------------------------------------------------
| ICircularSlider::armRange                                                    |
| Returns the range of values over which the arm can be moved.  This range     |
| contains the minimum and maximum value of the circular slider.               |
------------------------------------------------------------------------------*/
IRange ICircularSlider::armRange() const
{
  short low  = 0;
  short high = 0;

  IEventResult evt(handle().sendEvent(CSM_QUERYRANGE, &low, &high));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_QUERYRANGE");
  return IRange(low, high);
}

/*------------------------------------------------------------------------------
| ICircularSlider::value                                                       |
| Returns the current value of the circular slider.  This value is based on    |
| where the arm is within the current range.                                   |
------------------------------------------------------------------------------*/
long ICircularSlider::value() const
{
  short value = 0;

  IEventResult evt(handle().sendEvent(CSM_QUERYVALUE, &value));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_QUERYVALUE");
  return value;
}

/*------------------------------------------------------------------------------
| ICircularSlider::radius                                                      |
| Returns the radius of the dial in pixels.  This value is indirectly set by   |
| sizing the control.                                                          |
------------------------------------------------------------------------------*/
unsigned long  ICircularSlider::radius() const
{
  unsigned short rad = 0;

  IEventResult evt(handle().sendEvent(CSM_QUERYRADIUS, &rad));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_QUERYRADIUS");
  return rad;
}

/*------------------------------------------------------------------------------
| ICircularSlider::rotationIncrement                                           |
| Returns the increment that the arm is rotated when the user selects one of   |
| the circular slider increment or decrement buttons.                          |
------------------------------------------------------------------------------*/
unsigned long  ICircularSlider::rotationIncrement() const
{
  unsigned short buttonInc = 0;
  unsigned short tickInc   = 0;

  IEventResult evt(handle().sendEvent(CSM_QUERYINCREMENT,
                                      &buttonInc, &tickInc));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_QUERYINCREMENT");
  return buttonInc;
}

/*------------------------------------------------------------------------------
| ICircularSlider::tickSpacing                                                 |
| Returns the tick mark increment of the control.  This                        |
| represents the value between the tick marks around the circular slider.      |
------------------------------------------------------------------------------*/
unsigned long  ICircularSlider::tickSpacing() const
{
  unsigned short buttonInc = 0;
  unsigned short tickInc   = 0;

  IEventResult evt(handle().sendEvent(CSM_QUERYINCREMENT,
                                      &buttonInc, &tickInc));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_QUERYINCREMENT");
  return tickInc;
}

/*------------------------------------------------------------------------------
| ICircularSlider::setArmRange                                                 |
| Sets the range of values over which the arm can be moved.  This range        |
| contains the minimum and maximum value of the circular slider.               |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setArmRange(const IRange& rng)
{
  IEventResult evt(handle().sendEvent(CSM_SETRANGE,
                   (unsigned long)rng.lowerBound(),
                   (unsigned long)rng.upperBound()));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_SETRANGE");
  return *this;
}

/*------------------------------------------------------------------------------
| ICircularSlider::setValue                                                    |
| Sets the current value of the circular slider.  This value is based on       |
| where the arm is within the current range.                                   |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setValue(long value)
{
  IEventResult evt(handle().sendEvent(CSM_SETVALUE,(int)value,(unsigned long)0));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_SETVALUE");
  return *this;
}

/*------------------------------------------------------------------------------
| ICircularSlider::setRotationIncrement                                        |
| Sets the increment that the arm is rotated when the user selects one of      |
| the circular slider increment or decrement buttons.                          |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setRotationIncrement(unsigned long scroll)
{
  IEventResult evt(handle().sendEvent(CSM_SETINCREMENT,
                   (unsigned long)scroll,
                   (unsigned long)tickSpacing()));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_SETINCREMENT");
  return *this;
}

/*------------------------------------------------------------------------------
| ICircularSlider::setTickSpacing                                              |
| Sets the tick mark increment of the control.  This                           |
| represents the value between the tick marks around the circular slider.      |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setTickSpacing(unsigned long tick)
{
  IEventResult evt(handle().sendEvent(CSM_SETINCREMENT,
                   (unsigned long)rotationIncrement(),
                   (unsigned long)tick));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_SETINCREMENT");
  return *this;
}

/*------------------------------------------------------------------------------
| ICircularSlider::setDecrementBitmaps                                         |
| Sets the bitmaps used for decrement buttons of the circular slider.          |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setDecrementBitmaps(const IResourceId& leftUp,
                                                      const IResourceId& leftDown)
{
  return setDecrementBitmaps(leftUp.resourceLibrary().loadBitmap(leftUp.id()),
                             leftDown.resourceLibrary().loadBitmap(leftDown.id()) );
}

/*------------------------------------------------------------------------------
| ICircularSlider::setIncrementBitmaps                                         |
| Sets the bitmaps used for increment buttons of the circular slider.          |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setIncrementBitmaps(const IResourceId& rightUp,
                                                      const IResourceId& rightDown)
{
  return setIncrementBitmaps(rightUp.resourceLibrary().loadBitmap(rightUp.id()),
                             rightDown.resourceLibrary().loadBitmap(rightDown.id()) );
}

/*------------------------------------------------------------------------------
| ICircularSlider::setDecrementBitmaps                                         |
| Sets the bitmaps used for decrement buttons of the circular slider.          |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setDecrementBitmaps(const IBitmapHandle& leftUp,
                                                      const IBitmapHandle& leftDown)
{
  fData  = PCSBITMAPDATA(new CSBITMAPDATA);
  if (fData)
  {
     ((_CSBITMAPDATA*)fData)->hbmLeftUp    = leftUp;
     ((_CSBITMAPDATA*)fData)->hbmLeftDown  = leftDown;
     ((_CSBITMAPDATA*)fData)->hbmRightUp   = 0;
     ((_CSBITMAPDATA*)fData)->hbmRightDown = 0;
  }
  IEventResult evt(handle().sendEvent(CSM_SETBITMAPDATA,
                   PVOID(fData),(unsigned long)0));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_SETBITMAPNDATA");
  return *this;
}

/*------------------------------------------------------------------------------
| ICircularSlider::setIncrementBitmaps                                         |
| Sets the bitmaps used for increment buttons of the circular slider.          |
|                                                                              |
------------------------------------------------------------------------------*/
ICircularSlider& ICircularSlider::setIncrementBitmaps(const IBitmapHandle& rightUp,
                                                      const IBitmapHandle& rightDown)
{
  fData  = PCSBITMAPDATA(new CSBITMAPDATA);
  if (fData)
  {
     ((_CSBITMAPDATA*)fData)->hbmLeftUp    = 0;
     ((_CSBITMAPDATA*)fData)->hbmLeftDown  = 0;
     ((_CSBITMAPDATA*)fData)->hbmRightUp   = rightUp;
     ((_CSBITMAPDATA*)fData)->hbmRightDown = rightDown;
  }
  IEventResult evt(handle().sendEvent(CSM_SETBITMAPDATA,
                   PVOID(fData),(unsigned long)0));
  if (!(evt.asUnsignedLong()))
     ITHROWGUIERROR("CSM_SETBITMAPNDATA");
  return *this;
}

/*------------------------------------------------------------------------------
| ICircularSlider::calcMinimumSize                                             |
| Returns an ISize that indicates the minimum size that                        |
| the circular slider can be (used by the ICanvas hierarchy of classes).       |
------------------------------------------------------------------------------*/
ISize ICircularSlider::calcMinimumSize() const
{
  ISize i(100,70);
  return i;
}
