/*******************************************************************************
* FILE NAME: immaud.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immaud.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #include <string.h>
   #define INCL_MCIOS2
   #define INCL_MMIOOS2
   #define IC_MULTIMEDIA
   #include <iwindefs.h>
   }

#ifndef _ITRACE_
   #include <itrace.hpp>
#endif

#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif

#include <immowin.hpp>
#include <immaud.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMAUD_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::bytesPerSecond                                         |
| Returns the average number of bytes-per-second played or recorded (the       |
| rate that the data is transferred to the hardware).                          |
------------------------------------------------------------------------------*/
unsigned long IMMConfigurableAudio::bytesPerSecond(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long rc = itemStatus(translateAudioFlag(MCI_WAVE_STATUS_AVGBYTESPERSEC), 0, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::bitsPerSample                                          |
| Returns the number of bits-per-sample used to represent each sample of       |
| each channel.  The standard values are 4, 8, and 16.                         |
------------------------------------------------------------------------------*/
unsigned long IMMConfigurableAudio::bitsPerSample(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long rc =itemStatus(translateAudioFlag(MCI_WAVE_STATUS_BITSPERSAMPLE),0, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::blockAlignment                                         |
| Returns the block alignment of the data in bytes.                            |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMConfigurableAudio::blockAlignment(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long rc =itemStatus(translateAudioFlag(MCI_WAVE_STATUS_BLOCKALIGN),0, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::channels                                               |
| Returns the number of audio channels set.  For example:                      |
| mono returns 1; stereo returns 2.                                            |
------------------------------------------------------------------------------*/
unsigned long IMMConfigurableAudio::channels(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long rc = itemStatus(translateAudioFlag(MCI_WAVE_STATUS_CHANNELS), 0,call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::format                                                 |
| Returns the audio format being used.                                         |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioBuffer::Format IMMConfigurableAudio::format(CallType call) const
{
  IASSERTSTATE(isOpen());
  IMMAudioBuffer::Format rc = (IMMAudioBuffer::Format)(itemStatus(
            translateAudioFlag(MCI_WAVE_STATUS_FORMATTAG),0,call));
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::samplesPerSecond                                       |
| Returns the number of samples-per-second played or recorded.                 |
| Standard values are 11025, 22050, and 44100.                                 |
------------------------------------------------------------------------------*/
unsigned long IMMConfigurableAudio::samplesPerSecond(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long rc = itemStatus(translateAudioFlag(MCI_WAVE_STATUS_SAMPLESPERSEC), 0, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::setBytesPerSecond                                      |
| Sets the average bytes per second to be played or recorded.                  |
|                                                                              |
------------------------------------------------------------------------------*/
IMMConfigurableAudio& IMMConfigurableAudio::setBytesPerSecond(unsigned long averageBytes,
                                                     CallType call)
{
  IASSERTSTATE(isOpen());
  MCI_WAVE_SET_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback     = HWND(deviceWindow().handle());
  if (translateAudioFlag(MCI_WAVE_SET_AVGBYTESPERSEC) ==
               MCI_WAVE_SET_AVGBYTESPERSEC)
  {
  parms.ulAvgBytesPerSec = averageBytes;
     sendCommand(MCI_SET, MCI_WAVE_SET_AVGBYTESPERSEC | call, &parms);
  } else {
     parms.ulItem          = translateAudioFlag(MCI_WAVE_SET_AVGBYTESPERSEC);
     parms.ulValue         = averageBytes;
     sendCommand(MCI_SET,MCI_SET_ITEM | call, &parms);
  } /* endif */
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::setBitsPerSample                                       |
| Sets the bits-per-sample to be played or recorded.                           |
| The standard values are 4, 8, and 16.                                        |
------------------------------------------------------------------------------*/
IMMConfigurableAudio& IMMConfigurableAudio::setBitsPerSample(unsigned long bitsPer,
                                             CallType call)
{
  IASSERTSTATE(isOpen());
  MCI_WAVE_SET_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback    = HWND(deviceWindow().handle());
  if (translateAudioFlag(MCI_WAVE_SET_BITSPERSAMPLE) ==
               MCI_WAVE_SET_BITSPERSAMPLE)
  {
     parms.usBitsPerSample = (unsigned short)bitsPer;
     sendCommand(MCI_SET, MCI_WAVE_SET_BITSPERSAMPLE | call, &parms);
  } else {
     parms.ulItem          = translateAudioFlag(MCI_WAVE_SET_BITSPERSAMPLE);
     parms.ulValue         = bitsPer;
     sendCommand(MCI_SET,MCI_SET_ITEM | call, &parms);
  } /* endif */

  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::setBlockAlignment                                      |
| Sets the block alignment of the data in bytes.                               |
|                                                                              |
------------------------------------------------------------------------------*/
IMMConfigurableAudio& IMMConfigurableAudio::setBlockAlignment(unsigned long alignment,
                                                CallType call)
{
  IASSERTSTATE(isOpen());
  MCI_WAVE_SET_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback     = HWND(deviceWindow().handle());
  if (translateAudioFlag(MCI_WAVE_SET_BLOCKALIGN) ==
               MCI_WAVE_SET_BLOCKALIGN)
  {
     parms.usBlockAlign     = (unsigned short)alignment;
     sendCommand(MCI_SET, MCI_WAVE_SET_BLOCKALIGN | call, &parms);
  } else {
     parms.ulItem          = translateAudioFlag(MCI_WAVE_SET_BLOCKALIGN);
     parms.ulValue         = alignment;
     sendCommand(MCI_SET,MCI_SET_ITEM | call, &parms);
  } /* endif */
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::setChannels                                            |
| Sets the number of audio channels.                                           |
|                                                                              |
------------------------------------------------------------------------------*/
IMMConfigurableAudio& IMMConfigurableAudio::setChannels(unsigned long channels, CallType call)
{
  IASSERTSTATE(isOpen());
  MCI_WAVE_SET_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback     = HWND(deviceWindow().handle());
  if (translateAudioFlag(MCI_WAVE_SET_CHANNELS) ==
               MCI_WAVE_SET_CHANNELS)
  {
     parms.usChannels       = (unsigned short)channels;
     sendCommand(MCI_SET, MCI_WAVE_SET_CHANNELS | call, &parms);
  } else {
     parms.ulItem          = translateAudioFlag(MCI_WAVE_SET_CHANNELS);
     parms.ulValue         = channels;
     sendCommand(MCI_SET,MCI_SET_ITEM | call, &parms);
  } /* endif */
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::setFormat                                              |
| Sets the audio format to be used.                                            |
|                                                                              |
------------------------------------------------------------------------------*/
IMMConfigurableAudio& IMMConfigurableAudio::setFormat(IMMAudioBuffer::Format tag, CallType call)
{
  IASSERTSTATE(isOpen());
  MCI_WAVE_SET_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback     = HWND(deviceWindow().handle());
  if (translateAudioFlag(MCI_WAVE_SET_FORMATTAG) ==
               MCI_WAVE_SET_FORMATTAG)
  {
     parms.usFormatTag      = (unsigned short)tag;
     sendCommand(MCI_SET, MCI_WAVE_SET_FORMATTAG | call, &parms);
  } else {
     parms.ulItem          = translateAudioFlag(MCI_WAVE_SET_FORMATTAG);
     parms.ulValue         = tag;
     sendCommand(MCI_SET,MCI_SET_ITEM | call, &parms);
  } /* endif */
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::setSamplesPerSecond                                    |
| Sets the samples rate for playing and recording.                             |
|                                                                              |
------------------------------------------------------------------------------*/
IMMConfigurableAudio& IMMConfigurableAudio::setSamplesPerSecond(unsigned long samplesPer,
                                                CallType call)
{
  IASSERTSTATE(isOpen());
  MCI_WAVE_SET_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback     = HWND(deviceWindow().handle());
  if (translateAudioFlag(MCI_WAVE_SET_SAMPLESPERSEC) ==
               MCI_WAVE_SET_SAMPLESPERSEC)
  {
     parms.ulSamplesPerSec  = samplesPer;
     sendCommand(MCI_SET, MCI_WAVE_SET_SAMPLESPERSEC | call, &parms);
  } else {
     parms.ulItem          = translateAudioFlag(MCI_WAVE_SET_SAMPLESPERSEC);
     parms.ulValue         = samplesPer;
     sendCommand(MCI_SET,MCI_SET_ITEM | call, &parms);
  } /* endif */
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::translateAudioFlag                                     |
| Allows derived classes to map the general audio commands to                  |
| the device-specific values.                                                  |
------------------------------------------------------------------------------*/
unsigned long IMMConfigurableAudio::translateAudioFlag(unsigned long flag) const
{
  return flag;
}

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::IMMConfigurableAudio                                   |
| This constructor creates a new device object.                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMMConfigurableAudio::IMMConfigurableAudio(const IString &baseDeviceName,
                             Boolean openNow,
                             unsigned long instance,
                             Boolean shareable)
           : IMMRecordable(baseDeviceName, openNow, instance, shareable)
{ }

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::IMMConfigurableAudio                                   |
| This constructor wrappers an existing device, and gives a derived class the  |
| ability to specify a desired alias (instead of ICLUI generating it).         |
------------------------------------------------------------------------------*/
IMMConfigurableAudio::IMMConfigurableAudio(unsigned long newDeviceId,
                             const IString &newAlias)
                    : IMMRecordable(newDeviceId, newAlias)
{ }

/*------------------------------------------------------------------------------
| IMMConfigurableAudio::~IMMConfigurableAudio                                  |
| Virtual destructor - nothing special needed as parent does cleanup.          |
|                                                                              |
------------------------------------------------------------------------------*/
IMMConfigurableAudio::~IMMConfigurableAudio()
{ }
