/*******************************************************************************
* FILE NAME: immcdda.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immcdda.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #include <string.h>
   #define INCL_MCIOS2
   #define IC_MULTIMEDIA
   #include <iwindefs.h>
   #include <cdaudos2.h>
   }

#ifndef _ITRACE_
   #include <itrace.hpp>
#endif

#ifndef _IMMTTIME_
   #include <immttime.hpp>
#endif

#include <immcdda.hpp>

#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif

#ifndef _ITHREAD_
   #include <ithread.hpp>
#endif

#include <immowin.hpp>

#ifndef _ITIMER_
   #include <itimer.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif
#ifndef _IMMCDHDR_
  #include <immcdhdr.hpp>
#endif


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMCDDA_CPP_
  #include <ipagetun.h>
#endif


// Initialize static objects
#pragma data_seg(ICLStaticConst)
const INotificationId IMMAudioCD::positionTimerId   = "IMMAudioCD::positionTimerId";
const INotificationId IMMAudioCD::trackStartedId    = "IMMAudioCD::trackStartedId";

const IString IMMAudioCD::discTitleKey("IMMAudioCDDiscTitle");
const IString IMMAudioCD::discProgramKey("IMMAudioCDDiscProgram");
#pragma data_seg()

class TOCRep : public IBase
{
private:

  TOCRep();
  TOCRep(void *newToc, unsigned long num);
  ~TOCRep();

int
  count;
unsigned long
  numTracks;
void
  *toc;

friend class IMMAudioCDContents;
friend class IMMAudioCDContentsData;
};

class IMMAudioCDData : public IBase
{
public:
  IMMAudioCDData(IMMAudioCDContents *toc2,
                 IProfile *prof2,
                 Boolean cdPlaying2,
                 Boolean conPlay2,
                 Boolean loadAndGo2,
                 IMMAudioCDContents::Cursor cursor2,
                 IMMAudioCD::ScanStatus scanStatus2,
                 ITimer* pScanTimer2,
                 ITimer* pUpdateTimer2 )
      : toc(toc2),
        prof( prof2),
        cdPlaying(cdPlaying2),
        conPlay(conPlay2),
        loadAndGo(loadAndGo2),
        cursor(cursor2),
        scanTime(*toc2,1),
        scanStatus(scanStatus2),
        pScanTimer(pScanTimer2),
        pUpdateTimer(pUpdateTimer2),
        deviceHandler(0),
        needToDeleteProfile(false)
     { };

  ~IMMAudioCDData()
     {
       if (toc)
          delete toc;
       if (pScanTimer)
          delete pScanTimer;
       if (pUpdateTimer)
          delete pUpdateTimer;
       if (needToDeleteProfile)
          delete prof;
     };

IMMAudioCDContents
  *toc;
IMMAudioCD::ScanStatus
  scanStatus;
ITimer
  *pScanTimer,
  *pUpdateTimer;
Boolean
  needToDeleteProfile,
  conPlay,
  loadAndGo,
  cdPlaying;
IProfile
  *prof;
IMMAudioCDContents::Cursor
  cursor;
IMMTrackMinSecFrameTime
  scanTime;
IMMAudioCDHandler
  *deviceHandler;
};

class IMMCursorData : public IBase
{
public:
  IMMCursorData(unsigned long ind, Boolean val,
                IMMAudioCDContents* Toc)
         : index(ind),
           valid(val),
           table(Toc),
           sameValidation(0)
     { };
  ~IMMCursorData()
     { };

IMMAudioCDContents
  *table;
unsigned long
  index,
  sameValidation;
Boolean
  valid;
};

class IMMAudioCDContentsData : public IBase
{
public:
  IMMAudioCDContentsData(Boolean val,IString discID,unsigned long *list,
                         TOCRep *reps,unsigned long num)
         : rep(reps),
           discId(discID),
           trackList(list),
           valid(val),
           trackNum(num),
           needValidation(0)
     { }

  ~IMMAudioCDContentsData()
   {
      if (trackList)
         delete trackList;
      rep->count--;
      if (rep->count <= 0)
         delete rep;
   }

Boolean
  valid;
IString
  discId;
TOCRep
  *rep;
unsigned long
  *trackList;
unsigned long
  trackNum,
  needValidation;
};

/*------------------------------------------------------------------------------
| TOCRep::TOCRep                                                               |
| Use this function to create a Table of contents object.                      |
|                                                                              |
------------------------------------------------------------------------------*/
TOCRep::TOCRep()
      : count(0),
        numTracks(0),
        toc(0)
{ }

/*------------------------------------------------------------------------------
| TOCRep::TOCRep                                                               |
| Use this function to create a Table of contents object.                      |
|                                                                              |
------------------------------------------------------------------------------*/
TOCRep::TOCRep(void *newToc, unsigned long num)
      : count(0),
        numTracks(num),
        toc(newToc)
{ }

/*------------------------------------------------------------------------------
| TOCRep::~TOCRep                                                              |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
TOCRep::~TOCRep()
{
  if (toc)
     delete toc;
}

/*------------------------------------------------------------------------------
| Cursor::Cursor                                                               |
| Use this function to create an IMMAudioCDContents::Cursor.                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor::Cursor(const IMMAudioCDContents &Toc)
                          : fCursorData(new IMMCursorData(0,true,0))
{
  fCursorData->table = (IMMAudioCDContents*)(&Toc);
  if ((!Toc.numberOfEntries()) ||
      (!Toc.isValid()        )  )
      invalidate();
  else
      setToFirst();
}

/*------------------------------------------------------------------------------
| Cursor::Cursor                                                               |
| Use this function to create an IMMAudioCDContents::Cursor.                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor::Cursor(const IMMAudioCDContents::Cursor &cursor)
   : fCursorData(new IMMCursorData(cursor.fCursorData->index,
                                   cursor.fCursorData->valid,
                                   cursor.fCursorData->table))
{ }

/*------------------------------------------------------------------------------
| Cursor::Cursor                                                               |
| Use this function to create an IMMAudioCDContents::Cursor.                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor::Cursor()
                          : fCursorData(new IMMCursorData(0,false, 0))
{ }

/*------------------------------------------------------------------------------
| Cursor::operator =                                                           |
| Sets this cursor to be the same as the passed in cursor.                     |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::operator = (const IMMAudioCDContents::Cursor &newCursor)
{
   if (this != &newCursor)  // check for assignment to self
   {
      fCursorData->valid = newCursor.fCursorData->valid;
      fCursorData->index = newCursor.fCursorData->index;
      fCursorData->table = newCursor.fCursorData->table;
      fCursorData->sameValidation = newCursor.fCursorData->sameValidation;
   }
   return *this;
}

/*------------------------------------------------------------------------------
| Cursor::~Cursor                                                              |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor::~Cursor()
{
  if (fCursorData)
     delete fCursorData;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::IMMAudioCDContents                                       |
| Use this function to create an IMMAudioCDContents object.                    |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::IMMAudioCDContents()
  : fContentsData(new IMMAudioCDContentsData(false,IString(),0,new TOCRep,0))
{
   fContentsData->rep->count = 1;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::IMMAudioCDContents                                       |
| Use this function to create an IMMAudioCDContents object.                    |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::IMMAudioCDContents(const IMMAudioCDContents &newToc)
   : fContentsData(new IMMAudioCDContentsData(newToc.fContentsData->valid,
                                              newToc.fContentsData->discId,
                                              0,
                                              newToc.fContentsData->rep,
                                              newToc.numberOfEntries()))

{
  fContentsData->rep->count++;
  unsigned long tr = newToc.numberOfEntries();
  if (tr)
  {
     fContentsData->trackList = new unsigned long[tr];
     memcpy(fContentsData->trackList, newToc.fContentsData->trackList, sizeof(unsigned long) * tr);
  }
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::operator =                                               |
| Sets the contents to be the same as another table of contents.               |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents &IMMAudioCDContents::operator = (const IMMAudioCDContents &newToc)
{
   if (this != &newToc)  // check for assignment to self
   {
      fContentsData->valid = newToc.fContentsData->valid;
      fContentsData->discId = newToc.fContentsData->discId;
      fContentsData->trackNum = newToc.fContentsData->trackNum;

      fContentsData->rep->count--;
      if (fContentsData->rep->count <= 0)
         delete fContentsData->rep;

      newToc.fContentsData->rep->count++;
      fContentsData->rep = newToc.fContentsData->rep;
      if (newToc.fContentsData->trackList)
      {
         delete fContentsData->trackList;
         unsigned long tr = newToc.numberOfEntries();
         if (tr)
         {
            fContentsData->trackList = new unsigned long[tr];
            memcpy(fContentsData->trackList, newToc.fContentsData->trackList, sizeof(unsigned long) * tr);
         }
      }
     fContentsData->needValidation++;
   }
   return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::IMMAudioCDContents                                       |
| Use this function to create an IMMAudioCDContents object.                    |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::IMMAudioCDContents(void *newToc,
                                       const IString &id,
                                       unsigned long tracks)
   : fContentsData(new IMMAudioCDContentsData(true,id,0,new TOCRep(newToc,tracks),tracks))
{
   fContentsData->rep->count = 1;
   if (tracks)
   {
      fContentsData->trackList = new unsigned long[tracks];
      unsigned long *pul = fContentsData->trackList;
      for (unsigned long i = 1; i <= tracks; i++)
         *pul++ = i;
   } /* endif */
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::~IMMAudioCDContents                                      |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::~IMMAudioCDContents()
{
  if (fContentsData)
     delete fContentsData;
}


/*------------------------------------------------------------------------------
| IMMAudioCDContents::controlInformation                                       |
| Returns the track control information at the given cursor.  This value       |
| cannot be set because it is set by the manufacturer of the CD.               |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::controlInformation(const IMMAudioCDContents::Cursor &cursor) const
{
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  return (PTOCREC(fContentsData->rep->toc) - 1 +  //Need to subtract 1 since the tracks are 1 based
          fContentsData->trackList[cursor.asIndex()-1])->Control;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::country                                                  |
| Returns the country at the given cursor.  This value cannot be set because   |
| it is set by the manufacturer of the CD.                                     |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::country(const IMMAudioCDContents::Cursor &cursor) const
{
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  return (PTOCREC(fContentsData->rep->toc) - 1 +      //Need to subtract 1 since the tracks are 1 based
          fContentsData->trackList[cursor.asIndex()-1])->usCountry;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::timeAt                                                   |
| Returns the elapsed time of the entry at the given cursor.                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTime IMMAudioCDContents::timeAt(unsigned long track,Boolean begin) const
{
  IASSERTSTATE(isValid());
  if (begin)
     return IMMTime((PTOCREC(fContentsData->rep->toc) + track - 1)->ulStartAddr);
  else
     return IMMTime((PTOCREC(fContentsData->rep->toc) + track - 1)->ulEndAddr);
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::endOfTrack                                               |
| Returns the ending time (position) of the track.                             |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTime IMMAudioCDContents::endOfTrack(const IMMAudioCDContents::Cursor &cursor) const
{
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  return IMMTime((PTOCREC(fContentsData->rep->toc) - 1 +      //Need to subtract 1 since the tracks are 1 based
                  fContentsData->trackList[cursor.asIndex()-1])->ulEndAddr);
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::owner                                                    |
| Returns the owner of the track at the given cursor.  This value cannot be    |
| set because it is set by the manufacturer of the CD.                         |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::owner(const IMMAudioCDContents::Cursor &cursor) const
{
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  return (PTOCREC(fContentsData->rep->toc) - 1 +      //Need to subtract 1 since the tracks are 1 based
          fContentsData->trackList[cursor.asIndex()-1])->ulOwner;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::serialNumber                                             |
| Returns the serial number of the track at the given cursor.  This value      |
| cannot be set because it is set by the manufacturer of the CD.               |          |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::serialNumber(const IMMAudioCDContents::Cursor &cursor) const
{
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  return (PTOCREC(fContentsData->rep->toc) - 1 +      //Need to subtract 1 since the tracks are 1 based
          fContentsData->trackList[cursor.asIndex()-1])->ulSerialNum;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::startOfTrack                                             |
| Returns the starting time (position) of the track.                           |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTime IMMAudioCDContents::startOfTrack(const IMMAudioCDContents::Cursor &cursor) const
{
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  return IMMTime((PTOCREC(fContentsData->rep->toc) - 1 +      //Need to subtract 1 since the tracks are 1 based
                 fContentsData->trackList[cursor.asIndex()-1])->ulStartAddr);
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::track                                                    |
| Returns the track number at the given cursor.                                |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::track(const IMMAudioCDContents::Cursor &cursor) const
{
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  if (fContentsData->trackList)
     return fContentsData->trackList[cursor.asIndex()-1];
  else
     return 0;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::replaceEntryAt                                           |
| Replaces the track number at the cursor.                                     |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents& IMMAudioCDContents::replaceEntryAt(unsigned long newTrack,
                                                       const Cursor &cursor)
{
  IASSERTPARM(newTrack && newTrack<= numberOfTracks());
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  fContentsData->trackList[cursor.asIndex()-1] = newTrack;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::addEntryAsFirst                                          |
| Adds the track number to the beginning of the playback list.                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents& IMMAudioCDContents::addEntryAsFirst(unsigned long newTrack)
{
  IASSERTPARM(newTrack && newTrack<= numberOfTracks());
  IASSERTSTATE(isValid());
  fContentsData->needValidation++;

  unsigned long tr = numberOfEntries();
  if (tr)
  {
     unsigned long *newTrackList = new unsigned long[tr+1];
     newTrackList[0] = newTrack;
     newTrackList++;
     memcpy(newTrackList, fContentsData->trackList, sizeof(unsigned long) * tr);
     newTrackList--;
     delete fContentsData->trackList;
     fContentsData->trackList = newTrackList;
     fContentsData->trackNum++;
  } /* endif */
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::addEntryAsNext                                           |
| Adds the track number after the cursor in the playback list.                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents& IMMAudioCDContents::addEntryAsNext(unsigned long newTrack,
                                                       const Cursor &cursor)
{
  IASSERTPARM(newTrack && newTrack<= numberOfTracks());
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  fContentsData->needValidation++;

  unsigned long tr = numberOfEntries();
  if (tr)
  {
     unsigned long *newTrackList = new unsigned long[tr+1];
     unsigned long index = cursor.asIndex() ;  //Since the cursor is 0-based
     memcpy(newTrackList, fContentsData->trackList, sizeof(unsigned long) * index);
     newTrackList[index] = newTrack;
     newTrackList+=index + 1;
     fContentsData->trackList+=index;
     memcpy(newTrackList, fContentsData->trackList, sizeof(unsigned long) * (tr-index));
     newTrackList-=(index + 1);
     fContentsData->trackList-=index;
     delete fContentsData->trackList;
     fContentsData->trackList = newTrackList;
     fContentsData->trackNum++;
  } /* endif */
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::removeEntryAt                                            |
| Removes the track number at the cursor from the playback list.               |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents& IMMAudioCDContents::removeEntryAt( const Cursor &cursor)
{
  IASSERTPARM(cursor.isValid());
  IASSERTSTATE(isValid());
  fContentsData->needValidation++;

  unsigned long tr = numberOfEntries();
  if (tr)
  {
     unsigned long *newTrackList = new unsigned long[tr-1];
     unsigned long index = cursor.asIndex()-1;
     memcpy(newTrackList, fContentsData->trackList, sizeof(unsigned long) * index);
     newTrackList+=index;
     fContentsData->trackList+=index + 1;
     memcpy(newTrackList, fContentsData->trackList, sizeof(unsigned long) * (tr-index-1));
     newTrackList-=index;
     fContentsData->trackList-=(index + 1);
     delete fContentsData->trackList;
     fContentsData->trackList = newTrackList;
     fContentsData->trackNum--;
  } /* endif */
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::numberOfTracks                                           |
| Returns the number of tracks.                                                |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::numberOfTracks() const
{
  return fContentsData->rep->numTracks;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::numberOfEntries                                          |
| Returns the number of entries in the playback list.                          |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::numberOfEntries() const
{
  return fContentsData->trackNum;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::IMMAudioCD                                                       |
| Use this function to create an IMMAudioCD object.                            |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD::IMMAudioCD( IProfile* profile,
                        Boolean openNow,
                        unsigned long instance,
                        Boolean openShareable)
          :  IMMRemovableMedia("CDAUDIO", openNow, instance, openShareable),
             fAudioCDData(new IMMAudioCDData(new IMMAudioCDContents(),
                ((profile) ? profile : new IProfile(IProfile::userProfile())),
                false, false, false, IMMAudioCDContents::Cursor(),
                notScanning, 0, 0 )),
             fDelay(!openNow)
{
  if (!profile)
     fAudioCDData->needToDeleteProfile = true;
  if (openNow)
  {
     if (!(mode()==IMMDevice::notReady))
     {
     if (isMediaPresent())
     {
        setTimeFormat(IMMTime::trackMinSecFrame, wait);
        setTableOfContents(getTableFromDisc());
     }
     } /* endif */
  } /* endif */
  fAudioCDData->pScanTimer = new ITimer();
  fAudioCDData->pUpdateTimer = new ITimer();
  fAudioCDData->cursor.setToFirst();
  fAudioCDData->deviceHandler = new IMMAudioCDHandler();
  fAudioCDData->deviceHandler->handleEventsFor(this);
}

/*------------------------------------------------------------------------------
| IMMAudioCD::IMMAudioCD                                                       |
| Use this function to create an IMMAudioCD object.                            |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD::IMMAudioCD(unsigned long newDeviceId,
                        const IString &newAlias)
          : IMMRemovableMedia(newDeviceId, newAlias),
            fAudioCDData(new IMMAudioCDData(new IMMAudioCDContents(),
                               new IProfile(IProfile::userProfile()),
                               false, false, false, IMMAudioCDContents::Cursor(),
                               notScanning, 0, 0 )),
            fDelay(true)
{
  fAudioCDData->needToDeleteProfile = true;
  fAudioCDData->pScanTimer = new ITimer();
  fAudioCDData->pUpdateTimer = new ITimer();
  fAudioCDData->cursor.setToFirst();
  fAudioCDData->deviceHandler = new IMMAudioCDHandler();
  fAudioCDData->deviceHandler->handleEventsFor(this);
}

/*------------------------------------------------------------------------------
| IMMAudioCD::~IMMAudioCD                                                      |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD::~IMMAudioCD()
{
  if (fAudioCDData)
  {
     if (fAudioCDData->deviceHandler)
     {
        fAudioCDData->deviceHandler->stopHandlingEventsFor(this);
        delete fAudioCDData->deviceHandler;
     }
     delete fAudioCDData;
  }
}

/*------------------------------------------------------------------------------
| IMMAudioCD::setProfile                                                       |
| Sets a new profile.   The profile contains one or more compact disc titles   |
| and their IMMAudioCDContents.                                                |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::setProfile (IProfile &profile)
{
  if (fAudioCDData->needToDeleteProfile)
  {
     delete fAudioCDData->prof;
     fAudioCDData->needToDeleteProfile = false;
  }
  fAudioCDData->prof = &profile;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::discId                                                           |
| Returns the compact disc identifier.                                         |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMAudioCD::discId(CallType call) const
{
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());

  MCI_INFO_PARMS parms;
  MCI_CD_ID      cdid;

  memset(&parms, 0, sizeof(parms));
  memset(&cdid, 0, sizeof(cdid));
  parms.hwndCallback = HWND(deviceWindow().handle());
  parms.pszReturn    = PSZ(&cdid);
  parms.ulRetSize    = sizeof(cdid);
  ((IMMAudioCD*)this)->sendCommand(MCI_INFO,
              MCI_CD_INFO_ID | call,
              &parms,
              0);
  if (lastError()== MCIERR_INVALID_BUFFER)
     ((IMMAudioCD*)this)->setLastError(0);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");

  return IString(parms.pszReturn);
}

/*------------------------------------------------------------------------------
| IMMAudioCD::contents                                                         |
| Returns this CD's IMMAudioCDContents (table of contents).                    |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents IMMAudioCD::contents() const
{
  IMMAudioCDContents result;
  if (fAudioCDData->toc)
     result = *(fAudioCDData->toc);
  return result;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::numberOfTracks                                                   |
| Returns the number of tracks.                                                |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCD::numberOfTracks(CallType call) const
{
  unsigned long rc = itemStatus(MCI_STATUS_NUMBER_OF_TRACKS,0, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::uniqueDiscIdentifier                                             |
| Returns the unique disc identifier. The value is the name of this class plus |
| the ID of the disc. This can be useful since not all CDs have a upc.         |
------------------------------------------------------------------------------*/
IString IMMAudioCD::uniqueDiscIdentifier() const
{
  IString result;

  if (isMediaPresent())
     result = IString("IMMAudioCD") + discId(wait);
  return result;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::upc                                                              |
| Returns the disc's universal product code (serial number).  If               |
| the disc does not contain a upc, then a null IString is returned.            |
------------------------------------------------------------------------------*/
IString IMMAudioCD::upc(CallType call) const
{
  IASSERTSTATE(isOpen());

  MCI_INFO_PARMS parms;
  char    temp[8];
  IString result;

  memset(&parms, 0, sizeof(parms));
  memset(temp, 0, sizeof(temp));
  parms.hwndCallback = HWND(deviceWindow().handle());
  parms.pszReturn    = temp;
  parms.ulRetSize    = sizeof(temp);
  ((IMMAudioCD*)this)->sendCommand(MCI_INFO, MCI_CD_INFO_UPC | call,
                                &parms, 0);
  if (lastError()== MCIERR_INVALID_BUFFER)
     ((IMMAudioCD*)this)->setLastError(0);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return result;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::isOpenStringValid                                                |
| Returns true if the string passed in is valid for this device.               |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCD::isOpenStringValid(const IString &deviceName) const
{
  IString tester(deviceName);
  return (tester.upperCase().indexOf("CDAUDIO") == 1 ? true : false);
}

/*------------------------------------------------------------------------------
| IMMAudioCD::stopT                                                            |
| Stops the update timer.                                                      |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::stopT()
{
  fAudioCDData->pUpdateTimer->stop();
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::playing                                                          |
| Returns true if this CD is currently playing.                                |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCD::playing()
{
  return fAudioCDData->cdPlaying;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::getTableFromDisc                                                 |
| Generates a table of contents based on the hard-coded values stored on the   |
| compact disc (Only data stored on the CD are the tracks and their lengths).  |
------------------------------------------------------------------------------*/
IMMAudioCDContents IMMAudioCD::getTableFromDisc()
{
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());

  MCI_TOC_PARMS         parms;
  IMMAudioCDContents  Toc;
  PTOCREC               ptocrec = 0;
  unsigned long         numTracks = numberOfTracks(wait);

  ptocrec         = new (MCI_TOC_REC[numTracks]);
  parms.pBuf      = ptocrec;
  parms.ulBufSize = sizeof(MCI_TOC_REC) * numTracks;
  sendCommand(MCI_GETTOC, MCI_WAIT, &parms);
  if (lastError() == MCIERR_INVALID_BUFFER)
  {
     delete ptocrec;
     parms.pBuf = ptocrec = PTOCREC(new (char[parms.ulBufSize]));
     sendCommand(MCI_GETTOC,
                 MCI_WAIT,
                 &parms);
  }
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  else
     Toc = IMMAudioCDContents(ptocrec, uniqueDiscIdentifier(), numTracks);
  return Toc;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::setTableOfContents                                               |
| Sets this CD's table of contents to the one passed in.                       |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::setTableOfContents(const IMMAudioCDContents &newToc)
{
   if (fAudioCDData->toc)
      delete fAudioCDData->toc;
   IString discId = uniqueDiscIdentifier();
   if (fAudioCDData->prof->containsKeyName(discProgramKey, discId))
   {
      IString temp = fAudioCDData->prof->elementWithKey(discProgramKey, discId);
      fAudioCDData->toc = new IMMAudioCDContents(newToc);
      if (fAudioCDData->toc->fContentsData->trackList)
         delete fAudioCDData->toc->fContentsData->trackList;
      unsigned long * list = (unsigned long *)(unsigned char *)temp;
      unsigned long tr = list[0];
      list++;
      fAudioCDData->toc->fContentsData->trackNum = tr;
      fAudioCDData->toc->fContentsData->trackList = new unsigned long[tr];
      memcpy(fAudioCDData->toc->fContentsData->trackList, list, sizeof(unsigned long) * tr);
   }
   else
      fAudioCDData->toc = new IMMAudioCDContents(newToc);
   IMMAudioCDContents::Cursor temp(*(fAudioCDData->toc));
   fAudioCDData->cursor = temp;
   IMMTime::Format format = timeFormat(wait);
   setTimeFormat(IMMTime::mmTime, wait);
   seek(fAudioCDData->toc->startOfTrack(fAudioCDData->cursor), wait);
   setTimeFormat(format, wait);
   IMMTrackMinSecFrameTime  posTime(*(fAudioCDData->toc), position());
   notifyObservers(INotificationEvent(trackStartedId,*this,true,&posTime));
   return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::discTitle                                                        |
| Returns the disc title. A null IString is returned if you have not set a     |
| title.                                                                       |
------------------------------------------------------------------------------*/
IString IMMAudioCD::discTitle() const
{
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());

  IString result;
  IString discId(uniqueDiscIdentifier());
  if (fAudioCDData->prof->containsKeyName(discTitleKey, discId))
     result = fAudioCDData->prof->elementWithKey(discTitleKey, discId);
  return result;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::goToEntry                                                        |
| Moves the current position to the passed in location.  This location         |
| is a track number and the time into the track.                               |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::goToEntry(unsigned long index)
{
  fAudioCDData->cursor.setToIndex(index);
  IASSERTSTATE(fAudioCDData->cursor.isValid());
  goToCursor();
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::goToEntry                                                        |
| Moves the current position to the passed in location.  This location         |
| is a track number and the time into the track.                               |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::goToEntry(IMMAudioCDContents::Cursor reqCursor)
{
  IASSERTPARM(reqCursor.isValid());
  fAudioCDData->cursor.setToIndex(reqCursor.asIndex());
  IASSERTSTATE(fAudioCDData->cursor.isValid());
  goToCursor();
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::pause                                                            |
| Pauses the audio CD device, if the audio CD device is playing.               |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::pause( CallType call)
{
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());

  IMMPlayableDevice::pause(call);
  if (!lastError() && call == wait && IMMDevice::paused == mode())
     putInStoppedState();
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::play                                                             |
| Starts playing the CD device from the passed in start position               |
| to the passed in end position.                                               |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::play(const IMMTime &from,
                             const IMMTime &to,
                             Boolean resumeIfPaused,
                             CallType call)
{
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  if (fAudioCDData->scanStatus != notScanning)
  {
     fAudioCDData->pScanTimer->stop();
     seek(fAudioCDData->scanTime);
     fAudioCDData->scanStatus = notScanning;
  }
  // if wait put in playing state before play call.
  if (call == wait)
     putInPlayingState();

  if (from.isValid() || to.isValid())
     Inherited::play(from, to, resumeIfPaused, call);
  else
  {
     IMMTime::Format format = timeFormat(wait);
     setTimeFormat(IMMTime::mmTime, wait);
     IMMTime curPos(position());

     IMMTime startTrack = fAudioCDData->toc->startOfTrack(fAudioCDData->cursor);
     IMMTime endTrack   = fAudioCDData->toc->endOfTrack(fAudioCDData->cursor);
     if (curPos >= startTrack && curPos <= endTrack)
        startTrack = curPos;
     else if (curPos < startTrack)
     {
        fAudioCDData->cursor.setToFirst();
        startTrack = fAudioCDData->toc->startOfTrack(fAudioCDData->cursor);
        endTrack   = fAudioCDData->toc->endOfTrack(fAudioCDData->cursor);
     }
     else
     {
        fAudioCDData->cursor.setToLast();
        startTrack = fAudioCDData->toc->startOfTrack(fAudioCDData->cursor);
        endTrack   = fAudioCDData->toc->endOfTrack(fAudioCDData->cursor);
     }
     ITRACE_DEVELOP(IString("starttrack = ") + startTrack.asString());
     Inherited::play(startTrack, endTrack, call);
     setTimeFormat(format, wait);
  }

  // if called for play to complete, put in stopped state
  // else if play started ok, put in playing state
  if (call == wait && IMMDevice::paused == mode())
     putInStoppedState();
  else if (!lastError())
          putInPlayingState();
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::resume                                                           |
| Resumes playback of the compact disc from a paused state.                    |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::resume(Boolean resume, CallType call)
{
  if (!(!resume && IMMDevice::paused == mode()))
     IMMPlayableDevice::resume(call);
  if (call == wait && IMMDevice::playing == mode())
     putInPlayingState();
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::setDiscTitle                                                     |
| Sets the disc's title.  The disc title is stored in the currently set        |
| profile.                                                                     |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::setDiscTitle(const IString &title)
{
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  fAudioCDData->prof->addOrReplaceElementWithKey(discTitleKey, title,
                                   uniqueDiscIdentifier());
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::setProgram                                                       |
| Sets a new table of contents. The table of contents is stored in the         |
| currently set profile.                                                       |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::setProgram(const IMMAudioCDContents &prog)
{
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  if (IMMDevice::playing == mode())
     stop();
  // Need to make sure the user has the right program for the right
  // disc
  IASSERTPARM(uniqueDiscIdentifier() == prog.discId());

  unsigned long num=prog.numberOfEntries();
  IString temp(&num,sizeof(unsigned long),prog.fContentsData->trackList,
               prog.numberOfEntries()*sizeof(unsigned long));
  fAudioCDData->prof->addOrReplaceElementWithKey(discProgramKey, temp,
                                 prog.discId());
  setTableOfContents(prog);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::setTrackTitle                                                    |
| Sets the track's title.  The track title is stored in the currently set      |
| profile.                                                                     |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::setTrackTitle(const IString &trackTitle,
                                                    unsigned long track)
{
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  fAudioCDData->prof->addOrReplaceElementWithKey(IString(track), trackTitle,
                                   uniqueDiscIdentifier());
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::stop                                                             |
| Stops playback and backward or forward scanning of the compact disc.         |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::stop(CallType call)
{
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  if (fAudioCDData->scanStatus != notScanning)
  {
     fAudioCDData->pScanTimer->stop();
     seek(fAudioCDData->scanTime);
     fAudioCDData->scanStatus = notScanning;
  }
  putInStoppedState();
  IMMPlayableDevice::stop(call);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::trackBackward                                                    |
| Moves the current position backwards the passed in number of tracks. If the  |
| device was playing, it starts playing at the new position.                   |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::trackBackward(unsigned long number)
{
  if (number)
     moveOneTrack(false, number);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::trackForward                                                     |
| Moves the current position forewards the passed in number of tracks. If the  |
| device was playing, it starts playing at the new position.                   |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::trackForward(unsigned long number)
{
  if (number)
     moveOneTrack(true, number);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::trackTitle                                                       |
| Returns the track's title.  The track title is stored in the currently set   |
| profile.                                                                     |
------------------------------------------------------------------------------*/
IString IMMAudioCD::trackTitle(unsigned long track) const
{
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  IString result;

  IString discId(uniqueDiscIdentifier());
  IString trackNum(track);
  if (fAudioCDData->prof->containsKeyName(trackNum, discId))
     result = fAudioCDData->prof->elementWithKey(trackNum, discId);
  return result;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::processInternal                                                  |
| This function performs services for IMMAudioCDHandler::dispatchHandlerEvent. |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD& IMMAudioCD::processInternal(const IMMNotifyEvent &evt)
{
  switch (evt.command())
  {

    case IMMNotifyEvent::open:
         if (isOpen())
         {
            setTimeFormat(IMMTime::trackMinSecFrame);
            if (isMediaPresent())
               setTableOfContents(getTableFromDisc());
         } /* endif */
         break;
    // if play is finished see where we are and go on to the next
    // track in the program
    case IMMNotifyEvent::play:
         switch (evt.successCode())
         {
           case IMMNotifyEvent::successful:
                {
                  putInStoppedState();
                  Boolean doPlay = false;
                  if ((fAudioCDData->cursor.asIndex() != fAudioCDData->toc->numberOfEntries()) ||
                      (fAudioCDData->conPlay))
                  {
                     ++(fAudioCDData->cursor);
                     if (fAudioCDData->cursor.isValid())
                     {
                           doPlay = true;
                     }
                     else {
                        fAudioCDData->cursor = IMMAudioCDContents::Cursor(*(fAudioCDData->toc));
                        if (fAudioCDData->conPlay)
                           doPlay = true;
                     }
                  }
                  IMMTime::Format format = timeFormat(wait);
                  setTimeFormat(IMMTime::mmTime, wait);
                  seek(fAudioCDData->toc->startOfTrack(fAudioCDData->cursor), wait);
                  setTimeFormat(format, wait);
                  IMMTrackMinSecFrameTime  posTime(*(fAudioCDData->toc), position());
                  if (doPlay)
                  {
                     setTimeFormat(IMMTime::mmTime, wait);
                     play(fAudioCDData->toc->startOfTrack(fAudioCDData->cursor), fAudioCDData->toc->endOfTrack(fAudioCDData->cursor));
                     setTimeFormat(format, wait);
                  }
                  notifyObservers(INotificationEvent(trackStartedId,*this,true,&posTime));
                }
                break;

            // could have done if/else but may need these cases
            // later; leave for now
            case IMMNotifyEvent::preempted:
            case IMMNotifyEvent::aborted:
            case IMMNotifyEvent::error:
            default:
               if (IMMDevice::playing != mode())
                  putInStoppedState();
            }
         break;

      case IMMNotifyEvent::stop:
      case IMMNotifyEvent::pause:
         if (IMMNotifyEvent::successful == evt.successCode())
            putInStoppedState();
         break;

      case IMMNotifyEvent::resume:
         if (IMMNotifyEvent::successful == evt.successCode() &&
             IMMDevice::playing == mode())
            putInPlayingState();
         break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::handleScan                                                       |
| Handles the scan timer.  This function gets called for each of the scan      |
| timer clicks.                                                                |
------------------------------------------------------------------------------*/
void IMMAudioCD::handleScan(unsigned long timerId)
{
  Boolean trackChanged = false;

  if (fAudioCDData->scanStatus == scanningForward)
  {
     fAudioCDData->scanTime += IMMTime(3000);
     if (!fAudioCDData->scanTime.isValid())
     {
        fAudioCDData->cursor.setToNext();
        trackChanged = true;
        //if (!fAudioCDData->toc->overValidTrack(fAudioCDData->cursor))
        if (!fAudioCDData->cursor.isValid())
        {
           IMMAudioCDContents::Cursor temp(*(fAudioCDData->toc));
           fAudioCDData->cursor = temp;
        }
        fAudioCDData->scanTime = IMMTrackMinSecFrameTime(*(fAudioCDData->toc), fAudioCDData->toc->track(fAudioCDData->cursor), 0, 0, 0);
     }
  }
  else if (fAudioCDData->scanStatus == scanningBackward)
       {
          fAudioCDData->scanTime -= IMMTime(3000);
          if (!fAudioCDData->scanTime.isValid())
          {
             fAudioCDData->cursor.setToPrevious();
             trackChanged = true;
             if (!fAudioCDData->cursor.isValid())
             {
                IMMAudioCDContents::Cursor temp(*(fAudioCDData->toc));
                fAudioCDData->cursor = temp;
                fAudioCDData->cursor.setToLast();
             }
             IMMMinSecFrameTime msf(fAudioCDData->toc->endOfTrack(fAudioCDData->cursor) -
                     fAudioCDData->toc->startOfTrack(fAudioCDData->cursor));
             fAudioCDData->scanTime = IMMTrackMinSecFrameTime(*(fAudioCDData->toc),
                     fAudioCDData->toc->track(fAudioCDData->cursor),
                     msf.minutes(), msf.seconds(), msf.frames());
          }
       }
  if (trackChanged)
     notifyObservers(INotificationEvent(trackStartedId ,*this,true,&fAudioCDData->scanTime));

  notifyObservers(INotificationEvent(positionTimerId,*this,true,&fAudioCDData->scanTime));
}

/*------------------------------------------------------------------------------
| IMMAudioCD::goToCursor                                                       |
| Go to the position specified by the cursor.                                  |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCD::goToCursor()
{
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  IASSERTSTATE(fAudioCDData->toc);
  IASSERTSTATE(fAudioCDData->cursor.isValid());
  IMMDevice::Mode currentMode = mode(wait);
  if (IMMDevice::playing == currentMode)
     stop(wait);

  IMMTime::Format format = timeFormat(wait);
  setTimeFormat(IMMTime::mmTime, wait);
  seek(fAudioCDData->toc->startOfTrack(fAudioCDData->cursor), wait);
  setTimeFormat(format, wait);
  IMMTrackMinSecFrameTime  posTime(*(fAudioCDData->toc), position());
  notifyObservers(INotificationEvent(trackStartedId,*this,true,&posTime));
  if (IMMDevice::playing == currentMode)
  {
     setTimeFormat(IMMTime::mmTime, wait);
     play(IMMTime(), //fAudioCDData->toc->startOfTrack(fAudioCDData->cursor),
     fAudioCDData->toc->endOfTrack(fAudioCDData->cursor));
     setTimeFormat(format, wait);
  }
  return true;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::moveOneTrack                                                     |
| Moves one track forward or backward.                                         |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCD::moveOneTrack(Boolean forward, unsigned long number)
{
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  IASSERTSTATE(fAudioCDData->toc);
  unsigned long loop=number;

  while (loop)
  {
     if (forward)
        ++(fAudioCDData->cursor);
     else
        --(fAudioCDData->cursor);
     if (!fAudioCDData->cursor.isValid())
     {
        IMMAudioCDContents::Cursor temp(*(fAudioCDData->toc));
        fAudioCDData->cursor = temp;
        if (!forward)
           fAudioCDData->cursor.setToLast();
     }
     loop--;
  } /* endwhile */
  return goToCursor();
}

/*------------------------------------------------------------------------------
| IMMAudioCD::putInPlayingState                                                |
| Prepare CD for playing.                                                      |
|                                                                              |
------------------------------------------------------------------------------*/
void IMMAudioCD::putInPlayingState()
{
  fAudioCDData->cdPlaying = true;
  if (! fAudioCDData->pUpdateTimer->isStarted())
     fAudioCDData->pUpdateTimer->start(new ITimerMemberFn<IMMAudioCD>(*this,handleUpdate));
}

/*------------------------------------------------------------------------------
| IMMAudioCD::putInStoppedState                                                |
| Disable CD from playing.                                                     |
|                                                                              |
------------------------------------------------------------------------------*/
void IMMAudioCD::putInStoppedState()
{
  fAudioCDData->cdPlaying = false;
  fAudioCDData->pUpdateTimer->stop();
}

/*------------------------------------------------------------------------------
| IMMAudioCD::handleUpdate                                                     |
| Handles the update timer.  This function gets called for each of the update  |
| timer clicks, and notifies all of the attached observers of the new position.|
------------------------------------------------------------------------------*/
void IMMAudioCD::handleUpdate(unsigned long timerid)
{
   IMMTrackMinSecFrameTime  posTime(*(fAudioCDData->toc), position());
   notifyObservers(INotificationEvent(positionTimerId,*this,true,&posTime));
}

/*------------------------------------------------------------------------------
| IMMAudioCD::startScanningForward                                             |
| Causes the audio CD device to search forward at high speed.                  |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::startScanningForward()
{
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  // if not stopped, stop CD
  // set scanTime to the current position
  // set scanStatus variable, start scantimer
  if (fAudioCDData->scanStatus != notScanning)
  {
     fAudioCDData->pScanTimer->stop();
     seek(fAudioCDData->scanTime);
     fAudioCDData->scanStatus = notScanning;
  }
  if (mode() != IMMDevice::stopped)
     stop();

  IMMTime::Format temp = timeFormat(wait);
  setTimeFormat(IMMTime::trackMinSecFrame, wait);
  IMMTrackMinSecFrameTime mmTime(*(fAudioCDData->toc), position(wait));
  fAudioCDData->scanTime =  mmTime;
  setTimeFormat(temp, wait);

  fAudioCDData->scanStatus = scanningForward;
  if (! fAudioCDData->pScanTimer->isStarted())
     fAudioCDData->pScanTimer->start(new ITimerMemberFn<IMMAudioCD>(*this,handleScan),100);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::startScanningBackward                                            |
| Causes the audio CD device to search backward at high speed.                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::startScanningBackward()
{
  IASSERTSTATE(isOpen());
  IASSERTSTATE(isMediaPresent());
  // if not stopped, stop CD
  // set scanTime to the current position
  // set scanStatus variable, start scantimer
  if (fAudioCDData->scanStatus != notScanning)
  {
     fAudioCDData->pScanTimer->stop();
     seek(fAudioCDData->scanTime);
     fAudioCDData->scanStatus = notScanning;
  }
  if (mode() != IMMDevice::stopped)
     stop();
  IMMTime::Format temp = timeFormat(wait);
  setTimeFormat(IMMTime::trackMinSecFrame, wait);
  IMMTrackMinSecFrameTime mmTime(*(fAudioCDData->toc), position(wait));
  fAudioCDData->scanTime =  mmTime;
  setTimeFormat(temp, wait);
  fAudioCDData->scanStatus = scanningBackward;
  if (! fAudioCDData->pScanTimer->isStarted())
     fAudioCDData->pScanTimer->start(new ITimerMemberFn<IMMAudioCD>(*this,handleScan),100);
  return *this;
}

/*------------------------------------------------------------------------------
| Cursor::isValid                                                              |
| Returns true if the cursor is valid.                                         |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCDContents::Cursor::isValid() const
{
  if ((!fCursorData->valid) || (!fCursorData->table))
     return false;
  if ((fCursorData->index <  fCursorData->table->numberOfEntries()) &&
      (fCursorData->sameValidation == fCursorData->table->fContentsData->needValidation))
     return true;
  else
     return false;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::isValid                                                  |
| Returns true if the table of contents is valid.                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCDContents::isValid() const
{
  if (fContentsData->trackList && fContentsData->valid)
     return true;
  else
     return false;
}

/*------------------------------------------------------------------------------
| Cursor::invalidate                                                           |
| Flags this cursor as invalid.                                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::invalidate()
{
   fCursorData->valid = false;
   return *this;
}

/*------------------------------------------------------------------------------
| Cursor::setToFirst                                                           |
| Points to the first table of contents item and                               |
| validates the cursor.                                                        |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::setToFirst()
{
   fCursorData->index = 0;
   if (fCursorData->table)
   {
      fCursorData->valid = true;
      fCursorData->sameValidation = fCursorData->table->fContentsData->needValidation;
   } /* endif */
   return *this;
}

/*------------------------------------------------------------------------------
| Cursor::setToIndex                                                           |
| Points to the item with the given 0-based index                              |
| and validates the cursor.                                                    |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::setToIndex(unsigned long toIndex)
{
  if (fCursorData->table)
  {
     if ((toIndex - 1 < fCursorData->table->numberOfEntries()) && toIndex)
     {
        fCursorData->index = toIndex - 1;
        fCursorData->valid = true;
        fCursorData->sameValidation = fCursorData->table->fContentsData->needValidation;
     }
     else
        invalidate();
  }
  return *this;
}

/*------------------------------------------------------------------------------
| Cursor::setToLast                                                            |
| Points to the last table of contents item and                                |
| validates the cursor.                                                        |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::setToLast()
{
  if (fCursorData->table)
  {
     fCursorData->index = fCursorData->table->numberOfEntries()-1;
     fCursorData->valid = true;
     fCursorData->sameValidation = fCursorData->table->fContentsData->needValidation;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| Cursor::setToNext                                                            |
| Points to the next table of contents item. If none exists, the cursor is     |
| invalidated.                                                                 |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::setToNext()
{
  if (fCursorData->table)
  {
     if (!(isValid()))
        invalidate();
     else
     {
        if (fCursorData->index+1 >= fCursorData->table->numberOfEntries())
           setToFirst();
        else
        {
           fCursorData->index++;
           fCursorData->valid = true;
           fCursorData->sameValidation = fCursorData->table->fContentsData->needValidation;
        }
     }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| Cursor::setToPrevious                                                        |
| Points to the previous table of contents item. If none exists, the cursor is |
| invalidated.                                                                 |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::setToPrevious()
{
  if (fCursorData->table)
  {
     if (!(isValid()))
        invalidate();
     else
     {
        if (fCursorData->index == 0)
           setToLast();
        else
        {
           fCursorData->index--;
           fCursorData->valid = true;
           fCursorData->sameValidation = fCursorData->table->fContentsData->needValidation;
        }
     }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| Cursor::operator ++                                                          |
| Points to the next table of contents item. If none exists, the cursor is     |
| invalidated.                                                                 |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::operator ++()
{
  return setToNext();
}

/*------------------------------------------------------------------------------
| Cursor::operator --                                                          |
| Points to the previous table of contents item. If none exists, the cursor is |
| invalidated.                                                                 |
------------------------------------------------------------------------------*/
IMMAudioCDContents::Cursor &IMMAudioCDContents::Cursor::operator --()
{
  return setToPrevious();
}


/*------------------------------------------------------------------------------
| Cursor::asIndex                                                              |
| Returns the current index.                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMAudioCDContents::Cursor::asIndex() const
{
  return fCursorData->index + 1;
}

/*------------------------------------------------------------------------------
| IMMAudioCDContents::discId                                                   |
| Returns the compact disc identifier.                                         |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMAudioCDContents::discId() const
{
  return IString(fContentsData->discId);
}

/*------------------------------------------------------------------------------
| IMMAudioCD::isAutoPlayEnabled                                                |
| Returns true if autoplay is turned on.  Autoplay causes the audio CD device  |
| to start playing whenever a CD is loaded.                                    |
------------------------------------------------------------------------------*/
Boolean IMMAudioCD::isAutoPlayEnabled() const
{
  return fAudioCDData->loadAndGo;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::isContinuousPlayEnabled                                          |
| Returns true if continuous play is turned on.                                |
| Continuous play repeats the current track continuously on playback.          |
------------------------------------------------------------------------------*/
Boolean IMMAudioCD::isContinuousPlayEnabled() const
{
  return fAudioCDData->conPlay;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::enableContinuousPlay                                             |
| Enables or disables continuous play.                                         |
| Continuous play repeats the current track continuously on playback.          |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::enableContinuousPlay(Boolean continuous)
{
  fAudioCDData->conPlay = continuous;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::disableContinuousPlay                                            |
| Disables continuous play.                                                    |
| Continuous play repeats the current track continuously on playback.          |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::disableContinuousPlay()
{
  fAudioCDData->conPlay = false;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::disableAutoPlay                                                  |
| Disables autoplay.  Autoplay causes the audio CD device                      |
| to start playing whenever a CD is loaded.                                    |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::disableAutoPlay()
{
  fAudioCDData->loadAndGo = false;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::enableAutoPlay                                                   |
| Disables or enables autoplay.  Autoplay causes the audio CD device           |
| to start playing whenever a CD is loaded.                                    |
------------------------------------------------------------------------------*/
IMMAudioCD &IMMAudioCD::enableAutoPlay(Boolean autoPl)
{
  fAudioCDData->loadAndGo = autoPl;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCD::profile                                                          |
| Returns the current profile. The profile contains one or more compact disc   |
| titles and their IMMAudioCDContents.                                         |
------------------------------------------------------------------------------*/
IProfile IMMAudioCD::profile() const
{
  return *(fAudioCDData->prof);
}

/*------------------------------------------------------------------------------
| IMMAudioCD::changeTable                                                      |
| Removes the current table of contents.                                       |
|                                                                              |
------------------------------------------------------------------------------*/
void IMMAudioCD::changeTable ()
{
  if (fAudioCDData->toc)
     delete fAudioCDData->toc;
  fAudioCDData->toc = 0;
}
