/*******************************************************************************
* FILE NAME: immcdxa.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immcdxa.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #include <string.h>
   #define INCL_MCIOS2
   #define IC_MULTIMEDIA
   #include <iwindefs.h>
   }

#ifndef _ITRACE_
   #include <itrace.hpp>
#endif

#include <immcdxa.hpp>

#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif
#include <immowin.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMCDXA_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMCDXA::IMMCDXA                                                             |
| This constructor creates a new device object.                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMMCDXA::IMMCDXA( Boolean openNow,
                  unsigned long instance,
                  Boolean openShareable)
       : IMMRemovableMedia("CDXA", openNow, instance, openShareable)
{ }


/*------------------------------------------------------------------------------
| IMMCDXA::IMMCDXA                                                             |
| This constructor wrappers an existing device, and gives a derived class the  |
| ability to specify a desired alias (instead of ICLUI generating it).         |
------------------------------------------------------------------------------*/
IMMCDXA::IMMCDXA(unsigned long newDeviceId,
                 const IString &newAlias)
       : IMMRemovableMedia(newDeviceId, newAlias)
{ }

/*------------------------------------------------------------------------------
| IMMCDXA::~IMMCDXA                                                            |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMCDXA::~IMMCDXA()
{ }

/*------------------------------------------------------------------------------
| IMMCDXA::upc                                                                 |
| Returns the disc's universal product code (serial number).  If               |
| the disc does not contain a upc, then a null IString is returned.            |
------------------------------------------------------------------------------*/
IString IMMCDXA::upc(CallType call) const
{
  IASSERTSTATE(isOpen());

  MCI_INFO_PARMS parms;
  char    temp[256];
  IString result;

  memset(&parms, 0, sizeof(parms));
  memset(temp, 0, sizeof(temp));
  parms.hwndCallback = HWND(deviceWindow().handle());
  parms.pszReturn    = temp;
  parms.ulRetSize    = sizeof(temp);
  ((IMMCDXA*)this)->sendCommand(MCI_INFO,
              MCI_CD_INFO_UPC | call, &parms, 0);
  if (lastError()== MCIERR_INVALID_BUFFER)
     ((IMMCDXA*)this)->setLastError(0);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  else
     result = temp;
  return result;
}

/*------------------------------------------------------------------------------
| IMMCDXA::isOpenStringValid                                                   |
| Returns true if the string passed in is valid for this device.               |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMCDXA::isOpenStringValid(const IString &deviceName) const
{
  IString tester(deviceName);
  return (tester.upperCase().indexOf("CDXA") == 1 ? true : false);
}
