/*******************************************************************************
* FILE NAME: immdev.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immdev.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
  {
  #include <string.h>
  #define INCL_MCIOS2
  #define INCL_WINATOM
  #define INCL_WINMESSAGEMGR
  #define IC_MULTIMEDIA
  #include <iwindefs.h>
  }

#include <immdev.hpp>

#ifndef _ITHREAD_
   #include <ithread.hpp>
#endif

#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif

#ifndef _ITRACE_
   #include <itrace.hpp>
#endif

#ifndef _IMMEVT_
   #include <immevt.hpp>
#endif

#ifndef _IMMDVHDR_
   #include <immdvhdr.hpp>
#endif

#ifndef _IMMOWIN_
   #include <immowin.hpp>
#endif

#ifndef _IMULTSTA_
   #include <imultsta.hpp>
#endif
#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMDEV_CPP_
  #include <ipagetun.h>
#endif

// Initialize static objects
#pragma data_seg(ICLStaticConst)
const unsigned long
  IMMDevice::cdxa       = 17,
  IMMDevice::audioCD    = 3,
  IMMDevice::dat        = 4,
  IMMDevice::audioTape  = 5,
  IMMDevice::waveAudio  = 7,
  IMMDevice::sequencer  = 8,
  IMMDevice::ampMixer   = 9,
  IMMDevice::videoTape  = 1,
  IMMDevice::digitalVideo = 12,
  IMMDevice::videoDisc  = 2,
  IMMDevice::overlay    = 10,
  IMMDevice::animation  = 11,
  IMMDevice::speaker    = 13,
  IMMDevice::headphone  = 14,
  IMMDevice::microphone = 15,
  IMMDevice::monitor    = 16,
  IMMDevice::other      = 6,
  IMMDevice::allDevices = 0xFFFF;
#pragma data_seg()

class IMMDeviceData : public IBase
{
public:
  IMMDeviceData(IString anAlias, IString curDev)
         : id(0),
           alias(anAlias),
           errorId(0),
           aDeviceWindow(0),
           deviceHandler(0),
           closeOnDestroy(true),
           currentDevice(curDev),
           sawFirstPassDevice(false),
           openStatus(IMMDevice::openNotAttempted)
     { }

IMMMessageWindow
  *aDeviceWindow;
IMMDeviceHandler
  *deviceHandler;
Boolean
  closeOnDestroy,
  sawFirstPassDevice;
unsigned short
  userParm;
unsigned long
  id,
  errorId;
IString
  alias,
  currentDevice;
IMMDevice::OpenStatus
  openStatus;
static unsigned long
  aliasCount;

};

#pragma data_seg(ICLNonConst)
unsigned long IMMDeviceData::aliasCount = 0;
#pragma data_seg()




/*------------------------------------------------------------------------------
| IMMDevice::acquire                                                           |
| Requests assignment of (access to) the device.  If queued is true, the       |
| request is queued and executed as soon as the device resources are available.|
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::acquire(ShareMode acquire,
                              Boolean queued,
                              CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_GENERIC_PARMS parms;

  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  sendCommand(MCI_ACQUIREDEVICE,
              acquire | call | (queued ? MCI_ACQUIRE_QUEUE : 0),
              &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsStreaming                                                 |
| Returns true if the device can continuously transfer digital data to or from |
| another device, as determined by the device's connections.                   |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsStreaming(CallType call) const
{
  Boolean rc = itemCapability(MCI_GETDEVCAPS_CAN_STREAM, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsDigitalTransfer                                           |
| Returns true if the device can internally process digital data, for example  |
| a digital-to-analog converter on a CD player.                                |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsDigitalTransfer(CallType call) const
{
  Boolean rc =  itemCapability(MCI_GETDEVCAPS_CAN_PROCESS_INTERNAL, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsEject                                                     |
| Returns true if the device can eject media.                                  |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsEject(CallType call) const
{
  Boolean rc =  itemCapability(MCI_GETDEVCAPS_CAN_EJECT, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsDisableEject                                              |
| Returns true if the device can disable the manual                            |
| ejection of media.                                                           |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsDisableEject(CallType call) const
{
  Boolean rc = itemCapability(MCI_GETDEVCAPS_CAN_LOCKEJECT, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsPlay                                                      |
| Returns true if the device can play.                                         |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsPlay(CallType call) const
{
  Boolean rc =  itemCapability(MCI_GETDEVCAPS_CAN_PLAY, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsRecord                                                    |
| Returns true if the device can record.                                       |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsRecord(CallType call) const
{
  Boolean rc =  itemCapability(MCI_GETDEVCAPS_CAN_RECORD, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsRecordInsertion                                           |
| Returns true if the device supports insertion of data while recording.       |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsRecordInsertion(CallType call) const
{
  Boolean rc =  itemCapability(MCI_GETDEVCAPS_CAN_RECORD_INSERT, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsSave                                                      |
| Returns true if the device can save data to some type of media (for example  |
| a file).                                                                     |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsSave(CallType call) const
{
  Boolean rc =  itemCapability(MCI_GETDEVCAPS_CAN_SAVE, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsVolumeAdjustment                                          |
| Returns true if the device supports software control of its volume.          |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsVolumeAdjustment(CallType call) const
{
  Boolean rc =  itemCapability(MCI_GETDEVCAPS_CAN_SETVOLUME, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsAudio                                                     |
| Returns true if the device has support for audio playback.                   |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsAudio(CallType call) const
{
  Boolean rc =  itemCapability(MCI_GETDEVCAPS_HAS_AUDIO, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::supportsVideo                                                     |
| Returns true if the device has support for video playback.                   |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsVideo(CallType call) const
{
  Boolean rc =  itemCapability(MCI_GETDEVCAPS_HAS_VIDEO, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::itemCapability                                                    |
| Gets the device's capabilities.                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::itemCapability(unsigned long item,
                                        CallType call) const
{
  IASSERTSTATE(isOpen());

  MCI_GETDEVCAPS_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  parms.ulItem       = item;
  ((IMMDevice*)this)->sendCommand(MCI_GETDEVCAPS,
              MCI_GETDEVCAPS_ITEM | call,
              &parms);
  return parms.ulReturn;
}


/*------------------------------------------------------------------------------
| IMMDevice::supportsCommand                                                   |
| Returns true if the device supports the passed in command.                   |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::supportsCommand(IMMNotifyEvent::Command command,
                                  CallType call) const
{
  IASSERTSTATE(isOpen());

  MCI_GETDEVCAPS_PARMS parms;

  memset(&parms, 0, sizeof(MCI_GETDEVCAPS_PARMS));
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  parms.usMessage    = (unsigned short)command;
  ((IMMDevice*)this)->sendCommand(MCI_GETDEVCAPS,
              MCI_GETDEVCAPS_MESSAGE | call,
              &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return parms.ulReturn;
}

/*------------------------------------------------------------------------------
| IMMDevice::requiresFiles                                                     |
| Returns true if the device requires the use of files.                        |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::requiresFiles(CallType call) const
{
  Boolean rc =  itemCapability(MCI_GETDEVCAPS_USES_FILES, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::close                                                             |
| Closes the device.                                                           |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::close(CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_GENERIC_PARMS parms;

  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  sendCommand(MCI_CLOSE, call, &parms);
  if (lastError() && call == IMMDevice::wait)
     ITHROWMMERROR(lastError(),"sendCommand");
  releaseAlias(fDeviceData->alias);
  fDeviceData->id = 0;
  fDeviceData->currentDevice = "";
  fDeviceData->sawFirstPassDevice = false;
  fDeviceData->openStatus = openNotAttempted;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::deviceType                                                        |
| Returns the type of the device.                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::deviceType(CallType call) const
{
  unsigned long rc = itemCapability(MCI_GETDEVCAPS_DEVICE_TYPE, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}


/*------------------------------------------------------------------------------
| IMMDevice::mode                                                              |
| Returns the current mode of the device.                                      |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice::Mode IMMDevice::mode(CallType call) const
{
  IMMDevice::Mode rc = (IMMDevice::Mode)itemStatus(MCI_STATUS_MODE, 0, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::open                                                              |
| Opens the file or device. First creates an alias for the device, then calls  |
| sendString to delegate most of the work to the media control interface.      |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::open(const IString &fileOrDevice,
                                       Boolean shareable,
                                       CallType call)
{
  IString temp;
  if (fileOrDevice.length() == 0)
  {
     temp=deviceName();
  } else {
     temp=fileOrDevice;
  } /* endif */
  IString resultString(0,512);

  if (!isOpen() && isOpenStringValid(temp))
  {
     establishAlias();
     if (aliasName().length())
     {
        IString str("open ");
        if (temp.length() != 0)
        {
           str += temp;
           fDeviceData->currentDevice = temp;
        }
        else if (fDeviceData->currentDevice.length() !=0 )
                str += fDeviceData->currentDevice;
        if (shareable)
           str += " shareable";
        str += " alias " + aliasName() + (call == wait ? " wait" : " notify");
        resultString = sendCommand(str);
        fDeviceData->id = resultString.asUnsigned();
        if (call == nowait)
        {
           if (lastError())
              fDeviceData->aDeviceWindow->handle().postEvent(MM_MCINOTIFY,
                IEventParameter1((unsigned short)(lastError()),userParameter()), IEventParameter2(0,MCI_OPEN));
        }
        else if (call == wait)
        {
           if (lastError())
              ITHROWMMERROR(lastError(),"sendCommand");
           else
           {
              setOpenStatus(openSuccessful);
              fDeviceData->aDeviceWindow->handle().sendEvent(MM_MCINOTIFY,
                 IEventParameter1(IMMNotifyEvent::successful,userParameter()), IEventParameter2(0,MCI_OPEN));
           }
        } /* endif */
     }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::openOnThread                                                      |
| Opens the file or device, creating a thread to do the actual opening.  This  |
| allows pre-emption, thus freeing up the windowing system.                    |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::openOnThread( const IString &fileOrDevice,
                                    Boolean shareable,
                                    CallType call)
{
 IString temp;
  QMSG qmsg;

  class OpenSyncFn : public IThreadFn
  {
  public:
    OpenSyncFn(IMMDevice* dev, const IString& fileOrDev, Boolean shareable)
             : device(dev),
               file(fileOrDev),
               share(shareable)
               {;}
  void
    run() { device->open(file, share, nowait); }

  private:
  //
  //
  //
  IMMDevice
    *device;
  const IString
    file;
  Boolean
    share;
  };

  // Create a thread object
  //
  if (fileOrDevice.length() == 0)
  {
     temp=deviceName();
  } else {
     temp=fileOrDevice;
  } /* endif */
  OpenSyncFn *openSync = new OpenSyncFn(this, temp, shareable);
  IThread thread(openSync, true);

  while (openFailed != openStatus())
  {
    if (WinPeekMsg(IThread::current().anchorBlock(),
                   &qmsg,
                   (HWND)0,
                   0,
                   0,
                   PM_REMOVE))
    {
       WinDispatchMsg(IThread::current().anchorBlock(), &qmsg);
    }

    if (isOpen())
       break;
  }
  if (openFailed == openStatus())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::prerollTime                                                       |
| Returns the amount of preroll time.                                          |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTime  IMMDevice::prerollTime(CallType call) const
{
  unsigned long temp = itemCapability(MCI_GETDEVCAPS_PREROLL_TIME, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  if (temp)
     return IMMTime(temp);
  else
     return IMMTime();
}

/*------------------------------------------------------------------------------
| IMMDevice::prerollType                                                       |
| Returns the preroll characteristics of the device.                           |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice::PrerollType IMMDevice::prerollType(CallType call) const
{
  IMMDevice::PrerollType rc = IMMDevice::PrerollType(itemCapability(MCI_GETDEVCAPS_PREROLL_TYPE, call));
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::description                                                       |
| Returns the description of the hardware associated with this device.         |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMDevice::description(CallType call) const
{
  IASSERTSTATE(isOpen());

  char           product[MAX_PRODINFO + 1];
  MCI_INFO_PARMS parms;
  IString        result;

  memset(&parms, 0, sizeof(MCI_INFO_PARMS));
  memset(product, 0, sizeof(product));
  parms.pszReturn = product;
  parms.ulRetSize = MAX_PRODINFO;
  ((IMMDevice*)this)->sendCommand(MCI_INFO, MCI_INFO_PRODUCT | call,
                                  (void *)&parms, 0);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  result = product;
  return result;
}

/*------------------------------------------------------------------------------
| IMMDevice::isAcquired                                                        |
| Returns true if the device is ready.  A device must be acquired (in ready    |
| state) before it can be used.                                                |
------------------------------------------------------------------------------*/
Boolean IMMDevice::isAcquired(CallType call) const
{
  Boolean rc = itemStatus(MCI_STATUS_READY,0, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::speedFormat                                                       |
| Returns the currently set speed format for the device.                       |
|                                                                              |
------------------------------------------------------------------------------*/
IMMSpeed::Format IMMDevice::speedFormat(CallType call) const
{
  IMMSpeed::Format rc = (IMMSpeed::Format)itemStatus(MCI_STATUS_SPEED_FORMAT,0, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDevice::setSpeedFormat                                                    |
| Sets the speed format for the device.                                        |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setSpeedFormat(IMMSpeed::Format format,
                                     CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_SET_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback  = HWND(fDeviceData->aDeviceWindow->handle());
  parms.ulSpeedFormat = (unsigned long)format;
  sendCommand(MCI_SET,
              MCI_SET_SPEED_FORMAT | call,
              &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::setTimeFormat                                                     |
| Sets the time format for the device.                                         |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setTimeFormat(IMMTime::Format format,
                                    CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_SET_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  parms.ulTimeFormat = (unsigned long)format;
  sendCommand(MCI_SET, MCI_SET_TIME_FORMAT | call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::timeFormat                                                        |
| Returns the currently set time format for the device.                        |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTime::Format IMMDevice::timeFormat(CallType call) const
{
  IMMTime::Format rc = (IMMTime::Format)itemStatus(MCI_STATUS_TIME_FORMAT,0, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}


/*------------------------------------------------------------------------------
| IMMDevice::setVolume                                                         |
| Sets the volume for the device.                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setVolume(unsigned long volume,
                                IMMDevice::AudioChannel channel,
                                const IMMMillisecondTime &over,
                                CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_SET_PARMS parms;
  Boolean       includeOver = false;

  memset(&parms, 0, sizeof(MCI_SET_PARMS));
  includeOver = over.isValid();
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  parms.ulAudio      = (unsigned long)channel;
  parms.ulLevel      = (unsigned long)volume;
  parms.ulOver       = (includeOver ? (unsigned long)over : 0);
  sendCommand(MCI_SET, MCI_SET_AUDIO | MCI_SET_VOLUME |
              (includeOver ?  MCI_OVER : 0) | call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}


/*------------------------------------------------------------------------------
| IMMDevice::enableAudio                                                       |
| Turns on or turns off the audio for the device.                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::enableAudio( Boolean enable,
                                   IMMDevice::AudioChannel channel,
                                   const IMMMillisecondTime &over,
                                   CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_SET_PARMS parms;
  Boolean       includeOver;

  memset(&parms, 0, sizeof(MCI_SET_PARMS));
  includeOver = over.isValid();
  parms.ulOver       = (includeOver ? (unsigned long)over : 0);
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  parms.ulAudio      = channel;
  sendCommand(MCI_SET, MCI_SET_AUDIO |
              (enable ? MCI_SET_ON : MCI_SET_OFF) |
              (includeOver ?  MCI_OVER : 0) | call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::disableAudio                                                      |
| Turns off the audio for the device.                                          |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::disableAudio(IMMDevice::AudioChannel channel,
                                         const IMMMillisecondTime &over,
                                         CallType call)
{
  IASSERTSTATE(isOpen());
  enableAudio(false,channel,over,call);
  return *this;
}


/*------------------------------------------------------------------------------
| IMMDevice::sendCommand                                                       |
| Sends a command string to the device.  Ultimately calls the mciSendString    |
| function.                                                                    |
------------------------------------------------------------------------------*/
IString   IMMDevice::sendCommand(const IString &command)
{
  return sendCommand(command, userParameter());
}

/*------------------------------------------------------------------------------
| IMMDevice::sendCommand                                                       |
| Sends a command string to the device.  Ultimately calls the mciSendString    |
| function.                                                                    |
------------------------------------------------------------------------------*/
IString  IMMDevice::sendCommand(const IString &command,
                                unsigned short userParms)
{
  unsigned long rc;
  IString achResult(0,513);

  rc = IMultimediaStatics::sendString(command,
                     achResult,
                     (unsigned short)(achResult.length()),
                     HWND(fDeviceData->aDeviceWindow->handle()),
                     userParms);

  rc &= 0xFFFF;
  if (rc)
  {
     if (rc != MCIERR_INSTANCE_INACTIVE)
     {
        setLastError(rc);
     } else {
        acquire();
        return sendCommand(command,userParms);
     } /* endif */
  } /* endif */
  else
     setLastError(0);
  return achResult;
}

/*------------------------------------------------------------------------------
| IMMDevice::sendCommand                                                       |
| Sends a command to the device.  Ultimately calls the mciSendCommand          |
| function.                                                                    |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::sendCommand(unsigned short message,
                            unsigned long param1,
                            void *param2)
{
  sendCommand(message, param1, param2, userParameter());
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::sendCommand                                                       |
| Sends a command to the device.  Ultimately calls the mciSendCommand          |
| function.                                                                    |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::sendCommand(unsigned short message,
                            unsigned long param1,
                            void *param2,
                            unsigned short userParms)
{
  unsigned long rc;
  rc = IMultimediaStatics::sendCommand((unsigned short)deviceId(),
                      message,
                      param1,
                      param2,
                      userParms);

  rc &= 0xFFFF;
  if (rc)
  {
     if (rc == MCIERR_INSTANCE_INACTIVE)
     {
        acquire();
        sendCommand( message, param1, param2);
     } else {
        setLastError(rc);
     } /* endif */
  } else {
     setLastError(0);
     if (param1 & MCI_WAIT)
     {
        IMMNotifyEvent notifyEvent(IEvent(fDeviceData->aDeviceWindow->handle(),
                                          MM_MCINOTIFY,
                                          IEventParameter1(0,userParameter()),
                                          IEventParameter2(0,message)));
        notifyObservers(INotificationEvent(commandNotifyId,*this,true,&notifyEvent));
     } /* endif */
  }
  return *this;
}


/*------------------------------------------------------------------------------
| IMMDevice::setUserParameter                                                  |
| Sets the user parameter.  This parameter is automatically passed in          |
| most of the notification messages.                                           |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setUserParameter(unsigned short newUserParm)
{
  fDeviceData->userParm = newUserParm;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::userParameter                                                     |
| Returns the user parameter.                                                  |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned short IMMDevice::userParameter() const
{
  return fDeviceData->userParm;
}

/*------------------------------------------------------------------------------
| IMMDevice::lastError                                                         |
| Returns the return code of the last low-level function call.                 |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::lastError() const
{
  return fDeviceData->errorId;
}

/*------------------------------------------------------------------------------
| IMMDevice::setLastError                                                      |
| Sets the return code of the last low-level function call.                    |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setLastError(unsigned long errorId)
{
  fDeviceData->errorId=errorId;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::release                                                           |
| Releases exclusive use of the device resources.                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::release(CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_GENERIC_PARMS parms;

  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  sendCommand(MCI_RELEASEDEVICE, call, &parms);
  return *this;
}


/*------------------------------------------------------------------------------
| IMMDevice::uniqueAlias                                                       |
| Uses the atom table to create a unique alias.                                |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMDevice::uniqueAlias()
{
  HATOMTBL htbl;
  IString  base("IMMDevice");
  Boolean  foundUnique = false;
  IString  atom;

  htbl = WinQuerySystemAtomTable();
  while (!foundUnique)
  {
    fDeviceData->aliasCount++;
    atom = base + IString(fDeviceData->aliasCount);
    if (!WinFindAtom(htbl, atom))
    {
       WinAddAtom(htbl, atom);
       foundUnique = true;
       break;
    }
  }
  return atom;
}

/*------------------------------------------------------------------------------
| IMMDevice::volume                                                            |
| Returns the currently set volume for the device.                             |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::volume(AudioChannel channel ,CallType call) const
{
  unsigned long temp = itemStatus(MCI_STATUS_VOLUME, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  if (channel == right)
     return temp>>16;
  else
     return (temp & 0xFFFF);
}

/*------------------------------------------------------------------------------
| IMMDevice::releaseAlias                                                      |
| Frees the atom associated with the passed in alias.                          |
|                                                                              |
------------------------------------------------------------------------------*/
void IMMDevice::releaseAlias(const IString &aliasRel)
{
  ATOM     atom;
  HATOMTBL htbl;

  htbl = WinQuerySystemAtomTable();
  atom = WinFindAtom(htbl, aliasRel);
  if (atom)
     WinDeleteAtom(htbl, atom);
//
}

/*------------------------------------------------------------------------------
| IMMDevice::establishAlias                                                    |
| Assigns a unique alias to the device.                                        |
|                                                                              |
------------------------------------------------------------------------------*/
void IMMDevice::establishAlias()
{
  fDeviceData->alias = uniqueAlias();
}

/*------------------------------------------------------------------------------
| IMMDevice::deletePendingEvents                                               |
| Removes all of the events of the type passed in from the queue.              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::deletePendingEvents(EventType evtType)
{
  QMSG qmsg;

  while (WinPeekMsg(IThread::current().anchorBlock(),
                    &qmsg,
                    fDeviceData->aDeviceWindow->handle(),
                    (evtType == allEvents) ? notifyEvent : evtType,
                    (evtType == allEvents) ? synchEvent  : evtType,
                    PM_REMOVE));
  return *this;
}



/*------------------------------------------------------------------------------
| IMMDevice::itemStatus                                                        |
| Returns the status of the item passed.                                       |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::itemStatus(unsigned long item,
                                      unsigned long value,
                                      CallType call) const
{
  IASSERTSTATE(isOpen());

  MCI_STATUS_PARMS parms;

  memset(&parms, 0, sizeof(MCI_STATUS_PARMS));
  parms.ulItem       = item;
  parms.ulValue      = value;
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  ((IMMDevice*)this)->sendCommand(MCI_STATUS, MCI_STATUS_ITEM | call,
                                  (void *)&parms, 0);
  return parms.ulReturn;
}


/*------------------------------------------------------------------------------
| IMMDevice::isOpenStringValid                                                 |
| Returns true if the string passed in is valid for this device.               |
| Derived classes should override this function for more specific handling.    |
------------------------------------------------------------------------------*/
Boolean IMMDevice::isOpenStringValid(const IString &deviceName) const
{
  // By default, the string looks good. Let the open occur.
  return true;
}

/*------------------------------------------------------------------------------
| IMMDevice::IMMDevice                                                         |
| This constructor creates a new device object and sets up its event handling  |
| protocol.                                                                    |
------------------------------------------------------------------------------*/
IMMDevice::IMMDevice(const IString& fileOrDevice )
         : fDeviceData(new IMMDeviceData(IString(),fileOrDevice))
{
  fDeviceData->aDeviceWindow = new IMMMessageWindow(*this);
  fDeviceData->deviceHandler = new IMMDeviceHandler();
  fDeviceData->deviceHandler->handleEventsFor(this);
}

/*------------------------------------------------------------------------------
| IMMDevice::IMMDevice                                                         |
| This constructor wrappers an existing device, and gives a derived class the  |
| ability to specify a desired alias (instead of ICLUI generating it).         |
------------------------------------------------------------------------------*/
IMMDevice::IMMDevice(unsigned long newDeviceId, const IString &newAlias)
         : fDeviceData(new IMMDeviceData(newAlias,IString()))
{
  fDeviceData->id          = newDeviceId;
  fDeviceData->openStatus = openSuccessful;
  fDeviceData->sawFirstPassDevice = true;
  fDeviceData->closeOnDestroy = true;
  fDeviceData->aDeviceWindow = new IMMMessageWindow(*this);
  fDeviceData->deviceHandler = new IMMDeviceHandler();
  fDeviceData->deviceHandler->handleEventsFor(this);
}

/*------------------------------------------------------------------------------
| IMMDevice::~IMMDevice                                                        |
| This destructor performs final housekeeping.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice::~IMMDevice()
{
  if (fDeviceData)
  {
     if (isOpen() && fDeviceData->closeOnDestroy)
        close(nowait);
     if (fDeviceData->deviceHandler)
     {
        fDeviceData->deviceHandler->stopHandlingEventsFor(this);
        delete fDeviceData->deviceHandler;
     }

     if (fDeviceData->aDeviceWindow)
        delete fDeviceData->aDeviceWindow;
     delete fDeviceData;
  } /* endif */
}


/*------------------------------------------------------------------------------
| IMMDevice::open                                                              |
| Opens the device. First creates an alias for the device, then calls          |
| sendString to delegate most of the work to the media control interface.      |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::open(unsigned long instance,
                           Boolean shareable,
                           CallType call)
{
  IString devName = deviceName();

  if (instance)
  {
     devName += IString(instance).rightJustify(2, '0');
  }
  open(devName, shareable, call);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::openOnThread                                                      |
| Opens the device, creating a thread to do the actual opening.  This          |
| allows pre-emption, thus freeing up the windowing system.                    |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::openOnThread(unsigned long instance,
                                   Boolean shareable)
{
  IString devName = deviceName();

  if (instance)
  {
     devName += IString(instance).rightJustify(2, '0');
  }
  openOnThread(devName, shareable);
  return *this;
}



/*------------------------------------------------------------------------------
| IMMDevice::isCloseOnDestroy                                                  |
| Returns true if the device closes itself when the device object is deleted.  |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean   IMMDevice::isCloseOnDestroy() const
{
  return fDeviceData->closeOnDestroy;
}

/*------------------------------------------------------------------------------
| IMMDevice::setCloseOnDestroy                                                 |
| Closes the device when this object is destroyed.  This is the default.       |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setCloseOnDestroy(Boolean close)
{
  fDeviceData->closeOnDestroy = close;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::deviceWindow                                                      |
| Returns the window object that the device uses for communication with the    |
| operating system.                                                            |
------------------------------------------------------------------------------*/
IMMMessageWindow& IMMDevice::deviceWindow() const
{
  return *(fDeviceData->aDeviceWindow);
}

/*------------------------------------------------------------------------------
| IMMDevice::deviceId                                                          |
| Returns the ID of the current device.  This value is                         |
| used by the system to identify the device.                                   |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::deviceId() const
{
  return fDeviceData->id;
}

/*------------------------------------------------------------------------------
| IMMDevice::aliasName                                                         |
| Returns the alias associated with the device                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMDevice::aliasName() const
{
  return fDeviceData->alias;
}


/*------------------------------------------------------------------------------
| IMMDevice::deviceName                                                        |
| Returns the name of the device. This could be a file name or a device name.  |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMDevice::deviceName() const
{
  return fDeviceData->currentDevice;
}


/*------------------------------------------------------------------------------
| IMMDevice::wasPassDeviceRequested                                            |
| Returns true if a passDevice event has already been processed.               |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::wasPassDeviceRequested() const
{
  return fDeviceData->sawFirstPassDevice;
}

/*------------------------------------------------------------------------------
| IMMDevice::openStatus                                                        |
| Returns the result of the open attempt.                                      |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice::OpenStatus IMMDevice::openStatus() const
{
  return fDeviceData->openStatus;
}

/*------------------------------------------------------------------------------
| IMMDevice::setOpenStatus                                                     |
| Sets the open device flag.                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setOpenStatus(OpenStatus success)
{
  fDeviceData->openStatus = success;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::setPassDeviceRequested                                            |
| Sets the pass device flag.  This indicates that the                          |
| device object has received use of the actual hardware device.                |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::setPassDeviceRequested(Boolean sawPassDevice)
{
  fDeviceData->sawFirstPassDevice = sawPassDevice;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::isOpen                                                            |
| Returns true if the device is opened.                                        |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::isOpen() const
{
  return ((fDeviceData->id                            &&
           openStatus() == IMMDevice::openSuccessful
         &&  wasPassDeviceRequested()) ? true : false);
//
}

/*------------------------------------------------------------------------------
| IMMDevice::connectedDeviceId                                                 |
| Returns the Id of the device that is connected to the passed in              |
| connector type of this device.                                               |
------------------------------------------------------------------------------*/
unsigned long IMMDevice::connectedDeviceId(ConnectorType type,
                                    CallType call) const
{
  IASSERTSTATE(isOpen());

  MCI_CONNECTION_PARMS parms;

  memset(&parms, 0, sizeof(MCI_CONNECTION_PARMS));
  parms.ulConnectorType = type;
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  ((IMMDevice*)this)->sendCommand(MCI_CONNECTION,
                        MCI_CONNECTOR_TYPE |
                        MCI_QUERY_CONNECTION | call, (void *)&parms, 0);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return parms.usToDeviceID;
}

/*------------------------------------------------------------------------------
| IMMDevice::enableConnector                                                   |
| Turns on or turns off the connector specified by the passed in connector     |
| type.                                                                        |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::enableConnector(ConnectorType type,
                                      Boolean enable,CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_CONNECTOR_PARMS parms;

  memset(&parms, 0, sizeof(MCI_CONNECTOR_PARMS));
  parms.ulConnectorType = type;
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  sendCommand(MCI_CONNECTOR, (enable ? MCI_ENABLE_CONNECTOR : MCI_DISABLE_CONNECTOR) |
              MCI_CONNECTOR_TYPE | call, (void *)&parms, 0);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDevice::disableConnector                                                  |
| Turns off the connector specified by the passed in connector                 |
| type.                                                                        |
------------------------------------------------------------------------------*/
IMMDevice& IMMDevice::disableConnector(ConnectorType type, CallType call)
{
   return enableConnector(type,false,call);
}


/*------------------------------------------------------------------------------
| IMMDevice::isConnectionSupported                                             |
| Returns true if the connector is valid for this device.                      |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean    IMMDevice::isConnectionSupported(ConnectorType type,CallType call) const
{
  MCI_CONNECTORINFO_PARMS parms;

  memset(&parms, 0, sizeof(MCI_CONNECTORINFO_PARMS));
  parms.ulConnectorType = type;
  parms.ulDeviceTypeID  = deviceType();
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  ((IMMDevice*)this)->sendCommand(MCI_CONNECTORINFO, MCI_ENUMERATE_CONNECTORS |
                                  MCI_CONNECTOR_TYPE | call,
                                  (void *)&parms, 0);
  if (lastError())
  {
     if (lastError() == MCIERR_UNSUPPORTED_CONN_TYPE)
        return false;
     else
        ITHROWMMERROR(lastError(),"sendCommand");
  }
  return true;
}
/*------------------------------------------------------------------------------
| IMMDevice::isConnectorEnabled                                                |
| Returns true if the connector is enabled for this device.                    |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean    IMMDevice::isConnectorEnabled(ConnectorType type,CallType call) const
{
  MCI_CONNECTOR_PARMS parms;

  memset(&parms, 0, sizeof(MCI_CONNECTOR_PARMS));
  parms.ulConnectorType = type;
  parms.hwndCallback = HWND(fDeviceData->aDeviceWindow->handle());
  ((IMMDevice*)this)->sendCommand(MCI_CONNECTOR,
                                  MCI_QUERY_CONNECTOR_STATUS |
                                  MCI_CONNECTOR_TYPE | call,
                                  (void *)&parms, 0);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  if (parms.ulReturn)
     return true;
  else
     return false;
}

/*------------------------------------------------------------------------------
| IMMDevice::isAudioEnabled                                                    |
| Returns true if the audio for the passed in channel is turned on.            |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDevice::isAudioEnabled(AudioChannel channel,
                                  CallType call)
{
  IMODTRACE_DEVELOP("IMMDevice::isAudioEnabled");

  // value contains the correct MMPM constant for the audio channel
  // assume default of all
  unsigned long value = MCI_STATUS_AUDIO_ALL;

  // if not default, stuff the appropriate constant
  if (channel == left)
     value = MCI_STATUS_AUDIO_LEFT;

  if (channel == right)
     value = MCI_STATUS_AUDIO_RIGHT;

  Boolean rc = itemStatus(MCI_STATUS_AUDIO, value, call);

  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");

  return rc;
}
