/*******************************************************************************
* FILE NAME: immdigvd.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immdigvd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #include <string.h>
   #define INCL_MCIOS2
   #define IC_MULTIMEDIA
   #include <iwindefs.h>
   }

#include <immdigvd.hpp>

#ifndef _ITRACE_
   #include <itrace.hpp>
#endif

#ifndef _IMMTIME_
   #include <immtime.hpp>
#endif

#ifndef _IEXCEPT_
   #include <iexcept.hpp>
#endif
#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif
#include <immowin.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMDIGVD_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IMMDigitalVideo::IMMDigitalVideo                                             |
| This constructor creates a new device object.                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDigitalVideo::IMMDigitalVideo( Boolean openNow,
                                  unsigned long instance,
                                  Boolean openShareable)
               : IMMConfigurableAudio("DIGITALVIDEO",  openNow,
                                      instance, openShareable)
{ }

/*------------------------------------------------------------------------------
| IMMDigitalVideo::IMMDigitalVideo                                             |
| This constructor wrappers an existing device, and gives a derived class the  |
| ability to specify a desired alias (instead of ICLUI generating it).         |
------------------------------------------------------------------------------*/
IMMDigitalVideo::IMMDigitalVideo(unsigned long newDeviceId,
                                 const IString &newAlias)
               : IMMConfigurableAudio(newDeviceId, newAlias)
{ }

/*------------------------------------------------------------------------------
| IMMDigitalVideo::~IMMDigitalVideo                                            |
| Virtual destructor - nothing special needed as parent does cleanup.          |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDigitalVideo::~IMMDigitalVideo()
{ }

/*------------------------------------------------------------------------------
| IMMDigitalVideo::supportsSizing                                              |
| Returns true if the device can independently stretch the horizontal          |
| and vertical dimensions of the image.                                        |
------------------------------------------------------------------------------*/
Boolean IMMDigitalVideo::supportsSizing(CallType call) const
{
  IASSERTSTATE(isOpen());
  Boolean rc = itemCapability(MCI_DGV_GETDEVCAPS_CAN_STRETCH, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::supportsReverse                                             |
| Returns true if the device can play in reverse.                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDigitalVideo::supportsReverse(CallType call) const
{
  IASSERTSTATE(isOpen());
  Boolean rc = itemCapability(MCI_DGV_GETDEVCAPS_CAN_STRETCH, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::supportsStretchToFit                                        |
| Returns true if the device can stretch the video frames to fill the          |
| display rectangle.                                                           |
------------------------------------------------------------------------------*/
Boolean IMMDigitalVideo::supportsStretchToFit(CallType call) const
{
  IASSERTSTATE(isOpen());
  Boolean rc = itemCapability(MCI_DGV_GETDEVCAPS_CAN_STRETCH, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::fastSpeed                                                   |
| Returns the device's fast playback rate in the current speed format,         |
| either as a percentage or in frames-per-second.                              |
------------------------------------------------------------------------------*/
IMMSpeed IMMDigitalVideo::fastSpeed(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long temp = itemCapability(MCI_DGV_GETDEVCAPS_FAST_RATE, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"itemCapability");
  return IMMSpeed(speedFormat(),temp);
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::maximumSpeed                                                |
| Returns the device's maximum playback rate in the current speed format,      |
| either as a percentage or in frames-per-second.                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMSpeed IMMDigitalVideo::maximumSpeed(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long temp = itemCapability(MCI_DGV_GETDEVCAPS_MAXIMUM_RATE, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"itemCapability");
  return IMMSpeed(speedFormat(),temp);
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::minimumSpeed                                                |
| Returns the device's minimum playback rate in the current speed format,      |
| either as a percentage or in frames-per-second.                              |
------------------------------------------------------------------------------*/
IMMSpeed IMMDigitalVideo::minimumSpeed(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long temp = itemCapability(MCI_DGV_GETDEVCAPS_MINIMUM_RATE, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"itemCapability");
  return IMMSpeed(speedFormat(),temp);
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::normalSpeed                                                 |
| Returns the device's normal play rate, in the
| current speed format, either as a percentage or in frames-per-second.        |
------------------------------------------------------------------------------*/
IMMSpeed IMMDigitalVideo::normalSpeed(CallType call)  const
{
  IASSERTSTATE(isOpen());
  unsigned long temp = itemCapability(MCI_DGV_GETDEVCAPS_NORMAL_RATE, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"itemCapability");
  return IMMSpeed(speedFormat(),temp);
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::slowSpeed                                                   |
| Returns the device's slow playback rate in the current speed format,         |
| either as a percentage or in frames-per-second.                              |
------------------------------------------------------------------------------*/
IMMSpeed IMMDigitalVideo::slowSpeed(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long temp = itemCapability(MCI_DGV_GETDEVCAPS_SLOW_RATE, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"itemCapability");
  return IMMSpeed(speedFormat(),temp);
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::videoWidth                                                  |
| Returns the nominal (reasonable) width of the video for this machine.        |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDigitalVideo::videoWidth(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long rc = itemCapability(MCI_DGV_GETDEVCAPS_VIDEO_X_EXTENT, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::videoHeight                                                 |
| Returns the nominal (reasonable) height of the video for this machine.       |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDigitalVideo::videoHeight(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long rc = itemCapability(MCI_DGV_GETDEVCAPS_VIDEO_Y_EXTENT, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::maximumWindows                                              |
| Returns the maximum number of video windows allowed on this machine.         |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDigitalVideo::maximumWindows(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long rc = itemCapability(MCI_DGV_GETDEVCAPS_MAX_WINDOWS, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::record                                                      |
| Records from start to the end location.  If there was anything               |
| previously recorded it is deleted before the recording.                      |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::record(Boolean insert,
                                         const IMMTime &from,
                                         const IMMTime &to,
                                         Boolean resumeIfPaused,
                                         CallType call)
{
  IASSERTSTATE(isOpen());
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::record                                                      |
| Records from current to the end location.  If there was anything             |
| previously recorded it is deleted before the recording.                      |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::record(const IMMTime &to,
                                         Boolean resumeIfPaused,
                                         CallType call)
{
  IASSERTSTATE(isOpen());
  MCI_RECORD_PARMS parms;
  unsigned long    flags = 0;

  if (resumeIfPaused && IMMDevice::paused == mode())
     resume(call);
  else
  {
     memset(&parms, 0, sizeof(parms));
     parms.hwndCallback = HWND(deviceWindow().handle());
     if (to.isValid())
     {
        parms.ulTo = (unsigned long)to;
        flags     |= MCI_TO;
     }
     flags |=  call;
     sendCommand(MCI_RECORD, flags, &parms);
  }
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::isMonitoringEnabled                                         |
| Returns true if monitoring of the incoming video signal is enabled.          |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDigitalVideo::isMonitoringEnabled(CallType call) const
{
  IASSERTSTATE(isOpen());
  Boolean rc = itemStatus(MCI_STATUS_MONITOR, 0,call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::enableMonitoring                                            |
| Sets monitoring of the incoming video signal on or off.                      |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::enableMonitoring(Boolean enable, CallType call)
{
  IASSERTSTATE(isOpen());
  MCI_AMP_SET_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback     = HWND(deviceWindow().handle());
  parms.ulItem           = MCI_DGV_SET_MONITOR;
  sendCommand(MCI_SET,
              MCI_SET_ITEM | (enable ? MCI_SET_ON : MCI_SET_OFF) | call,
              &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::disableMonitoring                                           |
| Sets monitoring of the incoming video signal off.                            |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::disableMonitoring(CallType call)
{
  enableMonitoring(false,call);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::fileNormalSpeed                                             |
| Returns the normal play rate of the currently loaded video file, in the      |
| current speed format, either as a percentage or in frames-per-second.        |
------------------------------------------------------------------------------*/
IMMSpeed IMMDigitalVideo::fileNormalSpeed(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long temp = itemStatus(MCI_DGV_STATUS_NORMAL_RATE, 0, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"itemStatus");
  return IMMSpeed(speedFormat(),temp);
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::supportsOverlayGraphics                                     |
| Returns true if the device supports the display by an application            |
| of overlay graphics in a video window.                                       |
------------------------------------------------------------------------------*/
Boolean IMMDigitalVideo::supportsOverlayGraphics(CallType call) const
{
  IASSERTSTATE(isOpen());
  Boolean rc = itemCapability(MCI_DGV_GETDEVCAPS_OVERLAY_GRAPHICS, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::playAt                                                      |
| Plays the video at the specified speed from the start                        |
| position to the end position.                                                |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::playAt(const IMMSpeed& speed,
                                         const IMMTime &from,
                                         const IMMTime &to,
                                         CallType call)
{
  IASSERTSTATE(isOpen());
  return playDigital(from, to, call, MCI_DGV_PLAY_SPEED, speed.speed());
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::playFast                                                    |
| Plays the video at fast speed from the start                                 |
| position to the end position.                                                |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::playFast(const IMMTime &from,
                                           const IMMTime &to,
                                           CallType call)
{
  IASSERTSTATE(isOpen());
  return playDigital(from, to, call, MCI_DGV_PLAY_FAST);
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::isPlayingForward                                            |
| Returns true if the play direction is forward or if the                      |
| device is not playing.                                                       |
------------------------------------------------------------------------------*/
Boolean IMMDigitalVideo::isPlayingForward(CallType call) const
{
  IASSERTSTATE(isOpen());
  Boolean rc = itemStatus(MCI_DGV_STATUS_FORWARD, 0,call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::playScan                                                    |
| Plays frames when indexed; otherwise, plays the video as fast as             |
| possible from the start position to the end position with no audio.          |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::playScan(const IMMTime &from,
                                           const IMMTime &to,
                                           CallType call)
{
  IASSERTSTATE(isOpen());
  return playDigital(from, to, call, MCI_DGV_PLAY_SCAN);
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::playSlow                                                    |
| Plays the video at slow speed from the start                                 |
| position to the end position.                                                |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::playSlow(const IMMTime &from,
                                           const IMMTime &to,
                                           CallType call)
{
  IASSERTSTATE(isOpen());
  return playDigital(from, to, call, MCI_DGV_PLAY_SLOW);
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::refresh                                                     |
| Restores the video window by causing the video device to transfer a video    |
| image to the playback window (similar to a refresh of a normal window.       |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::refresh(const IRectangle& source,
                                          const IRectangle& destination,
                                          CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_RESTORE_PARMS parms;
  unsigned long flags = 0;
  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback = HWND(deviceWindow().handle());
  if (IPair() != source.area())
  {
     flags |= MCI_RESTORE_SRC_RECT;
     parms.SrcRect = source.asRECTL();
  }

  if (IPair() != destination.area())
  {
     flags |= MCI_RESTORE_DEST_RECT;
     parms.DestRect = destination.asRECTL();
  }
  sendCommand(MCI_RESTORE,
              flags | call,
              &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::setWindow                                                   |
| Use this function to supply your own window where the video is played.       |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::setWindow(const IWindowHandle& handle,
                                            CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_VID_WINDOW_PARMS parms;
  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback = HWND(deviceWindow().handle());
  parms.hwndDest     = handle;
  sendCommand(MCI_WINDOW, MCI_VID_WINDOW_HWND | call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::setWindow                                                   |
| Use this function to supply your own window where the video is played.       |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::setWindow(const IWindow& window,
                                            CallType call)
{
  IASSERTSTATE(isOpen());
  setWindow(window.handle(),call);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::setMonitorWindow                                            |
| Use this function to supply your own window for video monitoring.            |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::setMonitorWindow(const IWindowHandle& handle,
                                                   CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_VID_WINDOW_PARMS parms;
  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback = HWND(deviceWindow().handle());
  parms.hwndDest     = handle;
  sendCommand(MCI_WINDOW, MCI_VID_WINDOW_HWND | MCI_DGV_MONITOR | call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::setMonitorWindow                                            |
| Use this function to supply your own window for video monitoring.            |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::setMonitorWindow(const IWindow& window,
                                                   CallType call)
{
  IASSERTSTATE(isOpen());
  setMonitorWindow(window.handle(),call);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::useDefaultWindow                                            |
| Specifies that the digital video class creates and manages its own video     |
| window.                                                                      |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::useDefaultWindow(CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_VID_WINDOW_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback = HWND(deviceWindow().handle());
  sendCommand(MCI_WINDOW, MCI_VID_WINDOW_DEFAULT | call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::handle                                                      |
| Returns the window handle for the current video playback window.             |
|                                                                              |
------------------------------------------------------------------------------*/
IWindowHandle IMMDigitalVideo::handle(CallType call) const
{
  IASSERTSTATE(isOpen());
  IWindowHandle rc = itemStatus(MCI_DGV_STATUS_HWND, 0,call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::useDefaultMonitorWindow                                     |
| Specifies that the digital video class creates and manages its own video     |
| monitor window.                                                              |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::useDefaultMonitorWindow(CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_VID_WINDOW_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback = HWND(deviceWindow().handle());
  sendCommand(MCI_WINDOW, MCI_VID_WINDOW_DEFAULT | MCI_DGV_MONITOR |
              call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::monitorHandle                                               |
| Returns the window handle for the current video monitor window.              |
|                                                                              |
------------------------------------------------------------------------------*/
IWindowHandle IMMDigitalVideo::monitorHandle(CallType call) const
{
  IASSERTSTATE(isOpen());
  IWindowHandle rc = itemStatus(MCI_DGV_STATUS_HWND_MONITOR, 0,call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::videoFileWidth                                              |
| Returns the width of the video for the currently loaded video file.          |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDigitalVideo::videoFileWidth(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long rc = itemStatus(MCI_DGV_STATUS_VIDEO_X_EXTENT, 0,call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::videoFileHeight                                             |
| Returns the height of the video for the currently loaded video file.         |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDigitalVideo::videoFileHeight(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long rc = itemStatus(MCI_DGV_STATUS_VIDEO_Y_EXTENT,0, call);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::speed                                                       |
| Returns the currently set speed for the device.                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMSpeed IMMDigitalVideo::speed(CallType call) const
{
  IASSERTSTATE(isOpen());
  unsigned long temp = itemStatus(MCI_DGV_STATUS_SPEED, 0,call);
  if (lastError())
     ITHROWMMERROR(lastError(),"itemStatus");
  return IMMSpeed(speedFormat(),temp);
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::setDestination                                              |
| Sets the subrectangle within the video window where video is played.  This   |
| allows an application to move, grow, or shrink video inside the window.      |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::setDestination(const IRectangle& rect,
                                                 CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_DGV_RECT_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback = HWND(deviceWindow().handle());
  parms.rc           = rect.asRECTL();
  sendCommand(MCI_PUT,
              MCI_DGV_PUT_DESTINATION | MCI_DGV_PUT_RECT | call,
              &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::sourceRectangle                                             |
| Returns the subrectangle of the video from the currently loaded video        |
| file.                                                                        |
------------------------------------------------------------------------------*/
IRectangle IMMDigitalVideo::sourceRectangle(CallType call) const
{
  IASSERTSTATE(isOpen());
  MCI_DGV_RECT_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback = HWND(deviceWindow().handle());
  ((IMMDigitalVideo*)this)->sendCommand(MCI_WHERE,
                  MCI_VID_WHERE_SOURCE | call,
                  &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return IRectangle(parms.rc);
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::destinationRectangle                                        |
| Returns the subrectangle within the video window where video is played.      |
|                                                                              |
------------------------------------------------------------------------------*/
IRectangle IMMDigitalVideo::destinationRectangle(CallType call) const
{
  IASSERTSTATE(isOpen());

  MCI_DGV_RECT_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback = HWND(deviceWindow().handle());
  ((IMMDigitalVideo*)this)->sendCommand(MCI_WHERE,
                  MCI_VID_WHERE_DESTINATION | call,
                  &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return IRectangle(parms.rc);
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::videoFileName                                               |
| Returns the file name of the digital video file.                             |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMDigitalVideo::videoFileName() const
{
  IASSERTSTATE(isOpen());

  MCI_INFO_PARMS parms;
  char achResult[512 + 1];
  memset(&parms, 0, sizeof(parms));
  memset(achResult, 0, sizeof(achResult));
  parms.hwndCallback = HWND(deviceWindow().handle());
  parms.pszReturn    = achResult;
  parms.ulRetSize    = 512;
  ((IMMDigitalVideo*)this)->sendCommand(MCI_INFO,
                  MCI_DGV_INFO_VIDEO_FILE | IMMDevice::wait,
                  &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return IString(achResult);
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::playDigital                                                 |
| Allows the playing from a start to an end position with the specified        |
| flags and speed.                                                             |
------------------------------------------------------------------------------*/
IMMDigitalVideo& IMMDigitalVideo::playDigital(const IMMTime &from,
                                              const IMMTime &to,
                                              CallType call,
                                              unsigned long flagsIn,
                                              unsigned long speed)
{
  IASSERTSTATE(isOpen());

  MCI_DGV_PLAY_PARMS parms;
  unsigned long flags = flagsIn;
  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback = HWND(deviceWindow().handle());
  if (from.isValid())
  {
     parms.ulFrom = (unsigned long)from;
     flags |= MCI_FROM;
  }
  if (to.isValid())
  {
     parms.ulTo = (unsigned long)to;
     flags |= MCI_TO;
  }
  parms.ulSpeed = speed;
  sendCommand(MCI_PLAY, flags | call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::isOpenStringValid                                           |
| Returns true if the string passed in is valid for this device.               |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDigitalVideo::isOpenStringValid(const IString &deviceName) const
{
  IString tester(deviceName);
  return (tester.upperCase().indexOf("DIGITALVIDEO") == 1 ? true : false);
}

/*------------------------------------------------------------------------------
| IMMDigitalVideo::translateAudioFlag                                          |
| Maps the wave audio commands to the digital video command values.            |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMDigitalVideo::translateAudioFlag(unsigned long flag) const
{
  unsigned long rc = 0;

  switch (flag)
  {
      case MCI_WAVE_SET_SAMPLESPERSEC:
         rc = MCI_DGV_SET_SAMPLESPERSEC;
         break;

      case MCI_WAVE_SET_AVGBYTESPERSEC:
         rc = MCI_DGV_SET_AVGBYTESPERSEC;
         break;

      case MCI_WAVE_SET_BLOCKALIGN:
         rc = MCI_DGV_SET_BLOCKALIGN;
         break;

      case MCI_WAVE_SET_FORMATTAG:
         rc = MCI_DGV_SET_FORMATTAG;
         break;

      case MCI_WAVE_SET_CHANNELS:
         rc = MCI_DGV_SET_CHANNELS;
         break;

      case MCI_WAVE_SET_BITSPERSAMPLE:
         rc = MCI_DGV_SET_BITSPERSAMPLE;
         break;

      case MCI_WAVE_STATUS_SAMPLESPERSEC:
         rc = MCI_DGV_STATUS_SAMPLESPERSEC;
         break;

      case MCI_WAVE_STATUS_AVGBYTESPERSEC:
         rc = MCI_DGV_STATUS_AVGBYTESPERSEC;
         break;

      case MCI_WAVE_STATUS_BLOCKALIGN:
         rc = MCI_DGV_STATUS_BLOCKALIGN;
         break;

      case MCI_WAVE_STATUS_FORMATTAG:
         rc = MCI_DGV_STATUS_FORMATTAG;
         break;

      case MCI_WAVE_STATUS_CHANNELS:
         rc = MCI_DGV_STATUS_CHANNELS;
         break;

      case MCI_WAVE_STATUS_BITSPERSAMPLE:
         rc = MCI_DGV_STATUS_BITSPERSAMPLE;
         break;
  }
  return rc;
}

