/*******************************************************************************
* FILE NAME: immdvhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immdvhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
  {
  #include <string.h>
  #define INCL_MCIOS2
  #define INCL_WINATOM
  #define INCL_WINMESSAGEMGR
  #define IC_MULTIMEDIA
  #include <iwindefs.h>
  }

#include <immdvhdr.hpp>

#ifndef _ICCONST_
  #include <icconst.h>
#endif

#ifndef _IMMDEV_
  #include <immdev.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

#ifndef _IMMEVT_
   #include <immevt.hpp>
#endif

#ifndef _IMMOWIN_
   #include <immowin.hpp>
#endif

#ifndef _ITRACE_
   #include <itrace.hpp>
#endif

#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMDVHDR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMDeviceHandler::dispatchHandlerEvent                                       |
| Determine what message has been received, convert to the appropriate event,  |
| and call the virtual function for the event.                                 |
------------------------------------------------------------------------------*/
Boolean IMMDeviceHandler::dispatchHandlerEvent(IEvent &evt)
{
  Boolean result = false;

  // Do a switch here to see what type of notification is
  // taking place and construct the appropreate event. Then call the
  // virtual function to which the message corresponds.

  switch (evt.eventId())
  {
    case MM_MCINOTIFY:
         {
         IMMNotifyEvent notifyEvent(evt);
         IMMDevice* device = notifyEvent.device();
         if (notifyEvent.command() == IMMNotifyEvent::open)
         {
            if (notifyEvent.successCode() == IMMNotifyEvent::successful)
               device->setOpenStatus(IMMDevice::openSuccessful);
            else if (notifyEvent.errorId())
                 {
                    device->setLastError(notifyEvent.errorId());
                    device->setOpenStatus(IMMDevice::openFailed);
                 }
            else
               device->setOpenStatus(IMMDevice::openFailed);
         }
         break;
         }

    case MM_MCIPASSDEVICE:
         {
         IMMPassDeviceEvent passEvent(evt);
         IMMDevice* device = passEvent.device();
         result = passDevice(passEvent);
         break;
         }

    case MM_MCIPOSITIONCHANGE:
         {
         IMMPositionChangeEvent posEvent(evt);
         IMMDevice* device = posEvent.device();
         result = positionChange(posEvent);
         break;
         }

    case MM_MCICUEPOINT:
         {
         IMMCuePointEvent cueEvent(evt);
         IMMDevice* device = cueEvent.device();
         result = cuePoint(cueEvent);
         break;
         }

      case MM_MCIEVENT:
         {
         IMMDeviceEvent devEvent(evt);
         IMMDevice* device = devEvent.device();
         result = deviceEvent(devEvent);
         break;
         }
  }
  return result;
}

/*------------------------------------------------------------------------------
| IMMDeviceHandler::IMMDeviceHandler                                           |
| This constructor creates a new device handler object.                        |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceHandler::IMMDeviceHandler()
                : IHandler()
{}


/*------------------------------------------------------------------------------
| IMMDeviceHandler::~IMMDeviceHandler                                          |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceHandler::~IMMDeviceHandler()
{}


/*------------------------------------------------------------------------------
| IMMDeviceHandler::passDevice                                                 |
| Implemented by derived classes to process pass device events.                |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDeviceHandler::passDevice(const IMMPassDeviceEvent &evt)
{

  evt.device()->setPassDeviceRequested(true);
  return false;
}

/*------------------------------------------------------------------------------
| IMMDeviceHandler::positionChange                                             |
| Implemented by derived classes to process position                           |
| change events.                                                               |
------------------------------------------------------------------------------*/
Boolean IMMDeviceHandler::positionChange(const IMMPositionChangeEvent &evt)
{
  return false;
}

/*------------------------------------------------------------------------------
| IMMDeviceHandler::cuePoint                                                   |
| Implemented by derived classes to process cue point events                   |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDeviceHandler::cuePoint(const IMMCuePointEvent &evt)
{
  return false;
}


/*------------------------------------------------------------------------------
| IMMDeviceHandler::deviceEvent                                                |
| Implemented by derived classes to process device events                      |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMDeviceHandler::deviceEvent(const IMMDeviceEvent &evt)
{
  return false;
}

/*------------------------------------------------------------------------------
| IMMDeviceHandler::handleEventsFor                                            |
| Attaches the handler to the specified MMDevice object.                       |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceHandler& IMMDeviceHandler::handleEventsFor       ( IMMDevice* device )
{
   IHandler::handleEventsFor(&(device->deviceWindow()));
   return *this;
}

/*------------------------------------------------------------------------------
| IMMDeviceHandler::handleEventsFor                                            |
| Process illegal use of this member function.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceHandler& IMMDeviceHandler::handleEventsFor       ( IWindow* window )
{
   ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR, IErrorInfo::invalidRequest,
                      IException::recoverable);
   return *this;
}

/*------------------------------------------------------------------------------
| IMMDeviceHandler::stopHandlingEventsFor                                      |
| Detaches the handler to the specified MMDevice object.                       |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceHandler& IMMDeviceHandler::stopHandlingEventsFor ( IMMDevice* device )
{
   IHandler::stopHandlingEventsFor(&(device->deviceWindow()));
   return *this;
}

/*------------------------------------------------------------------------------
| IMMDeviceHandler::stopHandleEventsFor                                        |
| Process illegal use of this member function.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceHandler& IMMDeviceHandler::stopHandlingEventsFor ( IWindow* window )
{
   ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR, IErrorInfo::invalidRequest,
                      IException::recoverable);
   return *this;
}
