/*******************************************************************************
* FILE NAME: immevt.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immevt.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #include <string.h>
   #define INCL_MCIOS2
   #define IC_MULTIMEDIA
   #include <iwindefs.h>
   }

#ifndef _ITRACE_
   #include <itrace.hpp>
#endif

#include <immevt.hpp>


#ifndef _IMULTSTA_
   #include <imultsta.hpp>
#endif

#ifndef _IMMTIME_
   #include <immtime.hpp>
#endif

#ifndef _IMMOWIN_
   #include <immowin.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMEVT_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IMMNotifyEvent::IMMNotifyEvent                                               |
| Although you can construct objects of this class, typically                  |
| IMMDeviceHandler::dispatchHandlerEvent will do this.                         |
------------------------------------------------------------------------------*/
IMMNotifyEvent::IMMNotifyEvent(const IEvent &evt)
              : IEvent(evt)
{ }

/*------------------------------------------------------------------------------
| IMMNotifyEvent::successCode                                                  |
| Returns the success code, which indicates if the command succeeded or failed.|
|                                                                              |
------------------------------------------------------------------------------*/
IMMNotifyEvent::SuccessCode IMMNotifyEvent::successCode() const
{
  SuccessCode rc;

  if (parameter1().lowNumber() >= 3)
     rc = error;
  else
     rc = SuccessCode(parameter1().lowNumber());
  return rc;
}

/*------------------------------------------------------------------------------
| IMMNotifyEvent::userParameter                                                |
| Returns the user parameter that was set at the time of this event.           |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned short IMMNotifyEvent::userParameter() const
{
  return parameter1().highNumber();
}

/*------------------------------------------------------------------------------
| IMMNotifyEvent::command                                                      |
| Returns the command that generated this notification event.                  |
|                                                                              |
------------------------------------------------------------------------------*/
IMMNotifyEvent::Command IMMNotifyEvent::command() const
{
  return Command(parameter2().highNumber());
}


/*------------------------------------------------------------------------------
| IMMNotifyEvent::~IMMNotifyEvent                                              |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMNotifyEvent::~IMMNotifyEvent()
{ }

/*------------------------------------------------------------------------------
| IMMNotifyEvent::device                                                       |
| Returns a pointer to the device that this event is from.                     |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice* IMMNotifyEvent::device() const
{
  IMMMessageWindow* win = (IMMMessageWindow*)window();
  if (win)
     return &(win->device());
  else
     return 0;
}

/*------------------------------------------------------------------------------
| IMMNotifyEvent::errorId                                                      |
| Returns the error code from the command if there was one.                    |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned short IMMNotifyEvent::errorId() const
{
   return parameter1().lowNumber();
}

/*------------------------------------------------------------------------------
| IMMNotifyEvent::errorText                                                    |
| Returns the error text for a given error code.                               |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMNotifyEvent::errorText(unsigned long errorCode)
{
  IString errorString;
  IString achResult(0,513);
  if (!IMultimediaStatics::getErrorString(errorCode, achResult,
                                          (unsigned short)(achResult.length())))
     errorString = achResult;
  return errorString;
}

/*------------------------------------------------------------------------------
| IMMPassDeviceEvent::~IMMPassDeviceEvent                                      |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMPassDeviceEvent::~IMMPassDeviceEvent()
{ }

/*------------------------------------------------------------------------------
| IMMPassDeviceEvent::isGainingUse                                             |
| Returns true if the device is gaining use of the hardware device and returns |
| false if it is losing use of the device.                                     |
------------------------------------------------------------------------------*/
Boolean IMMPassDeviceEvent::isGainingUse() const
{
  return ((MCI_GAINING_USE == parameter2().lowNumber()) ? true : false);
}

/*------------------------------------------------------------------------------
| IMMPassDeviceEvent::device                                                   |
| Returns a pointer to the device that is being                                |
| requested to lose use of or gain use of.                                     |
------------------------------------------------------------------------------*/
IMMDevice* IMMPassDeviceEvent::device() const
{
  IMMMessageWindow* win = (IMMMessageWindow*)window();
  if (win)
     return &(win->device());
  else
     return 0;
}

/*------------------------------------------------------------------------------
| IMMPassDeviceEvent::IMMPassDeviceEvent                                       |
| Although you can construct objects of this class, typically                  |
| IMMDeviceHandler::dispatchHandlerEvent will do this.                         |
------------------------------------------------------------------------------*/
IMMPassDeviceEvent::IMMPassDeviceEvent(const IEvent &evt)
                  : IEvent(evt)
{ }

/*------------------------------------------------------------------------------
| IMMPositionChangeEvent::IMMPositionChangeEvent                               |
| Although you can construct objects of this class, typically                  |
| IMMDeviceHandler::dispatchHandlerEvent will do this.                         |
------------------------------------------------------------------------------*/
IMMPositionChangeEvent::IMMPositionChangeEvent(const IEvent &evt)
                      : IEvent(evt)
{ }

/*------------------------------------------------------------------------------
| IMMPositionChangeEvent::userParameter                                        |
| Returns the user parameter that was set at the time of this event.           |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned short IMMPositionChangeEvent::userParameter() const
{
  return parameter1().lowNumber();
}

/*------------------------------------------------------------------------------
| IMMPositionChangeEvent::~IMMPositionChangeEvent                              |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMPositionChangeEvent::~IMMPositionChangeEvent()
{ }

/*------------------------------------------------------------------------------
| IMMPositionChangeEvent::device                                               |
| Returns a pointer to the device that this event is from.                     |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice* IMMPositionChangeEvent::device() const
{
  IMMMessageWindow* win = (IMMMessageWindow*)window();
  if (win)
     return &(win->device());
  else
     return 0;
}

/*------------------------------------------------------------------------------
| IMMPositionChangeEvent::position                                             |
| Returns the position at the time the event was generated.                    |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTime IMMPositionChangeEvent::position() const
{
  return IMMTime((unsigned long)parameter2());
}

/*------------------------------------------------------------------------------
| IMMCuePointEvent::~IMMCuePointEvent                                          |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMCuePointEvent::~IMMCuePointEvent()
{ }

/*------------------------------------------------------------------------------
| IMMCuePointEvent::IMMCuePointEvent                                           |
| Although you can construct objects of this class, typically                  |
| IMMDeviceHandler::dispatchHandlerEvent will do this.                         |
------------------------------------------------------------------------------*/
IMMCuePointEvent::IMMCuePointEvent(const IEvent &evt)
                : IEvent(evt)
{ }

/*------------------------------------------------------------------------------
| IMMCuePointEvent::userParameter                                              |
| Returns the user parameter that was set at the time of this event.           |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned short IMMCuePointEvent::userParameter() const
{
  return parameter1().lowNumber();
}

/*------------------------------------------------------------------------------
| IMMCuePointEvent::device                                                     |
| Returns a pointer to the device that played or recorded over a cue point.    |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice* IMMCuePointEvent::device() const
{
  IMMMessageWindow* win = (IMMMessageWindow*)window();
  if (win)
     return &(win->device());
  else
     return 0;
}

/*------------------------------------------------------------------------------
| IMMCuePointEvent::position                                                   |
| Returns the cue point position.                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTime IMMCuePointEvent::position() const
{
  return IMMTime((unsigned long)parameter2());
}

/*------------------------------------------------------------------------------
| IMMDeviceEvent::IMMDeviceEvent                                               |
| Although you can construct objects of this class, typically                  |
| IMMDeviceHandler::dispatchHandlerEvent will do this.                         |
------------------------------------------------------------------------------*/
IMMDeviceEvent::IMMDeviceEvent(const IEvent &evt)
              : IEvent(evt)
{ }

/*------------------------------------------------------------------------------
| IMMDeviceEvent::eventCode                                                    |
| Returns the device-specific event code.                                      |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned short IMMDeviceEvent::eventCode() const
{
  return parameter1().lowNumber();
}

/*------------------------------------------------------------------------------
| IMMDeviceEvent::eventData                                                    |
| Returns the device-specific event data structure.                            |
|                                                                              |
------------------------------------------------------------------------------*/
void *IMMDeviceEvent::eventData() const
{
  return ((char *)parameter2());
}

/*------------------------------------------------------------------------------
| IMMDeviceEvent::~IMMDeviceEvent                                              |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDeviceEvent::~IMMDeviceEvent()
{ }

/*------------------------------------------------------------------------------
| IMMDeviceEvent::device                                                       |
| Returns a pointer to the device that this event is from.                     |
|                                                                              |
------------------------------------------------------------------------------*/
IMMDevice* IMMDeviceEvent::device() const
{
  IMMMessageWindow* win = (IMMMessageWindow*)window();
  if (win)
     return &(win->device());
  else
     return 0;
}
