/*******************************************************************************
* FILE NAME: immexcpt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immexcpt.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
{
   #define INCL_WINERRORS
   #define INCL_DOSFILEMGR
   #define INCL_DOSMISC
   #define INCL_WINWINDOWMGR
   #define IC_MULTIMEDIA
   #include <iwindefs.h>
   #include <string.h>
   #include <errno.h>
}

#include <icconst.h>

#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif

#ifndef _IMULTSTA_
   #include <imultsta.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMEXCPT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMErrorInfo::IMMErrorInfo                                                   |
------------------------------------------------------------------------------*/
IMMErrorInfo::IMMErrorInfo ( unsigned long errorId, const char *pMMFunctionName)
{
  IString achResult(0,513);
  ulClErrorId=errorId;
  if (!IMultimediaStatics::getErrorString(ulClErrorId, achResult,
                                          (unsigned short)(achResult.length())))
  {
     foundText=true;
  }
  else
  {
     achResult = "No error text is available.";
     foundText=false;
  }

  if (pMMFunctionName)
  {
    this->pClFuncNameText = new char[strlen(pMMFunctionName) +
                                     strlen(achResult) + 2];
    strcpy(this->pClFuncNameText, pMMFunctionName);
    strcat(this->pClFuncNameText, ":");
    strcat(this->pClFuncNameText, achResult);
  }
  else
  {
    this->pClFuncNameText = new char[strlen(achResult)+1];
    strcpy(this->pClFuncNameText, achResult);
  }
}

/*------------------------------------------------------------------------------
| IMMErrorInfo::~IMMErrorInfo                                                  |
------------------------------------------------------------------------------*/
IMMErrorInfo::~IMMErrorInfo ( )
{
  if (this->pClFuncNameText)
    delete [] this->pClFuncNameText;
}

/*------------------------------------------------------------------------------
| IMMErrorInfo::text                                                           |
------------------------------------------------------------------------------*/
const char* IMMErrorInfo::text ( ) const
{
  return( this->pClFuncNameText );
}

/*------------------------------------------------------------------------------
| IMMErrorInfo::errorId                                                        |
------------------------------------------------------------------------------*/
unsigned long IMMErrorInfo::errorId ( ) const
{
  return( ulClErrorId );
}

/*------------------------------------------------------------------------------
| IMMErrorInfo::operator const char*                                           |
------------------------------------------------------------------------------*/
IMMErrorInfo::operator const char* () const
{
  return( this->pClFuncNameText );
}

/*------------------------------------------------------------------------------
| IMMErrorInfo::isAvailable                                                    |
------------------------------------------------------------------------------*/
Boolean IMMErrorInfo::isAvailable ( ) const
{
  return foundText;
}

/*------------------------------------------------------------------------------
| IMMErrorInfo::throwMMError                                                   |
------------------------------------------------------------------------------*/
void IMMErrorInfo :: throwMMError ( unsigned long             errorId,
                                    const char                *pFunctionName,
                                    const IExceptionLocation  &location,
                                    IErrorInfo::ExceptionType name,
                                    IException::Severity      severity )
{
  IMMErrorInfo errInfo(errorId, pFunctionName);
  errInfo.throwError(location, name, severity);
}
