/*******************************************************************************
* FILE NAME: immmaud.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immmaud.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #include <string.h>
   #define INCL_MCIOS2
   #define IC_MULTIMEDIA
   #include <iwindefs.h>
   }

#ifndef _ITRACE_
   #include <itrace.hpp>
#endif
#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif

#include <immmaud.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMMAUD_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMMasterAudioData                                                           |
| This private class encapsulates functions used by member functions of the    |
| IMMMasterAudio class.                                                        |
------------------------------------------------------------------------------*/
class IMMMasterAudioData : public IMMDevice
{
public:
  IMMMasterAudioData()
    : IMMDevice(0, IString()) // master audio is device id 0
     { }
public:
IMMDevice
  &sendCommand(unsigned short message, unsigned long param1, void *param2)
         {return IMMDevice::sendCommand(message,param1,param2);};

IMMDevice
  &setLastError  (unsigned long errorId)
         {return IMMDevice::setLastError(errorId);};

unsigned long
  lastError() const
         {return IMMDevice::lastError();};

};

/*------------------------------------------------------------------------------
| IMMMasterAudio::IMMMasterAudio                                               |
| This constructor creates a new device object.                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio::IMMMasterAudio()
              : fMasterAudioData(new IMMMasterAudioData())
{ }

/*------------------------------------------------------------------------------
| IMMMasterAudio::~IMMMasterAudio                                              |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio::~IMMMasterAudio()
{
  if (fMasterAudioData)
     delete fMasterAudioData;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::setVolume                                                    |
| Sets the master volume for the operating system.                             |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::setVolume(unsigned long vol,
                                          IMMDevice::CallType call)
{
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.ulMasterVolume = (unsigned long)vol;
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO, MCI_MASTERVOL | call, &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::enableSpeakers                                               |
| Turns the speakers' setting either on or off.                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::enableSpeakers(Boolean enable,
                                               IMMDevice::CallType call)
{
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO,
              MCI_SPEAKERS | call | (unsigned long) enable, &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::disableSpeakers                                              |
| Turns the speakers' setting off.                                             |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::disableSpeakers(IMMDevice::CallType call)
{
  enableSpeakers(false,call);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::enableHeadphones                                             |
| Turns the headphones' setting on or off.                                     |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::enableHeadphones(Boolean enable,
                                                 IMMDevice::CallType call)
{
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO, MCI_HEADPHONES |
              call | (unsigned long)enable, &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::disableHeadphones                                            |
| Turns the headphones' setting off.                                           |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::disableHeadphones( IMMDevice::CallType call)
{
  enableHeadphones(false,call);
  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::saveVolume                                                   |
| Saves the current operating system setting for master volume                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::saveVolume( IMMDevice::CallType call)
{
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO,
              MCI_MASTERVOL | call | MCI_SAVESETTING, &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::saveSpeakersSetting                                          |
| Saves the current operating system setting for speakers                      |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::saveSpeakersSetting( IMMDevice::CallType call)
{
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO,
              MCI_SPEAKERS | call | MCI_SAVESETTING, &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::saveHeadphonesSetting                                        |
| Saves the current operating system setting for headphones                    |
|                                                                              |
------------------------------------------------------------------------------*/
IMMMasterAudio& IMMMasterAudio::saveHeadphonesSetting( IMMDevice::CallType call)
{
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO,
              MCI_HEADPHONES | call | MCI_SAVESETTING, &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::volume                                                       |
| Returns the master volume setting for either the current setting or the      |
| saved setting as a percent of the maximum audio level.                       |
------------------------------------------------------------------------------*/
unsigned long IMMMasterAudio::volume(SettingSource source,
                                               IMMDevice::CallType call ) const
{
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO,
              MCI_MASTERVOL | call | source,
              &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
  return parms.ulReturn;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::areSpeakersEnabled                                           |
| Returns true if the speakers' setting is enabled for the passed in           |
| source.                                                                      |
------------------------------------------------------------------------------*/
Boolean IMMMasterAudio::areSpeakersEnabled(SettingSource source,
                                           IMMDevice::CallType call ) const
{
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO,
              MCI_SPEAKERS | call | source,
              &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
  if (parms.ulReturn)
     return true;
  return false;
}

/*------------------------------------------------------------------------------
| IMMMasterAudio::areHeadphonesEnabled                                         |
| Returns true if the headphones' setting is enabled for the passed in         |
| source.                                                                      |
------------------------------------------------------------------------------*/
Boolean IMMMasterAudio::areHeadphonesEnabled(SettingSource source,
                                             IMMDevice::CallType call ) const
{
  MCI_MASTERAUDIO_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  fMasterAudioData->sendCommand(MCI_MASTERAUDIO,
                                MCI_HEADPHONES | call | source,
                                &parms);
  if (fMasterAudioData->lastError())
     ITHROWMMERROR(fMasterAudioData->lastError(),"sendCommand");
  if (parms.ulReturn)
     return true;
  return false;
}

