/*******************************************************************************
* FILE NAME: immplyhd.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immplyhd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
   {
   #define INCL_WINDIALOGS
   #define INCL_OS2MM
   #define INCL_SW
   #define INCL_SELSLIDE
   #define IC_MULTIMEDIA
   #include <iwindefs.h>
   }

#include <immplyhd.hpp>
#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif
#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif
#ifndef _IANIMBUT_
   #include <ianimbut.hpp>
#endif
#ifndef _IMMPLYPN_
   #include <immplypn.hpp>
#endif
#ifndef _IMMPLAYD_
   #include <immplayd.hpp>
#endif
#ifndef _ICCONST_
  #include <icconst.h>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMPLYHD_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::IMMPlayerPanelHandler                                 |
| Construct a new object.                                                      |
------------------------------------------------------------------------------*/
IMMPlayerPanelHandler::IMMPlayerPanelHandler()
{ }

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::~IMMPlayerPanelHandler                                |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IMMPlayerPanelHandler::~IMMPlayerPanelHandler()
{ }

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::command                                               |
| Calls the virtual functions to process a command event for a player panel.   |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::command(ICommandEvent& event)
{
  Boolean handled = false;
  IMMPlayerPanel* panel= (IMMPlayerPanel*) (event.window());

  switch (event.commandId())
  {
    case IC_FF:
         handled = fastForward(*panel);
         break;
    case IC_PAUSE:
         handled = pause(*panel);
         break;
    case IC_STOP:
         handled = stop(*panel);
         break;
    case IC_PLAY1:
         handled = play(*panel);
         break;
    case IC_REWIND:
         handled = rewind(*panel);
         break;
    case IC_STEPF:
         handled = stepForward(*panel);
         break;
    case IC_STEPB:
         handled = stepBackward(*panel);
         break;
  } /* endswitch */
  return handled;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::play                                                  |
| Gets called when the user presses the play button.                           |
------------------------------------------------------------------------------*/
 Boolean IMMPlayerPanelHandler::play(const IMMPlayerPanel& panel)
{
  IMMPlayableDevice* device = panel.playableDevice();
  IASSERTSTATE(device);
  if (panel.stopButton())
     panel.stopButton()->enable();
  if (panel.pauseButton())
  {
     panel.pauseButton()->enable();
     panel.pauseButton()->unlatch();
  }
  device->play();
  return true;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::stop                                                  |
| Gets called when the user presses the stop button.                           |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::stop(const IMMPlayerPanel& panel)
{
  IMMPlayableDevice* device = panel.playableDevice();
  IASSERTSTATE(device);
  if (panel.playButton())
  {
     panel.playButton()->unlatch();
  }
  if (panel.stopButton())
     panel.stopButton()->disable();
  if (panel.pauseButton())
  {
     panel.pauseButton()->unlatch();
     panel.pauseButton()->disable();
  }
  device->stop();
  device->seekToStart();
  return true;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::pause                                                 |
| Gets called when the user presses the pause button.                          |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::pause(const IMMPlayerPanel& panel)
{
  IMMPlayableDevice* device = panel.playableDevice();
  IASSERTSTATE(device);
  if (IMMDevice::paused == device->mode())
  {
     if (panel.pauseButton())
        panel.playButton()->latch();
     device->resume();
  } else
     device->pause();
  return true;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::fastForward                                           |
| Gets called when the user presses the fast forward button.                   |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::fastForward(const IMMPlayerPanel& panel)
{
  IMMPlayableDevice* device = panel.playableDevice();
  IASSERTSTATE(device);
  if (panel.playButton())
  {
     panel.playButton()->unlatch();
  }
  if (panel.stopButton())
     panel.stopButton()->disable();
  if (panel.pauseButton())
  {
     panel.pauseButton()->unlatch();
     panel.pauseButton()->disable();
  }
  device->seekToEnd();
  return true;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::rewind                                                |
| Gets called when the user presses the rewind button.                         |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::rewind(const IMMPlayerPanel& panel)
{
  IMMPlayableDevice* device = panel.playableDevice();
  IASSERTSTATE(device);
  if (panel.playButton())
  {
     panel.playButton()->unlatch();
  }
  if (panel.stopButton())
     panel.stopButton()->disable();
  if (panel.pauseButton())
  {
     panel.pauseButton()->unlatch();
     panel.pauseButton()->disable();
  }
  device->seekToStart();
  return true;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::stepForward                                           |
| Gets called when the user presses the step forward button.                   |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::stepForward(const IMMPlayerPanel& panel)
{
  IMMPlayableDevice* device = panel.playableDevice();
  IASSERTSTATE(device);
  if (panel.playButton())
  {
     panel.playButton()->unlatch();
  }
  IMMTime::Format format = device->timeFormat();
  device->setTimeFormat(IMMTime::mmTime);
  if (device->position() < device->length())
  {
     device->setTimeFormat(format);
     device->stepFrame();
  }
  else
     device->setTimeFormat(format);
  return true;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanelHandler::stepBackward                                          |
| Gets called when the user presses the step backward button.                  |
------------------------------------------------------------------------------*/
Boolean IMMPlayerPanelHandler::stepBackward(const IMMPlayerPanel& panel)
{
  IMMPlayableDevice* device = panel.playableDevice();
  IASSERTSTATE(device);
  if (panel.playButton())
  {
     panel.playButton()->unlatch();
  }
  IMMTime::Format format = device->timeFormat();
  device->setTimeFormat(IMMTime::mmTime);
  if (device->position())
  {
     device->setTimeFormat(format);
     device->stepFrame(1,false);
  }
  else
     device->setTimeFormat(format);
  return true;
}

