/*******************************************************************************
* FILE NAME: immplypn.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immplypn.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#ifndef _IHANDLE_
   #include <ihandle.hpp>
#endif
#ifndef _ICCONST_
  #include <icconst.h>
#endif
#ifndef _ICOLOR_
   #include <icolor.hpp>
#endif
#ifndef _IMMPLYHD_
   #include <immplyhd.hpp>
#endif
#ifndef _IANIMBUT_
   #include <ianimbut.hpp>
#endif
#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif
#include <immplypn.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMPLYPN_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMPlayerPanelData                                                           |
| This private class encapsulates data used by member functions of             |
| IMMPlayerPanel.                                                              |
------------------------------------------------------------------------------*/
class IMMPlayerPanelData : public IBase
{
public:
   IMMPlayerPanelData()
      : rewindBtn(0),
        ffBtn(0)    ,
        pauseBtn(0) ,
        stopBtn(0)  ,
        playBtn(0)  ,
        stepFBtn(0) ,
        stepBBtn(0) ,
        device(0)   ,
        pHandler(0) {};

IAnimatedButton
  *rewindBtn,
  *ffBtn   ,
  *pauseBtn,
  *stopBtn ,
  *playBtn ,
  *stepFBtn,
  *stepBBtn;

IMMPlayableDevice
  *device;

IMMPlayerPanelHandler
  *pHandler;
};


/*------------------------------------------------------------------------------
| IMMPlayerPanel::IMMPlayerPanel                                               |
| Construct a new player panel.                                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMMPlayerPanel::IMMPlayerPanel(unsigned long id,
                               IWindow* parent,
                               IWindow* owner,
                               unsigned long devType,
                               const IRectangle&     initial,
                               const Style& style)
              : IMultiCellCanvas(id, parent, owner, initial, style),
                fPanelData( new IMMPlayerPanelData())
{
   fPanelData->pHandler = new IMMPlayerPanelHandler();

   if ((devType == IMMDevice::videoDisc   ) ||
       (devType == IMMDevice::overlay     ) ||
       (devType == IMMDevice::animation   ) ||
       (devType == IMMDevice::digitalVideo))
   {
      fPanelData->stepBBtn = new IAnimatedButton(IC_STEPB, this, this,
                                           IRectangle(),
                                           IWindow::visible |
                                           IAnimatedButton::animateWhenLatched |
                                           IControl::group);
      fPanelData->stepBBtn->setBitmaps(IAnimatedButton::stepBackward);
      fPanelData->stepFBtn = new IAnimatedButton(IC_STEPF, this, this,
                                           IRectangle(),
                                           IWindow::visible |
                                           IAnimatedButton::animateWhenLatched);
      fPanelData->stepFBtn->setBitmaps(IAnimatedButton::stepForward);
   }
   IControl::Style tempAdd = IWindow::noStyle;
   if (!fPanelData->stepBBtn)
      tempAdd = IControl::group;
   fPanelData->ffBtn     = new IAnimatedButton(IC_FF, this, this,
                                   IRectangle(),
                                   IWindow::visible |
                                   tempAdd |
                                   IAnimatedButton::animateWhenLatched);
   fPanelData->ffBtn->setBitmaps(IAnimatedButton::fastForward);
   fPanelData->pauseBtn  = new IAnimatedButton(IC_PAUSE, this, this,
                                   IRectangle(),
                                   IWindow::visible |
                                   IAnimatedButton::animateWhenLatched |
                                   ICustomButton::autoLatch |
                                   ICustomButton::latchable);
   fPanelData->pauseBtn->setBitmaps(IAnimatedButton::pause);
   fPanelData->playBtn   = new IAnimatedButton(IC_PLAY1, this, this,
                                   IRectangle(),
                                   IWindow::visible |
                                   IAnimatedButton::animateWhenLatched |
                                   ICustomButton::latchable        |
                                   ICustomButton::autoLatch |
                                   IControl::tabStop);
   fPanelData->playBtn->setBitmaps(IAnimatedButton::play);
   fPanelData->rewindBtn = new IAnimatedButton(IC_REWIND, this, this,
                                   IRectangle(),
                                   IWindow::visible |
                                   IAnimatedButton::animateWhenLatched);
   fPanelData->rewindBtn->setBitmaps(IAnimatedButton::rewind);
   fPanelData->stopBtn   = new IAnimatedButton(IC_STOP, this, this,
                                   IRectangle(),
                                   IWindow::visible |
                                   IAnimatedButton::animateWhenLatched);
   fPanelData->stopBtn->setBitmaps(IAnimatedButton::stop);

   if (fPanelData->stepBBtn)
   {
      addToCell(fPanelData->stepBBtn,       1, 1, 1, 1);
      addToCell(fPanelData->rewindBtn,      2, 1, 1, 1);
      addToCell(fPanelData->stopBtn,        3, 1, 1, 1);
      addToCell(fPanelData->pauseBtn,       4, 1, 1, 1);
      addToCell(fPanelData->playBtn,        5, 1, 1, 1);
      addToCell(fPanelData->ffBtn,          6, 1, 1, 1);
      addToCell(fPanelData->stepFBtn,       7, 1, 1, 1);
   } else {
      addToCell(fPanelData->rewindBtn,      1, 1, 1, 1);
      addToCell(fPanelData->stopBtn,        2, 1, 1, 1);
      addToCell(fPanelData->pauseBtn,       3, 1, 1, 1);
      addToCell(fPanelData->playBtn,        4, 1, 1, 1);
      addToCell(fPanelData->ffBtn,          5, 1, 1, 1);
   }

   fPanelData->pHandler->handleEventsFor(this);

   // disable pause and stop buttons since we're not currently playing
   fPanelData->pauseBtn->disable();
   fPanelData->stopBtn->disable();
}

IMMPlayerPanel::~IMMPlayerPanel()
{
  if (fPanelData)
  {
     if (fPanelData->stepFBtn)
         delete fPanelData->stepFBtn;
     if (fPanelData->stepBBtn)
         delete fPanelData->stepBBtn;
     if (fPanelData->rewindBtn)
         delete fPanelData->rewindBtn;
     if (fPanelData->ffBtn)
         delete fPanelData->ffBtn;
     if (fPanelData->pauseBtn)
         delete fPanelData->pauseBtn;
     if (fPanelData->stopBtn)
         delete fPanelData->stopBtn;
     if (fPanelData->playBtn)
         delete fPanelData->playBtn;
     if (fPanelData->pHandler)
         delete fPanelData->pHandler;
     delete fPanelData;
  }
}

/*------------------------------------------------------------------------------
| IMMPlayerPanel::playableDevice                                               |
| Returns a pointer to the IMMPlayableDevice the player is controlling.        |
|                                                                              |
------------------------------------------------------------------------------*/
IMMPlayableDevice* IMMPlayerPanel::playableDevice() const
{
  return fPanelData->device;
}


/*------------------------------------------------------------------------------
| IMMPlayerPanel::setPlayableDevice                                            |
| Sets a pointer to the IMMPlayableDevice that the player will control.        |
|                                                                              |
------------------------------------------------------------------------------*/
IMMPlayerPanel& IMMPlayerPanel::setPlayableDevice(IMMPlayableDevice* dev)
{
  fPanelData->device = dev;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMPlayerPanel::rewindButton                                                 |
| Returns a pointer to the rewind animated button.                             |
|                                                                              |
------------------------------------------------------------------------------*/
IAnimatedButton* IMMPlayerPanel::rewindButton       () const
{
  return fPanelData->rewindBtn;
}
/*------------------------------------------------------------------------------
| IMMPlayerPanel::fastForwardButton                                            |
| Returns a pointer to the fast forward animated button.                       |
|                                                                              |
------------------------------------------------------------------------------*/
IAnimatedButton* IMMPlayerPanel::fastForwardButton  () const
{
  return fPanelData->ffBtn;
}
/*------------------------------------------------------------------------------
| IMMPlayerPanel::pauseButton                                                  |
| Returns a pointer to the pause animated button.                              |
|                                                                              |
------------------------------------------------------------------------------*/
IAnimatedButton* IMMPlayerPanel::pauseButton        () const
{
  return fPanelData->pauseBtn;
}
/*------------------------------------------------------------------------------
| IMMPlayerPanel::stopButton                                                   |
| Returns a pointer to the stop animated button.                               |
|                                                                              |
------------------------------------------------------------------------------*/
IAnimatedButton* IMMPlayerPanel::stopButton         () const
{
  return fPanelData->stopBtn;
}
/*------------------------------------------------------------------------------
| IMMPlayerPanel::playButton                                                   |
| Returns a pointer to the play animated button.                               |
|                                                                              |
------------------------------------------------------------------------------*/
IAnimatedButton* IMMPlayerPanel::playButton         () const
{
  return fPanelData->playBtn;
}
/*------------------------------------------------------------------------------
| IMMPlayerPanel::stepForwardButton                                            |
| Returns a pointer to the step forward animated button.                       |
|                                                                              |
------------------------------------------------------------------------------*/
IAnimatedButton* IMMPlayerPanel::stepForwardButton  () const
{
  return fPanelData->stepFBtn;
}
/*------------------------------------------------------------------------------
| IMMPlayerPanel::setBackwardButton                                            |
| Returns a pointer to the step backward animated button.                      |
|                                                                              |
------------------------------------------------------------------------------*/
IAnimatedButton* IMMPlayerPanel::stepBackwardButton () const
{
  return fPanelData->stepBBtn;
}
