/*******************************************************************************
* FILE NAME: immremed.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immremed.hpp                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
{
  #include <string.h>
  #define INCL_WINTIMER
  #define INCL_MCIOS2
  #define IC_MULTIMEDIA
  #include <iwindefs.h>
}

#include <immremed.hpp>

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif
#ifndef _ITIMER_
  #include <itimer.hpp>
#endif
#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif
#ifndef _IMMREMHD_
   #include <immremhd.hpp>
#endif
#include <immowin.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMREMED_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IMMRemovableMediaData                                                        |
| This private class encapsulates interaction with IMMRemovableMediaHandler.   |
|                                                                              |
------------------------------------------------------------------------------*/
class IMMRemovableMediaData : public IBase
{
public:
  IMMRemovableMediaData()
         : deviceHandler(0),
           fMediaTimer(0),
           fDiscIsInDrive(false)
     { }

IMMRemovableMediaHandler
  *deviceHandler;
ITimer
  *fMediaTimer;
Boolean
  fDiscIsInDrive;
};


/*------------------------------------------------------------------------------
| IMMRemovableMedia::IMMRemovableMedia                                         |
| This constructor creates a new device object.                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMedia::IMMRemovableMedia(const IString &baseDeviceName,
                                       Boolean openNow,
                                       unsigned long instance,
                                       Boolean shareable)
   : IMMPlayableDevice(baseDeviceName, openNow, instance, shareable),
     fRemovableMediaData(new IMMRemovableMediaData())
{
  fRemovableMediaData->deviceHandler = new IMMRemovableMediaHandler();
  fRemovableMediaData->deviceHandler->handleEventsFor(this);
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::~IMMRemovableMedia                                        |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMedia::~IMMRemovableMedia()
{
   if (fRemovableMediaData)
   {
      if (fRemovableMediaData->deviceHandler)
      {
         fRemovableMediaData->deviceHandler->stopHandlingEventsFor(this);
         delete fRemovableMediaData->deviceHandler;
      }
      if (fRemovableMediaData->fMediaTimer)
      {
         fRemovableMediaData->fMediaTimer->stop();
         delete fRemovableMediaData->fMediaTimer;
      }
      delete fRemovableMediaData;
   }
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::closeDoor                                                 |
| Retracts the tray and closes the door, if possible.                          |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMedia& IMMRemovableMedia::closeDoor(CallType call)
{
  return openDoor(false,call);
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::lockDoor                                                  |
| Locks the media cover on the device, if any.  This disables manual ejection  |
| of the media from the device.                                                |
------------------------------------------------------------------------------*/
IMMRemovableMedia& IMMRemovableMedia::lockDoor(Boolean lock, CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_SET_PARMS parms;
  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback = HWND(deviceWindow().handle());
  sendCommand(MCI_SET,(lock ? MCI_SET_DOOR_LOCK : MCI_SET_DOOR_UNLOCK) |
              call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::isMediaPresent                                            |
| Returns true if media is inserted in the device; otherwise, it returns false.|
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMRemovableMedia::isMediaPresent(CallType call) const
{
  IASSERTSTATE(isOpen());
  Boolean rc = itemStatus(MCI_STATUS_MEDIA_PRESENT, 0, call);
  if (lastError()== MCIERR_INVALID_MEDIA_TYPE)
     return false;
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::openDoor                                                  |
| Opens the door and ejects the tray, if possible.                             |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMedia& IMMRemovableMedia::openDoor(Boolean open, CallType call)
{
  IASSERTSTATE(isOpen());

  MCI_SET_PARMS parms;
  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback = HWND(deviceWindow().handle());
  sendCommand(MCI_SET,(open ? MCI_SET_DOOR_OPEN : MCI_SET_DOOR_CLOSED) |
              call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::unlockDoor                                                |
| Unlocks the media cover on the device, if any.  This enables manual ejection |
| of the media from the device.                                                |
------------------------------------------------------------------------------*/
IMMRemovableMedia& IMMRemovableMedia::unlockDoor(CallType call)
{
  return lockDoor(false,call);
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::IMMRemovableMedia                                         |
| This constructor wrappers an existing device, and gives a derived class the  |
| ability to specify a desired alias (instead of ICLUI generating it).         |
------------------------------------------------------------------------------*/
IMMRemovableMedia::IMMRemovableMedia(unsigned long newDeviceId,
                                       const IString &newAlias)
                  : IMMPlayableDevice(newDeviceId, newAlias),
                    fRemovableMediaData(new IMMRemovableMediaData())
{
  fRemovableMediaData->deviceHandler = new IMMRemovableMediaHandler();
  fRemovableMediaData->deviceHandler->handleEventsFor(this);
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::startCheckMediaTimer                                      |
| Starts a timer to periodically check the device and post a user event when   |
| a disc is inserted or removed.                                               |
------------------------------------------------------------------------------*/
IMMRemovableMedia& IMMRemovableMedia::startCheckMediaTimer()
{
  if ( wasPassDeviceRequested() )
  {
     fRemovableMediaData->fDiscIsInDrive = isMediaPresent();
     if (!fRemovableMediaData->fMediaTimer)
        fRemovableMediaData->fMediaTimer = new ITimer();
     if (!fRemovableMediaData->fMediaTimer->isStarted())
        fRemovableMediaData->fMediaTimer->start(new ITimerMemberFn<IMMRemovableMedia>(*this,checkMedia));
  }

  return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::stopCheckMediaTimer                                       |
| Stops the timer started in startCheckMediaTimer().                           |
------------------------------------------------------------------------------*/
IMMRemovableMedia& IMMRemovableMedia::stopCheckMediaTimer()
{
   fRemovableMediaData->fMediaTimer->stop();
   return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::checkMedia                                                |
| Check to see if media is present; send an IOC user event when state changes. |
|                                                                              |
------------------------------------------------------------------------------*/
void IMMRemovableMedia::checkMedia(unsigned long timerId)
{
   if (isMediaPresent())
   {
      if ( !(fRemovableMediaData->fDiscIsInDrive) )
      {
         fRemovableMediaData->fDiscIsInDrive = true;
         // Send (not post) event, since a) we already endured overhead of WM_TIMER, b) we're not within
         //   a handler-dispatch so we can safely do a send.
         deviceWindow().sendEvent(IC_UM_MEDIA_LOADED);
      }
   }
   else
   {
      if (fRemovableMediaData->fDiscIsInDrive)
      {
         fRemovableMediaData->fDiscIsInDrive = false;
         deviceWindow().sendEvent(IC_UM_MEDIA_UNLOADED);
      }
   }
}

