/*******************************************************************************
* FILE NAME: immremhd.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immremhd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
  {
  #include <string.h>
  #define INCL_MCIOS2
  #define INCL_WINATOM
  #define INCL_WINMESSAGEMGR
  #define IC_MULTIMEDIA
  #include <iwindefs.h>
  }

#include <immremhd.hpp>
#ifndef _IMMREMED_
  #include <immremed.hpp>
#endif
#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif
#ifndef _IMMEVT_
   #include <immevt.hpp>
#endif
#ifndef _ITIMER_
  #include <itimer.hpp>
#endif
#ifndef _IMMOWIN_
   #include <immowin.hpp>
#endif
#ifndef _ITRACE_
   #include <itrace.hpp>
#endif

#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMREMHD_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMRemovableMediaHandler::dispatchHandlerEvent                               |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
Boolean IMMRemovableMediaHandler::dispatchHandlerEvent ( IEvent& event )
{
  Boolean bRc = false;

  if ( event.eventId() == IC_UM_MEDIA_LOADED )
  {
//       IMMDeviceEvent devEvent(event);             //next release, pass the event like other handlers do
//       bRc = mediaLoaded();                        // & return a Boolean
       mediaLoaded();
  }
  else if ( event.eventId() == IC_UM_MEDIA_UNLOADED )
       {
//          IMMDeviceEvent devEvent(event);          //next release, pass the event like other handlers don
//          bRc = mediaUnloaded();                   // & return a Boolean
          mediaUnloaded();
       }

  if ( bRc )
  {                              // Processing done.
//     event.setResult( devEvent.result() );  // Copy event result.
  }

  return bRc;
}

/*------------------------------------------------------------------------------
| IMMRemovableMediaHandler::handleEventsFor                                    |
| Start handling events for the device passed, incl. emitting our own events.  |
------------------------------------------------------------------------------*/
IMMRemovableMediaHandler& IMMRemovableMediaHandler::handleEventsFor (IMMRemovableMedia* device )
{
   Inherited::handleEventsFor(device);
   device->startCheckMediaTimer();

   return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMediaHandler::stopHandlingEventsFor                              |
| Stop handling events for the device passed, stop emitting our own events.    |
------------------------------------------------------------------------------*/
IMMRemovableMediaHandler& IMMRemovableMediaHandler::stopHandlingEventsFor ( IMMRemovableMedia* device )
{
   device->stopCheckMediaTimer();
   Inherited::stopHandlingEventsFor(device);
   return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMedia::passDevice                                                |
| Process pass device events.                                                  |
------------------------------------------------------------------------------*/
Boolean IMMRemovableMediaHandler::passDevice(const IMMPassDeviceEvent &event)
{
  IMMRemovableMedia* device = (IMMRemovableMedia*)(event.device());
  if (device)
  {
     if (event.isGainingUse())
     {
        device->startCheckMediaTimer();
     }
     else
        device->stopCheckMediaTimer();
  }
  return false;
}

/*------------------------------------------------------------------------------
| IMMRemovableMediaHandler::~IMMRemovableMediaHandler                          |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMediaHandler::~IMMRemovableMediaHandler()
{
}

/*------------------------------------------------------------------------------
| IMMRemovableMediaHandler::IMMRemovableMediaHandler                           |
| Construct a handler.                                                         |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMediaHandler::IMMRemovableMediaHandler()
                         : IMMDeviceHandler()
{
}

/*------------------------------------------------------------------------------
| IMMRemovableMediaHandler::mediaLoaded                                        |
| Virtual function called when device gets media loaded into it.               |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMediaHandler& IMMRemovableMediaHandler::mediaLoaded()
{
  return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMediaHandler::mediaUnloaded                                      |
| Virtual function called when device gets media unloaded from it.             |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMediaHandler& IMMRemovableMediaHandler::mediaUnloaded()
{
  return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMediaHandler::handleEventsFor                                    |
| Process illegal use of this member function.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMediaHandler& IMMRemovableMediaHandler::handleEventsFor( IMMDevice* device )
{
   ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR, IErrorInfo::invalidRequest,
                      IException::recoverable);
   return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMediaHandler::stopHandlingEventsFor                              |
| Process illegal use of this member function.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMediaHandler& IMMRemovableMediaHandler::stopHandlingEventsFor( IMMDevice* device )
{
   ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR, IErrorInfo::invalidRequest,
                      IException::recoverable);
   return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMediaHandler::handleEventsFor                                    |
| Process illegal use of this member function.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMediaHandler& IMMRemovableMediaHandler::handleEventsFor( IWindow* window )
{
   ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR, IErrorInfo::invalidRequest,
                      IException::recoverable);
   return *this;
}

/*------------------------------------------------------------------------------
| IMMRemovableMediaHandler::stopHandlingEventsFor                              |
| Process illegal use of this member function.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMRemovableMediaHandler& IMMRemovableMediaHandler::stopHandlingEventsFor( IWindow* window )
{
   ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR, IErrorInfo::invalidRequest,
                      IException::recoverable);
   return *this;
}
