/*******************************************************************************
* FILE NAME: immspeed.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immspeed.hpp                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
#include <immspeed.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMSPEED_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMSpeed::IMMSpeed                                                           |
| Construct an object using the IMMSpeed format and value passed.              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMSpeed::IMMSpeed(IMMSpeed::Format form, unsigned long val)
   : fFormat(form),
     fSpeed(val)
{ }

/*------------------------------------------------------------------------------
| IMMSpeed::IMMSpeed                                                           |
| Construct an object using the percentage format from the value passed.       |
|                                                                              |
------------------------------------------------------------------------------*/
IMMSpeed::IMMSpeed(unsigned long percent)
   : fFormat(IMMSpeed::percentage),
     fSpeed(percent)
{ }

/*------------------------------------------------------------------------------
| IMMSpeed::~IMMSpeed                                                          |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMSpeed::~IMMSpeed()
{ }

/*------------------------------------------------------------------------------
| IMMSpeed::format                                                             |
| Returns the current speed format. This is either in frames-per-second        |
| or percentage.                                                               |
------------------------------------------------------------------------------*/
IMMSpeed::Format IMMSpeed::format() const
{
  return fFormat;
}

/*------------------------------------------------------------------------------
| IMMSpeed::speed                                                              |
| Returns the speed value as either a percentage or in frames-per-second.      |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long  IMMSpeed::speed() const
{
  return fSpeed;
}
