/*******************************************************************************
* FILE NAME: immttime.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immttime.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #include <string.h>
   #define INCL_MCIOS2
   #define IC_MULTIMEDIA
   #include <iwindefs.h>
   }

#ifndef _ITRACE_
#include <itrace.hpp>
#endif

#ifndef _IMMCDDA_
   #include <immcdda.hpp>
#endif

#include <immttime.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMTTIME_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime                             |
| This constructor creates a new device object.                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime(const IMMTrackMinSecFrameTime &tmsf)
                       : IMMTime(tmsf),
                         fStart(tmsf.fStart),
                         fEnd(tmsf.fEnd),
                         fTrackMinSecFrame(tmsf.fTrackMinSecFrame)
{ }

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime                             |
| This constructor creates a new device object.                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime(IMMAudioCDContents &toc,
                                                 unsigned long value)
                       : IMMTime(),
                         fTrackMinSecFrame(value)
{
  unsigned long track = value & 0xFF;
  if (track  &&  track <= toc.numberOfTracks())
  {
     fEnd = toc.timeAt(track,false);
     fStart = toc.timeAt(track,true);
     setMMTime(IMMTime(REDBOOKTOMM((value >> 8))) + fStart);
  } else {
    fEnd = IMMTime();
    fStart = IMMTime();
    setMMTime(IMMTime(REDBOOKTOMM((value >> 8))));
  } /* endif */
  if (asMMTime() <= fEnd)
     setValid(true);
  else
     setValid(false);
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime                             |
| This constructor creates a new device object.                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime(IMMAudioCDContents &toc,
                         unsigned long track,
                         unsigned long minutes,
                         unsigned long seconds,
                         unsigned long frames)
                       : IMMTime(),
                         fTrackMinSecFrame(track | (minutes << 8) |
                                           (seconds << 16) | (frames << 24))
{
  if (track && track <= toc.numberOfTracks())
  {
     fEnd = toc.timeAt(track,false);
     fStart = toc.timeAt(track,true);
     setMMTime(IMMTime(REDBOOKTOMM(minutes | (seconds << 8) | (frames << 16))) + fStart);
  } else {
     fEnd = IMMTime();
     fStart = IMMTime();
     setMMTime(IMMTime(REDBOOKTOMM(minutes | (seconds << 8) | (frames << 16))));
  } /* endif */
  if (asMMTime() <= fEnd)
     setValid(true);
  else
     setValid(false);
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::~IMMTrackMinSecFrameTime                            |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime::~IMMTrackMinSecFrameTime()
{ }

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::frames                                              |
| Returns the frames component of the time.                                    |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMTrackMinSecFrameTime::frames() const
{
  return ((fTrackMinSecFrame >> 24) & 0xFF);
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::minutes                                             |
| Returns the minutes component of the time.                                   |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMTrackMinSecFrameTime::minutes() const
{
  return ((fTrackMinSecFrame >> 8) & 0xFF);
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::seconds                                             |
| Returns the seconds component of the time.                                   |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMTrackMinSecFrameTime::seconds() const
{
  return ((fTrackMinSecFrame >> 16) & 0xFF);
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::track                                               |
| Returns the track component of the time.                                     |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMTrackMinSecFrameTime::track() const
{
  return (fTrackMinSecFrame & 0xFF);
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::asString                                            |
| Returns the time value as a string formatted as TT MM:SS.FF                  |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMTrackMinSecFrameTime::asString() const
{
  return IString(track()) + " " + IString(minutes()) + ":" +
         IString(seconds()).rightJustify(2, '0') + "." +
         IString(frames()).rightJustify(3, '0');
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::operator unsigned long                              |
| Returns the time as an unsigned long, 1 byte per component, in the following |
| order: (FFSSMMTT).                                                           |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime::operator unsigned long () const
{
  return fTrackMinSecFrame;
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::operator =                                          |
| Sets the time to be equal to the passed in time.                             |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime &IMMTrackMinSecFrameTime::operator = (const IMMTrackMinSecFrameTime &time)
{
  setMMTime(time.asMMTime());
  setValid(time.isValid());
  fStart = time.fStart;
  fEnd   = time.fEnd;
  fTrackMinSecFrame  = time.fTrackMinSecFrame;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::operator +=                                         |
| Adds the time to the current set time.                                       |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime &IMMTrackMinSecFrameTime::operator += (const IMMTime &time)
{
  setMMTime(asMMTime() + time.asMMTime());
  if (asMMTime() > fEnd.asMMTime() ||
      asMMTime() < fStart.asMMTime())
  {
     setValid(false);
  }
  else
  {
     fTrackMinSecFrame = (REDBOOKFROMMM(asMMTime() - fStart.asMMTime()) << 8) | (fTrackMinSecFrame & 0xFF);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::operator -=                                         |
| Subtracts the time from the current set time.                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime &IMMTrackMinSecFrameTime::operator -= (const IMMTime &time)
{
  setMMTime(asMMTime() - time.asMMTime());

  if (asMMTime() > fEnd.asMMTime() ||
      asMMTime() < fStart.asMMTime())
  {
     setValid(false);
  }
  else
  {
     fTrackMinSecFrame = (REDBOOKFROMMM(asMMTime() - fStart.asMMTime()) << 8) | (fTrackMinSecFrame & 0xFF);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::operator +                                          |
| Returns the sum of the current set time and the passed in time.              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime IMMTrackMinSecFrameTime::operator + (const IMMTime &time)
{
  IMMTrackMinSecFrameTime temp(*this);
  temp += time;
  return temp;
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::operator -                                          |
| Returns the result of the current set time minus the passed in time.         |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime IMMTrackMinSecFrameTime::operator - (const IMMTime &time)
{
  IMMTrackMinSecFrameTime temp(*this);
  temp -= time;
  return temp;
}

