/*******************************************************************************
* FILE NAME: immwave.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immwave.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #include <string.h>
   #define INCL_MCIOS2
   #define INCL_MMIOOS2
   #define IC_MULTIMEDIA
   #include <iwindefs.h>
   }

#ifndef _ITRACE_
   #include <itrace.hpp>
#endif

#include <immwave.hpp>

#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif
#include <immowin.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMWAVE_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMWaveAudio::IMMWaveAudio                                                   |
| This constructor creates a new device object.                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMMWaveAudio::IMMWaveAudio( Boolean openNow,
                            unsigned long instance,
                            Boolean openShareable )
            : IMMConfigurableAudio("WAVEAUDIO", openNow,
                                   instance, openShareable)
{ }

/*------------------------------------------------------------------------------
| IMMWaveAudio::~IMMWaveAudio                                                  |
| Virtual destructor - nothing special needed as parent does cleanup.          |
|                                                                              |
------------------------------------------------------------------------------*/
IMMWaveAudio::~IMMWaveAudio()
{ }

/*------------------------------------------------------------------------------
| IMMWaveAudio::copyFromBuffer                                                 |
| Copies all of the audio data from the audio buffer to the current            |
| location.                                                                    |
------------------------------------------------------------------------------*/
IMMWaveAudio& IMMWaveAudio::copyFromBuffer(const IMMAudioBuffer& audioBuffer,
                                           CallType call)
{
  IASSERTSTATE(isOpen());
  IString buf = audioBuffer.data();

  MCI_EDIT_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback   = HWND(deviceWindow().handle());
  parms.ulStructLen    = sizeof(parms);
  parms.pBuff          = PVOID((char*)buf);
  parms.ulBufLen       = audioBuffer.length();
  parms.pHeader        = PVOID((char*)(audioBuffer.headerData()));

  unsigned long flags = MCI_FROM_BUFFER;
  sendCommand(MCI_COPY, flags | call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMWaveAudio::copyToBuffer                                                   |
| Copies audio data into the audio buffer from the passed in begin             |
| position to the passed in end position.                                      |
------------------------------------------------------------------------------*/
IMMWaveAudio& IMMWaveAudio::copyToBuffer(IMMAudioBuffer& audioBuffer,
                                         const IMMTime &from,
                                         const IMMTime &to,
                                         CallType call) const
{
  IASSERTSTATE(isOpen());

  IString buf(0,audioBuffer.length());
  MCI_EDIT_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback   = HWND(deviceWindow().handle());
  parms.ulStructLen    = sizeof(parms);
  parms.pBuff          = PVOID((char*)buf);
  parms.ulBufLen       = audioBuffer.length();
  parms.pHeader        = PVOID((char*)(audioBuffer.headerData()));

  unsigned long flags = MCI_TO_BUFFER;
  if (from.isValid())
  {
     flags |= MCI_FROM;
     parms.ulFrom = (unsigned long)from;
  }
  if (to.isValid())
  {
     flags |= MCI_TO;
     parms.ulTo = (unsigned long)to;
  }
  ((IMMWaveAudio*)this)->sendCommand(MCI_COPY, flags | call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  audioBuffer.setData((char*)buf,audioBuffer.length());
  return *((IMMWaveAudio*)this);
}

/*------------------------------------------------------------------------------
| IMMWaveAudio::cutToBuffer                                                    |
| Cuts audio data into the audio buffer from the passed in                     |
| start position to the passed in end position.                                |
------------------------------------------------------------------------------*/
IMMWaveAudio& IMMWaveAudio::cutToBuffer(IMMAudioBuffer& audioBuffer,
                                        const IMMTime &from,
                                        const IMMTime &to,
                                        CallType call)
{
  IASSERTSTATE(isOpen());

  IString buf(0,audioBuffer.length());
  MCI_EDIT_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback   = HWND(deviceWindow().handle());
  parms.ulStructLen    = sizeof(parms);
  parms.pBuff          = PVOID((char*)buf);
  parms.ulBufLen       = audioBuffer.length();
  parms.pHeader        = PVOID((char*)(audioBuffer.headerData()));

  unsigned long flags = MCI_TO_BUFFER;
  if (from.isValid())
  {
     flags |= MCI_FROM;
     parms.ulFrom = (unsigned long)from;
  }

  if (to.isValid())
  {
     flags |= MCI_TO;
     parms.ulTo = (unsigned long)to;
  }

  sendCommand(MCI_CUT, flags | call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  audioBuffer.setData((char*)buf,audioBuffer.length());
  return *this;
}

/*------------------------------------------------------------------------------
| IMMWaveAudio::pasteFromBuffer                                                |
| Replaces the audio data with data from the audio buffer, from the passed     |
| in begin position to the passed in end position.                             |
------------------------------------------------------------------------------*/
IMMWaveAudio& IMMWaveAudio::pasteFromBuffer(const IMMAudioBuffer& audioBuffer,
                                            const IMMTime &from,
                                            const IMMTime &to,
                                            CallType call)
{
  IASSERTSTATE(isOpen());

  IString buf = audioBuffer.data();
  MCI_EDIT_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback   = HWND(deviceWindow().handle());
  parms.ulStructLen    = sizeof(parms);
  parms.pBuff          = PVOID((char*)buf);
  parms.ulBufLen       = audioBuffer.length();
  parms.pHeader        = PVOID((char*)(audioBuffer.headerData()));

  unsigned long flags = MCI_FROM_BUFFER;
  if (from.isValid())
  {
     flags |= MCI_FROM;
     parms.ulFrom = (unsigned long)from;
  }
  if (to.isValid())
  {
     flags |= MCI_TO;
     parms.ulTo = (unsigned long)to;
  }
  sendCommand(MCI_PASTE, flags | call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return *this;
}

/*------------------------------------------------------------------------------
| IMMWaveAudio::pasteToBuffer                                                  |
| Pastes audio data into the passed in audio buffer.                           |
|                                                                              |
------------------------------------------------------------------------------*/
IMMWaveAudio& IMMWaveAudio::pasteToBuffer(IMMAudioBuffer& audioBuffer,
                                          CallType call) const
{
  IASSERTSTATE(isOpen());

  IString buf(0,audioBuffer.length());
  MCI_EDIT_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback   = HWND(deviceWindow().handle());
  parms.ulStructLen    = sizeof(parms);
  parms.pBuff          = PVOID((char*)buf);
  parms.ulBufLen       = audioBuffer.length();
  parms.pHeader        = PVOID((char*)(audioBuffer.headerData()));

  unsigned long flags = MCI_TO_BUFFER;
  ((IMMWaveAudio*)this)->sendCommand(MCI_PASTE, flags | call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  audioBuffer.setData((char*)buf,audioBuffer.length());
  return *((IMMWaveAudio*)this);
}

/*------------------------------------------------------------------------------
| IMMWaveAudio::supportsWaveFormat                                             |
| Returns true if the specified format is supported.                           |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMWaveAudio::supportsWaveFormat(unsigned long bitsPerSample,
                                         unsigned long samplesPerSecond,
                                         unsigned long channels,
                                         IMMAudioBuffer::Format tag,
                                         CallType call) const
{
  IASSERTSTATE(isOpen());

  MCI_WAVE_GETDEVCAPS_PARMS parms;

  memset(&parms, 0, sizeof(parms));
  parms.hwndCallback    = HWND(deviceWindow().handle());
  parms.ulItem          = MCI_GETDEVCAPS_WAVE_FORMAT;
  parms.ulBitsPerSample = bitsPerSample;
  parms.ulFormatTag     = (unsigned long)tag;
  parms.ulSamplesPerSec = samplesPerSecond;
  parms.ulChannels      = channels;
  ((IMMWaveAudio*)this)->sendCommand(MCI_GETDEVCAPS,
                  MCI_GETDEVCAPS_ITEM | MCI_GETDEVCAPS_EXTENDED |
                  call, &parms);
  if (lastError())
     ITHROWMMERROR(lastError(),"sendCommand");
  return parms.ulReturn;
}

/*------------------------------------------------------------------------------
| IMMWaveAudio::isOpenStringValid                                              |
| Returns true if the string passed in is valid for this device.               |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMWaveAudio::isOpenStringValid(const IString &deviceName) const
{
  IString tester(deviceName);
  return (tester.upperCase().indexOf("WAVEAUDIO") == 1 ? true : false);
}


/*------------------------------------------------------------------------------
| IMMWaveAudio::cutCopyBufferSize                                              |
| Returns the size the audio buffer needs to be to contain the audio           |
| data from the passed in beginning position to the passed in ending position. |
------------------------------------------------------------------------------*/
unsigned long IMMWaveAudio::cutCopyBufferSize(const IMMTime &from,
                                              const IMMTime &to) const
{
  IASSERTSTATE(isOpen());

  unsigned long rc = 0;
  MCI_EDIT_PARMS parms;
  char dummy;
  MMAUDIOHEADER header;

  memset(&parms, 0, sizeof(parms));
  memset(&header, 0, sizeof(header));
  parms.ulStructLen = sizeof(parms);

  unsigned long flags = MCI_TO_BUFFER | MCI_WAIT;
  if (from.isValid())
  {
     flags |= MCI_FROM;
     parms.ulFrom = (unsigned long)from;
  }
  if (to.isValid())
  {
     flags |= MCI_TO;
     parms.ulTo = (unsigned long)to;
  }
  parms.pBuff    = &dummy;
  parms.ulBufLen = sizeof(dummy);
  parms.pHeader  = &header;
  ((IMMWaveAudio*)this)->sendCommand(MCI_COPY, flags, &parms);
  if (!lastError() || lastError() == MCIERR_INVALID_BUFFER)
     rc = parms.ulBufLen;
  else
     ITHROWMMERROR(lastError(),"sendCommand");
  return rc;
}
