/*******************************************************************************
* FILE NAME: imultsta.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imultsta.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Note we depend on ireslib.cpp being first
#pragma priority( -2147482102 )
extern "C"
  {
  #define INCL_MCIOS2
  #define IC_MULTIMEDIA
  #include <iwindefs.h>
  }

#include <imultsta.hpp>

#ifndef _IRESLIB_
  #include <ireslib.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMULTSTA_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLNonConst)
  static IMultimediaStatics mmpmMgr;
#pragma data_seg()

IGetErrorString*     IMultimediaStatics::entryGetErrorString  = 0;
ISendString*         IMultimediaStatics::entrySendString      = 0;
ISendCommand*        IMultimediaStatics::entrySendCommand     = 0;
IRegisterSelectionSlider* IMultimediaStatics::entryRegisterSelectionSlider = 0;
IRegisterCircularSlider*  IMultimediaStatics::entryRegisterCircularSlider  = 0;
IRegisterGraphicButton*   IMultimediaStatics::entryRegisterGraphicButton   = 0;
IDynamicLinkLibrary* IMultimediaStatics::mmpmDynLinkLib       = 0;
IDynamicLinkLibrary* IMultimediaStatics::mmctlDynLinkLib      = 0;

/*------------------------------------------------------------------------------
| IMultimediaStatics::~IMultimediaStatics                                      |
------------------------------------------------------------------------------*/
IMultimediaStatics ::  ~IMultimediaStatics ( )
{
  if (mmpmDynLinkLib)
     delete mmpmDynLinkLib;
  if (mmctlDynLinkLib)
     delete mmctlDynLinkLib;
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::mmpmDLL                                                  |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary& IMultimediaStatics :: mmpmDLL ( )
{
  if (!mmpmDynLinkLib)
     mmpmDynLinkLib = new IDynamicLinkLibrary("mdm");
  return *mmpmDynLinkLib;
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::mmctlDLL                                                 |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary& IMultimediaStatics :: mmctlDLL ( )
{
  if (!mmctlDynLinkLib)
     mmctlDynLinkLib = new IDynamicLinkLibrary("sw");
  return *mmctlDynLinkLib;
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::registerGraphicButton                                    |
------------------------------------------------------------------------------*/
Boolean
  IMultimediaStatics :: registerGraphicButton()
{
  if (!entryRegisterGraphicButton)
     entryRegisterGraphicButton =
        (IRegisterGraphicButton*)mmctlDLL().procAddress( "WinRegisterGraphicButton");
  return (*entryRegisterGraphicButton)();
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::registerSelectionSlider                                  |
------------------------------------------------------------------------------*/
Boolean
  IMultimediaStatics :: registerSelectionSlider()
{
  if (!entryRegisterSelectionSlider)
     entryRegisterSelectionSlider =
        (IRegisterSelectionSlider*)mmctlDLL().procAddress( "WinRegisterSelectionSlider");
  return (*entryRegisterSelectionSlider)();
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::registerCircularSlider                                   |
------------------------------------------------------------------------------*/
Boolean
  IMultimediaStatics :: registerCircularSlider()
{
  if (!entryRegisterCircularSlider)
     entryRegisterCircularSlider =
        (IRegisterCircularSlider*)mmctlDLL().procAddress( "WinRegisterCircularSlider");
  return (*entryRegisterCircularSlider)();
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::getErrorString                                           |
------------------------------------------------------------------------------*/
unsigned long
  IMultimediaStatics :: getErrorString( unsigned long error, IString &buffer,
                                        unsigned short bufferLength)
{
  if (!entryGetErrorString)
     entryGetErrorString =
        (IGetErrorString*)mmpmDLL().procAddress( "mciGetErrorString");
  return (*entryGetErrorString)(error,(char*)buffer,bufferLength);
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::sendString                                               |
------------------------------------------------------------------------------*/
unsigned long
  IMultimediaStatics :: sendString( const IString &command, IString &returnData,
                                    unsigned short returnLength,
                                    IWindowHandle window,
                                    unsigned short userParam)
{
  if (!entrySendString)
     entrySendString =
        (ISendString*)mmpmDLL().procAddress( "mciSendString");
  return (*entrySendString)((char*)command,(char*)returnData,returnLength,window,userParam);
}

/*------------------------------------------------------------------------------
| IMultimediaStatics::sendCommand                                              |
------------------------------------------------------------------------------*/
unsigned long
  IMultimediaStatics :: sendCommand( unsigned short deviceId,
                                     unsigned short message,
                                     unsigned long param1, void* param2,
                                     unsigned short userParam)
{
  if (!entrySendCommand)
        entrySendCommand =
          (ISendCommand*)mmpmDLL().procAddress("mciSendCommand");
  return (*entrySendCommand)(deviceId,message,param1,param2,userParam);
}

