/*******************************************************************************
* FILE NAME: ibappobs.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of functions that have been replaced *
*   and are now obsolete.  They have been placed here to provide compatibility *
*   with prior releases.                                                       *
*   This is a platform independent file.                                       *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
{
  #define INCL_WINFRAMEMGR
  #define INCL_WINHELP
  #define INCL_WINMENUS
  #define INCL_NLS
  #include <iwindefs.h>
}


#include <iframe.hpp>
#include <iframprv.hpp>
#include <imenu.hpp>
#include <iexcept.hpp>
#include <icolor.hpp>
#include <isubmenu.hpp>
#include <iwindow.hpp>
#include <ithread.hpp>
#include <ihelp.hpp>
#include <ihelphdr.hpp>
#include <imousevt.hpp>
#include <iaccel.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBAPPOBS_CPP_
  #include <ipagetun.h>
#endif

#if (IC_OBSOLETE <= IC_OBSOLETE_1)
//
// Block of obsolete code.  Obsolete level 1
//
#pragma data_seg(ICLStaticConst)
  const IMenu::Style IMenu::verticalFlip = MS_VERTICALFLIP;
#pragma data_seg()

/*------------------------------------------------------------------------------
| IFrameWindow::setColor                                                       |
|                                                                              |
| Change the window attribute color using Red, Green and Blue mix.             |
------------------------------------------------------------------------------*/
IFrameWindow& IFrameWindow::setColor(ColorArea value, const IColor& color)
{
   switch (value) {
     case foreground:
       setForegroundColor( color );
       break;
     case background:
       setBackgroundColor( color );
       break;
     case shadow:
       setShadowColor( color );
       break;
     case disableBackground:
       setDisabledBackgroundColor( color );
       break;
     case activeBorder:
       setActiveColor( color );
       break;
     case inactiveBorder:
       setInactiveColor( color );
       break;
     case frameBorder:
       setBorderColor( color );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                          IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   } /* endswitch */

   return *this;
}

/*------------------------------------------------------------------------------
| IFrameWindow::color                                                          |
|                                                                              |
| Return the color of the window attribute.                                    |
------------------------------------------------------------------------------*/
IColor IFrameWindow::color(ColorArea value) const
{
   IColor
     color(IColor::white);
   switch (value) {
     case foreground:
       color = foregroundColor();
       break;
     case background:
       color = backgroundColor();
       break;
     case shadow:
       color = shadowColor();
       break;
     case disableBackground:
       color = disabledBackgroundColor();
       break;
     case activeBorder:
       color = activeColor();
       break;
     case inactiveBorder:
       color = inactiveColor();
       break;
     case frameBorder:
       color = borderColor();
    default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                          IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   } /* endswitch */
   return color;
}

/*------------------------------------------------------------------------------
| IMenu::setColor                                                              |
------------------------------------------------------------------------------*/
IMenu& IMenu::setColor ( ColorArea  value, const IColor& color )
{
   switch (value) {
     case foreground:
       setForegroundColor( color );
       break;
     case background:
       setBackgroundColor( color );
       break;
     case disabledForeground:
       setDisabledForegroundColor( color );
       break;
     case disabledBackground:
       setDisabledBackgroundColor( color );
       break;
     case highlightForeground:
       setHiliteForegroundColor( color );
       break;
     case highlightBackground:
       setHiliteBackgroundColor( color );
       break;
     case border:
       setBorderColor( color );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                          IErrorInfo::invalidRequest,
                          IException::recoverable);
   } /* endswitch */
   return *this;
}

/*------------------------------------------------------------------------------
| IMenu::color                                                                 |
------------------------------------------------------------------------------*/
IColor IMenu::color   ( ColorArea value ) const
{
   IColor color(IColor::white);

   switch (value) {
     case foreground:
       color = foregroundColor();
       break;
     case background:
       color = backgroundColor();
       break;
     case disabledForeground:
       color = disabledForegroundColor();
       break;
     case disabledBackground:
       color = disabledBackgroundColor();
       break;
     case highlightForeground:
       color = hiliteForegroundColor();
       break;
     case highlightBackground:
       color = hiliteBackgroundColor();
       break;
     case border:
       color = borderColor();
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                          IErrorInfo::invalidRequest,
                          IException::recoverable);
       break;
   } /* endswitch */

   return color;
}

/*------------------------------------------------------------------------------
| IWindow::windowWithId                                                        |
|                                                                              |
| Search the internal collection for the iwindow with                          |
| the passed id and owner.                                                     |
------------------------------------------------------------------------------*/
IWindow* IWindow::windowWithId(unsigned long ulId,
                               const IWindow* owner,
                               Boolean allThreads)
{
   return (windowWithOwner(ulId, owner, allThreads));
}

/*-----------------------------------------------------------------------------|
| IWindow::isDisabled                                                          |
|                                                                              |
| Is the object disabled?                                                      |
------------------------------------------------------------------------------*/
IBase::Boolean IWindow::isDisabled() const
{
   return !isEnabled();
}

/*------------------------------------------------------------------------------
| IWindow::handleWithId                                                        |
|                                                                              |
| Return the window handle with the passed id and parent.                      |
------------------------------------------------------------------------------*/
IWindowHandle IWindow::handleWithId( unsigned long ulId,
                                     const IWindowHandle& parent)
{
   return handleWithParent (ulId, parent);
}

/*------------------------------------------------------------------------------
| IMenu::isItemDisabled                                                        |
------------------------------------------------------------------------------*/
IBase::Boolean IMenu::isItemDisabled(unsigned long ulItemId) const
{
  return (!(isItemEnabled(ulItemId)));
}

/*------------------------------------------------------------------------------
| IMenu::setVerticalFlip                                                       |
------------------------------------------------------------------------------*/
IMenu& IMenu::setVerticalFlip ( Boolean flip)
{
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (!flip)
  {
    ulStyle &= ~verticalFlip.asUnsignedLong();
  }
  else
  {
    ulStyle |= verticalFlip.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
  }
  return *this;
}
/*------------------------------------------------------------------------------
   Test for style.
------------------------------------------------------------------------------*/
IBase::Boolean IMenu::isVerticalFlip ( ) const
{
   return (style() & verticalFlip.asUnsignedLong()) ? true : false;
}



/*------------------------------------------------------------------------------
| IThread::setDefaultAutoInitPM                                                |
------------------------------------------------------------------------------*/
void IThread :: setDefaultAutoInitPM ( Boolean newFlag )
  {
  IThread::dfltAutoInitGUI = newFlag;
  }

/*------------------------------------------------------------------------------
| IThread::defaultAutoInitPM                                                   |
------------------------------------------------------------------------------*/
IBase::Boolean IThread :: defaultAutoInitPM ( )
  {
  return defaultAutoInitGUI();
  }


/*------------------------------------------------------------------------------
| IHelpWindow::associateWindow                                                 |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::associateWindow( IFrameWindow* frameWindow )
{
  return setAssociatedWindow (frameWindow);
}

/*------------------------------------------------------------------------------
| IHelpWindow::helpForWindow                                                   |
------------------------------------------------------------------------------*/
IFrameWindow* IHelpWindow :: helpForWindow( ) const
{
  unsigned long handle = this->sendEvent(HM_QUERY,
                                      IEventParameter1( 0, HMQW_ACTIVEVIEWPORT),
                                      IEventParameter2(0) );
  if (handle)
     return (IFrameWindow*)( IWindow::windowWithHandle(handle) );
  else
     return 0;
}

/*------------------------------------------------------------------------------
| IFrameWindow::flagsFrom                                                      |
|                                                                              |
| Convert the argument Style value to corresponding FCF_* flags.               |
| For WinNT, this function returns extended styles.                            |
------------------------------------------------------------------------------*/
unsigned long IFrameWindow::flagsFrom ( const Style &style )
  {
  unsigned long
    result = 0;
  if ( style & IFrameWindow::menuBar )
    result |= FCF_MENU;
  if ( style & IFrameWindow::hideButton )
    result |= FCF_HIDEBUTTON;
  if ( style & IFrameWindow::horizontalScroll )
    result |= FCF_HORZSCROLL;
  if ( style & IFrameWindow::verticalScroll )
    result |= FCF_VERTSCROLL;
  if ( style & IFrameWindow::maximizeButton )
    result |= FCF_MAXBUTTON;
  if ( style & IFrameWindow::minimizeButton )
    result |= FCF_MINBUTTON;
  if ( style & IFrameWindow::systemMenu )
    result |= FCF_SYSMENU;
  if ( style & IFrameWindow::titleBar )
    result |= FCF_TITLEBAR;
  if ( style & IFrameWindow::dialogBackground )
    result |= FCF_AUTOICON;
  if ( style & IFrameWindow::appDBCSStatus )
    result |= FCF_DBE_APPSTAT;
  return result;
  }

/*------------------------------------------------------------------------------
| IFrameWindow::styleFrom                                                      |
|                                                                              |
| Convert the argument Style value to corresponding WS_* and FS_* flags.       |
------------------------------------------------------------------------------*/
unsigned long IFrameWindow::styleFrom ( const Style &style )
  {
  return style.asUnsignedLong() & IFrameWindow::styleBits.asUnsignedLong();
  }

/*------------------------------------------------------------------------------
| IMouseClickEvent::handleOnTop                                                |
|                                                                              |
| Get the window handle that the mouse is positioned over.                     |
------------------------------------------------------------------------------*/
IWindowHandle IMouseClickEvent::handleOnTop ( ) const
{
  return windowUnderPointer();
}

/*------------------------------------------------------------------------------
| IMouseClickEvent::mouseNumber                                                |
|                                                                              |
| Determine which mouse button is generating the event.                        |
------------------------------------------------------------------------------*/
IMouseClickEvent::MouseNumber  IMouseClickEvent::mouseNumber ( ) const
{
  return mouseButton();
}

/*------------------------------------------------------------------------------
| IGUIColor::sysColor                                                          |
|                                                                              |
| Return the system color that this object represents.                         |
------------------------------------------------------------------------------*/
IGUIColor::SysColor IGUIColor::sysColor ( ) const
{
  return this->systemColor();
}

/*------------------------------------------------------------------------------
| IAccelerator::unset                                                          |
------------------------------------------------------------------------------*/
IAccelerator& IAccelerator :: unset()
{
  return this->reset();
}

//
// End block of obsolete code.  Obsolete level 1
//

#endif // IC_OBSOLETE
