/*******************************************************************************
* FILE NAME: ibctlobs.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of functions that have been replaced *
*   and are now obsolete.  They have been placed here to provide compatibility *
*   with prior releases.                                                       *
*   This is the platform independent file.  See ibctlob1.cpp and ibctlob2.cpp  *
*   for platform dependent code.                                               *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <ibutton.hpp>
#include <icanvas.hpp>
#include <icconst.h>
#include <icmnfun.hpp>
#include <icombobx.hpp>
#include <icombobs.hpp>
#include <ientryfd.hpp>
#include <iexcept.hpp>
#include <igroupbx.hpp>
#include <ilistbox.hpp>
#include <imcelcv.hpp>
#include <imle.hpp>
#include <inotebk.hpp>
#include <ioutlbox.hpp>
#include <ipushbut.hpp>
#include <iscroll.hpp>
#include <isetcv.hpp>
#include <islider.hpp>
#include <ispinbt.hpp>
#include <isplitcv.hpp>
#include <istattxt.hpp>
#include <ititle.hpp>
#include <ivport.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBCTLOBS_CPP_
  #include <ipagetun.h>
#endif

#if (IC_OBSOLETE <= IC_OBSOLETE_1)

//
//
//  Block of obsolete function for obsolete level 1
//
//

/******************************************************************************/
/* Public combobox styles - Replaced by IBaseComboBox::readOnlyDropDownType   */
/******************************************************************************/
#pragma data_seg(ICLStaticConst)
const IComboBox::Style
  IComboBox::dropDownListType = IBaseComboBox::readOnlyDropDownType;
#pragma data_seg()


/*------------------------------------------------------------------------------
| ICanvas::setColor                                                            |
|                                                                              |
| Set the background color of the canvas.                                      |
------------------------------------------------------------------------------*/
ICanvas &  ICanvas :: setColor ( ColorArea area,
                                 const IColor & color )
{
  IASSERTPARM(area == ICanvas::background);  // Only ColorArea value
  setBackgroundColor( color );
  return *this;
}

/*------------------------------------------------------------------------------
| ICanvas::color                                                               |
|                                                                              |
| Return the background color of the canvas.                                   |
------------------------------------------------------------------------------*/
IColor  ICanvas :: color ( ColorArea area ) const
{
  IASSERTPARM(area == ICanvas::background);  // Only ColorArea value

  return backgroundColor(  );
}

/*------------------------------------------------------------------------------
| Function Name: IMultiCellCanvas :: setColor                                  |
|                                                                              |
| Implementation:                                                              |
------------------------------------------------------------------------------*/
IMultiCellCanvas& IMultiCellCanvas :: setColor ( const IColor& color )
{
  setBackgroundColor( color );
  return *this;
}

/*------------------------------------------------------------------------------
| Function Name: IMultiCellCanvas :: Color                                     |
|                                                                              |
| Implementation:                                                              |
------------------------------------------------------------------------------*/
IColor IMultiCellCanvas :: color ( ) const
{
  return backgroundColor( );
}

/*------------------------------------------------------------------------------
| ISetCanvas::setColor                                                         |
|                                                                              |
| Return background color of the canvas.                                       |
------------------------------------------------------------------------------*/
ISetCanvas  &ISetCanvas :: setColor ( ColorArea area,
                                      const IColor& color )
{
  IASSERTPARM(area == ISetCanvas::background);
  setBackgroundColor( color );
  return *this;
}

/*------------------------------------------------------------------------------
| ISetCanvas::color                                                            |
|                                                                              |
| Return the specified color of the canvas.                                    |
------------------------------------------------------------------------------*/
IColor  ISetCanvas :: color ( ColorArea area ) const
{
  IASSERTPARM(area == ISetCanvas::background);

  return backgroundColor( );
}

/*------------------------------------------------------------------------------
| IViewPort::setColor                                                          |
|                                                                              |
| Sets the specified color of the specified area of the view port.             |
------------------------------------------------------------------------------*/
IViewPort  &IViewPort :: setColor ( ColorArea area,
                                    const IColor& color )
{
  IASSERTPARM(area == IViewPort::fillBackground);
  setBackgroundColor( color );
  return *this;
}

/*------------------------------------------------------------------------------
| IViewPort::color                                                             |
|                                                                              |
| Return the color of the specified area of the view port.                     |
------------------------------------------------------------------------------*/
IColor  IViewPort :: color ( ColorArea area ) const
{
  IASSERTPARM(area == IViewPort::fillBackground);

  return backgroundColor( );
}

/*------------------------------------------------------------------------------
| IListBox::setColor                                                           |
|                                                                              |
| Change the window attribute color using Red, Green, and Blue mix.            |
------------------------------------------------------------------------------*/
IListBox& IListBox::setColor(ColorArea value, const IColor& color)
{

   switch (value) {
     case foreground:
       setForegroundColor( color );
       break;
     case disabledForeground:
       setDisabledForegroundColor( color );
       break;
     case highlightForeground:
       setHiliteForegroundColor( color );
       break;
     case border:
       setBorderColor( color );
       break;
     case background:
       setBackgroundColor( color );
       break;
     default:
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,IErrorInfo::invalidParameter,
                         IException::recoverable);
       break;
   } /* endswitch */

   return *this;
}

/*------------------------------------------------------------------------------
| IListBox::color                                                              |
------------------------------------------------------------------------------*/
IColor IListBox::color(ColorArea value) const
{
   IColor color(IColor::white);

   switch (value) {
     case foreground:
       color = foregroundColor( );
       break;
     case disabledForeground:
       color = disabledForegroundColor( );
       break;
     case highlightForeground:
       color = hiliteForegroundColor( );
       break;
     case border:
       color = borderColor( );
       break;
     case background:
       color = backgroundColor( );
       break;
     default:
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,IErrorInfo::invalidParameter,
                         IException::recoverable);
       break;
   } /* endswitch */
   return color;
}

/*------------------------------------------------------------------------------
| INotebook::setColor                                                          |
|                                                                              |
| Sets the color of various regions of the notebook.                           |
------------------------------------------------------------------------------*/
INotebook& INotebook :: setColor ( ColorArea area, const IColor& color )
{
  switch (area)
     {
     case pageBackground           :
       setBackgroundPageColor( color );
       break;
     case majorTabBackground       :
       setBackgroundMajorColor( color );
       break;
     case majorTabForeground       :
       setForegroundMajorColor( color );
       break;
     case minorTabBackground       :
       setBackgroundMinorColor( color );
       break;
     case minorTabForeground       :
       setForegroundMinorColor( color );
       break;
     case notebookWindowBackground :
       setBackgroundColor( color );
       break;
     case notebookOutline          :
       setBorderColor( color );
       break;
     case statusLineText           :
       setForegroundColor( color );
       break;
     case selectionCursor          :
       setHiliteBackgroundColor( color );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
     }
  return *this;
}


/*------------------------------------------------------------------------------
| INotebook::color                                                             |
|                                                                              |
| Return the color of an area of the notebook.                                 |
------------------------------------------------------------------------------*/
IColor INotebook :: color ( ColorArea area ) const
{
  IColor color(IColor::white);
  switch (area)
     {
     case pageBackground           :
       color = backgroundPageColor( );
       break;
     case majorTabBackground       :
       color = backgroundMajorColor(  );
       break;
     case majorTabForeground       :
       color = foregroundMajorColor( );
       break;
     case minorTabBackground       :
       color = backgroundMinorColor( );
       break;
     case minorTabForeground       :
       color = foregroundMinorColor( );
       break;
     case notebookWindowBackground :
       color = backgroundColor( );
       break;
     case notebookOutline          :
       color = borderColor( );
       break;
     case statusLineText           :
       color = foregroundColor( );
       break;
     case selectionCursor          :
       color = hiliteBackgroundColor( );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
     }

  return color;
}



/*------------------------------------------------------------------------------
| IOutlineBox::setColor                                                        |
|                                                                              |
| Change the window attribute color using Red, Green, and Blue mix.            |
------------------------------------------------------------------------------*/
IOutlineBox& IOutlineBox::setColor(ColorArea value, const IColor& color)
{
   switch (value) {
     case fillRegion:
       setForegroundColor( color );
       break;
     default:
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                         IErrorInfo::invalidParameter,
                         IException::recoverable);
       break;
   }
   return *this;
}

/*------------------------------------------------------------------------------
| IOutlineBox::color                                                           |
|                                                                              |
| Return the color of window attribute.                                        |
------------------------------------------------------------------------------*/
IColor IOutlineBox::color(ColorArea value) const
{
   IColor color(IColor::white);

   switch (value)
   {
     case fillRegion:
       switch (outlineType())
       {
         case foreground:
              color = foregroundColor( );
              break;
         case background:
         case halftone:
              color = backgroundColor( );
              break;
         default:
              color = backgroundColor( );
              break;
       }
       break;
     default:
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                         IErrorInfo::invalidParameter,
                         IException::recoverable);
       break;
   }
   return color;
}

/*------------------------------------------------------------------------------
| IProgressIndicator::setColor                                                 |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator::setColor (ColorArea value,
                                                  const IColor& color)
{
   switch (value)
   {
     case foreground:
       setForegroundColor( color );
       break;
     case background:
       setBackgroundColor( color );
       break;
     case border:
       setBorderColor( color );
       break;
    default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                          IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   }
   return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator::color                                                    |
------------------------------------------------------------------------------*/
IColor IProgressIndicator :: color ( ColorArea value ) const
{
   IColor color(IColor::white);

   switch (value)
   {
     case foreground:
       color = foregroundColor( );
       break;
     case background:
       color = backgroundColor( );
       break;
     case border:
       color = borderColor( );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                          IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   }

   return color;
}

/*------------------------------------------------------------------------------
| IScrollBar::setColor                                                         |
------------------------------------------------------------------------------*/
IScrollBar &  IScrollBar :: setColor ( ColorArea area,
                                       const IColor & color)
{
  switch (area)
  {
     case shaft:
        setForegroundColor( color );
        break;
     case scrollBox:
        setHiliteForegroundColor( color );
        break;
     case border:
       setBorderColor( color );
       break;
     default:
        ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                           IErrorInfo::invalidParameter,
                           IException::recoverable);
        break;
  }

  return *this;
}


/*------------------------------------------------------------------------------
| IScrollBar::color                                                            |
|                                                                              |
| Return the color of window attributre.                                       |
------------------------------------------------------------------------------*/
IColor  IScrollBar :: color ( ColorArea area ) const
{
   IColor color(IColor::white);

   switch (area)
   {
      case shaft:
         color = foregroundColor( );
         break;
      case scrollBox:
         color = hiliteForegroundColor( );
         break;
      case border:
         color = borderColor( );
         break;
      default:
         ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                            IErrorInfo::invalidParameter,
                            IException::recoverable);
         break;
   }

   return color;
}

/*------------------------------------------------------------------------------
| ISpinButton::setColor                                                        |
|                                                                              |
| Change the color of a specific area.                                         |
------------------------------------------------------------------------------*/
ISpinButton& ISpinButton :: setColor ( ColorArea  colorValue,
                                       const IColor& color )
{
   switch (colorValue) {
     case foreground:
       setForegroundColor( color );
       break;
     case disabledForeground:
       setDisabledForegroundColor( color );
       break;
     case highlightForeground:
       setHiliteForegroundColor( color );
       break;
     case border:
       setBorderColor( color );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                          IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   }

   return *this;
}

/*------------------------------------------------------------------------------
| ISpinButton::color                                                           |
|                                                                              |
| Query a specific area color.                                                 |
------------------------------------------------------------------------------*/
IColor ISpinButton :: color   ( ColorArea colorValue ) const
{
   IColor color(IColor::white);

   switch (colorValue) {
     case foreground:
       color = foregroundColor( );
       break;
     case disabledForeground:
       color = disabledForegroundColor( );
       break;
     case highlightForeground:
       color = hiliteForegroundColor( );
       break;
     case border:
       color = borderColor( );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                          IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   }
   return color;
}

/*------------------------------------------------------------------------------
| IButton::setColor                                                            |
|                                                                              |
| Change the window attribute color using Red, Green and Blue mix              |
------------------------------------------------------------------------------*/
IButton& IButton :: setColor ( ColorArea value, const IColor& color )
{
   switch (value) {
     case foreground:
       setForegroundColor( color );
       break;
     case background:
       setBackgroundColor( color );
       break;
     case disabledForeground:
       setDisabledForegroundColor( color );
       break;
     case highlightForeground:
       setHiliteForegroundColor( color );
       break;
     case border:
       setBorderColor( color );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                          IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   }

   return *this;
}


/*------------------------------------------------------------------------------
| IButton::color                                                               |
|                                                                              |
| Return the color of window attribute.                                        |
------------------------------------------------------------------------------*/
IColor IButton :: color ( ColorArea value ) const
{
   IColor  color( IColor::white );

   switch (value) {
     case foreground:
       color = foregroundColor( );
       break;
     case background:
       color = backgroundColor( );
       break;
     case disabledForeground:
       color = disabledForegroundColor( );
       break;
     case highlightForeground:
       color = hiliteForegroundColor( );
       break;
     case border:
       color = borderColor( );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   }

   return color;
}

/*------------------------------------------------------------------------------
| IEntryField::isReadOnly                                                      |
------------------------------------------------------------------------------*/
Boolean  IEntryField :: isReadOnly() const
{
  return !(this->isWriteable());
}

/*------------------------------------------------------------------------------
| IEntryField::setColor                                                        |
|                                                                              |
| Change the window attribute color using Red, Green and Blue mix.             |
------------------------------------------------------------------------------*/
IEntryField& IEntryField::setColor(ColorArea value, const IColor& color)
{
   switch (value) {
     case foreground:
       setForegroundColor( color );
       break;
     case background:
       setBackgroundColor( color );
       break;
     case disabledForeground:
       setDisabledForegroundColor( color );
       break;
     case highlightForeground:
       setHiliteForegroundColor( color );
       break;
     case border:
       setBorderColor( color );
       break;
     default:
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                         IErrorInfo::invalidParameter,
                         IException::recoverable);
       break;
   }

   return *this;
}


/*------------------------------------------------------------------------------
| IEntryField::color                                                           |
|                                                                              |
| Return the color of the window attribute.                                    |
------------------------------------------------------------------------------*/
IColor IEntryField::color(ColorArea value) const
{
   IColor color(IColor::white);

   switch (value) {
     case foreground:
       color = foregroundColor( );
       break;
     case background:
       color = backgroundColor( );
       break;
     case disabledForeground:
       color = disabledForegroundColor( );
       break;
     case highlightForeground:
       color = hiliteForegroundColor( );
       break;
     case border:
       color = borderColor( );
       break;
     default:
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                         IErrorInfo::invalidParameter,
                         IException::recoverable);
       break;
   }
   return color;
}

/*------------------------------------------------------------------------------
| IGroupBox::setColor                                                          |
|                                                                              |
| Change the window attribute color using Red, Green and Blue mix.             |
------------------------------------------------------------------------------*/
IGroupBox& IGroupBox::setColor(ColorArea value, const IColor& color)
{

   switch (value) {
     case foreground:
       setForegroundColor( color );
       break;
     case border:
       setBorderColor( color );
       break;
     default:
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                         IErrorInfo::invalidParameter,
                         IException::recoverable);
       break;
   }
   return *this;
}


/*------------------------------------------------------------------------------
| IGroupBox::color                                                             |
|                                                                              |
| Return the color of the window attribute.                                    |
------------------------------------------------------------------------------*/
IColor IGroupBox::color(ColorArea value) const
{
   IColor color(IColor::white);

   switch (value) {
     case foreground:
       color = foregroundColor( );
       break;
     case border:
       color = borderColor( );
       break;
     default:
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                         IErrorInfo::invalidParameter,
                         IException::recoverable);
       break;
   }

   return color;
}

/*------------------------------------------------------------------------------
| IMultiLineEdit::setColor                                                     |
|                                                                              |
| Changes the color attribute of the specified color area.                     |
------------------------------------------------------------------------------*/
IMultiLineEdit&  IMultiLineEdit :: setColor(ColorArea value,
                                            const IColor& color)
{
   switch (value) {
     case foreground:
       setForegroundColor( color );
       break;
     case background:
       setBackgroundColor( color );
       break;
     case bordercolor:
       setBorderColor( color );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   } /* endswitch */

   return *this;
}

/*------------------------------------------------------------------------------
| IMultiLineEdit::color                                                        |
|                                                                              |
| Return the color of window attribute.                                        |
------------------------------------------------------------------------------*/
IColor IMultiLineEdit::color(ColorArea value) const
{
   IColor color(IColor::white);

   switch (value)
   {
     case foreground:
       color = foregroundColor( );
       break;
     case background:
       color = backgroundColor( );
       break;
     case bordercolor:
       color = borderColor( );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   } /* endswitch */

   return color;
}

/*------------------------------------------------------------------------------
| IStaticText::setColor                                                        |
|                                                                              |
| Change the window attribute color using Red, Green and Blue mix.             |
------------------------------------------------------------------------------*/
IStaticText& IStaticText :: setColor(ColorArea value, const IColor& color)
{
   switch (value) {
     case foreground:
       setForegroundColor( color );
       break;
     case background:
       setBackgroundColor( color );
       break;
     case fill:
       setFillColor( color );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                          IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   }
   return *this;
}

/*------------------------------------------------------------------------------
| IStaticText::color                                                           |
|                                                                              |
| Return the color of the window attribute.                                    |
------------------------------------------------------------------------------*/
IColor IStaticText :: color(ColorArea value) const
{
   IColor color(IColor::white);
   switch (value) {
     case foreground:
       color = foregroundColor( );
       break;
     case background:
       color = backgroundColor( );
       break;
     case fill:
       color = fillColor();
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                          IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   }
   return color;
}

/*------------------------------------------------------------------------------
| ITitle::setColor                                                             |
------------------------------------------------------------------------------*/
ITitle& ITitle::setColor(ColorArea value, const IColor& color)
{
   switch (value) {
     case activeFill:
       setActiveColor( color );
       break;
     case inactiveFill:
       setInactiveColor( color );
       break;
     case activeTextForeground:
       setActiveTextForegroundColor( color );
       break;
     case inactiveTextForeground:
       setInactiveTextForegroundColor( color );
       break;
     case activeTextBackground:
       setActiveTextBackgroundColor( color );
       break;
     case inactiveTextBackground:
       setInactiveTextBackgroundColor( color );
       break;
     case border:
       setBorderColor( color );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   } /* endswitch */

   return *this;
}

/*------------------------------------------------------------------------------
| ITitle::color                                                                |
------------------------------------------------------------------------------*/
IColor ITitle::color(ColorArea value) const
{
   IColor color(IColor::white);

   switch (value)
   {
     case activeFill:
       color = activeColor( );
       break;
     case inactiveFill:
       color = inactiveColor( );
       break;
     case activeTextForeground:
       color = activeTextForegroundColor( );
       break;
     case inactiveTextForeground:
       color = inactiveTextForegroundColor( );
       break;
     case activeTextBackground:
       color = activeTextBackgroundColor( );
       break;
     case inactiveTextBackground:
       color = inactiveTextBackgroundColor( );
       break;
     case border:
       color = borderColor( );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   } /* endswitch */

   return color;
}


/*------------------------------------------------------------------------------
| ITitle::viewNum                                                              |
------------------------------------------------------------------------------*/
unsigned long ITitle :: viewNum() const
{
  return viewNumber();
}

/*------------------------------------------------------------------------------
| ISplitCanvas::setColor                                                       |
|                                                                              |
| Sets the color to be used for the specified portion of the split bar.  It    |
| must be one of the values of the SplitBarArea enumeration.                   |
------------------------------------------------------------------------------*/
ISplitCanvas& ISplitCanvas :: setColor ( SplitBarArea value,
                                         const IColor& color )
{
   switch (value)
   {
     case splitBarEdge:
       setSplitBarEdgeColor( color );
       break;
     case splitBarMiddle:
       setSplitBarMiddleColor( color );
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   }

   return *this;
}

/*------------------------------------------------------------------------------
| ISplitCanvas::color                                                          |
|                                                                              |
| Returns the color used for the specified portion of the split bar.  It must  |
| be one of the values of the SplitBarArea enumeration.                        |
------------------------------------------------------------------------------*/
IColor ISplitCanvas :: color ( SplitBarArea value ) const
{
   IColor color(IColor::white);

   switch (value)
   {
     case splitBarEdge:
       color = splitBarEdgeColor();
       break;
     case splitBarMiddle:
       color = splitBarMiddleColor();
       break;
     default:
       ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,IErrorInfo::invalidParameter,
                          IException::recoverable);
       break;
   }
   return color;
}


/*------------------------------------------------------------------------------
| IMultiLineEdit::isReadOnly                                                   |
------------------------------------------------------------------------------*/
IBase::Boolean IMultiLineEdit :: isReadOnly ( ) const
{
  return !(this->isWriteable());
}


/*------------------------------------------------------------------------------
| IMultiLineEdit::cursor                                                       |
|                                                                              |
| Returns the line number where the cursor is currently positioned.            |
------------------------------------------------------------------------------*/
unsigned long IMultiLineEdit::cursor() const
{
   return this->cursorLinePosition();
}


/*------------------------------------------------------------------------------
| IPushButton::isBorder                                                        |
------------------------------------------------------------------------------*/
IBase::Boolean IPushButton :: isBorder ( ) const
{
  return this->hasBorder();
}

//
//
//  End of obsolete function for obsolete level 1
//
//
#endif
