/*******************************************************************************
* FILE NAME: ibmpctl.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ibmpctl.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINERRORS            // PMERR_HEAP_MAX_SIZE_REACHED
  #define INCL_WINSTATICS           // SS_xxx
  #define INCL_GPIBITMAPS
  #define INCL_WINSYS
  #include <iwindefs.h>
}

#include <ibase.hpp>
#include <ibmpctl.hpp>
#include <ibmphdr.hpp>
#include <iexcept.hpp>
#include <ihandle.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <iwcname.hpp>
#include <ifont.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBMPCTL_CPP_
  #include <ipagetun.h>
#endif


/***************************************************************/
/* Public bitmap styles.                                       */
/***************************************************************/
#pragma data_seg(ICLStaticConst)
  const IBitmapControl::Style IBitmapControl::sizeToBitmap    = SS_BITMAP;
const IBitmapControl::Style IBitmapControl::classDefaultStyle = WS_VISIBLE;

#pragma data_seg()


/***************************************************************/
/* Default style for new objects (initial value).              */
/***************************************************************/
#pragma data_seg(ICLNonConst)
IBitmapControl::Style IBitmapControl::currentDefaultStyle = WS_VISIBLE;
#pragma data_seg()

/*------------------------------------------------------------------------------
  This class wrappers an object pointer and destroys the pointed
  to object when the wrapper is destructed.
------------------------------------------------------------------------------*/
static struct DefaultHandler {
  operator IBitmapHandler* ();
IBitmapHandler
 *operator -> ()
    {
    return *this;
    }
  ~DefaultHandler ( )
    {
    if (ptr)
      {
      delete ptr;
      ptr = 0;
      }
    }
IBitmapHandler
 *ptr;
} defaultHandler;

/*------------------------------------------------------------------------------
| DefaultHandler::operator IBitmapHandler*                                     |
------------------------------------------------------------------------------*/
DefaultHandler :: operator IBitmapHandler* ()
    {
    if ( !ptr )
      ptr = new IBitmapHandler;
    return ptr;
    }


/*------------------------------------------------------------------------------
| IBitmapControl::IBitmapControl                                               |
------------------------------------------------------------------------------*/
IBitmapControl :: IBitmapControl(unsigned long ulId,
                                 IWindow* pwndParent,
                                 IWindow* pwndOwner,
                                 const IResourceId& bmpId,
                                 const IRectangle& rectInit,
                                 const Style& bmsStyle)
                    : IStaticText(ulId,pwndParent,pwndOwner, rectInit,
                        IWindow::noStyle), fBitmapControlData( 0 ),
                     bmpHandle(0), bSizeTo(false) , returnback(false)
{
  // assertions on input parms
  IASSERTPARM(pwndParent!=0);
  initialize(bmsStyle);
  setBitmap(bmpId);               // load and set bitmap
  if (bmsStyle & IWindow::visible)
  {
     show();
     refresh(IWindow::immediate);
  }
}

/*------------------------------------------------------------------------------
| IBitmapControl::IBitmapControl                                               |
------------------------------------------------------------------------------*/
IBitmapControl :: IBitmapControl(unsigned long ulId,
                                 IWindow* pwndParent,
                                 IWindow* pwndOwner,
                                 const IBitmapHandle& bmpId,
                                 const IRectangle& rectInit,
                                 const Style& bmsStyle)
                    : IStaticText(ulId,pwndParent,pwndOwner, rectInit,
                        IWindow::noStyle), fBitmapControlData( 0 ),
                     bmpHandle(0), bSizeTo(false) , returnback(false)
{
  // assertions on input parms
  IASSERTPARM(pwndParent!=0);
  initialize(bmsStyle);
  setBitmap(bmpId);               // load and set bitmap
  if (bmsStyle & IWindow::visible)
  {
     show();
     refresh(IWindow::immediate);
  }
}

/*------------------------------------------------------------------------------
| IBitmapControl::IBitmapControl                                               |
------------------------------------------------------------------------------*/
IBitmapControl :: IBitmapControl(unsigned long ulId,
                                 IWindow* pwndParent,
                                 IWindow* pwndOwner,
                                 unsigned long bmpId,
                                 const IRectangle& rectInit,
                                 const Style& bmsStyle)
                    : IStaticText(ulId,pwndParent,pwndOwner, rectInit,
                        IWindow::noStyle), fBitmapControlData( 0 ),
                     bmpHandle(0), bSizeTo(false) , returnback(false)
{
  // assertions on input parms
  IASSERTPARM(pwndParent!=0);
  initialize(bmsStyle);
  setBitmap(bmpId);               // load and set bitmap
  if (bmsStyle & IWindow::visible)
  {
     show();
     refresh(IWindow::immediate);
  }
}

/*------------------------------------------------------------------------------
| IBitmapControl::IBitmapControl                                               |
------------------------------------------------------------------------------*/
IBitmapControl :: IBitmapControl(unsigned long ulId,
                                 IWindow* pwndParent,
                                 IWindow* pwndOwner,
                                 ISystemBitmapHandle::Identifier bmp,
                                 const IRectangle& rectInit,
                                 const Style& bmsStyle)
                    : IStaticText(ulId,pwndParent,pwndOwner,rectInit,
                        IWindow::noStyle), fBitmapControlData( 0 ),
                      bmpHandle(0), bSizeTo(false), returnback(false)
{
  // assertions on input parms
  IASSERTPARM(pwndParent!=0);
  initialize(bmsStyle);
  setBitmap(bmp);                 // associate with control
  if (bmsStyle & IWindow::visible)
  {
     show();
     refresh(IWindow::immediate);
  }
}


/*------------------------------------------------------------------------------
| IBitmapControl::IBitmapControl                                               |
|  Instantiate a bitmap object for a template window.                          |
------------------------------------------------------------------------------*/
IBitmapControl :: IBitmapControl(unsigned long ulId,
                                 IWindow* pdlgwndParent)
                    : IStaticText(ulId,pdlgwndParent),
                      fBitmapControlData( 0 ),
                      bmpHandle(0), bSizeTo(true), returnback(true)
{
  disableFillBackground();
  IEventResult evt = handle().sendEvent(SM_QUERYHANDLE,
                                        IEventParameter1(0),
                                        IEventParameter2(0));
  HBITMAP bmp=evt.asUnsignedLong();
  unsigned long ulStyle = style();
  ulStyle &= ~SS_BITMAP;
  ulStyle |=  SS_TEXT;
  setStyle(ulStyle);
  this -> addHandler( defaultHandler );
  setBitmap(IBitmapHandle(bmp));
}


/*------------------------------------------------------------------------------
| IBitmapControl::IBitmapControl                                               |
|  Constructor to instantiate from an existing bitmap control.                 |
------------------------------------------------------------------------------*/
IBitmapControl :: IBitmapControl(const IWindowHandle& wh)
                    : IStaticText(wh) , bSizeTo(true) ,
                      fBitmapControlData( 0 ),
                      returnback(true)
{
  HBITMAP bmp=0;
  disableFillBackground();
  unsigned long ulStyle = style();
  if (ulStyle & (SS_BITMAP & ~SS_TEXT))
  {
     IEventResult evt = handle().sendEvent(SM_QUERYHANDLE,
                                           IEventParameter1(0),
                                           IEventParameter2(0));
     bmp=evt.asUnsignedLong();
  } /* endif */
  ulStyle &= ~SS_BITMAP;
  ulStyle |=  SS_TEXT;
  setStyle(ulStyle);
  this -> addHandler( defaultHandler );
  setBitmap(IBitmapHandle(bmp));
}

/*------------------------------------------------------------------------------
| IBitmapControl::~IBitmapControl                                              |
------------------------------------------------------------------------------*/
IBitmapControl :: ~IBitmapControl ( )
{
    if (returnback && isValid())
    {
       try
       {
          unsigned long ulStyle = style();
          ulStyle &= (SS_BITMAP | ~SS_TEXT);
          setStyle(ulStyle);
          handle().sendEvent(SM_SETHANDLE, IEventParameter1(bmpHandle),
                                           IEventParameter2(0));
       } catch (IException) {}
    }
  // delete fBitmapControlData;
    this -> removeHandler( defaultHandler );
}

/*------------------------------------------------------------------------------
| IBitmapControl::Style  IBitmapControl :: defaultStyle                        |
------------------------------------------------------------------------------*/
IBitmapControl::Style  IBitmapControl :: defaultStyle()
{
  return currentDefaultStyle;
}


/*------------------------------------------------------------------------------
| IBitmapControl::setDefaultStyle                                              |
------------------------------------------------------------------------------*/
void  IBitmapControl :: setDefaultStyle(const IBitmapControl::Style&
                                                        bmsStyle)
{
  currentDefaultStyle = bmsStyle;
}

/*------------------------------------------------------------------------------
| IBitmapControl::convertToGUIStyle                                            |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IBitmapControl :: convertToGUIStyle(const IBitFlag& guiStyle,
                                                  Boolean bExtOnly) const
{
  // Obtain the style from the class (IStaticText) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= extendedStyle() & IS_EXTMASK;
  }
  else
  {
    // Let the clean SS_BITMAP style, flow thru and add the static text style
    ulStyle |= guiStyle.asUnsignedLong() & ISS_BMPMASK;
    ulStyle |= SS_TEXT;
  }

  return( ulStyle );
}


/*------------------------------------------------------------------------------
| IBitmapControl::setBitmap                                                    |
------------------------------------------------------------------------------*/
IBitmapControl& IBitmapControl :: setBitmap(const IResourceId& bmpId)
{
  setBitmap(bmpId.resourceLibrary().loadBitmap(bmpId));
  return *this;
}


/*------------------------------------------------------------------------------
| IBitmapControl::setBitmap                                                    |
------------------------------------------------------------------------------*/
IBitmapControl& IBitmapControl :: setBitmap(unsigned long bmpId)
{

  setBitmap(IResourceId(bmpId));
  return *this;
}


/*------------------------------------------------------------------------------
| IBitmapControl::setBitmap                                                    |
------------------------------------------------------------------------------*/
IBitmapControl&  IBitmapControl :: setBitmap(const IBitmapHandle& bitmapHandle)
{
  bmpHandle = bitmapHandle;        // set instance variable
  setLayoutDistorted(IWindow::minimumSizeChanged,0);
  if (bSizeTo)
     sizeTo(calcMinimumSize());
  refresh();
  return *this;
}


/*------------------------------------------------------------------------------
| IBitmapControl::setBitmap                                                    |
------------------------------------------------------------------------------*/
IBitmapControl& IBitmapControl :: setBitmap(ISystemBitmapHandle::Identifier bmp)
{
  setBitmap(ISystemBitmapHandle(bmp));
  return *this;
}



/*------------------------------------------------------------------------------
| IBitmapControl::bitmap                                                       |
------------------------------------------------------------------------------*/
IBitmapHandle  IBitmapControl :: bitmap() const
{
  return bmpHandle;
}


/*------------------------------------------------------------------------------
| IBitmapControl::calcMinimumSize                                              |
|  Calculate the minimum screen size needed by the control                     |
|  which is based on the bitmap size.                                          |
------------------------------------------------------------------------------*/
ISize  IBitmapControl :: calcMinimumSize() const
{
  ISize minSize = Inherited::calcMinimumSize();

  if ( this->bitmap() )
  {
     BITMAPINFOHEADER   pbmpData;
     pbmpData.cbFix = sizeof( BITMAPINFOHEADER );
     pbmpData.cx = 0;
     pbmpData.cy = 0;
     // ignore return code since IIconControl calls this function and
     // it fails when its an icon.
     IQUERYBITMAPPARAMETERS( this->bitmap(), &pbmpData ) ;

     ISize bmpSize( pbmpData.cx, pbmpData.cy );
     minSize = minSize.maximum( bmpSize );
  }

  return minSize;
}


/*------------------------------------------------------------------------------
| IBitmapControl::initialize                                                   |
------------------------------------------------------------------------------*/
IBitmapControl&  IBitmapControl :: initialize(const Style& style)
{
  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

  unsigned long ulStyle = convertToGUIStyle( style );
  this -> addHandler( defaultHandler );
  if (style & IBitmapControl::sizeToBitmap)
     {
     bSizeTo = true;
     ulStyle &= ~SS_BITMAP;
     }

  if (ulStyle & IStaticText::fillBackground.asUnsignedLong())
     ulStyle &= (~IStaticText::fillBackground.asUnsignedLong());
  if (ulStyle & IWindow::visible.asUnsignedLong())
     ulStyle &= (~IWindow::visible.asUnsignedLong());

  setStyle(ulStyle);
  return( *this );
}


/*------------------------------------------------------------------------------
| IBitmapControl::setLayoutDistorted                                           |
------------------------------------------------------------------------------*/
IBitmapControl& IBitmapControl :: setLayoutDistorted 
                                  ( unsigned long layoutAttributesOn,
                                    unsigned long layoutAttributesOff )
{
  if (( layoutAttributesOn &  IWindow::colorChanged ) && this->parent())
  {

    IWindowClassName cname(this->parent()->handle());
    if ( cname == WC_BUTTON )
    {
       // We must be on a graphic push button, so propagate the color
       // change to the button 
       if ( this->backgroundColor() != parent()->backgroundColor() )
         parent()->setBackgroundColor(this->backgroundColor());
    }
  }
  if (( layoutAttributesOn &  IWindow::fontChanged ) && this->parent())
  {
    IWindowClassName cname(this->parent()->handle());
    if ( cname == WC_BUTTON )
    {
       // We must be on a graphic push button, so propagate the font
       // change to the button 
       IFont fnt(this);
       IFont fnt2(parent());
       if (( fnt.name() !=  fnt2.name() ) || 
           ( fnt.pointSize() != fnt2.pointSize() ))
       {
         parent()->setFont(fnt);
       }
    }
  }
  Inherited::setLayoutDistorted(layoutAttributesOn,layoutAttributesOff);
  return *this;
}
