/*******************************************************************************
* FILE NAME: ibmpdata.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IBitmapData                                                              *
*     IBitmapSet                                                               *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
{
  #include <iwindefs.h>
}

#include <istring.hpp>
#include <irect.hpp>
#include <ireslock.hpp>
#include <iresstat.hpp>
#include <ibmpdata.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBMPDATA_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IBitmapData::IBitmapData                                                     |
------------------------------------------------------------------------------*/
IBitmapData :: IBitmapData(unsigned long ulId,
                           unsigned long handle,
                           const IString& strFileName)
  : ulClId(ulId), ulClHandle(handle), strClFileName(strFileName),
    sizClBitmap(ISize(0,0)), ulClUseCount(0), chngd(false)
{}

/*------------------------------------------------------------------------------
| IBitmapData::IBitmapData                                                     |
------------------------------------------------------------------------------*/
IBitmapData :: IBitmapData(unsigned long ulId,
                           unsigned long handle,
                           const IString& strFileName,
                           const ISize& sizBitmap)
  : ulClId(ulId), ulClHandle(handle), strClFileName(strFileName),
    sizClBitmap(sizBitmap), ulClUseCount(0), chngd(false)
{}

/*------------------------------------------------------------------------------
| IBitmapData::~IBitmapData                                                    |
------------------------------------------------------------------------------*/
IBitmapData ::  ~IBitmapData() {}

/*------------------------------------------------------------------------------
| IBitmapSet::IBitmapSet                                                       |
------------------------------------------------------------------------------*/
IBitmapSet :: IBitmapSet()
       : ISet<IBitmapData*>() {}

/*------------------------------------------------------------------------------
| IBitmapSet::~IBitmapSet                                                      |
------------------------------------------------------------------------------*/
IBitmapSet :: ~IBitmapSet() {}


/*------------------------------------------------------------------------------
| IBitmapSet::find                                                             |
------------------------------------------------------------------------------*/
IBitmapData* IBitmapSet::find( const unsigned long ulBitmapId,
                               const IString& fileName,
                               const ISize& sizeBitmap )
{
  IBitmapData* pBmpReturn(0);
  IBitmapData* pBmpData;

     /* Lock Resource Table */
  IResourceLock reslibLock(IResLibStaticPtr::libraryKey());
  IBitmapSet::Cursor cursor(*this);
  for(cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
  {
     pBmpData = this->elementAt(cursor);
     if( (pBmpData->id() == ulBitmapId)               &&
         (pBmpData->resourceFileName() == fileName) &&
         (pBmpData->size() == sizeBitmap))
     {
       ITRACE_ALL("BMP Match found");
       pBmpReturn = pBmpData;
       break;
     }
  }
  return pBmpReturn;
}

/*------------------------------------------------------------------------------
| IBitmapSet::find                                                             |
------------------------------------------------------------------------------*/
IBitmapData* IBitmapSet::find( const IString& fileName )
{
  IBitmapData* pBmpReturn(0);
  IBitmapData* pBmpData;

     /* Lock Resource Table */
  IResourceLock reslibLock(IResLibStaticPtr::libraryKey());
  IBitmapSet::Cursor cursor(*this);
  for(cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
  {
     pBmpData = this->elementAt(cursor);
     if(pBmpData->resourceFileName() == fileName)
     {
       ITRACE_ALL("BMP Match found");
       pBmpReturn = pBmpData;
       break;
     }
  }
  return pBmpReturn;
}

/*------------------------------------------------------------------------------
| IBitmapSet::find                                                             |
------------------------------------------------------------------------------*/
IBitmapData* IBitmapSet::find( const IBitmapHandle& bmh )
{
  IBitmapData* pBmpReturn(0);
  IBitmapData* pBmpData;

     /* Lock Resource Table */
  IResourceLock reslibLock(IResLibStaticPtr::libraryKey());
  IBitmapSet::Cursor cursor(*this);
  for(cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
  {
     pBmpData = this->elementAt(cursor);
     if (pBmpData->handle() == bmh )
     {
       ITRACE_ALL("BMP Match found");
       pBmpReturn = pBmpData;
       break;
     }
  }
  return pBmpReturn;
}

/*------------------------------------------------------------------------------
| IBitmapSet::remove                                                           |
------------------------------------------------------------------------------*/
IBitmapSet& IBitmapSet::remove( IBitmapData* pBmpData )
{
     /* Lock Resource Table */
  IResourceLock reslibLock(IResLibStaticPtr::libraryKey());
  IBitmapSet::Cursor cursor(*this);
  for(cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
  {
    if (pBmpData == this->elementAt(cursor))
    {
      this->removeAt(cursor);
      break;
    }
  }
  return *this;
}
