/*******************************************************************************
* FILE NAME: ibutton.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ibutton.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
{
  #define INCL_WINBUTTONS
  #define INCL_WINSYS
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
}
#ifndef _IBASE_
  #include <ibase.hpp>
#endif
#ifndef _IBUTTON_
  #include <ibutton.hpp>
#endif
#ifndef _ICCONST_
  #include <icconst.h>
#endif
#ifndef _ICMNFUN_
  #include <icmnfun.hpp>
#endif
#ifndef _ICOLOR_
  #include <icolor.hpp>
#endif
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif
#ifndef _ISTRING_
  #include <istring.hpp>
#endif
#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBUTTON_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| Public button styles                                                         |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const IButton::Style
  IButton::noPointerFocus = BS_NOPOINTERFOCUS;
#pragma data_seg()

/*------------------------------------------------------------------------------
| IButton::IButton                                                             |
|                                                                              |
| Default constructor here for page tuning                                     |
------------------------------------------------------------------------------*/
IButton :: IButton ( )
                  : fButtonData(0)
{;}

/*------------------------------------------------------------------------------
| IButton::~IButton                                                            |
|                                                                              |
| Empty destructor here for page tuning                                        |
------------------------------------------------------------------------------*/
IButton :: ~IButton ( )
{
  // delete fButtonData;
}

/*********************************************************************/
/* No equivalent of BS_NOPOINTERFOCUS in Windows NT - One possible   */
/* solution is to keep this style internally, intercept WM_SETFOCUS  */
/* messages to button, save previous focus handle, execute button    */
/* and then restore focus to previous window handle                  */
/*********************************************************************/
/*------------------------------------------------------------------------------
| IButton::enableMouseClickFocus                                               |
|                                                                              |
| Enables the button to receive the focus when the user clicks on the button   |
| using the mouse.                                                             |
------------------------------------------------------------------------------*/
IButton&  IButton :: enableMouseClickFocus ( Boolean bTurnOn )
{
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (bTurnOn)
  {
    ulStyle &= ~noPointerFocus.asUnsignedLong();
  }
  else
  {
    ulStyle |= noPointerFocus.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
  }
  return( *this );
}

/*------------------------------------------------------------------------------
| IButton::disableMouseClickFocus                                              |
------------------------------------------------------------------------------*/
IButton&  IButton :: disableMouseClickFocus ( )
{
  enableMouseClickFocus(false);
  return *this;
}


/*------------------------------------------------------------------------------
| IButton::allowsMouseClickFocus                                               |
------------------------------------------------------------------------------*/
Boolean  IButton :: allowsMouseClickFocus ( ) const
{
  if (style() & noPointerFocus.asUnsignedLong())
    return( false );
  else
    return( true );
}


/*------------------------------------------------------------------------------
| IButton::highlight                                                           |
|                                                                              |
| Highlight the button.  A highlighted button appears as if the mouse          |
| selection button were pressed with the mouse pointer over the button.        |
------------------------------------------------------------------------------*/
IButton&  IButton :: highlight ( Boolean highlight )
{
  handle().sendEvent(BM_SETHILITE, IEventParameter1(highlight),
                                   IEventParameter2(0));
  return *this;
}


/*------------------------------------------------------------------------------
| IButton::unhighlight                                                         |
------------------------------------------------------------------------------*/
IButton&  IButton :: unhighlight ( )
{
  return highlight(false);
}


/*------------------------------------------------------------------------------
| IButton::isHighlighted                                                       |
------------------------------------------------------------------------------*/
Boolean  IButton :: isHighlighted ( ) const
{
  IEventResult evt = handle().sendEvent(BM_QUERYHILITE,
                                        IEventParameter1(0),
                                        IEventParameter2(0));
  if (evt.asUnsignedLong())
     return true;
  else
     return false;
}


/*------------------------------------------------------------------------------
| IButton::click                                                               |
------------------------------------------------------------------------------*/
IButton&  IButton :: click ( )
{
  ICLICKBUTTON(this->handle());
  return *this;
}

/*------------------------------------------------------------------------------
| IButton::foregroundColor                                                     |
|                                                                              |
| Returns the foreground color of the button.                                  |
------------------------------------------------------------------------------*/
IColor IButton::foregroundColor () const
{
  return (IWindow::color(PP_FOREGROUNDCOLOR,
                         IGUIColor(IGUIColor::menuText)));
}

/*------------------------------------------------------------------------------
| IButton::backgroundColor                                                     |
|                                                                              |
| Returns the background color of the button.                                  |
------------------------------------------------------------------------------*/
IColor IButton::backgroundColor () const
{
  //Return the correct background for a pushbutton.
  if (style() & BS_PRIMARYSTYLES)
  {
    return (IWindow::color(PP_BACKGROUNDCOLOR,
                           defaultbackgroundcolor(handle())));
  }
  return (IWindow::color(PP_BACKGROUNDCOLOR,
                         IGUIColor(IGUIColor::buttonMiddle)));
}

/*------------------------------------------------------------------------------
| IButton::disabledForegroundColor                                             |
|                                                                              |
| Returns the disabled foreground color of the button.                         |
------------------------------------------------------------------------------*/
IColor IButton::disabledForegroundColor () const
{
  return (IWindow::color(PP_DISABLEDFOREGROUNDCOLOR,
                         IGUIColor(IGUIColor::menuText)));
}

/*------------------------------------------------------------------------------
| IButton::hiliteForegroundColor                                               |
|                                                                              |
| Returns the hilite foreground color of the button.                           |
------------------------------------------------------------------------------*/
IColor IButton::hiliteForegroundColor () const
{
  return (IWindow::color(PP_HILITEFOREGROUNDCOLOR,
                         IGUIColor(IGUIColor::menuText)));
}

/*------------------------------------------------------------------------------
| IButton::hiliteBackgroundColor                                               |
|                                                                              |
| Returns the hilite background color of the button.                           |
------------------------------------------------------------------------------*/
IColor IButton::hiliteBackgroundColor () const
{
  return (IWindow::color(PP_HILITEBACKGROUNDCOLOR,
                         IGUIColor(IGUIColor::menuText)));
}

/*------------------------------------------------------------------------------
| IButton::setText                                                             |
|                                                                              |
| Sets the text used by the button, and notifies the parent canvas             |
------------------------------------------------------------------------------*/
IButton &  IButton :: setText ( const char* pszText )
{
  Inherited::setText(pszText);
  setLayoutDistorted(IWindow::minimumSizeChanged, 0);
  return *this;
}


/*------------------------------------------------------------------------------
| IButton::setText                                                             |
|                                                                              |
| Sets the button text from a string table.                                    |
------------------------------------------------------------------------------*/
IButton &  IButton :: setText ( const IResourceId & residText )
{
  Inherited::setText(residText);
  return *this;
}
