/*******************************************************************************
* FILE NAME: icmdevt.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icmdevt.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINMESSAGEMGR        // CMDSRC_xxx
extern "C" {
   #include <iwindefs.h>
}

#ifndef _ICMDEVT_
  #include <icmdevt.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICMDEVT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| ICommandEvent::~ICommandEvent                                                |
|                                                                              |
| Empty destructor here for page tuning                                        |
------------------------------------------------------------------------------*/
ICommandEvent::~ICommandEvent ( ) {}

/*------------------------------------------------------------------------------
| ICommandEvent::ICommandEvent                                                 |
|                                                                              |
| Construct ICommandEvent from an IEvent instance                              |
------------------------------------------------------------------------------*/
ICommandEvent::ICommandEvent ( const IEvent& event )
  : IEvent(event) { }

/*------------------------------------------------------------------------------
| ICommandEvent::source                                                        |
|                                                                              |
| Returns an enumerated value that maps to a CMDSRC_xxx value given by PM.     |
| This value identifies the source of the command message.                     |
------------------------------------------------------------------------------*/
ICommandEvent::Source  ICommandEvent::source ( ) const
{
  unsigned long ulSource = parameter2().number1();
  ulSource &= ~0x8000;  // take out special frame-client routing flag
  switch (ulSource)
  {
     case CMDSRC_PUSHBUTTON:
       return pushButton;
     case CMDSRC_MENU:
       return menu;
     case CMDSRC_ACCELERATOR:
       return accelerator;
     default:              //   CMDSRC_FONTDLG:
       return unknown;     //   CMDSRC_FILEDLG:
                           //   CMDSRC_PRINTDLG:
                           //   CMDSRC_COLORDLG:
                           //   CMDSRC_OTHER:
  } /* endswitch */
}
