/*******************************************************************************
* FILE NAME: icmnfun.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icmnfun.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINFRAMEMGR               // WM_QUERYFOCUSCHAIN.
#define INCL_WINWINDOWMGR              // QWS_FLAGS.
extern "C" {
  #include <iwindefs.h>
}

#include <icmnfun.hpp>
#include <icolor.hpp>
#include <ievtdata.hpp>
#include <ihandle.hpp>
#include <itrace.hpp>
#include <istring.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ICMNFUN_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| defaultbackgroundcolor                                                       |
|                                                                              |
| Determine the background color to use for the specified window,              |
| assuming that none has explicitly been assigned (choose between the          |
| system window background and system dialog background colors).               |
------------------------------------------------------------------------------*/
IColor defaultbackgroundcolor ( IWindowHandle hwndmain )
{
  IColor clrBackground = IGUIColor( IGUIColor::windowBgnd );
                                  // Default to system window background.

  /******************************************************************/
  /* The default background color will be based on the type of      */
  /* frame window we are sitting on (dialog background if the frame */
  /* is supposed to look like a dialog box, window background       */
  /* otherwise).                                                    */
  /******************************************************************/
  // Find the frame window.
  IEventResult mr = hwndmain.sendEvent( WM_QUERYFOCUSCHAIN,
                                        IEventParameter1( QFC_FRAME ),
                                        IEventParameter2( 0 ));
  IWindowHandle frame( mr.asUnsignedLong() );

  if ( frame  &&
       (IQUERYWINDOWUSHORT( frame, QWS_FLAGS ) & 0x100) )
  {           // Frame looks like a dialog box.
     clrBackground = IGUIColor( IGUIColor::dialogBgnd );
  }

  return clrBackground;
}

