/*******************************************************************************
* FILE NAME: icnrmhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icnrmhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINSTDCNR
#define INCL_WINWINDOWMGR
#define INCL_WININPUT
#define INCL_WINFRAMEMGR          // WM_FOCUSCHANGE, SC_CLOSE
#define INCL_WINMESSAGEMGR

extern "C" {
  #include <stdlib.h>
  #include <icconst.h>
  }

#include <icnrrec.hpp>  // Must be first for OS flags


#include <icnrmhdr.hpp>
#include <icmdevt.hpp>
#include <icnrctl.hpp>
#include <ipoint.hpp>
#include <itrace.hpp>
#include <ithread.hpp>
#include <iexcept.hpp>
#include <imenuevt.hpp>
#include <ictlevt.hpp>
#include <iexcept.hpp>
#include <ipopmenu.hpp>
#include <iwcname.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNRMHDR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| ICnrMenuHandler::~ICnrMenuHandler                                            |
------------------------------------------------------------------------------*/
ICnrMenuHandler :: ~ICnrMenuHandler      ( ) {}

/*------------------------------------------------------------------------------
| ICnrMenuHandler::ICnrMenuHandler() : IMenuHandler                            |
------------------------------------------------------------------------------*/
ICnrMenuHandler::ICnrMenuHandler() : IMenuHandler(),
                                       pcnrobjClMenu(0)
{ }

/*------------------------------------------------------------------------------
| ICnrMenuHandler::dispatchHandlerEvent                                        |
|                                                                              |
| Routes the edit events to the appropriate event functions.                   |
------------------------------------------------------------------------------*/
IBase::Boolean ICnrMenuHandler::dispatchHandlerEvent(IEvent& evt)
{
   switch (evt.eventId())
   {
     case WM_CONTROL:
     {
        IControlEvent ctlevt(evt);
        IContainerControl* pcnrctl =
                                (IContainerControl*)ctlevt.controlWindow();
        if(pcnrctl)
        {
          if ( pcnrctl->handle() )
          {
            IWindowClassName className(pcnrctl->handle());
            unsigned long ulMsg = evt.parameter1().number2();

            if (className == WC_CONTAINER)
            {                        // container
              switch(ulMsg)
              {
                case CN_CONTEXTMENU:
                {
                   // We need to simulate an IMenuEvent that would occur from
                   // a WM_CONTEXTMENU. Get the mouse position in
                   // container window coordinates.  Since the container
                   // does not tell us if this event is from a keyboard
                   // or a mouse, we assume the mouse (to get them from the
                   // keyboard requires an accelorator and the IMenuEvent does
                   // currently support this anyway). Note that we change
                   // the handle in the event from the owner of the container
                   // to the container itself so that the function window()
                   // returns the container as expected.
                   POINTL ptl;
                   POINTS pts;
                   IQUERYMSGPOS(IThread::current().anchorBlock(), &ptl);
                   IPoint pt(IWindow::mapPoint( ptl, HWND_DESKTOP, pcnrctl->handle()));
                   pts.x = (short)(pt.x());
                   pts.y = (short)(pt.y());
                   IEvent evtNew(pcnrctl, WM_CONTEXTMENU, &pts, true);
                   IMenuEvent mnEvt(evtNew);
                   if((void*)evt.parameter2())
                      pcnrobjClMenu =
                           IObjFromRec((IMiniCnrRecord*)(unsigned long)
                                                          (evt.parameter2()));
                   else
                      pcnrobjClMenu = 0;    //Menu meant for complete cnr.

                  if (makePopUpMenu(mnEvt)) {
                     addSourceEmphasis(mnEvt);
                     evt.setResult(true);   //Tell PM action taken.
                     return true;
                  } /* endif */
                }
              } /* end switch ulMsg */
            } /* endif  ulClassName == WC_CONTAINER */
          } /* endif handle valid */
        } /* pcnrctl != 0 */
     } /* endif WM_CONTROL */
   case WM_CONTEXTMENU:
   {
      return false;
   } /* endifelse */
   case WM_COMMAND:
   case WM_SYSCOMMAND:
   {                           // These aren't normally sent to owner
      /**********************************************************/
      /* Must ensure that WM_COMMAND and WM_SYSCOMMAND are not  */
      /* sent to the owner window if this cnr is the client     */
      /* area, in the cases where doing so would cause an       */
      /* endless loop between the frame and cnr handlers.       */
      /* The underlying problem is that the frame handler sends */
      /* WM_COMMAND and WM_SYSCOMMAND/SC_CLOSE events, and PM's */
      /* default frame procedure sends WM_COMMAND messages back */
      /* to the client area (FID_CLIENT), which the cnr         */
      /* would then send back to the owner frame, ad infinitum. */
      /* To remedy this problem, the cnr handler sets a bit     */
      /* reserved by the class library in mp2 of the event      */
      /* before sending it on to the owner frame, but doen't    */
      /* send on such events if the bit is already set (the bit */
      /* is used to signal whether the event has already been   */
      /* routed to the frame, and doesn't need to be sent again */
      /* since it would just come back from the frame).         */
      /**********************************************************/
      IContainerControl* pcnrctl = (IContainerControl*)evt.window();
      const IWindow* pwndOwner = pcnrctl->owner();
      ICommandEvent cmdevt(evt);
      Boolean bClient = (pcnrctl->id() == FID_CLIENT  &&
                         pwndOwner  &&
                         pwndOwner->isFrameWindow());
      if (!bClient  ||  cmdevt.isFromFrame() == false)
      {       // cnr is not client or event never sent to frame
         IWindowHandle whOwner = pwndOwner->handle();

         if (whOwner != IWindow::desktopWindow()->handle()  &&
             whOwner != IWindow::objectWindow()->handle())
         {                     // Valid owner window
            unsigned long ul2 = (unsigned long)evt.parameter2();
            if (bClient)
            {                  // Canvas is client window
               ul2 |= 0x8000;  // Set reserved bit in source value
            }
            evt.setResult(pwndOwner->handle().
                                sendEvent(evt.eventId(),
                                          evt.parameter1(),
                                          IEventParameter2(ul2)));
                               // Forward command event to owner window
            return true;
         }
      } /* endif */
      break;
   } /* endcase */
 } /* end switch */
   return Inherited::dispatchHandlerEvent(evt);
}


/*------------------------------------------------------------------------------
| ICnrMenuHandler::menuEnded                                                   |
|                                                                              |
| Reset the object pointer and call our parent to process menu end.            |
------------------------------------------------------------------------------*/
IBase::Boolean ICnrMenuHandler::menuEnded( IMenuEvent& menuEvent )
{
  //Instead of calling Inherited::menuEnded(), process here instead.
  //Do this so we can properly reset pcnrobjClMenu only when the main
  //popup menu is ending(not when the cascading sub menus go away).
  IWindow* popUpWindow =
           IWindow::windowWithHandle(IWindowHandle(menuEvent.parameter2()));

  //Only the main popup menu will satisfy this condition.
  if (popUpWindow && (popUpWindow->id() != FID_MENU))
  {
    removeSourceEmphasis(menuEvent);
    pcnrobjClMenu = 0;
  } /* endif */
  return false;
}

/*------------------------------------------------------------------------------
| ICnrMenuHandler::popupMenuObject                                             |
|                                                                              |
| Returns the object for which a pop-up menu was requested.                    |
------------------------------------------------------------------------------*/
IContainerObject* ICnrMenuHandler::popupMenuObject()
{
   return pcnrobjClMenu;
}

/*------------------------------------------------------------------------------
| ICnrMenuHandler::addSourceEmphasis                                           |
|                                                                              |
| If over an object, add source emphasis on the object, otherwise add source   |
| emphasis on the container.                                                   |
------------------------------------------------------------------------------*/
void ICnrMenuHandler::addSourceEmphasis(const IMenuEvent& menuEvent )
{
  IContainerControl* pcnrctl = (IContainerControl*)menuEvent.window();
  if(pcnrobjClMenu)
  {
     if(pcnrctl->isSelected(pcnrobjClMenu))
     {
        IContainerControl::ObjectCursor cursor(*pcnrctl,
                                           IContainerObject::selected);
        for(cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
        {
          pcnrctl->showSourceEmphasis(pcnrctl->objectAt(cursor));
        }
     }
     else
       pcnrctl->showSourceEmphasis(pcnrobjClMenu);
  }
  else
     pcnrctl->showSourceEmphasis();
}



/*------------------------------------------------------------------------------
| ICnrMenuHandler::removeSourceEmphasis                                        |
|                                                                              |
| If over an object, remove source emphasis on the object, otherwise remove    |
| source emphasis on the container.                                            |
------------------------------------------------------------------------------*/
void ICnrMenuHandler::removeSourceEmphasis(const IMenuEvent& menuEvent )
{
  IContainerControl* pcnrctl = (IContainerControl*)menuEvent.window();
  IContainerObject* pcnrobj;
  if(pcnrobjClMenu)
  {
     if(pcnrctl->isSelected(pcnrobjClMenu))
     {
        /************************************************************/
        /* Need to remove source emphasis on all selected objects.  */
        /* More than 1 object may be selected, so turn of refresh   */
        /* so that it doesn't flash every time we remove the        */
        /* source emphasis on each selected object.                 */
        /************************************************************/
        if (pcnrctl->isRefreshOn())
        {
          pcnrctl->setRefreshOff();
          IContainerControl::ObjectCursor cursor(*pcnrctl,
                                                IContainerObject::selected);
          for(cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
          {
            pcnrctl->hideSourceEmphasis(pcnrctl->objectAt(cursor));
          }
          pcnrctl->setRefreshOn();
          pcnrctl->refresh();
        }
        else
        {
          IContainerControl::ObjectCursor cursor(*pcnrctl,
                                                IContainerObject::selected);
          for(cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
          {
            pcnrctl->hideSourceEmphasis(pcnrctl->objectAt(cursor));
          }
        }
     }
     else
       /***************************************************************/
       /* Only need to remove source emphasis for the cursored object */
       /***************************************************************/
       pcnrctl->hideSourceEmphasis(pcnrobjClMenu);
  }
  else
     /***************************************************************/
     /* Remove source emphasis on the whole container.              */
     /***************************************************************/
     pcnrctl->hideSourceEmphasis();
}


/*------------------------------------------------------------------------------
| ICnrMenuHandler::handleEventsFor                                             |
|                                                                              |
| Attaches the handler to the container control passed in the argument.        |
------------------------------------------------------------------------------*/
ICnrMenuHandler& ICnrMenuHandler::handleEventsFor( IContainerControl *container)
{
  IASSERTPARM(container != 0);
  IHandler::handleEventsFor(container);
  return *this;
}


/*------------------------------------------------------------------------------
| ICnrMenuHandler::stopHandlingEventsFor                                       |
|                                                                              |
| Detaches the handler from the container control passed in the argument.      |
------------------------------------------------------------------------------*/
ICnrMenuHandler& ICnrMenuHandler::stopHandlingEventsFor(
                                                 IContainerControl *container )
{
  IASSERTPARM(container != 0);
  IHandler::stopHandlingEventsFor(container);
  return *this;
}

IHandler& ICnrMenuHandler :: handleEventsFor( IWindow *window )
{
    ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR,
                         IErrorInfo::invalidRequest,
                         IException::recoverable);
    return *this;
}

IHandler& ICnrMenuHandler :: stopHandlingEventsFor( IWindow *window )
{
    ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR,
                         IErrorInfo::invalidRequest,
                         IException::recoverable);
    return *this;
}
