/*******************************************************************************
* FILE NAME: icombobx.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icombobx.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

/**********************************************************************/
/* Need IC_USE_CB defined before iwindefs since Windows macros are    */
/* different if processing listboxes that are part of comboboxes      */
/**********************************************************************/
extern "C" {
  #define IC_USE_CB
  #define INCL_WINENTRYFIELDS       // ES_xxx, CBS_xxx
  #define INCL_WINERRORS            // PMERR_HEAP_MAX_SIZE_REACHED
  #define INCL_WINLISTBOXES         // LS_HORZSCROLL
  #define INCL_NLS                  // ES_ANY, ES_SBCS, ES_DBCS, ES_MIXED
  #define INCL_WINWINDOWMGR         // WinQueryClassName
  #define INCL_WINSYS               // SV_CYBORDER
  #define INCL_GPICONTROL           // From ILISTBX2.HPP
  #define INCL_GPILCIDS             //
  #define INCL_GPILOGCOLORTABLE     //
  #define INCL_GPIPRIMITIVES        //

  #include <iwindefs.h>
  #include <stdlib.h>            // atoi
  #include <string.h>
}

#include <inotifev.hpp>
#include <icombobx.hpp>

#include <iapp.hpp>
#include <icconst.h>
#include <icoordsy.hpp>
#include <iexcept.hpp>
#include <ifont.hpp>
#include <ihandle.hpp>
#include <ilistbox.hpp>
#include <imphdr.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICOMBOBX_CPP_
  #include <ipagetun.h>
#endif

/******************************************************************************/
/* Public entry field styles.                                                 */
/******************************************************************************/
#pragma data_seg(ICLStaticConst)
const IComboBox::Style
  IComboBox::classDefaultStyle ( (WS_VISIBLE | CBS_SIMPLE), IES_ANYDATA );
#pragma data_seg()

/******************************************************************************/
/* Default style for new objects (initial value)                              */
/******************************************************************************/
#pragma data_seg(ICLNonConst)
IComboBox::Style
  IComboBox::currentDefaultStyle ( (WS_VISIBLE | CBS_SIMPLE), IES_ANYDATA );
#pragma data_seg()


/*------------------------------------------------------------------------------
| IComboBox::IComboBox                                                         |
|                                                                              |
| Construct an IComboBox on an IWindow.                                        |
------------------------------------------------------------------------------*/
IComboBox::IComboBox( unsigned long id,
                      IWindow* pParent,
                      IWindow* pOwner,
                      const IRectangle& rectInit,
                      const Style& style) :
                      fComboBoxData( 0 ),
                      IBaseComboBox( id,
                                     pParent,
                                     pOwner,
                                     rectInit,
                                     (const IBaseComboBox::Style&) style )
{
}

/*------------------------------------------------------------------------------
| IComboBox::IComboBox                                                         |
|                                                                              |
| Construct an IComboBox on a dialog.                                          |
------------------------------------------------------------------------------*/
IComboBox::IComboBox( unsigned long id, IWindow* parent) :
                      fComboBoxData( 0 ),
                      IBaseComboBox( id, parent )
{
}

/*------------------------------------------------------------------------------
| IComboBox::IComboBox                                                         |
|                                                                              |
| Construct an IComboBox from an existing IComboBox.                           |
------------------------------------------------------------------------------*/
IComboBox::IComboBox( const IWindowHandle& handle) :
                      fComboBoxData( 0 ),
                      IBaseComboBox( handle )
{
}

/*------------------------------------------------------------------------------
| IComboBox::~IComboBox                                                        |
|                                                                              |
| Reomve the default mouse pointer handler.                                    |
------------------------------------------------------------------------------*/
IComboBox :: ~IComboBox()
{
  IMousePointerHandler::defaultHandler()->stopHandlingEventsFor(this);
  // delete fComboBoxData;
}

/*------------------------------------------------------------------------------
| IComboBox::defaultStyle                                                      |
------------------------------------------------------------------------------*/
IComboBox::Style  IComboBox::defaultStyle ( )
{
  return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IComboBox::setDefaultStyle                                                   |
------------------------------------------------------------------------------*/
void IComboBox::setDefaultStyle ( const Style& style )
{
  currentDefaultStyle = style;
}

#include <ilistbx4.hpp>

