/*******************************************************************************
* FILE NAME: icontrol.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icontrol.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
{
  #include <iwindefs.h>
}

#ifndef _ICONTROL_
  #include <icontrol.hpp>
#endif
#ifndef _IFONT_
  #include <ifont.hpp>
#endif
#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICONTROL_CPP_
  #include <ipagetun.h>
#endif

/******************************************************************************/
/* Public control styles.                                                     */
/******************************************************************************/
#pragma data_seg(ICLStaticConst)
const IControl::Style
  IControl::group   = WS_GROUP,
  IControl::tabStop = WS_TABSTOP;
#pragma data_seg()

/*------------------------------------------------------------------------------
| IControl::IControl                                                           |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IControl::IControl ( )
{;}

/*------------------------------------------------------------------------------
| IControl::~IControl                                                          |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IControl::~IControl ( )
{;}

/*------------------------------------------------------------------------------
| IControl::enableGroup                                                        |
------------------------------------------------------------------------------*/
IControl&  IControl::enableGroup ( Boolean enable )
{
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (enable)
  {
    ulStyle |= group.asUnsignedLong();
  }
  else
  {
    ulStyle &= ~group.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IControl::disableGroup                                                       |
------------------------------------------------------------------------------*/
IControl&  IControl::disableGroup ( )
{
  enableGroup(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IControl::isGroup                                                            |
------------------------------------------------------------------------------*/
Boolean  IControl::isGroup ( ) const
{
  return Inherited::isGroup();
}

/*------------------------------------------------------------------------------
| IControl::enableTabStop                                                      |
------------------------------------------------------------------------------*/
IControl&  IControl::enableTabStop ( Boolean enable )
{
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (enable)
  {
    ulStyle |= tabStop.asUnsignedLong();
  }
  else
  {
    ulStyle &= ~tabStop.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IControl::disableTabStop                                                     |
------------------------------------------------------------------------------*/
IControl&  IControl::disableTabStop ( )
{
  enableTabStop(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IControl::isTabStop                                                          |
------------------------------------------------------------------------------*/
Boolean  IControl::isTabStop ( ) const
{
  return Inherited::isTabStop();
}
