/*******************************************************************************
* FILE NAME: icustbev.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icustbev.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINBUTTONS
  #include <iwindefs.h>
}

#include <icustbev.hpp>
#include <icustbut.hpp>
#include <ictlevt.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICUSTBEV_CPP_
  #include <ipagetun.h>
#endif

ICustomButtonDrawEvent :: ICustomButtonDrawEvent ( const IEvent& event )
                        : IControlEvent ( event ),
                          fGraphicContext ( controlWindow()->handle() )
{
  fRectangle = controlWindow()->size();
}

ICustomButtonDrawEvent :: ICustomButtonDrawEvent
                          ( const ICustomButtonDrawEvent& event )
                        : IControlEvent ( event ),
                          fGraphicContext ( controlWindow()->handle() )
{
  fRectangle = controlWindow()->size();
}

ICustomButtonDrawEvent :: ~ICustomButtonDrawEvent ()
{
}

IGraphicContext& ICustomButtonDrawEvent :: graphicContext () const
{
  return (((ICustomButtonDrawEvent*)this)->fGraphicContext);
}

ICustomButtonDrawEvent::ButtonState ICustomButtonDrawEvent :: buttonState () const
{
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_HILITED )
    return buttonDown;
  if ( customButton()->isLatched() )
    return buttonLatched;
  return buttonUp;
}

Boolean ICustomButtonDrawEvent :: drawUp () const
{
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_HILITED )
    return false;
  if ( customButton()->isLatched() )
    return false;
  return true;
}

Boolean ICustomButtonDrawEvent :: drawDown () const
{
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_HILITED )
    return true;
  else
    return false;
}

Boolean ICustomButtonDrawEvent :: drawLatched () const
{
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_HILITED )
    return false;
  if ( customButton()->isLatched() )
    return true;
  return false;
}

Boolean ICustomButtonDrawEvent :: isButtonEnabled () const
{
  USERBUTTON *pUser = (USERBUTTON *)parameter2().asUnsignedLong();
  if ( pUser->fsState & BDS_DISABLED )
    return false;
  return true;
}

ICustomButton* ICustomButtonDrawEvent :: customButton () const
{
  return ((ICustomButton*)controlWindow());
}

IRectangle ICustomButtonDrawEvent :: drawingArea () const
{
  return fRectangle;
}

ICustomButtonDrawEvent& ICustomButtonDrawEvent :: setDrawingArea
                                                  ( const IRectangle& rectangle )
{
  fRectangle = rectangle;
  return *this;
}

