/*******************************************************************************
* FILE NAME: icustbhd.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icustbhd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINMESSAGEMGR
  #define INCL_WINBUTTONS
  #define INCL_WININPUT
  #define INCL_WINSYS
  #include <iwindefs.h>
}

#include <icustbut.hpp>
#include <icustbhd.hpp>
#include <icustbev.hpp>
#include <icolor.hpp>
#include <igrafctx.hpp>
#include <igrect.hpp>
#include <igstring.hpp>
#include <ifont.hpp>
#include <istring.hpp>
#include <igbundle.hpp>
#include <igregion.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICUSTBHD_CPP_
  #include <ipagetun.h>
#endif

ICustomButtonDrawHandler :: ICustomButtonDrawHandler ()
{
}

ICustomButtonDrawHandler :: ~ICustomButtonDrawHandler ()
{
}

Boolean ICustomButtonDrawHandler :: dispatchHandlerEvent ( IEvent &event )
{
  Boolean rc = false;

  switch ( event.eventId() )
  {
    // Check for paint message
    case WM_CONTROL:
         if ( event.parameter1().number2() == BN_PAINT )
         {
           ICustomButtonDrawEvent evt(event);
           if ( evt.drawDown() )
             evt.customButton()->sendEvent(IC_UM_DRAW_DOWN, true);
           else
             evt.customButton()->sendEvent(IC_UM_DRAW_DOWN, false);
           if ( rc = drawButton(evt) )
           {
             event.setResult(evt.result());
           }
         }
         break;
  }
  return rc;
}

Boolean ICustomButtonDrawHandler :: drawButton ( ICustomButtonDrawEvent& event )
{
  drawBorder ( event );
  drawBackground ( event );
  drawForeground ( event );
  if ( !event.isButtonEnabled() )
  {
    drawDisabledEmphasis ( event );
  }
  return true;
}

void ICustomButtonDrawHandler :: drawBorder ( ICustomButtonDrawEvent& event,
                                              ISize margin )
{
  // Get needed colors and sizes

  IColor  parentBgnd      = event.customButton()->parent()->backgroundColor();
  IColor  borderColor     = IGUIColor(IGUIColor::defaultButton);
  IColor  highlightDark   = IGUIColor(IGUIColor::buttonDark);
  IColor  highlightBright = IGUIColor(IGUIColor::buttonLight);

  unsigned long cxBorder = IQUERYSYSVALUE(SV_CXBORDER);
  unsigned long cyBorder = IQUERYSYSVALUE(SV_CYBORDER);

  IRectangle rect = event.drawingArea();

  // Get the current pen colors

  IColor oldPenColor = event.graphicContext().penColor();
  IColor oldFillColor = event.graphicContext().fillColor();

  // Clear the outermost border area to the parent background color
  // (This includes the margin and notched border area)

  unsigned long limit;
  unsigned long loop;
  event.graphicContext().setPenColor(parentBgnd)
                        .setDrawOperation(IGraphicBundle::frame);
  limit = cxBorder + margin.width();
  if (limit < (cyBorder + margin.height()) )
    limit = cyBorder + margin.height();
  for ( loop = 0; loop < limit; loop++ )
  {
     event.graphicContext().draw( IGRectangle ( IRectangle ( rect.left() + loop,
                                  rect.bottom() + loop,
                                  rect.right() - loop - 1,
                                  rect.top() - loop - 1 )));
  }
  rect.shrinkBy ( margin );

  // Draw a notched border for the button

  event.graphicContext().setPenColor(borderColor)
                        .setFillColor(borderColor)
                        .setDrawOperation(IGraphicBundle::fillAndFrame);

  event.graphicContext().draw( IGRectangle ( IRectangle ( rect.left() + cxBorder,
                               rect.top() - cyBorder,
                               rect.right() - cxBorder - 1,
                               rect.top() - 1 )));
  event.graphicContext().draw( IGRectangle ( IRectangle ( rect.left() + cxBorder,
                               rect.bottom(),
                               rect.right() - cxBorder - 1,
                               rect.bottom() + cyBorder - 1)));
  event.graphicContext().draw( IGRectangle ( IRectangle ( rect.left(),
                               rect.bottom() + cyBorder,
                               rect.left() + cxBorder - 1,
                               rect.top() - cyBorder - 1 )));
  event.graphicContext().draw( IGRectangle ( IRectangle ( rect.right() - cxBorder,
                               rect.bottom() + cyBorder,
                               rect.right() - 1,
                               rect.top() - cyBorder - 1)));
  rect.shrinkBy ( IPair ( cxBorder, cyBorder ));

  // Draw the 3-D borders

  if ( event.drawUp() )
  {
     // Draw bright highlight on left and top

     event.graphicContext().setPenColor(highlightBright);
     event.graphicContext().setFillColor(highlightBright);
     event.graphicContext().draw( IGRectangle ( IRectangle ( rect.left(),
                                  rect.top() - cyBorder,
                                  rect.right() - cxBorder - 1,
                                  rect.top() - 1 )));
     event.graphicContext().draw( IGRectangle ( IRectangle ( rect.left(),
                                  rect.bottom() + cyBorder,
                                  rect.left() + cxBorder - 1,
                                  rect.top() - cyBorder - 1 )));

     // Draw dark highlight on right and bottom (double wide)

     event.graphicContext().setPenColor(highlightDark);
     event.graphicContext().setFillColor(highlightDark);
     event.graphicContext().draw( IGRectangle ( IRectangle ( rect.left(),
                                  rect.bottom(),
                                  rect.right() - 1,
                                  rect.bottom() + cyBorder - 1)));
     event.graphicContext().draw( IGRectangle ( IRectangle ( rect.left() + cxBorder,
                                  rect.bottom() + cyBorder,
                                  rect.right() - 1,
                                  rect.bottom() + cyBorder + cyBorder - 1)));
     event.graphicContext().draw( IGRectangle ( IRectangle ( rect.right() - cxBorder,
                                  rect.bottom() + cyBorder + cyBorder,
                                  rect.right() - 1,
                                  rect.top() - 1)));
     event.graphicContext().draw( IGRectangle ( IRectangle ( rect.right() - cxBorder
                                                                          - cxBorder,
                                  rect.bottom() + cyBorder + cyBorder,
                                  rect.right() - cxBorder - 1,
                                  rect.top() - cyBorder - 1)));

     rect = IRectangle ( rect.left() + cxBorder,
                         rect.bottom() + cyBorder + cyBorder,
                         rect.right() - cxBorder - cxBorder,
                         rect.top() - cyBorder );
  }
  else
  {
     // Draw dark highlight on left and top

     event.graphicContext().setPenColor(highlightDark);
     event.graphicContext().setFillColor(highlightDark);
     event.graphicContext().draw( IGRectangle ( IRectangle ( rect.left(),
                                  rect.top() - cyBorder,
                                  rect.right() -  1,
                                  rect.top() - 1 )));
     event.graphicContext().draw( IGRectangle ( IRectangle ( rect.left(),
                                  rect.bottom(),
                                  rect.left() + cxBorder - 1,
                                  rect.top() - cyBorder - 1 )));

     rect = IRectangle ( rect.left() + cxBorder,
                         rect.bottom(),
                         rect.right(),
                         rect.top() - cyBorder );
  }

  // Reset colors back to the original value

  event.graphicContext().setFillColor(oldFillColor);
  event.graphicContext().setPenColor(oldPenColor);

  // Save the new rectangle as the drawing area

  event.setDrawingArea ( rect );

}

void ICustomButtonDrawHandler :: drawBackground ( ICustomButtonDrawEvent& event )
{
  IRectangle rect = event.drawingArea();

  // Get the current pen color

  IColor oldPenColor = event.graphicContext().penColor();
  IColor oldFillColor = event.graphicContext().fillColor();

  // Get the background color

  IColor bgndColor(0,0,0);
  if ( event.drawLatched() )
    bgndColor = event.customButton()->latchedBackgroundColor();
  else
    bgndColor = event.customButton()->backgroundColor();

  // Fill the background

  event.graphicContext().setPenColor(bgndColor)
                        .setFillColor(bgndColor)
                        .setDrawOperation(IGraphicBundle::fillAndFrame);
  event.graphicContext().draw( IGRectangle ( IRectangle ( rect.left(),
                               rect.bottom(),
                               rect.right() - 1,
                               rect.top() - 1)));

  // Halftone latched backgound (if needed)

  if ( event.drawLatched() &&
       event.customButton()->isLatchedBackgroundColorHalftone() )
  {
    unsigned long           oldPattern = event.graphicContext().fillPattern();
    IGraphicBundle::PenType oldType    = event.graphicContext().penType();
    IColor                  oldBgndClr = event.graphicContext().backgroundColor();

    event.graphicContext().setFillPattern(IGraphicBundle::halftone);
    event.graphicContext().setPenType(IGraphicBundle::alternate);
    event.graphicContext().setPenColor(bgndColor);
    event.graphicContext().setFillColor(IColor(IColor::white));

    event.graphicContext().draw( IGRectangle ( IRectangle ( rect.left(),
                                 rect.bottom(),
                                 rect.right() - 1,
                                 rect.top() - 1)));

    event.graphicContext().setPenType(oldType);
    event.graphicContext().setFillPattern(oldPattern);
    event.graphicContext().setBackgroundColor(oldBgndClr);
  }

  // Reset colors back to the original value

  event.graphicContext().setFillColor(oldFillColor);
  event.graphicContext().setPenColor(oldPenColor);

}

void ICustomButtonDrawHandler :: drawForeground ( ICustomButtonDrawEvent& event )
{
  IRectangle rect = event.drawingArea();

  // Get the old clip region and create a new clip region based on
  // the drawing area rectangle (so we do not draw outside of the
  // foreground area)

  IGRegion startRegion(event.graphicContext().clipRegion());
  IGRegion clipRegion(event.graphicContext(),rect);

  event.graphicContext().setClipRegion(clipRegion);

  // Get the current pen color

  IColor oldPenColor(event.graphicContext().penColor());

  // Get the current font

  IFont  fnt(event.graphicContext().handle());

  // Get the current border size

  unsigned long cyBorder = IQUERYSYSVALUE(SV_CYBORDER);

  // Start drawing text on the top of the button

  IPoint textPos;
  textPos.setY ( rect.top() - cyBorder - fnt.maxAscender() );

  // If button is not "up", shift foregroung drawing down 1 pel

  if ( !event.drawUp() )
    textPos.setY ( textPos.y() - 1 );

  // Use the foreground color to draw text

  if ( event.drawLatched() )
    event.graphicContext().setPenColor(event.customButton()->latchedForegroundColor());
  else
    event.graphicContext().setPenColor(event.customButton()->foregroundColor());

  // Loop through text to draw each line

  unsigned long index = 0;
  unsigned long start = 0;

  IString txt(event.customButton()->text());
  do
  {
    // Get the text for the current line
    IString subtxt;
    start = index + 1;
    index = txt.indexOf('\n',start);
    if (index != 0)
      subtxt = txt.subString(start,index-start).strip();
    else
      subtxt = txt.subString(start).strip();

    // Get the text width
    unsigned long txtWidth;
    if ( IBidiSettings::isBidiSupported() )
    {
      IBidiSettings settings(*(event.customButton()));
      txtWidth = fnt.textWidth(subtxt,settings);
    }
    else
    {
      txtWidth = event.customButton()->displaySize(subtxt).width();
    }

    // Center text (or clip if text is too big)
    if ( rect.width() >= txtWidth )
    {
      textPos.setX ( rect.left() + (( rect.width() - txtWidth ) / 2 ));
    }
    else
    {
      textPos.setX ( rect.left() );
      if ( IBidiSettings::isBidiSupported() )
      {
        IBidiSettings settings(*(event.customButton()));
        if ( settings.windowLayout() == IBidiSettings::layoutRightToLeft )
        {
          textPos.setX ( rect.right() - txtWidth );
        }
      }
    }

    // Draw text
    IGString gs(subtxt,textPos,fnt);
    event.graphicContext().draw(gs);

    // Move position for next line of text (overlap one pel)
    textPos.setY ( textPos.y() - fnt.maxCharHeight() + 1 );
  }
  while (index != 0);

  // Reset colors back to the original value

  event.graphicContext().setPenColor(oldPenColor);

  // Reset clip region back to original value (if it existed)

  event.graphicContext().clearClipRegion();

  if ((IRegionHandle)startRegion)
    event.graphicContext().setClipRegion(startRegion);
}

void ICustomButtonDrawHandler :: drawDisabledEmphasis
                                 ( ICustomButtonDrawEvent& event )
{
    IRectangle      rect = event.drawingArea();

    unsigned long           oldPattern   = event.graphicContext().fillPattern();
    IGraphicBundle::PenType oldType      = event.graphicContext().penType();
    IColor                  oldPenColor  = event.graphicContext().penColor();
    IColor                  oldFillColor = event.graphicContext().fillColor();

    event.graphicContext().setFillPattern(IGraphicBundle::halftone)
                          .setPenType(IGraphicBundle::alternate)
                          .setPenColor(event.customButton()->backgroundColor())
                          .setFillColor(IColor(IColor::white))
                          .setDrawOperation(IGraphicBundle::fillAndFrame);

    event.graphicContext().draw( IGRectangle ( IRectangle ( rect.left(),
                                  rect.bottom(),
                                  rect.right() - 1,
                                  rect.top() - 1)));

    event.graphicContext().setFillColor(oldFillColor)
                          .setPenColor(oldPenColor)
                          .setPenType(oldType)
                          .setFillPattern(oldPattern);
}

