/*******************************************************************************
* FILE NAME: idmhndlr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMHandler                                                               *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  /* define what subsections of PM headers to include */
  #define INCL_WINWINDOWMGR     // General window management
  #include <os2.h>
}
#ifndef _IDMHNDLR_
  #include <idmhndlr.hpp>
#endif

#ifndef _IDMSRCH_
  #include <idmsrch.hpp>
#endif

#ifndef _IDMTGTH_
  #include <idmtgth.hpp>
#endif

#ifndef _IDMRENDR_
  #include <idmrendr.hpp>
#endif

#ifndef _IDMPROV_
  #include <idmprov.hpp>
#endif

#ifndef _IENTRYFD_
  #include <ientryfd.hpp>
#endif

#ifndef _IDMEFIT_
  #include <idmefit.hpp>
#endif

#ifndef _IMLE_
  #include <imle.hpp>
#endif

#ifndef _IDMMLEIT_
  #include <idmmleit.hpp>
#endif

#ifndef _ICNRCTL_
  #include <icnrctl.hpp>
#endif

#ifndef _IDMCNRIT_
  #include <idmcnrit.hpp>
#endif

#ifndef _IMENUBAR_
  #include <imenubar.hpp>
#endif

#ifndef _ISUBMENU_
  #include <isubmenu.hpp>
#endif

#ifndef _IDMMENIT_
  #include <idmmenit.hpp>
#endif

#ifndef _ITBAR_
  #include <itbar.hpp>
#endif

#ifndef _IDMTBBIT_
  #include <idmtbbit.hpp>
#endif

#ifndef _IDMTBRIT_
  #include <idmtbrit.hpp>
#endif

#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

#ifndef _IDMSEQ_
  #include <idmseq.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMHNDLR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IDMRendererSeq                                                               |
------------------------------------------------------------------------------*/
class IDMRendererSeq : public ISequence< IDMRenderer* > {
public:
  IDMRendererSeq ( );
 ~IDMRendererSeq ( );
};

/*------------------------------------------------------------------------------
| IDMRendererSeq::IDMRendererSeq                                               |
| Constructor                                                                  |
------------------------------------------------------------------------------*/
IDMRendererSeq :: IDMRendererSeq ( )
  : ISequence< IDMRenderer* >( 10 )
{
}

/*------------------------------------------------------------------------------
| IDMRendererSeq::~IDMRendererSeq                                              |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IDMRendererSeq :: ~IDMRendererSeq ( )
{
}

#pragma data_seg(ICLNonConst)

/*------------------------------------------------------------------------------
| Static objects to ensure default handlers get deleted at shutdown.           |
------------------------------------------------------------------------------*/
static struct IDMHandler_Static {
static IDMSourceHandler
 *pSrcHandler;
static IDMTargetHandler
 *pTgtHandler;
static IDMItemProvider
 *pEFProvider,
 *pMLEProvider,
 *pCnrProvider,
 *pSMenuProvider,
 *pTBarProvider,
 *pTBarBtnProvider;

  ~IDMHandler_Static ( )
    {
    IMODTRACE_DEVELOP( "IDMHandler_Static dtor" );
    delete pSrcHandler;
    delete pTgtHandler;
    delete pEFProvider;
    delete pMLEProvider;
    delete pCnrProvider;
    delete pSMenuProvider;
    delete pTBarProvider;
    delete pTBarBtnProvider;
    }

} statics;

IDMSourceHandler
 *IDMHandler_Static::pSrcHandler = 0;
IDMTargetHandler
 *IDMHandler_Static::pTgtHandler = 0;
IDMItemProvider
 *IDMHandler_Static::pEFProvider = 0,
 *IDMHandler_Static::pMLEProvider = 0,
 *IDMHandler_Static::pCnrProvider = 0,
 *IDMHandler_Static::pSMenuProvider = 0,
 *IDMHandler_Static::pTBarProvider = 0,
 *IDMHandler_Static::pTBarBtnProvider = 0;

// IDMHandler static members:
IDMSourceHandler
 *IDMHandler::pSource = 0;
IDMTargetHandler
 *IDMHandler::pTarget = 0;

#pragma data_seg()

/*------------------------------------------------------------------------------
| IDMHandler::IDMHandler                                                       |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
IDMHandler :: IDMHandler () : IHandler (),
                              pRendererSeqCl(0)
{
}

/*------------------------------------------------------------------------------
| IDMHandler::~IDMHandler                                                      |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IDMHandler :: ~IDMHandler ()
{
  delete this->pRendererSeqCl;
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragFrom                                                   |
|                                                                              |
| Add the default source handler to the argument window.                       |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragFrom ( IWindow* pWin )
{
  IDMHandler::defaultSourceHandler().handleEventsFor( pWin );
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragFrom                                                   |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragFrom ( IEntryField* pEF )
{
  IDMHandler::defaultSourceHandler().handleEventsFor( pEF );
  IDMHandler::setItemProvider( pEF );
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragFrom                                                   |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragFrom ( IMultiLineEdit* pMLE )
{
  IDMHandler::defaultSourceHandler().handleEventsFor( pMLE );
  IDMHandler::setItemProvider( pMLE );
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragFrom                                                   |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragFrom ( IContainerControl* pCnr )
{
  IDMHandler::defaultSourceHandler().handleEventsFor( pCnr );
  IDMHandler::setItemProvider( pCnr );
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragFrom                                                   |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragFrom ( IMenuBar* pMenuBar )
{
  /********************************************************************/
  /* Iterate menu items for corresponding submenus                    */
  /********************************************************************/
  IMenu::Cursor  cursor( *pMenuBar );
  for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
  {
    IWindowHandle hSubmenu = pMenuBar->elementAt( cursor ).submenuHandle();
    ITRACE_DEVELOP( "Hooking submenu->" + hSubmenu.asString() );
    IWindow *pSubmenu = new IWindow( hSubmenu );
    pSubmenu->setAutoDeleteObject();

    IDMHandler::enableDragFrom( (ISubmenu *)pSubmenu );
  }
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragFrom                                                   |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragFrom ( ISubmenu* pSubmenu )
{
  /********************************************************************/
  /* Enable this submenu as a drag source, as well as setting the     */
  /* item provider.                                                   */
  /********************************************************************/
  IDMHandler::defaultSourceHandler().handleEventsFor( pSubmenu );
  IDMHandler::setItemProvider( pSubmenu );
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDropOn                                                     |
|                                                                              |
| Add the default target handler to the argument window.                       |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDropOn ( IWindow* pWin )
{
  IDMHandler::defaultTargetHandler().handleEventsFor( pWin );
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDropOn                                                     |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDropOn ( IEntryField* pEF )
{
  IDMHandler::defaultTargetHandler().handleEventsFor( pEF );
  IDMHandler::setItemProvider( pEF );
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDropOn                                                     |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDropOn ( IMultiLineEdit* pMLE )
{
  IDMHandler::defaultTargetHandler().handleEventsFor( pMLE );
  IDMHandler::setItemProvider( pMLE );
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDropOn                                                     |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDropOn ( IContainerControl* pCnr )
{
  IDMHandler::defaultTargetHandler().handleEventsFor( pCnr );
  IDMHandler::setItemProvider( pCnr );
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDropOn                                                     |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDropOn ( IToolBar* pTBar )
{
  /********************************************************************/
  /* Enable this toolbar button as a drag target, as well as setting  */
  /* the item provider.                                               */
  /********************************************************************/
  IDMHandler::defaultTargetHandler().handleEventsFor( pTBar );
  IDMHandler::setItemProvider( pTBar );
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragDropFor                                                |
|                                                                              |
| Add the default source and target handlers to the argument window.           |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragDropFor ( IWindow* pWin )
{
  IDMHandler::defaultSourceHandler().handleEventsFor( pWin );
  IDMHandler::defaultTargetHandler().handleEventsFor( pWin );
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragDropFor                                                |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragDropFor ( IEntryField* pEF )
{
  IDMHandler::defaultSourceHandler().handleEventsFor( pEF );
  IDMHandler::defaultTargetHandler().handleEventsFor( pEF );
  IDMHandler::setItemProvider( pEF );
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragDropFor                                                |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragDropFor ( IMultiLineEdit* pMLE )
{
  IDMHandler::defaultSourceHandler().handleEventsFor( pMLE );
  IDMHandler::defaultTargetHandler().handleEventsFor( pMLE );
  IDMHandler::setItemProvider( pMLE );
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragDropFor                                                |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragDropFor ( IContainerControl* pCnr )
{
  IDMHandler::defaultSourceHandler().handleEventsFor( pCnr );
  IDMHandler::defaultTargetHandler().handleEventsFor( pCnr );
  IDMHandler::setItemProvider( pCnr );
}

/*------------------------------------------------------------------------------
| IDMHandler::enableDragDropFor                                                |
------------------------------------------------------------------------------*/
void IDMHandler :: enableDragDropFor ( IToolBarButton* pTBarBtn )
{
  /********************************************************************/
  /* Enable this toolbar button as a drag target and source, as well  */
  /* as setting the item provider.                                    */
  /********************************************************************/
  IDMHandler::defaultSourceHandler().handleEventsFor( pTBarBtn );
  IDMHandler::defaultTargetHandler().handleEventsFor( pTBarBtn );
  IDMHandler::setItemProvider( pTBarBtn );
}

/*------------------------------------------------------------------------------
| IDMHandler::defaultSourceHandler                                             |
|                                                                              |
| Return default source handler, initializing if this is the first time.       |
------------------------------------------------------------------------------*/
IDMSourceHandler& IDMHandler :: defaultSourceHandler ( )
{
  if (!IDMHandler::pSource)
  {
    if (!statics.pSrcHandler)
      statics.pSrcHandler = new IDMSourceHandler;

    IDMHandler::pSource = statics.pSrcHandler;
  }
  return( *IDMHandler::pSource );
}

/*------------------------------------------------------------------------------
| IDMHandler::defaultTargetHandler                                             |
|                                                                              |
| Return default target handler, initializing if this is the first time.       |
------------------------------------------------------------------------------*/
IDMTargetHandler& IDMHandler :: defaultTargetHandler ( )
{
  if (!IDMHandler::pTarget)
  {
    if (!statics.pTgtHandler)
      statics.pTgtHandler = new IDMTargetHandler;

    IDMHandler::pTarget = statics.pTgtHandler;
  }
  return( *IDMHandler::pTarget );
}

/*------------------------------------------------------------------------------
| IDMHandler::setDefaultSourceHandler                                          |
|                                                                              |
| Set default source handler pointer to point to argument handler.             |
------------------------------------------------------------------------------*/
void IDMHandler :: setDefaultSourceHandler ( IDMSourceHandler& source )
{
  IDMHandler::pSource = &source;
}

/*------------------------------------------------------------------------------
| IDMHandler::setDefaultTargetHandler                                          |
|                                                                              |
| Set default target handler pointer to point to argument handler.             |
------------------------------------------------------------------------------*/
void IDMHandler :: setDefaultTargetHandler ( IDMTargetHandler& target )
{
  IDMHandler::pTarget = &target;
}

/*------------------------------------------------------------------------------
| IDMHandler::setItemProvider                                                  |
|                                                                              |
| If the argument window doesn't already have an item provider, use            |
| appropriate default.                                                         |
------------------------------------------------------------------------------*/
void IDMHandler :: setItemProvider ( IEntryField* pEF )
{
  if (!pEF->itemProvider())
  {
    if (!statics.pEFProvider)
      statics.pEFProvider = new IDMItemProviderFor< IDMEFItem >;

    pEF->setItemProvider( statics.pEFProvider );
  }
}

/*------------------------------------------------------------------------------
| IDMHandler::setItemProvider                                                  |
------------------------------------------------------------------------------*/
void IDMHandler :: setItemProvider ( IMultiLineEdit* pMLE )
{
  if (!pMLE->itemProvider())
  {
    if (!statics.pMLEProvider)
      statics.pMLEProvider = new IDMItemProviderFor< IDMMLEItem >;

    pMLE->setItemProvider( statics.pMLEProvider );
  }
}

/*------------------------------------------------------------------------------
| IDMHandler::setItemProvider                                                  |
------------------------------------------------------------------------------*/
void IDMHandler :: setItemProvider ( IContainerControl* pCnr )
{
  if (!pCnr->itemProvider())
  {
    if (!statics.pCnrProvider)
      statics.pCnrProvider = new IDMItemProviderFor< IDMCnrItem >;

    pCnr->setItemProvider( statics.pCnrProvider );
  }
}

/*------------------------------------------------------------------------------
| IDMHandler::setItemProvider                                                  |
------------------------------------------------------------------------------*/
void IDMHandler :: setItemProvider ( ISubmenu* pSubMenu )
{
  if (!pSubMenu->itemProvider())
  {
    if (!statics.pSMenuProvider)
      statics.pSMenuProvider = new IDMItemProviderFor< IDMMenuItem >;

    pSubMenu->setItemProvider( statics.pSMenuProvider );
  }
}

/*------------------------------------------------------------------------------
| IDMHandler::setItemProvider                                                  |
------------------------------------------------------------------------------*/
void IDMHandler :: setItemProvider ( IToolBar* pTBar )
{
  if (!pTBar->itemProvider())
  {
    if (!statics.pTBarProvider)
      statics.pTBarProvider = new IDMItemProviderFor< IDMToolBarItem >;

    pTBar->setItemProvider( statics.pTBarProvider );
  }
}

/*------------------------------------------------------------------------------
| IDMHandler::setItemProvider                                                  |
------------------------------------------------------------------------------*/
void IDMHandler :: setItemProvider ( IToolBarButton* pTBarBtn )
{
  if (!pTBarBtn->itemProvider())
  {
    if (!statics.pTBarBtnProvider)
      statics.pTBarBtnProvider = new IDMItemProviderFor< IDMTBarButtonItem >;

    pTBarBtn->setItemProvider( statics.pTBarBtnProvider );
  }
}

/*------------------------------------------------------------------------------
| IDMHandler::isContainerControl                                               |
|                                                                              |
| Determines if the window is a container control.                             |
------------------------------------------------------------------------------*/
Boolean IDMHandler :: isContainerControl(const IWindow* pCtrlWin) const
{
  /*-------------------------------------------------------------------
     Verify that we have a non-zero window pointer
   -------------------------------------------------------------------*/
  if (pCtrlWin == 0)
    return(false);

  /*-------------------------------------------------------------------
     Determine if its a container by checking its window class.
   -------------------------------------------------------------------*/
  char pszClassName[10];
  long lLen = WinQueryClassName(pCtrlWin->handle(),
                                sizeof(pszClassName),
                                (PSZ)pszClassName);

  if (lLen > 0  &&  lLen <= 6  &&  *pszClassName == '#')
  {
     unsigned long ulClassName = atoi(pszClassName + 1);
     if (ulClassName == (unsigned short)(unsigned long)WC_CONTAINER)
       return(true);
  }

  return(false);
}

/*------------------------------------------------------------------------------
| IDMHandler::numberOfRenderers                                                |
|                                                                              |
| Returns the number of renderers.                                             |
------------------------------------------------------------------------------*/
unsigned IDMHandler :: numberOfRenderers ()
{
  return( rendererCollection()->numberOfElements() );
}

/*------------------------------------------------------------------------------
| IDMHandler::renderer                                                         |
|                                                                              |
| Returns the renderer with the given position.                                |
------------------------------------------------------------------------------*/
IDMRenderer* IDMHandler :: renderer (unsigned position)
{
  return( rendererCollection()->elementAtPosition(position) );
}

/*------------------------------------------------------------------------------
| IDMHandler::replaceRenderer                                                  |
|                                                                              |
| Replaces a given IDMRenderer with another.                                   |
------------------------------------------------------------------------------*/
IDMHandler& IDMHandler :: replaceRenderer (unsigned position,
                                           const IDMRenderer &replacement)
{
  rendererCollection()->removeAtPosition(position);
  rendererCollection()->addAtPosition(position, (IDMRenderer *)&replacement);
  return( *this );
}

/*------------------------------------------------------------------------------
| IDMHandler::addRenderer                                                      |
|                                                                              |
| Adds another IDMRenderer to this operation.                                  |
------------------------------------------------------------------------------*/
IDMHandler& IDMHandler :: addRenderer (const IDMRenderer &newRenderer)
{
  rendererCollection()->add((IDMRenderer *)&newRenderer);
  return( *this );
}

/*------------------------------------------------------------------------------
| IDMHandler::removeRenderer                                                   |
|                                                                              |
| Removes an IDMRenderer from this operation.                                  |
------------------------------------------------------------------------------*/
IDMHandler& IDMHandler :: removeRenderer (unsigned position)
{
  rendererCollection()->removeAtPosition(position);
  return( *this );
}

/*------------------------------------------------------------------------------
| IDMHandler::removeRenderer                                                   |
|                                                                              |
| Removes an IDMRenderer from this operation.                                  |
------------------------------------------------------------------------------*/
IDMHandler& IDMHandler :: removeRenderer (const IDMRenderer &rendererToRemove)
{
  /************************************************************************/
  /* Get number of renderers                                              */
  /************************************************************************/
  unsigned count = numberOfRenderers();

  /************************************************************************/
  /* Loop thru the collection and remove renderer if a match is found     */
  /************************************************************************/
  for (unsigned position = 1; (position <= count); position++)
  {
    if ((rendererCollection()->elementAtPosition(position))
                                                 == &rendererToRemove)
    {
      removeRenderer(position);
      break;
    }
  }
  return( *this );
}

/*------------------------------------------------------------------------------
| IDMHandler::setRenderer                                                      |
|                                                                              |
| Sets the renderer for this operation.  Removes any existing renderers.       |
------------------------------------------------------------------------------*/
IDMHandler& IDMHandler :: setRenderer (const IDMRenderer &newRenderer)
{
  if (this->pRendererSeqCl)
  {
    delete this->pRendererSeqCl;
    this->pRendererSeqCl = 0;
  }

  IDMHandler::addRenderer(newRenderer);
  return( *this );
}

/*------------------------------------------------------------------------------
| IDMHandler::rendererCollection                                               |
|                                                                              |
| Returns a pointer to the collection of IDMRenderers.                         |
------------------------------------------------------------------------------*/
IDMRendererSeq* IDMHandler :: rendererCollection ()
{
  if (!this->pRendererSeqCl)
    this->pRendererSeqCl = new IDMRendererSeq;

  return( this->pRendererSeqCl );
}

