/*******************************************************************************
* FILE NAME: idmtbrit.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMToolBarItem                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #include <iwindefs.h>
}

#ifndef _IDMTBRIT_
  #include <idmtbrit.hpp>
#endif

#ifndef _IDMEVENT_
  #include <idmevent.hpp>
#endif

#ifndef _ITBAR_
  #include <itbar.hpp>
#endif

#ifndef _ITBARBUT_
  #include <itbarbut.hpp>
#endif

#ifndef _ISTRING_
  #include <istring.hpp>
#endif

// tracing functions
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

// Exception handling
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMTBRIT_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IDMToolBarItem::IDMToolBarItem                                               |
|                                                                              |
| Constructor for toolbar target items.                                        |
------------------------------------------------------------------------------*/
IDMToolBarItem::IDMToolBarItem( const IDMItem::Handle& pTBarItem )  :
                          IDMItem(pTBarItem)
{
  IMODTRACE_DEVELOP( "IDMToolBarItem::ctor (target)" );
}

/*------------------------------------------------------------------------------
| IDMToolBarItem::~IDMToolBarItem                                              |
|                                                                              |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IDMToolBarItem :: ~IDMToolBarItem( )
{
  IMODTRACE_DEVELOP( "IDMToolBarItem::dtor" );
}

/*------------------------------------------------------------------------------
| IDMToolBarItem::targetDrop                                                   |
|                                                                              |
| Sets button bitmap and text into the target toolbar button,  or moves the    |
| toolbar button to a new position within the toolbar.                         |
------------------------------------------------------------------------------*/
Boolean IDMToolBarItem :: targetDrop( IDMTargetDropEvent& evt )
{
  IMODTRACE_DEVELOP( "IDMToolBarItem::targetDrop" );

  /********************************************************************/
  /* Get the target toolbar                                           */
  /********************************************************************/
  IToolBar *pTgtTBar = (IToolBar *)evt.window();

  /********************************************************************/
  /* If type includes "menuItem" then process the menuitem link       */
  /* request.                                                         */
  /********************************************************************/
  if (types().includes( IDM::menuItem ))
  {
    /******************************************************************/
    /* Create the toolbar button (and make it the last item in the    */
    /* list for now)                                                  */
    /******************************************************************/
    IToolBarButton *pButton = new IToolBarButton( (unsigned long)object(),
                                                  pTgtTBar,
                                                  pTgtTBar );
    pButton->setAutoDeleteObject();
    pTgtTBar->addAsLast( pButton, true );

    return( true );
  } //end menuItem processing

  /********************************************************************/
  /* If type includes "toolBarButton" then process the move request   */
  /********************************************************************/
  if (types().includes( IDM::toolBarButton ))
  {
    /******************************************************************/
    /* Get the source button and the toolbar it resides in            */
    /******************************************************************/
    IToolBarButton *pSrcBtn = (IToolBarButton *)sourceWindow();
    IToolBar *pSrcTBar = (IToolBar *)pSrcBtn->parent();

    /******************************************************************/
    /* If we're in the same toolbar ...                               */
    /******************************************************************/
    if (pSrcTBar == pTgtTBar)
    {
      /******************************************************************/
      /* Move it to the end for now ...                                 */
      /******************************************************************/
      pTgtTBar->moveToLast ( pSrcBtn );
    }
    else
    {
      /****************************************************************/
      /* Add it to the new toolbar at the end for now and remove it   */
      /* from the old one ...                                         */
      /****************************************************************/
      pSrcTBar->remove( pSrcBtn );
      pSrcBtn->setParent( pTgtTBar );
      pSrcBtn->setOwner( pTgtTBar );
      pTgtTBar->addAsLast( pSrcBtn, true );
    }

    return( true );
  } //end toolBarButton processing

  return( false );
}

