/*******************************************************************************
* FILE NAME: iedithdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iedithdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define IC_MULTIMEDIA
#define INCL_CIRCULARSLIDER
#define INCL_SW
#define INCL_WINDIALOGS
#define INCL_WINENTRYFIELDS
#define INCL_WINMLE
#define INCL_WINSTDSLIDER
#define INCL_WINSTDSPIN
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
  #include <stdlib.h>
}

#ifndef _IEDITHDR_
  #include <iedithdr.hpp>
#endif
#ifndef _ICTLEVT_
  #include <ictlevt.hpp>
#endif
#ifndef _IEVENT_
  #include <ievent.hpp>
#endif
#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif
#ifndef _IWCNAME_
  #include <iwcname.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IEDITHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IEditHandler::IEditHandler                                                   |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IEditHandler::IEditHandler ( )
  : IEditHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| IEditHandler::~IEditHandler                                                  |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IEditHandler::~IEditHandler ( )
{ }

/*------------------------------------------------------------------------------
| IEditHandler::dispatchHandlerEvent                                           |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
Boolean IEditHandler::dispatchHandlerEvent ( IEvent& event )
{
  Boolean bRc = false;

  if ( event.eventId() == WM_CONTROL )
  {                                    // Control event.
     IControlEvent ctlevt( event );
     IWindowHandle hwndControl = ctlevt.controlHandle();

     if ( hwndControl )
     {
        unsigned long subMsg = ctlevt.parameter1().number2();
        IWindowClassName className( hwndControl );

        if ( className == WC_COMBOBOX )
        {                              // Combo box case.
           if ( subMsg == CBN_EFCHANGE )
           {
              bRc = this->edit( ctlevt );
           }
        }
        else if ( className == WC_ENTRYFIELD )
        {                              // Entry field case.
           if ( subMsg == EN_CHANGE )
           {
              bRc = this->edit( ctlevt );
           }
        }
        // Don't process MLE separately, since on Windows it is the
        // same as the entry field, which was just processed above.
        else if ( className == WC_MLE )
        {                              // Multi-line edit case.
           if ( subMsg == MLN_CHANGE )
           {
              bRc = this->edit( ctlevt );
           }
        }
        else if ( className == WC_SPINBUTTON )
        {                              // Spin button case.
           if ( subMsg == SPBN_CHANGE )
           {
              bRc = this->edit( ctlevt );
           }
        }
        else if ( className == WC_SLIDER )
        {                              // Slider case.
           if ( subMsg == SLN_CHANGE )
           {
              bRc = this->edit( ctlevt );
           }
        }
#ifdef WC_CIRCULARSLIDER
        else if ( className == WC_CIRCULARSLIDER )
        {                              // Circular slider case.
           if ( subMsg == CSN_CHANGED )
           {
              bRc = this->edit( ctlevt );
           }
        }
#endif

        if ( bRc )
        {                              // Processing done.
           event.setResult( ctlevt.result() );  // Copy event result.
        }
     }
  }

  return bRc;
}
