/*******************************************************************************
* FILE NAME: ievent.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ievent.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_WINERRORS
#define INCL_WINMESSAGEMGR
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
}

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif
#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif
#include <ievtpriv.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IEVENT_CPP_
  #include <ipagetun.h>
#endif


/**************************************************************/
/* Constructor.                                               */
/**************************************************************/
IEvent::IEvent ( IWindow* window,
                 unsigned long eventId,
                 const IEventParameter1& parameter1,
                 const IEventParameter2& parameter2 )
 : hCl( 0 ),
   p_dispatchingHandle( 0 ),
   p_controlHandle( 0 ),
   ulClEvtId( eventId ), epCl1( parameter1 ),
   epCl2( parameter2 ), erCl( 0 ),
   pData(new IEventPrivateData())
{
   if ( window )
   {
      this->hCl = window->handle();
      this->p_dispatchingHandle = window->handle();
      this->p_controlHandle = window->handle();
   }
}

/**************************************************************/
/* Constructor.                                               */
/**************************************************************/
IEvent::IEvent ( const IWindowHandle& handle,
                 unsigned long eventId,
                 const IEventParameter1& parameter1,
                 const IEventParameter2& parameter2 )
 : hCl( handle ),
   p_dispatchingHandle( handle ),
   p_controlHandle( handle),
   ulClEvtId( eventId ),
   epCl1( parameter1 ), epCl2( parameter2 ), erCl( 0 ),
   pData(new IEventPrivateData())
{ }

// Stop the compiler construction of static instances.
IEvent :: IEvent ( const IEvent& event )
 : hCl( event.hCl ),
   p_dispatchingHandle( event.p_dispatchingHandle ),
   p_controlHandle( event.p_controlHandle ),
   ulClEvtId( event.ulClEvtId ), epCl1( event.epCl1 ),
   epCl2( event.epCl2 ), erCl( event.erCl )
{
  pData = new IEventPrivateData();
  pData->setHandlerCursor(event.pData->handlerCursor());
  pData->setHandlerList(event.pData->handlerList());
}

IEvent& IEvent::operator= (const IEvent &event)
{
  if (this != &event)
  {
    hCl = event.hCl;
    p_dispatchingHandle = event.p_dispatchingHandle;
    p_controlHandle = event.p_controlHandle;
    ulClEvtId = event.ulClEvtId;
    epCl1 = event.epCl1;
    epCl2 = event.epCl2;
    erCl = event.erCl;
    delete pData;
    pData = new IEventPrivateData();
    pData->setHandlerCursor(event.pData->handlerCursor());
    pData->setHandlerList(event.pData->handlerList());
  }
  return *this;
}

/*------------------------------------------------------------------------------
   Destructor (here so compiler doesn't generate one).
------------------------------------------------------------------------------*/
IEvent :: ~IEvent ( )
{
  delete pData;
}

/**************************************************************************/
/* Return the IWindow object pointer for the window which had this event. */
/**************************************************************************/
IWindow* IEvent::window ( ) const
{
  IMODTRACE_ALL( "IEvent::window" );
  IWindow* pwin = IWindow::windowWithHandle( IWindowHandle( handle() ));
  return pwin;
}

/*****************************************************************/
/* Return the IWindow object pointer for the dispatching window. */
/*****************************************************************/
IWindow* IEvent::dispatchingWindow ( ) const
{
  IMODTRACE_ALL( "IEvent::dispatchingWindow" );
  IWindow* pwin = IWindow::windowWithHandle( p_dispatchingHandle );
  return pwin;
}

/***********************************************************************/
/* Return the IWindow object pointer for the original control window.  */
/***********************************************************************/
IWindow* IEvent::controlWindow ( ) const
{
  IMODTRACE_ALL( "IEvent::controlWindow" );
  IWindow* pwin = 0;
  if ( p_controlHandle )
  {
     pwin = IWindow::windowWithHandle( p_controlHandle );
  }
  return pwin;
}
