/*******************************************************************************
* FILE NAME: ifiledlg.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ifiledlg.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
{
  #define INCL_WINSTDFILE
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
  #include <string.h>
  #include <stdlib.h>
}

#ifndef _IFILEDLG_
  #include <ifiledlg.hpp>
#endif

#ifndef _IFLDPRIV_
  #include <ifldpriv.hpp>
#endif

#ifndef _IPMEXSTA_
  #include <ipmexsta.hpp>
#endif

#ifndef _IRESLIB_
  #include <ireslib.hpp>
#endif

#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

#ifndef _IHANDLER_
  #include <ihandler.hpp>
#endif

#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

#ifndef _ITHREAD_
  #include <ithread.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IFILEDLG_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLStaticConst)
const IFileDialog::Style
  IFileDialog::filter            = FDS_FILTERUNION,            // 0x00000004
  IFileDialog::helpButton        = FDS_HELPBUTTON,             // 0x00000008
  IFileDialog::applyButton       = FDS_APPLYBUTTON,            // 0x00000010
  IFileDialog::preload           = FDS_PRELOAD_VOLINFO,        // 0x00000020
  IFileDialog::modeless          = FDS_MODELESS,               // 0x00000040
  IFileDialog::includeEAS        = FDS_INCLUDE_EAS,            // 0x00000080
  IFileDialog::multiSelection    = FDS_MULTIPLESEL,            // 0x00000400
  IFileDialog::selectableListbox = FDS_ENABLEFILELB,           // 0x00000800
  IFileDialog::noStyle           = 0,
  IFileDialog::classDefaultStyle = 0;

const unsigned long
  IFileDialog::ok                = DID_OK,
  IFileDialog::cancel            = DID_CANCEL;
#pragma data_seg()

#pragma data_seg(ICLNonConst)
  IFileDialog::Style IFileDialog::currentDefaultStyle = 0;
#pragma data_seg()

extern "C"
{
  void* _System _IFileDlgProc(unsigned long hwnd, unsigned long ulMsg, void* mp1, void* mp2);
}

//
// Implementation for nested Settings class
//

/*------------------------------------------------------------------------------
| IFileDialog::Settings::Settings                                              |
------------------------------------------------------------------------------*/
IFileDialog::Settings::Settings( ) :
  fSettingsData( new IFileDialogSettingsData ),
  titleStr(),
  OKButtonStr(),
  initialDriveStr(),
  initialFileTypeStr(),
  fileNameStr(),
  isOpenDlg(true),
  isPosSet(false)
{
}

/*------------------------------------------------------------------------------
| IFileDialog::Settings& IFileDialog::Settings::setTitle                       |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::setTitle(const IResourceId& id)
{

    IString newText = id.resourceLibrary().loadString(id);
    if (newText.length())
      setTitle(newText);
    return *this;
}


/*------------------------------------------------------------------------------
| IFileDialog::Settings& IFileDialog::Settings::setOKButtonText                |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::setOKButtonText(const IResourceId& id)
{

    IString newText = id.resourceLibrary().loadString(id);
    if (newText.length())
      setOKButtonText(newText);
    return *this;
}


/*------------------------------------------------------------------------------
| IFileDialog::Settings& IFileDialog::Settings::addFileType                    |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::addFileType(const char* fileType)
{
   if (fSettingsData->fileTypes == 0)
     fSettingsData->fileTypes = new StringSeq;

   IString* pString = new IString(fileType);
   fSettingsData->fileTypes->add(pString);

   return *this;
}


/*------------------------------------------------------------------------------
| IFileDialog::Settings& IFileDialog::Settings::addDrive                       |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::addDrive(const char* drive)
{
   IMODTRACE_DEVELOP("IFileDialog::Settings::addDrive");
   if (fSettingsData->drives == 0)
     fSettingsData->drives = new StringSeq;

   IString* pString = new IString(drive);
   fSettingsData->drives->add(pString);

   return *this;
}



/*------------------------------------------------------------------------------
| IFileDialog::Settings::~Settings                                             |
------------------------------------------------------------------------------*/
IFileDialog::Settings::~Settings()
{
  IMODTRACE_DEVELOP("~IFileDialog::Settings");
  delete fSettingsData->fileTypes;
  delete fSettingsData->drives;
  delete fSettingsData;
}


/*------------------------------------------------------------------------------
| IFileDialog::Settings&  IFileDialog::Settings::setTitle                      |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::setTitle (const char* newTitle)
{
  titleStr = newTitle;
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialog::Settings&  IFileDialog::Settings::setOKButtonText               |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::setOKButtonText(const char* newText)
{
  OKButtonStr = newText;
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialog::Settings& IFileDialog::Settings::setFileName                    |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::setFileName(const char* fileName)
{
  fileNameStr = fileName;
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialog::Settings& IFileDialog::Settings::setInitialDrive                |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::setInitialDrive(const char* drive)
{
  initialDriveStr = drive;
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialog::Settings&  IFileDialog::Settings::setInitialFileType            |
------------------------------------------------------------------------------*/
IFileDialog::Settings& IFileDialog::Settings::setInitialFileType(const char* type)
{
  initialFileTypeStr = type;
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialog::Settings::setDialogTemplate                                     |
------------------------------------------------------------------------------*/
IFileDialog::Settings&
  IFileDialog::Settings::setDialogTemplate ( const IResourceId& resId )
{
  fSettingsData->isDlgTemplateSet = true;
  fSettingsData->dlgId = resId.id();
  fSettingsData->dlgHandle = resId.resourceLibrary().handle();
  return *this;
}

/*------------------------------------------------------------------------------
| StringSeq*  IFileDialog::Settings::fileTypes ( ) const                       |
------------------------------------------------------------------------------*/
StringSeq*  IFileDialog::Settings::fileTypes ( ) const
{
  return fSettingsData->fileTypes;
}

/*------------------------------------------------------------------------------
| StringSeq*  IFileDialog::Settings::drives( ) const                           |
------------------------------------------------------------------------------*/
StringSeq*  IFileDialog::Settings::drives( ) const
{
  return fSettingsData->drives;
}

/*------------------------------------------------------------------------------
| IBase::Boolean  IFileDialog::Settings::isDialogTemplateSet ( ) const         |
------------------------------------------------------------------------------*/
IBase::Boolean  IFileDialog::Settings::isDialogTemplateSet ( ) const
{
  return fSettingsData->isDlgTemplateSet;
}

/*------------------------------------------------------------------------------
| _IFileDlgProc                                                                |

| Override the WinFileDlg dialog procedure so that the handler scheme          |
| will work for Modal dialogs.                                                 |
------------------------------------------------------------------------------*/
void* _System _IFileDlgProc(HWND hwnd, unsigned long msg, void* mp1, void* mp2)
{
  // Find this pointer for IFileDialog object
  FILEDLG* pfildData = (FILEDLG*)WinQueryWindowPtr(hwnd, QWL_USER);
  IFileDialog* pfildlg = (IFileDialog*)pfildData->ulUser;

  if (pfildlg == (IFileDialog*)0)
       return IPMCtlStatics::defFileDlgProc(hwnd, msg, mp1, mp2);

  switch(msg)
  {
     case WM_INITDLG:

        pfildlg->addDefaultHandler();    // for IFrameWindow

        pfildlg->addToWindowSet(pfildlg, IWindowHandle(hwnd));

        if (pfildlg->fFileDialogData->handlerToAdd != 0)
           pfildlg->fFileDialogData->handlerToAdd->handleEventsFor(pfildlg);
        break;

     case WM_COMMAND:
     case WM_SYSCOMMAND:            // Keep these from going to default
     case WM_CLOSE:                 // frame handler
        return IPMCtlStatics::defFileDlgProc(hwnd, msg, mp1, mp2);

  } /* end switch */

  // call dispatch so that any handlers will get called
  void* result;
  IEvent newEvt(hwnd, msg, mp1, mp2);
  if (pfildlg->dispatch(newEvt))
     result = newEvt.result();
  else
     result = IPMCtlStatics::defFileDlgProc(hwnd, msg, mp1, mp2);

  if(msg==WM_DESTROY)
  {
     pfildlg->removeFromWindowSet(pfildlg);
  }

  return result;
}

//
// Implementation for IFileDialog
//

/*------------------------------------------------------------------------------
| IFileDialog::IFileDialog                                                     |
------------------------------------------------------------------------------*/
IFileDialog::IFileDialog ( IWindow*  parent,
                           IWindow*  owner,
                           const Style& style )
  : IFrameWindow ( IFrameWindow::deferCreation ),
    fFileDialogData(new IFileDialogData)
{
  Settings tempSettings;
  initialize(parent, owner, style, 0, tempSettings);
}

/*------------------------------------------------------------------------------
| IFileDialog::IFileDialog                                                     |
------------------------------------------------------------------------------*/
IFileDialog::IFileDialog ( IWindow*        parent,
                           IWindow*        owner,
                           const Settings& settings,
                           const Style&    style )
  : IFrameWindow ( IFrameWindow::deferCreation ),
    fFileDialogData(new IFileDialogData)
{
  initialize(parent, owner, style, 0, settings);
}

/*------------------------------------------------------------------------------
| IFileDialog::IFileDialog                                                     |
------------------------------------------------------------------------------*/
IFileDialog::IFileDialog ( IWindow*        parent,
                           IWindow*        owner,
                           const Style&    style,
                           const Settings& settings )
  : IFrameWindow ( IFrameWindow::deferCreation ),
    fFileDialogData(new IFileDialogData)
{
  initialize(parent, owner, style, 0, settings);
}

/*------------------------------------------------------------------------------
| IFileDialog::IFileDialog                                                     |
------------------------------------------------------------------------------*/
IFileDialog::IFileDialog ( IWindow*        parent,
                           IWindow*        owner,
                           IHandler*       handler,
                           const Style&    style,
                           const Settings& settings )
  : IFrameWindow ( IFrameWindow::deferCreation ),
    fFileDialogData(new IFileDialogData)
{
  initialize(parent, owner, style, handler, settings);
}


/*------------------------------------------------------------------------------
| IFileDialog::~IFileDialog                                                    |
------------------------------------------------------------------------------*/
IFileDialog::~IFileDialog( )
{
  IMODTRACE_DEVELOP("~IFileDialog");
  int i;

  delete [] pFileDlgData->pszTitle;
  delete [] pFileDlgData->pszOKButton;
  delete [] pFileDlgData->pszIType;
  delete [] pFileDlgData->pszIDrive;

  if(pFileDlgData->papszITypeList)
  {
    for(i=0; *pFileDlgData->papszITypeList[i]; i++)
       delete [] *pFileDlgData->papszITypeList[i];
    delete [] pFileDlgData->papszITypeList;
  }

  if(pFileDlgData->papszIDriveList)
  {
    for(i=0; *pFileDlgData->papszIDriveList[i]; i++)
       delete [] *pFileDlgData->papszIDriveList[i];
    delete [] pFileDlgData->papszIDriveList;
  }

  if (pFileDlgData->papszFQFilename)
     IPMCtlStatics::freeFileDlgList((char**)(pFileDlgData->papszFQFilename));

  delete pFileDlgData;
  if (fFileDialogData)
    delete fFileDialogData;
}

/*------------------------------------------------------------------------------
| IFileDialog::isModeless                                                      |
------------------------------------------------------------------------------*/
Boolean IFileDialog::isModeless () const
{
   return pFileDlgData->fl & FDS_MODELESS;
}

/*------------------------------------------------------------------------------
| IFileDialog::buttonPressedId                                                 |
------------------------------------------------------------------------------*/
unsigned long IFileDialog::buttonPressedId ( ) const
{
   return pFileDlgData->lReturn;
}

/*------------------------------------------------------------------------------
| IFileDialog::pressedOK                                                       |
------------------------------------------------------------------------------*/
Boolean IFileDialog::pressedOK ( ) const
{
   return buttonPressedId() == DID_OK ;
}

/*------------------------------------------------------------------------------
| IFileDialog::selectedFileCount                                               |
------------------------------------------------------------------------------*/
unsigned IFileDialog::selectedFileCount() const
{
   return pFileDlgData->ulFQFCount;
}


/*------------------------------------------------------------------------------
| IFileDialog::returnValue                                                     |
------------------------------------------------------------------------------*/
long IFileDialog::returnValue () const
{
   return pFileDlgData->lSRC;
}

/*------------------------------------------------------------------------------
| IFileDialog::fileName                                                        |
------------------------------------------------------------------------------*/
IString IFileDialog::fileName (unsigned fileNumber) const
{
   if (fileNumber == 0) {
     return IString(pFileDlgData->szFullFile);
   }

   if (fileNumber < 1 ||
       fileNumber > selectedFileCount() +1 ||
       !pFileDlgData->papszFQFilename )
        return IString();

  return IString((char*)(*pFileDlgData->papszFQFilename[fileNumber]));
}


/*------------------------------------------------------------------------------
| IFileDialog::saveAsEAType                                                    |
------------------------------------------------------------------------------*/
IString IFileDialog::saveAsEAType ( ) const
{
   if ((pFileDlgData->sEAType != -1) && pFileDlgData->papszITypeList)
      return IString((char*)(*pFileDlgData->papszITypeList[pFileDlgData->sEAType]));

   return IString();
}


/*------------------------------------------------------------------------------
| IFileDialog::Style  IFileDialog::defaultStyle                              |
------------------------------------------------------------------------------*/
IFileDialog::Style  IFileDialog::defaultStyle()
{
  return currentDefaultStyle;
}


/*------------------------------------------------------------------------------
| IFileDialog::convertToGUIStyle                                               |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IFileDialog::convertToGUIStyle(const IBitFlag& guiStyle,
                                             Boolean bExtOnly) const
{
  // Obtain the style from the class (IFrameWindow) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= extendedStyle() & IS_EXTMASK;
  }
  else
  {
    // FDS_ styles have a one-to-one correspondence to our style bits, and
    // inhabit the lower word of the base GUI style.  Therefore, obtain
    // that portion asis, masking out the upper word.
    ulStyle |= guiStyle.asUnsignedLong() & IFDS_MASK;
  }

  return( ulStyle );
}


/*------------------------------------------------------------------------------
| IFileDialog::initialize                                                          |
|                                                                              |
| Called by constructor to actually start the dialog.                          |
| For a modal dialog does not return until user has dismissed dialog.          |
| Because of this, we have override the PM dialog procedure with our own       |
| so that the handler scheme will work.                                        |
------------------------------------------------------------------------------*/
void IFileDialog::initialize (IWindow*        parent,
                              IWindow*        owner,
                              const Style&    style,
                              IHandler*       handler,
                              const Settings& settings)
{
   IMODTRACE_DEVELOP("IFileDialog::initialize");

   // It's possible that the first thing an application may do is
   // create a IFileDialog. Since we deferCreation of our parent IFrameWindow
   // we should be sure that PM has been initialized.
   IThread::current().initializeGUI();

   fFileDialogData->handlerToAdd = handler;

   pFileDlgData = new FILEDLG;
   memset(pFileDlgData, 0, sizeof(FILEDLG) );
   pFileDlgData->cbSize = sizeof(FILEDLG);

   // Save the extended style to make sure we have a copy of it stored
   setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

   // Convert the style bits into the GUI native style settings
   pFileDlgData->fl = convertToGUIStyle( style );

   if (settings.isOpenDialog()) {
      pFileDlgData->fl |= FDS_OPEN_DIALOG;
   } else {
      pFileDlgData->fl |= FDS_SAVEAS_DIALOG;
   } /* endif */

   if (settings.isPositionSet()) {
      pFileDlgData->x = (short)(settings.position().x());
      pFileDlgData->y = (short)(settings.position().y());
   } else {
      pFileDlgData->fl |= FDS_CENTER;
   } /* endif */

   if (settings.isDialogTemplateSet()) {
      pFileDlgData->fl |= FDS_CUSTOM;
      pFileDlgData->hMod = settings.fSettingsData->dlgHandle;
      pFileDlgData->usDlgId = (unsigned short)(settings.fSettingsData->dlgId);
   }

   pFileDlgData->ulUser = (unsigned long)this;

   if (settings.title().length()) {
      pFileDlgData->pszTitle = (PSZ)new char[settings.title().length()+1];
      strcpy(pFileDlgData->pszTitle, settings.title() );
   }

   if (settings.okButtonText().length()) {
      pFileDlgData->pszOKButton = (PSZ)new char[settings.okButtonText().length()+1];
      strcpy(pFileDlgData->pszOKButton, settings.okButtonText() );
   }

   if (settings.initialDrive().length()) {
      pFileDlgData->pszIDrive = (PSZ)new char[settings.initialDrive().length()+1];
      strcpy(pFileDlgData->pszIDrive, settings.initialDrive() );
   }

   if (settings.initialFileType().length()) {
      pFileDlgData->pszIType = (PSZ)new char[settings.initialFileType().length()+1];
      strcpy(pFileDlgData->pszIType, settings.initialFileType() );
   }

   strcpy(pFileDlgData->szFullFile, settings.fileName() );

   pFileDlgData->pfnDlgProc = (FNWP*)_IFileDlgProc;

   IString* curString;
   int i;

   // Settings maintains the list of Types and Drives as BB collections
   // of strings. These have to be filled into an array of char pointers
   // in the PM FILEDIALOG structure...
   StringSeq* curQ = settings.fileTypes();
   if (curQ) {
     i = 0;
     // allocate space for the table of char pointers
     pFileDlgData->papszITypeList = (PAPSZ) new char*[curQ->numberOfElements() + 1];
     StringSeq::Cursor current(*curQ);
     for (current.setToFirst(); current.isValid(); current.setToNext() ) {
        curString = curQ->elementAt(current);
        // allocate space for the current string
        *pFileDlgData->papszITypeList[i] = new char[curString->length() + 1];
        // copy current string into storage just allocated
        strcpy(*pFileDlgData->papszITypeList[i], *curString);
        i++;
     } /* end for */
     *pFileDlgData->papszITypeList[i] = 0;       // array terminator
   }

   curQ = settings.drives();
   if (curQ) {
     i = 0;
     // allocate space for the table of char pointers
     pFileDlgData->papszIDriveList = (PAPSZ) new char*[curQ->numberOfElements() + 1];
     StringSeq::Cursor current(*curQ);
     for (current.setToFirst(); current.isValid(); current.setToNext() ) {
        curString = curQ->elementAt(current);
        // allocate space for the current string
        *pFileDlgData->papszIDriveList[i] = new char[curString->length() + 1];
        // copy current string into storage just allocated
        strcpy(*pFileDlgData->papszIDriveList[i], *curString);
        i++;
     } /* end for */
     *pFileDlgData->papszIDriveList[i] = 0;
   }

   // Need to set a variable in IWindow that holds the default procedure
   // so that IFrameHandler will work.
   setDefaultProcedure(IPMCtlStatics::defFileDlgProcAddr());

   // Make call to WinFileDlg via IPMCtlStatics. Trying to avoid reference
   // to WinFileDlg until actually needed.
   if (!IPMCtlStatics::fileDlg(parent == 0 ? desktopWindow()->handle() : parent->handle(),
                   owner  == 0 ? IWindowHandle(0)        : owner->handle(),
                   pFileDlgData) )
   {
      ITHROWGUIERROR("WinFileDlg");
   }
}

