/*******************************************************************************
* FILE NAME: ifilehdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ifilehdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C"
{
  #define INCL_WINMESSAGEMGR
  #define INCL_WINSTDFILE
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
}

#ifndef _IPMEXSTA_
  #include <ipmexsta.hpp>
#endif

#ifndef _ISTRING_
  #include <istring.hpp>
#endif

#ifndef _IFILEHDR_
  #include <ifilehdr.hpp>
#endif

#ifndef _IFILEDLG_
  #include <ifiledlg.hpp>
#endif

#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IFILEHDR_CPP_
  #include <ipagetun.h>
#endif

//
// Implementation for IFileDialogHandler.
//

/*------------------------------------------------------------------------------
| IFileDialogHandler::IFileDialogHandler                                       |
------------------------------------------------------------------------------*/
IFileDialogHandler::IFileDialogHandler ( )
  : IFileDialogHandler::Inherited()
{ }

/*------------------------------------------------------------------------------
| IFileDialogHandler::~IFileDialogHandler                                      |
------------------------------------------------------------------------------*/
IFileDialogHandler::~IFileDialogHandler ( )
{ }

/*------------------------------------------------------------------------------
| IFileDialogHandler::handleEventsFor                                          |
------------------------------------------------------------------------------*/
IFileDialogHandler& IFileDialogHandler::handleEventsFor ( IFileDialog* fileDialog )
{
  IASSERTPARM( fileDialog != 0 );
  Inherited::handleEventsFor( fileDialog );
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::stopHandlingEventsFor                                    |
------------------------------------------------------------------------------*/
IFileDialogHandler& IFileDialogHandler::stopHandlingEventsFor
                                                   ( IFileDialog* fileDialog )
{
  IASSERTPARM( fileDialog != 0 );
  Inherited::stopHandlingEventsFor( fileDialog );
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::handleEventsFor                                          |
------------------------------------------------------------------------------*/
IHandler& IFileDialogHandler::handleEventsFor( IWindow *window )
{
  Inherited::handleEventsFor( window );
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::stopHandlingEventsFor                                    |
------------------------------------------------------------------------------*/
IHandler& IFileDialogHandler::stopHandlingEventsFor( IWindow *window )
{
  Inherited::stopHandlingEventsFor( window );
  return *this;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::dispatchHandlerEvent                                     |
------------------------------------------------------------------------------*/
Boolean IFileDialogHandler::dispatchHandlerEvent ( IEvent& event )
{
  Boolean retcond = false;

  IFileDialogEvent fileEvt( event );
  fileEvt.setResult( true );
  switch ( event.eventId() )
  {
     case FDM_FILTER:
        retcond = this->filter( fileEvt );
        break;
     case FDM_VALIDATE:
        retcond = this->validate( fileEvt );
        break;
     case WM_DESTROY:
     {
        FILEDLG* pfildData = (FILEDLG*)WinQueryWindowPtr( event.handle(), QWL_USER );
        IFileDialog* pfildlg = (IFileDialog*)pfildData->ulUser;

        if ( pfildlg != 0 )
        {
           // Even If dialog was cancelled, call modelessResults().
           if ( pfildlg->isModeless() )
              this->modelessResults( pfildlg );
        }
        break;
     }
     default:
        break;
   } /* endswitch */

   // filter() and validate() will call setResult on fileEvt to indicate if
   // the file is OK to process.  We have to pass this on to the original event.
   event.setResult( fileEvt.result() );

   if ( !(Boolean)fileEvt.result() )
   {
      // If validate() or filter() setResult to false, the file should not be
      // processed, and therefore no further handlers should be called.
      return true;
   }
   else
   {
      return retcond;
   }
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::filter                                                   |
------------------------------------------------------------------------------*/
Boolean IFileDialogHandler::filter ( IFileDialogEvent& event )
{
  event.setResult( this->filterName( event.fileName(), 
                                     (IFileDialog*)event.window() ));
  return false;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::filterName                                               |
------------------------------------------------------------------------------*/
Boolean IFileDialogHandler::filterName ( const IString& fileName,
                                         IFileDialog* dialog )
{
  return true;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::validate                                                 |
------------------------------------------------------------------------------*/
Boolean IFileDialogHandler::validate ( IFileDialogEvent& event )
{
  event.setResult( this->validateName( event.fileName(),
                                       (IFileDialog*)event.window() ));
  return false;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::validateName                                             |
------------------------------------------------------------------------------*/
Boolean IFileDialogHandler::validateName ( const IString& fileName,
                                           IFileDialog* dialog )
{
  return true;
}

/*------------------------------------------------------------------------------
| IFileDialogHandler::modelessResults                                          |
------------------------------------------------------------------------------*/
Boolean IFileDialogHandler::modelessResults ( IFileDialog* endingDialog )
{
  return true;
}


//
// Implementation for IFileDialogEvent.
//

/*------------------------------------------------------------------------------
| IFileDialogEvent::~IFileDialogEvent                                          |
------------------------------------------------------------------------------*/
IFileDialogEvent::~IFileDialogEvent ( )
{ }

/*------------------------------------------------------------------------------
| IFileDialogEvent::IFileDialogEvent                                           |
------------------------------------------------------------------------------*/
IFileDialogEvent::IFileDialogEvent ( IEvent& event )
  : IEvent( event )
{ }

/*------------------------------------------------------------------------------
| IFileDialogEvent::fileName                                                   |
------------------------------------------------------------------------------*/
IString IFileDialogEvent::fileName ( ) const
{
  char* temp = this->parameter1();
  return temp;
}

/*------------------------------------------------------------------------------
| IFileDialogEvent::fileLength                                                 |
------------------------------------------------------------------------------*/
unsigned long IFileDialogEvent::fileLength ( ) const
{
  char* temp = this->parameter1();
  IString str(temp);
  return str.length();
}
