/*******************************************************************************
* FILE NAME: iflyhhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iflyhhdr.hpp                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WIN
  #define INCL_WINMESSAGEMGR
  #define INCL_WINWINDOWMGR
  #define INCL_WINPOINTERS
  #define INCL_WININPUT
  #include <iwindefs.h>
}

#include <iflyhhdr.hpp>
#include <iflytext.hpp>
#include <iflypriv.hpp>
#include <iapp.hpp>
#include <icoordsy.hpp>
#include <iexcept.hpp>
#include <irect.hpp>
#include <ireslib.hpp>
#include <ictlevt.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IFLYHHDR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IHandleStringElement::IHandleStringElement                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IHandleStringElement::IHandleStringElement( unsigned long      handle,
                                            const IString&     flyText,
                                            const IString&     longText )
: handleKey(handle), flyStr(flyText), longStr(longText), flyResId(0),
  longResId(0), fStrings(true)
{}

/*------------------------------------------------------------------------------
| IHandleStringElement::IHandleStringElement                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IHandleStringElement::IHandleStringElement( unsigned long      handle,
                                            const IResourceId& flyText,
                                            const IResourceId& longText )
: handleKey(handle), flyStr(), longStr(), flyResId(flyText),
  longResId(longText), fStrings(false)
{}

/*------------------------------------------------------------------------------
| IHandleStringElement::~IHandleStringElement                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IHandleStringElement::~IHandleStringElement( )
{
}

/*------------------------------------------------------------------------------
| IHandleStringSet::IHandleStringSet                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IHandleStringSet::IHandleStringSet()
: IKeySet < IHandleStringElement*, unsigned long > ()
{}

/*------------------------------------------------------------------------------
| IHandleStringSet::~IHandleStringSet                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IHandleStringSet::~IHandleStringSet()
{}

/*------------------------------------------------------------------------------
| IFlyOverData::IFlyOverData                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverData::IFlyOverData( IFlyText* flyText, ITextControl* longText,
                            unsigned long delay, unsigned long initialDelay,
                            IFlyOverHelpHandler* flyHelp )
: fFlytxt(flyText), fLongtxt(longText),
  fLongOffset(0), fFlyOffset(0),
  fTimer(),
  fPrevPos(),
  fDefault(),
  fSetPrevToNull(false),
  fHandle(0),
  fPrevHandle(0),
  fControlHandle(0),
  fControlWindow(0),
  fFly(flyHelp),
  fResLib(0),
  fHandleStringSet(0),
  fTimerDelay(delay),            // Save the original timer value.
  fInitialTimerDelay(initialDelay)
{
}

/*------------------------------------------------------------------------------
| IFlyOverData::~IFlyOverData                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverData::~IFlyOverData()
{
  if (fHandleStringSet)
  {
    IHandleStringSet::Cursor cursor(*fHandleStringSet);
    forCursor (cursor)
    {
      delete fHandleStringSet->elementAt( cursor );
    }
    fHandleStringSet->removeAll();
  }
  delete fHandleStringSet;
  delete fResLib;
  fTimer.stop();
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::IFlyOverHelpHandler                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler::IFlyOverHelpHandler( IFlyText* flyText,
                                          unsigned long initialDelay,
                                          unsigned long delay )
: fFlyOverData( new IFlyOverData(flyText, 0, delay, initialDelay, this))
{
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::IFlyOverHelpHandler                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler::IFlyOverHelpHandler( ITextControl* longText,
                                          unsigned long initialDelay,
                                          unsigned long delay )
: fFlyOverData( new IFlyOverData(0, longText, delay, initialDelay, this))
{
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::IFlyOverHelpHandler                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler::IFlyOverHelpHandler( IFlyText* flyText,
                                          ITextControl* longText,
                                          unsigned long initialDelay,
                                          unsigned long delay )
: fFlyOverData( new IFlyOverData(flyText, longText, delay, initialDelay, this))
{
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::~IFlyOverHelpHandler                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler::~IFlyOverHelpHandler()
{
  delete fFlyOverData;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::flyTextControl                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyText* IFlyOverHelpHandler::flyTextControl( ) const
{
  return fFlyOverData->fFlytxt;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::longTextControl                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITextControl* IFlyOverHelpHandler::longTextControl( ) const
{
  return fFlyOverData->fLongtxt;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::setFlyTextControl                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::setFlyTextControl(IFlyText* flyText)
{
  fFlyOverData->fFlytxt = flyText;
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::setLongTextControl                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::setLongTextControl(
                                                        ITextControl* longText )
{
  fFlyOverData->fLongtxt = longText;
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::resourceLibrary                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IResourceLibrary& IFlyOverHelpHandler::resourceLibrary( ) const
{
  // If we've got our own, return it, else use the current user default.
  if ( fFlyOverData->fResLib )
    return *fFlyOverData->fResLib;
  else
    return IApplication::current().userResourceLibrary();
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::setResourceLibrary                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::setResourceLibrary(
                                                   const IModuleHandle &resMod )
{
  // Construct a DLL from the input handle.

  delete fFlyOverData->fResLib;
  fFlyOverData->fResLib = 0;
  if ( resMod )
    fFlyOverData->fResLib = new IDynamicLinkLibrary( resMod );
  else
    fFlyOverData->fResLib = new IResourceLibrary();

  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::setResourceLibrary                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::setResourceLibrary(
                                                        const char *resDLLName )
{
  // Construct a DLL from the input dll name.

  delete fFlyOverData->fResLib;
  fFlyOverData->fResLib = 0;
  if ( resDLLName )
    fFlyOverData->fResLib = new IDynamicLinkLibrary( resDLLName );
  else
    fFlyOverData->fResLib = new IResourceLibrary();

  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::setFlyStringTableOffset                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::setFlyTextStringTableOffset(
                                                                long newOffset )
{
  fFlyOverData->fFlyOffset = newOffset;
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::setLongStringTableOffset                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::setLongStringTableOffset(
                                                                long newOffset )
{
  fFlyOverData->fLongOffset = newOffset;
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::flyTextStringTableOffset                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
long IFlyOverHelpHandler::flyTextStringTableOffset ( ) const
{
  return fFlyOverData->fFlyOffset;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::longStringTableOffset                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
long IFlyOverHelpHandler::longStringTableOffset  ( ) const
{
  return fFlyOverData->fLongOffset;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::defaultText                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IFlyOverHelpHandler::defaultText ( ) const
{
  return fFlyOverData->fDefault;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::setDefaultText                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::setDefaultText( unsigned long id )
{
  fFlyOverData->fDefault = this->resourceLibrary().loadString( id );
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::setDefaultText                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::setDefaultText( const IString &id )
{
  fFlyOverData->fDefault = id;
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::flyHelpText                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IFlyOverHelpHandler::flyHelpText(const IWindowHandle& handle ) const
{
  if ( fFlyOverData->fHandleStringSet &&
       fFlyOverData->fHandleStringSet->containsElementWithKey( handle ))
  {
    IHandleStringElement*
      elementWithHandle(fFlyOverData->fHandleStringSet->elementWithKey( handle ));
    return elementWithHandle->flyStr;
  }
  else
  {
    return IString();
  }
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::longHelpText                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IFlyOverHelpHandler::longHelpText(const IWindowHandle& handle ) const
{
  if ( fFlyOverData->fHandleStringSet &&
       fFlyOverData->fHandleStringSet->containsElementWithKey( handle ))
  {
    IHandleStringElement*
      elementWithHandle(fFlyOverData->fHandleStringSet->elementWithKey( handle ));
    return elementWithHandle->longStr;
  }
  else
  {
    return IString();
  }
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::setHelpText                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::setHelpText
                                               ( const IWindowHandle& handle,
                                                 const IString&       flyText,
                                                 const IString&       longText )
{
  if ( !fFlyOverData->fHandleStringSet )
    fFlyOverData->fHandleStringSet = new IHandleStringSet();

  removeHelpText( handle );

  fFlyOverData->fHandleStringSet->add(
                           new IHandleStringElement(handle,flyText,longText));
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::setHelpText                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::setHelpText
                                               ( const IWindowHandle& handle,
                                                 const IResourceId&   flyText,
                                                 const IResourceId&   longText )
{
  if ( !fFlyOverData->fHandleStringSet )
    fFlyOverData->fHandleStringSet = new IHandleStringSet();

  removeHelpText( handle );

  fFlyOverData->fHandleStringSet->add(
                           new IHandleStringElement(handle,flyText,longText));
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::removeHelpText                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::removeHelpText
                                                 ( const IWindowHandle& handle )
{
  if ( fFlyOverData->fHandleStringSet &&
       fFlyOverData->fHandleStringSet->containsElementWithKey( handle ))
  {
    IHandleStringElement *oldElement((IHandleStringElement*)
                      fFlyOverData->fHandleStringSet->elementWithKey( handle ));
    fFlyOverData->fHandleStringSet->removeElementWithKey( handle );
    delete oldElement;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::setDelayTime                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::setDelayTime( unsigned long milliseconds )
{
  fFlyOverData->fTimer.setInterval( milliseconds/2 );
  // Save the new timer delay.
  fFlyOverData->fTimerDelay = milliseconds;
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::delayTime                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IFlyOverHelpHandler::delayTime( ) const
{
  // Return the last set time delay.
  // return fFlyOverData->fTimer.interval();
  return fFlyOverData->fTimerDelay;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::setInitialDelayTime                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::setInitialDelayTime( unsigned long milliseconds )
{
  // Save the new timer delay.
  fFlyOverData->fInitialTimerDelay = milliseconds;
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::initialDelayTime                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IFlyOverHelpHandler::initialDelayTime( ) const
{
  // Return the last set time delay.
  // return fFlyOverData->fTimer.interval();
  return fFlyOverData->fInitialTimerDelay;
}


/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::startHandlingEventsFor                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::handleEventsFor( IWindow* window )
{
  IASSERTPARM(window != 0);
  IFlyOverHelpHandler::Inherited::handleEventsFor( window );
  if (!fFlyOverData->fTimer.isStarted())
  {
    fFlyOverData->fTimer.setInterval( fFlyOverData->fInitialTimerDelay/2 );
    fFlyOverData->fTimer.start(
      new ITimerMemberFn<IFlyOverData>(*fFlyOverData, IFlyOverData::timerpop));

    fFlyOverData->fPrevPos = IWindow::pointerPosition();
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::stopHandlingEventsFor                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverHelpHandler& IFlyOverHelpHandler::stopHandlingEventsFor( IWindow* window )
{
  IASSERTPARM(window != 0);
  IFlyOverHelpHandler::Inherited::stopHandlingEventsFor( window );
  fFlyOverData->fTimer.stop();
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::dispatchHandlerEvent                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IFlyOverHelpHandler::dispatchHandlerEvent( IEvent& event )
{
  if (event.eventId() == WM_CONTROLPOINTER || event.eventId() == WM_MOUSEMOVE)
  {
    if (!fFlyOverData->fTimer.isStarted() &&
        WinIsChild( event.handle(), WinQueryActiveWindow( IWindow::desktopWindow()->handle() )))
    {
      // Kick the timer off.
      // Note: the interval is divided by two becuase it takes to timer pops
      //       in order to show fly over help for a window.
      fFlyOverData->fTimer.setInterval( fFlyOverData->fInitialTimerDelay/2 );
      fFlyOverData->fTimer.start(
        new ITimerMemberFn<IFlyOverData>(*fFlyOverData, IFlyOverData::timerpop));
      fFlyOverData->fPrevPos = IWindow::pointerPosition();
    }
    if (event.eventId() == WM_CONTROLPOINTER)
    {
      IControlEvent controlEvent(event);
      // If the mouse is of the fly-over text control then do NOT change
      // the fHandle data member
      if ( fFlyOverData->fFlytxt &&
         ( fFlyOverData->fFlytxt->handle() == controlEvent.controlHandle() ))
      {
         // Just do nothing for this case (in other words, leave the
         // fHandle data member at it's previous value)
      }
      else
      {
        fFlyOverData->fHandle = controlEvent.controlHandle();
        if (!fFlyOverData->fHandle)
          fFlyOverData->fHandle = event.handle();
      }
    }
    else
      fFlyOverData->fHandle = event.handle();
  }
  // If the window the handler is attached to is moving,
  // stop the timer and hide the flytext control.
  else if (event.eventId() == WM_ADJUSTWINDOWPOS)
  {
    PSWP pswp((PSWP)event.parameter1().asUnsignedLong());
    if (pswp->fl & SWP_MOVE)
    {
      if ( fFlyOverData->fFlytxt )
      {
         fFlyOverData->fFlytxt->hide();
      }
      fFlyOverData->fPrevHandle = 0;
    }
  }
  return false;
}

/*------------------------------------------------------------------------------
| IFlyOverData::timerpop                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
void IFlyOverData::timerpop( unsigned long timerId )
{
  // Get the mouse location.

  POINTL pt;
  WinQueryPointerPos( HWND_DESKTOP, &pt );

  // If either of the controls is no longer valid, return.

  if (fFlytxt && !fFlytxt->isValid())
    return;
  if (fLongtxt && !fLongtxt->isValid())
    return;

  // Get the handle of the control that is under the mouse.
  // Note:  controls that are disabled will not be returned since
  //        they do not respond to hit-test.
  fControlHandle = WinWindowFromPoint( IWindow::desktopWindow()->handle(), &pt, true );

  // If the control that is under the mouse is a descendant of the
  // handle that was in the control pointer or mousemove event or
  // the mouse is over the fly text control (this prevents the fly text
  // control from being removed once displayed for a window) and
  // the control is a child of the active window then proceed to show
  // fly over help.

  Boolean overFlyTextWindow(fFlytxt && (fFlytxt->handle() == fControlHandle));

  if (( WinIsChild( fControlHandle, fHandle ) &&
        WinIsChild( fControlHandle, WinQueryActiveWindow( IWindow::desktopWindow()->handle() ))) ||
        overFlyTextWindow )
  {
    if (overFlyTextWindow)
      return;
    if (pt.x == fPrevPos.x() && pt.y == fPrevPos.y())  // If the mouse has been
                                                       // at the same location.
    {
      fControlWindow = IWindow::windowWithHandle( fControlHandle );

      // If the controlWindow is null, then we are dealing with
      // an aggregate control that is not wrappered.

      if (!fControlWindow)
      {
        // Query the handle's parent and then try to get the
        // IWindow pointer that corresponds to the parent handle.

        IWindowHandle parentHandle(WinQueryWindow(fControlHandle, QW_PARENT ));
        fControlWindow = IWindow::windowWithHandle( parentHandle );
      }

      if (fControlWindow && (fControlWindow != fFlytxt))
      {
        // Look through the handle string set to see if we have
        // a hit based on the window handle.

        if ( fHandleStringSet &&
             fHandleStringSet->containsElementWithKey( fControlHandle ))
        {
          // We go a hit in the handle set.
          IHandleStringElement*
                   mouseOverElement(fHandleStringSet->elementWithKey(fControlHandle));

          if ( mouseOverElement->fStrings )
          {
            if (fFlytxt)
              setFlyText(mouseOverElement->flyStr);
            if (fLongtxt)
              setLongText(mouseOverElement->longStr);
          }
          else
          {
            unsigned long id(mouseOverElement->flyResId.id());
            if (id && fFlytxt)
              setFlyText(mouseOverElement->flyResId.resourceLibrary().tryToLoadString(id));
            id = mouseOverElement->longResId.id();
            if (id && fLongtxt)
              setLongText(mouseOverElement->longResId.resourceLibrary().tryToLoadString(id));
          }
        }
        else
        {
          unsigned long controlId(fControlWindow->id());

          if ( fFlytxt )
          {
            IString flyInfo = IString();
            flyInfo = fFly->resourceLibrary().tryToLoadString(controlId + fFlyOffset);
            if (flyInfo.length() == 0)
              setFlyText(fFly->defaultText());
            else
              setFlyText(flyInfo);
          }

          if ( fLongtxt )
          {
            IString longInfo = IString();
            longInfo = fFly->resourceLibrary().tryToLoadString(controlId + fLongOffset);
            if (longInfo.length() == 0)
              setLongText(fFly->defaultText());
            else
              setLongText(longInfo);
          }
        }
      }

      // Set the previous handle to the current handle for the next timerpop.
      fPrevHandle = fControlHandle;

      // set the interval to 1/3 second so we can tell "quickly" if we've
      // moved off of the window.
      fTimer.setInterval(333);
    }
    else // The mouse has moved.
    {
      // If we are no longer over the same control.
      if (fPrevHandle != fControlHandle)
      {
        // Reset previous handle to null and reset the interval.
        fPrevHandle = 0;
        fTimer.setInterval(fTimerDelay/2);
        if (fFlytxt)
          fFlytxt->hide();
        if (fLongtxt)
          setLongText( fDefault );
      }
    }
    // Update the previous point to the current point.
    fPrevPos.setX( pt.x ).setY( pt.y );
  }
  else
  {
    // Mouse is no longer over a window that is a child of the
    // window the flyOverHelpHandler what attached to.

    if (fFlytxt)
    {
      fFlytxt->hide();
    }
    if (fLongtxt)
    {
      setLongText( fDefault );
    }
    fPrevHandle = 0;
    fTimer.stop();
  }
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::setFlyText                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverData& IFlyOverData::setFlyText( const IString& helpText )
{
  if(helpText.size()!=0)
  {
    if(fFlytxt->text()!=helpText)
    {
      fFlytxt->hide();
      fFlytxt->setText(IString::change(helpText, (char*)"\xA", (char*)""));
    }
    // If the previous handle is null, this means the mouse has moved
    // onto a new window so show the fly text control.
    if ( fPrevHandle == 0 )
    {
      positionTextWindow();
    }
  }
  else
  {
    if(fFlytxt->isVisible())
      fFlytxt->hide();
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverHelpHandler::setLongText                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyOverData& IFlyOverData::setLongText( const IString& helpText )
{
  Boolean preventLongTextUpdate(fLongtxt->sendEvent(IC_UM_QRY_PREVENTUPDATE));
  if (!preventLongTextUpdate)
  {
    if (helpText.size()!=0)
      fLongtxt->setText( helpText );
    else
      if ( fDefault.length() != 0 )
        fLongtxt->setText( fDefault );
      else
        // At a minimum, set the long text to a single blank to avoid
        // the frame handler from hiding the long text control if the
        // long text control happens to be a frame extension.
        fLongtxt->setText( " " );
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyOverData::positionTextWindow                                             |
|                                                                              |
| Reposition and show the flyText window if the mouse has moved over a         |
| new window since the last timer pop.                                         |
|                                                                              |
| Note:  The window handle of the window the mouse is over and the IWindow*    |
|        is passed into this function.  The window handle will not correspond  |
|        to the IWindow object in the case of aggregate controls (spin button, |
|        combo box, mle, etc).  The IWindow pointer is needed to query the     |
|        layoutAdjustment for such controls as combo box.                      |
------------------------------------------------------------------------------*/
IFlyOverData& IFlyOverData::positionTextWindow( )
{
  if (fPrevHandle != fControlHandle)
  {
    fFlytxt->hide();
    SWP swp;

    // Get the location and size of the window.

    IQUERYWINDOWPOS(fControlHandle, &swp);

    // Map the location of the window the mouse is over to desktop
    // coordinates.
    IPoint pt(0,0);
    pt = IWindow::mapPoint(pt, fControlHandle,
                           IWindow::desktopWindow()->handle() );

    IRectangle controlRect( pt, ISize(swp.cx, swp.cy) );

    // Get the control's visible rectangle.
    IRectangle visibleRect(fControlWindow->visibleRectangle());

    // If the rectangle is empty then this might be a graphic push
    // button (so we would need the parents visible rect instead)
    if ( visibleRect.size() == ISize(0,0) )
    {
       IWindow* pParent = fControlWindow->parent();
       if ( pParent && ( fControlWindow->id() == pParent->id() ))
       {
          visibleRect = pParent->visibleRectangle();
          fControlWindow = pParent;
       }
    }

    // if the visible rectangle does not equal the rectangle
    // from query the control (as in the case of a drop down
    // combo box, then convert the origin point of the
    // visible rectangle to desktop coordinates and position
    // the fly text relative to it.
    if (controlRect.size() == visibleRect.size())
      fFlytxt->setRelativeWindowRect( controlRect );
    else
    {
      pt = IWindow::mapPoint( visibleRect.minXMinY(),
                              fControlWindow->parent()->handle(),
                              IWindow::desktopWindow()->handle());
      fFlytxt->setRelativeWindowRect( IRectangle( pt, visibleRect.size() ));
    }
    fFlytxt->postEvent(IC_UM_FLY_PAINT);
  }
  return *this;
}

