/*******************************************************************************
* FILE NAME: iflytext.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iflytext.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
{
  #define INCL_WINWINDOWMGR
  #define INCL_WINSTATICS
  #define INCL_WINSYS
  #define INCL_WINCOUNTRY
  #define INCL_WINHOOKS
  #include <iwindefs.h>
}

#include <iflytext.hpp>
#include <iflypain.hpp>
#include <icoordsy.hpp>
#include <igbundle.hpp>
#include <igpyline.hpp>
#include <iptarray.hpp>
#include <irect.hpp>
#include <ithread.hpp>
#include <icolor.hpp>
#include <ifont.hpp>
#include <iexcept.hpp>
#include <istring.hpp>
#include <ireslib.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IFLYTEXT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
  This class wrappers an object pointer and destroys the pointed
  to object when the wrapper is destructed.
------------------------------------------------------------------------------*/
static struct DefaultHandler {
  operator IFlyTextPaintHandler* ();
IFlyTextPaintHandler
 *operator -> ()
    {
    return *this;
    }
  ~DefaultHandler ( )
    {
    if (ptr)
      {
      delete ptr;
      ptr = 0;
      }
    }
IFlyTextPaintHandler
 *ptr;
} defaultHandler;

/*------------------------------------------------------------------------------
| DefaultHandler::operator IFlyTextPaintHandler*                               |
------------------------------------------------------------------------------*/
DefaultHandler :: operator IFlyTextPaintHandler* ()
{
  if ( !ptr )
    ptr = new IFlyTextPaintHandler;
  return ptr;
}


Boolean    IFlyText::fgRegistered(false);
IRectangle IFlyText::fgScreen(IRectangle(0,0,0,0));

/*------------------------------------------------------------------------------
| IFlyText::IFlyText                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyText::IFlyText ( unsigned long id, IWindow *owner )
  : IFlyText::Inherited( ), fFlyTextData(0), fHelpArrow(),
    fArrowPos( IFlyText::upperLeft )
{
  IGraphicBundle bundle;
  bundle.setDrawOperation( IGraphicBundle::frame );
  IPointArray ptArray( 11 );
  ptArray[0].setX( 5 ).setY( 0 );
  ptArray[1].setX( 5 ).setY( 2 );
  ptArray[2].setX( 4 ).setY( 3 );
  ptArray[3].setX( 1 ).setY( 0 );
  ptArray[4].setX( 0 ).setY( 0 );
  ptArray[5].setX( 0 ).setY( 12 );
  ptArray[6].setX( 12 ).setY( 12 );
  ptArray[7].setX( 12 ).setY( 11 );
  ptArray[8].setX( 9 ).setY( 8 );
  ptArray[9].setX( 10 ).setY( 7 );
  ptArray[10].setX( 12 ).setY( 7 );

  IGPolyline *polyline(new IGPolyline( ptArray ));
  polyline->setGraphicBundle( bundle );
  fHelpArrow.addAsFirst( *polyline );
  ptArray.resize( 9 );
  ptArray[0].setX( 6 ).setY( 0 );
  ptArray[1].setX( 6 ).setY( 2 );
  ptArray[2].setX( 4 ).setY( 4 );
  ptArray[3].setX( 1 ).setY( 1 );
  ptArray[4].setX( 1 ).setY( 11 );
  ptArray[5].setX( 11 ).setY( 11 );
  ptArray[6].setX( 8 ).setY( 8 );
  ptArray[7].setX( 10 ).setY( 6 );
  ptArray[8].setX( 12 ).setY( 6 );
  fBorderArrow = new IGPolyline( ptArray );
  fBorderArrow->setGraphicBundle( bundle );
  fHelpArrow.addAsFirst( *fBorderArrow );
  ptArray[0].setX( 7 ).setY( 0 );
  ptArray[1].setX( 7 ).setY( 2 );
  ptArray[2].setX( 4 ).setY( 5 );
  ptArray[3].setX( 2 ).setY( 4 );
  ptArray[4].setX( 2 ).setY( 10 );
  ptArray[5].setX( 9 ).setY( 10 );
  ptArray[6].setX( 7 ).setY( 8 );
  ptArray[7].setX( 9 ).setY( 5 );
  ptArray[8].setX( 12 ).setY( 5 );
  polyline = new IGPolyline( ptArray );
  polyline->setGraphicBundle( bundle );
  fHelpArrow.addAsFirst( *polyline );
  ptArray.resize( 10 );
  ptArray[0].setX( 8 ).setY( 0 );
  ptArray[1].setX( 8 ).setY( 2 );
  ptArray[2].setX( 4 ).setY( 6 );
  ptArray[3].setX( 3 ).setY( 5 );
  ptArray[4].setX( 3 ).setY( 9 );
  ptArray[5].setX( 7 ).setY( 9 );
  ptArray[6].setX( 6 ).setY( 8 );
  ptArray[7].setX( 9 ).setY( 4 );
  ptArray[8].setX( 12 ).setY( 4 );
  ptArray[9].setX( 12 ).setY( 0 );
  bundle.setDrawOperation( IGraphicBundle::fillAndFrame );
  fArrow = new IGPolygon( ptArray );
  fArrow->setGraphicBundle( bundle );
  fHelpArrow.addAsFirst( *fArrow );

  if (!fgRegistered)
  {
    fgRegistered = true;
    // query the dimension of the desktop

    fgScreen = IRectangle( 0,0,
                         IQUERYSYSVALUE(SV_CXSCREEN),
                         IQUERYSYSVALUE(SV_CYSCREEN));
  }

  IWindowHandle flytext(
  this -> create ( id,
                   0,
                   SS_TEXT | WS_SAVEBITS,
                   WC_STATIC,
                   IWindow::desktopWindow()->handle(),
                   owner->handle(),
                   IRectangle(),
                   0,
                   0 ));

  startHandlingEventsFor(flytext);
  this -> addHandler( defaultHandler );

  // Set default font to Helv.8
  // Note: DBCS OS/2 systems should use the default font instead of Helv
  //       since the double-byte characters do not exist fot this font

  unsigned long codepage = WinQueryCp(HMQ_CURRENT);
  switch (codepage)
  {
    // Note: The following list represents the DBCS code pages supported
    //       by OS/2 2.1 and Warp as of June 1995

    case  949: // Korean                      (replaces older 934/944)
    case  944: // Korean
    case  934: // Korean
    case 1381: // People's Republic of China  (replaces older 936/946)
    case  936: // People's Republic of China 
    case  946: // People's Republic of China 
    case  950: // Taiwan                      (replaces older 938/948)
    case  938: // Taiwan
    case  948: // Taiwan
    case  932: // Japanese
    case  942: // Japanese
    {
      IFont fnt(this);
      fnt.setPointSize(8);
      fnt.setWindowFont(this);
    }
    break;
    default:
    {
      IFont fnt("Helv",8);
      fnt.setWindowFont(this);
    }
    break;
  }

  // Set the default colors
  setBorderColor( IColor::black );
  setShadowColor( IColor::white );
  setBackgroundColor( IColor::white );
  setForegroundColor( IColor::black );

}

/*------------------------------------------------------------------------------
| IFlyText::~IFlyText                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyText::~IFlyText()
{
  this -> removeHandler( defaultHandler );
  IGList::Cursor cursor( fHelpArrow );
  for (cursor.setToFirst(); cursor.isValid() ; cursor.setToNext())
  {
    IGraphic* graphic(&fHelpArrow.graphicAt( cursor ));
    delete graphic;
  } /* endfor */
}

/*------------------------------------------------------------------------------
| IFlyText::setText                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyText& IFlyText::setText( const char* text )
{
  Inherited::setText( IString::change(text, (char*)"\xA", (char*)""));
  this->sizeTo( displaySize() + ISize(16,16) );
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyText::setText                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyText& IFlyText::setText( const IResourceId& text )
{
  IString str(text.resourceLibrary().loadString(text));
  setText((char*)str);
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyText::setRelativeWindowRect                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IFlyText& IFlyText::setRelativeWindowRect ( const IRectangle& rect )
{
  ArrowPos      bestPos;
  unsigned long bestArea(0),
                tempArea(0);
  IPoint        bestPoint;

  fWinrc = rect;

  IUPDATEWINDOW(IWindow::desktopWindow()->handle());

  // Added logic to move the text box in so that it overlaps more of
  // the control that help is being provided for.  The text box is
  // moved in 5 horizontally and 4 vertically below.

  IRectangle textRect(IPoint(0,0), this->size());
  IPoint newPos;
  if ( ICoordinateSystem::applicationOrientation() ==
       ICoordinateSystem::originLowerLeft )
     {
     newPos = IPoint(  fWinrc.maxX() - 5,
                       fWinrc.minY() + 4 - textRect.height());
     }
  else
     {
     newPos = IPoint(  fWinrc.maxX() - 5,
                       fWinrc.maxY() - 4 );
     }
  textRect.moveTo( newPos );

  fArrowPos = IFlyText::upperLeft;        // help text to lower right of window
  if (!fgScreen.contains( textRect ))
  {
    // determine the area of the fly text rectangle that
    // can be shown on the screen and save it to compare
    // with the other locations.
    bestArea  = IRectangle(fgScreen & textRect).area();
    bestPoint = textRect.minXMinY();
    bestPos   = fArrowPos;

    fArrowPos = IFlyText::upperRight;     // help text to lower left of window
    newPos.setX( fWinrc.minX() - textRect.width() + 5 );

    textRect.moveTo(newPos);
    if (!fgScreen.contains( textRect ))
    {
      // determine the area of the fly text rectangle that
      // can be shown.

      tempArea = IRectangle(fgScreen & textRect).area();

      // If the new area is larger, save it as the best position.
      if (bestArea < tempArea)
      {
        bestArea  = tempArea;
        bestPoint = textRect.minXMinY();
        bestPos   = fArrowPos;
      }

      fArrowPos = IFlyText::lowerRight;   // help text to upper left of window
      if ( ICoordinateSystem::applicationOrientation() ==
           ICoordinateSystem::originLowerLeft )
         newPos.setY( fWinrc.maxY() - 4);
      else
         newPos.setY( fWinrc.minY() + 4 - textRect.height() );
      textRect.moveTo(newPos);
      if (!fgScreen.contains( textRect ))
      {
        // determine the area of the fly text rectangle that
        // can be shown.
        tempArea = IRectangle(fgScreen & textRect).area();

        // If the new area is larger, save it as the best position.
        if (bestArea < tempArea)
        {
          bestArea  = tempArea;
          bestPoint = textRect.minXMinY();
          bestPos   = fArrowPos;
        }

        fArrowPos = IFlyText::lowerLeft;  // help text to upper right of window
        newPos.setX( fWinrc.maxX() - 5);
        textRect.moveTo(newPos);

        if (!fgScreen.contains( textRect ))
        {
          // determine the area of the fly text rectangle that
          // can be shown.
          tempArea = IRectangle(fgScreen & textRect).area();

          // If the new area is larger, save it as the best position.
          if (bestArea < tempArea)
          {
            bestArea  = tempArea;
            bestPoint = textRect.minXMinY();
            bestPos   = fArrowPos;
          }
        }
        else
        {
          bestArea = 0;
        }
      }
      else
      {
        bestArea = 0;
      }
    }
    else
    {
      bestArea = 0;
    }
  }
  if (bestArea != 0)
  {
    fArrowPos = bestPos;
    this->moveTo(bestPoint);
  }
  else
  {
    this->moveTo(textRect.minXMinY());
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IFlyText::relativeWindowRect                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IRectangle IFlyText::relativeWindowRect( ) const
{
  return fWinrc;
}

/*------------------------------------------------------------------------------
| IFlyText::hide                                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IWindow& IFlyText::hide( ) 
{
  Boolean invalidate = false;
  if (this->isVisible())
  {
    // Check to see if the entire fly text is visible (we do this by
    // checking each corner point to see if the window under that point
    // is the fly text control).  In the future, the WinQueryVisibleRegion
    // API may provide a better solution, but it is not available on
    // OS/2 2.1 systems.
    IRectangle rctl = this->rect().shrinkBy(ISize(1,1));
    POINTL pt = rctl.bottomLeft().asPOINTL();
    if ( IWINDOWFROMPOINT ( IWindow::desktopWindow()->handle(), &pt, true )
                            != this->handle() )
    {
      invalidate = true;
    }
    pt = rctl.topLeft().asPOINTL();
    if ( IWINDOWFROMPOINT ( IWindow::desktopWindow()->handle(), &pt, true )
                            != this->handle() )
    {
      invalidate = true;
    }
    pt = rctl.topRight().asPOINTL();
    if ( IWINDOWFROMPOINT ( IWindow::desktopWindow()->handle(), &pt, true )
                            != this->handle() )
    {
      invalidate = true;
    }
    pt = rctl.bottomRight().asPOINTL();
    if ( IWINDOWFROMPOINT ( IWindow::desktopWindow()->handle(), &pt, true )
                            != this->handle() )
    {
      invalidate = true;
    }
  }
  Inherited::hide();
  if ( invalidate )
  {
     IWindow::desktopWindow()->refresh(this->rect());
  }
  return *this;
}
