/*******************************************************************************
* FILE NAME: ifocshdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ifocshdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define IC_MULTIMEDIA
#define INCL_CIRCULARSLIDER
#define INCL_SW
#define INCL_WINDIALOGS
#define INCL_WINENTRYFIELDS
#define INCL_WINLISTBOXES
#define INCL_WINMLE
#define INCL_WINSTDCNR
#define INCL_WINSTDSLIDER
#define INCL_WINSTDSPIN
#define INCL_WINSTDVALSET
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
  #include <stdlib.h>
}

#ifndef _IFOCSHDR_
  #include <ifocshdr.hpp>
#endif
#ifndef _ICTLEVT_
  #include <ictlevt.hpp>
#endif
#ifndef _IEVENT_
  #include <ievent.hpp>
#endif
#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif
#ifndef _IWCNAME_
  #include <iwcname.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IFOCSHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IFocusHandler::IFocusHandler                                                 |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IFocusHandler::IFocusHandler ( )
  : IFocusHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| IFocusHandler::~IFocusHandler                                                |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IFocusHandler::~IFocusHandler ( )
{ }

/*------------------------------------------------------------------------------
| IFocusHandler::dispatchHandlerEvent                                          |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
Boolean IFocusHandler::dispatchHandlerEvent ( IEvent& event )
{
  Boolean bRc = false;

  if ( event.eventId() == WM_CONTROL )
  {                                    // Control event.
     IControlEvent ctlevt( event );
     IWindowHandle hwndControl = ctlevt.controlHandle();

     if ( hwndControl )
     {
        unsigned long subMsg = ctlevt.parameter1().number2();
        IWindowClassName className( hwndControl );

        if ( className == WC_ENTRYFIELD )
        {                              // Entry field case.
           if ( subMsg == EN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == EN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }
        else if ( className == WC_LISTBOX )
        {                              // List box case.
           if ( subMsg == LN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == LN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }
        // Don't process MLE separately, since on Windows it is the
        // same as the entry field, which was just processed above.
        else if ( className == WC_MLE )
        {                              // Multi-line edit case.
           if ( subMsg == MLN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == MLN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }
        else if ( className == WC_SPINBUTTON )
        {                              // Spin button case.
           if ( subMsg == SPBN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == SPBN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }
        else if ( className == WC_CONTAINER )
        {                              // Container case.
           if ( subMsg == CN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == CN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }
        else if ( className == WC_SLIDER )
        {                              // Slider case.
           if ( subMsg == SLN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == SLN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }
#ifdef WC_CIRCULARSLIDER
        else if ( className == WC_CIRCULARSLIDER )
        {                              // Circular slider case.
           if ( subMsg == CSN_SETFOCUS )
           {
              if ( ctlevt.parameter2().number1() )
              {
                 bRc = this->gotFocus( ctlevt );
              }
              else
              {
                 bRc = this->lostFocus( ctlevt );
              }
           }
        }
#endif
        else if ( className == WC_VALUESET )
        {                              // Value set case.
           if ( subMsg == VN_SETFOCUS )
           {
              bRc = this->gotFocus( ctlevt );
           }
           else if ( subMsg == VN_KILLFOCUS )
           {
              bRc = this->lostFocus( ctlevt );
           }
        }

        if ( bRc )
        {                              // Processing done.
           event.setResult( ctlevt.result() );  // Copy event result.
        }
     }
  } /* endif WM_CONTROL */

  return bRc;
}

Boolean IFocusHandler::gotFocus ( IControlEvent& event )
{
  return false;
}

Boolean IFocusHandler::lostFocus ( IControlEvent& event )
{
  return false;
}
