/*******************************************************************************
* FILE NAME: ifontdlg.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ifontdlg.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINSTDFONT
  #define INCL_WINWINDOWMGR
  #define INCL_WINSYS
  #include <iwindefs.h>
  #include <string.h>
}

#include <ifontdlg.hpp>

#ifndef _IPMEXSTA_
  #include <ipmexsta.hpp>
#endif

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif
#ifndef _IFONT_
  #include <ifont.hpp>
#endif
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif
#ifndef _IHANDLER_
  #include <ihandler.hpp>
#endif
#ifndef _ITHREAD_
  #include <ithread.hpp>
#endif
#ifndef _IRESLIB_
  #include <ireslib.hpp>
#endif

extern "C"
{
   void* _System _IFontDlgProc(unsigned long hwnd, unsigned long ulMsg, void* mp1, void* mp2);
}

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IFONTDLG_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLStaticConst)
const IFontDialog::Style
  IFontDialog::helpButton        = FNTS_HELPBUTTON,            // 0x00000008
  IFontDialog::applyButton       = FNTS_APPLYBUTTON,           // 0x00000010
  IFontDialog::resetButton       = FNTS_RESETBUTTON,           // 0x00000020
  IFontDialog::modeless          = FNTS_MODELESS,              // 0x00000040
  IFontDialog::bitmapOnly        = FNTS_BITMAPONLY,            // 0x00000100
  IFontDialog::vectorOnly        = FNTS_VECTORONLY,            // 0x00000200
  IFontDialog::fixedWidthOnly    = FNTS_FIXEDWIDTHONLY,        // 0x00000400
  IFontDialog::proportionalOnly  = FNTS_PROPORTIONALONLY,      // 0x00000800
  IFontDialog::noSynthesize      = FNTS_NOSYNTHESIZEDFONTS,    // 0x00001000
  IFontDialog::noStyle           = 0,
  IFontDialog::classDefaultStyle = FNTS_RESETBUTTON;

const unsigned long
  IFontDialog::ok                = DID_OK,
  IFontDialog::cancel            = DID_CANCEL;
#pragma data_seg()

#pragma data_seg(ICLNonConst)
  IFontDialog::Style IFontDialog::currentDefaultStyle = FNTS_RESETBUTTON;
#pragma data_seg()

//
// Implementation for nested Settings

/*------------------------------------------------------------------------------
| IFontDialog::Settings::Settings                                              |
------------------------------------------------------------------------------*/
IFontDialog::Settings::Settings (IFont* pfont)
{
   fData = new FONTDLG;
   memset(fData, 0, sizeof(FONTDLG) );

   fData->cbSize = sizeof(FONTDLG);
   fData->clrFore = SYSCLR_OUTPUTTEXT;
   fData->clrBack = SYSCLR_ENTRYFIELD;
   fData->usWeight = 5;
   fData->usWidth = 5;
   fData->fAttrs.usRecordLength = sizeof(FATTRS);

   fApplyFont = pfont;

   if (fApplyFont) {
      memcpy(&(fData->fAttrs), fApplyFont->fattrs(), sizeof(FATTRS));

      fData->fl |= FNTS_INITFROMFATTRS;
      fData->fxPointSize = MAKEFIXED(fApplyFont->pointSize(), 0);

      if (fApplyFont->isBitmapOnly())
         fData->fl |= FNTS_BITMAPONLY;

      if (fApplyFont->isVectorOnly())
         fData->fl |= FNTS_VECTORONLY;

      if (fApplyFont->isNonPropOnly())
         fData->fl |= FNTS_FIXEDWIDTHONLY;
   }
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings::~Settings                                             |
------------------------------------------------------------------------------*/
IFontDialog::Settings::~Settings ()
{
   delete [] fData->pszTitle;
   delete [] fData->pszPreview;
   delete [] fData->pszPtSizeList;
   delete [] fData->pszFamilyname;
   delete fData;
}

/*------------------------------------------------------------------------------
| IFontDialog::Settings::setFont                                               |
------------------------------------------------------------------------------*/
IFontDialog::Settings& IFontDialog::Settings::setFont( IFont* font )
{
   fApplyFont = font;

   if (fApplyFont)
   {
      memcpy(&(fData->fAttrs), fApplyFont->fattrs(), sizeof(FATTRS));

      fData->fl |= FNTS_INITFROMFATTRS;
      fData->fxPointSize = MAKEFIXED(fApplyFont->pointSize(), 0);

      if (fApplyFont->isBitmapOnly())
         fData->fl |= FNTS_BITMAPONLY;

      if (fApplyFont->isVectorOnly())
         fData->fl |= FNTS_VECTORONLY;

      if (fApplyFont->isNonPropOnly())
         fData->fl |= FNTS_FIXEDWIDTHONLY;
   }
   return *this;
}

/*------------------------------------------------------------------------------
| IFontDialog::Settings::setTitle                                              |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setTitle (const char* pszTitle)
{
   delete [] fData->pszTitle;
   if (pszTitle) {
      fData->pszTitle = (PSZ)new char[strlen(pszTitle)+1];
      strcpy((char*)fData->pszTitle, pszTitle);
   } else {
      fData->pszTitle = 0;
   } /* endif */
   return *this;
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings& IFontDialog::Settings::setTitle                       |
------------------------------------------------------------------------------*/
IFontDialog::Settings& IFontDialog::Settings::setTitle(const IResourceId& id)
{
    IString newText = id.resourceLibrary().loadString(id);
    if (newText.length())
      setTitle(newText);
    return *this;
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings::setPreviewText                                        |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setPreviewText (const char* pszPreview)
{
   delete [] fData->pszPreview;
   if (pszPreview) {
      fData->pszPreview = (PSZ)new char[strlen(pszPreview)+1];
      strcpy((char*)fData->pszPreview, pszPreview);
   } else {
      fData->pszPreview = 0;
   } /* endif */
   return *this;
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings::setSizeList                                           |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setSizeList (const char* pszSizeList)
{
   delete [] fData->pszPtSizeList;
   if (pszSizeList) {
      fData->pszPtSizeList = (PSZ)new char[strlen(pszSizeList)+1];
      strcpy((char*)fData->pszPtSizeList, pszSizeList);
   } else {
      fData->pszPtSizeList = 0;
   } /* endif */
   return *this;
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings::setDialogTemplate                                     |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setDialogTemplate(const IResourceId& resId)
{
   fData->usDlgId = (unsigned short)(resId.id());
   fData->hMod = resId.resourceLibrary().handle();
   return *this;
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings ::setPosition                                          |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings ::setPosition (const IPoint& position)
{
   fData->x = (short)(position.x());
   fData->y = (short)(position.y());
   return *this;
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings::setDisplayPS                                          |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setDisplayPS (const IPresSpaceHandle& hpsDisplay)
{
   fData->hpsScreen = hpsDisplay;
   return *this;
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings::setPrinterPS                                          |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setPrinterPS (const IPresSpaceHandle& hpsPrnter)
{
   fData->hpsPrinter = hpsPrnter;
   return *this;
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings::setFamily                                             |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setFamily (const char* pszFontFamily)
{
   delete [] fData->pszFamilyname;
   fData->pszFamilyname = new char[strlen(pszFontFamily) + 1];
   strcpy(fData->pszFamilyname, pszFontFamily);
   return *this;
}

/*------------------------------------------------------------------------------
| IFontDialog::Settings::setPointSize                                          |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setPointSize (unsigned long usPointSize)
{
   fData->fxPointSize = MAKEFIXED(usPointSize, 0);
   return *this;
}


//
// Implementation for IFontDialog
//

/*------------------------------------------------------------------------------
| IFontDialog::fontFamily                                                      |
------------------------------------------------------------------------------*/
IString IFontDialog::fontFamily () const
{
   IString strTemp((char*)fData->pszFamilyname);
   return strTemp;
}



/*------------------------------------------------------------------------------
| IFontDialog::~IFontDialog                                                    |
------------------------------------------------------------------------------*/
IFontDialog::~IFontDialog  ()
{
   delete [] fData->pszTitle;
   delete [] fData->pszPreview;
   delete [] fData->pszPtSizeList;
   delete [] fData->pszFamilyname;
   delete fData;
}


/*------------------------------------------------------------------------------
| IFontDialog::emHeight                                                        |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::emHeight () const
{
   return fData->lEmHeight;
}


/*------------------------------------------------------------------------------
| IFontDialog::xHeight                                                         |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::xHeight () const
{
   return fData->lXHeight;
}


/*------------------------------------------------------------------------------
| IFontDialog::externalLeading                                                 |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::externalLeading () const
{
   return fData->lExternalLeading;
}


/*------------------------------------------------------------------------------
| IFontDialog::pointSize                                                       |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::pointSize () const
{
   return FIXEDINT(fData->fxPointSize);
}


/*------------------------------------------------------------------------------
| IFontDialog::nominalPointSize                                                |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::nominalPointSize () const
{
   return fData->sNominalPointSize;
}


/*------------------------------------------------------------------------------
| IFontDialog::fontWeight                                                      |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::fontWeight () const
{
   return fData->usWeight;
}


/*------------------------------------------------------------------------------
| IFontDialog::fontWidth                                                       |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::fontWidth () const
{
   return fData->usWidth;
}


/*------------------------------------------------------------------------------
| IFontDialog::buttonPressedId                                                 |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::buttonPressedId () const
{
   return fData->lReturn;
}


/*------------------------------------------------------------------------------
| IFontDialog::isModeless                                                      |
------------------------------------------------------------------------------*/
Boolean IFontDialog::isModeless () const
{
   return fData->fl & FNTS_MODELESS;
}

/*------------------------------------------------------------------------------
| IFontDialog::pressedOK                                                       |
------------------------------------------------------------------------------*/
Boolean IFontDialog::pressedOK () const
{
   return buttonPressedId() == DID_OK;
}

/*------------------------------------------------------------------------------
| IFontDialog::returnValue                                                     |
| Returns the system return code from the font                                 |
| dialog.  FNTS_SUCCESSFUL means no error occured.                             |
| A positive return indicates an error.  The codes                             |
| are listed in the fcldlgp.h file.                                            |
------------------------------------------------------------------------------*/
long IFontDialog::returnValue () const
{
   return fData->lSRC;
}

/*------------------------------------------------------------------------------
| IFontDialog::Style  IFontDialog::defaultStyle                                |
------------------------------------------------------------------------------*/
IFontDialog::Style  IFontDialog::defaultStyle()
{
  return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IFontDialog::convertToGUIStyle                                               |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::convertToGUIStyle(const IBitFlag& guiStyle,
                                             Boolean bExtOnly) const
{
  // Obtain the style from the class (IFrameWindow) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= extendedStyle() & IS_EXTMASK;
  }
  else
  {
    // FNTS_ styles have a one-to-one correspondence to our style bits, and
    // inhabit the lower word of the base GUI style.  Therefore, obtain
    // that portion asis, masking out the upper word.
    ulStyle |= guiStyle.asUnsignedLong() & IFNTS_MASK;
  }

  return( ulStyle );
}

/*------------------------------------------------------------------------------
| IFontDialog::initialize                                                      |
------------------------------------------------------------------------------*/
void IFontDialog::initialize (IWindow*        parent,
                              IWindow*        owner,
                              const Style&    style,
                              IHandler*       handler,
                              const Settings& settings)
{
   // It's possible that the first thing an application may do is
   // create a IFontDialog. Since we deferCreation of our parent IFrameWindow
   // we should be sure that PM has been initialized.
   IThread::current().initializeGUI();

   Boolean fMadeHPS = false;

   fApplyFont = settings.fApplyFont;

   _handlerToAdd = handler;

   fData = new FONTDLG;
   memcpy(fData, settings.fData, sizeof(FONTDLG) );

   if (settings.fData->pszTitle) {
      char* src = settings.fData->pszTitle;
      fData->pszTitle = new char[strlen(src) + 1];
      strcpy(fData->pszTitle, src);
   }

   if (settings.fData->pszPreview) {
      char* src = settings.fData->pszPreview;
      fData->pszPreview = new char[strlen(src) + 1];
      strcpy(fData->pszPreview, src);
   }

   if (settings.fData->pszPtSizeList) {
      char* src = settings.fData->pszPtSizeList;
      fData->pszPtSizeList = new char[strlen(src) + 1];
      strcpy(fData->pszPtSizeList, src);
   }

   fData->pszFamilyname = new char[FACESIZE];
   fData->usFamilyBufLen = FACESIZE;
   if (settings.fData->pszFamilyname)
      strncpy(fData->pszFamilyname, settings.fData->pszFamilyname, FACESIZE);

   // Save the extended style to make sure we have a copy of it stored
   setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

   // Convert the style bits into the GUI native style settings
   fData->fl = convertToGUIStyle( style );

   if (fApplyFont) {
      fData->fl |= FNTS_INITFROMFATTRS;
      // Even with FNTS_INITFROMATTRS set, wants the name to be displayed
      // set in pszFamilyname.
      strncpy(fData->pszFamilyname, settings.fData->fAttrs.szFacename, FACESIZE);

   }

   if (fData->x == 0 && fData->y ==0)
      fData->fl |= FNTS_CENTER;

   if (fData->usDlgId) {
      fData->fl |= FNTS_CUSTOM;
   }

   fData->ulUser = (unsigned long)this;

   fData->pfnDlgProc = (FNWP*)_IFontDlgProc;

   if (!fData->hpsScreen && !fData->hpsPrinter) {
      fData->hpsScreen = WinGetPS(HWND_DESKTOP);
      fMadeHPS = TRUE;
   } /* endif */


   // Need to set a variable in IWindow that holds the default procedure
   // so that IFrameHandler will work.
   setDefaultProcedure(IPMCtlStatics::defFontDlgProcAddr());

   if (!IPMCtlStatics::fontDlg(parent == 0 ? desktopWindow()->handle() : parent->handle(),
                   owner  == 0 ? IWindowHandle(0)        : owner->handle(),
                   fData) )
   {
      ITHROWGUIERROR("WinFontDlg");
   }

   if (fMadeHPS)
      WinReleasePS(fData->hpsScreen);
}


/*------------------------------------------------------------------------------
| _IFontDlgProc                                                                |
------------------------------------------------------------------------------*/
void* _System _IFontDlgProc (HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
   /*****************************************************************/
   /* Get the "this" pointer for window object from the window word */
   /* which was set by WM_CREATE in above.                          */
   /*****************************************************************/
   FONTDLG* pfntstruct = (FONTDLG*)WinQueryWindowPtr(hwnd, QWL_USER);
   IFontDialog* pfntd = (IFontDialog*)pfntstruct->ulUser;

   if (pfntd == 0)                   // No window object pass to defWin.
       return IPMCtlStatics::defFontDlgProc(hwnd, msg, mp1, mp2);

   switch(msg)
   {
      case WM_INITDLG:

         pfntd->addDefaultHandler();    // default IFrameWindow handler needed since
                                        // we created Frame with deferCreation flag

         pfntd->addToWindowSet(pfntd, IWindowHandle(hwnd));

         if (pfntd->_handlerToAdd != 0)
            pfntd->_handlerToAdd->handleEventsFor(pfntd);

         break;

      case WM_COMMAND:
      case WM_SYSCOMMAND:
      case WM_CLOSE:
         return IPMCtlStatics::defFontDlgProc(hwnd, msg, mp1, mp2);

   } /* end switch */

  void* result;
  IEvent newEvt(hwnd, msg, mp1, mp2);
  if (pfntd->dispatch(newEvt))
     result = newEvt.result();
  else
     result = IPMCtlStatics::defFontDlgProc(hwnd, msg, mp1, mp2);

  if(msg==WM_DESTROY)
  {
     pfntd->removeFromWindowSet(pfntd);
     if (pfntd->pressedOK() && pfntd->fApplyFont)
     {
       HPS activeDevice;
       if (pfntd->fData->flFlags & FNTF_SCREENFONTSELECTED)
          activeDevice = pfntd->fData->hpsScreen;
       else if (pfntd->fData->flFlags & FNTF_PRINTERFONTSELECTED)
          activeDevice = pfntd->fData->hpsPrinter;
       else
          activeDevice = 0;

       pfntd->fApplyFont->setFontFromDialog(*pfntd, activeDevice);
     }
  }

  return result;
}
