/*******************************************************************************
* FILE NAME: iframe1.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iframnhd.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINFRAMEMGR

extern "C" {
   #include <iwindefs.h>
}

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifndef _IFRAME_
  #include <iframe.hpp>
#endif

#ifndef _IFRAMNHD_
  #include <iframnhd.hpp>
#endif

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IFRAME1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects
class IFrameWindowNotifyStatics
{
public:

~IFrameWindowNotifyStatics ()
{
  delete frameWindowNotifyHandler;
  frameWindowNotifyHandler = 0;
}
static IFrameWindowNotifyHandler
 &notificationHandler ( );

static IFrameWindowNotifyHandler
  *frameWindowNotifyHandler;
};

// Initialize static notification strings.
#pragma data_seg(ICLStaticConst)
static IFrameWindowNotifyStatics frameWindowStatics;
INotificationId const IFrameWindow::closeId = "IFrameWindow::close";
INotificationId const IFrameWindow::activateId = "IFrameWindow::activate";
INotificationId const IFrameWindow::deactivateId = "IFrameWindow::deactivate";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
IFrameWindowNotifyHandler*
  IFrameWindowNotifyStatics::frameWindowNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| IFrameWindowNotifyStatics::notificationHandler                               |
|                                                                              |
| Static function to create and return a notification handler for IFrameWindow.|
------------------------------------------------------------------------------*/
IFrameWindowNotifyHandler& IFrameWindowNotifyStatics::notificationHandler ()
{
  if (!(IFrameWindowNotifyStatics::frameWindowNotifyHandler))
  {
    IFrameWindowNotifyStatics::frameWindowNotifyHandler =
                                   new IFrameWindowNotifyHandler();
  }
  return (*IFrameWindowNotifyStatics::frameWindowNotifyHandler);
}

/*------------------------------------------------------------------------------
| IFrameWindow::enableNotification                                             |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IFrameWindow, does not send notifications to observer objects until it is    |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IFrameWindow& IFrameWindow::enableNotification (Boolean enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(IFrameWindowNotifyStatics::notificationHandler()));
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| IFrameWindowNotifyHandler::IFrameWindowNotifyHandler                         |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IFrameWindowNotifyHandler::IFrameWindowNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IFrameWindowNotifyHandler::~IFrameWindowNotifyHandler                        |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IFrameWindowNotifyHandler::~IFrameWindowNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IFrameWindowNotifyHandler::dispatchHandlerEvent                              |
|                                                                              |
| This notifies IFrameWindow Windows of the following events:                  |
|                                                                              |
| close(Id)                                                                    |
------------------------------------------------------------------------------*/

Boolean IFrameWindowNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
  IWindow* thisWindow = anEvent.window();

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      case WM_ACTIVATE:

        if (anEvent.parameter2() == thisWindow->handle())
        {
          /************************************************************/
          /* Post private message to delay notification until after   */
          /* the selection state has been modified                    */
          /************************************************************/
          thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                                 IEventParameter1( anEvent.parameter1() ),
                                 IEventParameter2( IC_NOTDEL_FRAME_ACTIVE ));
        }
        break;

      /****************************************************************/
      /* This is the delayed notification message.  Check to see which*/
      /* notification should be sent to the observers based on info   */
      /* stored in parameter 2 of the private message.  Parameter 1   */
      /* contains the information passed in the original event that   */
      /* needs to be sent with the notification                       */
      /****************************************************************/
      case IC_UM_DELAYNOTIFY:

        if (anEvent.parameter2() == IC_NOTDEL_FRAME_ACTIVE)
        {
          thisWindow->notifyObservers(INotificationEvent(
                     (anEvent.parameter1().asUnsignedLong() ?
                                 IFrameWindow::activateId :
                                 IFrameWindow::deactivateId),
                     *thisWindow, false));
        }
        break;

      default:
        break;
    }  /* end switch */
  }
  return (Inherited::dispatchHandlerEvent(anEvent));
}

