/*******************************************************************************
* FILE NAME: iframevt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iframevt.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #include <iwindefs.h>
  }

#include <iframevt.hpp>

#ifndef _IRECT_
  #include <irect.hpp>
#endif

#ifndef _IHANDLE_
  #include <ihandle.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IFRAMEVT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IFrameEvent::IFrameEvent                                                     |
------------------------------------------------------------------------------*/
IFrameEvent :: IFrameEvent ( const IEvent &baseEvent )
  : IEvent( baseEvent ),
    pFrame( (IFrameWindow*)baseEvent.window() )
  {
  }

/*------------------------------------------------------------------------------
| IFrameEvent::IFrameEvent                                                     |
|                                                                              |
| Copy constructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IFrameEvent :: IFrameEvent ( const IFrameEvent& frameEvent )
  : IEvent( frameEvent ),
    pFrame( frameEvent.pFrame )
  {
  }


/*------------------------------------------------------------------------------
| IFrameEvent::IFrameEvent::operator=                                          |
|                                                                              |
| Assignment operator                                                          |
------------------------------------------------------------------------------*/
IFrameEvent& IFrameEvent::operator= (const IFrameEvent &frameEvent)
{
  if (this != &frameEvent)
  {
    Inherited::operator= ( frameEvent );
    pFrame = frameEvent.pFrame;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IFrameEvent::~IFrameEvent                                                    |
------------------------------------------------------------------------------*/
IFrameEvent :: ~IFrameEvent ( )
  {
  }

/*------------------------------------------------------------------------------
| IFrameFormatEvent::IFrameFormatEvent                                         |
------------------------------------------------------------------------------*/
IFrameFormatEvent :: IFrameFormatEvent ( const IEvent &baseEvent )
  : IFrameEvent( baseEvent )
  {
  }

/*------------------------------------------------------------------------------
| IFrameFormatEvent::IFrameFormatEvent                                         |
|                                                                              |
| Copy constructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IFrameFormatEvent :: IFrameFormatEvent ( const IFrameFormatEvent& fmtEvent )
  : IFrameEvent( fmtEvent )
  {
  }

/*------------------------------------------------------------------------------
| IFrameFormatEvent::~IFrameFormatEvent                                        |
------------------------------------------------------------------------------*/
IFrameFormatEvent :: ~IFrameFormatEvent ( )
  {
  }

/*------------------------------------------------------------------------------
| IFrameFormatEvent::clientRect                                                |
|                                                                              |
| Return the client rectangle associated with this event.                      |
------------------------------------------------------------------------------*/
IRectangle IFrameFormatEvent :: clientRect ( ) const
  {
  RECTL
   *p = (PRECTL)(void*)( this->parameter2() );
  if ( p )
    return IRectangle( *p );
  else
    return IRectangle();
  }

/*------------------------------------------------------------------------------
| IFrameFormatEvent::setClientRect                                             |
|                                                                              |
| Sets the client rectangle value to a given argument.                         |
------------------------------------------------------------------------------*/
IFrameFormatEvent &IFrameFormatEvent :: setClientRect ( const IRectangle &aRect)
  {
  RECTL
   *clientRect = (PRECTL)(void*)( this -> parameter2() );
  if ( clientRect )
    {
    clientRect -> xLeft    = aRect.left();
    clientRect -> yBottom  = aRect.bottom();
    clientRect -> xRight   = aRect.right();
    clientRect -> yTop     = aRect.top();
    }
  return *this;
  }
