/*******************************************************************************
* FILE NAME: igbitm1.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igbitmap.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C"
  {
  #define INCL_DEV                /* Device Function definitions  */
  #define INCL_GPICONTROL         /* GPI control Functions        */
  #define INCL_GPITRANSFORMS      /* GPI transform functions      */
  #define INCL_GPICORRELATION
  #define INCL_GPIBITMAPS
  #define INCL_GPIPRIMITIVES
  #define INCL_WINPOINTERS
  #define INCL_WINSYS
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
  #include <math.h>
  #include <igbmodel.hpp>
  #undef word
  }

#include <igbitmap.hpp>
#include <ibmpdata.hpp>
#include <iexcept.hpp>
#include <igrafctx.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <iresstat.hpp>
#include <istring.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGBITM1_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IGBitmap::copyBitmap                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IGBitmap::copyBitmap( IGraphicContext& gc,
                                    unsigned long bmp, const ISize& newSize )
{
  BITMAPINFOHEADER2 bmpInfo;
  HBITMAP           hbmReturn(0);
  char*             pszError(0);
  Boolean           fEnlarge(newSize != ISize(0,0));
  unsigned          oldWidth, oldHeight;

  bmpInfo.cbFix = sizeof(BITMAPINFOHEADER2);
  if (!GpiQueryBitmapInfoHeader(bmp, &bmpInfo))
  {
    pszError = "GpiQueryBitmapInfoHeader";
  }
  else
  {
     if (fEnlarge)
     {
       oldWidth   = bmpInfo.cx;
       oldHeight  = bmpInfo.cy;
       bmpInfo.cx = newSize.width();
       bmpInfo.cy = newSize.height();
     }
     hbmReturn = GpiCreateBitmap( gc.handle(), &bmpInfo, 0, 0, 0);
     if (!hbmReturn)
       pszError = "GpiCreateBitmap";
     else
     {
       GpiSetBitmap(gc.handle(), hbmReturn);
       if (!fEnlarge)
       {
         POINTL aptl[4] = { 0,0, bmpInfo.cx - 1, bmpInfo.cy - 1,
                            0,0, bmpInfo.cx, bmpInfo.cy };
         if( GpiWCBitBlt( gc.handle(),
                          bmp, 4L, aptl, ROP_SRCCOPY, BBO_IGNORE) == GPI_ERROR)
           pszError = "GpiWCBitBlt";
       }
       else
       {
         POINTL aptl[4] = { 0,0, newSize.width() - 1, newSize.height() - 1,
                            0,0, oldWidth, oldHeight };
         if( GpiWCBitBlt( gc.handle(),
                          bmp, 4L, aptl, ROP_SRCCOPY, BBO_IGNORE) == GPI_ERROR)
           pszError = "GpiWCBitBlt";
       }
       GpiSetBitmap(gc.handle(), 0);
     } // else !GpiCreateBitmap
  } // else !GpiQueryBitmapInfoHeader

  if(pszError)
  {
    ITHROWGUIERROR(IString(pszError)+"Id: "+ IString(bmp)+" ");
  }
  return (unsigned long)hbmReturn;
}

