/*******************************************************************************
* FILE NAME: igbitm2.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igbitmap.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C"
  {
  #define INCL_DEV                /* Device Function definitions  */
  #define INCL_GPICONTROL         /* GPI control Functions        */
  #define INCL_GPITRANSFORMS      /* GPI transform functions      */
  #define INCL_GPICORRELATION
  #define INCL_GPIBITMAPS
  #define INCL_GPIPRIMITIVES
  #define INCL_WINPOINTERS
  #define INCL_WINSYS
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
  #include <math.h>
  #include <igbmodel.hpp>
  #undef word
  }

#include <igbitmap.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <iexcept.hpp>
#include <igrafctx.hpp>
#include <istring.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGBITM2_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IGBitmap::tryToLoadBitmap                                                    |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IGBitmap::tryToLoadBitmap( unsigned long bitmapId,
                                         const IModuleHandle& moduleHandle,
                                         const ISize&  bitmapSize )
{
  IGraphicContext gc;
  unsigned long   hbmReturn(0);
  HBITMAP         hbmOld(0);

  hbmOld = GpiLoadBitmap(gc.handle(), moduleHandle, bitmapId,
                         bitmapSize.width(), bitmapSize.height());

  // If load succeeded, make a copy of the bitmap and delete
  if (hbmOld)
  {
     hbmReturn = copyBitmap(gc, hbmOld, ISize(0,0));
     if(!GpiDeleteBitmap(hbmOld))
        ITHROWGUIERROR(IString("GpiDeleteBitmap")+"Id: "+ IString(bitmapId)+" ");
  }

  return hbmReturn;
}
