/*******************************************************************************
* FILE NAME: igraphdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igraphdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
#define INCL_WINBUTTONS
#define INCL_WINENTRYFIELDS
#define INCL_WINLISTBOXES
#define INCL_WINSTDCNR
#define INCL_WINSTDVALSET
#define INCL_WINWINDOWMGR

extern "C" {
  #include <iwindefs.h>
  #include <stdlib.h>
}

#include <igraphdr.hpp>
#include <igraphbt.hpp>
#include <ievent.hpp>
#include <iwindow.hpp>
#include <iiconctl.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGRAPHDR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IGraphicPushButtonHandler::IGraphicPushButtonHandler                         |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IGraphicPushButtonHandler::IGraphicPushButtonHandler ( )
{ ; }


/*------------------------------------------------------------------------------
| IGraphicPushButtonHandler::~IGraphicPushButtonHandler                        |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IGraphicPushButtonHandler::~IGraphicPushButtonHandler ( )
{ ; }


/************************************************************************/
/* dispatchHandler Event - Dispatch command functions for this handler. */
/************************************************************************/
IBase::Boolean IGraphicPushButtonHandler :: dispatchHandlerEvent(IEvent & evt)
{
   IGraphicPushButton* graphicbutton= (IGraphicPushButton*)evt.window();
   switch (evt.eventId())
   {
      case BM_SETHILITE:
         {
           IPoint location = graphicbutton->graphicWindow().position();
           if (evt.parameter1()==0)
              {
              location.setX(location.x()-1);
              location.setY(location.y()+1);
              // There is an OS/2 bug that does not repaint the top
              // and left outermost edge of the graphic when no-border
              // and no-margin is set.  So... we need to be sure that
              // the entire graphic is invalidated.
              if ( !graphicbutton->hasBorder() &&
                 ( graphicbutton->marginSize() == ISize(0,0) ))
                {
                   IRectangle rect1 = graphicbutton->graphicWindow().rect();
                   IRectangle rect2(rect1.left(),  rect1.top(),
                                    rect1.right()+1, rect1.top()+1);
                   graphicbutton->graphicWindow().refresh(rect2);
                   rect2 = IRectangle(rect1.left(),  rect1.bottom(),
                                      rect1.left()+1, rect1.top()+1);
                   graphicbutton->graphicWindow().refresh(rect2);
                }
              }
           else
              {
              location.setX(location.x()+1);
              location.setY(location.y()-1);
              } /* endif */
           graphicbutton->graphicWindow().moveTo(location);
         }
      case WM_SETWINDOWPARAMS:
      case BM_SETDEFAULT:
      case WM_SYSVALUECHANGED:
         graphicbutton->setStyle(graphicbutton->style() |
                                 IWindow::clipChildren.asUnsignedLong());
         defaultProcedure(evt);
         graphicbutton->setStyle(graphicbutton->style() &
                                 ~IWindow::clipChildren.asUnsignedLong());
         return true;
         break;
      default:
         return false;
      break;
   } /* endswitch */
   return false;
}

