/*******************************************************************************
* FILE NAME: igrect.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igrect.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <igrect.hpp>
#include <igrafctx.hpp>
#include <igbundle.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGRECT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IGRectangle::IGRectangle                                                     |
| Copy constructor.                                                            |
|                                                                              |
------------------------------------------------------------------------------*/
IGRectangle::IGRectangle( const IGRectangle& rect )
  : IGRectangle::Inherited(rect), fRect( rect.fRect ), fRounding(rect.fRounding)
{
}

/*------------------------------------------------------------------------------
| IGRectangle::IGRectangle                                                     |
| Constructor from a IRectangle object rect.                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IGRectangle::IGRectangle( const IRectangle& rect )
  : IGRectangle::Inherited(), fRect( rect ), fRounding(0,0)
{
}

/*------------------------------------------------------------------------------
| IGRectangle::~IGRectangle                                                    |
| Virtual destructor.                                                          |
|                                                                              |
------------------------------------------------------------------------------*/
IGRectangle::~IGRectangle()
{
}

/*------------------------------------------------------------------------------
| IGRectangle::drawOn                                                          |
| Draw itself on a given context.                                              |
| Return a reference of this object.                                           |
------------------------------------------------------------------------------*/
IGRectangle& IGRectangle::drawOn( IGraphicContext& graphicContext )
{
  graphicContext.draw( *this );
  return *this;
}

/*------------------------------------------------------------------------------
| IGRectangle::setRounding                                                     |
| Set the horizontal and vertical axes of the ellipse used to round the corner |
| of the rectangle.  Return a reference of this object.                        |
------------------------------------------------------------------------------*/
IGRectangle& IGRectangle::setRounding( const IPair& ellipseAxis )
{
  fRounding = ellipseAxis;
  return *this;
}

/*------------------------------------------------------------------------------
| IGRectangle::rounding                                                        |
| Get the horizontal and vertical axes of the ellipse used to round the corner |
| of the rectangle.                                                            |                                                                              |
------------------------------------------------------------------------------*/
IPair IGRectangle::rounding() const
{
  return fRounding;
}

/*------------------------------------------------------------------------------
| IGRectangle::enclosingRect                                                   |
| Get the IRectangle data member.                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IRectangle IGRectangle::enclosingRect( ) const
{
  return fRect;
}

/*------------------------------------------------------------------------------
| IGRectangle::setRectangle                                                    |
| Set the IRectangle data member.                                              |
| Return a reference of this object.                                           |
------------------------------------------------------------------------------*/
IGRectangle& IGRectangle::setEnclosingRect( const IRectangle& rectangle )
{
  fRect = rectangle;
  return *this;
}

/*------------------------------------------------------------------------------
| IGRectangle::operator=                                                       |
| Assignment operator.                                                         |
| Return a reference of this object.                                           |
------------------------------------------------------------------------------*/
IGRectangle& IGRectangle::operator= ( const IGRectangle& gRectangle )
{
  // If assignment to itself, return a reference to itself.
  if ( this == &gRectangle )
    return *this;

  // Call base class' overrided member function.
  this->Inherited::operator=(gRectangle);
  // Member-wise assignment.
  fRect     = gRectangle.fRect;
  fRounding = gRectangle.fRounding;
  return *this;
}

/*------------------------------------------------------------------------------
| IGRectangle::operator==                                                      |
| Boolean equal operator.                                                      |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IGRectangle::operator== ( const IGRectangle& gRectangle ) const
{
  // A member-wise comparison.
  return (fRect == gRectangle.fRect && fRounding == gRectangle.fRounding &&
          this->Inherited::operator==(gRectangle));
}

/*------------------------------------------------------------------------------
| IGRectangle::operator!=                                                      |
| Boolean unequal operator.                                                    |
|                                                                              |
------------------------------------------------------------------------------*/
IBase::Boolean IGRectangle::operator!= ( const IGRectangle& gRectangle ) const
{
  return !(*this == gRectangle);
}
