/*******************************************************************************
* FILE NAME: ihelp0.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ihelp.hpp.  (platform independent)                                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Note we depend on ireslib.cpp being first.
#pragma priority( -2147482102 )
#define INCL_WINHELP
#define INCL_WINMESSAGEMGR
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
}

#include <ihelp.hpp>
#include <ihelpprv.hpp>
#include <ihelpsta.hpp>
#include <icconst.h>
#include <iexcept.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <iwindow.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IHELP0_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IHelpWindowData::IHelpWindowData                                             |
|                                                                              |
------------------------------------------------------------------------------*/
IHelpWindowData::IHelpWindowData ( )
  : helpHandle( 0 ),
    helpLibraries( "" ),
    pHTResLib( 0 ),
    pMAResLib( 0 )
{ }

/*------------------------------------------------------------------------------
| IHelpWindowData::~IHelpWindowData                                            |
|                                                                              |
------------------------------------------------------------------------------*/
IHelpWindowData::~IHelpWindowData ( )
{
  if ( pHTResLib )
     delete pHTResLib;
  if ( pMAResLib )
     delete pMAResLib;
}


/*------------------------------------------------------------------------------
| IHelpWindow::Settings::Settings                                              |
|  Create a Settings object and initialize.                                    |
------------------------------------------------------------------------------*/
IHelpWindow::Settings::Settings ( )
  : title         (""),
    tutorial      (""),
    helpLibraries (""),
    HTResLib      (""),
    MAResLib      ("")
{
  usingHelpId   = 0;
  helpTableId   = 0;
  menuBarId     = 0;
  acceleratorId = 0;
  pHelpSettings = 0;
}


// IHelpWindow constructors and destructor in ihelp1.cpp.

/*------------------------------------------------------------------------------
| IHelpWindow::isIPFCompatible                                                 |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpWindow :: isIPFCompatible( ) const
{
  return true;
}


/*------------------------------------------------------------------------------
| IHelpWindow::setTitle                                                        |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::setTitle ( const char* title )
{
  this->sendEvent( HM_SET_HELP_WINDOW_TITLE,
                   IEventParameter1( (char*)title ), 0 );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::setTitle                                                        |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::setTitle ( const IResourceId& id )
{
  IString temp = id.resourceLibrary().loadString( id );
  return this->setTitle( temp );
}

/*------------------------------------------------------------------------------
| IHelpWindow::setHelpTable                                                    |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::setHelpTable ( const IResourceId& helpTable )
{
  helpTable.resourceLibrary().loadHelpTable( this, helpTable.id() );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::showPanelIds                                                    |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::showPanelIds ( Boolean visibleId )
{
  this->sendEvent( HM_SET_SHOW_PANEL_ID,
                   visibleId ? CMIC_SHOW_PANEL_ID : CMIC_HIDE_PANEL_ID,
                   0 );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::hidePanelIds                                                    |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::hidePanelIds ( )
{
  return this->showPanelIds( false );
}

// sizeTo, moveTo, moveSizeTo in ihelp1.cpp.

/*------------------------------------------------------------------------------
| IHelpWindow::handle                                                          |
------------------------------------------------------------------------------*/
IWindowHandle IHelpWindow::handle ( ) const
{
  return fHelpWindowData->helpHandle;
}

/*------------------------------------------------------------------------------
| IHelpWindow::addLibraries                                                    |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::addLibraries ( const char* helpLibraryNames )
{
  // Since the OS/2 message actually replaces the previous libraries,
  // we must maintain a list and keep adding to it, separating them
  // with a blank space.  If they pass NULL then we clear out all previously
  // set libraries.  The old list of libraries is saved in case an error
  // occurs (in which case we want to reset them back to what they were).

  IString oldLibraries;
  if ( helpLibraryNames && (*helpLibraryNames != '\0') )
  {
     oldLibraries = fHelpWindowData->helpLibraries;
     if ( fHelpWindowData->helpLibraries != "0" )
     {
        fHelpWindowData->helpLibraries += " ";
        fHelpWindowData->helpLibraries += helpLibraryNames;
     }
     else
        fHelpWindowData->helpLibraries = helpLibraryNames;
  }
  else
     fHelpWindowData->helpLibraries = 0;

  if ( this->sendEvent( HM_SET_HELP_LIBRARY_NAME,
                        IEventParameter1( (char*)fHelpWindowData->helpLibraries ),
                        IEventParameter2( 0 )) != 0 )
  {
     // Since an error occured, we need to reset the libraries and
     // throw an exception.
     if ( fHelpWindowData->helpLibraries != "0" )
     {
        fHelpWindowData->helpLibraries = oldLibraries;
        this->sendEvent( HM_SET_HELP_LIBRARY_NAME,
                         IEventParameter1( (char*)fHelpWindowData->helpLibraries ),
                         IEventParameter2( 0 ));
        ITHROWLIBRARYERROR( IC_HMERR_OPEN_LIB_FILE,
                            IErrorInfo::invalidRequest,
                            IException::recoverable );
     }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::hide                                                            |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::hide ( )
{
  this->sendEvent( HM_DISMISS_WINDOW,
                   IEventParameter1( 0 ),
                   IEventParameter2( 0 ) );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::show                                                            |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::show ( Boolean )
{
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::show                                                            |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::show ( HelpType helpType )
{
  switch ( helpType )
  {
     case index:
        this->sendEvent( HM_HELP_INDEX,
                         IEventParameter1( 0 ),
                         IEventParameter2( 0 ) );
        break;
     case contents:
        this->sendEvent( HM_HELP_CONTENTS,
                         IEventParameter1( 0 ),
                         IEventParameter2( 0 ) );
        break;
     case general:
        this->sendEvent( HM_GENERAL_HELP,
                         IEventParameter1( 0 ),
                         IEventParameter2( 0 ) );
        break;
     case keys:
        this->sendEvent( HM_KEYS_HELP,
                         IEventParameter1( 0 ),
                         IEventParameter2( 0 ) );
        break;
     case usingHelp:
        this->sendEvent( HM_DISPLAY_HELP,
                         IEventParameter1( 0 ),
                         IEventParameter2( 0 ) );
        break;
  } /* endswitch */
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::show                                                            |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::show ( const IResourceId& panelId )
{
  if ( panelId.id() )
  {
     this->sendEvent( HM_DISPLAY_HELP,
                      IEventParameter1( panelId.id() ),
                      IEventParameter2( HM_RESOURCEID ) );

  }
  else
  { // else - display system default help - "Using Help"
     this->sendEvent( HM_DISPLAY_HELP,
                      IEventParameter1( 0 ),
                      IEventParameter2( 0 ) );
  } /* endif */

  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::show                                                            |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::show ( const char* panelName )
{
  if ( panelName )
  {
     this->sendEvent( HM_DISPLAY_HELP,
                      IEventParameter1( (char*)panelName ),
                      IEventParameter2( HM_PANELNAME ) );
  }
  else
  {  // else - display system default help
     this->sendEvent( HM_DISPLAY_HELP,
                      IEventParameter1( 0 ),
                      IEventParameter2( 0 ) );
  } /* endif */
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::setUsingHelp                                                    |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::setUsingHelp ( unsigned long id )
{
  this->sendEvent( HM_REPLACE_USING_HELP,
                   IEventParameter1( id ),
                   IEventParameter2( 0 ) );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::indexWindow                                                     |
------------------------------------------------------------------------------*/
IWindowHandle IHelpWindow::indexWindow ( ) const
{
  unsigned long handle =
                  this->sendEvent( HM_QUERY,
                                   IEventParameter1( 0, HMQW_INDEX ),
                                   IEventParameter2( 0 ) );
  return IWindowHandle( handle );
}

/*------------------------------------------------------------------------------
| IHelpWindow::contentsWindow                                                  |
------------------------------------------------------------------------------*/
IWindowHandle IHelpWindow::contentsWindow ( ) const
{
  unsigned long handle =
                  this->sendEvent( HM_QUERY,
                                   IEventParameter1( 0, HMQW_TOC ),
                                   IEventParameter2( 0 ) );
  return IWindowHandle( handle );
}

/*------------------------------------------------------------------------------
| IHelpWindow::searchListWindow                                                |
------------------------------------------------------------------------------*/
IWindowHandle IHelpWindow::searchListWindow ( ) const
{
  unsigned long handle =
                  this->sendEvent( HM_QUERY,
                                   IEventParameter1( 0, HMQW_SEARCH ),
                                   IEventParameter2( 0 ) );
  return IWindowHandle( handle );
}

/*------------------------------------------------------------------------------
| IHelpWindow::viewedPagesWindow                                               |
------------------------------------------------------------------------------*/
IWindowHandle IHelpWindow::viewedPagesWindow ( ) const
{
  unsigned long handle =
                  this->sendEvent( HM_QUERY,
                                   IEventParameter1( 0, HMQW_VIEWPAGES ),
                                   IEventParameter2( 0 ) );
  return IWindowHandle( handle );
}

/*------------------------------------------------------------------------------
| IHelpWindow::communicationWindow                                             |
------------------------------------------------------------------------------*/
IWindowHandle IHelpWindow::communicationWindow ( ) const
{
  unsigned long handle =
                  this->sendEvent( HM_QUERY,
                                   IEventParameter1( 0, HMQW_OBJCOM_WINDOW ),
                                   IEventParameter2( 0 ) );
  return IWindowHandle( handle );
}

/*------------------------------------------------------------------------------
| IHelpWindow::coverPageWindow                                                 |
------------------------------------------------------------------------------*/
IWindowHandle IHelpWindow::coverPageWindow ( ) const
{
  unsigned long handle =
                  this->sendEvent( HM_QUERY,
                                   IEventParameter1( 0, HMQW_COVERPAGE ),
                                   IEventParameter2( 0 ) );
  return IWindowHandle( handle );
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setTitle                                              |
------------------------------------------------------------------------------*/
IHelpWindow::Settings& IHelpWindow::Settings::setTitle ( const char* title )
{
  this->title = IString( title );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setTitle                                              |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setTitle ( const IResourceId& titleId )
{
  this->title = titleId.resourceLibrary().loadString( titleId );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setHelpResLibrary                                     |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setHelpResLibrary ( const char* helpTableResLibrary )
{
  this->HTResLib = IString( helpTableResLibrary );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setAccelResLibrary                                    |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setAccelResLibrary ( const char* menuAccelResLibrary )
{
  this->MAResLib = IString( menuAccelResLibrary );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setTutorial                                           |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setTutorial ( const char* tutorial )
{
  this->tutorial = IString( tutorial );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setLibraries                                          |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setLibraries ( const char* helpLibraryNames )
{
  this->helpLibraries = IString( helpLibraryNames );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setHelpTable                                          |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setHelpTable ( unsigned long helpTable )
{
  this->helpTableId = helpTable;
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setUsingHelp                                          |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setUsingHelp ( unsigned long helpId )
{
  this->usingHelpId = helpId;
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setMenuBar                                            |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setMenuBar ( unsigned long menubar )
{
  this->menuBarId = menubar;
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::Settings::setAccelerator                                        |
------------------------------------------------------------------------------*/
IHelpWindow::Settings&
  IHelpWindow::Settings::setAccelerator ( unsigned long accelerator )
{
  this->acceleratorId = accelerator;
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::helpWindow                                                      |
------------------------------------------------------------------------------*/
IHelpWindow* IHelpWindow::helpWindow( const IWindow* window )
{
  if (window==0)
    return 0;

  IWindowHandle handle = IHelpStatics::queryHelpInstance( window->handle() );
  if (handle == 0)
     return 0;
  else
     return (IHelpWindow*)( IWindow::windowWithHandle( handle ) );
}
