/*******************************************************************************
* FILE NAME: ihelp1.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ihelp.hpp.  (platform dependent)                                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Note we depend on ireslib.cpp being first
#pragma priority( -2147482102 )
#define INCL_WINHELP
#define INCL_WININPUT
#define INCL_WINMESSAGEMGR
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
}

#include <ihelp.hpp>
#include <ihelpprv.hpp>
#include <ihelpsta.hpp>
#include <icoordsy.hpp>
#include <iframe.hpp>
#include <ipoint.hpp>
#include <iprocadr.hpp>
#include <irect.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <ithread.hpp>
#include <iwindow.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IHELP1_CPP_
  #include <ipagetun.h>
#endif

//***************************************************************
// Public styles.
//***************************************************************
#pragma data_seg(ICLStaticConst)
const IHelpWindow::Style
  IHelpWindow::noStyle            = 0,
  IHelpWindow::classDefaultStyle  = 0,
  IHelpWindow::ipfCompatible      = 0x00000001;
#pragma data_seg()

#pragma data_seg(ICLNonConst)
  IHelpWindow::Style IHelpWindow::currentDefaultStyle = IHelpWindow::ipfCompatible;
#pragma data_seg()


/*------------------------------------------------------------------------------
| IHelpWindow::IHelpWindow                                                     |
|  Initialize the PM help structure and create an instance of the help         |
|  facility.                                                                   |
------------------------------------------------------------------------------*/
IHelpWindow::IHelpWindow ( IFrameWindow* associatedWindow )
            : fHelpWindowData ( new IHelpWindowData )
{
  HELPINIT   helpstruct;
  helpstruct.cb              = sizeof(HELPINIT);
  helpstruct.ulReturnCode    = 0;
  helpstruct.pszTutorialName = 0;
  helpstruct.phtHelpTable    = 0;

  helpstruct.hmodHelpTableModule =
                  IApplication::current().userResourceLibrary().handle();
  helpstruct.idAccelTable        = 0;
  helpstruct.idActionBar         = 0;
  helpstruct.pszHelpWindowTitle  = 0;
  helpstruct.fShowPanelId        = CMIC_HIDE_PANEL_ID;
  helpstruct.pszHelpLibraryName  = 0;

  fHelpWindowData->helpHandle =
    IHelpStatics::createHelpInstance(IThread::current().anchorBlock(),
                                     &helpstruct);
  if (!fHelpWindowData->helpHandle)
  {
     ITHROWGUIERROR("WinCreateHelpInstance");
  }

  if ( associatedWindow )
  {
     setAssociatedWindow( associatedWindow );
  }

  startHandlingEventsFor(fHelpWindowData->helpHandle);
}


/*------------------------------------------------------------------------------
| IHelpWindow::IHelpWindow                                                     |
|  Initialize the PM help structure and create an instance of the help         |
|  facility.                                                                   |
------------------------------------------------------------------------------*/
IHelpWindow::IHelpWindow ( const IResourceId& helpTable,
                           IFrameWindow* associatedWindow )
            : fHelpWindowData ( new IHelpWindowData )
{
  HELPINIT   helpstruct;
  helpstruct.cb              = sizeof(HELPINIT);
  helpstruct.ulReturnCode    = 0;
  helpstruct.pszTutorialName = 0;
  helpstruct.phtHelpTable    = (PHELPTABLE)MAKELONG(
                                          (unsigned long)(helpTable.id() ),
                                          0xffff);

  helpstruct.hmodHelpTableModule =
                  IApplication::current().userResourceLibrary().handle();
  helpstruct.idAccelTable        = 0;
  helpstruct.idActionBar         = 0;
  helpstruct.pszHelpWindowTitle  = 0;
  helpstruct.fShowPanelId        = CMIC_HIDE_PANEL_ID;
  helpstruct.pszHelpLibraryName  = 0;

  fHelpWindowData->helpHandle =
    IHelpStatics::createHelpInstance(IThread::current().anchorBlock(),
                                     &helpstruct);
  if (!fHelpWindowData->helpHandle)
  {
     ITHROWGUIERROR("WinCreateHelpInstance");
  }

  if ( associatedWindow )
  {
     setAssociatedWindow( associatedWindow );
  }

  startHandlingEventsFor(fHelpWindowData->helpHandle);
}


/*------------------------------------------------------------------------------
| IHelpWindow::IHelpWindow                                                     |
|  Initialize the PM help structure from the given Settings class and          |
|  create an instance of the help facility.                                    |
------------------------------------------------------------------------------*/
IHelpWindow::IHelpWindow( const Settings& settings,
                          IFrameWindow* associatedWindow )
            : fHelpWindowData ( new IHelpWindowData )
{
  HELPINIT   helpstruct;
  helpstruct.cb              = sizeof(HELPINIT);
  helpstruct.ulReturnCode    = 0;
  helpstruct.pszTutorialName = settings.tutorial.length() ?
                                 (char*)(settings.tutorial) : 0;
  helpstruct.phtHelpTable    = (PHELPTABLE)MAKELONG(
                                 (unsigned long)(settings.helpTableId),
                                 0xffff);

  if ( settings.HTResLib.length() )
  {
     this->fHelpWindowData->pHTResLib =
              new IDynamicLinkLibrary( (char*)settings.HTResLib );
     helpstruct.hmodHelpTableModule = this->fHelpWindowData->pHTResLib->handle();
  }
  else
  {
     this->fHelpWindowData->pHTResLib = 0;
     helpstruct.hmodHelpTableModule =
                  IApplication::current().userResourceLibrary().handle();
  }

  if ( settings.MAResLib.length() )
  {
     this->fHelpWindowData->pMAResLib =
              new IDynamicLinkLibrary( (char*)settings.MAResLib );
     helpstruct.hmodAccelActionBarModule = this->fHelpWindowData->pMAResLib->handle();
  }
  else
  {
     this->fHelpWindowData->pMAResLib = 0;
     helpstruct.hmodAccelActionBarModule = 0;
  }

  helpstruct.idAccelTable       = settings.acceleratorId;
  helpstruct.idActionBar        = settings.menuBarId;
  helpstruct.pszHelpWindowTitle = (char*)settings.title;
  helpstruct.fShowPanelId       = CMIC_HIDE_PANEL_ID;
  helpstruct.pszHelpLibraryName = (char*)(settings.helpLibraries);

  //Store the initial libraries so we can concatenate when using addLibraries()
  fHelpWindowData->helpLibraries = settings.helpLibraries;

  fHelpWindowData->helpHandle =
    IHelpStatics::createHelpInstance(IThread::current().anchorBlock(),
                                     &helpstruct);
  if (!fHelpWindowData->helpHandle)
  {
     ITHROWGUIERROR("WinCreateHelpInstance");
  }
  startHandlingEventsFor(fHelpWindowData->helpHandle);

  if ( this->fHelpWindowData->pHTResLib )
     this->fHelpWindowData->pHTResLib->loadHelpTable( this,
                                            settings.helpTableId);

  if ( associatedWindow )
     setAssociatedWindow( associatedWindow );

  if (settings.usingHelpId)
     this->sendEvent(HM_REPLACE_USING_HELP,
                     IEventParameter1(settings.usingHelpId),
                     IEventParameter2(0) );
}


/*------------------------------------------------------------------------------
| IHelpWindow::IHelpWindow                                                     |
|  Initialize the PM help structure and create an instance of the help         |
|  facility.                                                                   |
------------------------------------------------------------------------------*/
IHelpWindow::IHelpWindow ( IFrameWindow*             associatedWindow,
                           const IHelpWindow::Style& style )
            : fHelpWindowData ( new IHelpWindowData )
{
  HELPINIT   helpstruct;
  helpstruct.cb              = sizeof(HELPINIT);
  helpstruct.ulReturnCode    = 0;
  helpstruct.pszTutorialName = 0;
  helpstruct.phtHelpTable    = 0;

  helpstruct.hmodHelpTableModule =
                  IApplication::current().userResourceLibrary().handle();
  helpstruct.idAccelTable        = 0;
  helpstruct.idActionBar         = 0;
  helpstruct.pszHelpWindowTitle  = 0;
  helpstruct.fShowPanelId        = CMIC_HIDE_PANEL_ID;
  helpstruct.pszHelpLibraryName  = 0;

  fHelpWindowData->helpHandle =
    IHelpStatics::createHelpInstance(IThread::current().anchorBlock(),
                                     &helpstruct);
  if (!fHelpWindowData->helpHandle)
  {
     ITHROWGUIERROR("WinCreateHelpInstance");
  }

  if ( associatedWindow )
  {
     setAssociatedWindow( associatedWindow );
  }

  startHandlingEventsFor(fHelpWindowData->helpHandle);
}


/*------------------------------------------------------------------------------
| IHelpWindow::IHelpWindow                                                     |
|  Initialize the PM help structure and create an instance of the help         |
|  facility.                                                                   |
------------------------------------------------------------------------------*/
IHelpWindow::IHelpWindow ( const IResourceId& helpTable,
                           IFrameWindow* associatedWindow,
                           const IHelpWindow::Style& style )
            : fHelpWindowData ( new IHelpWindowData )
{
  HELPINIT   helpstruct;
  helpstruct.cb              = sizeof(HELPINIT);
  helpstruct.ulReturnCode    = 0;
  helpstruct.pszTutorialName = 0;
  helpstruct.phtHelpTable    = (PHELPTABLE)MAKELONG(
                                          (unsigned long)(helpTable.id() ),
                                          0xffff);

  helpstruct.hmodHelpTableModule =
                  IApplication::current().userResourceLibrary().handle();
  helpstruct.idAccelTable        = 0;
  helpstruct.idActionBar         = 0;
  helpstruct.pszHelpWindowTitle  = 0;
  helpstruct.fShowPanelId        = CMIC_HIDE_PANEL_ID;
  helpstruct.pszHelpLibraryName  = 0;

  fHelpWindowData->helpHandle =
    IHelpStatics::createHelpInstance(IThread::current().anchorBlock(),
                                     &helpstruct);
  if (!fHelpWindowData->helpHandle)
  {
     ITHROWGUIERROR("WinCreateHelpInstance");
  }

  if ( associatedWindow )
  {
     setAssociatedWindow( associatedWindow );
  }

  startHandlingEventsFor(fHelpWindowData->helpHandle);
}


/*------------------------------------------------------------------------------
| IHelpWindow::IHelpWindow                                                     |
|  Initialize the PM help structure from the given Settings class and          |
|  create an instance of the help facility.                                    |
------------------------------------------------------------------------------*/
IHelpWindow::IHelpWindow( const Settings& settings,
                          IFrameWindow* associatedWindow,
                          const IHelpWindow::Style& style )
            : fHelpWindowData ( new IHelpWindowData )
{
  HELPINIT   helpstruct;
  helpstruct.cb              = sizeof(HELPINIT);
  helpstruct.ulReturnCode    = 0;
  helpstruct.pszTutorialName = settings.tutorial.length() ?
                                 (char*)(settings.tutorial) : 0;
  helpstruct.phtHelpTable    = (PHELPTABLE)MAKELONG(
                                 (unsigned long)(settings.helpTableId),
                                 0xffff);

  if ( settings.HTResLib.length() )
  {
     this->fHelpWindowData->pHTResLib =
              new IDynamicLinkLibrary( (char*)settings.HTResLib );
     helpstruct.hmodHelpTableModule = this->fHelpWindowData->pHTResLib->handle();
  }
  else
  {
     this->fHelpWindowData->pHTResLib = 0;
     helpstruct.hmodHelpTableModule =
                  IApplication::current().userResourceLibrary().handle();
  }

  if ( settings.MAResLib.length() )
  {
     this->fHelpWindowData->pMAResLib =
              new IDynamicLinkLibrary( (char*)settings.MAResLib );
     helpstruct.hmodAccelActionBarModule = this->fHelpWindowData->pMAResLib->handle();
  }
  else
  {
     this->fHelpWindowData->pMAResLib = 0;
     helpstruct.hmodAccelActionBarModule = 0;
  }

  helpstruct.idAccelTable       = settings.acceleratorId;
  helpstruct.idActionBar        = settings.menuBarId;
  helpstruct.pszHelpWindowTitle = (char*)settings.title;
  helpstruct.fShowPanelId       = CMIC_HIDE_PANEL_ID;
  helpstruct.pszHelpLibraryName = (char*)(settings.helpLibraries);

  //Store the initial libraries so we can concatenate when using addLibraries()
  fHelpWindowData->helpLibraries = settings.helpLibraries;

  fHelpWindowData->helpHandle =
    IHelpStatics::createHelpInstance(IThread::current().anchorBlock(),
                                     &helpstruct);
  if (!fHelpWindowData->helpHandle)
  {
     ITHROWGUIERROR("WinCreateHelpInstance");
  }
  startHandlingEventsFor(fHelpWindowData->helpHandle);

  if ( this->fHelpWindowData->pHTResLib )
     this->fHelpWindowData->pHTResLib->loadHelpTable( this,
                                            settings.helpTableId);

  if ( associatedWindow )
     setAssociatedWindow( associatedWindow );

  if (settings.usingHelpId)
     this->sendEvent(HM_REPLACE_USING_HELP,
                     IEventParameter1(settings.usingHelpId),
                     IEventParameter2(0) );
}


/*------------------------------------------------------------------------------
| IHelpWindow::IHelpWindow                                                     |
|  Create an IHelpWindow object from an existing window handle.                |
------------------------------------------------------------------------------*/
IHelpWindow::IHelpWindow(const IWindowHandle& helpWindowHandle)
            : fHelpWindowData ( new IHelpWindowData )
{
  IASSERTPARM(helpWindowHandle!=0);

  fHelpWindowData->helpHandle = helpWindowHandle;
  setAutoDestroyWindow(false);
  startHandlingEventsFor(fHelpWindowData->helpHandle);
}


/*------------------------------------------------------------------------------
| IHelpWindow::~IHelpWindow                                                    |
------------------------------------------------------------------------------*/
IHelpWindow::~IHelpWindow()
{
  if ( fHelpWindowData->helpHandle && !deleteIsInProcess() )
     IHelpStatics::destroyHelpInstance( fHelpWindowData->helpHandle );

  delete fHelpWindowData;
}

/*------------------------------------------------------------------------------
| IHelpWindow::sizeTo                                                          |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::sizeTo(const ISize& newSize)
{
  RECTL   rectl;
  IWindowHandle hwnd = coverPageWindow();
  if ( !hwnd )
  {
     ITHROWLIBRARYERROR(IC_NO_HELP_COVERPAGE, IErrorInfo::invalidRequest,
                        IException::recoverable);
  }
  WinQueryWindowRect(hwnd, &rectl);
  WinMapWindowPoints(hwnd, HWND_DESKTOP, (PPOINTL)&rectl, 2);

  IRectangle newRect =  ICoordinateSystem::convertToApplication(
                          IRectangle(rectl),
                          desktopWindow()->size() ) ;

  moveSizeTo( newRect.sizeTo( newSize ) );

  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::moveTo                                                          |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::moveTo(const IPoint& newPos)
{
  RECTL   rectl;
  IWindowHandle hwnd = coverPageWindow();
  if ( !hwnd )
  {
     ITHROWLIBRARYERROR(IC_NO_HELP_COVERPAGE, IErrorInfo::invalidRequest,
                        IException::recoverable);
  }
  WinQueryWindowRect(hwnd, &rectl);

  /* Note: WinQueryWindowRect always returns bottom left = (0,0)   */
  rectl.xLeft    = newPos.x();
  rectl.yBottom  = newPos.y();
  rectl.xRight  += newPos.x();
  rectl.yTop    += newPos.y();

  moveSizeTo( IRectangle( rectl ) );

  return *this;
}


/*------------------------------------------------------------------------------
| IHelpWindow::moveSizeTo                                                      |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::moveSizeTo(const IRectangle& newRect)
{
  RECTL rectl = ICoordinateSystem::convertToNative (
                  newRect,
                  desktopWindow()->size() ).asRECTL();
  this->sendEvent(HM_SET_COVERPAGE_SIZE,
                  IEventParameter1(&rectl),
                  IEventParameter2(0) );
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::setActiveWindow                                                 |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::setActiveWindow( IFrameWindow* activeWindow,
                                           IFrameWindow* relativeWindow )
{
  if (activeWindow)
    if (relativeWindow)
       this->sendEvent(HM_SET_ACTIVE_WINDOW,
                       IEventParameter1( activeWindow->handle() ),
                       IEventParameter2( relativeWindow->handle() ) );
    else
       this->sendEvent(HM_SET_ACTIVE_WINDOW,
                       IEventParameter1( activeWindow->handle() ),
                       IEventParameter2( HWND_PARENT ) );
  else
     this->sendEvent(HM_SET_ACTIVE_WINDOW,
                     IEventParameter1(0),
                     IEventParameter2(0) );

  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::setAssociatedWindow                                             |
------------------------------------------------------------------------------*/
IHelpWindow& IHelpWindow::setAssociatedWindow( IFrameWindow* associatedWindow )
{
  IASSERTPARM( associatedWindow != 0 );

  IWindowHandle frameHandle = associatedWindow->handle();

  if (!IHelpStatics::associateHelpInstance(fHelpWindowData->helpHandle, frameHandle))
     ITHROWGUIERROR("WinAssociateHelpInstance");

  return *this;
}

/*------------------------------------------------------------------------------
| IHelpWindow::sendEvent                                                       |
------------------------------------------------------------------------------*/
IEventResult IHelpWindow::sendEvent( unsigned long  eventId,
                                     const IEventParameter1 &parm1,
                                     const IEventParameter2 &parm2) const
{
   return fHelpWindowData->helpHandle.sendEvent(eventId, parm1, parm2 );
}

/*------------------------------------------------------------------------------
| IHelpWindow::sendEvent                                                       |
------------------------------------------------------------------------------*/
IEventResult IHelpWindow::sendEvent ( const IEvent& event ) const
{
  return fHelpWindowData->helpHandle.sendEvent(event.eventId(),
                                     event.parameter1(),
                                     event.parameter2());
}

/*------------------------------------------------------------------------------
| IHelpWindow::sendEvent                                                       |
------------------------------------------------------------------------------*/
IEventResult IHelpWindow::sendEvent ( EventType eventType,
                                      const IEventParameter1& parm1,
                                      const IEventParameter2& parm2 ) const
{
  unsigned long eventId;
  switch (eventType) {
  case IWindow::command:
     eventId = WM_COMMAND;
     break;
  case IWindow::systemCommand:
     eventId = WM_SYSCOMMAND;
     break;
  case IWindow::control:
     eventId = WM_CONTROL;
     break;
  case IWindow::help:
     eventId = WM_HELP;
     break;
  case IWindow::character:
     eventId = WM_CHAR;
     break;
  } /* endswitch */

  return fHelpWindowData->helpHandle.sendEvent(eventId, parm1, parm2);
}

/*------------------------------------------------------------------------------
| IHelpWindow::defaultStyle                                                    |
|                                                                              |
| Return the default style for new help window objects.                        |
------------------------------------------------------------------------------*/
IHelpWindow::Style  IHelpWindow::defaultStyle()
{
  return currentDefaultStyle;
}


