/*******************************************************************************
* FILE NAME: ihelphdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IHelpHandler - This class deals with help events                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINHELP
extern "C" {
  #include <iwindefs.h>
}

#ifndef _IHELPHDR_
  #include <ihelphdr.hpp>
#endif

#ifndef _ICCONST_
  #include <icconst.h>
#endif
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif
#ifndef _IFRAME_
  #include <iframe.hpp>
#endif
#ifndef _IMSGBOX_
  #include <imsgbox.hpp>
#endif
#ifndef _IRESLIB_
  #include <ireslib.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IHELPHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IHelpHandler::~IHelpHandler                                                  |
------------------------------------------------------------------------------*/
IHelpHandler::~IHelpHandler()
{ }


/*------------------------------------------------------------------------------
| IHelpHandler::dispatchHandlerEvent                                           |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler::dispatchHandlerEvent(IEvent& evt)
/**************************************************************/
/* After the event is dispatched to handlers in the list, this*/
/* function is dispatched. Window subclass may override this  */
/* function to handle the event.                              */
/**************************************************************/
{
  Boolean stopProcessingEvent = false;
    switch (evt.eventId())
    {
      case HM_ACTIONBAR_COMMAND:
      {
        IHelpMenuBarEvent menuBarEvent(evt);
        // Don't need to copy the event result (reserved 0 value).
        stopProcessingEvent = menuBarCommand(menuBarEvent);
        break;
      }

      case HM_ERROR:
      {
        IHelpErrorEvent errorEvent(evt);
        // Don't need to copy the event result (reserved 0 value).
        stopProcessingEvent = handleError(errorEvent);
        break;
      }

      case HM_HELPSUBITEM_NOT_FOUND:
      {
        IHelpSubitemNotFoundEvent hsnfEvent(evt);
        stopProcessingEvent = subitemNotFound(hsnfEvent);
        evt.setResult( hsnfEvent.result() );
        break;
      }

      case HM_GENERAL_HELP_UNDEFINED:
      {
        stopProcessingEvent = helpUndefined(evt);
        break;
      }

      case HM_INFORM:
      {
        IHelpHyperlinkEvent hyperlinkEvent(evt);
        // Don't need to copy the event result (reserved 0 value).
        stopProcessingEvent = hyperlinkSelect(hyperlinkEvent);
        break;
      }

      case HM_NOTIFY:
      {
        IHelpNotifyEvent notifyEvent(evt);

        switch (evt.parameter1().number1())
        {
          case CONTROL_SELECTED:
             stopProcessingEvent = controlSelect(notifyEvent);
             break;

          case OPEN_COVERPAGE:
             stopProcessingEvent = showCoverPage(notifyEvent);
             break;

          case OPEN_PAGE:
             stopProcessingEvent = showPage(notifyEvent);
             break;

          case SWAP_PAGE:
             stopProcessingEvent = swapPage(notifyEvent);
             break;

          case OPEN_INDEX:
             stopProcessingEvent = showIndex(notifyEvent);
             break;

          case OPEN_TOC:
             stopProcessingEvent = showContents(notifyEvent);
             break;

          case OPEN_HISTORY:
             stopProcessingEvent = showHistory(notifyEvent);
             break;

          case OPEN_LIBRARY:
             stopProcessingEvent = openLibrary(notifyEvent);
             break;

          case OPEN_SEARCH_HIT_LIST:
             stopProcessingEvent = showSearchList(notifyEvent);
             break;

          default:
             break;
          } /* endswitch */
        evt.setResult( notifyEvent.result() );
        break;
      } /* endcase HM_NOTIFY */

      case HM_QUERY_KEYS_HELP:
        stopProcessingEvent = keysHelpId(evt);
        break;

      case HM_TUTORIAL:
      {
        IHelpTutorialEvent tutorialEvent(evt);
        // Don't need to copy the event result (reserved 0 value).
        stopProcessingEvent = showTutorial(tutorialEvent);
        break;
      }

      default:
        break;
    } /* endswitch */

  return stopProcessingEvent;
}


/*------------------------------------------------------------------------------
| IHelpHandler::menuBarCommand                                                 |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: menuBarCommand(IHelpMenuBarEvent& menuBarEvent)
{
  return false;
}


/*------------------------------------------------------------------------------
| IHelpHandler::handleError                                                    |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: handleError(IHelpErrorEvent& errorEvent)
{
  unsigned long ulMessage = 0;
  switch (errorEvent.error())
  {
    case IHelpErrorEvent::loadingDLL:
       ulMessage = IC_HMERR_LOAD_DLL;
       break;
    case IHelpErrorEvent::noWindow:
       ulMessage = IC_HMERR_NO_FRAME_WND_IN_CHAIN;
       break;
    case IHelpErrorEvent::invalidAppHandle:
       ulMessage = IC_HMERR_INVALID_ASSOC_APP_WND;
       break;
    case IHelpErrorEvent::invalidInstanceHandle:
       ulMessage = IC_HMERR_INVALID_ASSOC_HELP_INST;
       break;
    case IHelpErrorEvent::invalidQueryHandle:
       ulMessage = IC_HMERR_INVALID_QUERY_APP_WND;
       break;
    case IHelpErrorEvent::noHelpTable:
       ulMessage = IC_HMERR_HELPTABLE_UNDEFINE;
       break;
    case IHelpErrorEvent::noHelpInstance:
       ulMessage = IC_HMERR_HELP_INSTANCE_UNDEFINE;
       break;
    case IHelpErrorEvent::invalidId:
       ulMessage = IC_HMERR_HELPITEM_NOT_FOUND;
       break;
    case IHelpErrorEvent::noIndex:
       ulMessage = IC_HMERR_INDEX_NOT_FOUND;
       break;
    case IHelpErrorEvent::noContent:
       ulMessage = IC_HMERR_CONTENT_NOT_FOUND;
       break;
    case IHelpErrorEvent::openHelpFile:
       ulMessage = IC_HMERR_OPEN_LIB_FILE;
       break;
    case IHelpErrorEvent::readHelpFile:
       ulMessage = IC_HMERR_READ_LIB_FILE;
       break;
    case IHelpErrorEvent::closeHelpFile:
       ulMessage = IC_HMERR_CLOSE_LIB_FILE;
       break;
    case IHelpErrorEvent::invalidHelpFile:
       ulMessage = IC_HMERR_INVALID_LIB_FILE;
       break;
    case IHelpErrorEvent::notEnoughMemory:
       ulMessage = IC_HMERR_NO_MEMORY;
       break;
    case IHelpErrorEvent::unableFreeMemory:
       ulMessage = IC_HMERR_FREE_MEMORY;
       break;
    case IHelpErrorEvent::invalidHelpWindow:
       ulMessage = IC_HMERR_PANEL_NOT_FOUND;
       break;
    case IHelpErrorEvent::unopenDatabase:
       ulMessage = IC_HMERR_DATABASE_NOT_OPEN;
       break;
    case IHelpErrorEvent::fontNotFound:
       ulMessage = IC_HMERR_FONT_ERROR;
       break;
    case IHelpErrorEvent::unknown:
    default:
       ulMessage = IC_HMERR_UNKNOWN;
       break;
  } /* endswitch */

  IMessageBox msgbox(errorEvent.window());
  msgbox.setTitle(IBase::messageText(IC_HELPERRORMSGBOX_TITLE));
  msgbox.show(IBase::messageText(ulMessage),
              IMessageBox::information);

  return true;
}


/*------------------------------------------------------------------------------
| IHelpHandler::helpUndefined                                                  |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: helpUndefined(IEvent& event)
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::hyperlinkSelect                                                |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: hyperlinkSelect(IHelpHyperlinkEvent& hyperlinkEvent)
{

#if (IC_OBSOLETE <= IC_OBSOLETE_1)
  return hypertextSelect(hyperlinkEvent);
#else
  return false;
#endif
}

/*------------------------------------------------------------------------------
| IHelpHandler::subitemNotFound                                                |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: subitemNotFound(IHelpSubitemNotFoundEvent& event)
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::showTutorial                                                   |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: showTutorial(IHelpTutorialEvent& tutorialEvent)
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::keysHelpId                                                     |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: keysHelpId(IEvent& event)
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::showContents                                                   |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: showContents(IHelpNotifyEvent& notifyEvent)
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::showCoverPage                                                  |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: showCoverPage(IHelpNotifyEvent& notifyEvent)
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::showPage                                                       |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: showPage(IHelpNotifyEvent& notifyEvent)
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::swapPage                                                       |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: swapPage(IHelpNotifyEvent& notifyEvent)
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::showIndex                                                      |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: showIndex(IHelpNotifyEvent& notifyEvent)
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::showHistory                                                    |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: showHistory(IHelpNotifyEvent& notifyEvent)
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::showSearchList                                                 |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: showSearchList(IHelpNotifyEvent& notifyEvent)
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::openLibrary                                                    |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: openLibrary(IHelpNotifyEvent& notifyEvent)
{
  return false;
}

/*------------------------------------------------------------------------------
| IHelpHandler::controlSelect                                                  |
------------------------------------------------------------------------------*/
IBase::Boolean IHelpHandler :: controlSelect(IHelpNotifyEvent& notifyEvent)
{
  return false;
}


/*------------------------------------------------------------------------------
| IHelpHandler::handleEventsFor                                                |
------------------------------------------------------------------------------*/
IHandler& IHelpHandler :: handleEventsFor( IWindow *window )
{
  ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR, IErrorInfo::invalidRequest,
                     IException::recoverable);
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpHandler::handleEventsFor                                                |
------------------------------------------------------------------------------*/
IHelpHandler& IHelpHandler :: handleEventsFor( IFrameWindow *applicationWindow )
{
  IASSERTPARM(applicationWindow != 0);
  IHandler::handleEventsFor(applicationWindow);
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpHandler::stopHandlingEventsFor                                          |
------------------------------------------------------------------------------*/
IHandler& IHelpHandler :: stopHandlingEventsFor( IWindow *window )
{
  ITHROWLIBRARYERROR(IC_MEMBER_ACCESS_ERROR, IErrorInfo::invalidRequest,
                     IException::recoverable);
  return *this;
}

/*------------------------------------------------------------------------------
| IHelpHandler::stopHandlingEventsFor                                          |
------------------------------------------------------------------------------*/
IHelpHandler& IHelpHandler :: stopHandlingEventsFor( IFrameWindow
                                                            *applicationWindow )
{
  IASSERTPARM(applicationWindow != 0);
  IHandler::stopHandlingEventsFor(applicationWindow);
  return *this;
}
