/*******************************************************************************
* FILE NAME: ihelpsta.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ihelpsta.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Note we depend on ireslib.cpp being first
#pragma priority( -2147482102 )
extern "C"
  {
  #define INCL_WINHELP
  #include <iwindefs.h>
  }

#include <ihelpsta.hpp>

#ifndef _IRESLIB_
  #include <ireslib.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IHELPSTA_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLNonConst)
  IHelpStatics helpMgr;
#pragma data_seg()

ICreateInstance*     IHelpStatics::entryCreate = 0;
IAssocInstance*      IHelpStatics::entryAssoc = 0;
IQueryInstance*      IHelpStatics::entryQuery = 0;
IDestroyInstance*    IHelpStatics::entryDestroy = 0;
ILoadHelpTable*      IHelpStatics::entryLoadHelpTable = 0;
IDynamicLinkLibrary* IHelpStatics::helpDynLinkLib = 0;

/*------------------------------------------------------------------------------
| IHelpStatics::~IHelpStatics                                                  |
------------------------------------------------------------------------------*/
IHelpStatics ::  ~IHelpStatics ( )
  {
    if (helpDynLinkLib)
      delete helpDynLinkLib;
  }

/*------------------------------------------------------------------------------
| IHelpStatics::helpDLL                                                        |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary& IHelpStatics :: helpDLL ( )
  {
     if (!helpDynLinkLib)
        helpDynLinkLib = new IDynamicLinkLibrary("helpmgr");
     return *helpDynLinkLib;
  }

/*------------------------------------------------------------------------------
| IHelpStatics::createHelpInstance                                             |
------------------------------------------------------------------------------*/
IWindowHandle
  IHelpStatics :: createHelpInstance ( const IAnchorBlockHandle& hab,
                                       void* helpstruct )
  {
     if (!entryCreate)
        entryCreate =
          (ICreateInstance*)helpDLL().procAddress( "Win32CreateHelpInstance");
     return (*entryCreate)(hab, helpstruct);
  }

/*------------------------------------------------------------------------------
| IHelpStatics::associateHelpInstance                                          |
------------------------------------------------------------------------------*/
Boolean
  IHelpStatics :: associateHelpInstance ( const IWindowHandle& helpHandle,
                                          const IWindowHandle& activeWindowHandle )
  {
     if (!entryAssoc)
        entryAssoc =
          (IAssocInstance*)helpDLL().procAddress("Win32AssociateHelpInstance");
     return (*entryAssoc)(helpHandle, activeWindowHandle);
  }

/*------------------------------------------------------------------------------
| IHelpStatics::queryHelpInstance                                              |
------------------------------------------------------------------------------*/
IWindowHandle
  IHelpStatics :: queryHelpInstance ( const IWindowHandle& windowHandle )
  {
     if (!entryQuery)
        entryQuery =
          (IQueryInstance*)helpDLL().procAddress("Win32QueryHelpInstance");
     return (*entryQuery)(windowHandle);
  }

/*------------------------------------------------------------------------------
| IHelpStatics::destroyHelpInstance                                            |
------------------------------------------------------------------------------*/
Boolean IHelpStatics :: destroyHelpInstance ( const IWindowHandle& helpHandle ) // $HT!c
  {
     if (!entryDestroy)
        entryDestroy =
          (IDestroyInstance*)helpDLL().procAddress("Win32DestroyHelpInstance");
     return (*entryDestroy)(helpHandle);
  }

/*------------------------------------------------------------------------------
| IHelpStatics::loadHelpTable                                                  |
------------------------------------------------------------------------------*/
Boolean IHelpStatics :: loadHelpTable ( const IWindowHandle& helpHandle,
                                        unsigned long id,
                                        const IModuleHandle& hmod )
  {
     if (!entryLoadHelpTable)
        entryLoadHelpTable =
          (ILoadHelpTable*)helpDLL().procAddress("Win32LoadHelpTable");
     return (*entryLoadHelpTable)(helpHandle, id, hmod);
  }
