/*******************************************************************************
* FILE NAME: iiconctl.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iiconctl.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147482112 )

#define INCL_GPIBITMAPS
#define INCL_WINPOINTERS
#define INCL_WINSTATICS           // SS_xxx
extern "C" {
  #include <iwindefs.h>
}

#include <iiconctl.hpp>
#include <iexcept.hpp>
#include <ihandle.hpp>
#include <iiconhdr.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <ithread.hpp>
#include <iiconctp.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IICONCTL_CPP_
  #include <ipagetun.h>
#endif

/***************************************************************/
/* Public icon styles.                                         */
/***************************************************************/
#pragma data_seg(ICLStaticConst)
  const IIconControl::Style IIconControl::sizeToIcon      = SS_BITMAP;
const IIconControl::Style IIconControl::classDefaultStyle = WS_VISIBLE;

#pragma data_seg()


/***************************************************************/
/* Default style for new objects (initial value).              */
/***************************************************************/
#pragma data_seg(ICLNonConst)
IIconControl::Style IIconControl::currentDefaultStyle = WS_VISIBLE;
#pragma data_seg()


/*------------------------------------------------------------------------------
  This class wrappers an object pointer and destroys the pointed
  to object when the wrapper is destructed.
------------------------------------------------------------------------------*/
static struct DefaultHandler {
  operator IIconHandler* ();
IIconHandler
 *operator -> ()
  {
    return *this;
  }
 ~DefaultHandler ( )
  {
    if (ptr)
    {
      delete ptr;
      ptr = 0;
    }
  }
IIconHandler
 *ptr;
} defaultHandler;

/*------------------------------------------------------------------------------
| DefaultHandler::operator IIconHandler*                                       |
------------------------------------------------------------------------------*/
DefaultHandler :: operator IIconHandler* ( )
{
  if ( !ptr )
    ptr = new IIconHandler;
  return ptr;
}

/*------------------------------------------------------------------------------
| IIconControl::IIconControl                                                   |
------------------------------------------------------------------------------*/
IIconControl :: IIconControl ( unsigned long ulId,
                               IWindow* pwndParent,
                               IWindow* pwndOwner,
                               const IResourceId& iconId,
                               const IRectangle& rectInit,
                               const Style& icsStyle )
  : IBitmapControl(ulId,pwndParent,pwndOwner,IBitmapHandle(),
                   rectInit,
                   IWindow::noStyle),
    fIconControlData( 0 ),
    iconHandle(0),
    bSizeTo(false) ,
    bHalf(false) ,
    returnback(false)
{
  // Assertions on input parms.
  IASSERTPARM(pwndParent!=0);
  initialize(icsStyle);
  setIcon(iconId);                // Load and set icon.
  this -> addHandler( defaultHandler );
  if (icsStyle & IWindow::visible)
  {
     show();
     refresh(IWindow::immediate);
  }
}

/*------------------------------------------------------------------------------
| IIconControl::IIconControl                                                   |
------------------------------------------------------------------------------*/
IIconControl :: IIconControl ( unsigned long ulId,
                               IWindow* pwndParent,
                               IWindow* pwndOwner,
                               unsigned long iconId,
                               const IRectangle& rectInit,
                               const Style& icsStyle )
  : IBitmapControl(ulId,pwndParent,pwndOwner,IBitmapHandle(),
                   rectInit,
                   IWindow::noStyle),
    fIconControlData( 0 ),
    iconHandle(0),
    bSizeTo(false) ,
    bHalf(false) ,
    returnback(false)
{
  // Assertions on input parms.
  IASSERTPARM(pwndParent!=0);
  initialize(icsStyle);
  setIcon(iconId);                // Load and set icon.
  this -> addHandler( defaultHandler );
  if (icsStyle & IWindow::visible)
  {
     show();
     refresh(IWindow::immediate);
  }
}

/*------------------------------------------------------------------------------
| IIconControl::IIconControl                                                   |
------------------------------------------------------------------------------*/
IIconControl :: IIconControl ( unsigned long ulId,
                               IWindow* pwndParent,
                               IWindow* pwndOwner,
                               const IPointerHandle& iconId,
                               const IRectangle& rectInit,
                               const Style& icsStyle )
  : IBitmapControl(ulId,pwndParent,pwndOwner,IBitmapHandle(),
                   rectInit,
                   IWindow::noStyle),
    fIconControlData( 0 ),
    iconHandle(0),
    bSizeTo(false) ,
    bHalf(false) ,
    returnback(false)
{
  // Assertions on input parms.
  IASSERTPARM(pwndParent!=0);
  initialize(icsStyle);
  setIcon(iconId);                // Load and set icon.
  this -> addHandler( defaultHandler );
  if (icsStyle & IWindow::visible)
  {
     show();
     refresh(IWindow::immediate);
  }
}

/*------------------------------------------------------------------------------
| IIconControl::IIconControl                                                   |
|  Construct an icon control on a standard window using a                      |
|  system-provided icon.                                                       |
------------------------------------------------------------------------------*/
IIconControl :: IIconControl ( unsigned long ulId,
                               IWindow* pwndParent,
                               IWindow* pwndOwner,
                               ISystemPointerHandle::Identifier icon,
                               const IRectangle& rectInit,
                               const Style& icsStyle )
  : IBitmapControl(ulId,pwndParent,pwndOwner,IBitmapHandle(),
                   rectInit,
                   IWindow::noStyle),
    fIconControlData( 0 ),
    iconHandle(0),
    bSizeTo(false) ,
    bHalf(false)  ,
    returnback(false)
{
  // Assertions on input parms.
  IASSERTPARM(pwndParent!=0);
  initialize(icsStyle);
  setIcon(icon);                  // Associate with control.
  this -> addHandler( defaultHandler );
  if (icsStyle & IWindow::visible)
  {
     show();
     refresh(IWindow::immediate);
  }
}

/*------------------------------------------------------------------------------
| IIconControl::IIconControl                                                   |
|  Construct on a tag template window.                                         |
------------------------------------------------------------------------------*/
IIconControl :: IIconControl ( unsigned long ulId,
                               IWindow* pdlgwndParent )
  : IBitmapControl(ulId,pdlgwndParent),
    fIconControlData( 0 ),
    iconHandle(0),
    bSizeTo(true) ,
    bHalf(false)  ,
    returnback(true)
{
  unsigned long ulStyle = style();
  IEventResult evt = handle().sendEvent( SM_QUERYHANDLE,
                                         IEventParameter1( 0 ),
                                         IEventParameter2( 0 ));
  HPOINTER pnt=evt.asUnsignedLong();
  ulStyle &= (unsigned long)(~SS_ICON);
  ulStyle |= SS_TEXT;
  setStyle(ulStyle);
  setIcon(IPointerHandle(pnt));
  this -> addHandler( defaultHandler );
}

/*------------------------------------------------------------------------------
| IIconControl::IIconControl                                                   |
|  Constructor to instantiate from an existing icon control.                   |
------------------------------------------------------------------------------*/
IIconControl :: IIconControl ( const IWindowHandle& wh )
  : IBitmapControl(wh),
    fIconControlData( 0 ),
    iconHandle(0),
    bSizeTo(true) ,
    bHalf(false)   ,
    returnback(true)
{
  unsigned long ulStyle = style();
  HPOINTER pnt=0;
  if (ulStyle & (SS_ICON & ~SS_TEXT))
  {
     IEventResult evt = handle().sendEvent( SM_QUERYHANDLE,
                                            IEventParameter1( 0 ),
                                            IEventParameter2( 0 ));
     pnt=evt.asUnsignedLong();
  } /* endif */
  ulStyle &= (unsigned long)(~SS_ICON);
  ulStyle |= SS_TEXT;
  setStyle(ulStyle);
  setIcon(IPointerHandle(pnt));
  this -> addHandler( defaultHandler );
}

/*------------------------------------------------------------------------------
| IIconControl::~IIconControl                                                  |
------------------------------------------------------------------------------*/
IIconControl :: ~IIconControl ( )
{
  if (returnback)
  {
     try
     {
        unsigned long ulStyle = style();
        ulStyle &= (SS_ICON | (unsigned long)(~SS_TEXT));
        setStyle(ulStyle);
        handle().sendEvent(SM_SETHANDLE, IEventParameter1(iconHandle),
                                         IEventParameter2(0));
     } catch (IException) {}
  }
  this -> removeHandler( defaultHandler );

}

/*------------------------------------------------------------------------------
| IIconControl::Style  IIconControl :: defaultStyle                            |
------------------------------------------------------------------------------*/
IIconControl::Style IIconControl :: defaultStyle ( )
{
  return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IIconControl::setDefaultStyle                                                |
------------------------------------------------------------------------------*/
void IIconControl :: setDefaultStyle ( const IIconControl::Style& icsStyle )
{
  currentDefaultStyle = icsStyle;
}

/*------------------------------------------------------------------------------
| IIconControl::convertToGUIStyle                                              |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IIconControl::convertToGUIStyle ( const IBitFlag& guiStyle,
                                                Boolean bExtOnly ) const
{
  // Obtain the style from the class (IBitmapControl) that we inherit from.
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range.
    ulStyle |= extendedStyle() & IS_EXTMASK;
  }

  // No additional processing as IBitmapControl::convertToGUIStyle will do all
  // the required processing.

  return ulStyle;
}

/*------------------------------------------------------------------------------
| IIconControl::setIcon                                                        |
------------------------------------------------------------------------------*/
IIconControl& IIconControl :: setIcon ( unsigned long iconId )
{
  setIcon(IResourceId(iconId));
  return *this;
}

/*------------------------------------------------------------------------------
| IIconControl::setIcon                                                        |
------------------------------------------------------------------------------*/
IIconControl& IIconControl :: setIcon ( const IResourceId& iconId )
{
  IPointerHandle ph = iconId.resourceLibrary().loadIcon(iconId);
                                  // Load from resource library.
  setIcon(ph);
  return *this;
}

/*------------------------------------------------------------------------------
| IIconControl::setIcon                                                        |
------------------------------------------------------------------------------*/
IIconControl& IIconControl :: setIcon ( const IPointerHandle& phHandle )
{

  iconHandle = phHandle;
  if ( iconHandle )
  {
     POINTERINFO pointerinfo;
     IQUERYPOINTERINFO( iconHandle, &pointerinfo );
     if ( pointerinfo.hbmColor == 0 )
     {
        bHalf = true;
        this->setBitmap( IBitmapHandle( pointerinfo.hbmPointer ));
     } else
     {
        bHalf = false;
        this->setBitmap( IBitmapHandle( pointerinfo.hbmColor ));
     }
  }
  else
  {
     this->setBitmap( IBitmapHandle(0) );
  }

  if ( bSizeTo )
  {
     this->sizeTo( this->calcMinimumSize() );
  }
  this->refresh();

  // Since the new icon being displayed may have transparent pels, we
  // need to force the parent window to repaint the background (unless
  // we have a style of fillBackground in which case we do
  // painting already).
  if (!this->hasFillBackground() && this->parent())
  {
     this->parent()->refresh(this->rect());
  }

  return *this;
}

/*------------------------------------------------------------------------------
| IIconControl::setIcon                                                        |
------------------------------------------------------------------------------*/
IIconControl& IIconControl :: setIcon ( ISystemPointerHandle::Identifier icon )
{
  setIcon(ISystemPointerHandle(icon));
  return *this;
}

/*------------------------------------------------------------------------------
| IIconControl::icon                                                           |
------------------------------------------------------------------------------*/
IPointerHandle IIconControl :: icon ( ) const
{
  return iconHandle;
}

/*------------------------------------------------------------------------------
| IIconControl::calcMinimumSize                                                |
|  Calculate the minimum screen size needed by the control                     |
|  which is based on the icon size.                                            |
------------------------------------------------------------------------------*/
ISize IIconControl :: calcMinimumSize ( ) const
{
  ISize minSize( 0, 0 );

  if ( ! this->icon() )
  {           // No graphic, or one from IBitmapControl::setBitmap.
     minSize = Inherited::calcMinimumSize();
  }           // Let IBitmapControl figure this case.
  else
  {           // Icon once passed to setIcon.
     POINTERINFO pointerinfo;
     IQUERYPOINTERINFO( this->icon(), &pointerinfo );
     if ( this->bitmap() != pointerinfo.hbmColor  &&
          this->bitmap() != pointerinfo.hbmPointer )
     {        // Bitmap set via IBitmapControl::setBitmap.
        minSize = Inherited::calcMinimumSize();
     }        // Let IBitmapControl figure this case too.
     else
     {        // bitmap() is graphic from IIconControl::setIcon.
        minSize = IStaticText::calcMinimumSize();

        BITMAPINFOHEADER pbmpData;
        pbmpData.cbFix = sizeof( BITMAPINFOHEADER );
        pbmpData.cx = 0;
        pbmpData.cy = 0;

        if ( !IQUERYBITMAPPARAMETERS( this->bitmap(), &pbmpData ) )
        {
           ITHROWGUIERROR( "IQUERYBITMAPPARAMETERS" );
        }

        ISize bmpSize( pbmpData.cx, pbmpData.cy );
        if ( bHalf )
        {
           bmpSize.scaleBy( 1, 0.5 );
        }
        minSize = minSize.maximum( bmpSize );
     }  // End base minimum size on icon.
  }

  return minSize;
}

/*------------------------------------------------------------------------------
| IIconControl::initialize                                                     |
------------------------------------------------------------------------------*/
IIconControl& IIconControl :: initialize ( const Style& style )
{
  // Save the extended style to make sure we have a copy of it stored.
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

  unsigned long ulStyle = convertToGUIStyle( style );
  if (style & IIconControl::sizeToIcon)
  {
     bSizeTo = true;
     ulStyle &= (unsigned long)(~SS_BITMAP);
  }

  if (ulStyle & IWindow::visible.asUnsignedLong())
     ulStyle &= (~IWindow::visible.asUnsignedLong());

  setStyle(ulStyle);
  return *this;
}

/*------------------------------------------------------------------------------
| IIconControl::visibleRectangle                                               |
|                                                                              |
| Returns the area painted by the icon control.                                |
------------------------------------------------------------------------------*/
IRectangle IIconControl :: visibleRectangle ( ) const
{
  return IRectangle( this->position(), ISize( 0, 0 ));
}
