/*******************************************************************************
* FILE NAME: iiconhdr.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iiconhdr.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #define INCL_GPIBITMAPS
  #define INCL_GPIPRIMITIVES
  #define INCL_WINPOINTERS
  #define INCL_GPILOGCOLORTABLE
  #include <iwindefs.h>
  }

#include <iiconhdr.hpp>

#include <ipainevt.hpp>
#include <iswp.hpp>
#include <itrace.hpp>
#include <icontrol.hpp>
#include <iexcept.hpp>
#include <ihandle.hpp>
#include <iiconctl.hpp>
#include <igrafctx.hpp>
#include <igrect.hpp>
#include <igbitmap.hpp>
#include <iiconctp.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IICONHDR_CPP_
  #include <ipagetun.h>
#endif

IIconHandler :: IIconHandler() {;}
IIconHandler :: ~IIconHandler() {;}

/*-------------------------------- Overrides -----------------------------------
| This inherited function is overridden:                                       |
|   paintWindow - Process the paint events for a Icon control window.          |
------------------------------------------------------------------------------*/
Boolean IIconHandler::paintWindow( IPaintEvent& evt )
{
  IMODTRACE_ALL("IIconHandler::paintWindow");
  IIconControl* icon = (IIconControl*)evt.window();

  // Before drawing the icon, check for fill background style
  if (icon->hasFillBackground())
  {
    ICREATELOGCOLORTABLE( evt.presSpaceHandle(), 0L,
                          LCOLF_RGB, 0L, 0L, 0 );
    IColor clrFill = icon->fillColor();
    RECTL lprc = IRectangle(IPoint(0,0),icon->size()).asRECTL();
    WinFillRect( evt.presSpaceHandle(), &lprc, clrFill.asRGBLong() );
  }

  if (icon->icon())
  {
     POINTERINFO
        pointerinfo;
     IQUERYPOINTERINFO(icon->icon(),&pointerinfo);
     if ( (icon->bitmap() != pointerinfo.hbmColor) &&
          (icon->bitmap() != pointerinfo.hbmPointer))
        return false;                 //Let the bitmap painthandler
                                      //have this since must have called
                                      //setBitmap on this IIconControl
     POINTL aptlPoints[4];
     BITMAPINFOHEADER   pbmpData;
     pbmpData.cbFix = sizeof(BITMAPINFOHEADER);
     if (pointerinfo.hbmColor == NULLHANDLE)
     {
        GpiQueryBitmapParameters(pointerinfo.hbmPointer,&pbmpData);
        pbmpData.cy *= .5;
     }
     else
        GpiQueryBitmapParameters(pointerinfo.hbmColor  ,&pbmpData);
     ISize winsize = icon->size();
     aptlPoints[0].x = 0;
     aptlPoints[0].y = 0;
     aptlPoints[1].x = winsize.width()-1;
     aptlPoints[1].y = winsize.height()-1;
     aptlPoints[2].x = 0L;
     aptlPoints[3].x = pbmpData.cx;

     //The following values are special for the AND MASK
     aptlPoints[2].y = pbmpData.cy;
     aptlPoints[3].y = aptlPoints[2].y + pbmpData.cy;

     //Set up the AND MASK colors
     unsigned long lcolor=GpiQueryColor(evt.presSpaceHandle());
     unsigned long lbcolor=GpiQueryBackColor(evt.presSpaceHandle());
     GpiSetColor(evt.presSpaceHandle(),CLR_WHITE);
     GpiSetBackColor(evt.presSpaceHandle(),CLR_BLACK);

     GpiWCBitBlt(evt.presSpaceHandle(),pointerinfo.hbmPointer,
                 4,aptlPoints,ROP_SRCAND,BBO_IGNORE);

     //Restore original presentantion space colors
     GpiSetColor(evt.presSpaceHandle(),lcolor);
     GpiSetBackColor(evt.presSpaceHandle(),lbcolor);

     // Reset array values since the Gpi call might have changed them
     aptlPoints[0].x = 0;
     aptlPoints[0].y = 0;
     aptlPoints[1].x = winsize.width()-1;
     aptlPoints[1].y = winsize.height()-1;
     aptlPoints[2].x = 0L;
     aptlPoints[3].x = pbmpData.cx;

     //The following values are special for the color bitmap and XOR MASK
     aptlPoints[2].y = 0L;
     aptlPoints[3].y = aptlPoints[2].y + pbmpData.cy;

     if (pointerinfo.hbmColor!=NULLHANDLE)
     {
        GpiWCBitBlt(evt.presSpaceHandle(),pointerinfo.hbmColor,
                    4,aptlPoints,ROP_SRCPAINT,BBO_IGNORE);
     } /* endif */

     //Set up the XOR MASK colors
     GpiSetColor(evt.presSpaceHandle(),CLR_WHITE);
     GpiSetBackColor(evt.presSpaceHandle(),CLR_BLACK);

     // Reset array values since the Gpi call might have changed them
     aptlPoints[0].x = 0;
     aptlPoints[0].y = 0;
     aptlPoints[1].x = winsize.width()-1;
     aptlPoints[1].y = winsize.height()-1;
     aptlPoints[2].x = 0L;
     aptlPoints[3].x = pbmpData.cx;
     aptlPoints[2].y = 0L;
     aptlPoints[3].y = aptlPoints[2].y + pbmpData.cy;

     GpiWCBitBlt(evt.presSpaceHandle(),pointerinfo.hbmPointer,
                 4,aptlPoints,ROP_SRCINVERT,BBO_IGNORE);

     //Restore original presentantion space colors
     GpiSetColor(evt.presSpaceHandle(),lcolor);
     GpiSetBackColor(evt.presSpaceHandle(),lbcolor);
  }
  else
  {
     return false;
  } /* endif */
  unsigned long length = icon->textLength();
  if (length)
  {
    if (icon->hasFillBackground())
    {
      // Turn off fill background so that we do not overpaint icon
      icon->disableUpdate();
      icon->disableFillBackground();
      icon->enableUpdate();
      Inherited::paintWindow(evt);
      icon->disableUpdate();
      icon->enableFillBackground();
      icon->enableUpdate();
    }
    else
    {
      Inherited::paintWindow(evt);
    }
  }
  return true;
}
