/*******************************************************************************
* FILE NAME: ikeyhdr.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ikeyhdr.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C"
  {
  #define INCL_WININPUT
  #include <os2.h>
  }

#include <ikeyhdr.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IKEYHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IKeyboardHandler::IKeyboardHandler                                           |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IKeyboardHandler :: IKeyboardHandler ( )
  : IKeyboardHandler::Inherited ( )
{}


/*------------------------------------------------------------------------------
| IKeyboardHandler::~IKeyboardHandler                                          |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IKeyboardHandler :: ~IKeyboardHandler ( )
{}


/*------------------------------------------------------------------------------
| IKeyboardHandler::dispatchHandlerEvent                                       |
|                                                                              |
| Distribute key events to the appropriate virtual functions.                  |
------------------------------------------------------------------------------*/
Boolean  IKeyboardHandler :: dispatchHandlerEvent ( IEvent & evt )
{
  Boolean bRc = false;

  if (evt.eventId() == WM_CHAR )
  {                               // key event found
     IKeyboardEvent keyevt(evt);
     if (keyevt.isUpTransition() == false)
     {                            // key press
        IKeyboardEvent::VirtualKey vkKey;
        if (keyevt.isScanCode()  &&
            scanCodeKeyPress(keyevt) )
        {                         // have a scan code
           bRc = true;            // no additional processing needed
        }
        else if (keyevt.isVirtual()  &&
                 (keyevt.isCharacter() == false  ||
                  ((vkKey = keyevt.virtualKey()) != IKeyboardEvent::insert  &&
                   vkKey != IKeyboardEvent::deleteKey  &&
                   vkKey != IKeyboardEvent::home  &&
                   vkKey != IKeyboardEvent::end  &&
                   vkKey != IKeyboardEvent::pageUp  &&
                   vkKey != IKeyboardEvent::pageDown  &&
                   vkKey != IKeyboardEvent::left  &&
                   vkKey != IKeyboardEvent::right  &&
                   vkKey != IKeyboardEvent::up  &&
                   vkKey != IKeyboardEvent::down))  &&
                 virtualKeyPress(keyevt) )
        {       // exclude NumLock + numeric keypad from virtual keys
           bRc = true;            // no additional processing needed
        }
        else if (keyevt.isCharacter()  &&
                 (keyevt.isVirtual() == false  ||
                  ((vkKey = keyevt.virtualKey()) != IKeyboardEvent::tab  &&
                   vkKey != IKeyboardEvent::enter  &&
                   vkKey != IKeyboardEvent::newLine  &&
                   vkKey != IKeyboardEvent::backSpace  &&
                   vkKey != IKeyboardEvent::esc))  &&
                 characterKeyPress(keyevt) )
        {     // exclude tab, new line, backspace, esc from char keys
           bRc = true;            // no additional processing needed
        }
     }
     if (bRc == false  &&  key(keyevt) )
     {                            // catch-all virtual function
        bRc = true;               // no additional processing needed
     }

     if (bRc)
     {                            // key should not be passed on
        evt.setResult(keyevt.result());
     }
  } /* endif WM_CHAR */

  return bRc;
}


/*------------------------------------------------------------------------------
| IKeyboardHandler::scanCodeKeyPress                                           |
|                                                                              |
| Process the press of a key with a scan code value.                           |
------------------------------------------------------------------------------*/
Boolean  IKeyboardHandler :: scanCodeKeyPress ( IKeyboardEvent & keyevt )
{
  return false;
}


/*------------------------------------------------------------------------------
| IKeyboardHandler::virtualKeyPress                                            |
|                                                                              |
| Process the press of a virtual key.                                          |
------------------------------------------------------------------------------*/
Boolean  IKeyboardHandler :: virtualKeyPress ( IKeyboardEvent & keyevt )
{
  return false;
}


/*------------------------------------------------------------------------------
| IKeyboardHandler::characterKeyPress                                          |
|                                                                              |
| Process the press of a character key.                                        |
------------------------------------------------------------------------------*/
Boolean  IKeyboardHandler :: characterKeyPress ( IKeyboardEvent & keyevt )
{
  return false;
}


/*------------------------------------------------------------------------------
| IKeyboardHandler::key                                                        |
|                                                                              |
| Process any and all key events.                                              |
------------------------------------------------------------------------------*/
Boolean  IKeyboardHandler :: key ( IKeyboardEvent & keyevt )
{
  return false;
}
