/*******************************************************************************
* FILE NAME: ilbdievt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ilbdievt.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #include <stdio.h>
  #define INCL_WINMENUS
  #include <iwindefs.h>
  }

#include <ilbdievt.hpp>

#ifndef  _IRECT_
  #include <irect.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ILBDIEVT_CPP_
  #include <ipagetun.h>
#endif

  #define OIDATA ((POWNERITEM)ownerItemData())

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::IListBoxDrawItemEvent                                 |
|                                                                              |
| Construct IListBoxDrawItemEvent object from an IEvent instance               |
------------------------------------------------------------------------------*/
IListBoxDrawItemEvent :: IListBoxDrawItemEvent ( IEvent& event )
  : IListBoxDrawItemEvent::Inherited( event ),
    fSelectionStateDrawn( false )
{ }

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::~IListBoxDrawItemEvent                                |
|                                                                              |
| Empty destructor here for page tuning                                        |
------------------------------------------------------------------------------*/
IListBoxDrawItemEvent :: ~IListBoxDrawItemEvent ( )
{ }

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::isDrawItem                                            |
|                                                                              |
| Test to see if the event is for drawing the item.                            |
------------------------------------------------------------------------------*/
Boolean IListBoxDrawItemEvent::isDrawItem ( ) const
{
  return (OIDATA->fsState == OIDATA->fsStateOld) ? true : false;
}

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::isSelected                                            |
------------------------------------------------------------------------------*/
Boolean IListBoxDrawItemEvent::isSelected ( ) const
{
  return (OIDATA->fsState) ? true : false;
}

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::isSetHighlight                                        |
|                                                                              |
| Test to see if the event is for highlighting the item.                       |
------------------------------------------------------------------------------*/
Boolean IListBoxDrawItemEvent::isSetHighlight ( ) const
{
  return (OIDATA->fsState && !OIDATA->fsStateOld) ? true : false;
}

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::isRemoveHighlight                                     |
------------------------------------------------------------------------------*/
Boolean IListBoxDrawItemEvent::isRemoveHighlight ( ) const
{
  return (!OIDATA->fsState && OIDATA->fsStateOld) ? true : false;
}

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::setSelectionStateDrawn                                |
------------------------------------------------------------------------------*/
IListBoxDrawItemEvent&
  IListBoxDrawItemEvent::setSelectionStateDrawn ( Boolean drawn )
{
  fSelectionStateDrawn = drawn;
  return *this;
}

/*------------------------------------------------------------------------------
| IListBoxDrawItemEvent::isSelectionStateDrawn                                 |
------------------------------------------------------------------------------*/
Boolean IListBoxDrawItemEvent::isSelectionStateDrawn  ( ) const
{
  return fSelectionStateDrawn;
}

/*------------------------------------------------------------------------------
| IListBoxSizeItemEvent::IListBoxSizeItemEvent                                 |
------------------------------------------------------------------------------*/
IListBoxSizeItemEvent::IListBoxSizeItemEvent ( IEvent& event )
  : IListBoxSizeItemEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IListBoxSizeItemEvent::~IListBoxSizeItemEvent                                |
------------------------------------------------------------------------------*/
IListBoxSizeItemEvent::~IListBoxSizeItemEvent ( )
{ }

/*------------------------------------------------------------------------------
| IListBoxSizeItemEvent::itemIndex                                             |
------------------------------------------------------------------------------*/
unsigned long IListBoxSizeItemEvent::itemIndex ( ) const
{
  return this->parameter2().number1();
}

/*------------------------------------------------------------------------------
| IListBoxSizeItemEvent::itemSize                                              |
------------------------------------------------------------------------------*/
ISize IListBoxSizeItemEvent::itemSize ( ) const
{
  return ISize( this->result().number2(), this->result().number1() );
}

/*------------------------------------------------------------------------------
| IListBoxSizeItemEvent::setItemSize                                           |
------------------------------------------------------------------------------*/
IListBoxSizeItemEvent&
  IListBoxSizeItemEvent::setItemSize ( const ISize& itemSize )
{
  this->setResult( IEventResult( (unsigned short)itemSize.height(),
                                 (unsigned short)itemSize.width() ));
  return *this;
}

/*------------------------------------------------------------------------------
| IListBoxSizeItemEvent::setResult                                             |
------------------------------------------------------------------------------*/
IEvent& IListBoxSizeItemEvent::setResult ( const IEventResult& eventResult )
{
  return Inherited::setResult( eventResult );
}
