/*******************************************************************************
* FILE NAME: ilistbox.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ilistbox.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
{
  #define INCL_WINLISTBOXES      // LIT_xxx, LM_xxx, LS_xxx
  #define INCL_WINSYS            // SV_xxx, PP_xxx
  #include <iwindefs.h>
}

#include <inotifev.hpp>
#include <ilistbox.hpp>
#include <istring.hpp>
#include <ireslib.hpp>
#include <ihandle.hpp>
#include <iexcept.hpp>
#include <icconst.h>
#include <itrace.hpp>
#include <icolor.hpp>
#include <ifont.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ILISTBOX_CPP_
  #include <ipagetun.h>
#endif

/***************************************************************/
/* Public list box styles.                                     */
/***************************************************************/
#pragma data_seg(ICLStaticConst)
const IListBox::Style
  IListBox::classDefaultStyle ( LS_HORZSCROLL  |
                                LS_NOADJUSTPOS |
                                WS_VISIBLE,
                                0 );
#pragma data_seg()

/***************************************************************/
/* Default style for new objects (initial value)               */
/***************************************************************/
#pragma data_seg(ICLNonConst)
  IListBox::Style IListBox::currentDefaultStyle ( LS_HORZSCROLL  |
                                                  LS_NOADJUSTPOS |
                                                  WS_VISIBLE,
                                                  0 );
#pragma data_seg()


/*------------------------------------------------------------------------------
| IListBox::IListBox                                                           |
|                                                                              |
| Construct an IListBox on an IWindow.                                         |
------------------------------------------------------------------------------*/
IListBox::IListBox ( unsigned long     Id,
                     IWindow*          pParent,
                     IWindow*          pOwner,
                     const IRectangle& rectInit,
                     const Style&      style )
  : fListBoxData( 0 ),
    IBaseListBox( Id,
                  pParent,
                  pOwner,
                  rectInit,
                  (const IBaseListBox::Style&) style )
{
}

/*------------------------------------------------------------------------------
| IListBox::IListBox                                                           |
|                                                                              |
| Construct an IListBox on a dialog.                                           |
------------------------------------------------------------------------------*/
IListBox::IListBox ( unsigned long id,
                     IWindow* parent )
  : fListBoxData( 0 ),
    IBaseListBox( id, parent )
{
}

/*------------------------------------------------------------------------------
| IListBox::IListBox                                                           |
|                                                                              |
| Wrapper an existing ListBox.                                                 |
------------------------------------------------------------------------------*/
IListBox::IListBox ( const IWindowHandle& handle )
  : fListBoxData( 0 ),
    IBaseListBox( handle )
{
}

/*------------------------------------------------------------------------------
| IListBox::~IListBox                                                          |
|                                                                              |
| Default destructor here for page tuning.                                     |
------------------------------------------------------------------------------*/
IListBox::~IListBox()
{
  // delete fListBoxData;
}

/*------------------------------------------------------------------------------
| IListBox::Style IListBox::defaultStyle                                       |
------------------------------------------------------------------------------*/
IListBox::Style IListBox::defaultStyle()
{
   return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IListBox::setDefaultStyle                                                    |
------------------------------------------------------------------------------*/
void IListBox::setDefaultStyle(const Style& style)
{
  currentDefaultStyle = style;
}

#include <ilistbx4.hpp>
