/*******************************************************************************
* FILE NAME: imenuevt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imenuevt.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
   #define INCL_WINMESSAGEMGR
   #define INCL_WINWINDOWMGR
   #define INCL_WININPUT
   #define INCL_WINPOINTERS
   #include <iwindefs.h>
}

#include <iapp.hpp>
#include <icoordsy.hpp>
#include <imnitem.hpp>
#include <imenuevt.hpp>
#include <ipoint.hpp>
#include <isubmenu.hpp>
#include <ithread.hpp>
#include <iwindow.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMENUEVT_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IMenuEvent::IMenuEvent                                                       |
------------------------------------------------------------------------------*/
IMenuEvent :: IMenuEvent(IEvent& evt)
  : IEvent(evt) {;}

/*------------------------------------------------------------------------------
| IMenuEvent::~IMenuEvent                                                      |
------------------------------------------------------------------------------*/
IMenuEvent :: ~IMenuEvent( ) {}

/*------------------------------------------------------------------------------
| IMenuEvent::menuItemId                                                       |
------------------------------------------------------------------------------*/
unsigned long IMenuEvent::menuItemId() const
{
  // Check for menu start or end and return id info
  if (eventId() == WM_INITMENU || eventId() == WM_MENUEND)
  {
    return parameter1();
  }
  else
  {
    // Otherwise menu select action, id info in different location
    if (eventId() == WM_MENUSELECT)
    {
      return parameter1().number1();
    }
  } /* endif */
  return 0;
}

/*------------------------------------------------------------------------------
| IMenuEvent::mousePosition                                                    |
|  Returns mouse point mapped to the menu control owner window.                |
------------------------------------------------------------------------------*/
IPoint IMenuEvent::mousePosition() const
{
   POINTL ptl;
   IQUERYMSGPOS(IThread::current().anchorBlock(), &ptl);
   return IWindow::mapPoint(
              ICoordinateSystem::convertToApplication(
                 ptl,
                 IWindow::desktopWindow()->size() ),
              HWND_DESKTOP,
              window()->handle() );
}

/*------------------------------------------------------------------------------
| IMenuEvent::menuItem                                                         |
------------------------------------------------------------------------------*/
IMenuItem IMenuEvent::menuItem() const
{
   // Only have the id of an actual MenuItem for WM_MENUSELECT
   // All other msgs pertain to an entire menu
   if (eventId() == WM_MENUSELECT ) {
      ISubmenu submenu((unsigned long)parameter2());
      return submenu.menuItem(menuItemId());
   } /* endif */
   return IMenuItem(0);
}
