/*******************************************************************************
* FILE NAME: imenuprv.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imenuprv.hpp                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1996             *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
{
  #include <iwindefs.h>
}

#include <imenuprv.hpp>
#include <iframe.hpp>
#include <iframprv.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMENUPRV_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| setMenuItemHelpId                                                            |
|  Local function used to build help id table for menu items                   |
------------------------------------------------------------------------------*/
void IMenuPrivate::setMenuItemHelpId( IFrameWindow*  owner,
                                      unsigned long  menuItemId,
                                      unsigned long  helpTopicId )
{
  /********************************************************************/
  /* If a valid owner passed, check table for id passed               */
  /********************************************************************/
  if (owner)
  {
    /******************************************************************/
    /* Start at top and keep searching as long as entries exist       */
    /******************************************************************/
    MIHELPTBL* pMiHTable = (MIHELPTBL*)(owner->fFrameWindowData->fpHelpList);
    while (pMiHTable)
    {
      /****************************************************************/
      /* If match is found, return match, otherwise go to next entry  */
      /****************************************************************/
      if ( pMiHTable->fId == menuItemId )
      {
        pMiHTable->fHelpId = helpTopicId;
        return;
      }
      pMiHTable = pMiHTable->pmitNext;
    }

    /******************************************************************/
    /* Create a new menu lookup table entry and fill with info passed */
    /******************************************************************/
    MIHELPTBL* pMi2Table = new MIHELPTBL;
    pMi2Table->fId = menuItemId;
    pMi2Table->fHelpId = helpTopicId;

    /******************************************************************/
    /* Chain list (if any) to list                                    */
    /******************************************************************/
    pMi2Table->pmitNext = (MIHELPTBL*)(owner->fFrameWindowData->fpHelpList);

    /******************************************************************/
    /* Finally, since this is now the start of the list, save it      */
    /******************************************************************/
    owner->fFrameWindowData->fpHelpList = (void*)pMi2Table;
  }
  return;
}


/*------------------------------------------------------------------------------
| queryMenuItemHelpId                                                          |
|  Local function used to build help id table for menu items                   |
------------------------------------------------------------------------------*/
unsigned long IMenuPrivate::queryMenuItemHelpId( IFrameWindow*  owner,
                                                 unsigned long  menuItemId )
{
  /********************************************************************/
  /* If a valid owner passed, check table for id passed               */
  /********************************************************************/
  if (owner)
  {
    /******************************************************************/
    /* Start at top and keep searching as long as entries exist       */
    /******************************************************************/
    MIHELPTBL* pMiHTable = (MIHELPTBL*)(owner->fFrameWindowData->fpHelpList);
    while (pMiHTable)
    {
      /****************************************************************/
      /* If match is found, return match, otherwise go to next entry  */
      /****************************************************************/
      if ( pMiHTable->fId == menuItemId )
        return pMiHTable->fHelpId;

      pMiHTable = pMiHTable->pmitNext;
    }
  }
  return 0;
}


/*------------------------------------------------------------------------------
| deleteHelpIdTable                                                            |
|  Local function to delete a previously built help id table                   |
------------------------------------------------------------------------------*/
void IMenuPrivate::deleteHelpIdTable( IFrameWindow* owner )
{
  if (owner->fFrameWindowData->fpHelpList)
  {
    MIHELPTBL* pMiHTable = (MIHELPTBL*)(owner->fFrameWindowData->fpHelpList);
    MIHELPTBL* pNext;
    do {
      pNext = pMiHTable->pmitNext;
      delete pMiHTable;
      pMiHTable = pNext;
    } while (pMiHTable);
  }
  owner->fFrameWindowData->fpHelpList = 0;
}

