/*******************************************************************************
* FILE NAME: imnitem.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imnitem.cpp                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#pragma priority( -2147482112 )

extern "C"
{
  #define INCL_WINMENUS
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
}
#ifndef _IMENU_
  #include <imenu.hpp>
#endif
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif
#ifndef _IMNITEM_
  #include <imnitem.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMNITEM_CPP_
  #include <ipagetun.h>
#endif

//***************************************************************
// Public styles.
//***************************************************************
#pragma data_seg(ICLStaticConst)
const IMenuItem::Style
  IMenuItem::noStyle            = 0,
  IMenuItem::textItem           = MIS_TEXT,               // 0x00000001
  IMenuItem::bitmapItem         = MIS_BITMAP,             // 0x00000002
  IMenuItem::separator          = MIS_SEPARATOR,          // 0x00000004
  IMenuItem::drawItem           = MIS_OWNERDRAW,          // 0x00000008
  IMenuItem::submenu            = MIS_SUBMENU,            // 0x00000010
  IMenuItem::postSystemCommand  = MIS_SYSCOMMAND,         // 0x00000040
  IMenuItem::postHelp           = MIS_HELP,               // 0x00000080
  IMenuItem::buttonSeparator    = MIS_BUTTONSEPARATOR,    // 0x00000200
  IMenuItem::unavailable        = MIS_STATIC,             // 0x00000100
  IMenuItem::split              = MIS_BREAK,              // 0x00000400
  IMenuItem::splitWithSeparator = MIS_BREAKSEPARATOR,     // 0x00000800
  IMenuItem::classDefaultStyle  = 0;

//#ifdef IC_WIN
//  const unsigned long IMenuItem::atEnd = MF_END;
//#else
  const unsigned long IMenuItem::atEnd = (unsigned long)ULONG_MAX;
//#endif
#pragma data_seg()

#pragma data_seg(ICLNonConst)
  IMenuItem::Style IMenuItem::currentDefaultStyle = 0;
#pragma data_seg()

//***************************************************************
// Public attributes
//***************************************************************
#pragma data_seg(ICLStaticConst)
const IMenuItem::Attribute
  IMenuItem::noAttribute           = 0,
  IMenuItem::noDismiss             = MIA_NODISMISS,       // 0x00000020
  IMenuItem::framed                = MIA_FRAMED,          // 0x00001000
  IMenuItem::checked               = MIA_CHECKED,         // 0x00002000
  IMenuItem::disabled              = MIA_DISABLED,        // 0x00004000
  IMenuItem::highlighted           = MIA_HILITED,         // 0x00008000
  IMenuItem::classDefaultAttribute = noAttribute;
#pragma data_seg()

#pragma data_seg(ICLNonConst)
  IMenuItem::Attribute IMenuItem::currentDefaultAttribute = noAttribute;
#pragma data_seg()

class IMenuItemData : public IBase {
/*------------------------------------------------------------------------------
| The IMenuItemData class encapsulates private data used by the IMenuItem      |
| class.  An object of this class is created in the IMenuItem constructors.    |
------------------------------------------------------------------------------*/
public:
  IMenuItemData ( );

virtual
 ~IMenuItemData ( );

unsigned long
  afHelpId;
}; // IMenuItemData

/*------------------------------------------------------------------------------
| IMenuItemData::IMenuItemData                                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMenuItemData::IMenuItemData ( )
  : afHelpId( 0 )
{ }

/*------------------------------------------------------------------------------
| IMenuItemData::~IMenuItemData                                                |
|                                                                              |
------------------------------------------------------------------------------*/
IMenuItemData::~IMenuItemData ( )
{ }

/*------------------------------------------------------------------------------
| IMenuItem::IMenuItem                                                         |
------------------------------------------------------------------------------*/
IMenuItem::IMenuItem  ( unsigned long       id,
                        const Style         &style,
                        const Attribute     &attribute)
 : strClItemText(),
   iPosition(IMenuItem::atEnd),
   afStyle(style.asUnsignedLong()),
   afExtendedStyle(style.asExtendedUnsignedLong()),
   afAttribute(attribute.asUnsignedLong()),
   itemId(id),
   hwndSubmenu(0),
   hBitmapHandle(0),
   fMenuItemData( new IMenuItemData )
{ }

/*------------------------------------------------------------------------------
| IMenuItem::IMenuItem                                                         |
------------------------------------------------------------------------------*/
IMenuItem::IMenuItem( const IMenuItem&  menuItem )
  : strClItemText( menuItem.strClItemText ),
    iPosition( menuItem.iPosition ),
    afStyle( menuItem.afStyle ),
    afExtendedStyle( menuItem.afExtendedStyle ),
    afAttribute( menuItem.afAttribute ),
    itemId( menuItem.itemId ),
    hwndSubmenu( menuItem.hwndSubmenu ),
    hBitmapHandle( menuItem.hBitmapHandle ),
    fMenuItemData( new IMenuItemData )
{ }

/*------------------------------------------------------------------------------
| IMenuItem::operator=                                                         |
------------------------------------------------------------------------------*/
IMenuItem& IMenuItem::operator=( const IMenuItem&  menuItem )
{
  if (this == &menuItem)
    return *this;

  strClItemText   = menuItem.strClItemText;
  iPosition       = menuItem.iPosition;
  afStyle         = menuItem.afStyle;
  afExtendedStyle = menuItem.afExtendedStyle;
  afAttribute     = menuItem.afAttribute;
  itemId          = menuItem.itemId;
  hwndSubmenu     = menuItem.hwndSubmenu;
  hBitmapHandle   = menuItem.hBitmapHandle;
  fMenuItemData->afHelpId = menuItem.fMenuItemData->afHelpId;

  return *this;
}

/*------------------------------------------------------------------------------
| IMenuItem::~IMenuItem                                                        |
------------------------------------------------------------------------------*/
IMenuItem::~IMenuItem()
{
  if (fMenuItemData)
    delete fMenuItemData;
}

/*------------------------------------------------------------------------------
| IMenuItem::setDrawItem                                                       |
------------------------------------------------------------------------------*/
IMenuItem&  IMenuItem::setDrawItem    ( Boolean  state)
{
   if (state)
   {
     afStyle &= ~(textItem.asUnsignedLong());
     afStyle &= ~(separator.asUnsignedLong() | bitmapItem.asUnsignedLong() );
     afStyle |= drawItem.asUnsignedLong();
     hBitmapHandle = IBitmapHandle();
     strClItemText = IString();
   }
   else
     afStyle &= ~drawItem.asUnsignedLong();

   return *this;
}

/*------------------------------------------------------------------------------
| IMenuItem::setSeparator                                                      |
------------------------------------------------------------------------------*/
IMenuItem& IMenuItem::setSeparator   ( )
{
   afStyle &= ~(textItem.asUnsignedLong());
   afStyle &= ~(drawItem.asUnsignedLong() | bitmapItem.asUnsignedLong() );
   afStyle |= separator.asUnsignedLong();
   hBitmapHandle = IBitmapHandle();
   strClItemText = IString();
   return *this;
}

/*------------------------------------------------------------------------------
| IMenuItem::setText                                                           |
------------------------------------------------------------------------------*/
IMenuItem&  IMenuItem::setText( const char*         newText )
{
   afStyle &= ~(bitmapItem.asUnsignedLong() | separator.asUnsignedLong() |
                drawItem.asUnsignedLong() );
   afStyle |= textItem.asUnsignedLong();
   strClItemText = IString(newText);
   hBitmapHandle = 0;
   return *this;
}

/*------------------------------------------------------------------------------
| IMenuItem::setBitmap                                                         |
------------------------------------------------------------------------------*/
IMenuItem&  IMenuItem::setBitmap(const IBitmapHandle& hbmpMenuItem)
{
   afStyle &= ~(textItem.asUnsignedLong());
   afStyle &= ~(drawItem.asUnsignedLong() | separator.asUnsignedLong() );
   afStyle |= bitmapItem.asUnsignedLong();
   strClItemText = IString();
   hBitmapHandle = hbmpMenuItem;
   return *this;
}


/*------------------------------------------------------------------------------
| IMenuItem::setLayout                                                         |
------------------------------------------------------------------------------*/
IMenuItem& IMenuItem::setLayout(IMenuItem::LayoutType newLayout)
{
   afStyle &= ~(split.asUnsignedLong() | splitWithSeparator.asUnsignedLong() |
                buttonSeparator.asUnsignedLong());

   switch(newLayout)
   {
     case splitLayout:
       afStyle |= split.asUnsignedLong();
       break;
     case splitWithSeparatorLayout:
       afStyle |= splitWithSeparator.asUnsignedLong();
       break;
     case buttonSeparatorLayout:
       afStyle |= buttonSeparator.asUnsignedLong();
       break;
   } /* endswitch */
   return *this;
}

/*------------------------------------------------------------------------------
| IMenuItem::setCommand                                                        |
------------------------------------------------------------------------------*/
IMenuItem& IMenuItem::setCommand(IMenuItem::CommandType newMessageType)
{
   afStyle &= ~( postSystemCommand.asUnsignedLong() |
                 postHelp.asUnsignedLong() );
   switch (newMessageType)
   {
     case systemCommand:
       afStyle |= postSystemCommand.asUnsignedLong();
       break;
     case helpCommand:
       afStyle |= postHelp.asUnsignedLong();
       break;
   } /* endswitch */
   return *this;
}

/*------------------------------------------------------------------------------
| IMenuItem::layoutType                                                        |
------------------------------------------------------------------------------*/
IMenuItem::LayoutType IMenuItem::layoutType  ( ) const
{
   if (afStyle & split.asUnsignedLong() )
      return splitLayout;
   else
      if (afStyle & splitWithSeparator.asUnsignedLong() )
         return splitWithSeparatorLayout;
      else
         if (afStyle & buttonSeparator.asUnsignedLong() )
            return buttonSeparatorLayout;
         else
            return normalLayout;
}


/*------------------------------------------------------------------------------
| IMenuItem::commandType                                                       |
------------------------------------------------------------------------------*/
IMenuItem::CommandType IMenuItem::commandType ( ) const
{
   if (afStyle & postSystemCommand.asUnsignedLong() )
      return systemCommand;
   else
      if (afStyle & postHelp.asUnsignedLong() )
         return helpCommand;
      else
         return command;
}

/*------------------------------------------------------------------------------
| IMenuItem::defaultAttribute                                                  |
------------------------------------------------------------------------------*/
IMenuItem::Attribute  IMenuItem::defaultAttribute()
{
  return currentDefaultAttribute;
}

/*------------------------------------------------------------------------------
|  IMenuItem::defaultStyle                                                     |
------------------------------------------------------------------------------*/
IMenuItem::Style  IMenuItem::defaultStyle()
{
  return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IMenuItem::convertToGUIStyle                                                 |
|                                                                              |
| Returns base/extended style for the underlying GUI.                          |
------------------------------------------------------------------------------*/
unsigned long IMenuItem::convertToGUIStyle(const IBitFlag& guiStyle,
                                           Boolean bExtOnly) const
{
  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    return( (extendedStyle() & IMIS_EXTMASK) );
  }

  // MIS_ styles have a one-to-one correspondence to our style bits, and
  // inhabit the lower word of the base GUI style.  Therefore, return
  // that portion asis, masking out the upper word.
  return( (guiStyle.asUnsignedLong() & IMIS_MASK) );
}

/*------------------------------------------------------------------------------
| IMenuItem::setSubmenuHandle                                                  |
------------------------------------------------------------------------------*/
IMenuItem& IMenuItem::setSubmenuHandle ( const IMenuHandle &menuHandle )
{
  // Set the handle to the one passed and then check if for validity
  hwndSubmenu = menuHandle;
  if ( hwndSubmenu )
    // If non-null, then set style bit for submenu
    afStyle |= submenu.asUnsignedLong();
  else
    // otherwise if null, remove style bit for submenu
    afStyle &= ~(submenu.asUnsignedLong());
  return *this;
}

/*------------------------------------------------------------------------------
| IMenuItem::setText                                                           |
------------------------------------------------------------------------------*/
IMenuItem&  IMenuItem::setText(const IResourceId& textResId)
{
   return(setText((char*)textResId.resourceLibrary().loadString(textResId)));
}

/*------------------------------------------------------------------------------
| IMenuItem::setBitmap                                                         |
------------------------------------------------------------------------------*/
IMenuItem&  IMenuItem::setBitmap(const IResourceId& bitmapResId)
{
  return(setBitmap(bitmapResId.resourceLibrary().loadBitmap(bitmapResId)) );
}

/*------------------------------------------------------------------------------
| IMenuItem::helpId                                                            |
------------------------------------------------------------------------------*/
unsigned long IMenuItem::helpId ( ) const
{
  return fMenuItemData->afHelpId;
}

/*------------------------------------------------------------------------------
| IMenuItem::setHelpId                                                         |
------------------------------------------------------------------------------*/
IMenuItem& IMenuItem::setHelpId( unsigned long newHelpTopicId )
{
  fMenuItemData->afHelpId = newHelpTopicId;
  return *this;
}

