/*******************************************************************************
* FILE NAME: inotebk0.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     INotebook::IPageSettings                                                 *
*     INotebook::Cursor                                                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define INCL_WINSTDBOOK
  #define INCL_WINDIALOGS           // Needed for PDLGTEMPLATE in BOOKPAGEINFO
  #include <iwindefs.h>
}

#ifndef _INOTEBK_
  #include <inotebk.hpp>
#endif
#ifndef _IRESLIB_
  #include <ireslib.hpp>
#endif
#ifndef _ISTRING_
  #include <istring.hpp>
#endif

// Platform specific files must be included here so the ibase.hpp has been
// included and defined IC_MOTIF or whatever.

// Segment definitions
#ifdef IC_PAGETUNE
  #define _INOTEBK0_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| INotebook::PageSettings::Pagesettings                                        |
|                                                                              |
| Construct an instance of this class using the default values.                |
------------------------------------------------------------------------------*/
INotebook::PageSettings :: PageSettings ( )
  : savedStatusText (IString ()), savedTabText (IString ()),
    pageStyle (0), savedUserData (0)
{ }


/*------------------------------------------------------------------------------
| INotebook::PageSettings::Pagesettings                                        |
|                                                                              |
| Construct an instance of this class using one or more page attributes.       |
------------------------------------------------------------------------------*/
INotebook::PageSettings :: PageSettings ( const Attribute& newPageStyle )
  : savedStatusText (IString ()), savedTabText (IString ()),
    pageStyle (newPageStyle), savedUserData (0)
{ }


/*------------------------------------------------------------------------------
| INotebook::PageSettings::Pagesettings                                        |
|                                                                              |
| Construct an instance of this class using the tab text, status text, and     |
| one or more attributes.                                                      |
------------------------------------------------------------------------------*/
INotebook::PageSettings :: PageSettings ( const char* tabText,
                                          const char* statusText,
                                          const Attribute& newPageStyle )
  : savedStatusText (IString (statusText)), pageStyle (newPageStyle),
    savedTabText (IString (tabText)), savedUserData (0)
{
  if (statusText)                                     // Make sure attribute is
    pageStyle |= (PageSettings::statusTextOn);        // on if text specified

  if(tabText)                                         // Make sure attribute is
    if ((!(pageStyle & (PageSettings::majorTab))) &&  // on if tab text is
        (!(pageStyle & (PageSettings::minorTab))))    // specified
      pageStyle |= (PageSettings::majorTab);
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::Pagesettings                                        |
|                                                                              |
| Construct an instance of this class using the tab bitmap, status text,       |
| and one or more attributes.                                                  |
------------------------------------------------------------------------------*/
INotebook::PageSettings :: PageSettings ( const IBitmapHandle& tabBitmap,
                                          const char* statusText,
                                          const Attribute& newPageStyle )
  : savedStatusText (IString (statusText)), pageStyle (newPageStyle),
    savedTabText (IString ()), savedTabBitmap (tabBitmap),
    savedUserData (0)
{
  if (statusText)                                     // Make sure attribute is
    pageStyle |= (PageSettings::statusTextOn);        // on if text specified

  if(tabBitmap)                                       // Make sure attribute is
    if ((!(pageStyle & (PageSettings::majorTab))) &&  // on if tab text is
        (!(pageStyle & (PageSettings::minorTab))))    // specified
      pageStyle |= (PageSettings::majorTab);
}

/*------------------------------------------------------------------------------
| INotebook::PageSettings::~PageSettings                                       |
|                                                                              |
| Default destructor here for page tuning.                                     |
------------------------------------------------------------------------------*/
INotebook :: PageSettings :: ~PageSettings ( )
{ }

/*------------------------------------------------------------------------------
| INotebook::PageSettings::PageSettings                                        |
|                                                                              |
| Copy constructor to keep compiler from generating one.                       |
------------------------------------------------------------------------------*/
INotebook::PageSettings:: PageSettings (
                              const INotebook::PageSettings& pageSettings)
                              : pageStyle (pageSettings.pageStyle),
                                savedTabBitmap ( pageSettings.savedTabBitmap),
                                savedUserData  ( pageSettings.savedUserData),
                                savedStatusText( pageSettings.savedStatusText),
                                savedTabText   ( pageSettings.savedTabText )
{ }

/*------------------------------------------------------------------------------
| INotebook::PageSettings::PageSettings                                        |
|                                                                              |
| Assignment operator.                                                         |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                      operator= (const INotebook::PageSettings& pageSettings)
{
  pageStyle       = pageSettings.pageStyle;
  savedTabBitmap  = pageSettings.savedTabBitmap;
  savedUserData   = pageSettings.savedUserData;
  savedStatusText = pageSettings.savedStatusText;
  savedTabText    = pageSettings.savedTabText;
  return( *this );
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::setStatusText                                       |
|                                                                              |
| Store the status text in the PageSetting object's instance data.             |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                                     setStatusText ( const char* text )
{
  pageStyle |= (PageSettings::statusTextOn);
  savedStatusText = IString (text);
  return *this;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::setStatusText                                       |
|                                                                              |
| Store the status text in the PageSetting object's instance data.             |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                           setStatusText ( const IResourceId& resId )
{
  return setStatusText(resId.resourceLibrary().loadString (resId.id()));
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::setTabText                                          |
|                                                                              |
| Store the tab text in the PageSetting object's instance data.                |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                                      setTabText ( const char* text )
{
  savedTabText = IString (text);
  return *this;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::setTabText                                          |
|                                                                              |
| Store the tab text in the PageSetting object's instance data.                |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                              setTabText ( const IResourceId& resId )
{
  return setTabText(resId.resourceLibrary().loadString (resId.id()));
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::setTabBitmap                                        |
|                                                                              |
| Store the tab bitmap in the PageSetting object's instance data.              |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                         setTabBitmap ( const IBitmapHandle& bitmap )
{
  savedTabBitmap = bitmap;
  return *this;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::setTabBitmap                                        |
|                                                                              |
| Store the tab bitmap in the PageSetting object's instance data.              |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                            setTabBitmap ( const IResourceId& resId )
{
  return setTabBitmap(resId.resourceLibrary().loadBitmap (resId.id()));
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::setUserData                                         |
|                                                                              |
| Store the user data in the PageSetting object's instance data.               |
------------------------------------------------------------------------------*/
INotebook::PageSettings& INotebook::PageSettings ::
                                 setUserData ( unsigned long ulData )
{
  savedUserData = ulData;
  return *this;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::statusText                                          |
|                                                                              |
| Return the text of the status line associated with this page.                |
------------------------------------------------------------------------------*/
IString INotebook::PageSettings :: statusText ( ) const
{
  return savedStatusText;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::tabText                                             |
|                                                                              |
| Return the text of the tab associated with this page.                        |
------------------------------------------------------------------------------*/
IString INotebook::PageSettings :: tabText ( ) const
{
  return savedTabText;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::tabBitmap                                           |
|                                                                              |
| Return the bitmap of the tab associated with this page.                      |
------------------------------------------------------------------------------*/
IBitmapHandle INotebook::PageSettings :: tabBitmap ( ) const
{
  return savedTabBitmap;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::userData                                            |
|                                                                              |
| Return the application data in the page's reserved storage.                  |
------------------------------------------------------------------------------*/
unsigned long INotebook::PageSettings :: userData ( ) const
{
  return savedUserData;
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::isAutoSize                                          |
------------------------------------------------------------------------------*/
IBase::Boolean INotebook::PageSettings :: isAutoSize ( ) const
{
  return( (pageStyle & PageSettings::autoPageSize) ? true : false );
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::isStatusTextOn                                      |
------------------------------------------------------------------------------*/
IBase::Boolean INotebook::PageSettings :: isStatusTextOn ( ) const
{
  return( (pageStyle & PageSettings::statusTextOn) ? true : false );
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::isMajorTab                                          |
|                                                                              |
| Query whether this page has a major tab.                                     |
------------------------------------------------------------------------------*/
IBase::Boolean INotebook::PageSettings :: isMajorTab ( ) const
{
  return( (pageStyle & PageSettings::majorTab) ? true : false );
}


/*------------------------------------------------------------------------------
| INotebook::PageSettings::isMinorTab                                          |
|                                                                              |
| Query whether this page has a minor tab.                                     |
------------------------------------------------------------------------------*/
IBase::Boolean INotebook::PageSettings :: isMinorTab ( ) const
{
  return( (pageStyle & PageSettings::minorTab) ? true : false );
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::Cursor                                                    |
|                                                                              |
| Cursor class constructor                                                     |
------------------------------------------------------------------------------*/
INotebook::Cursor :: Cursor ( const INotebook& notebook )
  : nbCl (notebook), ulClCurrentPageId (0),
    ulClReferences (0)
{ }

/*------------------------------------------------------------------------------
| INotebook::Cursor::~Cursor                                                   |
|                                                                              |
| Default destructor used for page tuning.                                     |
------------------------------------------------------------------------------*/
INotebook :: Cursor :: ~Cursor ( )
{ }


/*------------------------------------------------------------------------------
| INotebook::Cursor::setToFirst                                                |
|                                                                              |
| Sets the cursor to point to the first page of the notebook.                  |
------------------------------------------------------------------------------*/
IBase::Boolean INotebook::Cursor :: setToFirst ( )
{
  if (!nbCl.isEmpty())
  {
     ulClCurrentPageId = nbCl.firstPage();
     ulClReferences    = nbCl.ulClValidate;    // Mark cursor valid
     return true;
  }
  else
  {
     invalidate();
     return false;
  }
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::setToNext                                                 |
|                                                                              |
| Sets the cursor to point to the next page of the notebook.                   |
------------------------------------------------------------------------------*/
IBase::Boolean INotebook::Cursor :: setToNext ( )
{
  IPageHandle thisPage (ulClCurrentPageId);
  IPageHandle nextPage = nbCl.nextPage( thisPage );

  if ( nextPage )
  {
    ulClCurrentPageId = nextPage;
    ulClReferences    = nbCl.ulClValidate;    // Mark cursor valid
    return true;
  }
  else
  {
    invalidate();
    return false;
  }
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::setToPrevious                                             |
|                                                                              |
| Sets the cursor to point to the previous page in the notebook.               |
------------------------------------------------------------------------------*/
IBase::Boolean INotebook::Cursor :: setToPrevious ( )
{
  IPageHandle thisPage (ulClCurrentPageId);
  IPageHandle prevPage = nbCl.previousPage( thisPage );

  if ( prevPage )
  {
    ulClCurrentPageId = prevPage;
    ulClReferences    = nbCl.ulClValidate;    // Mark cursor valid
    return true;
  }
  else
  {
    invalidate();
    return false;
  }
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::setToLast                                                 |
|                                                                              |
| Sets the cursor to point to the last page in the notebook.                   |
------------------------------------------------------------------------------*/
IBase::Boolean INotebook::Cursor :: setToLast ( )
{
  if (!nbCl.isEmpty())
  {
     ulClCurrentPageId = nbCl.lastPage();
     ulClReferences    = nbCl.ulClValidate;
     return true;
  }
  else
  {
     invalidate();
     return false;
  }
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::first                                                     |
|                                                                              |
| Set the cursor to the first page of the notebook and return an IPageHandle   |
| object referencing the first notebook page.                                  |
------------------------------------------------------------------------------*/
IPageHandle INotebook::Cursor :: first ( )
{
  IPageHandle returnHandle (0);

  if (setToFirst())
    returnHandle = ulClCurrentPageId;

  return returnHandle;
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::last                                                      |
|                                                                              |
| Set the cursor to the last page of the notebook and return an IPageHandle    |
| object referencing the last notebook page.                                   |
------------------------------------------------------------------------------*/
IPageHandle INotebook::Cursor :: last ( )
{
  IPageHandle returnHandle (0);

  if (setToLast())
    returnHandle = ulClCurrentPageId;

  return returnHandle;
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::next                                                      |
|                                                                              |
| Set the cursor to the next page of the notebook and return an IPageHandle    |
| object referencing that next page.                                           |
------------------------------------------------------------------------------*/
IPageHandle INotebook::Cursor :: next ( )
{
  IPageHandle returnHandle(0);

  if (setToNext())
    returnHandle = ulClCurrentPageId;

  return returnHandle;
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::previous                                                  |
|                                                                              |
| Set the cursor to the previous page of the notebook and return an            |
| IPageHandle object referencing that previous page.                           |
------------------------------------------------------------------------------*/
IPageHandle INotebook::Cursor :: previous ( )
{
  IPageHandle returnHandle(0);

  if (setToPrevious())
    returnHandle = ulClCurrentPageId;

  return returnHandle;
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::current                                                   |
|                                                                              |
| Return an IPageHandle referencing the page currently pointed to by the       |
| cursor.                                                                      |
------------------------------------------------------------------------------*/
IPageHandle INotebook::Cursor :: current ( ) const
{
  IPageHandle currHandle(0);

  if (isValid())
    currHandle = ulClCurrentPageId;

  return currHandle;
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::setCurrent                                                |
|                                                                              |
| Set the cursor to point to the provided page.                                |
------------------------------------------------------------------------------*/
void INotebook::Cursor :: setCurrent ( const IPageHandle& current )
{
  ulClCurrentPageId = current;
  ulClReferences    = nbCl.ulClValidate;    // Mark cursor valid
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::isValid                                                   |
|                                                                              |
| Queries whether the cursor points to a valid item.                           |
------------------------------------------------------------------------------*/
IBase::Boolean INotebook::Cursor :: isValid ( ) const
{
  if ((ulClCurrentPageId) && (ulClReferences==nbCl.ulClValidate))
    return true;
  else
    return false;
}


/*------------------------------------------------------------------------------
| INotebook::Cursor::invalidate                                                |
|                                                                              |
| Flags this cursor as not valid.                                              |
------------------------------------------------------------------------------*/
void INotebook::Cursor :: invalidate ( )
{
  ulClCurrentPageId = 0;
}
