/*******************************************************************************
* FILE NAME: inotehdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     INotebookHandler                                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define INCL_WINFRAMEMGR
  #define INCL_WINSTDBOOK
  #define INCL_WINDIALOGS           // Needed for PDLGTEMPLATE in BOOKPAGEINFO
  #include <iwindefs.h>
}

#include <inotehdr.hpp>

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif


#ifndef _INOTEBK_
  #include <inotebk.hpp>
#endif


// Segment definitions
#ifdef IC_PAGETUNE
  #define _INOTEHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| INotebookHandler::dispatchHandlerEvent                                       |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
Boolean INotebookHandler :: dispatchHandlerEvent ( IEvent &event )
{
  Boolean stopProcessingEvent = false;

  if (event.eventId() == WM_HELP  &&  !(event.window()->hasFocus()))
  {          // Focus is not on the notebook (and hence, not a tab).
     IWindow* parent = event.window()->parent();
     if (parent)
     {
        /**********************************************************/
        /* The user must have requested contextual help from a    */
        /* page window.  We will avoid the notebook's processing  */
        /* of WM_HELP, which is geared for its tabs.  If the help */
        /* request gets here, the page window must not be a frame */
        /* (it is probably a canvas).  By routing the requests to */
        /* the notebook's parent, these contextual helps can be   */
        /* resolved through the help subtable of the notebook's   */
        /* frame window.                                          */
        /**********************************************************/
        parent->handle().postEvent(event.eventId(),
                                   event.parameter1(),
                                   event.parameter2());
        stopProcessingEvent = true;
     }
  }
  else

  /******************************************************************/
  /* Store the colors that are being set and then let this message  */
  /* pass on to the notebook control.                               */
  /******************************************************************/
  if (event.eventId() == BKM_SETNOTEBOOKCOLORS)
  {
     ((INotebook*)event.window())
        ->storeNotebookColors(event.parameter1(), event.parameter2());
  }

  return stopProcessingEvent;
}


/*------------------------------------------------------------------------------
| INotebookHandler::INotebookHandler                                           |
------------------------------------------------------------------------------*/
INotebookHandler :: INotebookHandler ( )
{
}

/*------------------------------------------------------------------------------
| INotebookHandler::~INotebookHandler                                          |
------------------------------------------------------------------------------*/
INotebookHandler :: ~INotebookHandler ( )
{
}
