/*******************************************************************************
* FILE NAME: ipageevt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IPageEvent                                                               *
*     IPageRemoveEvent                                                         *
*     IPageHelpEvent                                                           *
*     IPageSelectEvent                                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINDIALOGS           // Needed for PDLGTEMPLATE in BOOKPAGEINFO.
#define INCL_WINSTDBOOK
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
}

#ifndef _IPAGEEVT_
  #include <ipageevt.hpp>
#endif

#ifndef _IHELP_
  #include <ihelp.hpp>
#endif
#ifndef _INOTEBK_
  #include <inotebk.hpp>
#endif
#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IPAGEEVT_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IPageEvent::IPageEvent                                                       |
------------------------------------------------------------------------------*/
IPageEvent::IPageEvent ( const IEvent& event )
  : IPageEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IPageEvent::IPageEvent                                                       |
------------------------------------------------------------------------------*/
IPageEvent::IPageEvent ( const IControlEvent& event )
  : IPageEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IPageEvent::IPageEvent                                                       |
| Copy constructor                                                             |
------------------------------------------------------------------------------*/
IPageEvent::IPageEvent ( const IPageEvent& event )
  : IPageEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IPageEvent::~IPageEvent                                                      |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IPageEvent::~IPageEvent ( )
{ }

/*------------------------------------------------------------------------------
| IPageRemoveEvent::IPageRemoveEvent                                           |
------------------------------------------------------------------------------*/
IPageRemoveEvent::IPageRemoveEvent ( const IEvent& event )
  : IPageRemoveEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IPageRemoveEvent::IPageRemoveEvent                                           |
------------------------------------------------------------------------------*/
IPageRemoveEvent::IPageRemoveEvent ( const IControlEvent& event )
  : IPageRemoveEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IPageRemoveEvent::IPageRemoveEvent                                           |
------------------------------------------------------------------------------*/
IPageRemoveEvent::IPageRemoveEvent ( const IPageEvent& event )
  : IPageRemoveEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IPageRemoveEvent::~IPageRemoveEvent                                          |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IPageRemoveEvent::~IPageRemoveEvent ( )
{ }

/*------------------------------------------------------------------------------
| IPageHelpEvent::IPageHelpEvent                                               |
------------------------------------------------------------------------------*/
IPageHelpEvent::IPageHelpEvent ( const IEvent& event )
  : IPageHelpEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IPageHelpEvent::IPageHelpEvent                                               |
------------------------------------------------------------------------------*/
IPageHelpEvent::IPageHelpEvent ( const IControlEvent& event )
  : IPageHelpEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IPageHelpEvent::IPageHelpEvent                                               |
------------------------------------------------------------------------------*/
IPageHelpEvent::IPageHelpEvent ( const IPageEvent& event )
  : IPageHelpEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IPageHelpEvent::~IPageHelpEvent                                              |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IPageHelpEvent::~IPageHelpEvent ( )
{ }

/*------------------------------------------------------------------------------
| IPageSelectEvent::IPageSelectEvent                                           |
------------------------------------------------------------------------------*/
IPageSelectEvent::IPageSelectEvent ( const IEvent& event )
  : IPageSelectEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IPageSelectEvent::IPageSelectEvent                                           |
------------------------------------------------------------------------------*/
IPageSelectEvent::IPageSelectEvent ( const IControlEvent& event )
  : IPageSelectEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IPageSelectEvent::IPageSelectEvent                                           |
------------------------------------------------------------------------------*/
IPageSelectEvent::IPageSelectEvent ( const IPageEvent& event )
  : IPageSelectEvent::Inherited( event )
{ }

/*------------------------------------------------------------------------------
| IPageSelectEvent::~IPageSelectEvent                                          |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IPageSelectEvent::~IPageSelectEvent ( )
{ }

/*------------------------------------------------------------------------------
| IPageEvent::pageHandle                                                       |
|                                                                              |
| Return the id of the page appropriate for the event (e.g. the page being     |
| resized).                                                                    |
------------------------------------------------------------------------------*/
IPageHandle IPageEvent::pageHandle ( ) const
{
  // Don't assume we have a notebook object.
  return IPageHandle( notebook()->handle().sendEvent(
                                             BKM_QUERYPAGEID,
                                             IEventParameter1(),
                                             IEventParameter2( BKA_TOP, 0 )));
}

/*------------------------------------------------------------------------------
| IPageEvent::notebook                                                         |
|                                                                              |
| Return the notebook.                                                         |
------------------------------------------------------------------------------*/
INotebook* IPageEvent::notebook ( ) const
{
  return (INotebook*)IWindow::windowWithHandle( (unsigned long)parameter2() );
         // hwnd is in mp2.
}

/*------------------------------------------------------------------------------
| IPageRemoveEvent::pageWindow                                                 |
|                                                                              |
| Return the window or control associated with the page.                       |
------------------------------------------------------------------------------*/
IWindow* IPageRemoveEvent::pageWindow ( ) const
{
  DELETENOTIFY* pdn = (DELETENOTIFY*)(void*)parameter2();
  IWindowHandle hwndPageWindow = pdn->hwndPage;
  IWindow* pwndPageWindow = IWindow::windowWithHandle( hwndPageWindow );
  return pwndPageWindow;
}

/*------------------------------------------------------------------------------
| IPageRemoveEvent::tabBitmap                                                  |
|                                                                              |
| Returns the bitmap of the tab associated with the page.                      |
------------------------------------------------------------------------------*/
IBitmapHandle IPageRemoveEvent::tabBitmap ( ) const
{
  DELETENOTIFY* pdn = (DELETENOTIFY*)(void*)parameter2();
  return pdn->hbmTab;
}

/*------------------------------------------------------------------------------
| IPageRemoveEvent::notebook                                                   |
|                                                                              |
| Returns the notebook.                                                        |
------------------------------------------------------------------------------*/
INotebook* IPageRemoveEvent::notebook ( ) const
{
  return (INotebook*)(this->controlWindow());
}

/*------------------------------------------------------------------------------
| IPageRemoveEvent::pageHandle                                                 |
|                                                                              |
| The page handle cannot be returned.                                          |
------------------------------------------------------------------------------*/
IPageHandle IPageRemoveEvent::pageHandle ( ) const
{
  return 0;
}

// IC_NOTYET - Must wait until help is completed.
/*------------------------------------------------------------------------------
| IPageHelpEvent::helpWindow                                                   |
|                                                                              |
| Returns the help window for the notebook.                                    |
------------------------------------------------------------------------------*/
IHelpWindow  *IPageHelpEvent::helpWindow ( ) const
{
  IWindow* pwndNotebook = this->controlWindow();
  IHelpWindow* pHelp =
                 pwndNotebook ? IHelpWindow::helpWindow( pwndNotebook ) : 0;
  return pHelp;
}

/*------------------------------------------------------------------------------
| IPageHelpEvent::pageHandle                                                   |
|                                                                              |
| Returns the notebook page handle representing the page for which help        |
| is requested.                                                                |
------------------------------------------------------------------------------*/
IPageHandle IPageHelpEvent::pageHandle ( ) const
{
  return IPageHandle( this->parameter2() );
}

/*------------------------------------------------------------------------------
| IPageHelpEvent::notebook                                                     |
|                                                                              |
| Returns a pointer to the notebook control.                                   |
------------------------------------------------------------------------------*/
INotebook* IPageHelpEvent::notebook ( ) const
{
  IWindow* pwndNotebook = this->controlWindow();
  IWindowHandle hwndNotebook = 0;
  if ( pwndNotebook )
  {                               // Know the control object.
     hwndNotebook = pwndNotebook->handle();
  }
  else
  {                               // Assume parent == owner.
     hwndNotebook = IWINDOWFROMID( this->handle(), this->controlId() );
  }
  return (INotebook*)IWindow::windowWithHandle( hwndNotebook );
}

/*------------------------------------------------------------------------------
| IPageSelectEvent::pageHandle                                                 |
|                                                                              |
| Returns the notebook page handle representing the selected notebook page.    |
------------------------------------------------------------------------------*/
IPageHandle IPageSelectEvent::pageHandle ( ) const
{
  PAGESELECTNOTIFY* psn = (PAGESELECTNOTIFY*)(void*)parameter2();
  return IPageHandle( psn->ulPageIdNew );
}

/*------------------------------------------------------------------------------
| IPageSelectEvent::notebook                                                   |
|                                                                              |
| Returns a pointer to the notebook control.                                   |
------------------------------------------------------------------------------*/
INotebook* IPageSelectEvent::notebook ( ) const
{
  PAGESELECTNOTIFY* psn = (PAGESELECTNOTIFY*)(void*)parameter2();
  return (INotebook*)IWindow::windowWithHandle( psn->hwndBook );
}

/*------------------------------------------------------------------------------
| IPageSelectEvent::previousSelectedPageHandle                                 |
|                                                                              |
| Return the new id of the previously selected notebook page.                  |
------------------------------------------------------------------------------*/
IPageHandle IPageSelectEvent::previousSelectedPageHandle ( ) const
{
  PAGESELECTNOTIFY* psn = (PAGESELECTNOTIFY*)(void*)parameter2();
  return IPageHandle( psn->ulPageIdCur );
}
