/*******************************************************************************
* FILE NAME: ipagehdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IPageHandler                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINDIALOGS           // Needed for PDLGTEMPLATE in BOOKPAGEINFO.
#define INCL_WINHELP
#define INCL_WINSTDBOOK
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
  #include <stdlib.h>
}

#ifndef _IPAGEHDR_
  #include <ipagehdr.hpp>
#endif

#ifndef _ICTLEVT_
  #include <ictlevt.hpp>
#endif
#ifndef _IHELP_
  #include <ihelp.hpp>
#endif
#ifndef _INBDIEVT_
  #include <inbdievt.hpp>
#endif
#ifndef _INOTEBK_
  #include <inotebk.hpp>
#endif
#ifndef _IPAGEEVT_
  #include <ipageevt.hpp>
#endif
#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif
#ifndef _IWCNAME_
  #include <iwcname.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IPAGEHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IPageHandler::IPageHandler                                                   |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IPageHandler:: IPageHandler ( )
  : IPageHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| IPageHandler::~IPageHandler                                                  |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IPageHandler::~IPageHandler ( )
{ }

/*------------------------------------------------------------------------------
| IPageHandler::dispatchHandlerEvent                                           |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
Boolean IPageHandler::dispatchHandlerEvent ( IEvent& event )
{
  Boolean bRc = false;

  switch ( event.eventId() )
  {
     case WM_CONTROL:
     case WM_DRAWITEM:
     {                                 // Control/draw-item event.
        IControlEvent ctlevt( event );
        IWindowHandle hwndControl = ctlevt.controlHandle();

        if ( hwndControl  &&
             IWindowClassName( hwndControl ) == WC_NOTEBOOK )
        {                              // Notebook case.
           if ( event.eventId() == WM_CONTROL )
           {
              switch ( ctlevt.parameter1().number2() )
              {
                 case BKN_NEWPAGESIZE:
                 {
                    IPageEvent pageevt( ctlevt );
                    bRc = this->resize( pageevt );
                    if ( bRc )
                    {
                       event.setResult( pageevt.result() );
                    }
                    break;
                 }
                 case BKN_PAGESELECTED:
                 {
                    IPageSelectEvent pageevt( ctlevt );
                    bRc = this->select( pageevt );
                    if ( bRc )
                    {
                       event.setResult( pageevt.result() );
                    }
                    break;
                 }
                 case BKN_PAGEDELETED:
                 {
                    IPageRemoveEvent pageevt( ctlevt );
                    bRc = this->remove(pageevt );
                    if ( bRc )
                    {
                       event.setResult( pageevt.result() );
                    }
                    break;
                 }
                 case BKN_HELP:
                 {
                    IPageHelpEvent pageevt( ctlevt );
                    bRc = this->help( pageevt );
                    if ( bRc )
                    {
                       event.setResult( pageevt.result() );
                    }
                    break;
                 }
                 default:
                    break;
              } /* endswitch */
           } /* endif WM_CONTROL */
           else if ( event.eventId() == WM_DRAWITEM )
           {
              INotebookDrawItemEvent nbdievt( event );
              bRc = this->drawTab( nbdievt );
              if ( bRc )
              {
                 event.setResult( nbdievt.result() );
              }
           }
        } /* endif notebook control */
        break;
     } /* endcase WM_CONTROL, WM_DRAWITEM */
     default:
        break;
  } /* endswitch */

  return bRc;
}

/*------------------------------------------------------------------------------
| IPageHandler::resize                                                         |
|                                                                              |
| Implemented by derived classes to process a page size change notification.   |
------------------------------------------------------------------------------*/
Boolean IPageHandler::resize ( IPageEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IPageHandler::select                                                         |
|                                                                              |
| Implemented by derived classes to process the notification that a new page   |
| is at the top of the notebook.                                               |
------------------------------------------------------------------------------*/
Boolean IPageHandler::select ( IPageSelectEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IPageHandler::remove                                                         |
|                                                                              |
| Implemented by derived classes to process a page deletion notification.      |
------------------------------------------------------------------------------*/
Boolean IPageHandler::remove ( IPageRemoveEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IPageHandler::help                                                           |
|                                                                              |
| Implemented by derived classes to process a notification that help has       |
| been requested for a notebook page.                                          |
------------------------------------------------------------------------------*/
Boolean IPageHandler::help ( IPageHelpEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IPageHandler::drawTab                                                        |
|                                                                              |
| Implemented by derived classes to draw the page tab.                         |
------------------------------------------------------------------------------*/
Boolean IPageHandler:: drawTab ( INotebookDrawItemEvent& event )
{
  return false;
}
