/*******************************************************************************
* FILE NAME: ipainhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ipainhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_WINMESSAGEMGR
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
}
#ifndef _IPAINHDR_
  #include <ipainhdr.hpp>
#endif
#ifndef _IPAINEVT_
  #include <ipainevt.hpp>
#endif
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IPAINHDR_CPP_
  #include <ipagetun.h>
#endif


/**************************************************************/
/* Default constructor for page tuning.                       */
/**************************************************************/
IPaintHandler :: IPaintHandler ( )
{ ; }


/**************************************************************/
/* Empty destructor for page tuning.                          */
/**************************************************************/
IPaintHandler :: ~IPaintHandler ( )
{ ; }


/**************************************************************/
/* dispatchHandler Event - Dispatch command functions for this handler. */
/**************************************************************/
Boolean IPaintHandler :: dispatchHandlerEvent(IEvent & evt)
{
   IBoolean bRc = false;

   if (evt.eventId() == WM_PAINT) {
      IPaintEvent ptevt(evt);
      if (ptevt.presSpaceHandle())    // if we have a pres space
      {
         bRc = paintWindow(ptevt);    // call derived class (if any) to paint
         if (bRc)
         {
            RECTL  rct = ptevt.rect().asRECTL();
            IVALIDATERECT(evt.handle(), &rct );
            evt.setResult(ptevt.result());  // copy result
         }
      } else {
               // Insure default painting occurs
               WinEndPaint(WinBeginPaint(evt.handle(), 0, 0));
             }
   } /* endif */

   return bRc;
}
