/*******************************************************************************
* FILE NAME: ipopmenu.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ipopmenu.hpp                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
   #define INCL_WINPOINTERS
   #define INCL_WINWINDOWMGR
   #define INCL_WINMENUS
   #define INCL_WININPUT
   #include <iwindefs.h>
}

#include <ipopmenu.hpp>

#include <icoordsy.hpp>
#include <iexcept.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <ireslib.hpp>
#include <iwindow.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IPOPMENU_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IPopUpMenu::IPopUpMenu                                                       |
|  Constructor. Empty pop-up for dynamic menu.                                 |
------------------------------------------------------------------------------*/
IPopUpMenu::IPopUpMenu( IWindow*           owner,
                        unsigned long      windowId,
                        const Style&       style )
                      : fPopUpMenuData( 0 )
{
   IASSERTPARM(owner != 0);
   IWindowHandle whMenu =
      this -> create( windowId,
                      0,
                      style.asUnsignedLong(),
                      WC_MENU,
                      HWND_DESKTOP,
                      owner->handle(),
                      IRectangle(),
                      0,
                      0,
                      IWindow::onTopOfSiblings );

   startHandlingEventsFor(whMenu);
}


/*------------------------------------------------------------------------------
| IPopUpMenu::IPopUpMenu                                                       |
|  Constructor. Pop-up from .rc.                                               |
------------------------------------------------------------------------------*/
IPopUpMenu::IPopUpMenu( const IResourceId& menuResId,
                        IWindow* owner )
                      : fPopUpMenuData( 0 )
{
   IWindowHandle handleClPopUp =
       menuResId.resourceLibrary().loadMenu(menuResId, owner);
   WinSetParent(handleClPopUp, HWND_DESKTOP, FALSE);
   WinSetWindowUShort(handleClPopUp,           //PM load with FID_MENU
                      QWS_ID, menuResId);      //Change it.
   startHandlingEventsFor(handleClPopUp);
}


/*------------------------------------------------------------------------------
| IPopUpMenu::~IPopUpMenu                                                      |
|  Define to override IMenu abstract class definition.                         |
------------------------------------------------------------------------------*/
IPopUpMenu::~IPopUpMenu()
{}


/*------------------------------------------------------------------------------
| IPopUpMenu::show                                                             |
------------------------------------------------------------------------------*/
IPopUpMenu& IPopUpMenu::show( const IPoint& ptAt )
{
   IPoint nativePoint = ICoordinateSystem::convertToNative(
                           ptAt, owner()->size() );
   IWindowHandle ownerHwnd = owner()->handle();
   if (!WinPopupMenu(ownerHwnd, ownerHwnd,
                     handle(),
                     nativePoint.x(), nativePoint.y(),
                     0,
                     PU_HCONSTRAIN | PU_VCONSTRAIN |
                     PU_KEYBOARD | PU_MOUSEBUTTON1 | PU_MOUSEBUTTON2) )
   {
     ITHROWGUIERROR("WinPopUpMenu");
   }
   return *this;
}
