/*******************************************************************************
* FILE NAME: iprofile.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IProfile                                                                 *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define INCL_WINSHELLDATA
  #define INCL_SHLERRORS
  #include <iwindefs.h>
}

#include <stdio.h>

#ifndef _IPROFILE_
  #include <iprofile.hpp>
#endif
#ifndef _ISTRING_
  #include <istring.hpp>
#endif
#ifndef _ITHREAD_
  #include <ithread.hpp>
#endif
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IPROFILE_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IProfile::IProfile                                                           |
------------------------------------------------------------------------------*/
IProfile::IProfile ( const char *profName )
  : profileName(profName), defApplName(), fProfileData(0)
{
   IMODTRACE_DEVELOP("IProfile::ctor (char *)");
   IASSERTPARM(profName != 0);

   //Note:  Create a temporary IString for the profile name to
   //       workaround a PMSHAPI bug that attempts to write to
   //       read-only storage
   hIni = PrfOpenProfile( IThread::current().anchorBlock(),
                          IString( profName ) );
   if (!hIni) {
      ITHROWGUIERROR("PrfOpenProfile");
   } /* endif */
}

/*------------------------------------------------------------------------------
| IProfile::IProfile                                                           |
| Copy contructor                                                              |
------------------------------------------------------------------------------*/
IProfile::IProfile ( const IProfile& aProfile )
{
   if ((aProfile.hIni != HINI_PROFILE) &&
       (aProfile.hIni != HINI_USERPROFILE) &&
       (aProfile.hIni != HINI_SYSTEMPROFILE)) {
       hIni = PrfOpenProfile(IThread::current().anchorBlock(),
                             (PSZ)aProfile.profileName);
       if (!hIni) {
          ITHROWGUIERROR("PrfOpenProfile");
       } /* endif */
   } else {
      this->hIni = aProfile.hIni;
   } /* endif */

   this->defApplName = aProfile.defApplName;
   this->profileName = aProfile.profileName;
}

/*------------------------------------------------------------------------------
| IProfile::IProfile                                                           |
|  Private Ctor. to create from Profile Handle                                 |
------------------------------------------------------------------------------*/
IProfile::IProfile ( const IProfileHandle& profile )
  : hIni(profile), defApplName(), fProfileData(0)
{
   HAB hab = WinQueryAnchorBlock(HWND_DESKTOP);
   PRFPROFILE Profile;
   Profile.cchUserName = 0;
   Profile.pszUserName = 0;
   Profile.cchSysName = 0;
   Profile.pszSysName = 0;
   char Buffer[100];

   if (PrfQueryProfile(hab, &Profile)) {
      if (hIni == HINI_USERPROFILE) {
         char* pszData = new char[Profile.cchUserName+1];
         Profile.pszUserName = (PSZ)pszData;
         Profile.cchSysName = 100;
         Profile.pszSysName = (PSZ)Buffer;
         PrfQueryProfile(hab, &Profile);
         profileName = IString(pszData);
         delete [] pszData;
      } else {
         char* pszData = new char[Profile.cchSysName];
         Profile.cchUserName = 100;
         Profile.pszUserName = (PSZ)Buffer;
         Profile.pszSysName = (PSZ)pszData;
         PrfQueryProfile(hab, &Profile);
         profileName = IString(pszData);
         delete [] pszData;
      } /* endif */
   } /* endif */
}

/*------------------------------------------------------------------------------
| IProfile::systemProfile                                                      |
------------------------------------------------------------------------------*/
IProfile IProfile::systemProfile ( )
{
   IMODTRACE_DEVELOP("IProfile::systemProfile");
   return IProfile(HINI_SYSTEMPROFILE);
}

/*------------------------------------------------------------------------------
| IProfile::userProfile                                                        |
------------------------------------------------------------------------------*/
IProfile IProfile::userProfile ( )
{
   IMODTRACE_DEVELOP("IProfile::userProfie");
   return IProfile(HINI_USERPROFILE);
}


/*------------------------------------------------------------------------------
| IProfile::operator =                                                         |
------------------------------------------------------------------------------*/
IProfile& IProfile::operator =(const IProfile& aProfile)
{
   /* Note: Ensure self assignment works */
   if(this->hIni != aProfile.hIni)
   {
     if ((hIni != HINI_PROFILE) &&
         (hIni != HINI_USERPROFILE) &&
         (hIni != HINI_SYSTEMPROFILE)) {

         PrfCloseProfile(hIni);           // close file if not system & user

         hIni = PrfOpenProfile(IThread::current().anchorBlock(),
                              (PSZ)aProfile.profileName);
         if (!hIni) {
            ITHROWGUIERROR("PrfOpenProfile");
         } /* endif */
     } else {
        this->hIni = aProfile.hIni;
     } /* endif */

     this->defApplName = aProfile.defApplName;
     this->profileName = aProfile.profileName;
   } /* endif */
   return *this;
}

/*------------------------------------------------------------------------------
| IProfile::name                                                               |
| Return profile name                                                          |
------------------------------------------------------------------------------*/
IString IProfile::name ( ) const
{
   return profileName;
}

/*------------------------------------------------------------------------------
| IProfile::handle                                                             |
| Return the profile handle                                                    |
------------------------------------------------------------------------------*/
IProfileHandle IProfile::handle() const
{
  return hIni;
}

/*------------------------------------------------------------------------------
| IProfile::defaultApplicationName                                             |
| Return default application name                                              |
------------------------------------------------------------------------------*/
IString IProfile::defaultApplicationName ( ) const
{
   return defApplName;
}

/*------------------------------------------------------------------------------
| IProfile::setDefaultApplicationName                                          |
| Sets the default application name                                            |
------------------------------------------------------------------------------*/
IProfile& IProfile::setDefaultApplicationName ( const char *applName )
{
   defApplName = IString(applName);
   return *this;
}

/*------------------------------------------------------------------------------
| IProfile::~IProfile                                                          |
------------------------------------------------------------------------------*/
IProfile::~IProfile ( )
{
   IMODTRACE_DEVELOP("IProfile::dtor");
   if ((hIni != HINI_PROFILE) &&
       (hIni != HINI_USERPROFILE) &&
       (hIni != HINI_SYSTEMPROFILE)) {
      PrfCloseProfile(hIni);           // close file if not system & user
   } /* endif */
// delete fProfileData; when IProfileData is defined.
}

/*------------------------------------------------------------------------------
| IProfile::elementWithKey                                                     |
------------------------------------------------------------------------------*/
IString IProfile::elementWithKey ( const char *key, const char *applName ) const
{
   IMODTRACE_DEVELOP("IProfile::elementWithKey");
   IASSERTPARM(key != 0);

   if (!applName) {
      IASSERTPARM(defApplName.length() > 0);
   } /* endif */

   unsigned long ulLen = length(key, applName);

   if (ulLen > 0) {
      char* buffer = new char[ulLen + 1];
      /**************************/
      /* Query string first.    */
      /**************************/
      IGUIErrorInfo clear("Clear Last Error");
      PrfQueryProfileString(hIni,
                (applName) ? (PSZ)applName : (PSZ)defApplName,
                (PSZ)key, (PSZ)"", (PSZ)buffer, ulLen + 1);

      IGUIErrorInfo err("PrfQueryProfileString");
      if (err.errorId() != 0) {
         if (err.errorId() == PMERR_INVALID_ASCIIZ ) {
            /**************************/
            /* Query binary data.     */
            /**************************/
            if (!PrfQueryProfileData(hIni,
                   (applName) ? (PSZ)applName : (PSZ)defApplName,
                   (PSZ)key, buffer, &ulLen + 1) ) {
                ITHROWGUIERROR("PrfQueryProfileData");
            } /* endif */
         } else {
            IInvalidRequest ex(err.text(), err.errorId(), unrecoverable);
            ITHROW(ex);
         } /* endif */

      } /* endif */

      IString str(buffer, ulLen);
      delete buffer;
      return str;

   } /* endif */
   return IString();
}

/*------------------------------------------------------------------------------
| IProfile::integerWithKey                                                     |
------------------------------------------------------------------------------*/
long IProfile::integerWithKey ( const char *key, const char *applName ) const
{
   IMODTRACE_DEVELOP("IProfile::integerWithKey");
   IASSERTPARM(key != 0);
   if (!applName) {
      IASSERTPARM(defApplName.length() > 0);
   } /* endif */

   long lResult;
   unsigned long resultSize = sizeof(lResult);

   if (!PrfQueryProfileData(hIni,
               (applName) ? (PSZ)applName : (PSZ)defApplName,
               (PSZ)key, (PVOID)&lResult, &resultSize)) {
      ITHROWGUIERROR("PrfQueryProfileData");
   } /* endif */

   return lResult;
}


/*------------------------------------------------------------------------------
| IProfile::addOrReplaceElementWithKey                                         |
|   Write string data to file.                                                 |
------------------------------------------------------------------------------*/
IProfile& IProfile::addOrReplaceElementWithKey ( const char *key,
                                                const IString& data,
                                                const char *applName)
{
   IMODTRACE_DEVELOP("IProfile::addOrReplaceElementWithKey");
   IASSERTPARM(key != 0);
   if (!applName) {
      IASSERTPARM(defApplName.length() > 0);
   } /* endif */

   if (!PrfWriteProfileData(hIni,
                             (applName) ? (PSZ)applName : (PSZ)defApplName,
                             (PSZ)key, (PSZ)data, data.length() )) {
      ITHROWGUIERROR("PrfWriteProfileString");
   } /* endif */
   return *this;
}


/*------------------------------------------------------------------------------
| IProfile::addOrReplaceElementWithKey                                         |
|   Write binary data to file.                                                 |
------------------------------------------------------------------------------*/
IProfile& IProfile::addOrReplaceElementWithKey
                            ( const char *key, long data, const char *applName)
{
   IMODTRACE_DEVELOP("IProfile::addOrReplaceElementWithKey");
   IASSERTPARM(key != 0);
   if (!applName) {
      IASSERTPARM(defApplName.length() > 0);
   } /* endif */

   Boolean bSuccess;

   if (!PrfWriteProfileData(hIni,
                           (applName) ? (PSZ)applName : (PSZ)defApplName,
                           (PSZ)key, (void*)&data, sizeof(data))) {
      ITHROWGUIERROR("PrfWriteProfileData");
   } /* endif */
   return *this;
}

/*------------------------------------------------------------------------------
| IProfile::deleteElementWithKey                                               |
------------------------------------------------------------------------------*/
IProfile& IProfile::deleteElementWithKey ( const char *key,
                                           const char *applName)
{
   IMODTRACE_DEVELOP("IProfile::deleteElementWithKey");
   IASSERTPARM(key != 0);
   if (!applName) {
      IASSERTPARM(defApplName.length() > 0);
   } /* endif */

   if (!PrfWriteProfileString(hIni,
                   (applName) ? (PSZ)applName : (PSZ)defApplName,
                   (PSZ)key, 0) ) {
      ITHROWGUIERROR("PrfWriteProfileString");
   } /* endif */

   return *this;
}


/*------------------------------------------------------------------------------
| IProfile::deleteElementWithApplication                                       |
------------------------------------------------------------------------------*/
IProfile& IProfile::deleteElementWithApplication ( const char *applName)
{
   IMODTRACE_DEVELOP("IProfile::deleteElementWithApplication");
   if (!applName) {
      IASSERTPARM(defApplName.length() > 0);
   } /* endif */

   if (!PrfWriteProfileString(hIni,
                   (applName) ? (PSZ)applName : (PSZ)defApplName, 0, 0 )) {
      ITHROWGUIERROR("PrfWriteProfileString");
   } /* endif */
   return *this;
}

/*------------------------------------------------------------------------------
| IProfile::numberOfApplications                                               |
------------------------------------------------------------------------------*/
unsigned long IProfile::numberOfApplications () const
{
   IMODTRACE_DEVELOP("IProfile::numberOfApplications");

   unsigned long
     result = 0;

   IProfile::Cursor cursor((IProfile&)*this);

   for ( cursor.setToFirst(); cursor.isValid(); cursor.setToNext() )
     result++;

   return result;
}

/*------------------------------------------------------------------------------
| IProfile::numberOfKeys                                                       |
------------------------------------------------------------------------------*/
unsigned long IProfile::numberOfKeys ( const char *applName) const
{
   IMODTRACE_DEVELOP("IProfile::numberOfKeys");
   if (!applName) {
      IASSERTPARM(defApplName.length() > 0);
   } /* endif */

   IProfile::Cursor cursor((IProfile&)*this,
                           (applName) ? (PSZ)applName : (PSZ)defApplName );

   unsigned long
     result = 0;

   for ( cursor.setToFirst(); cursor.isValid(); cursor.setToNext() )
     result++;

   return result;
}

/*------------------------------------------------------------------------------
| IProfile::containsApplication                                                |
------------------------------------------------------------------------------*/
Boolean IProfile::containsApplication ( const char *applName ) const
{
  IMODTRACE_DEVELOP("IProfile::containsApplication");
  IASSERTPARM(applName != 0);

  IProfile::Cursor cursor((IProfile&)*this);

  for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext() ) {
      if (applicationOrKeyAt(cursor) == applName) {
         return true;
      } /* endif */
  } /* endfor */
  return false;
}

/*------------------------------------------------------------------------------
| IProfile::containsKeyName                                                    |
------------------------------------------------------------------------------*/
Boolean IProfile::containsKeyName( const char *key,
                                   const char *applName ) const
{
  IMODTRACE_DEVELOP("IProfile::containsKeyName");

  IProfile::Cursor cursor((IProfile&)*this,
                          (applName) ? applName : (const char*)defApplName );

  for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext() ) {
      if (applicationOrKeyAt(cursor) == key) {
         return true;
      } /* endif */
  } /* endfor */
  return false;
}


/*------------------------------------------------------------------------------
| IProfile::length                                                             |
|   Private function to return the length of the profile data                  |
------------------------------------------------------------------------------*/
unsigned long IProfile::length(const char *keyName, const char *applName) const
{
   unsigned long ulSize;
   if (PrfQueryProfileSize(hIni,
              (applName) ? (PSZ)applName : (PSZ)defApplName,
              (PSZ)keyName, (unsigned long *) &ulSize)) {
      return ulSize;
   } else {
      ITHROWGUIERROR("PrfQueryProfileSize");
   } /* endif */
   return 0;
}

/*------------------------------------------------------------------------------
| IProfile::applicationOrKeyAt                                                 |
------------------------------------------------------------------------------*/
IString IProfile::applicationOrKeyAt ( const Cursor &cursor ) const
{
   IMODTRACE_DEVELOP("IProfile::applicationOrKeyAt");
   IASSERTPARM(cursor.isValid());

   // cursor pos marks start of app/key name
   return IString( ((char*)cursor.strCl) + cursor.pos - 1 );
}

/*------------------------------------------------------------------------------
| IProfile::Cursor::~Cursor                                                    |
------------------------------------------------------------------------------*/
IProfile::Cursor::~Cursor ( )
  {
//delete fCursorData; when IProfileCursorData is defined.
  }

/*------------------------------------------------------------------------------
| IProfile::Cursor::Cursor                                                     |
------------------------------------------------------------------------------*/
IProfile::Cursor::Cursor ( IProfile  &profile )
  : pos(1), strCl(), applNameCl(), fCursorData(0)
{
   IMODTRACE_DEVELOP("IProfile::Cursor::ctor1");
   initCursor(profile.handle(), 0);
}

/*------------------------------------------------------------------------------
| IProfile::Cursor:: Cursor                                                    |
------------------------------------------------------------------------------*/
IProfile::Cursor:: Cursor ( IProfile  &profile, const char *applName)
  : pos(1), strCl(), fCursorData(0)
{
   IMODTRACE_DEVELOP("IProfile::Cursor::ctor2");
   if (applName) {
      applNameCl = applName;
   } else {
      applNameCl = profile.defaultApplicationName();
   } /* endif */
   initCursor(profile.handle(), applNameCl);
}

/*------------------------------------------------------------------------------
| IProfile::Cursor::setToFirst                                                 |
| Have the cursor point to the first element                                   |
------------------------------------------------------------------------------*/
IBase::Boolean IProfile::Cursor :: setToFirst ()
{
this->pos = 1;
// Valid if not off end...
return this->pos < this->strCl.length();
}

/*------------------------------------------------------------------------------
| IProfile::Cursor::setToNext                                                  |
| Have the cursor point to the next element                                    |
------------------------------------------------------------------------------*/
IBase::Boolean IProfile::Cursor :: setToNext ()
{
  // Skip to next null...
  this->pos = this->strCl.indexOf( '\0', this->pos );
  // If not found, set position off end...
  if ( this->pos == 0 )
    this->pos == this->strCl.length() + 1;
  // Else, point to next byte...
  else
    this->pos++;
  // Valid if not off end...
  return this->pos < this->strCl.length();
}

/*------------------------------------------------------------------------------
| IProfile::Cursor::setToPrevious                                              |
| Have the cursor point to the previous element                                |
------------------------------------------------------------------------------*/
IBase::Boolean IProfile::Cursor :: setToPrevious ()
{
  // If not at start, back up to previous null...
  if ( this->pos > 1 && this->pos < this->strCl.length() )
    this->pos = this->strCl.lastIndexOf( '\0', this->pos - 2 );
  // Else, leave invalid...
  else
    this->pos = this->strCl.length();
  // If not found, then we must now be at start...
  if ( this->pos == 0 )
    this->pos = 1;
  // Else, position at byte past the null...
  else
    this->pos++;

  // Valid if not off end...
  return this->pos < this->strCl.length();
}

/*------------------------------------------------------------------------------
| IProfile::Cursor::setToLast                                                  |
| Have the cursor point to the last element                                    |
------------------------------------------------------------------------------*/
IBase::Boolean IProfile::Cursor :: setToLast ()
{
   // Position at last null (before the terminating one).
   this->pos = this->strCl.lastIndexOf( '\0', this->strCl.length() - 2 );
   // Advance to next byte.
   this->pos++;

   // Valid if not off end...
   return this->pos < this->strCl.length();
}

/*------------------------------------------------------------------------------
| IProfile::Cursor :: isValid                                                  |
------------------------------------------------------------------------------*/
Boolean IProfile::Cursor :: isValid (Boolean bCheckFile) const
{
   if ( this->pos >= this->strCl.length() ) {
      return false;
   } else {
      if (bCheckFile) {

         unsigned long ulLen = 0;
         const char
          *p = ( applNameCl.length() == 0 ) ? (char*)0 : (char*)applNameCl;
         PrfQueryProfileSize(hProfileCl, p, 0, &ulLen);
         IString
           current( 0, ulLen );
         if (ulLen) {
           PrfQueryProfileString(hProfileCl, p, 0,
                                 "", (char *)current, ulLen+1);
         }
         return (current == strCl);
      } else {
      return true;
      }
   } /* endif */
}

/*------------------------------------------------------------------------------
| IProfile::Cursor::invalidate                                                 |
| Invalidate the cursor                                                        |
------------------------------------------------------------------------------*/
void IProfile::Cursor :: invalidate ()
{
  this->pos = this->strCl.length();
}

/*------------------------------------------------------------------------------
| IProfile::Cursor::initCursor                                                 |
|   Read profile for all application names or key names                        |
|                                                                              |
|   On exit, strCl will contain:                                               |
|      | entry1 | 0 | entry2 | 0 | ... | entry n | 0 | 0 |                     |
|                                                      A                       |
|                  Terminating null of the IString -----                       |
------------------------------------------------------------------------------*/
void IProfile::Cursor::initCursor (const IProfileHandle& hProfile,
                                   const char *applName )
{
   IMODTRACE_DEVELOP("IProfile::Cursor::initCursor");

   hProfileCl = hProfile;
   unsigned long ulLen;

   PrfQueryProfileSize(hProfile, applName, 0, &ulLen);

   this->strCl = IString( 0, ulLen );

   if (ulLen) {
      PrfQueryProfileString(hProfile, applName, 0, "", (char*)strCl, ulLen+1);
   } else {
      strCl = "";
   }

}

/*------------------------------------------------------------------------------
| IProfile::deleteProfile                                                      |
------------------------------------------------------------------------------*/
void IProfile::deleteProfile ( const char* profileName )
{
  // Do not allow the user or system profiles to be removed
  if (( IProfile::systemProfile().name() == profileName ) ||
      ( IProfile::userProfile().name()   == profileName ))
  {
    return;
  }

  // Delete the profile file
  remove(profileName);

}

