/*******************************************************************************
* FILE NAME: ireslib1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of IDynamicLinkLibrary and it's      *
*   related classes.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define INCL_DOSMODULEMGR
  #define INCL_DOSRESOURCES    /* Resource types */
  #define INCL_DOSINFOSEG
  #define INCL_DOSPROCESS
  #define INCL_WINERRORS
  #include <iwindefs.h>
}

#include <ireslib.hpp>
#include <iresstat.hpp>
#include <istring.hpp>
#include <irect.hpp>
#include <itrace.hpp>
#include <iset.h>
#include <iexcept.hpp>
#include <ireslock.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IRESLIB1_CPP_
  #include <ipagetun.h>
#endif


/* Used to manage shared Handles */
class IDLLModule : public IBase
{
  public:
   IDLLModule(const char* pszModuleName);
   IDLLModule(const char* pszModuleName,const  IModuleHandle& handle);
   ~IDLLModule();


       IString name() const      { return strClName;  }

 Boolean  operator ==(const IDLLModule& hmod) const;
 unsigned long useCount() const { return ulClUseCount;    }
 unsigned long addRef();
 unsigned long removeRef();
IModuleHandle  handle() const    { return hmodCl; }
      IString asDebugInfo() const;
 private:
   IString       strClName;
   IModuleHandle hmodCl;
   unsigned long ulClUseCount;
};

/*------------------------------------------------------------------------------
| IDLLModule::IDLLModule                                                       |
------------------------------------------------------------------------------*/
IDLLModule :: IDLLModule(const char* pszModuleName)
        : strClName(pszModuleName), ulClUseCount(0) {}

/*------------------------------------------------------------------------------
| IDLLModule::IDLLModule                                                       |
------------------------------------------------------------------------------*/
IDLLModule ::   IDLLModule(const char* pszModuleName, const IModuleHandle& handle)
        : strClName(pszModuleName), ulClUseCount(1), hmodCl(handle) {}

/*------------------------------------------------------------------------------
| IDLLModule::~IDLLModule                                                      |
------------------------------------------------------------------------------*/
IDLLModule ::   ~IDLLModule() {}

/*------------------------------------------------------------------------------
| IDLLModule::operator ==                                                      |
------------------------------------------------------------------------------*/
Boolean IDLLModule :: operator ==(const IDLLModule& hmod) const
{
  return( (hmod.name() == name()) ? true : false );
}


class IDLLSet : public ISet<IDLLModule*>
{
public:
   IDLLSet ( );
  ~IDLLSet ( );
};

/*------------------------------------------------------------------------------
| IDLLSet::IDLLSet() : ISet<IDLLModule*>                                       |
------------------------------------------------------------------------------*/
IDLLSet :: IDLLSet() : ISet<IDLLModule*>(5) {}

/*------------------------------------------------------------------------------
| IDLLSet::~IDLLSet                                                            |
------------------------------------------------------------------------------*/
IDLLSet :: ~IDLLSet() {}


/*------------------------------------------------------------------------------
| IResLibStaticPtr::deleteDLLSet                                               |
------------------------------------------------------------------------------*/
void IResLibStaticPtr :: deleteDLLSet()
{
  if(pDLLSet)
    delete pDLLSet;
}

/*------------------------------------------------------------------------------
| IResLibStaticPtr::dllSet                                                     |
------------------------------------------------------------------------------*/
IDLLSet&  IResLibStaticPtr ::   dllSet()
{
  if(pDLLSet==0)
    pDLLSet = new IDLLSet;
  return *pDLLSet;
}

/***************************************************************/
/*                                                             */
/* IDLLModule Strategy:                                        */
/*  Calling dllSet() will create IDLLSet if it doesn't exist   */
/*                                                             */
/*  IDynamicLinkLibrary ctor adds IDLLModule object if it      */
/*     doesn't already exist                                   */
/*  addRef() by IDynamicLinkLibrary::open()                    */
/*  removeRef() by IDynamicLinkLibrary::close()                */
/*  IDllModule deleteted by IDynamicLinkLibrary dtor           */
/*    if 0 useCount.                                           */
/*                                                             */
/*                                                             */
/***************************************************************/


/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::IDynamicLinkLibrary                                     |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary :: IDynamicLinkLibrary(const char* pszResourceFile)
{
  IMODTRACE_DEVELOP("DynLnkLib::DynLnkLib");
  if(pszResourceFile!=0)
  {
     ITRACE_ALL(IString("DLL: ")+pszResourceFile);
     /* Lock Resource Table */
     IResourceLock reslibLock(IResLibStaticPtr::libraryKey());
     Boolean fFound = false;

     IDLLModule* pdllNew = new IDLLModule(pszResourceFile);
     IDLLModule* pdllCollection;
     IDLLSet::Cursor rescur(IResLibStaticPtr::dllSet());
     for(rescur.setToFirst(); rescur.isValid(); rescur.setToNext())
     {
        pdllCollection = IResLibStaticPtr::dllSet().elementAt(rescur);
        ITRACE_ALL(pdllCollection->asDebugInfo());
        if (*pdllCollection==*pdllNew)
        {
            pmodClResource = pdllCollection;
            fFound=true;
            break;
        }
     }
     if(!fFound)
     {
        pmodClResource = pdllNew;
        IResLibStaticPtr::dllSet().add(pmodClResource);
     }
     else
        delete pdllNew;
  open();
  }
}


/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::IDynamicLinkLibrary                                     |
|   Copy Constructor                                                           |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary :: IDynamicLinkLibrary(const IDynamicLinkLibrary& reslib)
              : pmodClResource(reslib.pmodClResource)
{
  IMODTRACE_DEVELOP("DynLnkLib::DynLnkLib(const &)");
  if(pmodClResource!=0)
  {
     /* Lock Resource Table */
     IResourceLock reslibLock(IResLibStaticPtr::libraryKey());
     pmodClResource->addRef();
  }
}

/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::IDynamicLinkLibrary                                     |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary :: IDynamicLinkLibrary   (const IModuleHandle& modH)
{
  IMODTRACE_DEVELOP("DynLnkLib::DynLnkLib(modH)");

  /* Lock Resource Table */
  IResourceLock reslibLock(IResLibStaticPtr::libraryKey());
  Boolean fFound = false;

  IDLLModule* pdll;
  IDLLSet::Cursor rescur(IResLibStaticPtr::dllSet());
  for(rescur.setToFirst(); rescur.isValid(); rescur.setToNext())         // BB
  {
    pdll = IResLibStaticPtr::dllSet().elementAt(rescur);
    if (pdll->handle()!= 0 && pdll->handle()==modH)
    {
        fFound=true;
        pmodClResource = pdll;
        break;
    }
  }
  if(!fFound)
  {
     char name[512];
     unsigned long ulErrorCode;
     ulErrorCode = DosQueryModuleName(modH, sizeof(name), (PCHAR)name);
     if(ulErrorCode!=0)
     {
          ITHROWSYSTEMERROR( ulErrorCode,
                             "DosQueryModuleName",
                             IErrorInfo::accessError,
                             IException::recoverable);
     }
     // Special constructor for existing hmod
     pmodClResource = new IDLLModule(name, modH);
     IResLibStaticPtr::dllSet().add(pmodClResource);
  }
  pmodClResource->addRef();

}


/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::operator=                                               |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary& IDynamicLinkLibrary ::
                        operator=(const IDynamicLinkLibrary& reslib)
{

  IMODTRACE_DEVELOP("DynLnkLib::operator=");
  if(this != &reslib)
  {
     IResourceLock reslibLock(IResLibStaticPtr::libraryKey());
     if(reslib.pmodClResource!=0)
        reslib.pmodClResource->addRef();
     if(pmodClResource!=0)
     {
       pmodClResource->removeRef();
       if(pmodClResource->useCount()==0)
       {
         IResLibStaticPtr::dllSet().remove(pmodClResource);
         delete pmodClResource;
         pmodClResource = 0;
       }
     }
     pmodClResource = reslib.pmodClResource;
  }
  return *this;
}



/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::~IDynamicLinkLibrary                                    |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary :: ~IDynamicLinkLibrary()
{
  IMODTRACE_DEVELOP("DynLnkLib::~DynLnkLibrary");
  if (handle())
    close();

  if(pmodClResource!=0 && pmodClResource->useCount()==0)
  {
     if ( IResLibStaticPtr::dllSet().contains(pmodClResource) )
     {
       IResLibStaticPtr::dllSet().remove(pmodClResource);
       delete pmodClResource;
     }
     pmodClResource = 0;
  }
}

/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::open                                                    |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary& IDynamicLinkLibrary :: open()
{
  IMODTRACE_ALL("DynLnkLib::open");
  if(pmodClResource!=0)
     pmodClResource->addRef();
  return *this;
}

/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::close                                                   |
------------------------------------------------------------------------------*/
IDynamicLinkLibrary& IDynamicLinkLibrary :: close()
{
  IMODTRACE_ALL("DynLnkLib::close");
   /* free the reference to the DLL */
   if (pmodClResource)
      pmodClResource->removeRef();
  return *this;
}


/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::fileName                                                |
------------------------------------------------------------------------------*/
IString IDynamicLinkLibrary::fileName() const
{
   IMODTRACE_ALL("DynLnkLib::fileName");
   char temp[512];
   unsigned long ulErrorCode;
   ulErrorCode = DosQueryModuleName(handle(), sizeof(temp), (PCHAR)temp);
   if(ulErrorCode!=0)
   {
      ITHROWSYSTEMERROR( ulErrorCode,
                         "DosQueryModuleName",
                         IErrorInfo::accessError,
                         IException::recoverable);
   }
   ITRACE_ALL(temp);

   return IString(temp);};


/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::isOpen                                                  |
------------------------------------------------------------------------------*/
Boolean IDynamicLinkLibrary::isOpen() const
{
  IMODTRACE_ALL("DynLnkLib::isOpen");
  if(pmodClResource!=0 && pmodClResource->handle()!=0)
    return true;
  else
    return false;
}


/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::addRef                                                  |
------------------------------------------------------------------------------*/
unsigned long IDynamicLinkLibrary::addRef(const IModuleHandle& modh)
{
     IMODTRACE_ALL("DynLnkLib::addRef");
     IResourceLock reslibLock(IResLibStaticPtr::libraryKey());
     unsigned long ulUseCount =0;
     IDLLModule* pdllCollection;

     IDLLSet::Cursor rescur(IResLibStaticPtr::dllSet());
     for(rescur.setToFirst(); rescur.isValid(); rescur.setToNext())
     {
        pdllCollection = IResLibStaticPtr::dllSet().elementAt(rescur);
        if ((unsigned long)(pdllCollection->handle()) == (unsigned long)modh)
        {
            ulUseCount = pdllCollection->addRef();
            break;
        }
     }
     return ulUseCount;
}


/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::removeRef                                               |
------------------------------------------------------------------------------*/
unsigned long IDynamicLinkLibrary::removeRef(const IModuleHandle& modh)
{
     IMODTRACE_ALL("DynLnkLib::removeRef");
     IResourceLock reslibLock(IResLibStaticPtr::libraryKey());
     unsigned long ulUseCount =0;
     IDLLModule* pdllCollection;

     IDLLSet::Cursor rescur(IResLibStaticPtr::dllSet());
     for(rescur.setToFirst(); rescur.isValid(); rescur.setToNext())
     {
        pdllCollection = IResLibStaticPtr::dllSet().elementAt(rescur);
        if ((unsigned long)(pdllCollection->handle()) == (unsigned long)modh)
        {
            ulUseCount = pdllCollection->removeRef();
            break;
        }
     }
     return ulUseCount;
}


/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::procAddress                                             |
|   Get proceudure address by name                                             |
------------------------------------------------------------------------------*/
void* IDynamicLinkLibrary :: procAddress(const char* pszProcName) const
{
   IMODTRACE_ALL("DynLnkLib::procAddress(psz)");
   IASSERT(handle()!=0);
   unsigned long
     ulErrorCode = 0;
   int (_System *pfnTemp)() = 0;
   ulErrorCode = DosQueryProcAddr(handle(), 0, (PSZ)pszProcName, &pfnTemp);
   if(ulErrorCode!=0)
   {
     // If call failed, upper case string and try again
     IString tryUpper(pszProcName);
     tryUpper.upperCase();
     ulErrorCode = DosQueryProcAddr(handle(), 0, (PSZ)(char *)tryUpper,
                                    &pfnTemp);
   }
   if(ulErrorCode!=0)
   {
      ITHROWSYSTEMERROR( ulErrorCode,
                         IString("DosQueryProcAddr: ") +
                                  pszProcName,
                         IErrorInfo::accessError,
                         IException::recoverable);
   }
   return pfnTemp;
}

/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::procAddress                                             |
|  Get procedure address from Ordinal                                          |
------------------------------------------------------------------------------*/
void* IDynamicLinkLibrary :: procAddress(unsigned long ulOrdinal) const
{
   IMODTRACE_ALL("DynLnkLib::procAddress(ul)");
   IASSERT(handle()!=0);
   unsigned long
     ulErrorCode = 0;
   int (_System *pfnTemp)() = 0;
   ulErrorCode = DosQueryProcAddr(handle(), ulOrdinal, (PSZ)0,
                                  &pfnTemp);
   if(ulErrorCode!=0)
   {
      ITHROWSYSTEMERROR( ulErrorCode,
                         IString("DosQueryProcAddr #: ") +
                                  IString(ulOrdinal),
                         IErrorInfo::accessError,
                         IException::recoverable);
   }
   return pfnTemp;
}

/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::isEntryPoint32Bit                                       |
------------------------------------------------------------------------------*/
Boolean IDynamicLinkLibrary :: isEntryPoint32Bit
                                 ( const char *procedureName ) const
  {
  unsigned long
    result,
    ulErrorCode = 1;
  ulErrorCode = DosQueryProcType( this->handle(),
                                  0,
                                  (PSZ)procedureName,
                                  &result );
  if ( ulErrorCode )
    {
      ITHROWSYSTEMERROR( ulErrorCode,
                         IString("DosQueryProcType: ") +
                                  procedureName,
                         IErrorInfo::accessError,
                         IException::recoverable);
    }
  return result;
  }

/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::isEntryPoint32Bit                                       |
------------------------------------------------------------------------------*/
Boolean IDynamicLinkLibrary :: isEntryPoint32Bit
                                 ( unsigned long procedureOrdinal ) const
  {
  unsigned long
    result,
    ulErrorCode = 1;
  ulErrorCode = DosQueryProcType( this->handle(),
                                  procedureOrdinal,
                                  0,
                                  &result );
  if ( ulErrorCode )
    {
    ITHROWSYSTEMERROR( ulErrorCode,
                       IString("DosQueryProcType #: ") +
                                IString(procedureOrdinal),
                       IErrorInfo::accessError,
                       IException::recoverable);
    }
  return result;
  }

/*------------------------------------------------------------------------------
| IDynamicLinkLibrary::handle                                                  |
|  Return the Handle used to load Resources                                    |
------------------------------------------------------------------------------*/
IModuleHandle IDynamicLinkLibrary::handle() const
{
   IMODTRACE_ALL("DynLnkLib::handle");
   if(pmodClResource!=0)
     return pmodClResource->handle();
   else
     return IModuleHandle(0);
}

/*------------------------------------------------------------------------------
| IDLLModule::addRef                                                           |
|   Add a reference to a DLL Module. Create if 0.                              |
------------------------------------------------------------------------------*/
unsigned long IDLLModule :: addRef()
{
   IMODTRACE_ALL("DLL::addRef");
   if(useCount() == 0)
   {
     HMODULE hmod;
     char  szObjectNameBuf[100];
     unsigned long ulErrorCode = 0;
     ulErrorCode = DosLoadModule ((PSZ)szObjectNameBuf, sizeof(szObjectNameBuf),
                (PSZ)name(), &hmod);

     if (ulErrorCode != 0 || hmod==0)
     {
        ITHROWSYSTEMERROR( ulErrorCode,
                           IString("DosLoadModule : ") +
                                    name(),
                           IErrorInfo::accessError,
                           IException::recoverable);
     }
     hmodCl = hmod;
   }
   ulClUseCount++;
   return ulClUseCount;
}


/*------------------------------------------------------------------------------
| IDLLModule::removeRef                                                        |
|  Remove a reference. Delete if 0.                                            |
------------------------------------------------------------------------------*/
unsigned long IDLLModule :: removeRef()
{
   IMODTRACE_ALL("DLL::removeRef");
   ulClUseCount--;
   if(ulClUseCount == 0)
   {
     unsigned long
       ulErrorCode;
     ulErrorCode = DosFreeModule(handle());
     hmodCl = 0;
     if(ulErrorCode!=0)
     {
        ITHROWSYSTEMERROR( ulErrorCode,
                           IString("DosFreeModule : ") +
                                    name(),
                           IErrorInfo::accessError,
                           IException::recoverable);
     }
   } /* end (ulClUseCount == 0)  */
   return ulClUseCount;

}

/*------------------------------------------------------------------------------
| IDLLModule::asDebugInfo                                                      |
------------------------------------------------------------------------------*/
IString IDLLModule :: asDebugInfo() const
{
  IString str="IDLLModule(Name(";
  str += strClName;
  str += "),Handle(";
  str += IString(hmodCl.asString());
  str += "),UseCount(";
  str += IString(ulClUseCount);
  str += ")";
  return str;
}
