/*******************************************************************************
* FILE NAME: ireslib2.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of Resource Library                  *
*   bitmap functions.                                                          *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define INCL_GPICONTROL   //rem /* GPI control Functions        */
  #define INCL_GPIBITMAPS   //rem
  #define INCL_WINPOINTERS
  #include <iwindefs.h>
}

#ifndef _IRESLIB_
  #include <ireslib.hpp>
#endif
#ifndef _IRESSTAT_
  #include <iresstat.hpp>
#endif
#ifndef _ISTRING_
  #include <istring.hpp>
#endif
#ifndef _IRECT_
  #include <irect.hpp>
#endif
#ifndef _IRESLOCK_
  #include <ireslock.hpp>
#endif
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif
#ifndef _IBMPDATA_
  #include <ibmpdata.hpp>
#endif
#ifndef _IGBITMAP_
  #include <igbitmap.hpp>
#endif
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif


// Segment definitions
#ifdef IC_PAGETUNE
//#define _IRESLIB2_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IResLibStaticPtr::deleteBitmapSet                                            |
------------------------------------------------------------------------------*/
void IResLibStaticPtr::deleteBitmapSet()
{
  if(pBmpSet)
    delete pBmpSet;
}


/*------------------------------------------------------------------------------
| IResLibStaticPtr::bitmapSet                                                  |
------------------------------------------------------------------------------*/
IBitmapSet& IResLibStaticPtr::bitmapSet()
{
  if(pBmpSet==0)
    pBmpSet = new IBitmapSet;
  return *pBmpSet;
}

/*------------------------------------------------------------------------------
| IResourceLibrary::loadBitmap                                                 |
------------------------------------------------------------------------------*/
IBitmapHandle IResourceLibrary::loadBitmap(unsigned long ulBitmapId,
                                           const ISize& sizeBitmap,
                                           Boolean fManaged) const
{
  IMODTRACE_ALL("ResLib::loadBitmap");

  IBitmapHandle ulBmpHandle = tryToLoadBitmap(ulBitmapId, sizeBitmap, fManaged);
  if(!ulBmpHandle)
     ITHROWGUIERROR(IString("GpiLoadBitmap")+"Id: "+ IString(ulBitmapId)+" ");

  return ulBmpHandle;
}


/*------------------------------------------------------------------------------
| IResourceLibrary::tryToLoadBitmap                                            |
------------------------------------------------------------------------------*/
IBitmapHandle IResourceLibrary::tryToLoadBitmap(unsigned long ulBitmapId,
                                                const ISize& sizeBitmap,
                                                Boolean fManaged) const
{
  IMODTRACE_ALL("ResLib::tryToLoadBitmap");
  unsigned long ulBmpHandle;
  IBitmapData* pBmpData = 0;

  /* If resources are managed, build a cursor to see if we have already */
  /* loaded this Icon.  If so, just update the use count                */
  Boolean fFound = false;
  if(fManaged)
  {
    pBmpData = IResLibStaticPtr::bitmapSet().find( ulBitmapId,
                                                   fileName(),
                                                   sizeBitmap );
    if ( pBmpData )
    {
      fFound = true;
      ulBmpHandle = pBmpData->handle();
    } /* endif */
  }
  if(!fFound)
  {
    ulBmpHandle = IGBitmap::tryToLoadBitmap( ulBitmapId, handle(), sizeBitmap );

    if(ulBmpHandle && fManaged)
    {
      // Lock Resource Table
      IResourceLock reslibLock(IResLibStaticPtr::libraryKey());
      IBitmapData* pbmpNew = new IBitmapData(ulBitmapId, ulBmpHandle,
                                             fileName(), sizeBitmap);
      IResLibStaticPtr::bitmapSet().add(pbmpNew);

    } /* endif (fManaged) */
  }  /* endif !found */

  return IBitmapHandle(ulBmpHandle);
}


/*------------------------------------------------------------------------------
| IResourceLibrary::loadBitmap                                                 |
------------------------------------------------------------------------------*/
IBitmapHandle IResourceLibrary::loadBitmap(unsigned long ulBitmapId,
                                           Boolean fManaged) const
{
  ISize sizBitmap(0,0);
  return loadBitmap(ulBitmapId, sizBitmap, fManaged);
}

/*------------------------------------------------------------------------------
| IResourceLibrary::tryToLoadBitmap                                                 |
------------------------------------------------------------------------------*/
IBitmapHandle IResourceLibrary::tryToLoadBitmap(unsigned long ulBitmapId,
                                                Boolean fManaged) const
{
  ISize sizBitmap(0,0);
  return tryToLoadBitmap(ulBitmapId, sizBitmap, fManaged);
}


/*------------------------------------------------------------------------------
| IResourceLibrary::addRef                                                     |
------------------------------------------------------------------------------*/
unsigned long IResourceLibrary::addRef(const IBitmapHandle& bmh)
{
  IMODTRACE_ALL("ResLib::addRef");
  ITRACE_ALL(IString("BMPHandle is: ") + IString(bmh.asUnsigned()));

  unsigned long ulUseCount=0;
  IBitmapData* pBmpData;
  if(bmh!=0)
  {
    pBmpData = IResLibStaticPtr::bitmapSet().find( bmh );
    if ( pBmpData )
    {
      ulUseCount = pBmpData->addRef();
      ITRACE_ALL(IString("Handle is: ")+IString(pBmpData->handle()) +
                 " UseCount now: " + IString(ulUseCount));
    }
  } /* endif */
  return(ulUseCount);
}

/*------------------------------------------------------------------------------
| IResourceLibrary::removeRef                                                  |
------------------------------------------------------------------------------*/
unsigned long IResourceLibrary::removeRef(const IBitmapHandle& bmh)
{
  IMODTRACE_ALL("ResLib::removeRef");
  ITRACE_ALL(IString("BMPHandle is: ") + IString(bmh.asUnsigned()));

  unsigned long ulUseCount=0;
  IBitmapData* pBmpData;
  if(bmh!=0)
  {
    pBmpData = IResLibStaticPtr::bitmapSet().find( bmh );
    if ( pBmpData )
    {
      ulUseCount = pBmpData->removeRef();
      ITRACE_ALL(IString("Handle is: ")+IString(pBmpData->handle()) +
                 " UseCount now: " + IString(ulUseCount));
      if(ulUseCount==0)
      {
        Boolean fSuccess = IDELETEBITMAP(pBmpData->handle());
        ITRACE_ALL(IString("Bitmap deleted, handle: ")+IString(bmh.asUnsigned()));

        if(!fSuccess)
        {
            ITHROWGUIERROR("IDELETEBITMAP");
        }
        IResLibStaticPtr::bitmapSet().remove(pBmpData);
        delete(pBmpData);
      }
    }
  } /* endif */
  return ulUseCount;
}

/*------------------------------------------------------------------------------
| IResourceLibrary::systemBitmap                                               |
------------------------------------------------------------------------------*/
unsigned long
   IResourceLibrary::systemBitmap ( unsigned long systemBitmapId )
{
  unsigned long hbmp;
  IBitmapData*
    pBmpData = IResLibStaticPtr::bitmapSet().find( systemBitmapId,
                                                   IString(),
                                                   ISize(0,0));
  if ( pBmpData )
  {
    hbmp = pBmpData->handle();
  }
  else
  {
    hbmp = IGETSYSBITMAP( systemBitmapId );

    if (!hbmp)
    {
      ITHROWGUIERROR("IGETSYSBITMAP");
    }
    // If we return a copy we will manage reference count this resource
    // otherwise not (don't want to destroy a system pointer).
    pBmpData = new IBitmapData( systemBitmapId, hbmp, IString());
    IResLibStaticPtr::bitmapSet().add(pBmpData);
  }
  return hbmp;
}
