/*******************************************************************************
* FILE NAME: iscrlhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iscrlhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINMESSAGEMGR
#define INCL_WINSCROLLBARS
extern "C" {
  #include <iwindefs.h>
}

#ifndef _ISCRLHDR_
  #include <iscrlhdr.hpp>
#endif

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif
#ifndef _ISCRLEVT_
  #include <iscrlevt.hpp>
#endif
#ifndef _ISCROLL_
  #include <iscroll.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ISCRLHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| IScrollHandler::IScrollHandler                                               |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
IScrollHandler::IScrollHandler ( )
  : IScrollHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| IScrollHandler::~IScrollHandler                                              |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IScrollHandler::~IScrollHandler ( )
{ }

/*------------------------------------------------------------------------------
| IScrollHandler::dispatchHandlerEvent                                         |
|                                                                              |
| Dispatch scroll event handler functions.                                     |
------------------------------------------------------------------------------*/
Boolean IScrollHandler::dispatchHandlerEvent ( IEvent& event )
{
  Boolean bRc = false;

  switch ( event.eventId() )
  {
     case WM_VSCROLL:
     {
        IScrollEvent scrlevt( event );
        switch ( event.parameter2().number2() )
        {
           case SB_LINEUP:
              bRc = this->lineUp( scrlevt );
              break;
           case SB_LINEDOWN:
              bRc = this->lineDown( scrlevt );
              break;
           case SB_PAGEUP:
              bRc = this->pageUp( scrlevt );
              break;
           case SB_PAGEDOWN:
              bRc = this->pageDown( scrlevt );
              break;
           case SB_SLIDERTRACK:
              bRc = this->scrollBoxTrack( scrlevt );
              break;
           case SB_SLIDERPOSITION:
              bRc = this->scrollBoxTrackEnd( scrlevt );
              break;
           case SB_ENDSCROLL:
              bRc = this->scrollEnd( scrlevt );
              break;
           default:
              break;
        }

        if ( bRc )
        {                              // Processing done.
           event.setResult( scrlevt.result());  // Copy event result.
        }
        break;
     }

     case WM_HSCROLL:
     {
        IScrollEvent scrlevt( event );
        switch ( event.parameter2().number2() )
        {
           case SB_LINELEFT:
              bRc = this->lineLeft( scrlevt );
              break;
           case SB_LINERIGHT:
              bRc = this->lineRight( scrlevt );
              break;
           case SB_PAGELEFT:
              bRc = this->pageLeft( scrlevt );
              break;
           case SB_PAGERIGHT:
              bRc = this->pageRight( scrlevt );
              break;
           case SB_SLIDERTRACK:
              bRc = this->scrollBoxTrack( scrlevt );
              break;
           case SB_SLIDERPOSITION:
              bRc = this->scrollBoxTrackEnd( scrlevt );
              break;
           case SB_ENDSCROLL:
              bRc = this->scrollEnd( scrlevt );
              break;
           default:
              break;
        }

        if ( bRc )
        {                              // Processing done.
           event.setResult( scrlevt.result() );  // Copy event result.
        }
        break;
     }

     default:
        break;
  }

  return bRc;
}

/*------------------------------------------------------------------------------
| IScrollHandler::lineUp                                                       |
------------------------------------------------------------------------------*/
Boolean IScrollHandler::lineUp ( IScrollEvent& event )
{
  this->moveScrollBox( event );
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::lineDown                                                     |
------------------------------------------------------------------------------*/
Boolean IScrollHandler::lineDown ( IScrollEvent& event )
{
  this->moveScrollBox( event );
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::pageUp                                                       |
------------------------------------------------------------------------------*/
Boolean IScrollHandler::pageUp ( IScrollEvent& event )
{
  this->moveScrollBox( event );
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::pageDown                                                     |
------------------------------------------------------------------------------*/
Boolean IScrollHandler::pageDown ( IScrollEvent& event )
{
  this->moveScrollBox( event );
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::lineLeft                                                     |
------------------------------------------------------------------------------*/
Boolean IScrollHandler::lineLeft ( IScrollEvent& event )
{
  this->moveScrollBox( event );
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::lineRight                                                    |
------------------------------------------------------------------------------*/
Boolean IScrollHandler::lineRight ( IScrollEvent& event )
{
  this->moveScrollBox( event );
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::pageLeft                                                     |
------------------------------------------------------------------------------*/
Boolean IScrollHandler::pageLeft ( IScrollEvent& event )
{
  this->moveScrollBox( event );
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::pageRight                                                    |
------------------------------------------------------------------------------*/
Boolean IScrollHandler::pageRight ( IScrollEvent& event )
{
  this->moveScrollBox( event );
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::scrollBoxTrack                                               |
------------------------------------------------------------------------------*/
Boolean IScrollHandler::scrollBoxTrack ( IScrollEvent& event )
{
  this->moveScrollBox( event );
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::scrollBoxTrackEnd                                            |
------------------------------------------------------------------------------*/
Boolean IScrollHandler::scrollBoxTrackEnd ( IScrollEvent& event )
{
  this->moveScrollBox( event );
  return true;
}

/*------------------------------------------------------------------------------
| IScrollHandler::scrollEnd                                                    |
------------------------------------------------------------------------------*/
Boolean IScrollHandler::scrollEnd ( IScrollEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| IScrollHandler::moveScrollBox                                                |
|                                                                              |
| Move the scroll box according to the information in the scroll event.        |
------------------------------------------------------------------------------*/
IScrollHandler& IScrollHandler::moveScrollBox ( IScrollEvent& event )
{
  IScrollBar* pscrbar = event.scrollBarWindow();
  if ( pscrbar )
  {
     switch ( event.parameter2().number2() )
     {                                 // Switch on scroll bar event.
        case SB_LINEUP:                // (Same as SB_LINELEFT.)
        case SB_LINEDOWN:              // (Same as SB_LINERIGHT.)
        case SB_PAGEUP:                // (Same as SB_PAGELEFT.)
        case SB_PAGEDOWN:              // (Same as SB_PAGERIGHT.)
           pscrbar->moveScrollBoxTo( pscrbar->scrollBoxPosition() +
                                       event.scrollAmount() );
           break;
        case SB_SLIDERPOSITION:        // Fall into next case.
        case SB_SLIDERTRACK:           // Fall into next case.
           pscrbar->moveScrollBoxTo( event.newScrollBoxPosition() );
           break;
        case SB_ENDSCROLL:             // Fall into next case.
        default:
           break;
     }
  }

  return *this;
}
