/*******************************************************************************
* FILE NAME: iselhdr.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iselhdr.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINBUTTONS
#define INCL_WINENTRYFIELDS
#define INCL_WINLISTBOXES
#define INCL_WINSTDCNR
#define INCL_WINSTDVALSET
#define INCL_WINWINDOWMGR

extern "C" {
  #include <iwindefs.h>
  #include <stdlib.h>
}

#ifndef _ISELHDR_
  #include <iselhdr.hpp>
#endif
#ifndef _ICTLEVT_
  #include <ictlevt.hpp>
#endif
#ifndef _IEVENT_
  #include <ievent.hpp>
#endif
#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif
#ifndef _IWCNAME_
  #include <iwcname.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ISELHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| ISelectHandler::ISelectHandler                                               |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
ISelectHandler::ISelectHandler ( )
  : ISelectHandler::Inherited ( )
{ }

/*------------------------------------------------------------------------------
| ISelectHandler::~ISelectHandler                                              |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ISelectHandler::~ISelectHandler ( )
{ }

/*------------------------------------------------------------------------------
| ISelectHandler::dispatchHandlerEvent                                         |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
Boolean ISelectHandler::dispatchHandlerEvent ( IEvent& event )
{
  Boolean bRc = false;

  if ( event.eventId() == WM_CONTROL )
  {                                    // Control event.
     IControlEvent ctlevt( event );
     IWindowHandle hwndControl = ctlevt.controlHandle();

     if ( hwndControl )
     {
        unsigned long subMsg = ctlevt.parameter1().number2();
        IWindowClassName className( hwndControl );

        if ( className == WC_COMBOBOX )
        {                              // Combo-box case.
           if ( subMsg == CBN_LBSELECT )
           {
              bRc = this->selected( ctlevt );
           }
           else if ( subMsg == CBN_ENTER )
           {
              bRc = this->enter( ctlevt );
           }
        }
        else if ( className == WC_BUTTON )
        {                              // Button case.
           unsigned long flStyle = ISTYLEOF( hwndControl );
           if ( (flStyle & BS_PRIMARYSTYLES) != BS_PUSHBUTTON )
           {                           // Not a push button.
              if ( subMsg == BN_CLICKED )
              {
                 bRc = this->selected( ctlevt );
              }
              else if ( subMsg == BN_DBLCLICKED )
              {
                 bRc = this->enter( ctlevt );
              }
           }
        }
        else if ( className == WC_LISTBOX )
        {                              // List box case.
           if ( subMsg == LN_SELECT )
           {
              bRc = this->selected( ctlevt );
           }
           else if ( subMsg == LN_ENTER )
           {
              bRc = this->enter( ctlevt );
           }
        }
        else if ( className == WC_CONTAINER )
        {                              // Container case.
           if ( subMsg == CN_EMPHASIS )
           {                           // Check for selected emphasis.
              NOTIFYRECORDEMPHASIS* pnre =
                  (NOTIFYRECORDEMPHASIS*)(void*)ctlevt.parameter2();
              if ( pnre->fEmphasisMask & CRA_SELECTED )
              {
                 bRc = this->selected( ctlevt );
              }
           }
           else if ( subMsg == CN_ENTER )
           {
              bRc = this->enter( ctlevt );
           }
        }
        else if ( className == WC_VALUESET )
        {                              // Value set case.
           if ( subMsg == VN_SELECT )
           {
              bRc = this->selected( ctlevt );
           }
           else if ( subMsg == VN_ENTER )
           {
              bRc = this->enter( ctlevt );
           }
        }

        if ( bRc )
        {                              // Processing done.
           event.setResult( ctlevt.result() );  // Copy event result.
        }
     }
  } /* endif WM_CONTROL */

  return bRc;
}

/*------------------------------------------------------------------------------
| ISelectHandler::enter                                                        |
|                                                                              |
| Dummy virtual handler function.  If a button generated the event,            |
| call selected.                                                               |
------------------------------------------------------------------------------*/
Boolean ISelectHandler::enter ( IControlEvent& event )
{
  Boolean bRc = false;
  if ( IWindowClassName( event.controlHandle() ) == WC_BUTTON )
  {
     bRc = this->selected( event );
  }
  return bRc;
}

/*------------------------------------------------------------------------------
| ISelectHandler::selected                                                     |
|                                                                              |
| Dummy virtual handler function.                                              |
------------------------------------------------------------------------------*/
Boolean ISelectHandler::selected ( IControlEvent& event )
{
  return false;
}
