/*******************************************************************************
* FILE NAME: isizeevt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in isizeevt.hpp.  This encapsulates a WM_WINDOWPOSCHANGED                  *
*    message.  It provides member functions for querying the previous and      *
*    new size of the window.                                                   *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #include <iwindefs.h>           // for PSWP
}

#ifndef _ISIZEEVT_
  #include <isizeevt.hpp>
#endif

#ifndef _IEVTDATA_
  #include <ievtdata.hpp>
#endif
#ifndef _IPOINT_
  #include <ipoint.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISIZEEVT_CPP_
  #include <ipagetun.h>
#endif


/*******************************************************************/
/* Constructor here for page tuning.                               */
/*******************************************************************/
IResizeEvent :: IResizeEvent ( const IEvent & event )
  : IEvent ( event )
{ ; }


/*******************************************************************/
/* Empty destructor here for page tuning.                          */
/*******************************************************************/
IResizeEvent :: ~IResizeEvent ( )
{ ; }


/*******************************************************************/
/* oldSize - Return the previous size of the window.               */
/*******************************************************************/
ISize  IResizeEvent :: oldSize ( ) const
{
   PSWP pswp = (PSWP)(char*)parameter1();
                                  // array of two SWP structures
   pswp++;                        // old window is second SWP struct
   return ISize(pswp->cx, pswp->cy);    // old size
}


/*******************************************************************/
/* newSize - Return the new size of the window.                    */
/*******************************************************************/
ISize  IResizeEvent :: newSize ( ) const
{

   PSWP pswp = (PSWP)(char*)parameter1();
                                  // array of two SWP structures
   return ISize(pswp->cx, pswp->cy);    // new size
}
