/*******************************************************************************
* FILE NAME: isizehdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in isizehdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_WINFRAMEMGR          // for WM_WINDOWPOSCHANGED
extern "C" {
  #include <iwindefs.h>
}

#ifndef _ISIZEHDR_
  #include <isizehdr.hpp>
#endif
#ifndef _ISIZEEVT_
  #include <isizeevt.hpp>
#endif
#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISIZEHDR_CPP_
  #include <ipagetun.h>
#endif


/*******************************************************************/
/* Default constructor here for page tuning.                       */
/*******************************************************************/
IResizeHandler :: IResizeHandler ( )
  : IResizeHandler::Inherited ( )
{}


/*******************************************************************/
/* Empty destructor here for page tuning.                          */
/*******************************************************************/
IResizeHandler :: ~IResizeHandler ( )
{}


/************************************************************************/
/* dispatchhandlerEvent - Dispatch command functions for this handler.  */
/************************************************************************/
Boolean  IResizeHandler :: dispatchHandlerEvent ( IEvent& evt )
{
   IBoolean bRc = false;

   if (evt.eventId() == WM_WINDOWPOSCHANGED)
   {                              // size/move event found
      PSWP pswp = (PSWP)(char*)evt.parameter1();
      if (pswp->fl & SWP_SIZE)
      {                           // size event found
         IResizeEvent resevt(evt);
         bRc = windowResize(resevt);      // call derived class (if any)
         if ( bRc )       // call derived class (if any)
         {                        // event has been processed
            // Generate a user message if the resize handler is stopping
            // processing of the message.   IRecoordHandler needs to
            // see resize events to work properly.  If this handler
            // is stopping the message we generate a substitute.
            // Note that this logic relies on the fact that there is
            // no ICLUI handler of WM_WINDOWPOSCHANGED which returns
            // true.  This means that the IC_UM... message ONLY occurs
            // if a user supplied handler returns true before the
            // IRecoordHandler has run.
            // This is not a problem in Windows since we use different
            // messages in IRecoordHandler and IResizeHandler.
            resevt.handle().sendEvent(
               IC_UM_WINDOWPOSCHANGED,
               resevt.parameter1(),
               resevt.parameter2() );

            /*******************************************************/
            /* The handler has said not to pass on the event, so   */
            /* don't pass it to another handler.  However, we do   */
            /* go ahead and pass it to the window procedure of the */
            /* current window, since it is only a notification     */
            /* message and we could break something by not doing   */
            /* this (this message is PM's way of generating the    */
            /* WM_SIZE, WM_SHOW, and WM_HIDE messages, if the      */
            /* message gets sent on to the default window          */
            /* procedure, and there's no harm in not eliminating   */
            /* these messages).                                    */
            /*******************************************************/
            defaultProcedure(evt);
            evt.setResult(resevt.result());  // copy the derived result
         }
      } /* endif SWP_SIZE */
   } /* endif WM_WINDOWPOSCHANGED */

   return bRc;
}
