/*******************************************************************************
* FILE NAME: isldahdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in isldahdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define IC_MULTIMEDIA
#define INCL_CIRCULARSLIDER
#define INCL_SW
#define INCL_WINDIALOGS
#define INCL_WINENTRYFIELDS
#define INCL_WINMLE
#define INCL_WINSTDSLIDER
#define INCL_WINSTDSPIN
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
  #include <stdlib.h>
}

#ifndef _ISLDAHDR_
  #include <isldahdr.hpp>
#endif
#ifndef _ICTLEVT_
  #include <ictlevt.hpp>
#endif
#ifndef _IEVENT_
  #include <ievent.hpp>
#endif
#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif
#ifndef _IWCNAME_
  #include <iwcname.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ISLDAHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| ISliderArmHandler::ISliderArmHandler                                                   |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
ISliderArmHandler::ISliderArmHandler ( )
  : ISliderArmHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| ISliderArmHandler::~ISliderArmHandler                                                  |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ISliderArmHandler::~ISliderArmHandler ( )
{ }

/*------------------------------------------------------------------------------
| ISliderArmHandler::dispatchHandlerEvent                                           |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
Boolean ISliderArmHandler::dispatchHandlerEvent ( IEvent& event )
{
  Boolean bRc = false;

  if ( event.eventId() == WM_CONTROL )
  {                                    // Control event.
     IControlEvent ctlevt( event );
     IWindowHandle hwndControl = ctlevt.controlHandle();

     if ( hwndControl )
     {
        unsigned long subMsg = ctlevt.parameter1().number2();
        IWindowClassName className( hwndControl );

        if ( className == WC_SLIDER )
        {                              // Slider case.
           if ( subMsg == SLN_SLIDERTRACK )
           {
              bRc = this->moving( ctlevt );
           }
           else if ( subMsg == SLN_CHANGE )
           {
              bRc = this->positionChanged( ctlevt );
           }
        }
#ifdef WC_CIRCULARSLIDER
        else if ( className == WC_CIRCULARSLIDER )
        {                              // Circular slider case.
           if ( subMsg == CSN_TRACKING )
           {
              bRc = this->moving( ctlevt );
           }
           if ( subMsg == CSN_CHANGED )
           {
              bRc = this->positionChanged( ctlevt );
           }
        }
#endif

        if ( bRc )
        {                              // Processing done.
           event.setResult( ctlevt.result() );  // Copy event result.
        }
     }
  }

  return bRc;
}

/*------------------------------------------------------------------------------
| ISliderArmHandler::moving                                                         |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ISliderArmHandler::moving ( IControlEvent& event )
{
  return false;
}
/*------------------------------------------------------------------------------
| ISliderArmHandler::positionChanged                                                |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ISliderArmHandler::positionChanged ( IControlEvent& event )
{
  return false;
}
