/*******************************************************************************
* FILE NAME: islider.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in islider.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
{
   #define INCL_WINERRORS
   #define INCL_WINSTDSLIDER
   #define INCL_DOSMODULEMGR
   #define INCL_WINMESSAGEMGR
   #define INCL_WINSYS
   #include <iwindefs.h>
}

#include <islider.hpp>
#include <ipoint.hpp>
#include <ihandle.hpp>
#include <ievent.hpp>
#include <istring.hpp>
#include <irect.hpp>
#include <iexcept.hpp>
#include <icolor.hpp>
#include <ireslib.hpp>
#include <icconst.h>
#include <itrace.hpp>
#include <icmnfun.hpp>
#include <inotifev.hpp>
#include <icoordsy.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISLIDER_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public progress indicator styles.                                            |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const IProgressIndicator::Style
  IProgressIndicator::horizontal        ( 0, ISLS_HORIZONTAL ),
  IProgressIndicator::vertical          = SLS_VERTICAL,
  IProgressIndicator::alignCentered     ( 0, ISLS_CENTER ),
  IProgressIndicator::alignBottom       = SLS_BOTTOM,
  IProgressIndicator::alignTop          = SLS_TOP,
  IProgressIndicator::alignLeft         = SLS_LEFT,
  IProgressIndicator::alignRight        = SLS_RIGHT,
  IProgressIndicator::snapToTickMark    = SLS_SNAPTOINCREMENT,
  IProgressIndicator::handleDrawItem    = SLS_OWNERDRAW,
  IProgressIndicator::ribbonStrip       = SLS_RIBBONSTRIP,
  IProgressIndicator::homeBottom        ( 0, ISLS_HOMEBOTTOM ),
  IProgressIndicator::homeTop           = SLS_HOMETOP,
  IProgressIndicator::homeLeft          ( 0, ISLS_HOMELEFT ),
  IProgressIndicator::homeRight         = SLS_HOMERIGHT,
  IProgressIndicator::primaryScale1     ( 0, ISLS_PRIMARYSCALE1 ),
  IProgressIndicator::primaryScale2     = SLS_PRIMARYSCALE2,
  IProgressIndicator::pmCompatible      ( 0, ISLS_PMCOMPATIBILITY),
  IProgressIndicator::border3D          ( 0, IWS_BORDER3D ),
  IProgressIndicator::classDefaultStyle ( (WS_VISIBLE      |
                                           SLS_RIBBONSTRIP |
                                           SLS_READONLY),
                                                         (ISLS_HORIZONTAL |
                                                          ISLS_CENTER     |
                                                          ISLS_HOMELEFT   |
                                                          ISLS_PRIMARYSCALE1) );
#pragma data_seg()


/*------------------------------------------------------------------------------
| Default style for new progress indicator objects (initial value).            |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
IProgressIndicator::Style
  IProgressIndicator::currentDefaultStyle ( (WS_VISIBLE      |
                                             SLS_RIBBONSTRIP |
                                             SLS_READONLY),
                                                         (ISLS_HORIZONTAL |
                                                          ISLS_CENTER     |
                                                          ISLS_HOMELEFT   |
                                                          ISLS_PRIMARYSCALE1) );
#pragma data_seg()


/*------------------------------------------------------------------------------
| IProgressIndicator::IProgressIndicator                                       |
------------------------------------------------------------------------------*/
IProgressIndicator :: IProgressIndicator ( unsigned long     windowId,
                                           IWindow*          parent,
                                           IWindow*          owner,
                                           const IRectangle& initial,
                                           unsigned long     numberOfTicks,
                                           unsigned long     tickSpacing,
                                           const Style&      style )
{
   IMODTRACE_DEVELOP("IProgressIndicator::ctor1");
   IASSERTPARM(parent!=0);             //Assertions on input parms

   Style readonlyStyle(style);
   readonlyStyle |= SLS_READONLY;

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | readonlyStyle.asExtendedUnsignedLong() );

   SLDCDATA sldcData;                  //Slider control data
   sldcData.cbSize = sizeof(SLDCDATA);
   if (style & primaryScale2)
   {
      sldcData.usScale2Increments = (unsigned short)numberOfTicks;
      sldcData.usScale2Spacing = (unsigned short)tickSpacing;
      sldcData.usScale1Increments = (unsigned short)0;
      sldcData.usScale1Spacing = (unsigned short)0;
   }
   else
   {
      sldcData.usScale1Increments = (unsigned short)numberOfTicks;
      sldcData.usScale1Spacing = (unsigned short)tickSpacing;
      sldcData.usScale2Increments = (unsigned short)0;
      sldcData.usScale2Spacing = (unsigned short)0;
   }
   unsigned long hwndOwner = 0;
   if (owner != 0)
      hwndOwner = owner->handle();

   initialize(windowId, parent->handle(), hwndOwner,
              convertToGUIStyle( readonlyStyle ), initial,
              (void*)&sldcData);
}

/*------------------------------------------------------------------------------
| IProgressIndicator::IProgressIndicator                                       |
------------------------------------------------------------------------------*/
IProgressIndicator :: IProgressIndicator ( unsigned long   windowId,
                                           IWindow*        parent,
                                           IWindow*        owner,
                                           const IRectangle& initial,
                                           unsigned long   scale1NumberOfTicks,
                                           unsigned long   scale1TickSpacing,
                                           unsigned long   scale2NumberOfTicks,
                                           unsigned long   scale2TickSpacing,
                                           const Style&    style )
{
   IMODTRACE_DEVELOP("IProgressIndicator::ctor2");
   IASSERTPARM(parent!=0);             //Assertions on input parms

   Style readonlyStyle(style);
   readonlyStyle |= SLS_READONLY;

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | readonlyStyle.asExtendedUnsignedLong() );

   SLDCDATA sldcData;                  //Slider control data
   sldcData.cbSize = sizeof(SLDCDATA);
   sldcData.usScale1Increments = (unsigned short)scale1NumberOfTicks;
   sldcData.usScale2Increments = (unsigned short)scale2NumberOfTicks;
   sldcData.usScale1Spacing = (unsigned short)scale1TickSpacing;
   sldcData.usScale2Spacing = (unsigned short)scale2TickSpacing;
   unsigned long hwndOwner = 0;
   if (owner != 0)
      hwndOwner = owner->handle();
   initialize(windowId, parent->handle(), hwndOwner,
              convertToGUIStyle( readonlyStyle ), initial,
              (void*)&sldcData);
}

/*------------------------------------------------------------------------------
| IProgressIndicator::IProgressIndicator                                       |
------------------------------------------------------------------------------*/
IProgressIndicator :: IProgressIndicator( unsigned long ulId,
                                          IWindow* pwndParent )
{
  IMODTRACE_DEVELOP("IProgressIndicator::ctor3");
  setAutoDestroyWindow(false);
  startHandlingEventsFor(ulId, pwndParent);
  setBackgroundColor(defaultbackgroundcolor(handle()));

}

/*------------------------------------------------------------------------------
| IProgressIndicator :: ProgressIndicator                                      |
------------------------------------------------------------------------------*/
IProgressIndicator :: IProgressIndicator ( )
{
}

/*------------------------------------------------------------------------------
| IProgressIndicator::~IProgressIndicator                                      |
------------------------------------------------------------------------------*/
IProgressIndicator :: ~IProgressIndicator()
{ }

/*------------------------------------------------------------------------------
| IProgressIndicator::defaultStyle                                             |
|                                                                              |
| Return the default style for new progress indicator objects.                 |
------------------------------------------------------------------------------*/
IProgressIndicator::Style  IProgressIndicator :: defaultStyle ( )
{
   return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IProgressIndicator::convertToGUIStyle                                        |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IProgressIndicator :: convertToGUIStyle(const IBitFlag& guiStyle,
                                                      Boolean bExtOnly) const
{
  // Obtain the style from the class (IControl) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= extendedStyle() & IS_EXTMASK;
  }
  else
  {
    // SLS_ styles have a one-to-one correspondence to our style bits, and
    // inhabit the lower word of the base GUI style.  Therefore, obtain
    // that portion asis, masking out the upper word. Five of the styles
    // were denoted as 0 (OS/2 PM), due to the lack of available bits.
    // We are representing those as extended bits so we or the developer
    // can accurately test for them later.  Since their native values are 0,
    // we do not have to do any additional processing here.  Note that
    // since this class only deals with shaft related styles, we only
    // mask those out here.
    ulStyle |= guiStyle.asUnsignedLong() & ISLS_SHAFTMASK;

  }

  return( ulStyle );
}

/*------------------------------------------------------------------------------
| IProgressIndicator::tickPosition                                             |
|                                                                              |
| Return the pixel position of the tick at the specified index.  Ticks are     |
| numbered starting with zero.                                                 |
------------------------------------------------------------------------------*/
IPoint IProgressIndicator :: tickPosition ( unsigned long ulTickNum ) const
{
   IMODTRACE_DEVELOP("IProgressIndicator::tickPosition");

   IEventResult evtTickData = handle().sendEvent(SLM_QUERYTICKPOS,
                                 IEventParameter1(ulTickNum),
                                 IEventParameter2((unsigned long)0));

   if (evtTickData.number1() == 0 || evtTickData.number2() == 0)
      ITHROWGUIERROR2("SLM_QUERYTICKPOS",
                      IErrorInfo::invalidParameter,
                      IException::recoverable);

   IPoint pt((unsigned short)evtTickData.number1(),
             (unsigned short)evtTickData.number2());

   // Always returns end of tick closest to shaft
   return ICoordinateSystem::convertToApplication( pt, this->size() );
}

/*------------------------------------------------------------------------------
| IProgressIndicator::tickLength                                               |
|                                                                              |
| Return the length of the tick at the specified index, in pixels.             |
------------------------------------------------------------------------------*/
unsigned long IProgressIndicator :: tickLength ( unsigned long ulTickNum ) const
{
   IMODTRACE_DEVELOP("IProgressIndicator::tickLength");
   IEventResult evtTickData =
           handle().sendEvent(SLM_QUERYTICKSIZE,
                              IEventParameter1(ulTickNum),
                              IEventParameter2((unsigned long)0));

   if (evtTickData.number1() == (USHORT)SLDERR_INVALID_PARAMETERS)
      ITHROWGUIERROR2("SLM_QUERYTICKSIZE",
                      IErrorInfo::invalidParameter,
                      IException::recoverable);

   return evtTickData.number1();
}

/*------------------------------------------------------------------------------
| IProgressIndicator::tickSpacing                                              |
|                                                                              |
| Returns the number of pixels between ticks for the specified scale.          |
------------------------------------------------------------------------------*/
unsigned long IProgressIndicator :: tickSpacing  ( Scale scale ) const
{
   WNDPARAMS wndParams;
   SLDCDATA sldcData;

   // Set up buffer with query information
   wndParams.fsStatus = WPM_CTLDATA | WPM_CBCTLDATA;
   wndParams.cbCtlData = sizeof( SLDCDATA );
   wndParams.pCtlData = (PVOID)&sldcData;

   unsigned long ulSuccess =
           handle().sendEvent(WM_QUERYWINDOWPARAMS,
                              IEventParameter1(&wndParams),
                              IEventParameter2((unsigned long)0));

   if (!ulSuccess)
      ITHROWLIBRARYERROR(IC_SLDR_GET_CTLDATA,
                         IErrorInfo::accessError,
                         IException::recoverable);

   if (scale == scale1)
      return ((PSLDCDATA)(wndParams.pCtlData))->usScale1Spacing;
   else
      return ((PSLDCDATA)(wndParams.pCtlData))->usScale2Spacing;
}

/*------------------------------------------------------------------------------
| IProgressIndicator::numberOfTicks                                            |
|                                                                              |
| Returns the number of ticks for the specified scale.                         |
------------------------------------------------------------------------------*/
unsigned long IProgressIndicator :: numberOfTicks ( Scale scale ) const
{
   WNDPARAMS wndParams;
   SLDCDATA sldcData;

   // Set up buffer with query information
   wndParams.fsStatus = WPM_CTLDATA | WPM_CBCTLDATA;
   wndParams.cbCtlData = sizeof( SLDCDATA );
   wndParams.pCtlData = (PVOID)&sldcData;

   unsigned long ulSuccess =
           handle().sendEvent(WM_QUERYWINDOWPARAMS,
                              IEventParameter1(&wndParams),
                              IEventParameter2((unsigned long)0));

   if (!ulSuccess)
      ITHROWLIBRARYERROR(IC_SLDR_GET_CTLDATA,
                         IErrorInfo::accessError,
                         IException::recoverable);
   if (scale == scale1)
      return ((PSLDCDATA)(wndParams.pCtlData))->usScale1Increments;
   else
      return ((PSLDCDATA)(wndParams.pCtlData))->usScale2Increments;
}

/*------------------------------------------------------------------------------
| IProgressIndicator::setTicks                                                 |
|                                                                              |
| Sets the number of ticks and number of pixels between ticks.                 |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator :: setTicks (
                                             unsigned long scale1NumberOfTicks,
                                             unsigned long scale2NumberOfTicks,
                                             unsigned long scale1TickSpacing,
                                             unsigned long scale2TickSpacing )
{
   WNDPARAMS wndParams;
   SLDCDATA sldcData;

   sldcData.cbSize = sizeof(SLDCDATA);
   sldcData.usScale1Increments = (unsigned short)scale1NumberOfTicks;
   sldcData.usScale1Spacing =  (unsigned short)scale1TickSpacing;
   sldcData.usScale2Increments = (unsigned short)scale2NumberOfTicks;
   sldcData.usScale2Spacing = (unsigned short)scale2TickSpacing;
   wndParams.fsStatus = WPM_CTLDATA;
   wndParams.cbCtlData = sizeof(SLDCDATA);
   wndParams.pCtlData = (void*)(&sldcData);

   unsigned long ulSuccess =
           handle().sendEvent(WM_SETWINDOWPARAMS,
                              IEventParameter1(&wndParams),
                              IEventParameter2((unsigned long)0));
   if (!ulSuccess)
      ITHROWLIBRARYERROR(IC_SLDR_SET_CTLDATA,
                         IErrorInfo::accessError,
                         IException::recoverable);

   refresh();
   return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator::setTicks                                                 |
|                                                                              |
| Sets the number of ticks and number of pixels between ticks.                 |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator :: setTicks (Scale scale,
                                                    unsigned long numberOfTicks,
                                                    unsigned long tickSpacing )
{
   WNDPARAMS wndParams;
   SLDCDATA sldcData;
   unsigned long ulSuccess;

  // Set up buffer with query information
   wndParams.fsStatus = WPM_CTLDATA | WPM_CBCTLDATA;
   wndParams.cbCtlData = sizeof( SLDCDATA );
   wndParams.pCtlData = (PVOID)&sldcData;
   ulSuccess = handle().sendEvent(WM_QUERYWINDOWPARAMS,
                                  IEventParameter1(&wndParams),
                                  IEventParameter2((unsigned long)0));
   if (!ulSuccess)
      ITHROWLIBRARYERROR(IC_SLDR_GET_CTLDATA,
                         IErrorInfo::accessError,
                         IException::recoverable);

   sldcData.cbSize = sizeof(SLDCDATA);
   if (scale == scale1)
   {
      sldcData.usScale1Increments = (unsigned short)numberOfTicks;
      sldcData.usScale1Spacing =  (unsigned short)tickSpacing;
   }
   else
   {
      sldcData.usScale2Increments = (unsigned short)numberOfTicks;
      sldcData.usScale2Spacing = (unsigned short)tickSpacing;
   }
   wndParams.fsStatus = WPM_CTLDATA;
   wndParams.cbCtlData = sizeof(SLDCDATA);
   wndParams.pCtlData = (void*)&sldcData;

   ulSuccess = handle().sendEvent(WM_SETWINDOWPARAMS,
                                  IEventParameter1(&wndParams),
                                  IEventParameter2((unsigned long)0));
   if (!ulSuccess)
      ITHROWLIBRARYERROR(IC_SLDR_SET_CTLDATA,
                         IErrorInfo::accessError,
                         IException::recoverable);

   refresh();
   return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator::setTickLength                                            |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator :: setTickLength (
                                                    unsigned long ulTickNum,
                                                    unsigned long ulTickLength )
{
   IMODTRACE_DEVELOP("IProgressIndicator::setTickLength");
   unsigned long ulResult =
           handle().sendEvent(SLM_SETTICKSIZE,
                              IEventParameter1(ulTickNum, ulTickLength),
                              IEventParameter2((unsigned long)0 ));
   if (!ulResult)
   {
      ITHROWGUIERROR2("SLM_SETTICKSIZE",
                      IErrorInfo::invalidParameter,
                      IException::recoverable);
   }

   return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator::setTickLength                                            |
|                                                                              |
| Set the size of ALL ticks (in pixels).                                       |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator :: setTickLength (
                                                  unsigned long ulTickLength )
{
   IMODTRACE_DEVELOP("IProgressIndicator::setTickLength (all)");
   unsigned long ulResult =
           handle().sendEvent(SLM_SETTICKSIZE,
                              IEventParameter1((unsigned long)SMA_SETALLTICKS,
                                                ulTickLength),
                              IEventParameter2((unsigned long)0 ));

   if (!ulResult)
   {
      ITHROWGUIERROR2("SLM_SETTICKSIZE",
                      IErrorInfo::invalidParameter,
                      IException::recoverable);
   }
   return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator::setTickText                                              |
|                                                                              |
| Set the text above or text to a tick.                                        |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator :: setTickText ( unsigned long ulTickNum,
                                                        const char* pszTickText)
{
   IMODTRACE_DEVELOP("IProgressIndicator::setTickText");
   unsigned long ulResult =
           handle().sendEvent(SLM_SETSCALETEXT,
                              IEventParameter1(ulTickNum),
                              IEventParameter2((void*)pszTickText));

   if (!ulResult)
   {
      ITHROWGUIERROR2("SLM_SETSCALETEXT",
                      IErrorInfo::invalidParameter,
                      IException::recoverable);
   }
   return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator::setTickText                                              |
|                                                                              |
| Set the text above or below the tick.                                        |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator :: setTickText ( unsigned long ulTickNum,
                                                        const IResourceId& text)
{
   IMODTRACE_DEVELOP("IProgressIndicator::setTickText");
   IString str = text.resourceLibrary().loadString(text);
   setTickText(ulTickNum,(char*)str);

   return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator::shaftPosition                                            |
|                                                                              |
| Return the lower left point of the shaft relative to the lower left          |
| of the control window.                                                       |
------------------------------------------------------------------------------*/
IPoint IProgressIndicator :: shaftPosition ( ) const
{
   IMODTRACE_DEVELOP("IProgressIndicator::shaftPosition");

   IEventResult evtShaftData =
          handle().sendEvent(SLM_QUERYSLIDERINFO,
                             IEventParameter1((unsigned long)SMA_SHAFTPOSITION),
                             IEventParameter2((unsigned long)0));

   IRectangle shaftRect(
                 IPoint((unsigned short)evtShaftData.number1(),
                        (unsigned short)evtShaftData.number2() ),
                 this->shaftSize() );
   return ICoordinateSystem::convertToApplication(
             shaftRect,
             this->size() ).minXMinY() ;
}


/*------------------------------------------------------------------------------
| IProgressIndicator::shaftSize                                                |
------------------------------------------------------------------------------*/
ISize IProgressIndicator :: shaftSize ( ) const
{
   IMODTRACE_DEVELOP("IProgressIndicator::shaftSize");

   IEventResult evtShaftData =
        handle().sendEvent(SLM_QUERYSLIDERINFO,
                           IEventParameter1((unsigned long)SMA_SHAFTDIMENSIONS),
                           IEventParameter2((unsigned long)0));

   ISize sz;
   if (style() & vertical.asUnsignedLong())
   {
      sz.setWidth((unsigned short)evtShaftData.number2());
      sz.setHeight((unsigned short)evtShaftData.number1());
   }
   else
   {
      sz.setWidth((unsigned short)evtShaftData.number1());
      sz.setHeight((unsigned short)evtShaftData.number2());
   }
   return sz;
}

/*------------------------------------------------------------------------------
| IProgressIndicator::setShaftPosition                                         |
|                                                                              |
| Set the position of the progress indicator / slider shaft.                   |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator :: setShaftPosition ( const IPoint& position )
{
   IMODTRACE_DEVELOP("IProgressIndicator::setShaftPosition");

   IPoint nativePt = ICoordinateSystem::convertToNative(
                        IRectangle( position, this->shaftSize() ),
                        this->size() ).minXMinY() ;

   unsigned long ulResult =
        handle().sendEvent(SLM_SETSLIDERINFO,
                           IEventParameter1((unsigned long)SMA_SHAFTPOSITION),
                           IEventParameter2(nativePt.x(), nativePt.y()));
   if (!ulResult)
      ITHROWGUIERROR2("SLM_SETSLIDERINFO",
                      IErrorInfo::invalidParameter,
                      IException::recoverable);

   return *this;

}

/*------------------------------------------------------------------------------
| IProgressIndicator::setShaftBreadth                                          |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator :: setShaftBreadth (
                                                  unsigned long ulShaftBreadth )
{
   IMODTRACE_DEVELOP("IProgressIndicator::setShaftBreadth");

   unsigned long ulResult =
        handle().sendEvent(SLM_SETSLIDERINFO,
                           IEventParameter1((unsigned long)SMA_SHAFTDIMENSIONS),
                           IEventParameter2(ulShaftBreadth));
   if (!ulResult)
      ITHROWGUIERROR2("SLM_SETSLIDERINFO",
                      IErrorInfo::invalidParameter,
                      IException::recoverable);

   return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator::armRange                                                 |
|                                                                              |
| Query the range over which the arm can be moved.                             |
------------------------------------------------------------------------------*/
unsigned long IProgressIndicator :: armRange ( ) const
{
   IMODTRACE_DEVELOP("IProgressIndicator::armRange");

   IEventResult evtArmData =
      handle().sendEvent(SLM_QUERYSLIDERINFO,
                         IEventParameter1((unsigned long)SMA_SLIDERARMPOSITION,
                                          (unsigned long)SMA_RANGEVALUE ),
                         IEventParameter2((unsigned long)0 ));
   return evtArmData.number2();
}

/*------------------------------------------------------------------------------
| IProgressIndicator::armPixelOffset                                           |
|                                                                              |
| Get position of arm as an offset from home in pixels.                        |
------------------------------------------------------------------------------*/
unsigned long IProgressIndicator :: armPixelOffset ( ) const
{
   IMODTRACE_DEVELOP("IProgressIndicator::armPixelOffset");
   IEventResult evtArmData =
      handle().sendEvent(SLM_QUERYSLIDERINFO,
                         IEventParameter1((unsigned long)SMA_SLIDERARMPOSITION,
                                          (unsigned long)SMA_RANGEVALUE),
                         IEventParameter2((unsigned long)0));
   return evtArmData.number1();
}

/*------------------------------------------------------------------------------
| IProgressIndicator::armTickOffset                                            |
|                                                                              |
| Return the position of the arm as a tick number.  Ticks are numbered         |
| starting at 0.                                                               |
------------------------------------------------------------------------------*/
unsigned long IProgressIndicator :: armTickOffset ( ) const
{
   IMODTRACE_DEVELOP("IProgressIndicator::armTickOffset");

   IEventResult evtArmData =
      handle().sendEvent(SLM_QUERYSLIDERINFO,
                         IEventParameter1((unsigned long)SMA_SLIDERARMPOSITION,
                                          (unsigned long)SMA_INCREMENTVALUE),
                         IEventParameter2((unsigned long)0));

   return evtArmData.number1();
}

/*------------------------------------------------------------------------------
| IProgressIndicator::moveArmToTick                                            |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator :: moveArmToTick (
                                                  unsigned long ulTickNum )
{
  IMODTRACE_DEVELOP("IProgressIndicator::moveArmToTick");

  /*******************************************************************/
  /* Only attempt to set it if we are attempting to set a new value. */
  /*******************************************************************/
  if (armTickOffset() != ulTickNum)
  {
    unsigned long ulResult =
       handle().sendEvent(SLM_SETSLIDERINFO,
                IEventParameter1((unsigned long)SMA_SLIDERARMPOSITION,
                                 (unsigned long)SMA_INCREMENTVALUE),
                                 IEventParameter2(ulTickNum));
    if (ulResult)
    {
      notifyObservers(INotificationEvent(IProgressIndicator::armTickOffsetId,
                                         *this, true, (void*)ulTickNum));
    }
    else
    {
      ITHROWGUIERROR2("SLM_SETSLIDERINFO",
                      IErrorInfo::invalidParameter,
                      IException::recoverable);
    }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| Function Name: IProgressIndicator :: moveArmToPixel                          |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator :: moveArmToPixel (
                                                  unsigned long ulArmOffset )
{
  IMODTRACE_DEVELOP("IProgressIndicator::moveArmToPixel");

  /*******************************************************************/
  /* Only attempt to set it if we are attempting to set a new value. */
  /*******************************************************************/
  if (armPixelOffset() != ulArmOffset)
  {
    unsigned long ulResult =
       handle().sendEvent(SLM_SETSLIDERINFO,
                          IEventParameter1((unsigned long)SMA_SLIDERARMPOSITION,
                                           (unsigned long)SMA_RANGEVALUE),
                          IEventParameter2(ulArmOffset));

    if (ulResult)
    {
      notifyObservers(INotificationEvent(IProgressIndicator::armPixelOffsetId,
                                         *this, true, (void*)ulArmOffset));
    }
    else
    {
      ITHROWGUIERROR2("SLM_SETSLIDERINFO",
                       IErrorInfo::invalidParameter,
                       IException::recoverable);
    }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator::initialize                                               |
------------------------------------------------------------------------------*/
void IProgressIndicator :: initialize ( unsigned long     windowId,
                                        unsigned long     parent,
                                        unsigned long     owner,
                                        unsigned long     ulStyle,
                                        const IRectangle& initial,
                                        void *            sliderData )
{

  IWindowHandle wndhSlider =
      this -> create( windowId,
                      0,
                      ulStyle,
                      WC_SLIDER,
                      parent,
                      owner,
                      initial,
                      sliderData,
                      0 );


  startHandlingEventsFor(wndhSlider);
  setBackgroundColor(defaultbackgroundcolor(wndhSlider));
}

/*------------------------------------------------------------------------------
| IProgressIndicator::backgroundColor                                          |
|                                                                              |
| Returns the background color of the progress indicator.                      |
------------------------------------------------------------------------------*/
IColor IProgressIndicator::backgroundColor () const
{
  return (IWindow::color(PP_BACKGROUNDCOLOR,
                         IGUIColor(IGUIColor::defaultControl)));
}

/*------------------------------------------------------------------------------
| IProgressIndicator::calMinimumSize                                           |
|                                                                              |
| Calculate the minimum screen size needed by the control.                     |
------------------------------------------------------------------------------*/
ISize  IProgressIndicator :: calcMinimumSize ( ) const
{
   if (style() & vertical.asUnsignedLong())
      return ISize(30,100);

   return ISize(100,30);
}

/*------------------------------------------------------------------------------
| IProgressIndicator::getTickText                                              |
|                                                                              |
| Return the text associated with a tick.                                      |
------------------------------------------------------------------------------*/
char* IProgressIndicator :: getTickText ( unsigned long ulTickNum,
                                          char* pszBuffer,
                                          unsigned long ulBufLength ) const
{
   IMODTRACE_DEVELOP("IProgressIndicator::getTickText");
   char* pszTemp;

   if(pszBuffer==0 || ulBufLength==0)
   {
      ulBufLength = tickTextLength(ulTickNum)+1;
      pszTemp = new char[ulBufLength];
      pszBuffer = pszTemp;
   }

   IEventResult evtTickData =
           handle().sendEvent(SLM_QUERYSCALETEXT,
                              IEventParameter1(ulTickNum, ulBufLength),
                              IEventParameter2((void*)pszBuffer));

   if (evtTickData.number1() == (USHORT)SLDERR_INVALID_PARAMETERS)
      ITHROWGUIERROR2("SLM_QUERYSCALETEXT",
                      IErrorInfo::invalidParameter,
                      IException::recoverable);

   return pszBuffer;
}

/*------------------------------------------------------------------------------
| IProgressIndiactor::tickTextLength                                           |
|                                                                              |
| Return the length of the text associated with a tick.                        |
------------------------------------------------------------------------------*/
unsigned long IProgressIndicator :: tickTextLength (
                                                unsigned long ulTickNum ) const
{
   IMODTRACE_DEVELOP("IProgressIndicator::tickTextLength");
   IEventResult evtTickData =
           handle().sendEvent(SLM_QUERYSCALETEXT,
                              IEventParameter1(ulTickNum, (unsigned long)0),
                              IEventParameter2((unsigned long)0));

   if (evtTickData.number1() == (USHORT)SLDERR_INVALID_PARAMETERS)
      ITHROWGUIERROR2("SLM_QUERYSCALETEXT",
                      IErrorInfo::invalidParameter,
                      IException::recoverable);

   return evtTickData.number1();
}

/*------------------------------------------------------------------------------
| IProgressIndicator::IProgressIndicator                                       |
|                                                                              |
| Constructor from an existing progress indicator.                             |
------------------------------------------------------------------------------*/
IProgressIndicator :: IProgressIndicator( const IWindowHandle& wh)
{
  setAutoDestroyWindow(false);
  startHandlingEventsFor(wh);
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: alignment                                              |
------------------------------------------------------------------------------*/
IProgressIndicator::Alignment IProgressIndicator :: alignment ( ) const
{
   unsigned long ulStyle = style();
   if (ulStyle & alignLeft.asUnsignedLong())
      return bottomLeft;
   if (ulStyle & alignRight.asUnsignedLong())
      return topRight;

   return centered;
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: homePosition                                           |
------------------------------------------------------------------------------*/
IProgressIndicator::HomePosition IProgressIndicator :: homePosition ( ) const
{
   return( (style() & homeTop.asUnsignedLong() ? homeTopRight
                                               : homeBottomLeft) );
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: setHomePosition                                        |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator :: setHomePosition ( HomePosition home )
{
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (home == homeBottomLeft)
  {
    ulStyle &= ~(homeTop.asUnsignedLong());
  }
  else
  {
    ulStyle |= homeTop.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    refresh();
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: primaryScale                                           |
------------------------------------------------------------------------------*/
IProgressIndicator::Scale IProgressIndicator :: primaryScale ( ) const
{
   return( (style() & primaryScale2.asUnsignedLong() ? scale2 : scale1) );
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: setPrimaryScale                                        |
|                                                                              |
| Replaces the primary scale style of the progress indicator.                  |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator :: setPrimaryScale ( Scale primaryScale )
{
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (primaryScale == scale1)
  {
    ulStyle &= ~(primaryScale2.asUnsignedLong());
  }
  else
  {
    ulStyle |= primaryScale2.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    refresh();
    notifyObservers(INotificationEvent(IProgressIndicator::scaleId,
                                       *this, true, (void*)primaryScale));
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: isVertical                                             |
------------------------------------------------------------------------------*/
IBase::Boolean IProgressIndicator :: isVertical ( ) const
{
   return( (style() & vertical.asUnsignedLong() ? true : false) );
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: isRibbonStripEnabled                                   |
------------------------------------------------------------------------------*/
IBase::Boolean IProgressIndicator :: isRibbonStripEnabled ( ) const
{
   return( (style() & ribbonStrip.asUnsignedLong() ? true : false) );
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: isPMCompatible                                         |
------------------------------------------------------------------------------*/
IBase::Boolean IProgressIndicator :: isPMCompatible ( ) const
{
   return true;
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: enableRibbonStrip                                      |
|                                                                              |
| Sets the style to be ribbon-strip or not depending on the                    |
| input flag.                                                                  |
------------------------------------------------------------------------------*/
IProgressIndicator&  IProgressIndicator :: enableRibbonStrip ( Boolean ribbon)
{
  unsigned long ulStyle = style();      // get the current style
  unsigned long ulOldStyle = ulStyle;   // save it

  if (ribbon)
  {
    ulStyle |= ribbonStrip.asUnsignedLong();     //add ribbon-strip style
  }
  else
  {
    ulStyle &= ~(ribbonStrip.asUnsignedLong());  //remove ribbon-strip style
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    refresh();
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: disableRibbonStrip                                     |
------------------------------------------------------------------------------*/
IProgressIndicator&  IProgressIndicator :: disableRibbonStrip ( )
{
   enableRibbonStrip( false );
   return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: isDrawItemEnabled                                      |
------------------------------------------------------------------------------*/
IBase::Boolean IProgressIndicator :: isDrawItemEnabled    ( ) const
{
   return( (style() & handleDrawItem.asUnsignedLong() ? true : false) );
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: enableDrawItem                                         |
------------------------------------------------------------------------------*/
IProgressIndicator&  IProgressIndicator :: enableDrawItem( Boolean drawItem )
{
  unsigned long ulStyle = style();    // get current style
  unsigned long ulOldStyle = ulStyle;   // save it

  if (drawItem)
  {
     ulStyle |= handleDrawItem.asUnsignedLong();
     // add owner draw style
  }
  else
  {
     ulStyle &= ~(handleDrawItem.asUnsignedLong());
     // remove previous draw style
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    refresh();
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: disableDrawItem                                        |
------------------------------------------------------------------------------*/
IProgressIndicator&  IProgressIndicator :: disableDrawItem ( )
{
   enableDrawItem( false );
   return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: isSnapToTickEnabled                                    |
------------------------------------------------------------------------------*/
IBase::Boolean IProgressIndicator :: isSnapToTickEnabled  ( ) const
{
   return( (style() & snapToTickMark.asUnsignedLong() ? true : false) );
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: enableSnapToTick                                       |
|                                                                              |
| Sets the style to be snap-to-tick or not depending on the                    |
| input flag.                                                                  |
------------------------------------------------------------------------------*/
IProgressIndicator& IProgressIndicator::enableSnapToTick ( Boolean snapToTick )
{
  unsigned long ulStyle = style();        // get the current style
  unsigned long ulOldStyle = ulStyle;     // save it

  if (snapToTick)
  {
    ulStyle |= snapToTickMark.asUnsignedLong();
                                              //add snap-to-increment style
  }
  else
  {
    ulStyle &= ~(snapToTickMark.asUnsignedLong());
                                              //remove snap-to-increment style
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    refresh();
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: disableSnapToTick                                      |
------------------------------------------------------------------------------*/
IProgressIndicator&  IProgressIndicator :: disableSnapToTick ( )
{
   enableSnapToTick( false );
   return *this;
}

/*------------------------------------------------------------------------------
| IProgressIndicator :: tickText                                               |
------------------------------------------------------------------------------*/
IString IProgressIndicator :: tickText(unsigned long ulTickNum) const
{
   char* pszTickText = getTickText(ulTickNum);

   IString strTickText(pszTickText);
   delete(pszTickText);
   return strTickText;
}

/*------------------------------------------------------------------------------
| Public slider styles                                                         |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const ISlider::Style
  ISlider::buttonsLeft       = SLS_BUTTONSLEFT,
  ISlider::buttonsRight      = SLS_BUTTONSRIGHT,
  ISlider::buttonsBottom     = SLS_BUTTONSBOTTOM,
  ISlider::buttonsTop        = SLS_BUTTONSTOP,
  ISlider::classDefaultStyle ( (WS_VISIBLE |
                                SLS_BUTTONSLEFT),
                                                  (ISLS_HORIZONTAL |
                                                   ISLS_CENTER     |
                                                   ISLS_HOMELEFT   |
                                                   ISLS_PRIMARYSCALE1) );
#pragma data_seg()

/*------------------------------------------------------------------------------
| Default style for new slider objects (initial value)                         |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
ISlider::Style
  ISlider::currentDefaultStyle ( (WS_VISIBLE |
                                  SLS_BUTTONSLEFT),
                                                    (ISLS_HORIZONTAL |
                                                     ISLS_CENTER     |
                                                     ISLS_HOMELEFT   |
                                                     ISLS_PRIMARYSCALE1) );
#pragma data_seg()


/*------------------------------------------------------------------------------
| ISlider::ISlider                                                             |
|                                                                              |
| Constructor to create a scale control on a standard window.                  |
------------------------------------------------------------------------------*/
ISlider :: ISlider( unsigned long     windowId,
                    IWindow*          parent,
                    IWindow*          owner,
                    const IRectangle& initial,
                    unsigned long     numberOfTicks,
                    unsigned long     tickSpacing,
                    const Style&      style )
                 : IProgressIndicator()
{
   IMODTRACE_DEVELOP("ISlider::ctor1");
   IASSERTPARM(parent!=0);             //Assertions on input parms

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

   SLDCDATA sldcData;                  //Slider control data
   sldcData.cbSize = sizeof(SLDCDATA);
   if (style & primaryScale2)
   {
      sldcData.usScale2Increments = (unsigned short)numberOfTicks;
      sldcData.usScale2Spacing = (unsigned short)tickSpacing;
      sldcData.usScale1Increments = (unsigned short)0;
      sldcData.usScale1Spacing = (unsigned short)0;
   }
   else
   {
      sldcData.usScale1Increments = (unsigned short)numberOfTicks;
      sldcData.usScale1Spacing = (unsigned short)tickSpacing;
      sldcData.usScale2Increments = (unsigned short)0;
      sldcData.usScale2Spacing = (unsigned short)0;
   }
   unsigned long hwndOwner = 0;
   if (owner != 0)
      hwndOwner = owner->handle();

   initialize(windowId, parent->handle(), hwndOwner,
              convertToGUIStyle( style ), initial, (void*)&sldcData);
}

/*------------------------------------------------------------------------------
| ISlider::ISlider                                                             |
|                                                                              |
| Constructor to create a scale control on a standard window.                  |
------------------------------------------------------------------------------*/
ISlider :: ISlider ( unsigned long         windowId,
                     IWindow*              parent,
                     IWindow*              owner,
                     const IRectangle&     initial,
                     unsigned long         scale1NumberOfTicks,
                     unsigned long         scale1TickSpacing,
                     unsigned long         scale2NumberOfTicks,
                     unsigned long         scale2TickSpacing,
                     const Style&          style )
                 : IProgressIndicator()
{
   IMODTRACE_DEVELOP("ISlider::ctor");
   IASSERTPARM(parent!=0);             //Assertions on input parms

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

   SLDCDATA sldcData;                  //Slider control data
   sldcData.cbSize = sizeof(SLDCDATA);
   sldcData.usScale1Increments = (unsigned short)scale1NumberOfTicks;
   sldcData.usScale2Increments = (unsigned short)scale2NumberOfTicks;
   sldcData.usScale1Spacing = (unsigned short)scale1TickSpacing;
   sldcData.usScale2Spacing = (unsigned short)scale2TickSpacing;

   unsigned long hwndOwner = 0;
   if (owner != 0)
      hwndOwner = owner->handle();

   initialize(windowId, parent->handle(), hwndOwner,
              convertToGUIStyle( style ), initial, (void*)&sldcData);
}

/*------------------------------------------------------------------------------
| ISlider :: ISlider                                                           |
------------------------------------------------------------------------------*/
ISlider ::  ISlider( unsigned long ulId,
                      IWindow* pwndParent)
               : IProgressIndicator(ulId, pwndParent)
{ }

/*------------------------------------------------------------------------------
| ISlider :: ISlider                                                           |
------------------------------------------------------------------------------*/
ISlider :: ISlider( const IWindowHandle& wh )
               : IProgressIndicator(wh)
{ }

/*------------------------------------------------------------------------------
| ISlider::~ISlider                                                            |
------------------------------------------------------------------------------*/
ISlider :: ~ISlider ( )
{ }

/*------------------------------------------------------------------------------
| ISlider::defaultStyle                                                        |
------------------------------------------------------------------------------*/
ISlider::Style  ISlider :: defaultStyle ( )
{
   return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| ISlider::convertToGUIStyle                                                   |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long ISlider::convertToGUIStyle(const IBitFlag& guiStyle,
                                         Boolean bExtOnly) const
{
  // Obtain the style from the class (IProgressIndicator) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= extendedStyle() & IS_EXTMASK;
  }
  else
  {
    // Note that since this class only deals with button related styles,
    // we only mask those out here.
    ulStyle |= guiStyle.asUnsignedLong() & ISLS_BUTTONMASK;
  }

  return( ulStyle );
}

/*------------------------------------------------------------------------------
| ISlider::buttonsPosition                                                     |
|                                                                              |
| Returns the position of the slider's buttons.                                |
------------------------------------------------------------------------------*/
ISlider::ButtonsPosition ISlider :: buttonsPosition ( ) const
{
   IMODTRACE_DEVELOP("ISlider::buttonsPosition");

   unsigned long ulStyle = style();
   if (ulStyle & buttonsTop.asUnsignedLong())
   {
      if (ulStyle & vertical.asUnsignedLong())
         return top;
      else
         return right;
   }
   else if (ulStyle & buttonsBottom.asUnsignedLong())
   {
      if (ulStyle & vertical.asUnsignedLong())
         return bottom;
      else
         return left;
   }
   else
     return none;
}

/*------------------------------------------------------------------------------
| ISlider::detentPosition                                                      |
|                                                                              |
| Returns the offset of a detent from the home position, in pixels.            |
------------------------------------------------------------------------------*/
unsigned long ISlider :: detentPosition ( unsigned long ulDetentId ) const
{
   IMODTRACE_DEVELOP("ISlider::detentPosition");
   IEventResult evtDetentData =
           handle().sendEvent(SLM_QUERYDETENTPOS,
                              IEventParameter1(ulDetentId),
                              IEventParameter2((unsigned long)0));

   if ((short)(evtDetentData.number1()) == SLDERR_INVALID_PARAMETERS)
      ITHROWGUIERROR2("SLM_QUERYDETENTPOS",
                      IErrorInfo::invalidParameter,
                      IException::recoverable);

   return evtDetentData.number1();
}

/*------------------------------------------------------------------------------
| ISlider::addDetent                                                           |
------------------------------------------------------------------------------*/
unsigned long ISlider :: addDetent (unsigned long ulOffset )
{
   IMODTRACE_DEVELOP("ISlider::addDetent");
   unsigned long ulDetentID =
           handle().sendEvent(SLM_ADDDETENT,
                              IEventParameter1(ulOffset),
                              IEventParameter2((unsigned long)0));
   if (!ulDetentID)
   {
      ITHROWGUIERROR2("SLM_ADDDETENT",
                      IErrorInfo::invalidParameter,
                      IException::recoverable);
   }

   return ulDetentID;
}

/*------------------------------------------------------------------------------
| ISlider::removeDetent                                                        |
------------------------------------------------------------------------------*/
ISlider& ISlider :: removeDetent ( unsigned long ulDetentId )
{
   IMODTRACE_DEVELOP("ISlider::removeDetent");
   unsigned long ulResult =
           handle().sendEvent(SLM_REMOVEDETENT,
                              IEventParameter1(ulDetentId),
                              IEventParameter2((unsigned long)0));
   if (!ulResult)
      ITHROWGUIERROR2("SLM_REMOVEDETENT",
                      IErrorInfo::invalidParameter,
                      IException::recoverable);

   return *this;
}

/*------------------------------------------------------------------------------
| ISlider::armSize                                                             |
------------------------------------------------------------------------------*/
ISize ISlider :: armSize ( ) const
{
   IMODTRACE_DEVELOP("ISlider::armSize");

   IEventResult evtShaftData =
      handle().sendEvent(SLM_QUERYSLIDERINFO,
                      IEventParameter1((unsigned long)SMA_SLIDERARMDIMENSIONS),
                      IEventParameter2((unsigned long)0));

   ISize sz;
   if (style() & vertical.asUnsignedLong())
   {
      sz.setWidth((unsigned short)evtShaftData.number2());
      sz.setHeight((unsigned short)evtShaftData.number1());
   }
   else
   {
      sz.setWidth((unsigned short)evtShaftData.number1());
      sz.setHeight((unsigned short)evtShaftData.number2());
   }
   return sz;
}

/*------------------------------------------------------------------------------
| ISlider::setArmSize                                                          |
------------------------------------------------------------------------------*/
ISlider& ISlider :: setArmSize(const ISize& sizSliderArm)
{
   IMODTRACE_DEVELOP("ISlider::setArmSize");
   unsigned short us1;
   unsigned short us2;
   if (style() & vertical.asUnsignedLong())
   {
      us1 = sizSliderArm.height();
      us2 = sizSliderArm.width();
   }
   else
   {
      us1 = sizSliderArm.width();
      us2 = sizSliderArm.height();
   }

   unsigned long ulResult =
      handle().sendEvent(SLM_SETSLIDERINFO,
                       IEventParameter1((unsigned long)SMA_SLIDERARMDIMENSIONS),
                       IEventParameter2(us1, us2));
   if (!ulResult)
      ITHROWGUIERROR2("SLM_SETSLIDERINFO",
                      IErrorInfo::invalidParameter,
                      IException::recoverable);

   return *this;
}

