/*******************************************************************************
* FILE NAME: ispinhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ispinhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINSTDSPIN
#define INCL_WINWINDOWMGR
extern "C" {
  #include <iwindefs.h>
  #include <stdlib.h>
}

#ifndef _ISPINHDR_
  #include <ispinhdr.hpp>
#endif
#ifndef _ICTLEVT_
  #include <ictlevt.hpp>
#endif
#ifndef _IEVENT_
  #include <ievent.hpp>
#endif
#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif
#ifndef _IWCNAME_
  #include <iwcname.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ISPINHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| ISpinHandler::ISpinHandler                                                   |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
ISpinHandler::ISpinHandler ( )
  : ISpinHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| ISpinHandler::~ISpinHandler                                                  |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ISpinHandler::~ISpinHandler ( )
{ }

/*------------------------------------------------------------------------------
| ISpinHandler::dispatchHandlerEvent                                           |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
Boolean ISpinHandler::dispatchHandlerEvent ( IEvent& event )
{
  Boolean bRc = false;

  if ( event.eventId() == WM_CONTROL )
  {                                    // Control event.
     IControlEvent ctlevt( event );
     IWindowHandle hwndControl = ctlevt.controlHandle();

     if ( hwndControl  &&
         IWindowClassName( hwndControl ) == WC_SPINBUTTON )
     {                                 // Spin button case.
        unsigned long subMsg = ctlevt.parameter1().number2();

        if ( subMsg == SPBN_UPARROW )
        {
           bRc = this->arrowUp( ctlevt );
        }
        else if ( subMsg == SPBN_DOWNARROW )
        {
           bRc = this->arrowDown( ctlevt );
        }
        else if ( subMsg == SPBN_ENDSPIN )
        {
           bRc = this->spinEnded( ctlevt );
        }

        if ( bRc )
        {                              // Processing done.
           event.setResult( ctlevt.result() );  // Copy event result.
        }
     }
  }

  return bRc;
}

/*------------------------------------------------------------------------------
| ISpinHandler::arrowUp                                                        |
------------------------------------------------------------------------------*/
Boolean ISpinHandler::arrowUp ( IControlEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| ISpinHandler::arrowDown                                                      |
------------------------------------------------------------------------------*/
Boolean ISpinHandler::arrowDown ( IControlEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| ISpinHandler::spinEnd                                                        |
------------------------------------------------------------------------------*/
Boolean ISpinHandler::spinEnded ( IControlEvent& event )
{
  return false;
}
