/*******************************************************************************
* FILE NAME: ispinszh.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ispinszh.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINMESSAGEMGR   // For WM_ADJUSTWINDOWPOS.
extern "C" {
  #include <iwindefs.h>
}

#include <ispinszh.hpp>
#include <ispinbas.hpp>
#include <iexcept.hpp>
#include <icconst.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISPINSZH_CPP_
  #include <ipagetun.h>
#endif


ISpinButtonSizeHandler* ISpinButtonStatics :: pClSizeHandler = 0;

/*------------------------------------------------------------------------------
| ISpinButtonStatics::~ISpinButtonStatics                                      |
|                                                                              |
| Destructor to clean up.                                                      |
------------------------------------------------------------------------------*/
ISpinButtonStatics :: ~ISpinButtonStatics ( )
{
  if (pClSizeHandler)
  {
     delete pClSizeHandler;
     pClSizeHandler = 0;
  }
}

/*------------------------------------------------------------------------------
| ISpinButtonStatics::sizeHandler                                              |
|                                                                              |
| Returns the common spin button size handler.                                 |
------------------------------------------------------------------------------*/
ISpinButtonSizeHandler& ISpinButtonStatics :: sizeHandler ( )
{
  if (pClSizeHandler == 0)
  {
     pClSizeHandler = new ISpinButtonSizeHandler;
  }
  return *pClSizeHandler;
}

/*------------------------------------------------------------------------------
| ISpinButtonSizeHandler::ISpinButtonSizeHandler                               |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
ISpinButtonSizeHandler :: ISpinButtonSizeHandler ( )
  : ISpinButtonSizeHandler::Inherited ( )
{ }

/*------------------------------------------------------------------------------
| ISpinButtonSizeHandler::~ISpinButtonSizeHandler                              |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ISpinButtonSizeHandler :: ~ISpinButtonSizeHandler ( )
{ }

/*------------------------------------------------------------------------------
| ISpinButtonSizeHandler::windowResize                                         |
|                                                                              |
| Gives the spin button the opportunity to properly resize its child entry     |
| field and reposition its child spin arrows.                                  |
------------------------------------------------------------------------------*/
Boolean ISpinButtonSizeHandler :: windowResize( IResizeEvent& event )
{
  // IC_NOTYET - Need to figure out what to catch in NT
  PSWP pswp = (PSWP)(char*)(event.parameter1());
  if (pswp->fl & SWP_NOADJUST  ||  event.parameter2() == 0)
  {                // No WM_ADJUSTWINDOWPOS message yet.
     event.window()->sendEvent( WM_ADJUSTWINDOWPOS,
                                event.parameter1() );
  }
  return false;
}

/*------------------------------------------------------------------------------
| ISpinButtonSizeHandler::handleEventsFor                                      |
|                                                                              |
| Attach the handler to a spin button.                                         |
------------------------------------------------------------------------------*/
ISpinButtonSizeHandler& ISpinButtonSizeHandler :: handleEventsFor
                                               ( IBaseSpinButton* spinButton )
{
  IASSERTPARM( spinButton != 0 );
  Inherited::handleEventsFor( spinButton );
  return *this;
}

/*------------------------------------------------------------------------------
| ISpinButtonSizeHandler::stopHandlingEventsFor                                |
|                                                                              |
| Detach the handler from a spin button.                                       |
------------------------------------------------------------------------------*/
ISpinButtonSizeHandler&
  ISpinButtonSizeHandler :: stopHandlingEventsFor
                              ( IBaseSpinButton* spinButton )
{
  IASSERTPARM( spinButton != 0 );
  Inherited::stopHandlingEventsFor( spinButton );
  return *this;
}

/*------------------------------------------------------------------------------
| ISpinButtonSizeHandler::handleEventsFor                                      |
------------------------------------------------------------------------------*/
IHandler& ISpinButtonSizeHandler :: handleEventsFor ( IWindow *window )
{                  // Private to hide version in IHandler.
  ITHROWLIBRARYERROR( IC_MEMBER_ACCESS_ERROR,
                      IErrorInfo::invalidRequest,
                      IException::recoverable );
  return *this;
}

/*------------------------------------------------------------------------------
| ISpinButtonSizeHandler::stopHandlingEventsFor                                |
------------------------------------------------------------------------------*/
IHandler& ISpinButtonSizeHandler :: stopHandlingEventsFor
                                       ( IWindow *window )
{                  // Private to hide version in IHandler.
  ITHROWLIBRARYERROR( IC_MEMBER_ACCESS_ERROR,
                      IErrorInfo::invalidRequest,
                      IException::recoverable );
  return *this;
}
