/*******************************************************************************
* FILE NAME: istathdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in istathdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
{
  #define INCL_WINSYS
  #define INCL_GPIPRIMITIVES
  #define INCL_GPILOGCOLORTABLE
  #include <iwindefs.h>
}

#include <istathdr.hpp>

#ifndef _IPAINEVT_
  #include <ipainevt.hpp>
#endif

#ifndef _ISWP_
  #include <iswp.hpp>
#endif

#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

#ifndef _ICONTROL_
  #include <icontrol.hpp>
#endif

#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif

#ifndef _IHANDLE_
  #include <ihandle.hpp>
#endif

#ifndef _ISTATTXT_
  #include <istattxt.hpp>
#endif

#ifndef _ISTRING_
  #include <istring.hpp>
#endif

  #include <ipmbidi.hpp>        // OS/2 specific BIDI functions

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISTATHDR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IStaticTextHandler::IStaticTextHandler                                       |
------------------------------------------------------------------------------*/
IStaticTextHandler :: IStaticTextHandler()
{ }

/*------------------------------------------------------------------------------
| IStaticTextHandler::~IStaticTextHandler                                      |
------------------------------------------------------------------------------*/
IStaticTextHandler :: ~IStaticTextHandler()
{ }

/*------------------------------------------------------------------------------
| IStaticTextHandler::paintWindow                                              |
|                                                                              |
| Process the paint events for a static window.                                |
------------------------------------------------------------------------------*/
Boolean IStaticTextHandler::paintWindow( IPaintEvent& evt )
{

  IStaticText* pStatic = (IStaticText *)evt.window();
  ISize winsize = pStatic->size();
  unsigned long ulLength = pStatic->textLength();
  unsigned long ulStyles = pStatic->style();
  unsigned long ulExtStyles = pStatic->extendedStyle();
  RECTL lprc, lprc2;

  lprc.MINX = 0;
  lprc.MINY = 0;
  lprc.MAXX = winsize.width();
  lprc.MAXY = winsize.height();

  ICREATELOGCOLORTABLE( evt.presSpaceHandle(), 0L,
                        LCOLF_RGB, 0L, 0L, 0 );

  IColor clrFgnd = pStatic->foregroundColor();
  IColor clrBgnd = pStatic->backgroundColor();

  if (ulStyles & IStaticText::fillBackground.asUnsignedLong())
  {
    IColor clrFill = pStatic->fillColor();
    WinFillRect( evt.presSpaceHandle(), &lprc, clrFill.asRGBLong() );
  }


  if (ulLength)
  {
    unsigned long ulStyle = 0;

    IString*      bidiText         = 0;
    LAYOUT_OBJECT bidiLayoutObject = 0;

    //---------------------------------------------------------------
    // Check for BIDI support
    //---------------------------------------------------------------
    if ( IBidiSettings::isBidiSupported() )
    {
      //-------------------------------------------------------------
      // Query the current BIDI attributes
      //-------------------------------------------------------------
      IBidiSettings settings(*pStatic);

      //-------------------------------------------------------------
      // BIDI text strings need to be converted before display, so
      // we need to make a copy of the text (to work with)
      //-------------------------------------------------------------
      bidiText = new IString(pStatic->text());

      //-------------------------------------------------------------
      // Create a layout object to use when drawing BIDI text
      //-------------------------------------------------------------
      bidiLayoutObject = bidiCreateLayoutObject();

      //-------------------------------------------------------------
      // Convert the BIDI text for display
      // Note: We need to turn off symmetric swapping because during
      //       conversion (it will be handled later)
      //-------------------------------------------------------------
      {
        IBidiSettings tmpSettings(*pStatic);
        unsigned long tilde = bidiText->indexOf('~');
        if (tilde)
           bidiText->remove(tilde,1);
        tmpSettings.disableSymmetricSwapping();
        bidiPrepareForDisplay(bidiLayoutObject,*bidiText,tmpSettings,tilde);
        if (tilde)
           bidiText->insert("~",tilde);
      }

      //-------------------------------------------------------------
      // If right-to-left layout, swap left and right justify flags
      //-------------------------------------------------------------
      if ( settings.windowLayout() == IBidiSettings::layoutRightToLeft )
      {
        if ( ulStyles & IStaticText::right.asUnsignedLong() )
        {
          ulStyles &= ~IStaticText::right.asUnsignedLong();
          ulExtStyles |= IStaticText::left.asExtendedUnsignedLong();
        }
        else
        {
          if ( !(ulStyles & IStaticText::center.asUnsignedLong()) )
          {
            ulStyles |= IStaticText::right.asUnsignedLong();
            ulExtStyles &= ~IStaticText::left.asExtendedUnsignedLong();
          }
        }
      }
    } // isBidiSupported

    if (ulExtStyles & IStaticText::underscore.asExtendedUnsignedLong())
      ulStyles |= DT_UNDERSCORE;
    if (ulExtStyles & IStaticText::strikeout.asExtendedUnsignedLong())
      ulStyles |= DT_STRIKEOUT;

    ulStyle |= (ulStyles & (DT_RIGHT | DT_CENTER | DT_BOTTOM |
                            DT_VCENTER | DT_WORDBREAK | DT_MNEMONIC |
                            DT_HALFTONE | DT_UNDERSCORE | DT_STRIKEOUT));

    if (!pStatic->isEnabled())
      ulStyle |= DT_HALFTONE;


    // Draw the text
    IString pszText = pStatic->text();
    if ( bidiText )
       pszText = *bidiText;

    ISize fm = pStatic->characterSize();
    unsigned long cchDrawn = 0;

    // Loop to handle multiple line static text
    while (cchDrawn != ulLength)
    {
      lprc2.MINX = lprc.MINX;
      lprc2.MAXX = lprc.MAXX;
      lprc2.MINY = lprc.MINY;
      lprc2.MAXY = lprc.MAXY;

      //-------------------------------------------------------------
      // Check for BIDI support 
      //-------------------------------------------------------------
      if ( IBidiSettings::isBidiSupported() )
      {
         // For BIDI systems, call specialized draw routine
         IBidiSettings settings(*pStatic);
         cchDrawn += bidiDrawText( evt.presSpaceHandle(),
                                   pszText,
                                   cchDrawn,
                                   bidiLayoutObject,
                                   settings,
                                   ulStyle,
                                   clrFgnd,
                                   clrBgnd,
                                   IRectangle(lprc2) );
      }
      else
      {
        // Calculate the size of the drawing rectangle
        WinDrawText( evt.presSpaceHandle(),
                     ulLength - cchDrawn,
                     (PCHAR)pszText + cchDrawn,
                     &lprc2,
                     0,
                     0,
                     ulStyle | DT_QUERYEXTENT );
        WinFillRect( evt.presSpaceHandle(), &lprc2, clrBgnd.asRGBLong() );
  
        // Draw the static text
        cchDrawn += WinDrawText( evt.presSpaceHandle(),
                                 ulLength - cchDrawn,
                                 (PCHAR)pszText + cchDrawn,
                                 &lprc,
                                 clrFgnd.asRGBLong(),
                                 clrBgnd.asRGBLong(),
                                 ulStyle );
      }

       // Don't loop if no word breaking, or if it is not top aligned.
      if (!(ulStyle & DT_WORDBREAK))
        break;

      if (ulStyle & (DT_VCENTER | DT_BOTTOM))
        break;

       // If out of room for drawing, exit.
      if ((lprc.MAXY -= fm.height()) <= lprc.MINY)
        break;
    } /* end while */

    //---------------------------------------------------------------
    // Clean up any temporary storage used for BIDI support
    //---------------------------------------------------------------
    if ( bidiText )
      delete bidiText;
    if ( bidiLayoutObject )
        bidiDeleteLayoutObject(bidiLayoutObject);

  }  // if length > 0
  return( true );
}

