/*******************************************************************************
* FILE NAME: itbarcnr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in itbarcnr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINFRAMEMGR
  #include <iwindefs.h>
}

#include <itbar.hpp>
#include <itbarcnr.hpp>
#include <iframe.hpp>
#include <itrace.hpp>
#include <ipoint.hpp>
#include <icolor.hpp>
#include <istring.hpp>
#include <iexcept.hpp>
#include <itbpriv.hpp>

#include <iseq.h>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITBARCNR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public tool bar container styles                                             |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const IToolBarContainer::Style
  IToolBarContainer::classDefaultStyle ( WS_VISIBLE, ICNV_PACKTIGHT |
                                                     ICNV_DECKSBYGROUP );
#pragma data_seg()

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
IToolBarContainer::Style
  IToolBarContainer::fgCurrentDefaultStyle ( WS_VISIBLE, ICNV_PACKTIGHT |
                                                         ICNV_DECKSBYGROUP );
#pragma data_seg()

/*------------------------------------------------------------------------------
| Class:   IToolBarContainerData                                               |
|                                                                              |
| Purpose: Store private data used by the class.                               |
|                                                                              |
------------------------------------------------------------------------------*/
class IToolBarContainerData : public IBase
{
public:
  IToolBarContainerData ( IFrameWindow* frame, IFrameWindow::Location location );

  ~IToolBarContainerData() {}

IFrameWindow
 *frame;

IFrameWindow::Location
  loc;

Boolean
  layoutOn;
};

IToolBarContainerData::IToolBarContainerData
                       ( IFrameWindow* frame, IFrameWindow::Location location )
                       : frame(frame),
                         loc(location),
                         layoutOn(true)
{
}

/*------------------------------------------------------------------------------
| IToolBarContainer::IToolBarContainer                                         |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarContainer::IToolBarContainer ( unsigned long          id,
                                       IFrameWindow*          frame,
                                       IFrameWindow::Location location,
                                       const Style&           style )
                  :ISetCanvas(id, frame, frame, IRectangle(), IWindow::noStyle),
                   fToolBarContainerData(new IToolBarContainerData(frame,location))
{
  IMODTRACE_DEVELOP("ITBarCnr::ITBarCnr");

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

  // Set the correct orientation and alignment
  setDeckOrientation ( ISetCanvas::horizontal );
  setAlignment ( ISetCanvas::centerLeft );
  if (( location == IFrameWindow::leftOfClient ) ||
      ( location == IFrameWindow::rightOfClient ))
  {
    setDeckOrientation ( ISetCanvas::vertical );
    setAlignment ( ISetCanvas::topCenter );
  }

  ITRACE_DEVELOP(IString("style =")+
                 IString(style.asUnsignedLong()).d2x());
  ITRACE_DEVELOP(IString("extended style =")+
                 IString(style.asExtendedUnsignedLong()).d2x());

  // Process styles
  convertToGUIStyle( style );

  // Disable group style (not required by IToolBarContainer)
  if (!(style.asUnsignedLong() & IControl::group.asUnsignedLong()))
  {                // Style not explicitly specified.
     setStyle( IWindow::style() & ~IControl::group.asUnsignedLong() );
  }                // Take it off (turned on by ICanvas).

  ITRACE_DEVELOP(IString("window style =")+
                 IString(this->style()).d2x());
  ITRACE_DEVELOP(IString("window extended style =")+
                 IString(this->extendedStyle()).d2x());


  // Turn layout off during initialization
  fToolBarContainerData->layoutOn = false;

  Inherited::setPad(ISize(0,0));
  Inherited::setMargin(ISize(0,0));
  setBackgroundColor(IGUIColor(IGUIColor::menuBgnd));
  frame->addExtension(this, location, IFrameWindow::none);

  // Turn layout back on and cause layout
  fToolBarContainerData->layoutOn = true;
  setLayoutDistorted(IWindow::layoutChanged, 0);
}

/*------------------------------------------------------------------------------
| IToolBarContainer::~IToolBarContainer                                        |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarContainer::~IToolBarContainer ()
{
  IMODTRACE_DEVELOP("ITBarCnr::~ITBarCnr");

  delete fToolBarContainerData;
}

/*------------------------------------------------------------------------------
| IToolBarContainer::setLocation                                               |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarContainer& IToolBarContainer::setLocation
                                      ( IFrameWindow::Location location )
{
  IMODTRACE_DEVELOP("ITBarCnr::setLocation");

  if ( location != fToolBarContainerData->loc )
  {
    // remove the current frame extension
    fToolBarContainerData->frame->disableUpdate();
    fToolBarContainerData->frame->removeExtension(this);
    fToolBarContainerData->frame->enableUpdate();

    // Set the correct style for the canvas
    setDeckOrientation ( ISetCanvas::horizontal );
    setAlignment ( ISetCanvas::centerLeft );

    if (( location == IFrameWindow::leftOfClient ) ||
        ( location == IFrameWindow::rightOfClient ))
    {
      setDeckOrientation ( ISetCanvas::vertical );
      setAlignment ( ISetCanvas::topCenter );
    }

    // Set the correct orientation and margin for child toolbars
    IWindow::ChildCursor cursor(*this);
    for ( cursor.setToFirst(); cursor.isValid(); cursor.setToNext() )
    {
      IToolBar* toolbar = (IToolBar*)IWindow::windowWithHandle(childAt(cursor));
      if ( toolbar )
      {
        // Set the correct new location for the toolbar
        if (location == IFrameWindow::aboveClient)
          toolbar->sendEvent ( IC_UM_TOOLBAR, IC_UM_TBAR_SET_LOCATION,
                               IToolBar::aboveClient );
        if (location == IFrameWindow::belowClient)
          toolbar->sendEvent ( IC_UM_TOOLBAR, IC_UM_TBAR_SET_LOCATION,
                               IToolBar::belowClient );
        if (location == IFrameWindow::leftOfClient)
          toolbar->sendEvent ( IC_UM_TOOLBAR, IC_UM_TBAR_SET_LOCATION,
                               IToolBar::leftOfClient );
        if (location == IFrameWindow::rightOfClient)
          toolbar->sendEvent ( IC_UM_TOOLBAR, IC_UM_TBAR_SET_LOCATION,
                               IToolBar::rightOfClient );

        // Set the correct margin and orientation
        if (( location == IFrameWindow::aboveClient ) ||
            ( location == IFrameWindow::belowClient ))
        {
          if ( toolbar->deckOrientation() == ISetCanvas::vertical )
          {
            toolbar->setDeckOrientation(ISetCanvas::horizontal);
            toolbar->setMargin(ISize(toolbar->margin().height(),
                                     toolbar->margin().width()));
          }
        }
        else
        {
          if ( toolbar->deckOrientation() == ISetCanvas::horizontal )
          {
            toolbar->setDeckOrientation(ISetCanvas::vertical);
            toolbar->setMargin(ISize(toolbar->margin().height(),
                                     toolbar->margin().width()));
          }
        }

        // Check for misfits
        toolbar->filter();
      }
    }

    // Add the new extension
    fToolBarContainerData->frame->addExtension(this, location, IFrameWindow::none);

    // Save location and force layout
    fToolBarContainerData->loc = location;
    setLayoutDistorted(IWindow::layoutChanged |
                       IWindow::immediateUpdate, 0);

  }
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarContainer::location                                                  |
|                                                                              |
------------------------------------------------------------------------------*/
IFrameWindow::Location IToolBarContainer::location ( ) const
{
  return fToolBarContainerData->loc;
}

/*------------------------------------------------------------------------------
|  IToolBarContainer::setLayoutDistorted                                       |
|                                                                              |
| Provide the response specific for a tool bar container to various distortion |
| notifications.  Some will cause the canvas to need to update how it has      |
| layed out its child windows.                                                 |
------------------------------------------------------------------------------*/
IToolBarContainer& IToolBarContainer::setLayoutDistorted
                                      ( unsigned long layoutAttributeOn,
                                        unsigned long layoutAttributeOff )
{
  IMODTRACE_DEVELOP("ITBarCnr::setLayoutDistorted");
  ITRACE_DEVELOP(IString("Layout Attribute is: ")+IString(layoutAttributeOn).d2x());

  if(fToolBarContainerData->layoutOn || layoutAttributeOn != IWindow::layoutChanged)
  {
    Inherited::setLayoutDistorted(layoutAttributeOn, layoutAttributeOff);
    ITRACE_DEVELOP("Called Inherited::setLayoutDistorted");
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarContainer::defaultStyle                                              |
|                                                                              |
| Returns the current default style.                                           |
| Notes: Can not inline this since it is in the constructor declaration.       |
------------------------------------------------------------------------------*/
IToolBarContainer::Style  IToolBarContainer::defaultStyle ( )
{
  return fgCurrentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IToolBarContainer::setDefaultStyle                                           |
|                                                                              |
| Sets the current default style.                                              |
------------------------------------------------------------------------------*/
void IToolBarContainer::setDefaultStyle ( const Style& style )
{
  fgCurrentDefaultStyle = style;
}

/*------------------------------------------------------------------------------
| IToolBarContainer::convertToGUIStyle                                         |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IToolBarContainer :: convertToGUIStyle(const IBitFlag& guiStyle,
                                                     Boolean bExtOnly) const
{
  // Obtain the style from the class (ISetCanvas) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Same as extended style mask for ISetCanvas, so we do not mask again.
    return( ulStyle );
  }
  else
  {
    // All the styles for the toolbar are extended styles, so we do not mask.
    ulStyle |= guiStyle.asUnsignedLong();
  }

  return( ulStyle );
}

/*------------------------------------------------------------------------------
| IToolBarContainer::frameToolBarContainer                                     |
|                                                                              |
| Returns the tool bar container for the specified frame and location.         |
------------------------------------------------------------------------------*/
IToolBarContainer* IToolBarContainer::frameToolBarContainer
                                      ( IFrameWindow*          frame,
                                        IFrameWindow::Location frameLocation )
{
  IToolBar::FrameCursor cursor(frame);
  for ( cursor.setToFirst(); cursor.isValid(); cursor.setToNext() )
  {
     IToolBar* toolbar = IToolBar::toolBarAt(cursor);

     if (( toolbar->location() == IToolBar::aboveClient ) &&
         ( frameLocation == IFrameWindow::aboveClient ) &&
         ( toolbar->toolBarContainer() != 0 ))
     {
       return toolbar->toolBarContainer();
     }

     if (( toolbar->location() == IToolBar::belowClient ) &&
         ( frameLocation == IFrameWindow::belowClient ) &&
         ( toolbar->toolBarContainer() != 0 ))
     {
       return toolbar->toolBarContainer();
     }

     if (( toolbar->location() == IToolBar::leftOfClient ) &&
         ( frameLocation == IFrameWindow::leftOfClient ) &&
         ( toolbar->toolBarContainer() != 0 ))
     {
       return toolbar->toolBarContainer();
     }

     if (( toolbar->location() == IToolBar::rightOfClient ) &&
         ( frameLocation == IFrameWindow::rightOfClient ) &&
         ( toolbar->toolBarContainer() != 0 ))
     {
       return toolbar->toolBarContainer();
     }
  }
  return 0;
}

