/*******************************************************************************
* FILE NAME: itbarfrm.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in itbarfrm.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
{
  #define INCL_WINSYS
  #define INCL_WINFRAMEMGR
  #define INCL_WINFRAMECTLS
  #define INCL_WINWINDOWMGR
  #define INCL_WINCOUNTRY
  #define INCL_WINHOOKS

  #include <iwindefs.h>
}

#include <itbarfrm.hpp>
#include <itbfpriv.hpp>
#include <itbar.hpp>
#include <iswp.hpp>
#include <ifont.hpp>
#include <igstring.hpp>
#include <igbitmap.hpp>
#include <igregion.hpp>
#include <igbundle.hpp>
#include <itbpriv.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITBARFRM_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLStaticConst)
static const unsigned long titleBarHeight(19);
#pragma data_seg()

#pragma data_seg(ICLNonConst)
unsigned long IToolBarTitle::offSet(0);
#pragma data_seg()

/*------------------------------------------------------------------------------
| IToolBarFrameData::IToolBarFrameData                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameData::IToolBarFrameData( IToolBarFrameWindow* toolBarFrameWindow )
  : systemMenu( toolBarFrameWindow ),
    toolBarTitle( toolBarFrameWindow, "" ),
    titleFrameHandler(),
    systemMenuDrawHandler(),
    titleBarCommandHandler(),
    showHideButton( IC_ID_SHOWHIDE,
                    toolBarFrameWindow,
                    toolBarFrameWindow,
                    IRectangle(),
                    IToolBarButton::bitmapVisible |
                    IWindow::visible              |
                    IButton::noPointerFocus ),
    moveButton( IC_ID_OWNERMOVE,
                toolBarFrameWindow,
                toolBarFrameWindow,
                IRectangle(),
                IToolBarButton::bitmapVisible  |
                ICustomButton::latchable       |
                IWindow::visible               |
                IButton::noPointerFocus )
{ }

/*------------------------------------------------------------------------------
| IToolBarFrameData::~IToolBarFrameData                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameData::~IToolBarFrameData( )
{ }

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::IToolBarFrameWindow                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameWindow::IToolBarFrameWindow( unsigned long windowId,
                                          IFrameWindow* owner )
  : IToolBarFrameWindow::Inherited( windowId,
                                    IWindow::desktopWindow(),
                                    owner,
                                    IRectangle(),
                                    IFrameWindow::titleBar      |
                                    IFrameWindow::systemMenu    |
                                    IFrameWindow::border        |
                                    IFrameWindow::noMoveWithOwner ),
    fToolBarFrameData( new IToolBarFrameData( this ))
{
  IToolBarTitle::offSet = IQUERYSYSVALUE( SV_CYTITLEBAR ) - titleBarHeight;

  // Set up bitmaps for move with owner button
  fToolBarFrameData->moveButton.setBitmap(IApplication::current().
                    resourceLibrary().tryToLoadBitmap(IC_ID_NOOWNERMOVE));
  fToolBarFrameData->moveButton.setLatchedBitmap(IApplication::current().
                    resourceLibrary().tryToLoadBitmap(IC_ID_OWNERMOVE));

  // Make the system menu bitmap owner draw &
  // Remove all the unnecessary menu options.

  fToolBarFrameData->systemMenu.setItem(
                fToolBarFrameData->systemMenu.menuItem(SC_SYSMENU).setDrawItem())
                              .deleteItem(ISystemMenu::idRestore)
                              .deleteItem(ISystemMenu::idMinimize)
                              .deleteItem(ISystemMenu::idMaximize)
                              .deleteItem(ISystemMenu::idHide);

  fToolBarFrameData->showHideButton.setUserData( false );

  fToolBarFrameData->showHideButton.setBitmap(IApplication::current()
                                   .resourceLibrary().tryToLoadBitmap(IC_ID_HIDE));

//.setLatchedColor( fToolBarFrameData->showHideButton.backgroundColor(), false );

  fToolBarFrameData->titleFrameHandler.handleEventsFor( this );
  fToolBarFrameData->systemMenuDrawHandler.handleEventsFor(&fToolBarFrameData->systemMenu);
  fToolBarFrameData->titleBarCommandHandler.handleEventsFor( this );

  addExtension( &fToolBarFrameData->showHideButton,
                IFrameWindow::rightOfTitleBar, titleBarHeight );
  addExtension( &fToolBarFrameData->moveButton,
                IFrameWindow::rightOfTitleBar, titleBarHeight );

}

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::~IToolBarFrameWindow                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameWindow::~IToolBarFrameWindow( )
{
  fToolBarFrameData->titleFrameHandler.stopHandlingEventsFor( this );
  fToolBarFrameData->systemMenuDrawHandler.stopHandlingEventsFor(&fToolBarFrameData->systemMenu);
  fToolBarFrameData->titleBarCommandHandler.stopHandlingEventsFor( this );
  delete fToolBarFrameData;
}

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::setTitleText                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameWindow& IToolBarFrameWindow::setTitleText( const char* text )
{
  fToolBarFrameData->toolBarTitle.setText( text );
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::setTitleText                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarFrameWindow& IToolBarFrameWindow::setTitleText( const IResourceId& text)
{
  fToolBarFrameData->toolBarTitle.setText( text );
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarFrameWindow::titleText                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IToolBarFrameWindow::titleText( ) const
{
  return fToolBarFrameData->toolBarTitle.text();
}

/*------------------------------------------------------------------------------
| ITitleBarFrameHandler::ITitleBarFrameHandler                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitleBarFrameHandler::ITitleBarFrameHandler()
 : ITitleBarFrameHandler::Inherited()
{}

/*------------------------------------------------------------------------------
| ITitleBarFrameHandler::~ITitleBarFrameHandler                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitleBarFrameHandler::~ITitleBarFrameHandler()
{}

/*------------------------------------------------------------------------------
| ITitleBarFrameHandler::format                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ITitleBarFrameHandler::format( IFrameFormatEvent &formatEvent )
{
  RECTL
    temp = { 0, 0 };

  IFrameFormatEvent
    localEvent = formatEvent;
  if ( formatEvent.parameter2() == 0 )
  {
    localEvent = IEvent( formatEvent.handle(),
                         formatEvent.eventId(),
                         formatEvent.parameter1(),
                         &temp );
  }

  // Get count of standard controls (and thus index of extensions)...
  unsigned long
    controls(this -> defaultProcedure( localEvent ));

  ISWPArray
   swpArray( formatEvent.swpArray(),
             controls + this->numberOfExtensions( formatEvent.frame() ) );

  ISWP &titleSWP     (swpArray[ swpArray.indexOf( FID_TITLEBAR )]);
  ISWP &clientSWP    (swpArray[ swpArray.indexOf( FID_CLIENT )]);
  ISWP &systemMenuSWP(swpArray[ swpArray.indexOf( FID_SYSMENU )]);

  // make the titlebar wider and smaller in height
  titleSWP.setSize( ISize( titleSWP.size().width() + IToolBarTitle::offSet,
                           titleSWP.size().height() - IToolBarTitle::offSet));

  // move the title bar up and over
  titleSWP.setPosition( IPoint( titleSWP.position().x() - IToolBarTitle::offSet,
                                titleSWP.position().y() + IToolBarTitle::offSet));

  // make the system menu smaller in both x and y direction
  systemMenuSWP.setSize( ISize( systemMenuSWP.size().width() - IToolBarTitle::offSet,
                                systemMenuSWP.size().height() - IToolBarTitle::offSet));
  // move the system menu up
  systemMenuSWP.setPosition( IPoint( systemMenuSWP.position().x(),
                                     systemMenuSWP.position().y() + IToolBarTitle::offSet));

  // make the client taller
  clientSWP.setSize( ISize( clientSWP.size().width(),
                            clientSWP.size().height() + IToolBarTitle::offSet));

  unsigned long widthBeforePositioningCtrls(titleSWP.size().width());

  // position the remaining extensions
  formatEvent.setResult( this->positionExtensions( localEvent, controls ));

  // Remove the two buttons to the left of the titlebar if there
  // is insufficient room to display them.
  if (widthBeforePositioningCtrls < 2 * titleBarHeight)
  {
    ISWP &moveButton    (swpArray[controls]);
    ISWP &showHideButton(swpArray[controls+1]);
    moveButton.setSize(ISize(0,0));
    showHideButton.setSize(ISize(0,0));
    titleSWP.setSize( ISize(widthBeforePositioningCtrls, titleSWP.size().height()));
  } /* endif */
  return true;
}

/*------------------------------------------------------------------------------
| ITitlePaintHandler::ITitlePaintHandler                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitlePaintHandler::ITitlePaintHandler()
 : ITitlePaintHandler::Inherited()
{}

/*------------------------------------------------------------------------------
| ITitlePaintHandler::~ITitlePaintHandler                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitlePaintHandler::~ITitlePaintHandler()
{}

/*------------------------------------------------------------------------------
| ITitlePaintHandler::paintWindow                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ITitlePaintHandler::paintWindow( IPaintEvent& event )
{
  IToolBarTitle* toolBarTitle((IToolBarTitle*)event.window());
  IGraphicContext gc(event.presSpaceHandle());

  this->defaultProcedure( event );
  paintTitleText( *toolBarTitle, gc );

  return true;
}

/*------------------------------------------------------------------------------
| ITitlePaintHandler::dispatchHandlerEvent                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ITitlePaintHandler::dispatchHandlerEvent( IEvent& event )
{
  // If the BIDI attributes change, repaint the title bar
  if (event.eventId() == 0xBD0)
     event.window()->postEvent(TBM_SETHILITE);

  if (event.eventId() == TBM_SETHILITE)
  {
    IToolBarTitle* toolBarTitle((IToolBarTitle*)event.window());
    IGraphicContext gc(event.window()->handle());
    this->defaultProcedure( event );

    // Get the title bar rectangle and set it as the
    // clip region to ensure the title text string does
    // not paint outside the title bar window.
    IRectangle toolBarTitleRect(toolBarTitle->rect());
    toolBarTitleRect.moveTo( IPoint(0,0) );

    // Set the clip region to the window rect and destroy
    // any existing clip region.
    IGRegion titleBarRegion( gc, toolBarTitleRect );

    gc.setClipRegion( titleBarRegion );

    // Paint the title text string.
    paintTitleText( *toolBarTitle, gc );

    // Clear the clip region
    gc.clearClipRegion();

    return true;
  }
  else
  {
    return ITitlePaintHandler::Inherited::dispatchHandlerEvent( event );
  }
}

/*------------------------------------------------------------------------------
| ITitlePaintHandler::paintWindow                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitlePaintHandler& ITitlePaintHandler::paintTitleText( const IToolBarTitle& toolBarTitle,
                                                        IGraphicContext& gc )
{
  IGraphicBundle titleStr;

  IFont titleFont(&toolBarTitle);

  if (WinQueryActiveWindow( HWND_DESKTOP ) == toolBarTitle.owner()->handle())
  {
    titleStr.setPenColor( IGUIColor(IGUIColor::activeTitleText ))
            .setBackgroundColor( IGUIColor(IGUIColor::activeTitleTextBgnd ));
  }
  else
  {
    titleStr.setPenColor( IGUIColor(IGUIColor::inactiveTitleText ))
            .setBackgroundColor( IGUIColor(IGUIColor::inactiveTitleTextBgnd ));
  }
  titleStr.setBackgroundMixMode( IGraphicBundle::backOverPaint );

  // Assume we should start the text from the left side but check for
  // right-to-left layout
  IPoint pt(titleFont.avgCharWidth(), titleFont.maxLowercaseDescender()+1);
  if ( IBidiSettings::isBidiSupported() )
  {
    IBidiSettings bidi(toolBarTitle);
    if ( bidi.windowLayout() == IBidiSettings::layoutRightToLeft )
    {
      unsigned long txtWidth = titleFont.textWidth(toolBarTitle.text(),bidi);
      pt.setX(toolBarTitle.size().width() - txtWidth - pt.x());
    }
  }
  IGString titleText(toolBarTitle.text(), pt);

  titleText.setGraphicBundle( titleStr );
  titleText.drawOn(gc);
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarTitle::IToolBarTitle                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarTitle::IToolBarTitle( IWindow* owner, char* titleText )
                            : IToolBarTitle::Inherited( owner, "" ),
                              paintHandler(),
                              title(titleText)
{
  this->Inherited::setText("");
  paintHandler.handleEventsFor( this );

  // Set the font to Helv.8
  // Note: DBCS OS/2 systems should use the default font instead of Helv
  //       since the double-byte characters do not exist fot this font

  unsigned long codepage = WinQueryCp(HMQ_CURRENT);
  switch (codepage)
  {
    // Note: The following list represents the DBCS code pages supported
    //       by OS/2 2.1 and Warp as of June 1995

    case  949: // Korean                      (replaces older 934/944)
    case  944: // Korean
    case  934: // Korean
    case 1381: // People's Republic of China  (replaces older 936/946)
    case  936: // People's Republic of China
    case  946: // People's Republic of China
    case  950: // Taiwan                      (replaces older 938/948)
    case  938: // Taiwan
    case  948: // Taiwan
    case  932: // Japanese
    case  942: // Japanese
    {
      IFont fnt(this);
      fnt.setPointSize(8);
      fnt.setWindowFont(this);
    }
    break;
    default:
    {
      IFont fnt("Helv",8);
      fnt.setWindowFont(this);
    }
    break;
  }

}

/*------------------------------------------------------------------------------
| IToolBarTitle::~IToolBarTitle                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarTitle::~IToolBarTitle( )
{
  paintHandler.stopHandlingEventsFor( this );
}

/*------------------------------------------------------------------------------
| IToolBarTitle::setText                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarTitle& IToolBarTitle::setText( const char* text )
{
  title = text;
  refresh();
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarTitle::setText                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IToolBarTitle& IToolBarTitle::setText( const IResourceId& residText )
{
  title = residText.resourceLibrary().loadString(residText);
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarTitle::text                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IToolBarTitle::text( ) const
{
  return title;
}

/*------------------------------------------------------------------------------
| ISystemMenuDrawHandler::ISystemMenuDrawHandler                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ISystemMenuDrawHandler::ISystemMenuDrawHandler() :
  ISystemMenuDrawHandler::Inherited()
{}

/*------------------------------------------------------------------------------
| ISystemMenuDrawHandler::~ISystemMenuDrawHandler                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ISystemMenuDrawHandler::~ISystemMenuDrawHandler()
{}

/*------------------------------------------------------------------------------
| ISystemMenuDrawHandler::draw                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ISystemMenuDrawHandler::draw(IMenuDrawItemEvent& evt, DrawFlag& flag)
{
  IBitmapHandle bmp = IApplication::current().resourceLibrary()
                      .tryToLoadBitmap ( IC_SMALLSYSTEMMENU );
  if ( bmp )
  {
    IGraphicContext gc(evt.itemPresSpaceHandle());
    IGBitmap systemMenu( bmp );
    systemMenu.drawOn( gc );
    return true;
  }
  return false;
}

/*------------------------------------------------------------------------------
| ISystemMenuDrawHandler::setSize                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ISystemMenuDrawHandler::setSize(IMenuDrawItemEvent& evt, ISize& newSize)
{
  newSize.setWidth(IQUERYSYSVALUE( SV_CYTITLEBAR )).setHeight( titleBarHeight );
  return true;
}


/*------------------------------------------------------------------------------
| ITitleBarCommandHandler::ITitleBarCommandHandler                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitleBarCommandHandler::ITitleBarCommandHandler() {}

/*------------------------------------------------------------------------------
| ITitleBarCommandHandler::~ITitleBarCommandHandler                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ITitleBarCommandHandler::~ITitleBarCommandHandler() {}

/*------------------------------------------------------------------------------
| ITitleBarCommandHandler::command                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ITitleBarCommandHandler::command( ICommandEvent& event )
{
  switch (event.commandId())
  {
    case IC_ID_SHOWHIDE:
    {
      IToolBar* toolBar(((IToolBar*)((IToolBarFrameWindow*)event.window())->client()));
      IToolBarButton *showHideButton((IToolBarButton*)
                     IWindow::windowWithOwner( IC_ID_SHOWHIDE, event.window()));
      if (showHideButton->userData())
      {
        showHideButton->setBitmap( IC_ID_HIDE );
        showHideButton->setUserData( false );
        toolBar->expand( );
        // show the tool bar
      }
      else
      {
        showHideButton->setBitmap( IC_ID_SHOW );
        showHideButton->setUserData( true );
        toolBar->collapse( );
        // hide the tool bar
      }
    }
    break;
    case IC_ID_OWNERMOVE:
    {
      IToolBarFrameWindow *toolBarFrame((IToolBarFrameWindow*)event.window());
      IToolBarButton *moveButton((IToolBarButton*)
                   IWindow::windowWithOwner( IC_ID_OWNERMOVE, event.window()));

      unsigned long tbfStyle(IQUERYWINDOWULONG(toolBarFrame->handle(), QWL_STYLE));
      if (moveButton->isLatched())
      {
        ISETWINDOWULONG(toolBarFrame->handle(), QWL_STYLE,
                        (tbfStyle & ~FS_NOMOVEWITHOWNER ));
      }
      else
      {
        ISETWINDOWULONG(toolBarFrame->handle(), QWL_STYLE,
                        (tbfStyle | FS_NOMOVEWITHOWNER ));
        // move with owner
      }
    }
    break;
    default:
    {
      // Route the command event to the frame which owns the toolbar frame,
      // if there is one.
      IToolBarFrameWindow *toolBarFrame((IToolBarFrameWindow*)event.window());
      IWindow             *owner (toolBarFrame ? toolBarFrame->owner() : 0);
      if (owner)
         owner->sendEvent( event );
    }
    break;
  } /* endswitch */
  return false;
}

/*------------------------------------------------------------------------------
| ITitleBarCommandHandler::dispatchHandlerEvent                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean ITitleBarCommandHandler::dispatchHandlerEvent( IEvent& event )
{
  if ( event.eventId() == IC_UM_TOOLBAR &&
       event.parameter1() == IC_UM_TBAR_SETEXPAND_BTN )
  {
    IToolBarButton *showHideButton((IToolBarButton*)
                   IWindow::windowWithOwner( IC_ID_SHOWHIDE, event.window()));
    if (event.parameter2() == true)
    {
      showHideButton->setUserData( false );
      showHideButton->setBitmap( IC_ID_HIDE );
    }
    else
    {
      showHideButton->setUserData( true );
      showHideButton->setBitmap( IC_ID_SHOW );
    }
    return false;
  }
  else
  {
    return Inherited::dispatchHandlerEvent( event );
  }
}
