/*******************************************************************************
* FILE NAME: ivport.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ivport.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512  (from iwindow.cpp)
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WININPUT                // WinQueryFocus for iwindefs.h
  #include <iwindefs.h>
}

#ifndef _IVPORT_
  #include <ivport.hpp>
#endif

#include <icconst.h>
#include <icoordsy.hpp>
#include <iexcept.hpp>
#include <inotifev.hpp>
#include <irecohdr.hpp>
#include <iscrlevt.hpp>
#include <iscroll.hpp>
#include <itrace.hpp>
#include <ivporhdr.hpp>
#include <ivrect.hpp>
#include <iwposbuf.hpp>

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IVPORT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public view port styles.                                                     |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const IViewPort::Style
  IViewPort::asNeededHorizontalScrollBar ( 0, ICNV_ASNEEDEDHORZSB ),
  IViewPort::alwaysHorizontalScrollBar   ( 0, ICNV_ALWAYSHORZSB ),
  IViewPort::noHorizontalScrollBar       ( 0, ICNV_NOHORZSB ),
  IViewPort::asNeededVerticalScrollBar   ( 0, ICNV_ASNEEDEDVERTSB ),
  IViewPort::alwaysVerticalScrollBar     ( 0, ICNV_ALWAYSVERTSB ),
  IViewPort::noVerticalScrollBar         ( 0, ICNV_NOVERTSB ),
  IViewPort::noViewWindowFill            ( 0, ICNV_NOVIEWWINFILL ),
  IViewPort::expandableViewWindow        ( 0, ICNV_EXPANDVIEWWIN ),
  IViewPort::classDefaultStyle           ( WS_VISIBLE, ICNV_ASNEEDEDHORZSB |
                                                       ICNV_ASNEEDEDVERTSB );
#pragma data_seg()


/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
IViewPort::Style
           IViewPort::currentDefaultStyle ( WS_VISIBLE, ICNV_ASNEEDEDHORZSB |
                                                        ICNV_ASNEEDEDVERTSB );

/*------------------------------------------------------------------------------
| ViewPortStatics                                                              |
|                                                                              |
| Manage static handlers.                                                      |
------------------------------------------------------------------------------*/
static IViewPortStatics ViewPortStatics;
#pragma data_seg()

/*------------------------------------------------------------------------------
| Class:   IViewPortData                                                       |
|                                                                              |
| Store private data used by the class.                                        |
------------------------------------------------------------------------------*/
class IViewPortData : public IBase
{
public:
  IViewPortData  ( );
  ~IViewPortData ( );
IBidiSettings::BidiLayout
  bidiLayout;
unsigned long
  rightJustifyIndex;
unsigned long
  insideLayout;
};

IViewPortData::IViewPortData ( )
  : bidiLayout( IBidiSettings::layoutLeftToRight ),
    rightJustifyIndex( 0 ),
    insideLayout( 0 )
{ }

IViewPortData::~IViewPortData ( )
{ }


/*------------------------------------------------------------------------------
| IViewRectangle::IViewRectangle                                               |
|                                                                              |
| Constructor here for page tuning.                                            |
------------------------------------------------------------------------------*/
IViewRectangle::IViewRectangle ( unsigned long id,
                                 IWindow* parent, IWindow* owner,
                                 const IRectangle& initial,
                                 const Inherited::Style& style )
  : IViewRectangle::Inherited( id, parent, owner, initial, style )
{ }

/*------------------------------------------------------------------------------
| IViewRectangle::~IViewRectangle                                              |
|                                                                              |
| Destructor here for page tuning.                                             |
------------------------------------------------------------------------------*/
IViewRectangle::~IViewRectangle ( )
{ }

/*------------------------------------------------------------------------------
| IViewRectangle::setLayoutDistorted                                           |
------------------------------------------------------------------------------*/
IViewRectangle& IViewRectangle::setLayoutDistorted (
                                            unsigned long layoutAttributeOn,
                                            unsigned long layoutAttributeOff )
{
  unsigned long ulFlagsOn = (layoutAttributeOn &
                               (unsigned long)
                                  (~(IWindow::childMinimumSizeChanged |
                                     IWindow::childWindowCreated |
                                     IWindow::childWindowDestroyed)));

  // Notify view port if view window's minimum size is changing.
  if ( layoutAttributeOn & IWindow::childMinimumSizeChanged )
  {                               // Change to child window size.
     ulFlagsOn |= IWindow::minimumSizeChanged;
  }                               // Reset flag to pass up to view port.

  // Notify view port if new or destroyed view window.
  if ( layoutAttributeOn & IWindow::childWindowCreated )
  {                               // New view window.
     ulFlagsOn |= IWindow::windowCreated;
  }                               // Reset flag to pass up to view port.
  if ( layoutAttributeOn & IWindow::childWindowDestroyed )
  {                               // Removed view window.
     ulFlagsOn |= IWindow::windowDestroyed;
  }                               // Reset flag to pass up to view port.

  Inherited::setLayoutDistorted( ulFlagsOn, layoutAttributeOff );

  return *this;
}


#define MAX_SCROLL_LIMIT          0x8000


/*------------------------------------------------------------------------------
| IViewPort::IViewPort                                                         |
|                                                                              |
| Constructor for IViewPort.                                                   |
------------------------------------------------------------------------------*/
IViewPort::IViewPort ( unsigned long windowId,
                       IWindow* parent, IWindow* owner,
                       const IRectangle& initial, const Style& style )
//: IViewPort::Inherited( windowId, parent, owner, initial ),   $HT!d
  : IViewPort::Inherited( ),                                 // $HT!a
    ptClTopLeftView( 0, 0 ),
    psbClVert( 0 ), psbClHorz( 0 ), pcvClViewRect( 0 ),
    sizClViewWindow( -1, -1 ),
    ulClFlag( 0 ),
    fViewPortData( 0 )
{
  IMODTRACE_DEVELOP( "IViewPort::ctor" );

  // Check for valid style combination.
  if ( ((style & asNeededHorizontalScrollBar)  &&
        (style & alwaysHorizontalScrollBar))  ||
       ((style & asNeededHorizontalScrollBar)  &&
        (style & noHorizontalScrollBar))  ||
       ((style & alwaysHorizontalScrollBar)  &&
        (style & noHorizontalScrollBar))  ||
       ((style & asNeededVerticalScrollBar)  &&
        (style & alwaysVerticalScrollBar))  ||
       ((style & asNeededVerticalScrollBar)  &&
        (style & noVerticalScrollBar))  ||
       ((style & alwaysVerticalScrollBar)  &&
        (style & noVerticalScrollBar)) )
  {                               // Mutually exclusive styles.
     ITHROWLIBRARYERROR( IC_INVALIDSTYLE,
                         IErrorInfo::invalidParameter,
                         IException::recoverable );
  } /* endif */

  // Save the extended style to make sure we have a copy of it stored.
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

  // Process styles.                                    $HT!d
//this->convertToGUIStyle( style );                     $HT!d

  // Remove the default recoordination handler set in the public
  // ICanvas ctor, since ISetCanvas positions its child windows.
  IRecoordHandler::defaultHandler()->stopHandlingEventsFor( this );

  // Create the presentation system window.             $HT!a
  this->initialize( windowId,                        // $HT!a
                    parent,                          // $HT!a
                    owner,                           // $HT!a
                    initial,                         // $HT!a
                    this->convertToGUIStyle( style ),   // $HT!a
                    this->convertToGUIStyle( style, true ) ); // $HT!a

  /*****************************************************************/
  /* Create the "standard" child windows, omitting the visible     */
  /* style and size/position for now.                              */
  /*****************************************************************/
  psbClHorz =
    new IScrollBar( IC_VIEWPORT_HORZSCROLLBAR,
                    this, this,
                    IRectangle( 0, 0,
                                0, IScrollBar::systemScrollBarWidth(false) ),
                    IScrollBar::horizontal | IControl::group );
  psbClVert =
    new IScrollBar( IC_VIEWPORT_VERTSCROLLBAR,
                    this, this,
                    IRectangle( 0, 0,
                                IScrollBar::systemScrollBarWidth( true ), 0 ),
                    IScrollBar::vertical | IControl::group );
  pcvClViewRect =
    new IViewRectangle( IC_VIEWPORT_VIEWRECTANGLE,
                        this, this, IRectangle(), IWindow::visible );
  pcvClViewRect->positionOnSiblings();  // For proper input focus.

  /*****************************************************************/
  /* If bidirectional support is set,                              */
  /* Create private data class and initialize BIDI setting         */
  /* and be sure that our vertical                                 */
  /* scroll bar works left-to-right.  This is needed due to the    */
  /* logic for scrolling the view window.                          */
  /*****************************************************************/
  if ( IBidiSettings::isBidiSupported() )
  {
     fViewPortData = new IViewPortData;
     fViewPortData->bidiLayout = IBidiSettings(*this).windowLayout();
     IBidiSettings settings( *psbClHorz );
     settings.setWindowLayout( IBidiSettings::layoutLeftToRight );
     settings.apply(*psbClHorz);
  }

  /*****************************************************************/
  /* Set the scroll bar increments based off of the current font   */
  /* (use half of a character).                                    */
  /*****************************************************************/
  ISize sizLineScroll = characterSize() / 2;
  psbClHorz->setMinScrollIncrement( sizLineScroll.width() );
  psbClVert->setMinScrollIncrement( sizLineScroll.height() );

  /*****************************************************************/
  /* Add the private resize, scroll, and keyboard handlers.        */
  /*****************************************************************/
  ViewPortStatics.resizeHandler().handleEventsFor( this );
  ulClFlag |= IViewPort::resizeHandlerAdded;

  ViewPortStatics.scrollHandler().handleEventsFor( this );
  ulClFlag |= IViewPort::scrollHandlerAdded;

  ViewPortStatics.keyboardHandler().handleEventsFor( this );
  ulClFlag |= IViewPort::keyboardHandlerAdded;

  if ( style & noViewWindowFill )
  {                               // Add special paint handlers.
     ViewPortStatics.paintHandler().handleEventsFor( this );
     ulClFlag |= IViewPort::paintHandlerAdded;

     ViewPortStatics
      .rectanglePaintHandler()
      .handleEventsFor( pcvClViewRect );
     ulClFlag |= IViewPort::rectanglePaintHandlerAdded;
  }

  /************************************************************/
  /* The view window is to be sized to fill the view port,    */
  /* but no smaller than its minimum size.                    */
  /************************************************************/
  if ( this->extendedStyle() &
         IViewPort::expandableViewWindow.asExtendedUnsignedLong() )
  {           // Flag to size the view window based on its min size.
     ulClFlag |= IViewPort::useMinimumSizeView;
  }

  ulClFlag |= IViewPort::initialized;
}

/*------------------------------------------------------------------------------
| IViewPort::~IViewPort                                                        |
|                                                                              |
| Destructor for clean-up.                                                     |
------------------------------------------------------------------------------*/
IViewPort::~IViewPort ( )
{
  IMODTRACE_DEVELOP( "IViewPort::dtor" );

  if ( fViewPortData )
  {
     delete fViewPortData;
  }
  if ( psbClVert )
  {
     delete psbClVert;
  }
  if ( psbClHorz )
  {
     delete psbClHorz;
  }
  if ( pcvClViewRect )
  {
     if ( ulClFlag & IViewPort::rectanglePaintHandlerAdded )
     {                            // Remove the private paint handler.
        ViewPortStatics
         .rectanglePaintHandler()
         .stopHandlingEventsFor( pcvClViewRect );
        ulClFlag &= (unsigned long)(~IViewPort::rectanglePaintHandlerAdded);
     }
     delete pcvClViewRect;
  }

  if ( ulClFlag & IViewPort::resizeHandlerAdded )
  {                              // Remove the private resize handler.
     ViewPortStatics.resizeHandler().stopHandlingEventsFor( this );
     ulClFlag &= (unsigned long)(~IViewPort::resizeHandlerAdded);
  }

  if ( ulClFlag & IViewPort::scrollHandlerAdded )
  {                              // Remove the private scroll handler.
     ViewPortStatics.scrollHandler().stopHandlingEventsFor( this );
     ulClFlag &= (unsigned long)(~IViewPort::scrollHandlerAdded);
  }

  if ( ulClFlag & IViewPort::keyboardHandlerAdded )
  {                            // Remove the private keyboard handler.
     ViewPortStatics.keyboardHandler().stopHandlingEventsFor( this );
     ulClFlag &= (unsigned long)(~IViewPort::keyboardHandlerAdded);
  }

  if ( ulClFlag & IViewPort::paintHandlerAdded )
  {                            // Remove the private paint handler.
     ViewPortStatics.paintHandler().stopHandlingEventsFor( this );
     ulClFlag &= (unsigned long)(~IViewPort::paintHandlerAdded);
  }
}

/*------------------------------------------------------------------------------
| IViewPort::defaultStyle                                                      |
------------------------------------------------------------------------------*/
IViewPort::Style IViewPort::defaultStyle ( )
{
  return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IViewPort::convertToGUIStyle                                                 |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IViewPort::convertToGUIStyle ( const IBitFlag& guiStyle,
                                             Boolean bExtOnly ) const
{
  // Obtain the style from the class (ICanvas) that we inherit from.
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if ( bExtOnly )
  {
     // Use mask to only return extended styles in the user defined range.
     ulStyle |= extendedStyle() & IS_EXTMASK;
  }
  else
  {
     // No styles are defined for the base canvas, so we do not mask.
     ulStyle |= guiStyle.asUnsignedLong();

     // No additional processing required since ICanvas::convertToGUIStyle is
     // doing all the required work ...
  }

  return ulStyle;
}

/*------------------------------------------------------------------------------
| IViewPort::viewWindow                                                        |
|                                                                              |
| Enumerate child windows to find the view window (it's the one that isn't one |
| of the scroll bars or the canvas view rectangle).  This function will throw  |
| an exception if more than one is found.                                      |
------------------------------------------------------------------------------*/
IWindowHandle IViewPort::viewWindow ( )
{
  Boolean bChangeParent = false;
  IWindowHandle hwndView = 0;
  IWindowHandle hwndChild = 0;

  /*****************************************************************/
  /* First enumerate all child windows of the view port itself.    */
  /* If this function has been called before, the view window has  */
  /* already been moved to be a child of the view area canvas.     */
  /*****************************************************************/
  Boolean bDone = false;
  IWindow::ChildCursor cursor( *this );
  for ( cursor.setToFirst();
        bDone == false  &&  cursor.isValid();
        cursor.setToNext() )
  {                               // Enumerate all child controls.
     hwndChild = this->childAt( cursor );
     if ( hwndChild != psbClHorz->handle()  &&
          hwndChild != psbClVert->handle()  &&
          hwndChild != pcvClViewRect->handle() )
     {                            // Not a standard child.
        if ( hwndView )
        {                         // Already found a view window.
           ITHROWLIBRARYERROR( IC_VIEWPORT_INVALID_SCROLL,
                               IErrorInfo::invalidRequest,
                               IException::recoverable );
        }
        else
        {
           hwndView = hwndChild;
           bChangeParent = true;
        }
     } /* endif */
  }

  /*****************************************************************/
  /* Now enumerate all child windows of the canvas view area.      */
  /* If this function has been called before, we've changed the    */
  /* parent of the view window to the canvas.                      */
  /*****************************************************************/
  IWindow::ChildCursor viewRectCursor( *pcvClViewRect );
  bDone = false;
  for ( viewRectCursor.setToFirst();
        bDone == false  &&  viewRectCursor.isValid();
        viewRectCursor.setToNext() )
  {                               // Enumerate all child controls.
     hwndChild = pcvClViewRect->childAt( viewRectCursor );
     if ( hwndView )
     {                            // Already found a view window.
        ITHROWLIBRARYERROR( IC_VIEWPORT_INVALID_SCROLL,
                            IErrorInfo::invalidRequest,
                            IException::recoverable );
     }
     else
     {
        hwndView = hwndChild;
     }
  }

  /*****************************************************************/
  /* Now change the parent of the view window to the canvas, if    */
  /* this hasn't already been done.                                */
  /*****************************************************************/
  if ( bChangeParent )
  {                      // Need view window to be child of view area.
     Boolean bNew = false;
     IWindow* wndView = IWindow::windowWithHandle( hwndView );
     if ( ! wndView )
     {                            // Not in window list.
        wndView = new IWindow( hwndView );  // Wrapper it now.
        bNew = true;
     }

     wndView->setParent( pcvClViewRect );
     if ( pcvClViewRect->handle() ==
            IQUERYFOCUS( IWindow::desktopWindow()->handle() ))
     {                            // View rectangle has focus.
        wndView->setFocus();      // Shift focus to view window.
     }

     if ( bNew )
     {                            // Memory clean-up.
        delete wndView;
     }
  }

  return hwndView;
}

/*------------------------------------------------------------------------------
| IViewPort::scrollViewVerticallyTo                                            |
|                                                                              |
| Scroll the view window by moving it relative to the view area.  The          |
| specified zero-based value is used as the offset of the top-most visible row |
| from the top edge of the view window.                                        |
------------------------------------------------------------------------------*/
IViewPort& IViewPort::scrollViewVerticallyTo ( unsigned long ulScrollTo )
{
  unsigned long ulViewWindowHeight = viewWindowSize().height();
  unsigned long ulViewRectHeight = pcvClViewRect->size().height();
  if ( ulViewWindowHeight > pcvClViewRect->size().height() )
  {                         // View window bigger than view rectangle.
     /**************************************************************/
     /* Note that some function is duplicated between here and     */
     /* IViewPortHandler, since the user could call this directly. */
     /**************************************************************/
     unsigned long ulMaxScrollTo =
                      ulViewWindowHeight - ulViewRectHeight;
     unsigned long ul0BasedScrollTo =
                      (ulScrollTo > ulMaxScrollTo) ?
                                 ulMaxScrollTo : ulScrollTo;
                        // Adjust scroll request if scrolling too far.
     ptClTopLeftView.setY( ul0BasedScrollTo );
     IWindowHandle hwndView = viewWindow();

     if ( hwndView )
     {                  // Scroll the view window.
        this->positionViewWindow( hwndView, pcvClViewRect->rect() );
     }

     // Adjust for huge logical height.
     if ( ulViewWindowHeight >= MAX_SCROLL_LIMIT )
     {             // Scroll bar units are ratio of view port units.
        unsigned long yRatio =
                        (ulViewWindowHeight / MAX_SCROLL_LIMIT) + 1;
        unsigned long ulSave = ul0BasedScrollTo;
        ul0BasedScrollTo /= yRatio;
        if ( ulSave != 0  &&  ul0BasedScrollTo == 0 )
        {               // Adjust for rounding to zero.
           ul0BasedScrollTo++;
        }
        else if ( ulSave != (ulViewWindowHeight - ulViewRectHeight)  &&
                  ul0BasedScrollTo ==
                    (ulViewWindowHeight - ulViewRectHeight) / yRatio )
        {               // Adjust for rounding to max.
           ul0BasedScrollTo--;
        }
     }
     psbClVert->moveScrollBoxTo( ul0BasedScrollTo + 1 );
                                  // Adjust scroll bar thumb
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IViewPort::scrollViewHorizontallyTo                                          |
|                                                                              |
| Scroll the view window by moving it relative to the view area.  The          |
| specified zero-based value is used as the offset of the left-most visible    |
| column from the left edge of the view window.                                |
------------------------------------------------------------------------------*/
IViewPort& IViewPort :: scrollViewHorizontallyTo ( unsigned long ulScrollTo )
{
  unsigned long ulViewWindowWidth = viewWindowSize().width();
  unsigned long ulViewRectWidth = pcvClViewRect->size().width();
  if ( ulViewWindowWidth > pcvClViewRect->size().width() )
  {                         // View window bigger than view rectangle.
     /**************************************************************/
     /* Note that some function is duplicated between here and     */
     /* IViewPortHandler, since the user could call this directly. */
     /**************************************************************/
     unsigned long ulMaxScrollTo =
                        ulViewWindowWidth - ulViewRectWidth;
     unsigned long ul0BasedScrollTo =
                      (ulScrollTo > ulMaxScrollTo) ?
                                 ulMaxScrollTo : ulScrollTo;
                        // Adjust scroll request if scrolling too far.
     ptClTopLeftView.setX( ul0BasedScrollTo );
     IWindowHandle hwndView = viewWindow();

     if ( hwndView )
     {                  // Scroll the view window.
        this->positionViewWindow( hwndView, pcvClViewRect->rect() );
     }

     // Adjust for huge logical width.
     if ( ulViewWindowWidth >= MAX_SCROLL_LIMIT )
     {             // Scroll bar units are ratio of view port units.
        unsigned long xRatio =
                        (ulViewWindowWidth / MAX_SCROLL_LIMIT) + 1;
        unsigned long ulSave = ul0BasedScrollTo;
        ul0BasedScrollTo /= xRatio;
        if ( ulSave != 0  &&  ul0BasedScrollTo == 0 )
        {               // Adjust for rounding to zero.
           ul0BasedScrollTo++;
        }
        else if ( ulSave != (ulViewWindowWidth - ulViewRectWidth)  &&
                  ul0BasedScrollTo ==
                    (ulViewWindowWidth - ulViewRectWidth) / xRatio )
        {               // Adjust for rounding to max.
           ul0BasedScrollTo--;
        }
     }
     psbClHorz->moveScrollBoxTo( ul0BasedScrollTo + 1 );
                        // Adjust scroll bar thumb.
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IViewPort::setViewWindowSize                                                 |
|                                                                              |
| Set the logical size of the view window.                                     |
------------------------------------------------------------------------------*/
IViewPort& IViewPort :: setViewWindowSize ( const ISize& siz )
{
  if ( sizClViewWindow != siz )
  {
     sizClViewWindow = siz;
     this->setLayoutSize( siz );
     ulClFlag &= (unsigned long)(~IViewPort::useMinimumSizeView);
     this->setLayoutDistorted( IWindow::layoutChanged |
                               IWindow::immediateUpdate, 0 );
     ISize vWinSize = siz;
     notifyObservers( INotificationEvent( IViewPort::viewWindowSizeId,
                                          *this, true, &vWinSize ));
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IViewPort::viewWindowSize                                                    |
|                                                                              |
| Return the logical/actual size of the view window.                           |
------------------------------------------------------------------------------*/
ISize IViewPort :: viewWindowSize ( ) const
{
  ISize siz = sizClViewWindow;
  if ( siz == ISize( -1, -1 ))
  {                     // No logical size used.
     ((IViewPort*)this)->layout();   // Cast away const.
     siz = this->layoutSize();
  }
  return siz;
}

/*------------------------------------------------------------------------------
| IViewPort::viewWindowDrawRectangle                                           |
|                                                                              |
| Return the rectangle surrounding the visible portion of the view window.     |
------------------------------------------------------------------------------*/
IRectangle IViewPort :: viewWindowDrawRectangle ( ) const
{
   if ( ICoordinateSystem::applicationOrientation() ==
                       ICoordinateSystem::originLowerLeft )
      return IRectangle( this->topLeftViewPoint() -
                           IPoint( 0, pcvClViewRect->size().height() ),
                         pcvClViewRect->size() );
   else
      return IRectangle( this->topLeftViewPoint(),
                         pcvClViewRect->size() );
}

/*------------------------------------------------------------------------------
| IViewPort::topLeftViewPoint                                                  |
|                                                                              |
| Return the view window point displayed in the top left corner of the view    |
| rectangle.                                                                   |
------------------------------------------------------------------------------*/
IPoint IViewPort :: topLeftViewPoint ( ) const
{
   return IPoint( ptClTopLeftView.x(),
                  this->layoutSize().height() - ptClTopLeftView.y() );
}

/*------------------------------------------------------------------------------
| IViewPort::setTopLeftViewPoint                                               |
|                                                                              |
| Set the view window point displayed in the top left corner of the view       |
| rectangle.                                                                   |
------------------------------------------------------------------------------*/
IViewPort& IViewPort :: setTopLeftViewPoint ( const IPoint& topLeft )
{
   ptClTopLeftView = IPoint( topLeft.x(),
                             this->layoutSize().height() - topLeft.y() );

  return *this;
}

/*------------------------------------------------------------------------------
| IViewPort::setLayoutDistorted                                                |
|                                                                              |
| Treats a size change like a layout change.                                   |
------------------------------------------------------------------------------*/
IViewPort& IViewPort :: setLayoutDistorted( unsigned long layoutAttributeOn,
                                            unsigned long layoutAttributeOff )
{
  unsigned long ulFlagsOn = (layoutAttributeOn &
                               (unsigned long)
                                  (~(IWindow::sizeChanged |
                                     IWindow::childMinimumSizeChanged)));
  if ( (layoutAttributeOn & IWindow::sizeChanged)  ||
       ((ulClFlag & IViewPort::useMinimumSizeView)  &&
        (layoutAttributeOn & IWindow::childMinimumSizeChanged)) )
  {                               // Need to update scroll bars.
     ulFlagsOn |= IWindow::layoutChanged;  // Flag for ICanvas.
     if ( !(layoutAttributeOn & IWindow::fontPropogated)  &&
          !isLayoutDistorted(IWindow::fontPropogated) )
     {                            // Change buffered til now?
        ulFlagsOn |= IWindow::immediateUpdate;
     }
  }
  Inherited::setLayoutDistorted( ulFlagsOn, layoutAttributeOff );

  return *this;
}

/*------------------------------------------------------------------------------
| IViewPort::calcMinimumSize                                                   |
|                                                                              |
| Determine the minimum size of the view port based on the minimum needed to   |
| display scroll bars.                                                         |
------------------------------------------------------------------------------*/
ISize IViewPort :: calcMinimumSize ( ) const
{
  return ( psbClHorz->minimumSize() + psbClVert->minimumSize() );
}

/*------------------------------------------------------------------------------
| IViewPort::layout                                                            |
|                                                                              |
| Position the view window and size and position the scroll bars around it in  |
| the view rectangle.                                                          |
------------------------------------------------------------------------------*/
IViewPort& IViewPort :: layout ( )
{
  if ( isLayoutDistorted( IWindow::layoutChanged )  &&
       (ulClFlag & IViewPort::initialized) )
  {                               // Something has changed.

     // Set layout flag for BIDI systems.
     if ( fViewPortData )
     {
        fViewPortData->insideLayout++;
        if ( fViewPortData->insideLayout == 1 )
        {
           fViewPortData->rightJustifyIndex =
                            this->viewWindowDrawRectangle().right();
        }
     }

     IWindowHandle hwndView = this->viewWindow();
     this->isTabStop();           // Substitute for fixupChildren().

     ISize sizView( 0, 0 );       // Assume no view window.
     if ( hwndView )
     {                            // Have a view window.
        /***********************************************************/
        /* Set the layout size (used by setupScrollBars()) equal   */
        /* to the size of the view window.  This is done here      */
        /* since fixupChildren() will include all children (i.e.   */
        /* the scroll bars) in the layout size and is sensitive to */
        /* the window position of the children, which is not what  */
        /* we want here.                                           */
        /***********************************************************/
        IWindow* pwndView = IWindow::windowWithHandle( hwndView );
        if ( pwndView )
        {                         // Have child control object.
           if ( sizClViewWindow == ISize(-1, -1) )
           {                      // No logical window size.
              sizView = pwndView->size();

              if ( (ulClFlag & IViewPort::useMinimumSizeView)  ||
                   sizView == ISize() )  // 0-sized child window.
              {                   // Assume this isn't right.
                 sizView = pwndView->minimumSize();

                 pwndView->sizeTo(sizView);  // Use min size instead.

                 ulClFlag |= IViewPort::useMinimumSizeView;
              }     // (Needed to allow child canvas windows to grow.)
              else
              {                   // Use real window size.
                 ulClFlag &= (unsigned long)(~IViewPort::useMinimumSizeView);
              } /* endif */
           }
           else
           {                      // Use logical window size.
              sizView = sizClViewWindow;
              ulClFlag &= (unsigned long)(~IViewPort::useMinimumSizeView);
           } /* endif */
        }
        else                      // A non-wrappered control.
        {                         // Need to support these also.
           IWindow* wndView = new IWindow(hwndView);  // Wrapper it.
           sizView = wndView->size();
           delete wndView;
        }
     } /* endif */

     // Check if horizontal scroll was needed before this layout change
     // (this is needed to handle BIDI requirements for right-to-left systems).
     Boolean
       hScrollPrevEnabled = psbClHorz->isEnabled(),
       hScrollBidiChange = false;

     // Check if the view port window layout changed
     if ( IBidiSettings::isBidiSupported() )
     {
        IBidiSettings settings( *this );
        if ( settings.windowLayout() != fViewPortData->bidiLayout )
        {
           hScrollBidiChange = true;
           fViewPortData->bidiLayout = settings.windowLayout();
        }
     }

     // Check if scroll bar BIDI settings have been changed to right-to-left
     // and change it back to left-to-right (so our scrolling works).
     if ( IBidiSettings::isBidiSupported() )
     {
        IBidiSettings settings( *psbClHorz );
        if ( settings.windowLayout() != IBidiSettings::layoutLeftToRight )
        {
           settings.setWindowLayout( IBidiSettings::layoutLeftToRight );
           settings.apply( *psbClHorz );
        }
     }

     this->setLayoutSize( sizView );
     this->setupScrollBars();     // Add scroll bars if necessary.

     if ( hwndView )
     {                            // Have a view window.
        positionViewWindow( hwndView, pcvClViewRect->rect() );
                                  // Position client window.
     } /* endif */

     // Check for special BIDI horizontal scroll processing.
     if ( hScrollPrevEnabled && psbClHorz->isEnabled() &&
          hScrollBidiChange && (fViewPortData->insideLayout==1))
     {
        scrollViewHorizontallyTo( sizView.width() -
                                  viewWindowDrawRectangle().width() -
                                  viewWindowDrawRectangle().left() );
     }

     // Check for bidi horizontal scroll on right-to-left system.
     if ( IBidiSettings::isBidiSupported() && psbClHorz->isEnabled() &&
          (fViewPortData->insideLayout==1) )
     {
        // Check for right to left bidi support.
        IBidiSettings settings( *this );
        if ( settings.windowLayout() == IBidiSettings::layoutRightToLeft )
        {
          if ( hScrollPrevEnabled )
          {
            // The scroll bar was already enabled, so we just need to
            // maintain the right justification (unless this was a
            // layout change due to BIDI changing in which case it has
            // already been handled above).
            if ( !hScrollBidiChange )
            {
              scrollViewHorizontallyTo( fViewPortData->rightJustifyIndex -
                                        viewWindowDrawRectangle().width() );
            }
          }
          else
          {
            // The scroll bar was not enabled, so we need to scroll
            // all the way to the right of the view window.
            scrollViewHorizontallyTo( sizView.width() + 1 -
                                      viewWindowDrawRectangle().width() );
          }
        }
     }

     if ( fViewPortData )
     {
        fViewPortData->insideLayout--;
     }

     this->setLayoutDistorted( 0, IWindow::layoutChanged );
  }   /* end isDistorted */

  return *this;
}

/*------------------------------------------------------------------------------
| IViewPort::setupScrollBars                                                   |
|                                                                              |
| Position, size, and show/hide scroll bars.  Additionally, set the size and   |
| position of the scroll boxes.                                                |
------------------------------------------------------------------------------*/
IViewPort& IViewPort :: setupScrollBars ( )
{
  Boolean
    bNeedHScroll = false,
    bNeedVScroll = false;
  unsigned long
    ulHScrollHeight = 0,
    ulVScrollWidth = 0,
    ulExtStyle = this->extendedStyle();

  ISize
    sizViewArea( this->size() ),
    viewWindowSize( this->layoutSize() ),
    viewWindowMinSize( viewWindowSize );

  /*****************************************************************/
  /* Check for special case where the view window is to be sized   */
  /* to fill the view port, unless that would shrink the view      */
  /* window below its minimum size.  In this case, leave the view  */
  /* window at its minimum size and let it be scrolled by the view */
  /* port.  This behavior allows a multicell canvas with           */
  /* expandable rows and/or columns to work well in a view port.   */
  /*****************************************************************/
  Boolean expandableViewWindow = false;
  if ( sizClViewWindow == ISize(-1, -1)  &&
       ( ulExtStyle &
           IViewPort::expandableViewWindow.asExtendedUnsignedLong() ) )
  {                // Special case for sizing the view window.
     expandableViewWindow = true;
     viewWindowMinSize = viewWindowSize;
     viewWindowSize = sizViewArea.maximum( viewWindowMinSize );
  }

  /*****************************************************************/
  /* Check if the scroll bars are needed (the size of the view     */
  /* window is bigger than the size of the view area).             */
  /*****************************************************************/
  if ( (ulExtStyle & alwaysHorizontalScrollBar.asExtendedUnsignedLong())  ||
       ((ulExtStyle & asNeededHorizontalScrollBar.asExtendedUnsignedLong())  &&
        (sizViewArea.width() < viewWindowSize.width())) )
  {                               // Need a horizontal scroll bar.
     bNeedHScroll = true;         // Flag it.
     ulHScrollHeight = psbClHorz->size().height();
     sizViewArea -= ISize( 0, ulHScrollHeight );
     if ( expandableViewWindow )
     {
        viewWindowSize = sizViewArea.maximum( viewWindowMinSize );
     }
  }

  if ( (ulExtStyle & alwaysVerticalScrollBar.asExtendedUnsignedLong())  ||
       ((ulExtStyle & asNeededVerticalScrollBar.asExtendedUnsignedLong())  &&
        (sizViewArea.height() < viewWindowSize.height())) )
  {                               // Need a vertical scroll bar.
     bNeedVScroll = true;         // Flag it.
     ulVScrollWidth = psbClVert->size().width();
     sizViewArea -= ISize( ulVScrollWidth, 0 );
     if ( expandableViewWindow )
     {
        viewWindowSize = sizViewArea.maximum( viewWindowMinSize );
     }

     if ( bNeedHScroll == false  &&
          (ulExtStyle &
             asNeededHorizontalScrollBar.asExtendedUnsignedLong())  &&
          (sizViewArea.width() < viewWindowSize.width()) )
     {                            // Need a horiz scroll bar now.
        bNeedHScroll = true;
        ulHScrollHeight = psbClHorz->size().height();
        sizViewArea -= ISize( 0, ulHScrollHeight );
        if ( expandableViewWindow )
        {
           viewWindowSize = sizViewArea.maximum( viewWindowMinSize );
        }
     }
  }

  /*****************************************************************/
  /* Size the rectangle where the child controls can be drawn (the */
  /* size of the view port window, minus the size needed by any    */
  /* scroll bars).  This is the area within which the user's       */
  /* window is scrolled.                                           */
  /*****************************************************************/
  IWindowPosBuffer wposbuf( this );  // To size/pos scroll bars.

  IRectangle rectViewArea( IPoint( 0, ulHScrollHeight ),
                           ISize( sizViewArea ) );
  wposbuf.moveSizeTo( pcvClViewRect, rectViewArea );
                                  // Position/size scroll area.
  wposbuf.show( pcvClViewRect );

  /*****************************************************************/
  /* Special case where the view window is sized to the maximum of */
  /* its minimum size or what is necessary to fill the view        */
  /* rectangle.                                                    */
  /*****************************************************************/
  if ( expandableViewWindow  &&
       viewWindowSize != this->layoutSize() )
  {
     IWindow* child = IWindow::windowWithHandle( this->viewWindow() );
     child->sizeTo( viewWindowSize );
     this->setLayoutSize( viewWindowSize );  // New size of view window.
  }

  /*****************************************************************/
  /* Size the horizontal scroll bar and set its scroll box.        */
  /*****************************************************************/
  // Adjust for huge logical width.
  unsigned long xRatio = 1;
  unsigned long cxView = viewWindowSize.width();
  if ( cxView >= MAX_SCROLL_LIMIT  &&
       sizClViewWindow != ISize( -1, -1 ))
  {                     // Too big for scroll bar to handle.
     xRatio = (cxView / MAX_SCROLL_LIMIT) + 1; // Adjusting ratio.
     cxView /= xRatio;  // New scroll bar value.
     long pageScroll = rectViewArea.width() -
                         psbClHorz->minScrollIncrement();
     if ( pageScroll < 0 )
     {                  // Don't use a negative value.
        pageScroll = psbClHorz->minScrollIncrement();
     }
     psbClHorz->setPageScrollIncrement( pageScroll );
                        // User value must be set afterwards.
  }

  psbClHorz->setScrollBar( IRange( 1, cxView ),
                           rectViewArea.width() / xRatio );

                 // Total size is range, visible window is scroll bar.
  Boolean bEnableScrollBar = true;
  if ( ptClTopLeftView.x() + rectViewArea.width() >
                                       viewWindowSize.width() )
  {                               // Adjust for smaller window.
     if ( rectViewArea.width() >= viewWindowSize.width() )
     {                            // All of window is visible.
        bEnableScrollBar = false; // Disable scroll bar.
        ptClTopLeftView.setX( 0 );
     }
     else
     {                            // Fill all of view rectangle.
        ptClTopLeftView.setX( viewWindowSize.width() -
                                  rectViewArea.width() );
     }
  }
  psbClHorz->enable( bEnableScrollBar );  // En/disable scroll bar.
  psbClHorz->moveScrollBoxTo( ptClTopLeftView.x() / xRatio );

  if ( bNeedHScroll )
  {                               // View port shows horizontal scroll bar.
     wposbuf.moveSizeTo( psbClHorz,  // Position/size horizontal scroll bar.
                         IRectangle( IPoint(),
                                     ISize( rectViewArea.width(),
                                            ulHScrollHeight )));
     wposbuf.show( psbClHorz );
  }
  else
  {                               // Hide the horizontal scroll bar.
     wposbuf.hide( psbClHorz );
  }

  /*****************************************************************/
  /* Size the vertical scroll bar and set its scroll box.          */
  /*****************************************************************/
  // Adjust for huge logical height.
  unsigned long yRatio = 1;
  unsigned long cyView = viewWindowSize.height();
  if ( cyView >= MAX_SCROLL_LIMIT  &&
       sizClViewWindow != ISize( -1, -1 ))
  {                     // Too big for scroll bar to handle.
     yRatio = (cyView / MAX_SCROLL_LIMIT) + 1; // Adjusting ratio.
     cyView /= yRatio;  // New scroll bar value.
     long pageScroll = rectViewArea.height() -
                         psbClVert->minScrollIncrement();
     if ( pageScroll < 0 )
     {                  // Don't use a negative value.
        pageScroll = psbClVert->minScrollIncrement();
     }
     psbClVert->setPageScrollIncrement( pageScroll );
                        // User value must be set afterwards.
  }

  psbClVert->setScrollBar( IRange( 1, cyView ),
                           rectViewArea.height() / yRatio );

                 // Total size is range, visible window is scroll bar.
  bEnableScrollBar = true;
  if ( ptClTopLeftView.y() + rectViewArea.height() >
                                       viewWindowSize.height() )
  {                               // Adjust for smaller window.
     if ( rectViewArea.height() >= viewWindowSize.height() )
     {                            // All of window is visible.
        bEnableScrollBar = false; // Disable scroll bar.
        ptClTopLeftView.setY( 0 );
     }
     else
     {                            // Fill all of view rectangle.
        ptClTopLeftView.setY( viewWindowSize.height() -
                                 rectViewArea.height() );
     }
  }
  psbClVert->enable( bEnableScrollBar );  // En/disable scroll bar.
  psbClVert->moveScrollBoxTo( ptClTopLeftView.y() / yRatio );

  if ( bNeedVScroll )
  {                               // View port shows vertical scroll bar.
     wposbuf.moveSizeTo( psbClVert,
                         IRectangle( IPoint( rectViewArea.width(),
                                             ulHScrollHeight ),
                                     ISize( ulVScrollWidth,
                                            rectViewArea.height() )));
     wposbuf.show( psbClVert );
  }
  else
  {                               // Hide the vertical scroll bar.
     wposbuf.hide( psbClVert );
  }
  wposbuf.apply();                // Reposition/size scroll bars.

  return *this;
}

/*------------------------------------------------------------------------------
| IViewPort::positionViewWindow                                                |
|                                                                              |
| Move and position the client view window.                                    |
------------------------------------------------------------------------------*/
IViewPort& IViewPort :: positionViewWindow ( const IWindowHandle& hwndView,
                                             const IRectangle& rectView )
{
  IMODTRACE_DEVELOP( "IViewPort::positionViewWindow()" );
  IPoint ptBottomLeft( 0 - ptClTopLeftView.x(), 0 );

  // Check for BIDI right-to-left requirements so that the view window
  // is positioned on the right if the horizonal scroll bar is disabled.
  if (( ptClTopLeftView.x() == 0 ) && IBidiSettings::isBidiSupported() &&
        !psbClHorz->isEnabled() )
  {
     IBidiSettings settings( *this );
     if ( settings.windowLayout() == IBidiSettings::layoutRightToLeft )
     {
        ptBottomLeft.setX( rectView.width() - this->layoutSize().width() );
     }
  }

  if ( ptClTopLeftView.y() == 0 )
  {                               // Top align in view area.
     ptBottomLeft.setY( rectView.height() - this->layoutSize().height() );
  }
  else
  {                               // Position according to ptClTopLeftView.
     ptBottomLeft.setY( ptClTopLeftView.y() +
                          psbClVert->visibleCount() -
                          this->layoutSize().height() );
  }

  // (Assume the application will replace this function if using a
  // logical-sized view window, since we aren't looking at the view
  // window's physical size at all.)

#ifdef IC_TRACE_ALL
  ITRACE_DEVELOP( ">> view area height = " + IString( rectView.height() ));
  ITRACE_DEVELOP( ">> visible count = "
                       + IString( psbClVert->visibleCount() ));
  ITRACE_DEVELOP( ">> scroll position = "
                       + IString( psbClVert->scrollBoxPosition() ));
  ITRACE_DEVELOP( ">> ptClTopLeftView = " + ptClTopLeftView.asString() );
  ITRACE_DEVELOP( ">> position at: " + ptBottomLeft.asString() );
#endif

  IWindowPosBuffer wposbuf( pcvClViewRect );
  wposbuf.moveTo( hwndView, ptBottomLeft );

  wposbuf.apply( false );              // Reposition/size view window.
  return *this;
}
