/*******************************************************************************
* FILE NAME: iwcname.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iwcname.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
  }

#include <iwcname.hpp>

#ifndef _IHANDLE_
  #include <ihandle.hpp>
#endif

#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif

#include <iostream.h>



// Segment definitions
#ifdef IC_PAGETUNE
  #define _IWCNAME_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IWindowClassName::IWindowClassName                                           |
|                                                                              |
| Construct the class name from argument.  char* values are converted to       |
| corresponding "#n" "class names."  The class name for a window handle is     |
| obtained via WinQueryWindowClass.                                            |
------------------------------------------------------------------------------*/
IWindowClassName :: IWindowClassName ( const char *className )
  : name( )
  {
  this -> initialize( className );
  }

IWindowClassName :: IWindowClassName ( const IWindowHandle &handle )
  : name( )
  {
  // Validate handle...
  IASSERTPARM( handle.isValid() );
  unsigned
    len = 0;
  do
    {
    this->name = IString( 0, len + 16 );
    len = IQUERYCLASSNAME( handle, this->name.length()+1, this->name );
    }
  while ( len == this->name.length() );
  this->name = (char*)(this->name);
  }

IWindowClassName :: IWindowClassName ( const IWindowClassName &className )
  : name( className.name )
  {
  }

/*------------------------------------------------------------------------------
| IWindowClassName::~IWindowClassName                                          |
------------------------------------------------------------------------------*/
IWindowClassName :: ~IWindowClassName ( )
  {
  }

/*------------------------------------------------------------------------------
| IWindowClassName::asString                                                   |
|                                                                              |
| Simply returns the name data member.                                         |
------------------------------------------------------------------------------*/
IString IWindowClassName :: asString ( ) const
  {
  return this->name;
  }

/*------------------------------------------------------------------------------
| IWindowClassName::operator const char *                                      |
|                                                                              |
| If the class name is one of the #nnnnn values, convert to corresponding      |
| WC_* value.                                                                  |
------------------------------------------------------------------------------*/
IWindowClassName :: operator const char * ( ) const
  {
  const char
   *result = this->name;
  if ( result[0] == '#' )
    {
    unsigned short
      nnnnn = 0;
    const char
     *p = result;
    while ( *(++p) )
      nnnnn = nnnnn * 10 + ( *p % 16 );
    result = (const char *)( 0xFFFF0000ul + nnnnn );
    }
  return result;
  }

/*------------------------------------------------------------------------------
| IWindowClassName::operator ==                                                |
| IWindowClassName::operator !=                                                |
|                                                                              |
| If either argument is 0xFFFFnnnn, convert both to const char * and           |
| compare, otherwise, compare names.                                           |
|                                                                              |
| For the "not equal" comparisons", call the "equal" member functions and      |
| negate the result.                                                           |
------------------------------------------------------------------------------*/
Boolean IWindowClassName :: operator == ( const char *className ) const
  {
  if ( (unsigned long)className >> 16 == 0xFFFF )
    {
    const char
     *self = *this;
    return self == className;
    }
  else
    {
    return this->name == className;
    }
  }

Boolean IWindowClassName :: operator != ( const char *className ) const
  {
  return !( *this == className );
  }

Boolean IWindowClassName :: operator == ( const IWindowClassName &className ) const
  {
  return this->name == className.name;
  }

Boolean IWindowClassName :: operator != ( const IWindowClassName &className ) const
  {
  return this->name != className.name;
  }

Boolean operator == ( const char *className1, const IWindowClassName &className2 )
  {
  // Invoke member function...
  return className2 == className1;
  }

Boolean operator != ( const char *className1, const IWindowClassName &className2 )
  {
  return !( className2 == className1 );
  }

/*------------------------------------------------------------------------------
| ostream << IWindowClassName                                                  |
|                                                                              |
| Puts the asString representation (as the default conversion to const         |
| char * won't work for WC_* class names).                                     |
------------------------------------------------------------------------------*/
ostream &operator << ( ostream &aStream, const IWindowClassName &className )
  {
  return aStream << className.asString();
  }

/*------------------------------------------------------------------------------
| IWindowClassName::initialize                                                 |
|                                                                              |
| If the argument is of the form 0xFFFFnnnn, convert to "#nnnn".               |
------------------------------------------------------------------------------*/
IWindowClassName &IWindowClassName :: initialize ( const char *className )
  {
  unsigned
    ptr = (unsigned long)className >> 16;
  /* If upper word of pointer is max (reserved value) */
  if ( ptr == 0xFFFF )
    {
    /* Then keep just the lower word by stripping off upper word      */
    ptr = (unsigned long)className & 0xFFFF;
    this->name = "#" + IString( ptr );
    }
  else
    this->name = className;
  return *this;
  }
