/*******************************************************************************
* FILE NAME: iwindow8.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions used for the    *
*   IWindow bidirectional support.                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WINCOUNTRY
#define INCL_WINHOOKS
extern "C" {
   #include <iwindefs.h>
}

#ifndef _IRESLIB_
  #include <ireslib.hpp>
#endif
#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif
#ifndef _ICCONST_
  #include <icconst.h>
#endif
#ifndef _ITBPRIV_
  #include <itbpriv.hpp>
#endif
#ifndef _IGRAFCTX_
  #include <igrafctx.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IWINDOW8_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| The following and defined as part of the OS/2 bidectional support toolkit    |
| (pmbidi.h).  These values are not currently a part of the base OS/2 toolkit  |
| but are planned to be added in the future.  In the meantime, define them     |
| here.                                                                        |
------------------------------------------------------------------------------*/
#ifndef PM_BIDI_INCLUDED
#define WM_SETBIDIATTR          0x0BD0
#define WM_QUERYBIDIATTR        0x0BD1
#define BDAM_INIT               0x80000000
#define BDAM_LEVEL              0x70000000
#define BDAM_TEXTTYPE           0x01000000
#define BDAM_TEXT_ORIENTATION   0x00030000
#define BDAM_WND_ORIENTATION    0x00100000
#define BDAM_NUMERALS           0x00003000
#define BDAM_SYM_SWAP           0x00000100
#define BDAM_WORD_BREAK         0x00000200
#define BDAM_TEXT_SHAPE         0x000000FF
#define BDA_INIT                0x80000000
#define BDA_LEVEL               0x30000000
#define BDA_TEXTTYPE_VISUAL     0x00000000
#define BDA_TEXTTYPE_IMPLICIT   0x01000000
#define BDA_WND_ORIENT_LTR      0x00000000
#define BDA_WND_ORIENT_RTL      0x00100000
#define BDA_TEXT_ORIENT_LTR     0x00000000  
#define BDA_TEXT_ORIENT_RTL     0x00010000
#define BDA_TEXT_ORIENT_DYNAMIC 0x00020000
#define BDA_TEXT_ORIENT_CONTEXT 0x00020000
#define BDA_NUMERALS_NOMINAL    0x00000000
#define BDA_NUMERALS_PASSTHRU   0x00001000
#define BDA_NUMERALS_NATIONAL   0x00002000
#define BDA_NUMERALS_CONTEXTUAL 0x00003000
#define BDA_SYM_SWAP_OFF        0x00000000
#define BDA_SYM_SWAP_ON         0x00000100
#define BDA_WORD_BREAK_OFF      0x00000000
#define BDA_WORD_BREAK_ON       0x00000200
#define BDA_TEXT_DISPLAY_SHAPED 0x00000000
#define BDA_TEXT_SAVE_SHAPED    0x00000001
#define BDA_TEXT_NOMINAL        0x00000010
#define BDA_TEXT_INITIAL        0x00000011
#define BDA_TEXT_MIDDLE         0x00000012
#define BDA_TEXT_FINAL          0x00000013
#define BDA_TEXT_ISOLATED       0x00000014
#define LI_BD_WND_ATTR          0x00000020
#define LIF_CHILD_INHERIT       0x00000002
#define LIF_WND_REFRESH         0x00000004
#endif

typedef unsigned long
     (_System  IWinSetLangInfo)( unsigned long,   // hwnd
                                 unsigned long,   // effect
                                 unsigned long,   // data
                                 unsigned long,   // mask
                                 unsigned long,   // flags
                                 unsigned long ); // reserved

typedef unsigned long
     (_System  IWinQueryLangInfo)( unsigned long,   // hwnd
                                   unsigned long,   // effect
                                   unsigned long,   // flags
                                   unsigned long ); // reserved

typedef unsigned long
     (_System  IGpiQueryBidiAttr)( HPS ); // hps

typedef unsigned long
     (_System  IGpiSetBidiAttr)( HPS,             // hps
                                 unsigned long ); // attributes

/*------------------------------------------------------------------------------
| class IWindowBidiData                                                        |
|                                                                              |
| Private class used for implementation of bidi support                        |
------------------------------------------------------------------------------*/
class IBidiData : public IBase {
typedef IBase
  Inherited;
public:
  IBidiData ( );
Boolean
  support;
unsigned long
  codepage;
IDynamicLinkLibrary
  *pmBidiDynLinkLib;
IWinSetLangInfo
  *setInfo;
IWinQueryLangInfo
  *queryInfo;
IGpiSetBidiAttr
  *setBidiAttr;
IGpiQueryBidiAttr
  *queryBidiAttr;
};

IBidiData :: IBidiData ( )
           : support(false),
             codepage(0),
             pmBidiDynLinkLib(0),
             setInfo(0),
             queryInfo(0),
             setBidiAttr(0),
             queryBidiAttr(0)
{
}

/*------------------------------------------------------------------------------
| class IBidiSettingsData                                                      |
|                                                                              |
| Private class used for implementation of bidi support                        |
------------------------------------------------------------------------------*/
class IBidiSettingsData : public IBase {
typedef IBase
  Inherited;
public:
unsigned long
  data;
IBidiSettings
 *settings;
static IBidiData
 *bidi;
};

/*------------------------------------------------------------------------------
| static data                                                                  |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
IBidiData* IBidiSettingsData::bidi(0);
#pragma data_seg()

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IWindow::isBidiSupported () const
{
  return IBidiSettings::isBidiSupported();
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow& IWindow::applyBidiSettings ( const IWindow::BidiSettings& settings,
                                      Boolean childInherit, Boolean refresh )
{

  settings.fBidiSettingsData->settings->apply(*this,childInherit,refresh);
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings::BidiSettings ( const IWindow& window )
                      :fBidiSettingsData(0)
{
  if ( IBidiSettings::isBidiSupported() == false )
  {
    ITHROWLIBRARYERROR(IC_UNEXPECTED_ERROR,
                       IErrorInfo::invalidRequest,
                       IException::recoverable);
  }

  fBidiSettingsData = new IBidiSettingsData;
  fBidiSettingsData->settings = new IBidiSettings(window);
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings::~BidiSettings ()
{
   delete fBidiSettingsData->settings;
   delete fBidiSettingsData;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::setTextType
                       ( IWindow::BidiTextType textType )
{
  switch ( textType )
  {
    case visual:
      fBidiSettingsData->settings->setTextType(IBidiSettings::visual);
      break;
    case implicit:
      fBidiSettingsData->settings->setTextType(IBidiSettings::implicit);
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::setTextOrientation
                       ( IWindow::BidiTextOrientation textOrientation )
{
  switch ( textOrientation )
  {
    case textLeftToRight:
      fBidiSettingsData->settings->setTextOrientation(IBidiSettings::textLeftToRight);
      break;
    case textRightToLeft:
      fBidiSettingsData->settings->setTextOrientation(IBidiSettings::textRightToLeft);
      break;
    case textContextual:
      fBidiSettingsData->settings->setTextOrientation(IBidiSettings::textContextual);
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::setWindowLayout
                       ( IWindow::BidiLayout windowLayout )
{
  switch ( windowLayout )
  {
    case layoutLeftToRight:
      fBidiSettingsData->settings->setWindowLayout(IBidiSettings::layoutLeftToRight);
      break;
    case layoutRightToLeft:
      fBidiSettingsData->settings->setWindowLayout(IBidiSettings::layoutRightToLeft);
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::setNumeralDisplay
                       ( IWindow::BidiNumeralType numeralDisplay )
{
  switch ( numeralDisplay )
  {
    case arabic:
      fBidiSettingsData->settings->setNumeralDisplay(IBidiSettings::arabic);
      break;
    case asStored:
      fBidiSettingsData->settings->setNumeralDisplay(IBidiSettings::asStored);
      break;
    case national:
      fBidiSettingsData->settings->setNumeralDisplay(IBidiSettings::national);
      break;
    case contextual:
      fBidiSettingsData->settings->setNumeralDisplay(IBidiSettings::contextual);
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::enableSymmetricSwapping
                       ( IWindow::Boolean enable )
{
  fBidiSettingsData->settings->enableSymmetricSwapping(enable);
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::disableSymmetricSwapping ( )
{
   return enableSymmetricSwapping(false);
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::enableWordByWordReordering
                       ( IWindow::Boolean enable )
{
  fBidiSettingsData->settings->enableWordByWordReordering(enable);
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::disableWordByWordReordering ( )
{
   return enableWordByWordReordering(false);
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiSettings& IWindow::BidiSettings::setTextShape
                       ( IWindow::BidiTextShape textShape )
{
  switch ( textShape )
  {
    case displayShaped:
      fBidiSettingsData->settings->setTextShape(IBidiSettings::displayShaped);
      break;
    case saveShaped:
      fBidiSettingsData->settings->setTextShape(IBidiSettings::saveShaped);
      break;
    case nominalShape:
      fBidiSettingsData->settings->setTextShape(IBidiSettings::nominalShape);
      break;
    case initialShape:
      fBidiSettingsData->settings->setTextShape(IBidiSettings::initialShape);
      break;
    case middleShape:
      fBidiSettingsData->settings->setTextShape(IBidiSettings::middleShape);
      break;
    case finalShape:
      fBidiSettingsData->settings->setTextShape(IBidiSettings::finalShape);
      break;
    case isolatedShape:
      fBidiSettingsData->settings->setTextShape(IBidiSettings::isolatedShape);
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiTextType IWindow::BidiSettings::textType () const
{
  if ( fBidiSettingsData->settings->textType() == IBidiSettings::implicit )
    return implicit;
  else
    return visual;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiTextOrientation IWindow::BidiSettings::textOrientation () const
{
  if ( fBidiSettingsData->settings->textOrientation() == 
       IBidiSettings::textContextual )
    return textContextual;
  if ( fBidiSettingsData->settings->textOrientation() == 
       IBidiSettings::textRightToLeft )
    return textRightToLeft;
  return textLeftToRight;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiLayout IWindow::BidiSettings::windowLayout () const
{
  if ( fBidiSettingsData->settings->windowLayout() == 
       IBidiSettings::layoutRightToLeft )                   
    return layoutRightToLeft;
  else
    return layoutLeftToRight;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiNumeralType IWindow::BidiSettings::numeralDisplay () const
{
  if ( fBidiSettingsData->settings->numeralDisplay() == 
       IBidiSettings::contextual )
    return contextual;
  if ( fBidiSettingsData->settings->numeralDisplay() == 
       IBidiSettings::asStored )
    return asStored;
  if ( fBidiSettingsData->settings->numeralDisplay() == 
       IBidiSettings::national )
    return national;
  return arabic;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IWindow::BidiSettings::isSymmetricSwappingEnabled () const
{
  return fBidiSettingsData->settings->isSymmetricSwappingEnabled();
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IWindow::BidiSettings::isWordByWordReorderingEnabled () const
{
  return fBidiSettingsData->settings->isWordByWordReorderingEnabled();
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IWindow::BidiTextShape IWindow::BidiSettings::textShape () const
{
  if ( fBidiSettingsData->settings->textShape() == IBidiSettings::isolatedShape )
    return isolatedShape;
  if ( fBidiSettingsData->settings->textShape() == IBidiSettings::finalShape )
    return finalShape;
  if ( fBidiSettingsData->settings->textShape() == IBidiSettings::middleShape )
    return middleShape;
  if ( fBidiSettingsData->settings->textShape() == IBidiSettings::initialShape )
    return initialShape;
  if ( fBidiSettingsData->settings->textShape() == IBidiSettings::nominalShape )
    return nominalShape;
  if ( fBidiSettingsData->settings->textShape() == IBidiSettings::saveShaped )
    return saveShaped;
  return displayShaped;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IBidiSettings::isBidiSupported ()
{
  if ( !IBidiSettingsData::bidi )
  {
    IBidiSettingsData::bidi = new IBidiData;

    // If the codepage is 862(Hebrew) or 864(Arabic)
    IBidiSettingsData::bidi->codepage = WinQueryCp(HMQ_CURRENT);

    if (( IBidiSettingsData::bidi->codepage == 862 ) ||
        ( IBidiSettingsData::bidi->codepage == 864 ))
    {
      // Try to load the PMBIDI.DLL and get the address of APIs
      try
      {
        IBidiSettingsData::bidi->pmBidiDynLinkLib = new IDynamicLinkLibrary("pmbidi");
        IBidiSettingsData::bidi->setInfo = (IWinSetLangInfo*)
                           IBidiSettingsData::bidi->pmBidiDynLinkLib->procAddress
                           ("Win32SetLangInfo");
        IBidiSettingsData::bidi->queryInfo = (IWinQueryLangInfo*)
                           IBidiSettingsData::bidi->pmBidiDynLinkLib->procAddress
                           ("Win32QueryLangInfo");
        IBidiSettingsData::bidi->setBidiAttr = (IGpiSetBidiAttr*)
                           IBidiSettingsData::bidi->pmBidiDynLinkLib->procAddress
                           ("Gpi32SetBidiAttr");
        IBidiSettingsData::bidi->queryBidiAttr = (IGpiQueryBidiAttr*)
                           IBidiSettingsData::bidi->pmBidiDynLinkLib->procAddress
                           ("Gpi32QueryBidiAttr");
        IBidiSettingsData::bidi->support = true;
      }
      catch ( IException& exc ) {}
    }
  }
  return IBidiSettingsData::bidi->support;


}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::apply ( const IWindow& window,
                                      Boolean childInherit, Boolean refresh )
{
  if ( isBidiSupported() == false )
  {
    ITHROWLIBRARYERROR(IC_UNEXPECTED_ERROR,
                       IErrorInfo::invalidRequest,
                       IException::recoverable);
  }

  //-----------------------------------------------------------------
  // There is a bug in PM that does not update frame extensions
  // when the BIDI attributes of the frame have changed (and the
  // child inherit flag is true).
  //
  // A fix for this problem is to send a private message to the frame
  // before we change the BIDI attributes.  This will enable the
  // window to decide if it should force the frame extensions to be
  // updated.  The following cases are covered:
  //
  // 1. IWindow::applyBidiSettings (no child inherit)
  //    For this case we first send the private message (to set a
  //    flag in the IFrameWindowData).  When IFrameHandler receives
  //    the BIDI attribute change, it will not update frame
  //    extensions (but will turn off the IFrameWindowData flag)
  //
  // 2. IWindow::applyBidiSettings (child inherit)
  //    We do not send the private message and when IFrameHandler
  //    received the BIDI attribute change, it will update all
  //    of the frame extensions.
  //
  // 3. IWindow::applyBidiSettings (to parent of the frame)
  //    (same processing as item 2)
  //
  // 4. User update through Language Viewer (ALT-ScrollLock) popup
  //    (same processing as item 2)
  //-----------------------------------------------------------------
  if ( !childInherit && window.isFrameWindow() )
     window.sendEvent ( IC_UM_BIDI_CHANGE );

  //-----------------------------------------------------------------
  // Let the base window handle case do the real work
  //-----------------------------------------------------------------
  apply ( window.handle(), childInherit, refresh );

  //-----------------------------------------------------------------
  // If the window is a frame, we need to update any toolbars as well
  // in case they are floating or hidden.  We do this by sending a
  // private message to the frame and letting the tool bar handler
  // (attached to the frame automatically by the tool bar) do the
  // work of updating BIDI attributes.
  //-----------------------------------------------------------------
  if ( childInherit && window.isFrameWindow() )
     window.sendEvent ( IC_UM_TOOLBAR, IC_UM_TBAR_BIDI_CHANGE );

  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::apply ( const IWindowHandle windowHandle,
                                      Boolean childInherit, Boolean refresh )
{

  unsigned long ulFlags = 0;

  if (refresh)
    ulFlags |= LIF_WND_REFRESH;

  if (childInherit)
    ulFlags |= LIF_CHILD_INHERIT;

  IBidiSettingsData::bidi->setInfo( windowHandle.asUnsigned(),
                                    LI_BD_WND_ATTR,
                                    fBidiSettingsData->data |
                                    BDA_INIT | BDA_LEVEL,
                                    BDAM_INIT | BDAM_LEVEL |
                                    BDAM_TEXTTYPE | BDAM_TEXT_ORIENTATION |
                                    BDAM_WND_ORIENTATION | BDAM_NUMERALS |
                                    BDAM_SYM_SWAP | BDAM_WORD_BREAK |
                                    BDAM_TEXT_SHAPE,
                                    ulFlags, 0 );

  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::apply ( const IGraphicContext& graphicContext )
{
  return apply ( graphicContext.handle() );
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::apply ( const IPresSpaceHandle presSpace )
{
  IBidiSettingsData::bidi->setBidiAttr( presSpace.asUnsigned(), 
                                        fBidiSettingsData->data );
  return *this;
}


/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::IBidiSettings ( const IWindow& window )
              :fBidiSettingsData(0)
{
  if ( isBidiSupported() == false )
  {
    ITHROWLIBRARYERROR(IC_UNEXPECTED_ERROR,
                       IErrorInfo::invalidRequest,
                       IException::recoverable);
  }

  fBidiSettingsData = new IBidiSettingsData;
  fBidiSettingsData->data = IBidiSettingsData::bidi->queryInfo
                    ( window.handle().asUnsigned(), LI_BD_WND_ATTR, 0L, 0L);
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::IBidiSettings ( const IWindowHandle windowHandle )
              :fBidiSettingsData(0)
{
  if ( isBidiSupported() == false )
  {
    ITHROWLIBRARYERROR(IC_UNEXPECTED_ERROR,
                       IErrorInfo::invalidRequest,
                       IException::recoverable);
  }

  fBidiSettingsData = new IBidiSettingsData;
  fBidiSettingsData->data = IBidiSettingsData::bidi->queryInfo
                    ( windowHandle.asUnsigned(), LI_BD_WND_ATTR, 0L, 0L);
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::IBidiSettings ( const IGraphicContext& graphicContext )
              :fBidiSettingsData(0)
{
  if ( isBidiSupported() == false )
  {
    ITHROWLIBRARYERROR(IC_UNEXPECTED_ERROR,
                       IErrorInfo::invalidRequest,
                       IException::recoverable);
  }

  fBidiSettingsData = new IBidiSettingsData;
  fBidiSettingsData->data = IBidiSettingsData::bidi->queryBidiAttr
                    ( graphicContext.handle().asUnsigned() );
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::IBidiSettings ( const IPresSpaceHandle presSpace )
              :fBidiSettingsData(0)
{
  if ( isBidiSupported() == false )
  {
    ITHROWLIBRARYERROR(IC_UNEXPECTED_ERROR,
                       IErrorInfo::invalidRequest,
                       IException::recoverable);
  }

  fBidiSettingsData = new IBidiSettingsData;
  fBidiSettingsData->data = IBidiSettingsData::bidi->queryBidiAttr
                    ( presSpace.asUnsigned() );
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::~IBidiSettings ()
{
   delete fBidiSettingsData;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::setTextType
               ( IBidiSettings::BidiTextType textType )
{
  switch ( textType )
  {
    case visual:
      fBidiSettingsData->data &= ~BDA_TEXTTYPE_IMPLICIT;
      fBidiSettingsData->data |= BDA_TEXTTYPE_VISUAL;
      break;
    case implicit:
      fBidiSettingsData->data &= ~BDA_TEXTTYPE_VISUAL;
      fBidiSettingsData->data |= BDA_TEXTTYPE_IMPLICIT;
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::setTextOrientation
               ( IBidiSettings::BidiTextOrientation textOrientation )
{
  switch ( textOrientation )
  {
    case textLeftToRight:
      fBidiSettingsData->data &= ~BDA_TEXT_ORIENT_RTL & ~BDA_TEXT_ORIENT_DYNAMIC;
      fBidiSettingsData->data |= BDA_TEXT_ORIENT_LTR;
      break;
    case textRightToLeft:
      fBidiSettingsData->data &= ~BDA_TEXT_ORIENT_LTR & ~BDA_TEXT_ORIENT_DYNAMIC;
      fBidiSettingsData->data |= BDA_TEXT_ORIENT_RTL;
      break;
    case textContextual:
      fBidiSettingsData->data &= ~BDA_TEXT_ORIENT_LTR & ~BDA_TEXT_ORIENT_RTL;
      fBidiSettingsData->data |= BDA_TEXT_ORIENT_DYNAMIC;
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::setWindowLayout
               ( IBidiSettings::BidiLayout windowLayout )
{
  switch ( windowLayout )
  {
    case layoutLeftToRight:
      fBidiSettingsData->data &= ~BDA_WND_ORIENT_RTL;
      fBidiSettingsData->data |= BDA_WND_ORIENT_LTR;
      break;
    case layoutRightToLeft:
      fBidiSettingsData->data &= ~BDA_WND_ORIENT_LTR;
      fBidiSettingsData->data |= BDA_WND_ORIENT_RTL;
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::setNumeralDisplay
               ( IBidiSettings::BidiNumeralType numeralDisplay )
{

  fBidiSettingsData->data &= ~BDA_NUMERALS_NOMINAL & ~BDA_NUMERALS_PASSTHRU &
                             ~BDA_NUMERALS_NATIONAL & ~BDA_NUMERALS_CONTEXTUAL;

  switch ( numeralDisplay )
  {
    case arabic:
      fBidiSettingsData->data |= BDA_NUMERALS_NOMINAL;
      break;
    case asStored:
      fBidiSettingsData->data |= BDA_NUMERALS_PASSTHRU;
      break;
    case national:
      fBidiSettingsData->data |= BDA_NUMERALS_NATIONAL;
      break;
    case contextual:
      fBidiSettingsData->data |= BDA_NUMERALS_CONTEXTUAL;
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::enableSymmetricSwapping ( Boolean enable )
{
  if ( enable )
  {
    fBidiSettingsData->data &= ~BDA_SYM_SWAP_OFF;
    fBidiSettingsData->data |= BDA_SYM_SWAP_ON;
  }
  else
  {
    fBidiSettingsData->data &= ~BDA_SYM_SWAP_ON;
    fBidiSettingsData->data |= BDA_SYM_SWAP_OFF;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::disableSymmetricSwapping ( )
{
   return enableSymmetricSwapping(false);
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::enableWordByWordReordering ( Boolean enable )
{
  if ( enable )
  {
    fBidiSettingsData->data &= ~BDA_WORD_BREAK_OFF;
    fBidiSettingsData->data |= BDA_WORD_BREAK_ON;
  }
  else
  {
    fBidiSettingsData->data &= ~BDA_WORD_BREAK_ON;
    fBidiSettingsData->data |= BDA_WORD_BREAK_OFF;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::disableWordByWordReordering ( )
{
   return enableWordByWordReordering(false);
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings& IBidiSettings::setTextShape
               ( IBidiSettings::BidiTextShape textShape )
{

  fBidiSettingsData->data &= ~BDA_TEXT_DISPLAY_SHAPED & ~BDA_TEXT_SAVE_SHAPED &
                             ~BDA_TEXT_NOMINAL & ~BDA_TEXT_MIDDLE & ~BDA_TEXT_FINAL &
                             ~BDA_TEXT_ISOLATED;

  switch ( textShape )
  {
    case displayShaped:
      fBidiSettingsData->data |= BDA_TEXT_DISPLAY_SHAPED;
      break;
    case saveShaped:
      fBidiSettingsData->data |= BDA_TEXT_SAVE_SHAPED;
      break;
    case nominalShape:
      fBidiSettingsData->data |= BDA_TEXT_NOMINAL;
      break;
    case initialShape:
      fBidiSettingsData->data |= BDA_TEXT_INITIAL;
      break;
    case middleShape:
      fBidiSettingsData->data |= BDA_TEXT_MIDDLE;
      break;
    case finalShape:
      fBidiSettingsData->data |= BDA_TEXT_FINAL;
      break;
    case isolatedShape:
      fBidiSettingsData->data |= BDA_TEXT_ISOLATED;
      break;
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::BidiTextType IBidiSettings::textType () const
{
  if ( fBidiSettingsData->data & BDA_TEXTTYPE_IMPLICIT )
    return implicit;
  else
    return visual;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::BidiTextOrientation IBidiSettings::textOrientation () const
{
  if ( fBidiSettingsData->data & BDA_TEXT_ORIENT_DYNAMIC )
    return textContextual;
  if ( fBidiSettingsData->data & BDA_TEXT_ORIENT_RTL )
    return textRightToLeft;
  return textLeftToRight;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::BidiLayout IBidiSettings::windowLayout () const
{
  if ( fBidiSettingsData->data & BDA_WND_ORIENT_RTL )
    return layoutRightToLeft;
  else
    return layoutLeftToRight;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::BidiNumeralType IBidiSettings::numeralDisplay () const
{
  if (( fBidiSettingsData->data & BDA_NUMERALS_CONTEXTUAL ) == BDA_NUMERALS_CONTEXTUAL )
    return contextual;
  if (( fBidiSettingsData->data & BDA_NUMERALS_PASSTHRU ) == BDA_NUMERALS_PASSTHRU )
    return asStored;
  if (( fBidiSettingsData->data & BDA_NUMERALS_NATIONAL ) == BDA_NUMERALS_NATIONAL )
    return national;
  return arabic;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IBidiSettings::isSymmetricSwappingEnabled () const
{
  if ( fBidiSettingsData->data & BDA_SYM_SWAP_ON )
    return true;
  else
    return false;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IBidiSettings::isWordByWordReorderingEnabled () const
{
  if ( fBidiSettingsData->data & BDA_WORD_BREAK_ON )
    return true;
  else
    return false;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
IBidiSettings::BidiTextShape IBidiSettings::textShape () const
{
  if (( fBidiSettingsData->data & BDA_TEXT_ISOLATED ) == BDA_TEXT_ISOLATED )
    return isolatedShape;
  if (( fBidiSettingsData->data & BDA_TEXT_FINAL ) == BDA_TEXT_FINAL )
    return finalShape;
  if (( fBidiSettingsData->data & BDA_TEXT_MIDDLE ) == BDA_TEXT_MIDDLE )
    return middleShape;
  if (( fBidiSettingsData->data & BDA_TEXT_INITIAL ) == BDA_TEXT_INITIAL )
    return initialShape;
  if (( fBidiSettingsData->data & BDA_TEXT_NOMINAL ) == BDA_TEXT_NOMINAL )
    return nominalShape;
  if (( fBidiSettingsData->data & BDA_TEXT_SAVE_SHAPED ) == BDA_TEXT_SAVE_SHAPED )
    return saveShaped;
  return displayShaped;
}

