/* NOSHIP */
#ifndef _IFLDPRIV_
#define _IFLDPRIV_
/*******************************************************************************
* FILE NAME: ifldpriv.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IFileDialogSettingsData                                                  *
*     IFileDialogData                                                          *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ihandle.hpp>
#include <iqueue.h>
#include <itrace.hpp>

class IHandler;
class IString;

#pragma pack(4)

// StringSeq class is used to maintain sequences of IStrings
// needed for Drives and Types lists.
class StringSeq : public IQueue<IString*> {
public:
  StringSeq ( );
 ~StringSeq ( );
}; // StringSeq

StringSeq::StringSeq ( )
  : IQueue<IString*>()
{ }

StringSeq::~StringSeq ( )
{
  IString* curString;

  // Delete the IStrings that were stored in the collection.
  StringSeq::Cursor current(*this);
  for ( current.setToFirst(); current.isValid(); current.setToNext() )
  {
     curString = this->elementAt(current);
     delete curString;
  } /* end for */
}

/*******************************************************************************
|  Objects of IFileDialogSettingsData contain the private data for Settings    |
*******************************************************************************/
class IFileDialogSettingsData : public IBase {
typedef IBase
  Inherited;
public:
/*------------------------------- Constructors -------------------------------*/
  IFileDialogSettingsData ( )
    : fileTypes(0), drives(0), isDlgTemplateSet(false)
  { }

StringSeq
 *fileTypes,
 *drives;

unsigned long
  dlgId;

IModuleHandle
  dlgHandle;

Boolean
  isDlgTemplateSet;
}; // IFileDialogSettingsData


/*******************************************************************************
|  Objects of IFileDialogData contain the private data for the IFileDialog     |
|  class.                                                                      |
*******************************************************************************/
class IFileDialogData : public IBase {
typedef IBase
  Inherited;
public:
/*------------------------------- Constructors -------------------------------*/
  IFileDialogData ( )
    : handlerToAdd(0)
  { }

IHandler
 *handlerToAdd;
}; // IFileDialogData

#pragma pack()

#endif /* _IFLDPRIV_ */
