/* NOSHIP */
#ifndef _IFLYPRIV_
#define _IFLYPRIV_
/*******************************************************************************
* FILE NAME: iflypriv.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of classes that are private              *
*   to the implementation of iflyhhdr.hpp                                      *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <ihandle.hpp>
#include <itimer.hpp>
#include <irect.hpp>
#include <ireslib.hpp>
#include <istring.hpp>
#include <ikeyset.h>

class IFlyText;
class ITextControl;

class IHandleStringElement {
public:
  IHandleStringElement( unsigned long      handle,
                        const IString&     flyText,
                        const IString&     longText );

  IHandleStringElement( unsigned long      handle,
                        const IResourceId& flyText,
                        const IResourceId& longText );

  ~IHandleStringElement( );

  // Note:  A union would appear to be storage representation of choice
  //        for this class since it will contain either strings or
  //        resource Ids.  It is my understanding though, that the compiler
  //        will generate 32 bytes of storage at a minumum for each
  //        structure which negates any savings by using a union.
  unsigned long handleKey;
  IString       flyStr;
  IString       longStr;
  IResourceId   flyResId;
  IResourceId   longResId;
  Boolean       fStrings;
};

inline unsigned long const& key (IHandleStringElement* const& k)
{
  return k->handleKey;
}

class IHandleStringSet : public IKeySet < IHandleStringElement*, unsigned long >
{
public:
  IHandleStringSet();
  ~IHandleStringSet();
};

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

// Forward declarations for other classes:
class IResourceLibrary;

class IFlyOverData : public IBase {
typedef IBase
  Inherited;
/*******************************************************************************
* Objects of IFontData contain the private data for the IFont class.           *
* In addition, IFontData provides reference counting for IFont objects.        *
*******************************************************************************/
friend class IFlyOverHelpHandler;
public:
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There are 1 ways to construct instances of this class:                       |
|   1. From a pointer to a fly text control, long text control and             |
|      an unsigned long indicating the timer delay.                            |
------------------------------------------------------------------------------*/
  IFlyOverData( IFlyText*            flyText,
                ITextControl*        longText,
                unsigned long        delay,
                unsigned long        initialDelay,
                IFlyOverHelpHandler* flyHelp );

  ~IFlyOverData();

void
  timerpop( unsigned long timerId );

/*------------------------------ Data Members ----------------------------------
|  The following data members are used to different font values:               |
|   flytxt      - Pointer to fly text control.                                 |
|   longtxt     - Pointer to long text control.                                |
|   flyOffset - Offset into the string table for the short text control.       |
|   longOffset  - Offset into the string table for the long text control.      |
|   timer       - Timer object.                                                |
|   prevPos     - last position of the mouse from a WM_CONTROL pointer.        |
|   controlId   - Id of window that the mouse was over from the last control   |
|                 pointer message.                                             |
------------------------------------------------------------------------------*/
IFlyText
 *fFlytxt;
ITextControl
 *fLongtxt;
long
  fFlyOffset,
  fLongOffset;
ITimer
  fTimer;
IPoint
  fPrevPos;
IString
  fDefault;
Boolean
  fSetPrevToNull;
IWindowHandle
  fHandle,
  fPrevHandle,
  fControlHandle;
IWindow*
  fControlWindow;
IFlyOverHelpHandler*
  fFly;
IResourceLibrary
 *fResLib;
IHandleStringSet
 *fHandleStringSet;
unsigned long
  fTimerDelay,
  fInitialTimerDelay;
private:
IFlyOverData
  &operator=(const IFlyOverData&);
IFlyOverData(const IFlyOverData&);
IFlyOverData
  &positionTextWindow( ),
  &setFlyText( const IString& helpText ),
  &setLongText( const IString& helpText );


}; // IFlyOverData;

#pragma pack()

#endif /* _IFLYPRIV_ */
