/* NOSHIP */
#ifndef _IFNTPRIV_
#define _IFNTPRIV_
/*******************************************************************************
* FILE NAME: ifntpriv.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of classes that are private              *
*   to the implementation of ifont.                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <ipoint.hpp>

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

// Forward declarations for other classes:

class IFontPrivateData : public IBase {
typedef IBase
  Inherited;
/*******************************************************************************
* Objects of IFontData contain the private data for the IFont class.           *
* In addition, IFontData provides reference counting for IFont objects.        *
*******************************************************************************/
friend class IFont;
public:
/*------------------------ CONSTRUCTORS/DESTRUCTORS ----------------------------
| There are n ways to construct instances of this class:                       |
|   1. default                                                                 |
|   2. Copy constructor.                                                       |
------------------------------------------------------------------------------*/
  IFontPrivateData( );

  IFontPrivateData( const IFontPrivateData& fpd );

unsigned long
  refCount() const { return refs; }

IFontPrivateData
  &addRef() { refs++; return *this; }
void
  removeRef();

private:
/*------------------------------ Data Members ----------------------------------
|  The following data members are used to different font values:               |
|   pfontmetCl - Holds the font metrics for the font.                          |
|   plClCharWidths - Holds the array of character widths for the font.         |
|   pfattrsCl - Holds the font attributes structure.                           |
|   fxClPointSize - Holds the point size for the font.                         |
|   fxClHeight - Holds the character height.                                   |
|   fxClWidth - Holds the character width.                                     |
|   lClFontDir - Holds the direction to draw the font.                         |
|   lClXAngle - Holds the X portion of the font angle.                         |
|   lClYAngle - Holds the Y portion of the font angle.                         |
|   ptlClShear - Holds the font shear.                                         |
|   lClLCId - Holds the ID uses for the logical font.                          |
------------------------------------------------------------------------------*/
unsigned long
   flClStyles;

struct _FONTMETRICS
   *pfontmetCl;

struct _FATTRS
   *pfattrsCl;

IPoint
   pntClAngle,
   pntClShear;

long
   *plClCharWidths,
   fxClPointSize,
   fxClHeight,
   fxClWidth,
   lClFontDir,
   lClLCId;
unsigned long
   refs;

IFontPrivateData
  &operator=( const IFontPrivateData&);


}; // IFontData

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

/*--------------------------------- INLINES ----------------------------------*/

#endif /* _IFNTPRIV_ */
