#ifndef _IGBUNDLE_INL_
#define _IGBUNDLE_INL_ 0
/*******************************************************************************
* FILE NAME: igbundle.inl                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of the inline functions for the          *
*   classes declared in igbundle.hpp.                                          *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IGBUNDLE_
  #undef  _IGBUNDLE_INL_
  #define _IGBUNDLE_INL_ 1
  #include <igbundle.hpp>
#endif

#if _IGBUNDLE_INL_
  #define inline
#endif

// Set the filling and framing draw operations. They only affect closed graphics.
// Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::setDrawOperation( DrawOperation drawOperation )
{
  fDrawOp = drawOperation;
  fMask  |= IGraphicBundle::drawOp;
  return *this;
}

// Get the filling and framing draw operations. They only affect closed graphics.
inline IGraphicBundle::DrawOperation IGraphicBundle::drawOperation ( ) const
{
  return fDrawOp;
}

//Clear the draw operation. The current graphic context draw operation will be used
//in the drawing. Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::resetDrawOperation ( )
{
  fMask &= ~(unsigned long)IGraphicBundle::drawOp;
  return *this;
}

//Return whether you have set the draw operation since last reset.
inline IBase::Boolean IGraphicBundle::hasDrawOperation ( )  const
{
  return (IBase::Boolean)(fMask & IGraphicBundle::drawOp);
}

//Set pen color. Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::setPenColor( const IColor& penColor )
{
  fPen = penColor;
  fMask |= IGraphicBundle::pColor;
  return *this;
}

//Get pen color.
inline IColor IGraphicBundle::penColor ( ) const
{
  return fPen;
}

//Clear pen color. The current graphic context pen color will be used
//in the drawing. Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::resetPenColor ( )
{
  fMask &= ~(unsigned long)IGraphicBundle::pColor;
  return *this;
}

//Return whether you have set the pen color since last reset.
inline IBase::Boolean IGraphicBundle::hasPenColor ( ) const
{
  return (IBase::Boolean)(fMask & IGraphicBundle::pColor);
}

//Set filling color for closed graphics.
// Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::setFillColor( const IColor& fillColor )
{
  fFill = fillColor;
  fMask |= IGraphicBundle::fColor;
  return *this;
}

//Get filling color for closed graphics.
inline IColor IGraphicBundle::fillColor ( ) const
{
  return fFill;
}

//Clear the filling color. The current graphic context filling color will be used
//in the drawing. Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::resetFillColor ( )
{
  fMask &= ~(unsigned long)IGraphicBundle::fColor;
  return *this;
}

//Return whether you have set the filling color since last reset.
inline IBase::Boolean IGraphicBundle::hasFillColor ( ) const
{
  return (IBase::Boolean)(fMask & IGraphicBundle::fColor);
}

//Set background color. Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::setBackgroundColor(
                                                 const IColor& backgroundColor )
{
  fBack = backgroundColor;
  fMask |= IGraphicBundle::bColor;
  return *this;
}

//Return the background color.
inline IColor IGraphicBundle::backgroundColor ( ) const
{
  return fBack;
}

//Clear the background color. The current graphic context background color will be used
//in the drawing.
inline IGraphicBundle& IGraphicBundle::resetBackgroundColor ( )
{
  fMask &= ~(unsigned long)IGraphicBundle::bColor;
  return *this;
}

//Return whether you have set the background color since last reset.
inline IBase::Boolean IGraphicBundle::hasBackgroundColor ( ) const
{
  return (IBase::Boolean)(fMask & IGraphicBundle::bColor);
}

//Set the pen width. Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::setPenWidth( unsigned long width )
{
  fWidth = width;
  fMask |= IGraphicBundle::pWidth;
  return *this;
}

//Return the current pen width.
inline unsigned long IGraphicBundle::penWidth( ) const
{
  return fWidth;
}

//Clear the pen width. The current graphic context pen width will be used
//in the drawing. Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::resetPenWidth( )
{
  fMask &= ~(unsigned long)IGraphicBundle::pWidth;
  return *this;
}

//Return whether you have set the pen width since last reset.
inline IBase::Boolean IGraphicBundle::hasPenWidth( ) const
{
  return (IBase::Boolean)(fMask & IGraphicBundle::pWidth);
}

//Set the mix mode which is defined in igbundle.hpp
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::setMixMode( MixMode mixMode )
{
  fMask |= IGraphicBundle::pMix;
  fPmix = mixMode;
  return *this;
}

//Get the mix mode.
inline IGraphicBundle::MixMode IGraphicBundle::mixMode( ) const
{
  return fPmix;
}

//Clear the mix mode. The current graphic context mix mode will be used
//in the drawing. Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::resetMixMode( )
{
  fMask &= ~(unsigned long)IGraphicBundle::pMix;
  return *this;
}

//Return whether you have set the mix mode since last reset.
inline IBase::Boolean IGraphicBundle::hasMixMode( ) const
{
  return (IBase::Boolean)(fMask & IGraphicBundle::pMix);
}

//Set the background mix mode defined in igbundle.hpp.
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::setBackgroundMixMode(
                                           BackgroundMixMode backgroundMixMode )
{
  fMask |= IGraphicBundle::bMix;
  fBmix = backgroundMixMode;
  return *this;
}

//Get the background mix mode. Return a reference of this object.    
inline IGraphicBundle::BackgroundMixMode IGraphicBundle::backgroundMixMode( ) const
{
  return fBmix;
}


//Clear the background mix mode. The current graphic context background
//mix mode will be used in the drawing.
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::resetBackgroundMixMode( )
{
  fMask &= ~(unsigned long)IGraphicBundle::bMix;
  return *this;
}

//Return whether you have set the background mix mode since last reset.
inline IBase::Boolean IGraphicBundle::hasBackgroundMixMode( ) const
{
  return (IBase::Boolean)(fMask & IGraphicBundle::bMix);
}

//Set pen type defined in igbundle.hpp.
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::setPenType( PenType penType )
{
  fMask |= IGraphicBundle::pType;
  fPtype = penType;
  return *this;
}

//Get pen type.
inline IGraphicBundle::PenType IGraphicBundle::penType( ) const
{
  return fPtype;
}

//Clear the pen type. The current graphic context pen type
//will be used in the drawing.
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::resetPenType( )
{
  fMask &= ~(unsigned long)IGraphicBundle::pType;
  return *this;
}

//Return whether you have set the pen type since last reset.
inline IBase::Boolean IGraphicBundle::hasPenType( ) const
{
  return (IBase::Boolean)(fMask & IGraphicBundle::pType);
}

//Set the pen ending style.
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::setPenEndingStyle( PenEndingStyle penEndingStyle )
{
  fMask |= IGraphicBundle::pEnd;
  fPend = penEndingStyle;
  return *this;
}

//Get the pen ending style.
inline IGraphicBundle::PenEndingStyle IGraphicBundle::penEndingStyle( ) const
{
  return fPend;
}

//Clear the pen ending style. The current graphic context pen ending style
//will be used in the drawing.
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::resetPenEndingStyle( )
{
  fMask &= ~(unsigned long)IGraphicBundle::pEnd;
  return *this;
}

//Return whether you have set the pen ending style since last reset.
inline IBase::Boolean IGraphicBundle::hasPenEndingStyle( ) const
{
  return (IBase::Boolean)(fMask & IGraphicBundle::pEnd);
}

//Set pen joining style.
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::setPenJoiningStyle(
                                               PenJoiningStyle penJoiningStyle )
{
  fMask |= IGraphicBundle::pJoin;
  fPjoin = penJoiningStyle;
  return *this;
}

//Get pen joining style.
inline IGraphicBundle::PenJoiningStyle IGraphicBundle::penJoiningStyle( ) const
{
  return fPjoin;
}

//Clear the pen joining style. The current graphic context pen joining style
//will be used in the drawing.
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::resetPenJoiningStyle( )
{
  fMask &= ~(unsigned long)IGraphicBundle::pJoin;
  return *this;
}

//Return whether you have set the pen joining style since last reset.
inline IBase::Boolean IGraphicBundle::hasPenJoiningStyle( ) const
{
  return (IBase::Boolean)(fMask & IGraphicBundle::pJoin);
}

//Set pen pattern.
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::setPenPattern( unsigned long penPattern )
{
  fMask |= IGraphicBundle::pPat;
  fPpat = penPattern;
  return *this;
}

//Get pen pattern.
inline unsigned long IGraphicBundle::penPattern( ) const
{
  return fPpat;
}

//Clear the pen pattern. The current graphic context pen pattern
//will be used in the drawing.
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::resetPenPattern( )
{
  fMask &= ~(unsigned long)IGraphicBundle::pPat;
  return *this;
}

//Return whether you have set the pen pattern since last reset.
inline IBase::Boolean IGraphicBundle::hasPenPattern( ) const
{
  return (IBase::Boolean)(fMask & IGraphicBundle::pPat);
}

//Set filling pattern for closed graphics.
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::setFillPattern( unsigned long fillPattern )
{
  fMask |= IGraphicBundle::fPat;
  fFpat = fillPattern;
  return *this;
}

//Get filling pattern for closed graphics.
inline unsigned long IGraphicBundle::fillPattern( ) const
{
  return fFpat;
}

//Clear the filling pattern. The current graphic context filling pattern
//will be used in the drawing.
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::resetFillPattern( )
{
  fMask &= ~(unsigned long)IGraphicBundle::fPat;
  return *this;
}

//Return whether you have set the filling pattern since last reset.
inline IBase::Boolean IGraphicBundle::hasFillPattern( ) const
{
  return (IBase::Boolean)(fMask & IGraphicBundle::fPat);
}

//Set the pattern origin from where it spreads out.
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::setPatternOrigin( const IPoint& pt )
{
  fOrg = pt;
  fMask |= IGraphicBundle::pOrg;
  return *this;
}

//Get the pattern origin.
inline IPoint IGraphicBundle::patternOrigin( ) const
{
  return fOrg;
}

//Clear the pattern origin. The current graphic context pattern origin
//will be used in the drawing.
//Return a reference of this object.
inline IGraphicBundle& IGraphicBundle::resetPatternOrigin( )
{
  fMask &= ~(unsigned long)IGraphicBundle::pOrg;
  return *this;
}

//Return whether you have set the pattern origin since last reset.
inline IBase::Boolean IGraphicBundle::hasPatternOrigin( ) const
{
  return (IBase::Boolean)(fMask & IGraphicBundle::pOrg);
}


#endif // _IGBUNDLE_INL_
