/* NOSHIP */
#ifndef _IMCELCVC_
#define _IMCELCVC_
/*******************************************************************************
* FILE NAME: imcelcvc.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IMultiCellCanvasCell                                                     *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <irect.hpp>

#pragma pack(4)

// forward declarations here


class IWindow;

class IMCCvCell : public IBase {
typedef IBase
  Inherited;
/*******************************************************************************
| This class represents the data contained in the cell of a Multi-Cell Canvas. |
*******************************************************************************/
public:
/*-------------------------- Constructor/Destructor ----------------------------
| You can create instances of this class in the following way:                 |
|    - From a starting column, row, window, the column range and row range.    |
------------------------------------------------------------------------------*/
  IMCCvCell ( unsigned long  startingColumn,
              unsigned long  startingRow,
              IWindow*       window = 0,
              unsigned long  columnRange = 1,
              unsigned long  rowRange = 1);

IMCCvCell
 &setHeight           ( unsigned long height ),
 &setWidth            ( unsigned long width ),
 &setMinimumHeight    ( unsigned long minimumHeight ),  // should be set by
 &setMinimumWidth     ( unsigned long miminumWidth );   // calling setMinimum

unsigned long
  column              ( ) const,
  row                 ( ) const,
  columnRange         ( ) const,
  rowRange            ( ) const,
  width               ( ) const,
  height              ( ) const,
  minimumWidth        ( ) const,
  minimumHeight       ( ) const;

IRectangle
  location            ( const IPoint& origin);

IWindow
 *window              ( ) const;

static IMCCvCell
 &nullCell            ( );

Boolean
  isInUse             ( ) const;

unsigned long
  key                 ( ) const;

IString
 asString             ( ) const,
 asDebugInfo          ( ) const;

private: /*------------------------ PRIVATE ----------------------------------*/

friend unsigned long const& key (IMCCvCell* const& pcell);
IWindow
 *pWindow;
unsigned long
 ulColumnRange,
 ulRowRange;
union {
  struct {
    unsigned short usColumn;
    unsigned short usRow;
  } colRow;
  unsigned long ulKey;
};
unsigned long
 ulCurrentWidth,
 ulCurrentHeight,
 ulMinimumWidth,
 ulMinimumHeight;

static IMCCvCell
 *pnull;
};


/* Cell key function */
inline unsigned long const& key (IMCCvCell* const& cell) {
  return cell->ulKey; }

/*------------------------------------------------------------------------------
| IMCCvCell::IMCCvCell                                                         |
|                                                                              |
| Constructor.                                                                 |
------------------------------------------------------------------------------*/
inline IMCCvCell::IMCCvCell(
                            unsigned long startingColumn,
                            unsigned long startingRow,
                            IWindow*      window,
                            unsigned long columnRange,
                            unsigned long rowRange)
            : pWindow          (window),
              ulColumnRange    (columnRange),
              ulRowRange       (rowRange),
              ulCurrentWidth   (0),
              ulCurrentHeight  (0),
              ulMinimumWidth   (0),
              ulMinimumHeight  (0)
{
  colRow.usColumn =  (unsigned short)(startingColumn);
  colRow.usRow    =  (unsigned short)(startingRow);

}

/*------------------------------------------------------------------------------
| IMCCvCell::setMinimumHeight                                                  |
------------------------------------------------------------------------------*/
inline IMCCvCell& IMCCvCell::setMinimumHeight( unsigned long height)
{
  ulMinimumHeight = height;
  return *this;
}

/*------------------------------------------------------------------------------
| IMCCvCell::setMinimumWidth                                                   |
------------------------------------------------------------------------------*/
inline IMCCvCell& IMCCvCell::setMinimumWidth( unsigned long width )
{
  ulMinimumWidth = width;
  return *this;
}

/*------------------------------------------------------------------------------
| IMCCvCell::setHeight                                                         |
------------------------------------------------------------------------------*/
inline IMCCvCell& IMCCvCell:: setHeight ( unsigned long height )
{
  ulCurrentHeight = height;
  return *this;
}

/*------------------------------------------------------------------------------
| IMCCvCell::setWidth                                                          |
------------------------------------------------------------------------------*/
inline IMCCvCell& IMCCvCell::setWidth(
                                                   unsigned long width)
{
  ulCurrentWidth = width;
  return *this;
}

/*------------------------------------------------------------------------------
| IMCCvCell::width                                                             |
------------------------------------------------------------------------------*/
inline unsigned long IMCCvCell:: width            ( ) const
  { return ulCurrentWidth;  }

/*------------------------------------------------------------------------------
| IMCCvCell::height                                                            |
------------------------------------------------------------------------------*/
inline unsigned long IMCCvCell:: height           ( ) const
  { return ulCurrentHeight; }


/*------------------------------------------------------------------------------
| IMCCvCell::column                                                            |
------------------------------------------------------------------------------*/
inline unsigned long IMCCvCell:: column           ( ) const
  { return (unsigned long)(colRow.usColumn); }

/*------------------------------------------------------------------------------
| IMCCvCell::columnRange                                                       |
------------------------------------------------------------------------------*/
inline unsigned long IMCCvCell:: columnRange      ( ) const
  { return ulColumnRange;   }

/*------------------------------------------------------------------------------
| IMCCvCell::minimumWidth                                                      |
------------------------------------------------------------------------------*/
inline unsigned long IMCCvCell:: minimumWidth   ( ) const
  { return ulMinimumWidth;}

/*------------------------------------------------------------------------------
| IMCCvCell::row                                                               |
------------------------------------------------------------------------------*/
inline unsigned long IMCCvCell:: row              ( ) const
  { return (unsigned long)(colRow.usRow);    }

/*------------------------------------------------------------------------------
| IMCCvCell::rowRange                                                          |
------------------------------------------------------------------------------*/
inline unsigned long IMCCvCell:: rowRange         ( ) const
  { return ulRowRange; }

/*------------------------------------------------------------------------------
| IMCCvCell::minimumHeight                                                     |
------------------------------------------------------------------------------*/
inline unsigned long IMCCvCell:: minimumHeight  ( ) const
  { return ulMinimumHeight; }

/*------------------------------------------------------------------------------
| IMCCvCell::isInUse                                                           |
------------------------------------------------------------------------------*/
inline IBase::Boolean IMCCvCell:: isInUse  ( ) const
  { return (pWindow!=0); }

/*------------------------------------------------------------------------------
| IMCCvCell::window                                                            |
------------------------------------------------------------------------------*/
inline IWindow*  IMCCvCell::window( ) const
  { return pWindow;    }

/*------------------------------------------------------------------------------
| IMCCvCell::key                                                               |
------------------------------------------------------------------------------*/
inline unsigned long IMCCvCell::key( ) const
  { return ulKey;    }

#define CellFromCoords(col, row) ((unsigned long)(((unsigned short)(col)) | \
                                 ((unsigned long)((unsigned short)(row))) << 16))

#pragma pack()

#endif // _IMCELCVC_
