/* NOSHIP */
#ifndef _IOBSPTR_
#define _IOBSPTR_
/*******************************************************************************
* FILE NAME: iobsptr.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IObserverPtr                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ivbase.hpp>
#include <ievtdata.hpp>

class IObserver;
class IObserverDataList;
class INotificationEvent;
class IObserverPtrData;

#pragma pack(4)

class IObserverPtr
{
public:
/*------------------------------- Constructors -------------------------------*/
  IObserverPtr(IObserver*   observerIn=0,
               IEventData   observerDataIn=0)
       : observerData(observerDataIn),
         obsrvr(observerIn) {}
 ~IObserverPtr () {}

IObserver
 &observer ( ) const
   { return *obsrvr; }
IEventData
  observerData;
IObserver
 *obsrvr;

/*-------------------------------- Operators ---------------------------------*/
Boolean
  operator== (const IObserverPtr obsptr) const
  {
    return (obsptr.obsrvr == this->obsrvr);
  }

/**********************************************************************/
/* NOTE: This operator= function as written may seem to be broken.    */
/*       It does not allocate any new memory for the obsrvr pointer   */
/*       it is copying.  This was written this way for a reason,      */
/*       namely efficiency.  This is a private, no ship, header and   */
/*       this function is used in a specific optimized implementation */
/*       of the IObserverList::notifyObservers() function.  The fact  */
/*       that this function is not allocating new memory does NOT     */
/*       break the implementation.  If you change it, it will.  This  */
/*       function purposefully points to the existing IObserver       */
/*       object to use during notification.  It is pointed to from a  */
/*       temporary array created to do the notifications from.  There */
/*       is no reason to create a copy of the observer object...for   */
/*       efficiency we just use what's already there.   See           */
/*       IObserverList::notifyObservers() for implementation details. */
/**********************************************************************/
IObserverPtr
  &operator=( const IObserverPtr& observerPtr )
  {
    if (this != &observerPtr)
    {
      observerData = observerPtr.observerData;
      obsrvr = observerPtr.obsrvr;
    }
    return *this;
  }

private:
/*--------------------------------- Private ----------------------------------*/
IObserverPtrData
 *fObserverPtrData;
}; // IObserverPtr

#pragma pack()

#endif /* _IOBSPTR_ */
