/* NOSHIP */
#ifndef _ISPINSZH_
  #define _ISPINSZH_
/*******************************************************************************
* FILE NAME: ispinszh.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     ISpinButtonSizeHandler - This class allows a spin button to properly     *
*                              size its child entry field.                     *
*     ISpinButtonStatics     - This class manages the creation and deletion of *
*                              the spin button size handler object.            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <isizehdr.hpp>

#pragma pack(4)

// Forward declarations for other classes:
class ISpinButton;
class IBaseSpinButton;

class ISpinButtonSizeHandler : public IResizeHandler {
typedef IResizeHandler
  Inherited;
/*******************************************************************************
* This class allows an IBaseSpinButton window to properly change its size      *
* (actually the size of its child entry field and reposition its spin arrows)  *
* to compensate for the use of SWP_NOADJUST in IWindow::sizeTo.                *
*******************************************************************************/
public:
/*-------------------------- Constructor/Destructor ----------------------------
| The only way to construct instances of this class is to use the default      |
| constructor, which does not take any arguments.                              |
------------------------------------------------------------------------------*/
  ISpinButtonSizeHandler ( );
virtual
 ~ISpinButtonSizeHandler ( );

/*---------------------------- Window Attachment -------------------------------
| These functions permit attaching and detaching the handler object to/from    |
| a spin button:                                                               |
|   handleEventsFor       - Attaches the handler to the specified control.     |
|   stopHandlingEventsFor - Detaches the handler from the specified control.   |
------------------------------------------------------------------------------*/
virtual ISpinButtonSizeHandler
 &handleEventsFor        ( IBaseSpinButton* spinButton ),
 &stopHandlingEventsFor  ( IBaseSpinButton* spinButton );

protected:
/*----------------------------- Event Processing -------------------------------
| These functions are supplied to implement specific event processing.         |
|   windowResize - Allows the spin button to correctly resize and position     |
|                  its child entry field and spin arrows.                      |
------------------------------------------------------------------------------*/
virtual Boolean
  windowResize           ( IResizeEvent& event );

private:
/*--------------------------------- Private ----------------------------------*/
IHandler
 &handleEventsFor        ( IWindow *window ),
 &stopHandlingEventsFor  ( IWindow *window );

#if (IC_OBSOLETE <= IC_OBSOLETE_1 )
public:
/*--------------------- Obsolete data and Functions ----------------------------
| These functions permit attaching and detaching the handler object to/from    |
| a spin button:                                                               |
|   handleEventsFor       - Attaches the handler to the specified control.     |
|   stopHandlingEventsFor - Detaches the handler from the specified control.   |
------------------------------------------------------------------------------*/
virtual ISpinButtonSizeHandler
 &handleEventsFor        ( ISpinButton*     spinButton ),
 &stopHandlingEventsFor  ( ISpinButton*     spinButton );
#endif

}; // class ISpinButtonSizeHandler


class ISpinButtonStatics : public IBase {
typedef IBase
  Inherited;
/*******************************************************************************
* This class manages the lifetime of the static spin button size handler.  An  *
* object of this class is constructed during static object initialization.     *
* The handler object is constructed only if needed, and is deleted at          *
* termination of the application.                                              *
*******************************************************************************/
public:
/*-------------------------- Constructor/Destructor ----------------------------
| The only way to construct instances of this class is to use the default      |
| constructor, which does not take any arguments.                              |
------------------------------------------------------------------------------*/
 ~ISpinButtonStatics ( );

/*-------------------------------- Accessors -----------------------------------
| This function is used to access the static spin button size handler.  This   |
| allows one object to be constructed when needed, and shared by all spin      |
| buttons.                                                                     |
|   sizeHandler   - Returns the spin button size handler.                      |
------------------------------------------------------------------------------*/
static ISpinButtonSizeHandler
 &sizeHandler        ( );

private:
/*--------------------------------- Private ----------------------------------*/
static ISpinButtonSizeHandler
 *pClSizeHandler;
}; // class ISpinButtonStatics

#pragma pack()

#endif /* _ISPINSZH_ */
