/* NOSHIP */
#ifndef _ISPLITPC_
  #define _ISPLITPC_
/*******************************************************************************
* FILE NAME: isplitpc.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IPaneObj         - This class keeps up with the windows occupying the    *
*                        panes of a canvas.                                    *
*     ISplitPercentage - This class keeps up with the percentage data for      *
*                        all windows in the canvas.                            *
*     INumberSplitBar                                                          *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <icanvas.hpp>

#pragma pack(4)

class IPaneSeq;
class ISplitBar;
class ISplitCanvas;

class IPaneObj {
public:
  IPaneObj ( IWindow* window )
    : pwinCl( window ), ulControlSize( 0 ), ulClPercentage( 10 ), pNext( 0 )
  { }
private:
friend ISplitCanvas;
friend IPaneSeq;
IWindow
 *pwinCl;
unsigned long
  ulControlSize,
  ulClPercentage;
IPaneObj
 *pNext;
}; // IPaneObj

class ISplitPercentage {
public:
  ISplitPercentage ( IWindow* window,
                     unsigned long percentage )
    : pwinCl( window ), ulClPercentage( percentage ),
      ulClResolvedPercentage( 0 )
  { }
private:
friend ISplitCanvas;
friend IWindow* const
 &key ( ISplitPercentage* const& element );
IWindow
 *pwinCl;
unsigned long
  ulClPercentage,
  ulClResolvedPercentage;
}; // ISplitPercentage


inline IWindow* const& key ( ISplitPercentage* const& element )
{
  return element->pwinCl;
}



#pragma pack()

#endif /* _ISPLITPC_ */
