/* NOSHIP */
#ifndef _ISTPDATA_
#define _ISTPDATA_
/*******************************************************************************
* FILE NAME: istpdata.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IStringParserData                                                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ivbase.hpp>
#include <istring.hpp>

/*----------------------------------------------------------------------------*/
/* Define token pointer type                                                  */
/*----------------------------------------------------------------------------*/
typedef IString* ITokenPointer;

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

class IStringParserData : public IVBase {
typedef IVBase
  Inherited;
/*******************************************************************************
* Objects of this class are used to store and manage the data for parsing      *
* strings.                                                                     *
*******************************************************************************/
public:
/*------------------------- Constructor and Destructor -------------------------
| The constructor accepts the text string to be parsed and initialized the     |
| parse history information.  The parse history information is used to track   |
| what parsing has been performed (but may not be the final results).          |
|                                                                              |
| The destructor is used to delete possible parse history information.         |
------------------------------------------------------------------------------*/
  IStringParserData ( const IString &text );

 ~IStringParserData ();

/*--------------------------------- Parsing ------------------------------------
| These methods are used to perform the actual string parsing and to store a   |
| record of the actions performed.  These methods are intended to be called    |
| only by IStringParse objects.                                                |
------------------------------------------------------------------------------*/
IStringParserData
  &processToken   ( IString &token, Boolean reparse = false ),
  &processPattern ( const IString &pattern ),
  &processPattern ( const char *pattern ),
  &processPattern ( char pattern ),
  &processSkip    ( Boolean reparse = false ),
  &processTest    ( const IStringTest &test ),
  &setPosition    ( unsigned long position ),
  &changePosition ( int delta ),
  &changePosition ( unsigned long delta );

/*------------------------------ Implementation --------------------------------
| The following methods are used to implement common actions performed during  |
| string parsing.                                                              |
------------------------------------------------------------------------------*/
IStringParserData
  &reparseLastToken (),
  &reparseTokens    ( unsigned long startPos, unsigned long stopPos),
  &saveToken        ( IString *token ),
  &clearSavedTokens ();

/*---------------------------- Reference Counting ------------------------------
| The following functions manage the buffer reference count:                   |
|   addRef    - Increments the usage count.                                    |
|   removeRef - Decrements the usage count and deletes the buffer when the     |
|               usage count goes to 0.                                         |
------------------------------------------------------------------------------*/
void
  addRef    ( ),
  removeRef ( );

/*--------------------------------- Private ----------------------------------*/
private:

IString 
  parseText;
unsigned long
  currentPosition,
  patternPosition,
  patternLength,
  tokenCount,
  tokenArraySize;
IString
 *lastToken;
Boolean
  lastSkip;
ITokenPointer
  *usedTokens;
unsigned long
  refs;
};

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

#endif /* _ISTPDATA_ */
