/* NOSHIP */
#ifndef _IWINLSTS_
#define _IWINLSTS_
/*******************************************************************************
* FILE NAME: iwinlsts.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of classes that are used to              *
*   manage the collection of windows and the collection of handlers.           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ihandler.hpp>
#include <ikeyset.h>
#include <ikssts.h>
#include <ireslock.hpp>
#include <iseq.h>
#include <ihshks.h>
#include <iwindow.hpp>

class IThreadId;

#pragma pack(4)

/*------------------------------------------------------------------------------
| IHandlerListObject class                                                     |
------------------------------------------------------------------------------*/
class IHandlerListObject : public IBase {
public:
  IHandlerListObject ( IHandler* handler );
 ~IHandlerListObject ();
  IHandlerListObject ( const IHandlerListObject& object );
IHandlerListObject
  &operator= ( const IHandlerListObject& object );
IHandler
 *fHandler;
Boolean
  fRemovePending;
};

/*------------------------------------------------------------------------------
| IHandlerList class                                                           |
------------------------------------------------------------------------------*/
class IHandlerList : public ISequence<IHandlerListObject> {
public:
  IHandlerList ();
 ~IHandlerList();

unsigned long
  totalHandlers,
  maxHandlers;
}; // IHandlerList

/*------------------------------------------------------------------------------
| IWindowList class                                                            |
------------------------------------------------------------------------------*/
class IWindowList : public IHashKeySet<IWindow*, unsigned long> {
public:
  IWindowList ( );
 ~IWindowList ( );

#ifdef IC_DEVELOP
static void
  incrementEventCount ( )
  { totalEvents++; }
static void
  incrementWindowAddedCount ( )
  { totalWindowsAdded++; }
static void
  incrementWindowDeletedCount ( )
  { totalWindowsDeleted++; }
static void
  incrementCacheCount ( )
  { cacheUsedCount++; }
static void
  dump ( );
#endif

static IWindowList
 *listForThread ( const IThreadId& threadId ),
 *current       ( );

static IPrivateResource
 &libraryKey ( );

#ifdef IC_DEVELOP
static unsigned long
  totalEvents,
  totalWindowsAdded,
  totalWindowsDeleted,
  cacheUsedCount;
#endif

static IPrivateResource
  libKey;
}; // IWindowList

/*------------------------------------------------------------------------------
| hash                                                                         |
------------------------------------------------------------------------------*/
inline unsigned long hash(unsigned long const& key, unsigned long size)
{
   return key % size;

}


#pragma pack()

#endif // _IWINLSTS_
