/* NOSHIP */
#ifndef _IWINPRIV_
#define _IWINPRIV_
/*******************************************************************************
* FILE NAME: iwinpriv.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the definition of classes that are private              *
*   to the implementation of iwindow.                                          *
*                                                                              *
*   It also contains a cross reference containing the location of functions    *
*   used to implement IWindow.                                                 *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ihandle.hpp>
#include <ipoint.hpp>
#include <iwindow.hpp>

class IHandlerList;
class IObserverList;
class IWindowNotifyHandler;

#pragma pack(4)


/*------------------------------------------------------------------------------
| ITheDesktopWindow class                                                      |
------------------------------------------------------------------------------*/
class ITheDesktopWindow : public IWindow {
typedef IWindow
  Inherited;

public:
  ITheDesktopWindow ( );
virtual
 ~ITheDesktopWindow ( );

virtual IWindowHandle
  handle   ( ) const;

private:
  ITheDesktopWindow ( const ITheDesktopWindow& );
ITheDesktopWindow
 &operator=         ( const ITheDesktopWindow& );

IWindowHandle
  hwnd;
}; // ITheDesktopWindow

/*------------------------------------------------------------------------------
| ITheObjectWindow class                                                       |
------------------------------------------------------------------------------*/
class ITheObjectWindow : public IWindow {
typedef IWindow
  Inherited;

public:
  ITheObjectWindow ( );
virtual
 ~ITheObjectWindow ( );
virtual IWindowHandle
  handle ( ) const;

private:
  ITheObjectWindow ( const ITheObjectWindow& );
ITheObjectWindow
 &operator=        ( const ITheObjectWindow& );

IWindowHandle
  hwnd;
}; // ITheObjectWindow

/*------------------------------------------------------------------------------
| IObserverData class                                                          |
| The IObserverData member will contain an IObserverList following the         |
| first INotifier::add(IObserver) request and an IWindowNotifyHandler          |
| following the first INotifier::enableNotification.                           |
------------------------------------------------------------------------------*/
class IObserverData : public IBase {
public:
  IObserverData ( );
 ~IObserverData ( );

IObserverList
 *observers;
IWindowNotifyHandler
 *notifyHandler;
Boolean
  enabled;
}; // IObserverData

/*------------------------------------------------------------------------------
| IWindowData class                                                            |
------------------------------------------------------------------------------*/
class IWindowData : public IBase {
public:
  IWindowData  ( IWinProc*     windowProcedure,
                 unsigned long defState        );
 ~IWindowData  ( );
IObserverData
 &observerData ( );

IWinProc
 *defaultProc;
unsigned long
  state,
  layoutChange,
  helpId,
  ulExtGUIStyle;  //Extended GUI Style
IHandlerList
 *handlerList;
ISize
  minimumSize,
  cachedMinimumSize,
  sizClChar;       // Character size in the current font.
IObserverData
 *_observerData;
Boolean
  fHandlerRemovePending;
Boolean
  fSetMinSizeInProgress;
IAccelTblHandle
 fAccelHandle;


// The following supports recoordination for Motif


static IWindow::ExceptionFn
 *exceptionFunction;

static IWindow::SiblingOrder
  siblingCreateOrder;

static ITheDesktopWindow
 *desktopWindow;

static ITheObjectWindow
 *objectWindow;

enum StaticFlags {
  fastWindowWithHandle=1   // enables use of GWL_USER window word by IOC
  };
static unsigned long
  staticFlags;             // values are 0 and StaticFlags enum

};

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

/*----------------------------- Inline Functions -----------------------------*/
#ifndef I_NO_INLINES
  #include <iwinpriv.inl>
#endif


#endif /* _IWINPRIV_ */

#if 0
---------------------------------------------------------------
  Function Locator
---------------------------------------------------------------
Key
 ALL  (iwindow0.cpp) - Function portable to all environments
 AIX  (iwindow1.cpp) - Function is redefined for AIX
 PMNT (iwindow2.cpp) - Function portable to PM & NT
 PM   (iwindow3.cpp) - Function is redefined for PM
 NT   (iwindow4.cpp) - Function is redefined for NT
 ALL5 (iwindow5.cpp) - Function portable to all environments

ALL        IHandlerList::IHandlerList ()
ALL        IHandlerList::~IHandlerList()
PMNT  AIX  ITheDesktopWindow::ITheDesktopWindow()
ALL        ITheDesktopWindow::handle() const
ALL        ITheDesktopWindow::~ITheDesktopWindow()
PMNT  AIX  ITheObjectWindow::ITheObjectWindow()
ALL        ITheObjectWindow::handle() const
ALL        ITheObjectWindow::~ITheObjectWindow()
NT PM AIX  IWindow::ChildCursor::ChildCursor ( IWindow &parent )
NT PM AIX  IWindow::ChildCursor::invalidate ( )
NT PM AIX  IWindow::ChildCursor::isValid ( ) const
NT PM AIX  IWindow::ChildCursor::setToFirst ( )
NT PM AIX  IWindow::ChildCursor::setToNext ( )
NT PM AIX  IWindow::ChildCursor::~ChildCursor ( )
ALL        IWindow::ExceptionFn*
ALL        IWindow::ExceptionFn* IWindow::exceptionFunction ( )
ALL        IWindow::ExceptionFn::~ExceptionFn ()
PMNT  AIX  IWindow::IWindow ( const IWindowHandle& handle)
PMNT  AIX  IWindow::IWindow( unsigned long id,
PMNT  AIX  IWindow::IWindow()
ALL        IWindow::SiblingOrder IWindow::defaultOrdering ( )
PMNT       IWindow::acceleratorHandle()
PMNT       IWindow::acceleratorTable()
PMNT  AIX  IWindow::addHandler(IHandler* pshNew)
PMNT  AIX  IWindow::addToWindowSet(IWindow* window,
ALL5       IWindow::asDebugInfo() const
PMNT       IWindow::asGUIStyle( const IBitFlag& style, Boolean bExt )
ALL5       IWindow::asString() const
ALL        IWindow::calcMinimumSize() const
???   AIX  IWindow::calculateRecoordinationHeightPMtoNon()
ALL        IWindow::childAt ( const ChildCursor &cursor ) const
PMNT  AIX  IWindow::cleanUp()
ALL        IWindow::cleanUpHandler ( IHandler &handler )
NT PM AIX  IWindow::color(unsigned long ulColorArea) const
NT PM AIX  IWindow::color(unsigned long ulColorArea, const IColor& defaultColor) const
PMNT  AIX  IWindow::create( unsigned long        id,
ALL        IWindow::defaultProcedure(IEvent& evt)
PMNT  AIX  IWindow::defaultPushButton ( ) const
ALL        IWindow::deleteIsInProcess ( ) const
PMNT  AIX  IWindow::desktopWindow()
ALL        IWindow::disable()
ALL        IWindow::disableUpdate( )
PMNT  AIX  IWindow::dispatch(IEvent& evt)
PMNT  AIX  IWindow::enable(Boolean bEnable)
PMNT  AIX  IWindow::enableUpdate(Boolean enableWindow)
PMNT       IWindow::extendedStyle() const
ALL        IWindow::handle() const
ALL        IWindow::handleException (IException& exc,
PMNT  AIX  IWindow::handleWithId( unsigned long ulId,
PMNT  AIX  IWindow::hasFocus() const
ALL        IWindow::hide()
ALL        IWindow::hideSourceEmphasis()
PMNT  AIX  IWindow::id() const
      AIX  IWindow::idName() const
ALL        IWindow::isAutoDeleteObject ( ) const
ALL        IWindow::isAutoDestroyWindow ( ) const
PMNT  AIX  IWindow::isDisabled() const
PMNT  AIX  IWindow::isFrameWindow() const
ALL        IWindow::isLayoutDistorted( unsigned long layoutAttribute) const
ALL        IWindow::isPrimaryWindow ( ) const
      AIX  IWindow::isRealized()
      AIX  IWindow::isRelatedHandle(const IWindowHandle& wndh) const
NT PM AIX  IWindow::isShowing() const
ALL        IWindow::isTabStop ( ) const
PMNT  AIX  IWindow::isValid() const
PMNT  AIX  IWindow::isVisible() const
PMNT  AIX  IWindow::mapPoint ( const IPoint        &aPoint,
PMNT  AIX  IWindow::matchForMnemonic ( unsigned short character ) const
PMNT       IWindow::messageQueue ( ) const
ALL        IWindow::minimumSize() const
NT PM AIX  IWindow::moveSizeTo(const IRectangle& rect)
AIX        IWindow::moveSizeToForCompoundCtl(const IRectangle& rect)
NT PM AIX  IWindow::moveTo(const IPoint& pt)
      AIX  IWindow::moveToForCompoundCtl(const IPoint& pt)
PMNT  AIX  IWindow::objectWindow()
PMNT  AIX  IWindow::owner() const
PMNT  AIX  IWindow::parent() const
NT PM AIX  IWindow::position() const
NT PM AIX  IWindow::positionBehindSibling ( const IWindowHandle& siblingWindow)
NT PM AIX  IWindow::positionBehindSiblings ( )
      AIX  IWindow::positionForCompoundCtl() const
NT PM AIX  IWindow::positionOnSiblings ( )
PMNT  AIX  IWindow::postEvent ( EventType    eventType,
ALL        IWindow::postEvent ( const IEvent&          event) const
ALL        IWindow::postEvent ( unsigned long           eventId,
PMNT  AIX  IWindow::presSpace() const
      AIX  IWindow::realize()
      AIX  IWindow::recoordinateChildren()
      AIX  IWindow::recoordinationHeight(IPair::Coord proposedNewFarpoint) const
NT PM AIX  IWindow::rect() const
PMNT  AIX  IWindow::refresh(Boolean fRefreshChildren)
PMNT  AIX  IWindow::refresh(const IRectangle& rectArea)
      AIX  IWindow::registerCallbacks()
PMNT  AIX  IWindow::releasePresSpace(const IPresSpaceHandle& gc) const
PMNT  AIX  IWindow::removeFromWindowSet(IWindow* window)
PMNT  AIX  IWindow::removeHandler(IHandler* pshOld)
PMNT       IWindow::savedMinimumSize ( ) const
PMNT       IWindow::saveMinimumSize ( const ISize& size )
PMNT  AIX  IWindow::sendEvent ( EventType    eventType,
ALL        IWindow::sendEvent ( const IEvent& event) const
ALL        IWindow::sendEvent ( unsigned long           eventId,
PMNT       IWindow::setAcceleratorHandle( const IAccelTblHandle& )
PMNT       IWindow::setAcceleratorTable( const IAcceleratorTable*)
ALL        IWindow::setAutoDeleteObject(Boolean fDelete)
ALL        IWindow::setAutoDestroyWindow(Boolean fDestroy)
NT PM AIX  IWindow::setColor(unsigned long ulColorArea, const IColor& color)
ALL        IWindow::setDefaultOrdering  ( IWindow::SiblingOrder order)
ALL        IWindow::setDefaultProcedure ( IWinProc *newDefaultProc )
ALL        IWindow::setDeleteInProcess ( )
PMNT       IWindow::setExtendedStyle(unsigned long flExtStyle)
PMNT  AIX  IWindow::setFocus()
PMNT  AIX  IWindow::setLayoutDistorted( unsigned long layoutAttributeOn,
ALL        IWindow::setMinimumSize(const ISize& sizMin)
PM    AIX  IWindow::setOwner(const IWindow* newOwner)
PMNT  AIX  IWindow::setParent(const IWindow* newParent)
PMNT  AIX  IWindow::setStyle(unsigned long flStyle)
PMNT       IWindow::setWindowData ( long index, unsigned long ulong )
PMNT       IWindow::setWindowData ( long index, unsigned short ushort )
PMNT  AIX  IWindow::show(Boolean bShow)
ALL        IWindow::showSourceEmphasis(Boolean show)
ALL        IWindow::startHandlingEventsFor(unsigned long id,
NT PM AIX  IWindow::size() const
NT PM AIX  IWindow::sizeTo(const ISize& siz)
PMNT  AIX  IWindow::startHandlingEventsFor(const IWindowHandle& windowHandle)
PMNT  AIX  IWindow::style() const
      AIX  IWindow::sizeForCompoundCtl() const
      AIX  IWindow::unregisterCallbacks()
PMNT       IWindow::windowULong ( long index ) const
PMNT       IWindow::windowUShort ( long index ) const
PMNT  AIX  IWindow::windowWithHandle(const IWindowHandle& wndh)
ALL        IWindow::windowWithId(unsigned long ulId, const IWindow* owner)
      AIX  IWindow::windowWithRelatedHandle(const IWindowHandle& wndh)
PMNT  AIX  IWindow::~IWindow()
ALL        IWindowList::IWindowList() : IKeySet<IWindow*, unsigned long>(300)
ALL        IWindowList::current()
ALL        IWindowList::dump()
ALL        IWindowList::libraryKey()
ALL        IWindowList::~IWindowList()

#endif
