/*******************************************************************************
* FILE NAME: idmoper.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMOperation                                                             *
*       IDMSourceOperation                                                             *
*       IDMTargetOperation                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  /* define what subsections of PM headers to include */
  #define INCL_WINPOINTERS
  #define INCL_WINSYS
  #define INCL_WININPUT
  #define INCL_GPIBITMAPS
  #define INCL_WINSTDDRAG       // Standard drag stuff
  #define INCL_WINSTDCNR        // Standard container stuff
  #include <os2.h>
}

// Exception handling
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif

// Trace Facility
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

#ifndef _IPOINT_
  #include <ipoint.hpp>
#endif

#ifndef _IDMSRCOP_
  #include <idmsrcop.hpp>
#endif

#ifndef _IDMTGTOP_
  #include <idmtgtop.hpp>
#endif

#ifndef _IDMITEM_
  #include <idmitem.hpp>
#endif

#ifndef _IDMPROV_
  #include <idmprov.hpp>
#endif

#ifndef _IDMEVENT_
  #include <idmevent.hpp>
#endif

#ifndef _IDMCOMM_
  #include <idmcomm.hpp>
#endif

#ifndef _ICTLEVT_
  #include <ictlevt.hpp>
#endif

#ifndef _ICNRCTL_
  #include <icnrctl.hpp>
#endif

#ifndef _ICNROBJ_
  #include <icnrobj.hpp>
#endif

#ifndef _ICNRREC_
  #include <icnrrec.hpp>
#endif

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifndef _IHANDLE_
  #include <ihandle.hpp>
#endif

#ifndef _IDMIMAGE_
  #include <idmimage.hpp>
#endif

#ifndef _IDMSEQ_                           //PRIVATE
  #include <idmseq.hpp>
#endif

#ifndef _IGBITMAP_
  #include <igbitmap.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMOPER_CPP_
  #include <ipagetun.h>
#endif

#define DRG_DEBUG        ((PVOID) 0x80000000L)    /* prevents screen locking */

typedef IDM::DragImageStyle DragImageStyle;
typedef IDM::DropStyle      DropStyle;

/* Initialize static */
#pragma data_seg(ICLNonConst)
IDMTargetOperation::Handle IDMTargetOperation::pDMTgtOpHandle(0);
#pragma data_seg()


/*------------------------------------------------------------------------------
| Static operation definitions.                                                |
|                                                                              |
| These values exploit the specific values used for the PM toolkit             |
| DO_* values.  Item creation must support the operations corresponding        |
| to the DO_* values.                                                          |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const unsigned long IDMOperation::drag    (DO_DEFAULT);
const unsigned long IDMOperation::move    (DO_MOVE);
const unsigned long IDMOperation::copy    (DO_COPY);
const unsigned long IDMOperation::link    (DO_LINK);
const unsigned long IDMOperation::unknown (DO_UNKNOWN);
#pragma data_seg()

/*------------------------------------------------------------------------------
| IDMOperation::IDMOperation                                                   |
|                                                                              |
| Constructor                                                                  |
------------------------------------------------------------------------------*/
IDMOperation :: IDMOperation(IDMSourceBeginEvent &event)  :
                             src(event.source()),
                             pos(event.position()),
                             op(drag),
                             srcWindowHandle(0),
                             tgtWindowHandle(0),
                             pCnrObj(0),
                             pCnrRefresh(0),
                             pPMDragInfo(0),
                             bDragResult(true),
                             pDMItemSeqCl(0)
{
}

/*------------------------------------------------------------------------------
| IDMOperation::IDMOperation                                                   |
|                                                                              |
| Constructor                                                                  |
------------------------------------------------------------------------------*/
IDMOperation :: IDMOperation(IDMTargetEnterEvent &event)  :
                             src(IDM::pointingDevice),
                             pos(IPoint(0,0)),
                             op(drag),
                             srcWindowHandle(0),
                             tgtWindowHandle(0),
                             pCnrObj(0),
                             pCnrRefresh(0),
                             pPMDragInfo(0),
                             bDragResult(true),
                             pDMItemSeqCl(0)
{
}

/*------------------------------------------------------------------------------
| IDMOperation::~IDMOperation                                                  |
|                                                                              |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IDMOperation :: ~IDMOperation ( )
{
  delete this->pDMItemSeqCl;
}

/*------------------------------------------------------------------------------
| IDMOperation::operation                                                      |
|                                                                              |
| Returns the current direct manipulation operation.                           |
------------------------------------------------------------------------------*/
unsigned long IDMOperation :: operation () const
{
  return (this->op);
}

/*------------------------------------------------------------------------------
| IDMOperation::setOperation                                                   |
|                                                                              |
| Sets the operation of the Direct Manipulation operation.                     |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setOperation (unsigned long Operation)
{
  this->op = Operation;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::position                                                       |
|                                                                              |
| Returns the drag or drop position within the source or target window.        |
------------------------------------------------------------------------------*/
IPoint IDMOperation :: position () const
{
  return (this->pos);
}

/*------------------------------------------------------------------------------
| IDMOperation::source                                                         |
|                                                                              |
| Returns the source of the direct manipulation operation                      |
| (i.e. always pointing device)                                                |
------------------------------------------------------------------------------*/
IDM::Source IDMOperation :: source() const
{
  return (this->src);
}

/*------------------------------------------------------------------------------
| IDMOperation::sourceWindowHandle                                             |
|                                                                              |
| Returns the handle of the source window for the direct manipulation          |
| operation.                                                                   |
------------------------------------------------------------------------------*/
IWindowHandle IDMOperation :: sourceWindowHandle () const
{
  return (this->srcWindowHandle);
}

/*------------------------------------------------------------------------------
| IDMOperation::targetWindowHandle                                             |
|                                                                              |
| Returns the handle of the target window for the direct manipulation          |
| operation.                                                                   |
------------------------------------------------------------------------------*/
IWindowHandle IDMOperation :: targetWindowHandle () const
{
  return (this->tgtWindowHandle);
}

/*------------------------------------------------------------------------------
| IDMOperation::sourceWindow                                                   |
|                                                                              |
| Returns a pointer to the source window for the direct manipulation           |
| operation.                                                                   |
------------------------------------------------------------------------------*/
IWindow* IDMOperation :: sourceWindow () const
{
  return (IWindow::windowWithHandle(this->srcWindowHandle));
}

/*------------------------------------------------------------------------------
| IDMOperation::targetWindow                                                   |
|                                                                              |
| Returns a pointer to the target window for the direct manipulation           |
| operation.                                                                   |
------------------------------------------------------------------------------*/
IWindow* IDMOperation :: targetWindow () const
{
  return (IWindow::windowWithHandle(this->tgtWindowHandle));
}

/*------------------------------------------------------------------------------
| IDMOperation::numberOfItems                                                  |
|                                                                              |
| Returns the number of items in the collection.                               |
------------------------------------------------------------------------------*/
unsigned IDMOperation :: numberOfItems ()
{
  return (itemCollection()->numberOfElements());
}

/*------------------------------------------------------------------------------
| IDMOperation::item                                                           |
|                                                                              |
| Returns the drag item with a given index.  0 is returned if the index is     |
| invalid.                                                                     |
------------------------------------------------------------------------------*/
IDMItem::Handle IDMOperation :: item(unsigned index)
{
  /*------------------------------------------------------------------------
    Get number of items
  ------------------------------------------------------------------------*/
  unsigned count = numberOfItems();

  /*------------------------------------------------------------------------
    Verify index
  ------------------------------------------------------------------------*/
  if ((index < 1)  ||  (index > count))
  {
//  Throw exception
    ITHROWLIBRARYERROR(IC_INDEX_OUT_OF_RANGE,
                       IErrorInfo::invalidRequest,
                       IException::recoverable);
    return(0);
  }

  /*------------------------------------------------------------------------
    Return item handle at index
  ------------------------------------------------------------------------*/
  return(itemCollection()->elementAtPosition(index));
}

/*------------------------------------------------------------------------------
| IDMOperation::replaceItem                                                    |
|                                                                              |
| Replace a given drag item with another.                                      |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: replaceItem (unsigned index,
                                           const IDMItem::Handle &pDI)
{
  /*------------------------------------------------------------------------
    Get number of items
  ------------------------------------------------------------------------*/
  unsigned count = numberOfItems();

  /*------------------------------------------------------------------------
    Verify index
  ------------------------------------------------------------------------*/
  if ((index < 1)  ||  (index > count))
  {
//  Throw exception
    ITHROWLIBRARYERROR(IC_INDEX_OUT_OF_RANGE,
                       IErrorInfo::invalidRequest,
                       IException::recoverable);
    return(*this);
  }

  /*------------------------------------------------------------------------
    Replace the item
  ------------------------------------------------------------------------*/
  removeItem(index);
  itemCollection()->addAtPosition(index, pDI);

  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::addItem                                                        |
|                                                                              |
| Associate another drag item with this operation.                             |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: addItem (const IDMItem::Handle &pDI)
{
  IMODTRACE_DEVELOP( "IDMOperation::addItem" );

  /*-------------------------------------------------------------------
     Store item in collection and bump the operation's use count.
   -------------------------------------------------------------------*/
  itemCollection()->add(pDI);
  ITRACE_DEVELOP( "IDMOperation::addItem - addRef" );
  this->addRef();
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::removeItem                                                     |
|                                                                              |
| Remove a drag item from the operation.                                       |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: removeItem (unsigned index)
{
  /*------------------------------------------------------------------------
    Define a cursor for the drag item sequence and set to the proper
    position.
  ------------------------------------------------------------------------*/
  IDMItemSeq::Cursor cursor(*(itemCollection()));
  itemCollection()->setToPosition(index, cursor);

  /*-----------------------------------------------------------------------
      Set the item handle to 0 and remove item from the collection
   -----------------------------------------------------------------------*/
  itemCollection()->elementAt(cursor) = 0;
  itemCollection()->removeAtPosition(index);
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::removeItem                                                     |
|                                                                              |
| Remove a drag item from the operation.                                       |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: removeItem (const IDMItem::Handle &pDI)
{
  /*------------------------------------------------------------------------
    Get number of items
  ------------------------------------------------------------------------*/
  unsigned count = numberOfItems();

  /*------------------------------------------------------------------------
    Loop thru the collection and remove item if a match is found
  ------------------------------------------------------------------------*/
  for (unsigned index = 1; (index <= count); index++)
  {
    if ((itemCollection()->elementAtPosition(index)) == pDI)
    {
      removeItem(index);
      break;
    }
  }

  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::containerObject                                                |
|                                                                              |
| Returns a container object.                                                  |
------------------------------------------------------------------------------*/
IContainerObject* IDMOperation :: containerObject() const
{
  return (this->pCnrObj);
}

/*------------------------------------------------------------------------------
| IDMOperation::setContainerObject                                             |
|                                                                              |
| Sets a pointer to a container object.                                        |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setContainerObject(IContainerObject *object)
{
  this->pCnrObj = object;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::setContainerRefreshOff                                         |
|                                                                              |
| Disables the container refreshes, will be re-enable it after processing.     |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setContainerRefreshOff(IContainerControl* pCnr)
{
  IMODTRACE_DEVELOP("IDMOperation::setContainerRefreshOff");

  this->pCnrRefresh = pCnr;
  pCnr->setRefreshOff();

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::setContainerRefreshOn                                          |
|                                                                              |
| Re-enables container refreshes.                                              |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setContainerRefreshOn()
{
  IMODTRACE_DEVELOP("IDMOperation::setContainerRefreshOn");

  if (this->pCnrRefresh)
  {
    if (targetWindowHandle())
    {
      this->pCnrRefresh->setRefreshOn().refresh();
      this->pCnrRefresh = 0;
    }
    else
    {
      DRAGINFO*  pPMDragInfo = dragInfo();
      if (!pPMDragInfo ||
         ((unsigned long)pPMDragInfo->usOperation == IDMOperation::copy) ||
         ((unsigned long)pPMDragInfo->usOperation == IDMOperation::link) ||
          dragWasInterrupted())
      {
        this->pCnrRefresh->setRefreshOn();
      }
      else
      {
        this->pCnrRefresh->setRefreshOn().refresh();
      }
    }
  }

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::setSourceWindowHandle                                          |
|                                                                              |
| Sets the source window of the direct manipulation operation during a source  |
| begin event.                                                                 |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation ::
              setSourceWindowHandle (IWindowHandle srcWindow)
{
  this->srcWindowHandle = srcWindow;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::setTargetWindowHandle                                          |
|                                                                              |
| Sets the target window of the direct manipulation operation during a target  |
| enter event.                                                                 |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation ::
              setTargetWindowHandle (IWindowHandle tgtWindow)
{
  this->tgtWindowHandle = tgtWindow;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::setSource                                                      |
|                                                                              |
| Sets the source of the Direct Manipulation operation.                        |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setSource (Source source)
{
  this->src = source;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::setPosition                                                    |
|                                                                              |
| Sets the pointing device position for the Direct Manipulation operation.     |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setPosition (IPoint position)
{
  this->pos = position;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::setDragInfo                                                    |
|                                                                              |
| Sets the pointer to the PM DRAGINFO structure.                               |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setDragInfo (_DRAGINFO *pDragInfo)
{
  this->pPMDragInfo = pDragInfo;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::setDragResult                                                  |
|                                                                              |
| Sets the result of the drag operation.                                       |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setDragResult (Boolean result)
{
  this->bDragResult = result;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::itemCollection                                                 |
|                                                                              |
| Returns a pointer to the source collection of IDMItem::Handles.              |
| Notes:                                                                       |
|   1) Private                                                                 |
------------------------------------------------------------------------------*/
IDMItemSeq* IDMOperation :: itemCollection()
{
  if (!this->pDMItemSeqCl)
    this->pDMItemSeqCl = new IDMItemSeq;

  return (this->pDMItemSeqCl);
}


/*------------------------------ Source Operation ----------------------------*/


/*------------------------------------------------------------------------------
| IDMSourceOperation::IDMSourceOperation                                       |
|                                                                              |
| Constructor                                                                  |
------------------------------------------------------------------------------*/
IDMSourceOperation :: IDMSourceOperation(IDMSourceBeginEvent &event,
                                         DragImageStyle dIStyle)
  : IDMOperation(event),
    drgImgStyle(dIStyle),
    totalImgOffset(0,0),
    ptrOffset(0,0),
    stackingPct(IPair(50,50))
{
  IMODTRACE_DEVELOP("IDMSourceOperation::ctor");

  if (event.eventId() == WM_BEGINDRAG)
    setSourceWindowHandle(event.handle());
  else
  {
    /*-----------------------------------------------------------------
       Verify that this is the correct message and notification code
     -----------------------------------------------------------------*/
    IASSERT(event.eventId() == WM_CONTROL &&
            event.parameter1().number2() == CN_INITDRAG);

    IControlEvent ctlevt(event);
    setSourceWindowHandle(ctlevt.controlWindow()->handle());
  }
}

/*------------------------------------------------------------------------------
| IDMSourceOperation:: ~IDMSourceOperation                                     |
|                                                                              |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IDMSourceOperation :: ~IDMSourceOperation ()
{
  IMODTRACE_DEVELOP("IDMSourceOperation::dtor");

  /*-------------------------------------------------------------------
    Enable container refresh (this function can detect whether or
    not we're processing a container) ...
   -------------------------------------------------------------------*/
  setContainerRefreshOn();

  DRAGINFO*  pPMDragInfo = dragInfo();
  DrgFreeDraginfo (pPMDragInfo);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::begin                                                    |
|                                                                              |
| Begin the drag operation.                                                    |
------------------------------------------------------------------------------*/
IDMSourceOperation& IDMSourceOperation::begin ()
{
  _DRAGINFO       *pDInfo;
  _DRAGIMAGE      DragImage;
  _DRAGIMAGE      *pDrgImg;
  _DRAGIMAGE      *pDragImage = 0;
  Boolean         done = false;
  unsigned long   arraySize = 0;
  unsigned short  numberOfImagesFound = 0;
  unsigned long   ulItemCnt = numberOfItems();
  DragImageStyle  diStyle   = imageStyle();

  /*-------------------------------------------------------------------
     Make sure we have some items to drag...
   -------------------------------------------------------------------*/
  if (ulItemCnt)
  {
    pDInfo = DrgAllocDraginfo (ulItemCnt);
    if (!pDInfo)
      ITHROWGUIERROR("DrgAllocDraginfo");

    /*-------------------------------------------------------------------
       Set the window handle of the source and the drag operation
       in DRAGINFO.   All other fields in DRAGINFO are initialized
       by PM.
     -------------------------------------------------------------------*/
    pDInfo->hwndSource = sourceWindowHandle();
    pDInfo->usOperation = operation();

    /*-------------------------------------------------------------------
       Set up the drag image arrary if needed.
     -------------------------------------------------------------------*/
    if (diStyle == IDM::stack3AndFade)
    {
      arraySize = 4;
    }
    else if (diStyle == IDM::allStacked)
    {
      arraySize = ulItemCnt;
    }

    if (arraySize)
    {
      pDrgImg = pDragImage = new _DRAGIMAGE[arraySize];
      IASSERT(pDrgImg!=0);
    }

    /*-----------------------------------------------------------------
       Store "handle" to our drag information
     -----------------------------------------------------------------*/
    setDragInfo (pDInfo);

    /*-----------------------------------------------------------------
       Loop through the collection of IDMItems.  For each, copy
       the corresponding data, and set each one into the PM DragInfo
       structure.  Also, build the array of drag images to be displayed.
     -----------------------------------------------------------------*/
    for (unsigned long ulItem = 0; (ulItem < ulItemCnt); ulItem++)
    {
      IDMItem::Handle pItemHandle = item((ulItem+1));
      _DRAGITEM      PMDragItem;

      /*---------------------------------------------------------------
         Create a PM DragItem from the corresponding IDMItem.
         Then, set the PM DragItem into the PM DragInfo structure.
       ---------------------------------------------------------------*/
      pItemHandle->asPMDragItem(&PMDragItem);
      if (!DrgSetDragitem (pDInfo, &PMDragItem, sizeof(_DRAGITEM), ulItem))
        ITHROWGUIERROR("DrgSetDragitem");

      /*---------------------------------------------------------------
         Query the PM DragItem pointer from shared memory, and set
         its value in the corresponding ICLUI drag item.
       ---------------------------------------------------------------*/
      _DRAGITEM* pdi = DrgQueryDragitemPtr (pDInfo, ulItem);
      if (pdi)
      {
        pItemHandle->setPMDragItem(pdi);
      }
      else
      {
        ITHROWGUIERROR("DrgQueryDragitemPtr");
      }

      /*---------------------------------------------------------------
         Set up the dragimage for this drag item.
       ---------------------------------------------------------------*/
      if ((diStyle == IDM::allStacked) ||
          (diStyle == IDM::stack3AndFade))
      {
        IDMImage &image = pItemHandle->image();

        /*-------------------------------------------------------------
           Determine if we need to add the fade icon.
         -------------------------------------------------------------*/
        if ((diStyle == IDM::stack3AndFade) && (!done) &&
            (numberOfImagesFound == 3))
        {
          /*-----------------------------------------------------------
             Add the special fade icon if we have already found the
             3 we stack.
           -----------------------------------------------------------*/
          image.asPMDragImage(pDrgImg, false, true);
          ISize imageOffset = totalImageOffset();
          pDrgImg->cxOffset = (short)imageOffset.width();
          pDrgImg->cyOffset = (short)imageOffset.height();
          done = true;
          numberOfImagesFound++;
        }
        else if (!done)
        {
          image.asPMDragImage(pDrgImg);

          /*-----------------------------------------------------------
             If we got an image, increase the count.
           -----------------------------------------------------------*/
          if (pDrgImg->hImage)
          {
            ISize imageOffset;
            ISize  ptrOffset;
            numberOfImagesFound++;
            ptrOffset = image.pointerOffset();
            if (ptrOffset == 0)
            {
              imageOffset = totalImageOffset();
              pDrgImg->cxOffset = (short)imageOffset.width();
              pDrgImg->cyOffset = (short)imageOffset.height();
              storeImageOffset (&image);
            }
            else
            {
              pDrgImg->cxOffset = (short)ptrOffset.width();
              pDrgImg->cyOffset = (short)ptrOffset.height();
            }
            pDrgImg++;
          }
        }
      }
    }

    /*-----------------------------------------------------------------
       Give them system icons if they ask for them, or if they did not
       supply any images.
     -----------------------------------------------------------------*/
    if ((diStyle == IDM::systemImages) || (!numberOfImagesFound))
    {
      IDMImage image = IDMImage();
      image.asPMDragImage(&DragImage, true);

      /*---------------------------------------------------------------
         If we are dragging more than 1 item, give them the multiple
         file icon.  Otherwise, give them the single file icon.
       ---------------------------------------------------------------*/
      if (ulItemCnt > 1)
      {
        DragImage.hImage = (unsigned long)*(new ISystemPointerHandle (
                           ISystemPointerHandle::multipleFile));
      }
      else
      {
        DragImage.hImage = (unsigned long)*(new ISystemPointerHandle (
                           ISystemPointerHandle::singleFile));
      }
      numberOfImagesFound = 1;
      pDragImage = &DragImage;
    }

    /*-----------------------------------------------------------------
       Start the drag - DrgDrag executes in a synchronous manner

       Note:  Check the debug flag in order to fully enable debug
              support.
     -----------------------------------------------------------------*/
    unsigned rc = DrgDrag (pDInfo->hwndSource,
                           pDInfo,
                           pDragImage,
                           numberOfImagesFound,
                           VK_ENDDRAG,
                           ((IDM::debugSupport) ? DRG_DEBUG
                                                : 0));

    /*-----------------------------------------------------------------
       Delete the array if we created one.
     -----------------------------------------------------------------*/
    if ((diStyle != IDM::systemImages) && (pDragImage))
    {
      delete [] pDragImage;
    }
    ITRACE_DEVELOP("IDMSourceOperation:  DrgDrag rc = "+IString(rc));

    /*-----------------------------------------------------------------
       Remove any remaining items from the source operation's
       collection.  There will be remaining items if one of the
       following conditions occurs:
          - the operation was cancelled
          - help was requested
          - the drop was not allowed
       All of these conditions return NULL
     -----------------------------------------------------------------*/
    if (!rc)
    {
      setDragResult( false );
      unsigned count = numberOfItems();
      for (unsigned i = count; i; i--)
        removeItem(i);
    }
  } /* end if */

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::imageStyle                                               |
|                                                                              |
| Returns the source operation drag image style.                               |
------------------------------------------------------------------------------*/
DragImageStyle IDMSourceOperation::imageStyle () const
{
  return (this->drgImgStyle);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::pointerOffset                                            |
|                                                                              |
| Returns the position of the pointing device relative to the source container |
| object's origin.                                                             |
------------------------------------------------------------------------------*/
ISize IDMSourceOperation::pointerOffset () const
{
  return(this->ptrOffset);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::operation                                                |
|                                                                              |
| Returns the current direct manipulation operation.                           |
------------------------------------------------------------------------------*/
unsigned long IDMSourceOperation :: operation () const
{
  _DRAGINFO* di = dragInfo();
  if (di)
  {
    return((unsigned long)di->usOperation);
  }
  else
  {
    return( Inherited::operation() );
  }
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::stackingPercentage                                       |
|                                                                              |
| Returns the stacking percentage.                                             |
------------------------------------------------------------------------------*/
IPair IDMSourceOperation::stackingPercentage () const
{
  return (this->stackingPct);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::setImageStyle                                            |
|                                                                              |
| Sets the source operation drag image style.                                  |
------------------------------------------------------------------------------*/
IDMSourceOperation& IDMSourceOperation::setImageStyle(DragImageStyle dIStyle)
{
  this->drgImgStyle = dIStyle;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::setPointerOffset                                         |
|                                                                              |
| Sets the position of the pointing device relative to the source container    |
| object's origin.                                                             |
------------------------------------------------------------------------------*/
IDMSourceOperation& IDMSourceOperation::setPointerOffset (const ISize &pOff)
{
  this->ptrOffset = pOff;
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::setStackingPercentage                                    |
|                                                                              |
| Sets the stacking percentage.                                                |
------------------------------------------------------------------------------*/
IDMSourceOperation& IDMSourceOperation::
                    setStackingPercentage (const IPair& stackingPercent)
{
  this->stackingPct = stackingPercent;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::operationFrom                                            |
|                                                                              |
| Retrieves a handle to the source drag operation from the drag information    |
| structure.                                                                   |
------------------------------------------------------------------------------*/
IDMSourceOperation::Handle IDMSourceOperation ::
                           operationFrom(_DRAGINFO *pDInfo)
{
  /*-------------------------------------------------------------------
     Access the PM DRAGINFO structure which was passed to us.
   -------------------------------------------------------------------*/
  if (!DrgAccessDraginfo (pDInfo))
    ITHROWGUIERROR("DrgAccessDraginfo");

  /*-------------------------------------------------------------------
     Get count of PM drag items
   -------------------------------------------------------------------*/
  unsigned count = pDInfo->cditem;

  /*-------------------------------------------------------------------
     Loop thru PM Drag Items until we find one we can construct
     source operation form ...
   -------------------------------------------------------------------*/
  IDMSourceOperation::Handle pSrcOpH = 0;
  for (unsigned i=0; i < count; i++)
  {
    /*-----------------------------------------------------------------
       Get PM drag item
     -----------------------------------------------------------------*/
    _DRAGITEM *pPMDItem = DrgQueryDragitemPtr (pDInfo, i);

    /*-----------------------------------------------------------------
       If the pointer and ulItemID are not NULL, assume ulItemID
       contains a pointer to the source item which in turn, can
       give us the pointer to the source operation object,
       which we can construct an IDMSourceOperation::Handle from.
     -----------------------------------------------------------------*/
    if (pPMDItem)
    {
      if (pPMDItem->ulItemID)
      {
        pSrcOpH = ((IDMItem *)(pPMDItem->ulItemID))->sourceOperation();
        break;
      }
    }
  }

  return(pSrcOpH);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::storeImageOffset                                         |
|                                                                              |
| Store the offset at which to place the next image.                           |
| Notes:                                                                       |
|   1) Private                                                                 |
------------------------------------------------------------------------------*/
void IDMSourceOperation::storeImageOffset(IDMImage *pIDMImage)
{
  unsigned long style = (pIDMImage->style()).asUnsignedLong();
  ISize stretchSize = pIDMImage->stretchSize();
  IPointerHandle ptr;
  IBitmapHandle  bmp;
  POINTERINFO    PointerInfo;
  long           cxPtr,
                 cyPtr;

  /*-------------------------------------------------------------------
     Calculate the percentage that is used to position the next
     image in relation to the current image.  Default is (50,50)
     which means the nth image (where n=2, 3, or 4) is positioned
     with its lower left corner in the middle of the current image.
   -------------------------------------------------------------------*/
  if (style & DRG_ICON)
  {
    ptr = pIDMImage->pointer();
    if ((unsigned long)ptr)
    {
      if (WinQueryPointerInfo ((HPOINTER)ptr, &PointerInfo))
      {
        if (PointerInfo.fPointer)
        {
          cxPtr = WinQuerySysValue (HWND_DESKTOP, SV_CXPOINTER);
          cyPtr = WinQuerySysValue (HWND_DESKTOP, SV_CYPOINTER);
          if ((!cxPtr) || (!cyPtr))
            ITHROWGUIERROR("WinQuerySysValue");
        }
        else
        {
          cxPtr = WinQuerySysValue (HWND_DESKTOP, SV_CXICON);
          cyPtr = WinQuerySysValue (HWND_DESKTOP, SV_CYICON);
          if ((!cxPtr) || (!cyPtr))
            ITHROWGUIERROR("WinQuerySysValue");
        }
        this->totalImgOffset.setWidth(this->totalImgOffset.width() +
                             (stackingPercentage().coord1() *
                             (cxPtr + stretchSize.width())) / 100);
        this->totalImgOffset.setHeight(this->totalImgOffset.height() +
                             (stackingPercentage().coord2() *
                             (cyPtr + stretchSize.height())) / 100);
      }
      else
        ITHROWGUIERROR("WinQueryPointerInfo");
    }
  }
  else
  {
    bmp = pIDMImage->bitmap();
    if ((unsigned long)bmp)
    {
      IGBitmap bitmap(bmp);
      ISize    bmpSize = bitmap.size();
        this->totalImgOffset.setWidth(this->totalImgOffset.width() +
                             (stackingPercentage().coord1() *
                             (bmpSize.width() + stretchSize.width())) / 100);
        this->totalImgOffset.setHeight(this->totalImgOffset.height() +
                             (stackingPercentage().coord2() *
                             (bmpSize.height() + stretchSize.height())) / 100);
    }
  }
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::totalImageOffset                                         |
|                                                                              |
| Return the total image offset.                                               |
| Notes:                                                                       |
|   1) Private                                                                 |
------------------------------------------------------------------------------*/
ISize IDMSourceOperation::totalImageOffset()
{
  return (this->totalImgOffset);
}


/*------------------------------ Target Operation ----------------------------*/

/*------------------------------------------------------------------------------
| IDMTargetOperation::IDMTargetOperation                                       |
|                                                                              |
| Constructor                                                                  |
------------------------------------------------------------------------------*/
IDMTargetOperation :: IDMTargetOperation(IDMTargetEnterEvent &event)  :
                      IDMOperation(event),
                      bWasDragAfter(0),
                      sizeDropOffset(0,0),
                      dropStyle(IDM::notContainer),
                      bNotDragLeave(true),
                      bDrop(false)
{
  IMODTRACE_DEVELOP("IDMTargetOperation::ctor");

  _DRAGINFO *pDInfo;
  IDMItemProvider *pProvider;

  if (event.eventId() == DM_DRAGOVER)
  {
    /*-----------------------------------------------------------------
       Set the target window handle
     -----------------------------------------------------------------*/
    setTargetWindowHandle(event.handle());

    /*-----------------------------------------------------------------
       Get the PM DRAGINFO structure
     -----------------------------------------------------------------*/
    pDInfo = (_DRAGINFO *)event.parameter1().asUnsignedLong();

    /*-----------------------------------------------------------------
      Get pointer to the item provider
     -----------------------------------------------------------------*/
    pProvider = event.window()->itemProvider();
  }
  else
  {
    /*-----------------------------------------------------------------
       Verify that this is the correct message and notification code
     -----------------------------------------------------------------*/
    IASSERT(event.eventId() == WM_CONTROL &&
            (event.parameter1().number2() == CN_DRAGOVER ||
             event.parameter1().number2() == CN_DRAGAFTER));

    /*-----------------------------------------------------------------
       Set the target window handle
     -----------------------------------------------------------------*/
    IControlEvent ctlevt(event);
    setTargetWindowHandle(ctlevt.controlWindow()->handle());

    /*-----------------------------------------------------------------
       Get flag for over/after event and set our default drop style ...
     -----------------------------------------------------------------*/
    this->bWasDragAfter = event.isDragAfter();
    if (this->bWasDragAfter)
      setStyle(IDM::alignFlow);
    else
      setStyle(IDM::dropAtPosition);

    /*-----------------------------------------------------------------
       Set the container object pointer for the object that the over
       or after event occurred over.  0 if over container white space.
     -----------------------------------------------------------------*/
    setContainerObject(event.object());

    /*-----------------------------------------------------------------
       Get the PM DRAGINFO structure
     -----------------------------------------------------------------*/
    _CNRDRAGINFO *pCnrDInfo =
                 (_CNRDRAGINFO *)event.parameter2().asUnsignedLong();
    pDInfo = pCnrDInfo->pDragInfo;

    /*-----------------------------------------------------------------
      Get pointer to the item provider
     -----------------------------------------------------------------*/
    pProvider = ctlevt.controlWindow()->itemProvider();
  }

  /*-------------------------------------------------------------------
     Access the PM DRAGINFO structure which was passed to us.
   -------------------------------------------------------------------*/
  if (!DrgAccessDraginfo (pDInfo))
    ITHROWGUIERROR("DrgAccessDraginfo");

  /*-------------------------------------------------------------------
     Store "handle" to our drag information
   -------------------------------------------------------------------*/
  setDragInfo (pDInfo);

  /*-------------------------------------------------------------------
     Set the source window handle
   -------------------------------------------------------------------*/
  setSourceWindowHandle(pDInfo->hwndSource);

  /*-------------------------------------------------------------------
     Set the target enter (i.e. drag over) position of the pointing
     device.
   -------------------------------------------------------------------*/
  setPosition (IPoint(pDInfo->xDrop, pDInfo->yDrop));

  /*-------------------------------------------------------------------
     Set the current drag operation.
   -------------------------------------------------------------------*/
  setOperation (pDInfo->usOperation);

  /*-------------------------------------------------------------------
     Get first PM drag item
   -------------------------------------------------------------------*/
  unsigned long ulItem = 0;
  _DRAGITEM *pPMDItem = DrgQueryDragitemPtr (pDInfo, ulItem);

  /*-------------------------------------------------------------------
     For each dragitem in the draginfo, we need to create a
     corresponding IDMItem.  We construct each based upon the
     information in the PM DRAGINFO structure.
   -------------------------------------------------------------------*/
  while (pPMDItem)
  {
    /*-----------------------------------------------------------------
      Get the target operation handle.
     -----------------------------------------------------------------*/
    Handle pTgtOpH;
    pTgtOpH = this;

    /*-----------------------------------------------------------------
      Create the generic drag item based upon the PM DRAGITEM ...
     -----------------------------------------------------------------*/
    IDMItem tempItem(pTgtOpH, pPMDItem);
    IDMItem::Handle pDIH;
    pDIH = &tempItem;

    /*-----------------------------------------------------------------
      Obtain a derived target item based upon the generic drag
      item via the item provider  - Throw an exception if the
      provider fails ...
     -----------------------------------------------------------------*/
    IDMItem::Handle pTgtItemH = pProvider->provideTargetItemFor(pDIH);
    if (!pTgtItemH)
    {
      ITRACE_DEVELOP("IDMTargetOperation: target item provider for failed");
      //Throw exception here
      ITHROWLIBRARYERROR(IC_TARGET_ITEM_PROVIDER_FAILED,
                         IErrorInfo::invalidRequest,
                         IException::recoverable);
      break;
    }

    /*-----------------------------------------------------------------
      Add the target item to the collection ...
     -----------------------------------------------------------------*/
    addItem (pTgtItemH);

    /*-----------------------------------------------------------------
      Get the next PM DRAGITEM, it it exists ...
     -----------------------------------------------------------------*/
    pPMDItem = DrgQueryDragitemPtr (pDInfo, ++ulItem);
  }

  /*-------------------------------------------------------------------
     Free the PM DRAGINFO structure
   -------------------------------------------------------------------*/
  DrgFreeDraginfo (pDInfo);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::~IDMTargetOperation                                      |
|                                                                              |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IDMTargetOperation :: ~IDMTargetOperation ()
{
  IMODTRACE_DEVELOP("IDMTargetOperation::dtor");

  /*-------------------------------------------------------------------
     Enable container refresh (this function can detect whether or
     not we're processing a container) ...
   -------------------------------------------------------------------*/
  setContainerRefreshOn();

  DRAGINFO*  pPMDragInfo = dragInfo();
  if (this->bNotDragLeave)
    DrgDeleteDraginfoStrHandles (pPMDragInfo);

  if (this->bDrop)
    DrgFreeDraginfo (pPMDragInfo);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::instanceFor                                              |
|                                                                              |
| Returns the IDMTargetOperation object for the given target enter event.      |
------------------------------------------------------------------------------*/
IDMTargetOperation::Handle IDMTargetOperation ::
                           instanceFor (IDMTargetEnterEvent &event)
{
  /*-------------------------------------------------------------------
     If the target operation has not been created yet, create it and
     store it as part of the IDMTargetOperation class.
   -------------------------------------------------------------------*/
  IDMTargetOperation::Handle pTgtOpH = targetOperation();
  if (!pTgtOpH)
    return (0);

  /*------------------------------------------------------------------------
    Extract pointer to PM DRAGINFO struct from event parms
  ------------------------------------------------------------------------*/
  _DRAGINFO *pDInfo;
  if (event.eventId() == DM_DRAGOVER)
  {
    /*-----------------------------------------------------------------
       Get the PM DRAGINFO structure
     -----------------------------------------------------------------*/
    pDInfo = (_DRAGINFO *)event.parameter1().asUnsignedLong();
 }
  else
  {
    /*-----------------------------------------------------------------
       Verify that this is the correct message and notification code
     -----------------------------------------------------------------*/
    IASSERT(event.eventId() == WM_CONTROL &&
            (event.parameter1().number2() == CN_DRAGOVER ||
             event.parameter1().number2() == CN_DRAGAFTER));

    /*-----------------------------------------------------------------
       Get flag for over/after event and set our default drop style ...
     -----------------------------------------------------------------*/
    pTgtOpH->bWasDragAfter = event.isDragAfter();
    if (pTgtOpH->bWasDragAfter)
      pTgtOpH->setStyle(IDM::alignFlow);
    else
      pTgtOpH->setStyle(IDM::dropAtPosition);

    /*-------------------------------------------------------------------
       Set the container object pointer for the object that the over
       or after event occurred over.  0 if over container white space.
     -------------------------------------------------------------------*/
    pTgtOpH->setContainerObject(event.object());

    pDInfo = ((_CNRDRAGINFO *)event.parameter2().asUnsignedLong())->pDragInfo;
  }

  /*-------------------------------------------------------------------
     Access the PM DRAGINFO structure which was passed in the event.
   -------------------------------------------------------------------*/
  if (!DrgAccessDraginfo (pDInfo))
    ITHROWGUIERROR("DrgAccessDraginfo");

  /*-------------------------------------------------------------------
     Set the target enter (i.e. drag over) position of the pointing
     device.
   -------------------------------------------------------------------*/
  pTgtOpH->setPosition (IPoint(pDInfo->xDrop, pDInfo->yDrop));

  /*-------------------------------------------------------------------
     Set the current drag operation.
   -------------------------------------------------------------------*/
  pTgtOpH->setOperation (pDInfo->usOperation);

  /*-------------------------------------------------------------------
     Free the PM DRAGINFO structure
   -------------------------------------------------------------------*/
  DrgFreeDraginfo (pDInfo);

  return (pTgtOpH);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::targetOperation                                          |
|                                                                              |
| Returns the target operation object handle.                                  |
------------------------------------------------------------------------------*/
IDMTargetOperation::Handle IDMTargetOperation :: targetOperation()
{
  return (pDMTgtOpHandle);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::setTargetOperation                                       |
|                                                                              |
| Sets the target operation object handle.                                     |
------------------------------------------------------------------------------*/
void IDMTargetOperation ::
     setTargetOperation(IDMTargetOperation::Handle targetOp)
{
  pDMTgtOpHandle = targetOp;
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::wasDragAfter                                             |
|                                                                              |
| Returns the state of the latest DRAGOVER or DRAGAFTER.                       |
------------------------------------------------------------------------------*/
Boolean IDMTargetOperation :: wasDragAfter() const
{
  return(this->bWasDragAfter);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::dropOffset                                               |
|                                                                              |
| Queries the current drop offset.                                             |
------------------------------------------------------------------------------*/
ISize IDMTargetOperation :: dropOffset() const
{
  return(this->sizeDropOffset);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::style                                                    |
|                                                                              |
| Returns the drop style.                                                      |
------------------------------------------------------------------------------*/
DropStyle IDMTargetOperation :: style() const
{
  return(this->dropStyle);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::setDropOffset                                            |
|                                                                              |
| Sets the current drop offset.                                                |
------------------------------------------------------------------------------*/
IDMTargetOperation& IDMTargetOperation ::
                       setDropOffset(const ISize &offset)
{
  this->sizeDropOffset = offset;
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::setStyle                                                 |
|                                                                              |
| Sets the drop style.                                                         |
------------------------------------------------------------------------------*/
IDMTargetOperation& IDMTargetOperation ::
                       setStyle(DropStyle drpStyle)
{
  this->dropStyle = drpStyle;
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::isStyle                                                  |
|                                                                              |
| Queries whether a specified style flag is set.                               |
------------------------------------------------------------------------------*/
Boolean IDMTargetOperation :: isStyle(DropStyle drpStyle)
{
  return((style() & drpStyle) == drpStyle);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::dropPosition                                             |
|                                                                              |
| Computes the drop position for this item, in container workspace             |
| coordinates, and sets the offset for the next item, according to             |
| the operation's drop style.                                                  |
------------------------------------------------------------------------------*/
IPoint IDMTargetOperation :: dropPosition(const IDMItem::Handle &pDIH,
                                          IDMTargetDropEvent &event)
{
  IMODTRACE_DEVELOP("IDMTargetOperation dropPosition");

  IPoint ptDrop;

  if (isStyle(IDM::dropAtPosition))
  {
    /*-----------------------------------------------------------------
       Drop at position
     -----------------------------------------------------------------*/
    ptDrop = positionRelativeTo(pDIH, event);
  }
  else
  {
    /*-----------------------------------------------------------------
       Drop-align
     -----------------------------------------------------------------*/
    ISize sizOff = dropOffset();
    IPoint ptMouse = positionRelativeTo(0, event);

    /*-----------------------------------------------------------------
       Initial drop guess is mouse position plus current offset
     -----------------------------------------------------------------*/
    ptDrop = ptMouse + sizOff;

    /*-----------------------------------------------------------------
       If flow, check that we are not out of screen
     -----------------------------------------------------------------*/
    IRectangle rectObject;
    if (event.object())
      rectObject = event.container()->iconRectangle(event.object(), true);
    else
      rectObject = IRectangle();

    if (isStyle(IDM::alignFlow))
    {
      IRectangle rectWorkspace = event.container()->viewPortOnWorkspace();

      /*---------------------------------------------------------------
         Correct next position if out of container window
       ---------------------------------------------------------------*/
      if (isStyle(IDM::alignVertical))
      {
        /*-------------------------------------------------------------
           Vertical alignment
         -------------------------------------------------------------*/
        if (rectWorkspace.bottom() > ptDrop.y())
        {
          sizOff = ISize((sizOff.width() + rectObject.width()), 0);
          if (!isStyle(IDM::flowBeside))
          {
            sizOff.setHeight(rectWorkspace.top() - ptMouse.y() -
                             rectObject.height());
          }

          /*-----------------------------------------------------------
           Re-calc drop point
           -----------------------------------------------------------*/
          ptDrop = ptMouse + sizOff;
        }
      }
      else
      {
        /*-------------------------------------------------------------
           Horizontal alignment
         -------------------------------------------------------------*/
        if (rectWorkspace.right() < (ptDrop.x() + rectObject.width()))
        {
          sizOff = ISize(0, (sizOff.height() - rectObject.height()));
          if (!isStyle(IDM::flowBeside))
            sizOff.setWidth(rectWorkspace.left() - ptMouse.x());

          /*-----------------------------------------------------------
           Re-calc drop point
           -----------------------------------------------------------*/
          ptDrop = ptMouse + sizOff;
        }
      }
    }

    /*-----------------------------------------------------------------
       Compute next drop offset
     -----------------------------------------------------------------*/
    if (isStyle(IDM::alignVertical))
    {
      /*---------------------------------------------------------------
         Align vertically (-y)
       ---------------------------------------------------------------*/
      sizOff.setHeight(sizOff.height() - rectObject.height());
    }
    else
    {
      /*---------------------------------------------------------------
         Align horizontally (+x)
       ---------------------------------------------------------------*/
      sizOff.setWidth(sizOff.width() + rectObject.width());
    }

    setDropOffset(sizOff);
  }

  return(ptDrop);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::positionRelativeTo                                       |
|                                                                              |
| Returns a drag item's position relative to the target container.  If the     |
| item pointer is 0, the drop point relative to the container is returned.     |
------------------------------------------------------------------------------*/
IPoint IDMTargetOperation ::
       positionRelativeTo(const IDMItem::Handle &pDIH,
                          IDMTargetDropEvent &event)
{
  IPoint ptDrop;
  IContainerControl *pCnrTgt = event.container();

  if (pCnrTgt)
  {
    /*-----------------------------------------------------------------
       Map to container window position
     -----------------------------------------------------------------*/
    ptDrop = IWindow::mapPoint(event.dropPosition(),
                               HWND_DESKTOP,
                               pCnrTgt->handle() );

    /*-----------------------------------------------------------------
       Adjustment for workspace displacement relative to viewPort
     -----------------------------------------------------------------*/
    ptDrop += pCnrTgt->viewPortOnWorkspace().bottomLeft();

    ptDrop -= pCnrTgt->viewPortOnWindow().bottomLeft();

    /*-----------------------------------------------------------------
       Adjustment for icon offset
     -----------------------------------------------------------------*/
    if (pDIH)
      ptDrop += pDIH->imageOffset();
  }

  return(ptDrop);
}

